#include "StdAfx.h"
#include "windows.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
#include "DSBeep.h"

using namespace System;
using namespace System::Runtime::InteropServices;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Collections::Generic;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;

///////////////////////////////////////////////////
// GLOBALS FOR INFO EXCHANGING WITH THE CALLBACK
///////////////////////////////////////////////////
/*************************************************
* Acquisition Options
*************************************************/
unsigned int ObjToScan;
unsigned int OptionMask;
unsigned int DisplayOptionMask;
unsigned char ContrastLimitToDisplay;
unsigned char CompletenessLimitToDisplay;
unsigned int ClipRegionW;
unsigned int ClipRegionH;
unsigned int RollPreviewTimeout;
unsigned char ArtefactCleaningDepth;
unsigned char ContrastIntermediateLimitToDisplay;
unsigned char CompletenessIntermediateLimitToDisplay;
// ver 2.10.0.0
unsigned int AcquisitionScanArea;
bool IsRollAreaStandardGa;
// end ver 2.10.0.0
// ver 3.2.0.0
bool RollBlockCompositionIsEnabled = false;
// end ver 3.2.0.0

bool UseImageFinalization = true;
// Image dimension
unsigned int PreviewImSX,PreviewImSY,FullResImSX,FullResImSY;
/*************************************************
* Acquisition State Indicators
*************************************************/
int LastErrorCode;
BOOL AcquisitionStarted = FALSE;
unsigned int LastEventInfo;
int LastFrameSizeX = 0;
int LastFrameSizeY = 0;
double LastFrameCurrentFrameRate = .0;
double LastFrameNominalFrameRate = .0;
unsigned int LastDiagnosticValue = 0;
int AutoCapturePhase = GBMSAPI_ACP_OFF;
int ClippingRegionPosX = 0, ClippingRegionPosY = 0;
unsigned int ClippingRegionSizeX = 0, ClippingRegionSizeY = 0;
unsigned int RolledArtefactSize = 0;
unsigned char *MarkerFrame = NULL;
unsigned char *NotWipedArtefactFrame = NULL;
unsigned int FlatFingerprintSize = 0;
unsigned char ImageContrast = 0;
unsigned int ImageSize = 0;
unsigned char HalfLowerPalmCompleteness = 0;
BOOL SkipRequested = FALSE;
BOOL PreviewEnded = FALSE;

BOOL AcquisitionButtonPressed = FALSE;
BOOL ScannerStarted = FALSE;
BOOL AcquisitionEnded = FALSE;
BOOL BusyFrame = FALSE;
unsigned int AcquisitionStatus;
unsigned int PreviousAcquisitionStatus;

unsigned long AcquiredFramesNumber = 0, LostFramesNumber = 0;
/*************************************************
* Acquisition Buffers
*************************************************/
unsigned char *AcquisitionPreviewBuffer = NULL;
unsigned char *AcquisitionFullResBuffer = NULL;
unsigned char *CalibrationBuffer = NULL;
// used because sometimes can happen that
// calibrationImageSizeX is not a multiple of 4
unsigned char *DummyCalibrationBuffer = NULL; 
unsigned char *ClippedBuffer = NULL;

/*************************************************
* Frame Rate Settings
*************************************************/
// ver 2.10.0.0: frame rate parameters loaded in real time
//unsigned int FrameRateOptions = 0;
// end ver 2.10.0.0: frame rate parameters loaded in real time
// ver 2.10.0.0
bool IsRolled(unsigned int ObjTypeToScan);
// end ver 2.10.0.0


/*************************************************
* Dry/wet finger percent
*************************************************/
// ver 3.1.0.1
unsigned char DryFingerPercent = 0;
unsigned char WetFingerPercent = 0;
// end ver 3.1.0.1


void GBMSAPI_Example_LogError(char *ErrorString)
{
	//if (LogRequested == TRUE)
	{
		FILE *LogFilePtr = NULL;

		LogFilePtr = fopen("GBMSAPI_EX_LOG.log","a");
		if (LogFilePtr != NULL)
		{
			fputs(ErrorString,LogFilePtr);
			fprintf(LogFilePtr,"\n");
			fclose(LogFilePtr);
		}
	}
	return;
}

void GBMSAPI_Example_LogStart()
{
	//if (LogRequested == TRUE)
	{
		FILE *LogFilePtr = NULL;

		LogFilePtr = fopen("GBMSAPI_EX_LOG.log","w");
		if (LogFilePtr != NULL)
		{
			fclose(LogFilePtr);
		}
	}
	return;
}

///////////////////////////////////////////////////////
// ACQUISITION CALLBACK
///////////////////////////////////////////////////////
BOOL __stdcall GBMSAPIExample_AcquisitionEventsManagerCallback(
	unsigned int OccurredEventCode,
	int GetFrameErrorCode,
	unsigned int EventInfo,
	unsigned char *FramePtr,
	int FrameSizeX,
	int FrameSizeY,
	double CurrentFrameRate,
	double NominalFrameRate,
	unsigned int GB_Diagnostic,
	void *UserDefinedParameters
	)
{
	// LastEventOccurred = OccurredEventCode;
	if (SkipRequested == TRUE)
	{
		SkipRequested = FALSE;
		return FALSE;
	}

	//////////////////////////////////
	// Get Optional external equipment
	//////////////////////////////////
	unsigned int ExternalEquipment;
	int RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		GBMSAPI_StopAcquisition();
		LastErrorCode = RetVal;
		return FALSE;
	}
	

	////////////////
	// check error
	////////////////
	if (GetFrameErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		// dummy pedal read in order to clear
		unsigned char PedalState = FALSE;
		if ((ExternalEquipment & GBMSAPI_OED_PEDAL) != 0)
		{
			// check pedal state
			GBMSAPI_CheckPedalState(&PedalState);
		}
		LastErrorCode = GetFrameErrorCode;
		// Sound to advise operator that the fingerprint can be
		// released
		if ((ExternalEquipment & GBMSAPI_OED_SOUND) != 0)
		{
			GBMSAPI_Sound(12,2,1);
		}
		else
		{
			//Beep(4000,200);
			DSBeep(4000,200);
		}
		AcquisitionEnded = TRUE;
		return TRUE;
	}
				

	//////////////////////////////
	// check event:
	// ERROR EVENT ALREASY CHECKED
	//////////////////////////////
	switch (OccurredEventCode)
	{
	case GBMSAPI_AE_SCANNER_STARTED:
		{
			GBMSAPI_Example_LogStart();
			// set diagnostic sensitivity
			unsigned char paramValue = 80;
			int paramType = 3;

			//MessageBox::Show("1");
			GBMSAPI_SetDiagnosticSensitivity(TRUE,paramType,(void *)(&paramValue));
			//MessageBox::Show("2");
			// Set clipping region
			if (ClipRegionW > 0 && ClipRegionH > 0)
			{
				int RetVal = GBMSAPI_SetClippingRegionSize(ClipRegionW,ClipRegionH);
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_StopAcquisition();
					LastErrorCode = RetVal;
					return FALSE;
				}
			}
			//MessageBox::Show("3");

			// Set Roll preview timeout and artefact cleaning depth
			if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
			{
				if (RollPreviewTimeout > 0)
				{
					int RetVal = GBMSAPI_ROLL_SetPreviewTimeout(RollPreviewTimeout);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
				}

				if (ArtefactCleaningDepth > 0)
				{
					int RetVal = GBMSAPI_ROLL_SetArtefactCleaningDepth(ArtefactCleaningDepth);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
				}
			}
			//MessageBox::Show("4");
			
			ScannerStarted = TRUE;
			break;
		}
	case GBMSAPI_AE_PREVIEW_PHASE_END:
		{
			// Sound to advise operator that the preview has ended
			GBMSAPI_Example_LogError("PREVIEW END");
			if(IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
			{
				// ver 3.1.0.0: sound if not external roll procedure
				// or adaptative roll position
				if (!(OptionMask & GBMSAPI_AO_EXTERNAL_ROLL_COMPOSITION) ||
					(OptionMask & GBMSAPI_AO_ADAPT_ROLL_AREA_POSITION))
				{
					if ((ExternalEquipment & GBMSAPI_OED_SOUND) != 0)
					{
						GBMSAPI_Sound(12,2,1);
					}
					else
					{
						//Beep(4000,200);
						DSBeep(4000,200);
					}
				}
				// end ver 3.1.0.0
			}

			PreviewEnded = TRUE;
			break;
		}
	case GBMSAPI_AE_VALID_FRAME_ACQUIRED:
		{
			LastEventInfo = EventInfo;
			//FrameHasArrived = TRUE;
			if (BusyFrame == FALSE)
			{
				if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0)
				{
					memcpy(AcquisitionPreviewBuffer,FramePtr,FrameSizeX * FrameSizeY);
				}
				else
				{
					memcpy(AcquisitionFullResBuffer,FramePtr,FrameSizeX * FrameSizeY);
				}
				BusyFrame = TRUE;

				// check diagnostic
				if (GB_Diagnostic != 0 && 
					(
						((GB_Diagnostic & GBMSAPI_DM_VSROLL_ROLL_DIRECTION_RIGHT) == 0) &&
						((GB_Diagnostic & GBMSAPI_DM_VSROLL_ROLL_DIRECTION_LEFT) == 0) &&
						// ver 3.2.0.0
						((GB_Diagnostic & GBMSAPI_DM_VSROLL_ROLL_DIRECTION_DOWN) == 0) &&
						((GB_Diagnostic & GBMSAPI_DM_VSROLL_ROLL_DIRECTION_UP) == 0)
						// end ver 3.2.0.0
					)
				)
				{
					// if LED supported blink
					if ((ExternalEquipment & GBMSAPI_OED_VUI_LED) != 0)
					{
						GBMSAPI_VUI_LED_BlinkDuringAcquisition(TRUE);
					}
				}
				else
				{
					// if LED supported unblink
					if ((ExternalEquipment & GBMSAPI_OED_VUI_LED) != 0)
					{
						GBMSAPI_VUI_LED_BlinkDuringAcquisition(FALSE);
					}
				}

				
                // ver 3.1.0.1: dry/wet area percent
                RetVal = GBMSAPI_GetDryWetFingerAreaPercent(&DryFingerPercent,&WetFingerPercent);
                if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
                {
                    DryFingerPercent = WetFingerPercent = 0;
                }
                // end ver 3.1.0.1: dry/wet area percent

				// if autocapture get autocapture phase
				if ((!IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan))) &&
					(GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
				{
					// check Auto-Capture option set and not ignored
					if (((OptionMask & GBMSAPI_AO_AUTOCAPTURE) != 0)
						)
					{
						int RetVal = GBMSAPI_GetAutoCapturePhase(&AutoCapturePhase);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}

						// if diagnostic block autocapture
						unsigned int DeviceFeatures;
						RetVal = GBMSAPI_GetDeviceFeatures(&DeviceFeatures);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{							
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}
						if ((DeviceFeatures & GBMSAPI_DF_AUTO_CAPTURE_BLOCKING) != 0)
						{
							if (GB_Diagnostic != 0)
							{
								GBMSAPI_SetAutoCaptureBlocking(TRUE);
								AutoCapturePhase = -1;
							}
							else
							{
								GBMSAPI_SetAutoCaptureBlocking(FALSE);
							}
						}
					}					
				}

				// if clipping get clipping region
				if ((!IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan))) &&
					(GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO) &&
					(ClipRegionW > 0 && ClipRegionH > 0))
				{
					int RetVal = GBMSAPI_GetClippingRegionPosition(
						&ClippingRegionPosX,
						&ClippingRegionPosY,
						&ClippingRegionSizeX,
						&ClippingRegionSizeY
						);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
				}

				// if not rolled or photo get contrast
				if ((!IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan))) &&
					(GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
				{
					unsigned int AvailableImageInfo;
					int RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_GetTypeFromObject(ObjToScan),&AvailableImageInfo);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
					if ((AvailableImageInfo & GBMSAPI_AII_FINGERPRINT_CONTRAST) != 0)
					{
						RetVal = GBMSAPI_GetFingerprintContrast(&ImageContrast);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}

						// if customized contrast, set on the LCD
						if (((DisplayOptionMask & GBMSAPI_DO_CUSTOMIZED_CONTRAST) != 0) &&
							((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
							)
						{
							RetVal = GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen(ImageContrast);
							if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
							{
								GBMSAPI_StopAcquisition();
								LastErrorCode = RetVal;
								return FALSE;
							}
						}
					}
				}

				// if not rolled or photo get size
				if ((!IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan))) &&
					(GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
				{
					unsigned int AvailableImageInfo;
					int RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_GetTypeFromObject(ObjToScan),&AvailableImageInfo);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
					if ((AvailableImageInfo & GBMSAPI_AII_FINGERPRINT_SIZE) != 0)
					{
						RetVal = GBMSAPI_GetFingerprintSize(&ImageSize);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}
					}
				}

				// if half lower palm get completeness
				if ((GBMSAPI_GetTypeFromObject(ObjToScan) == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM))
				{
					unsigned int AvailableImageInfo;
					int RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_GetTypeFromObject(ObjToScan),&AvailableImageInfo);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
					if ((AvailableImageInfo & GBMSAPI_AII_LOWER_HALF_PALM_COMPLETENESS) != 0)
					{
						RetVal = GBMSAPI_GetLowerHalfPalmCompleteness(&HalfLowerPalmCompleteness);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}

						// if customized completeness, set on the LCD
						if ((DisplayOptionMask & GBMSAPI_DO_CUSTOMIZED_COMPLETENESS) != 0 &&
							((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
							)
						{
							RetVal = GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen(HalfLowerPalmCompleteness);
							if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
							{
								GBMSAPI_StopAcquisition();
								LastErrorCode = RetVal;
								return FALSE;
							}
						}
					}
				}

				LastFrameSizeX = FrameSizeX;
				LastFrameSizeY = FrameSizeY;
				LastFrameCurrentFrameRate = CurrentFrameRate;
				LastFrameNominalFrameRate = NominalFrameRate;
				LastDiagnosticValue = GB_Diagnostic;
			}
			break;
		}
	case GBMSAPI_AE_ACQUISITION_END:
		{
			LastEventInfo = EventInfo;
			AcquisitionEnded = TRUE;

			// Sound to advise operator that the fingerprint can be
			// released
			if ((ExternalEquipment & GBMSAPI_OED_SOUND) != 0)
			{
				GBMSAPI_Sound(12,2,1);
			}
			else
			{
				//Beep(4000,200);
				DSBeep(4000,200);
			}

			// dummy pedal read in order to clear
			unsigned char PedalState = FALSE;
			if ((ExternalEquipment & GBMSAPI_OED_PEDAL) != 0)
			{
				// check pedal state
				GBMSAPI_CheckPedalState(&PedalState);
			}

			GBMSAPI_GetFrameStatistic(&AcquiredFramesNumber,&LostFramesNumber);

			// Get Type for GetImageSize(...) function
			unsigned int ObjType = GBMSAPI_GetTypeFromObject(ObjToScan);
			if (ObjType != GBMSAPI_SBT_NO_OBJECT_TYPE)
			{
				if ((GB_Diagnostic & GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA) == 0 &&
					(GB_Diagnostic & GBMSAPI_DM_SCANNER_FAILURE) == 0
				)
				{
					if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) != 0 &&
						(!IsRolled(ObjType)) &&
						(ObjType != GBMSAPI_SBT_PHOTO)
						)
					{
						if (UseImageFinalization) GBMSAPI_ImageFinalization(FramePtr);
					}
					else if (IsRolled(ObjType) && 
						(OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP))
					{
						if (UseImageFinalization) GBMSAPI_ImageFinalization(FramePtr);
					}
				}

				
				if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0)
				{
					memcpy(AcquisitionPreviewBuffer,FramePtr,FrameSizeX * FrameSizeY);
				}
				else
				{
					memcpy(AcquisitionFullResBuffer,FramePtr,FrameSizeX * FrameSizeY);
				}
				BusyFrame = TRUE;

				LastFrameSizeX = FrameSizeX;
				LastFrameSizeY = FrameSizeY;
				LastFrameCurrentFrameRate = CurrentFrameRate;
				LastFrameNominalFrameRate = NominalFrameRate;
				LastDiagnosticValue = GB_Diagnostic;

				// if autocapture get autocapture phase
				if (!IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)) &&
					(GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
				{
					// check Auto-Capture option set and not ignored
					if (((OptionMask & GBMSAPI_AO_AUTOCAPTURE) != 0)
						)
					{
						int RetVal = GBMSAPI_GetAutoCapturePhase(&AutoCapturePhase);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}
					}
				}

				// if clipping get clipping region
				if ((GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO) &&
					(ClipRegionW > 0 && ClipRegionH > 0))
				{
					int RetVal = GBMSAPI_GetClippingRegionPosition(
						&ClippingRegionPosX,
						&ClippingRegionPosY,
						&ClippingRegionSizeX,
						&ClippingRegionSizeY
						);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
				}

				// if not rolled or photo get contrast
				if ((!IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan))) &&
					(GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
				{
					unsigned int AvailableImageInfo;
					int RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_GetTypeFromObject(ObjToScan),&AvailableImageInfo);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
					if ((AvailableImageInfo & GBMSAPI_AII_FINGERPRINT_CONTRAST) != 0)
					{
						RetVal = GBMSAPI_GetFingerprintContrast(&ImageContrast);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}
					}
				}

				// if not rolled or photo get size
				if ((!IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan))) &&
					(GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
				{
					unsigned int AvailableImageInfo;
					int RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_GetTypeFromObject(ObjToScan),&AvailableImageInfo);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
					if ((AvailableImageInfo & GBMSAPI_AII_FINGERPRINT_SIZE) != 0)
					{
						RetVal = GBMSAPI_GetFingerprintSize(&ImageSize);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}
					}
				}

				// if half lower palm get completeness
				if ((GBMSAPI_GetTypeFromObject(ObjToScan) == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM))
				{
					unsigned int AvailableImageInfo;
					int RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_GetTypeFromObject(ObjToScan),&AvailableImageInfo);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
					if ((AvailableImageInfo & GBMSAPI_AII_LOWER_HALF_PALM_COMPLETENESS) != 0)
					{
						RetVal = GBMSAPI_GetLowerHalfPalmCompleteness(&HalfLowerPalmCompleteness);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_StopAcquisition();
							LastErrorCode = RetVal;
							return FALSE;
						}
					}
				}

				// if Roll object get rolled info
				if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
				{
					// ver 2.10.0.0: use "2" function, comment next lines
					//int RetVal = GBMSAPI_ROLL_GetCompositeImageInfo(
					//	&RolledArtefactSize,
					//	&MarkerFrame,
					//	&NotWipedArtefactFrame,
					//	&ImageSize,
					//	&ImageContrast,
					//	&FlatFingerprintSize
					//	);
					// end ver 2.10.0.0: use "2" function, comment next lines
					// ver 2.10.0.0: use "2" function, add next lines
					unsigned int imgsx, imgsy;
					int RetVal = GBMSAPI_ROLL_GetCompositeImageInfo2(
						&RolledArtefactSize,
						&MarkerFrame,
						&NotWipedArtefactFrame,
						&ImageSize,
						&ImageContrast,
						&FlatFingerprintSize,
						&imgsx,&imgsy
						);
					// end ver 2.10.0.0: use "2" function, add next lines
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR
						&& RetVal != GBMSAPI_ERROR_CODE_UNAVAILABLE_OPTION // means that rolling has been stopped before roll begins
						)
					{
						GBMSAPI_StopAcquisition();
						LastErrorCode = RetVal;
						return FALSE;
					}
				}
			}
			
			break;
		}
	case GBMSAPI_AE_ACQUISITION_ERROR:
		{
			// dummy pedal read in order to clear
			unsigned char PedalState = FALSE;
			if ((ExternalEquipment & GBMSAPI_OED_PEDAL) != 0)
			{
				// check pedal state
				GBMSAPI_CheckPedalState(&PedalState);
			}
			LastErrorCode = GetFrameErrorCode;
			// Sound to advise operator that the fingerprint can be
			// released
			if ((ExternalEquipment & GBMSAPI_OED_SOUND) != 0)
			{
				GBMSAPI_Sound(12,2,1);
			}
			else
			{
				//Beep(4000,200);
				DSBeep(4000,200);
			}
			AcquisitionEnded = TRUE;
			return TRUE;
		}


	}

	return true;
}