#include "stdafx.h"
#include "windows.h"
#include "GBMSAPIExampleMainForm.h"
#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
#include "GBMSAPIExample_Util.h"
#include "SelectDeviceForm.h"
#include "CalibrationForm.h"
#include "VUI_LED_form.h"
#include "AcquisitionForm.h"
#include "PermanentDataStorageForm.h"



using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Collections::Generic;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;

using namespace GBMSAPICExample;

namespace GBMSAPICExample {
	System::Void AcquisitionForm::AcquiredImagePictureBox_Paint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e)		
	{
		try
		{
			// Adjust picture box to frame size
			float XFactor = ((float)(this->AcquiredImagePictureBox->Width) / LastFrameSizeX);
			float YFactor = ((float)(this->AcquiredImagePictureBox->Height) / LastFrameSizeY);

			// check for clipping
			if (
			 ClippingRegionPosX >= 0 &&
			 ClippingRegionPosY >= 0 &&
			 ClippingRegionSizeX != 0 &&
			 ClippingRegionSizeY != 0
			 )
			{
				float ClipPosX = ClippingRegionPosX * XFactor;
				float ClipPosY = ClippingRegionPosY * YFactor;
				float ClipSizeX = ClippingRegionSizeX * XFactor;
				float ClipSizeY = ClippingRegionSizeY * YFactor;
				
				System::Drawing::Pen ^ClipPen = gcnew Pen(Color::Blue,3.0);
				e->Graphics->DrawRectangle(
					ClipPen,
					ClipPosX,
					ClipPosY,
					ClipSizeX,
					ClipSizeY
				);
			}

			// at the end of rolled acquisition, draw roll info
			if (this->DrawRollInfo)
			{
				if (MarkerFrame != nullptr)
				{
					System::Drawing::Pen ^WipedPen = gcnew Pen(Color::Red,1.0);
					System::Drawing::Pen ^ArtefactPen = gcnew Pen(Color::Green,1.0);
					for (int x = 0; x < LastFrameSizeX; x ++)
					{
						for (int y = 0; y < LastFrameSizeY; y ++)
						{
							int PositionInVector = x + y * LastFrameSizeX;
							if ((MarkerFrame[PositionInVector] & GBMSAPI_ARTEFACT_REGION_MARKED_BORDER) )
							{
								float xPoint = x * XFactor;
								float yPoint = y * YFactor;
								e->Graphics->DrawRectangle(ArtefactPen,xPoint,yPoint,2.0,2.0);
							}
							if ((MarkerFrame[PositionInVector] & GBMSAPI_ARTEFACT_REGION_WIPED_BORDER))
							{
								float xPoint = x * XFactor;
								float yPoint = y * YFactor;
								e->Graphics->DrawRectangle(WipedPen,xPoint,yPoint,2.0,2.0);
							}								
						}
					}
				}
			}

            // Ver 3.1.0.0
            unsigned int DeviceFeaturesMask;
            int RetVal = GBMSAPI_GetDeviceFeatures(& DeviceFeaturesMask);
            if (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR &&
                ((DeviceFeaturesMask & GBMSAPI_DF_ROLL_OBJECT_STRIPE) != 0)
                )
            {
                int leftX, rightX;
                RetVal = GBMSAPI_ROLL_GetCurrentStripeCoord(&leftX, &rightX);
                if (
                    (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR) &&
                    (leftX >= 0) && (rightX >= 0)
                    )
                {
                    System::Drawing::Pen ^SlicePen = gcnew Pen(Color::Aquamarine, (float)1.0);
                    e->Graphics->DrawRectangle(SlicePen,
						(int)(leftX * XFactor), (int)0,
						(int)((rightX - leftX) * XFactor),
                        (int)(this->AcquiredImagePictureBox->Height));
                    String ^dbgStr = "L " + leftX + "; R " + rightX;
                    this->DiagnosticMessageListBox->Items->Add(dbgStr);
                }
            }
            // end Ver 3.1.0.0
		
		}			
		catch (Exception ^ex)
		{
			MessageBox::Show("Exception in AcquiredImagePictureBox_Paint: " + ex->Message);
			GBMSAPI_StopAcquisition();
			this->Close();
		}
	}

	System::Void AcquisitionForm::AcquisitionManagementGeneralTimer_Tick(System::Object^  sender, System::EventArgs^  e)
	{
		try
		{
			this->AcquisitionManagementGeneralTimer->Stop();

			this->AcquiredFramesNumTextBox->Text = "" + AcquiredFramesNumber;
			this->LostFramesNumTextBox->Text = "" + LostFramesNumber;

			TimeSpan x = DateTime::Now.Subtract(this->DelayBeforeStartTimer);

			if (x.TotalMilliseconds > 1000)
			{
				if (this->FirstStart == true)
				{
					this->StartStopButton_Click(nullptr,nullptr);
					//this->StartStopButton->Enabled = true;
					this->FirstStart = false;
				}					
			}

			this->AcquisitionStateManagement();

			this->AcquisitionManagementGeneralTimer->Start();
		}
		catch (Exception ^ex)
		{
			MessageBox::Show("Exception in management timer: " + ex->Message);
			GBMSAPI_StopAcquisition();
			this->Close();
		}
	}


	AcquisitionForm::AcquisitionForm(GBMSAPIExampleMainForm ^CallingWindow)
	{
		InitializeComponent();
		//
		//TODO: Add the constructor code here
		//
		UseImageFinalization = true;
		this->cbExcludeFinalization->Checked = false;

		///////////////////////////////////
		// SET CALLING WINDOW
		///////////////////////////////////
		if (CallingWindow == nullptr)
		{
			this->Close();
		}
		this->AcquisitionFormCallingWindow = CallingWindow;			

		///////////////////////////////////
		// load scan options
		///////////////////////////////////
		Boolean res = this->AcquisitionFormCallingWindow->GetAcquisitionOptions(
			&(ObjToScan),
			&OptionMask,
			&DisplayOptionMask,
			&ContrastLimitToDisplay,
			&CompletenessLimitToDisplay,
			&ClipRegionW,
			&ClipRegionH,
			&RollPreviewTimeout,
			&ArtefactCleaningDepth,
			&ContrastIntermediateLimitToDisplay,
			&CompletenessIntermediateLimitToDisplay,
			&IsRollAreaStandardGa,
			&RollBlockCompositionIsEnabled
			);

		if (res == false)
		{
			this->Close();
		}

		////////////////////////////////
		// SHOW OBJECT TO SCAN NAME
		////////////////////////////////
		String ^ObjName = GBMSAPI_Example_Util::GetObjectToScanNameFromID(ObjToScan);
		this->ObjectNameTextBox->Text = ObjName;

		//////////////////
		// ALLOC BITMAPS
		//////////////////
		
		// Get Type for GetImageSize(...) function
		unsigned int ObjType = GBMSAPI_GetTypeFromObject(ObjToScan);
		if (ObjType == GBMSAPI_SBT_NO_OBJECT_TYPE)
		{
			MessageBox::Show("Object to scan not recognized","ERROR");
			this->Close();
		}
		// Set scanning area
		// ver 2.10.0.0: use Scan instead of Flat/Roll area
		AcquisitionScanArea = GBMSAPI_Example_Util::GetScanAreaFromObjectType(
			ObjType, 
			((OptionMask & GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA) != 0), 
			IsRollAreaStandardGa
		);
		// end ver 2.10.0.0: use Scan instead of Flat/Roll area

		// ver 3.1.0.0 use "3" functions
        /*******************************
         * Get image size
         * ****************************/
		int RetVal = GBMSAPI_GetImageSize3(ObjType,OptionMask,AcquisitionScanArea,
			&FullResImSX,&FullResImSY,
			&PreviewImSX,&PreviewImSY
			);
        // end ver 3.1.0.0
		if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
		{
			GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"AcquisitionForm Constructor");
			this->Close();
		}
		/****************************
		// ALLOC PREVIEW IMAGE BITMAP
		****************************/
		// image allocation
		PreviewImage = gcnew Bitmap(PreviewImSX,PreviewImSY,Imaging::PixelFormat::Format8bppIndexed);
		if (PreviewImage == nullptr)
		{
			MessageBox::Show("AcquisitionForm Constructor: Memory allocation failed","FATAL ERROR");
			this->Close();
		}
        // ver 3.1.0.0 size and location for paint box
		GBMSAPI_Example_Util::CalculatePaintBoxSizeAndLocation(
			PreviewImSX, PreviewImSY,
            GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxOriginalSize, GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxOriginalLocation,
            GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForPreview, GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForPreview);
        // end ver 3.1.0.0 size and location for paint box
		
		/*****************************
		// ALLOC FULL RES IMAGE BITMAP
		*****************************/
		FullResImage = gcnew Bitmap(FullResImSX,FullResImSY,Imaging::PixelFormat::Format8bppIndexed);
		if (FullResImage == nullptr)
		{
			MessageBox::Show("AcquisitionForm Constructor: Memory allocation failed","FATAL ERROR");
			this->Close();
		}

        // ver 3.1.0.0 size and location for paint box
        GBMSAPI_Example_Util::CalculatePaintBoxSizeAndLocation(
			FullResImSX, FullResImSY,
            GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxOriginalSize, GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxOriginalLocation,
            GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForAcq, GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForAcq);

        this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForPreview;
        this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForPreview;
        // end ver 3.1.0.0 size and location for paint box

        // ver 3.1.0.0
        /////////////////////////////////
        // UPDATE BG IMAGE BUTTON
        /////////////////////////////////
        unsigned int devFeatures = 0;
        RetVal = GBMSAPI_GetDeviceFeatures(&devFeatures);
        if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
        {
			GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                RetVal, "AcquisitionForm,GBMSAPI_NET_GetDeviceFeatures");
            return;
        }
        if ((devFeatures & GBMSAPI_DF_UPDATE_BACKGROUND_IMAGE) != 0 &&
			!(IsRolled(ObjType))) // flat objects only
        {
            this->bUpdateBackgroundImage->Enabled = true;
        }
        else
        {
            this->bUpdateBackgroundImage->Enabled = false;
        }
        // end ver 3.1.0.0

		/////////////////////////////////
		// INIT ACQUISITION STATE GLOBALS
		/////////////////////////////////
		AcquisitionStatus = WAIT_START_STATUS;
		PreviousAcquisitionStatus = WAIT_START_STATUS;
		//AcquisitionInProgress = false;
		// LastEventOccurred = 0;
		LastErrorCode = GBMSAPI_ERROR_CODE_NO_ERROR;
		//FrameHasArrived = FALSE;
		
		
		LastFrameSizeX = 0;
		LastFrameSizeY = 0;
		LastFrameCurrentFrameRate = .0;
		LastFrameNominalFrameRate = .0;

		AutoCapturePhase = GBMSAPI_ACP_OFF;
		ClippingRegionPosX = 0;
		ClippingRegionPosY = 0;
		ClippingRegionSizeX = 0;
		ClippingRegionSizeY = 0;
		RolledArtefactSize = 0;
		MarkerFrame = nullptr;
		NotWipedArtefactFrame = nullptr;
		FlatFingerprintSize = 0;
		ImageContrast = 0;
		ImageSize = 0;
		HalfLowerPalmCompleteness = 0;
		SkipRequested = FALSE;

		// ver 3.1.0.1
		DryFingerPercent = WetFingerPercent = 0;
		// end ver 3.1.0.1
		
		DrawRollInfo = false;

		this->AutoCaptureAccettedLabel->BackColor = Color::White;
		this->FullResolutionPreviewAcceptedLabel->BackColor = Color::White;
		this->AcquireFlatObjectOnRollAreaAcceptedLabel->BackColor = Color::White;

		if (IsRolled(ObjType))
		{
			if ((OptionMask & GBMSAPI_AO_NO_ROLL_PREVIEW) != 0)
			{
				this->RollPreviewModeTextBox->Text = "NO PREVIEW";
			}
			else if ((OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP) != 0)
			{
				this->RollPreviewModeTextBox->Text = "MANUAL";
			}
			else if ((OptionMask & GBMSAPI_AO_ROLL_PREVIEW_TYPE) != 0)
			{
				this->RollPreviewModeTextBox->Text = "CENTER (Auto)";
			}
			else
			{
				this->RollPreviewModeTextBox->Text = "SIDE (Auto)";
			}
		}
		else
		{
			this->RollPreviewModeTextBox->Text = "";
		}

		// Start acquisition immediately
		DelayBeforeStartTimer = DateTime::Now;
		this->StartStopButton->Enabled = false;
		FirstStart = true;
		ScannerStarted = FALSE;
		AcquisitionEnded = FALSE;
		BusyFrame = FALSE;
		PreviewEnded = FALSE;

		

		/////////////////////
		// start timer
		/////////////////////
		this->AcquisitionManagementGeneralTimer->Start();
	}


	System::Void AcquisitionForm::StartStopButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		try
		{
			AcquisitionButtonPressed = TRUE;
		}			
		catch (Exception ^ex)
		{
			MessageBox::Show("Exception in StartStopButton_Click: " + ex->Message);
			GBMSAPI_StopAcquisition();
			this->Close();
		}
	}

	// ver 3.1.0.0
	System::Void AcquisitionForm::bUpdateBackgroundImage_Click(System::Object^  sender, System::EventArgs^  e)
	{
		unsigned int Diag;
        int RetVal;
        if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0)
        {
			RetVal = GBMSAPI_UpdateBackgroundImage(AcquisitionPreviewBuffer, &Diag);
        }
        else
        {
			RetVal = GBMSAPI_UpdateBackgroundImage(AcquisitionFullResBuffer, &Diag);
        }
		if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
		{
			GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"bUpdateBackgroundImage_Click,GBMSAPI_UpdateBackgroundImage");
		}
	}
	// end ver 3.1.0.0

	// ver 3.1.0.0
	System::Void AcquisitionForm::ShowRotationNotCompletedMessage()
	{
		if (!(OptionMask & GBMSAPI_AO_EXTERNAL_ROLL_COMPOSITION)
			//|| (OptionMask & GBMSAPI_AO_ADAPT_ROLL_AREA_POSITION)
		)
		{
			MessageBox::Show("Rotation not completed","WARNING");
		}
	}
	// end ver 3.1.0.0

	System::Void AcquisitionForm::AcquisitionStateManagement(void)
	{
		try {
			switch (AcquisitionStatus)
			{
			case WAIT_START_STATUS:
				{
					this->PreviewLabel->Visible = false;
					this->WaitFirstFrameLabel->Visible = false;
					if (this->FirstStart == false &&
						AcquisitionButtonPressed == FALSE)
					{
						this->StartStopButton->Text = "REPEAT";
						this->StartStopButton->Enabled = true;
					}
					else
					{
						this->StartStopButton->Text = "START";
					}
					

					unsigned int OptionalExternalEquipment;
					int RetVal = GBMSAPI_GetOptionalExternalEquipment(&OptionalExternalEquipment);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetOptionalExternalEquipment");
						return;
					}

					unsigned char PedalState = FALSE;
					Byte PressedButton = 0;
					// check pedal support
					
					if ((OptionalExternalEquipment & GBMSAPI_OED_PEDAL) != 0)
					{
						if (this->FirstStart == false)
						{
							this->StartStopButton->Text += "(or press pedal)";
						}
						// check pedal state
						GBMSAPI_CheckPedalState(&PedalState);
					}

					
					if ((OptionalExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
					{
						RetVal = GBMSAPI_VUI_LCD_GetPressedButton(&PressedButton);
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_VUI_LCD_GetPressedButton");
							return;
						}

						// if OK Button is pressed, close window
						if (PressedButton == GBMSAPI_VILCD_TOUCHSCREEN_OK_BUTTON)
						{
							this->Close();
							return;
						}
					}

					// check acquisition start request
					if (this->FirstStart == false &&
						AcquisitionButtonPressed == TRUE || 
						PedalState == TRUE ||
						PressedButton == GBMSAPI_VILCD_TOUCHSCREEN_START_BUTTON || // from logo screen
						PressedButton == GBMSAPI_VILCD_TOUCHSCREEN_REPEAT_BUTTON // from stop screen
						)
					{
						// Clear Image
						memset(AcquisitionPreviewBuffer,0xFF,(this->PreviewImage->Width)*(this->PreviewImage->Height));
						GBMSAPI_Example_Util::CopyRawImageIntoBitmap(
									(IntPtr)AcquisitionPreviewBuffer,PreviewImage);

						// ver 3.1.0.0 size and location for paint box
						this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForPreview;
						this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForPreview;
						// end ver 3.1.0.0 size and location for paint box
						this->AcquiredImagePictureBox->Image = PreviewImage;
						this->AcquiredImagePictureBox->SizeMode = PictureBoxSizeMode::StretchImage;

						// Reset variables
						// LastEventOccurred = 0;
						LastErrorCode = GBMSAPI_ERROR_CODE_NO_ERROR;
						BusyFrame = FALSE;
						
						LastFrameSizeX = 0;
						LastFrameSizeY = 0;
						LastFrameCurrentFrameRate = .0;
						LastFrameNominalFrameRate = .0;

						ClippingRegionPosX = 0;
						ClippingRegionPosY = 0;
						ClippingRegionSizeX = 0;
						ClippingRegionSizeY = 0;

						AutoCapturePhase = GBMSAPI_ACP_OFF;

						RolledArtefactSize = 0;
						MarkerFrame = nullptr;
						NotWipedArtefactFrame = nullptr;
						FlatFingerprintSize = 0;
						ImageContrast = 0;
						ImageSize = 0;
						HalfLowerPalmCompleteness = 0;
						// ver 3.1.0.1
						DryFingerPercent = WetFingerPercent = 0;
						// end ver 3.1.0.1

						// Reset State conditions
						ScannerStarted = FALSE;
						AcquisitionEnded = FALSE;
						SkipRequested = FALSE;
						PreviewEnded = FALSE;

						// Clear text boxes
						this->StopTypeTextBox->Text = "";
						this->ImSXTextBox->Text = "";
						this->ImSYTextBox->Text = "";
						this->AutoCapturePhaseTextBox->Text = "";
						this->ContrastTextBox->Text = "";
						this->SizeTextBox->Text = "";
						this->HLPCompletenessTextBox->Text = "";
						
						// clear Accepted options labels
						this->AutoCaptureAccettedLabel->BackColor = Color::White;
						this->FullResolutionPreviewAcceptedLabel->BackColor = Color::White;
						this->AcquireFlatObjectOnRollAreaAcceptedLabel->BackColor = Color::White;

						DrawRollInfo = false;
                        // ver 3.2.0.0
                        if (RollBlockCompositionIsEnabled)
                        {
                            GBMSAPI_ROLL_EnableBlockComposition(1);
                        }
                        else
                        {
                            GBMSAPI_ROLL_EnableBlockComposition(0);
                        }
                        // end ver 3.2.0.0	

						////////////////////////////////////
						// START ACQUISITION
						////////////////////////////////////
						// ver 2.10.0.0: use "2" function, comment next lines
						//int RetVal = GBMSAPI_StartAcquisition(
						//	ObjToScan,
						//	OptionMask,
						//	(GBMSAPI_AcquisitionEventsManagerCallback)
						//	(GBMSAPIExample_AcquisitionEventsManagerCallback),
						//	nullptr,
						//	DisplayOptionMask,
						//	ContrastLimitToDisplay,
						//	CompletenessLimitToDisplay
						//	);
						// end ver 2.10.0.0: use "2" function, comment next lines
						// ver 2.10.0.0: use "2" function, add next lines
						int RetVal = GBMSAPI_StartAcquisition2(
							ObjToScan,
							OptionMask,
							AcquisitionScanArea,
							(GBMSAPI_AcquisitionEventsManagerCallback)
							(GBMSAPIExample_AcquisitionEventsManagerCallback),
							nullptr,
							DisplayOptionMask,
							ContrastLimitToDisplay,
							CompletenessLimitToDisplay
							);
						// end ver 2.10.0.0: use "2" function, add next lines
						if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
								RetVal,"GBMSAPI_StartAcquisition in WAIT_START_STATUS");
						}
						else
						{
							// disable Start button until scanner effectively started
							this->StartStopButton->Enabled = false;
							AcquisitionStatus = WAIT_SCANNER_STATUS;
						}
						AcquisitionButtonPressed = FALSE;
						
					}

					this->Update();
					break;
				}

			case WAIT_SCANNER_STATUS:
				{
					this->WaitFirstFrameLabel->Visible = true;
					this->StartStopButton->Enabled = false;
					this->PreviewLabel->Visible = false;

					// check errors
					if (LastErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR || AcquisitionEnded == TRUE)
					{
						// message to be shown: in case of error, it will be set to 0
						unsigned char MessageToShowOnStopScreen = GBMSAPI_VILCD_MSG_ACQUISITION_SUCCESSFUL;
						// return to WAIT_START
						if (LastErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
								LastErrorCode,"WAIT_SCANNER_STATUS");
							MessageToShowOnStopScreen = 0;
						}
						LastErrorCode = GBMSAPI_ERROR_CODE_NO_ERROR;
						AcquisitionStatus = WAIT_START_STATUS;
						if (AcquisitionEnded == TRUE)
						{
							// check stop type
							if ((LastEventInfo & GBMSAPI_EI_STOP_TYPE) == 0)
							{
								// manual
								this->StopTypeTextBox->Text = "MANUAL STOP";
								if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
								{
									if ((OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP) == 0)
											ShowRotationNotCompletedMessage(); 
								}
								else if
									(
										(OptionMask & GBMSAPI_AO_AUTOCAPTURE)									
									)
								{
									MessageBox::Show("Image capture manually stopped before automatic stop: check image quality",
										"WARNING"); 
								}
							}
							else
							{
								this->StopTypeTextBox->Text = "AUTO STOP";
								if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_TIMEOUT) != 0)
								{
									MessageBox::Show("ACQUISITION TIMEOUT HAS EXPIRED","WARNING");
								}
							}

							// If LCD supported, enable OK button on stop screen
							unsigned int ExternalEquipment = 0;
							int RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
							if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
							{
								GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
									RetVal,"WAIT_ACQUISITION_END_STATUS");
								ExternalEquipment = 0;
							}
							if ((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
							{
								if (
									(DisplayOptionMask & GBMSAPI_DO_FINAL_SCREEN) != 0 // stop screen enabled after acquisition
								)
								{
									// enable OK button
									RetVal = GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
									if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
									{
										GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
											RetVal,"WAIT_ACQUISITION_END_STATUS");
									}
									// show message
									if (MessageToShowOnStopScreen != 0)
									{
										RetVal = GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(
											MessageToShowOnStopScreen,
											GBMSAPI_VILCD_MSG_AREA);
										if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
										{
											GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
												RetVal,"WAIT_ACQUISITION_END_STATUS");
										}
									}
									else
									{
										RetVal = GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(
											"ERROR IN ACQUISITION",
											GBMSAPI_VILCD_CUSTOM_MSG_TYPE_ERROR_MSG);
										if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
										{
											GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
												RetVal,"WAIT_ACQUISITION_END_STATUS");
										}
									}
								}
								else
								{
									RetVal = GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();
									if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
									{
										GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
											RetVal,"WAIT_ACQUISITION_END_STATUS");
									}
								}
							}

							AcquisitionEnded = FALSE;
						}
						
					}
					else
					{
						if (ScannerStarted == TRUE)
						{
							// update accepted options labels
							// check ignored options						
							if ((OptionMask & GBMSAPI_AO_AUTOCAPTURE) != 0)
							{
								this->AutoCaptureAccettedLabel->BackColor = Color::Green;
								this->AutoCaptureAccettedLabel->Update();
							}

							//if ((OptionMask & GBMSAPI_AO_FULL_RES_PREVIEW) != 0)
							//{
							//	this->FullResolutionPreviewAcceptedLabel->BackColor = Color::Green;
							//	// realloc image preview
							//	this->PreviewImage = gcnew Bitmap(
							//		FullResImSX,FullResImSY,FullResImSX,
							//		Imaging::PixelFormat::Format8bppIndexed,
							//		IntPtr(AcquisitionPreviewBuffer)
							//	);
							//	if (PreviewImage == nullptr)
							//	{
							//		MessageBox::Show("AcquisitionForm Constructor: Memory allocation failed","FATAL ERROR");
							//		this->Close();
							//	}
							//	Imaging::ColorPalette ^pal = this->PreviewImage->Palette;
							//	for (int i = 0; i < pal->Entries->Length; i++)
							//	{
							//		pal->Entries[i] = Color::FromArgb(255, i, i, i);
							//	}
							//	this->PreviewImage->Palette = pal;
							//}

							if ((OptionMask & GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA) != 0)
							{
								this->AcquireFlatObjectOnRollAreaAcceptedLabel->BackColor = Color::Green;
							}

							// enable start button
							this->StartStopButton->Text = "STOP";
							this->StartStopButton->Enabled = true;
							this->StartStopButton->Update();

							// switch to next state
							AcquisitionStatus = WAIT_FIRST_FRAME_STATUS;
						}
					}

					this->Update();
					break;
				}

			case WAIT_FIRST_FRAME_STATUS:
				{
					this->WaitFirstFrameLabel->Visible = true;
					this->PreviewLabel->Visible = false;

					// check errors
					if (LastErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR || AcquisitionEnded == TRUE)
					{
						// message to be shown: in case of error, it will be set to 0
						unsigned char MessageToShowOnStopScreen = GBMSAPI_VILCD_MSG_ACQUISITION_SUCCESSFUL;
						// return to WAIT_START
						if (LastErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR)
						{
							GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
								LastErrorCode,"WAIT_FIRST_FRAME_STATUS");
							MessageToShowOnStopScreen = 0;
						}
						LastErrorCode = GBMSAPI_ERROR_CODE_NO_ERROR;
						// return to WAIT_START
						AcquisitionStatus = WAIT_START_STATUS;
						if (AcquisitionEnded == TRUE)
						{
							// check stop type
							if ((LastEventInfo & GBMSAPI_EI_STOP_TYPE) == 0)
							{
								// manual
								this->StopTypeTextBox->Text = "MANUAL STOP";
								if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
								{
									ShowRotationNotCompletedMessage(); 
								}
								else if
									(
										(OptionMask & GBMSAPI_AO_AUTOCAPTURE)								
									)
								{
									MessageBox::Show("Image capture manually stopped before automatic stop: check image quality",
										"WARNING"); 
								}
							}
							else
							{
								this->StopTypeTextBox->Text = "AUTO STOP";
								if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_TIMEOUT) != 0)
								{
									MessageBox::Show("ACQUISITION TIMEOUT HAS EXPIRED","WARNING");
								}
							}

							// If LCD supported, enable OK button on stop screen
							unsigned int ExternalEquipment = 0;
							int RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
							if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
							{
								GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
									RetVal,"WAIT_ACQUISITION_END_STATUS");
								ExternalEquipment = 0;
							}
							if ((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
							{
								if (
									(DisplayOptionMask & GBMSAPI_DO_FINAL_SCREEN) != 0 // stop screen enabled after acquisition
								)
								{
									RetVal = GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
									if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
									{
										GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
											RetVal,"WAIT_ACQUISITION_END_STATUS");
									}
									// show message
									if (MessageToShowOnStopScreen != 0)
									{
										RetVal = GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(
											MessageToShowOnStopScreen,
											GBMSAPI_VILCD_MSG_AREA);
										if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
										{
											GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
												RetVal,"WAIT_ACQUISITION_END_STATUS");
										}
									}
									else
									{
										RetVal = GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(
											"ERROR IN ACQUISITION",
											GBMSAPI_VILCD_CUSTOM_MSG_TYPE_ERROR_MSG);
										if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
										{
											GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
												RetVal,"WAIT_ACQUISITION_END_STATUS");
										}
									}
								}
								else
								{
									RetVal = GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();
									if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
									{
										GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
											RetVal,"WAIT_ACQUISITION_END_STATUS");
									}
								}
							}

							AcquisitionEnded = FALSE;
						}
					}
					else if (AcquisitionButtonPressed == TRUE)
					{								
						// VER2500
						if (OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP && 
							(IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan))))
						{
							int RetStopPreview = GBMSAPI_ROLL_StopPreview();
							if (RetStopPreview != GBMSAPI_ERROR_CODE_NO_ERROR)
							{
								GBMSAPI_StopAcquisition();
							}
						}
						else
						{
							GBMSAPI_StopAcquisition();
						}
						//GBMSAPI_StopAcquisition();
						this->StartStopButton->Enabled = false;
						AcquisitionButtonPressed = FALSE;
					}
					else
					{
						if (BusyFrame == TRUE)
						{
							UInt32 NextState;
							// a valid frame has been acquired
							// visualize frame
							// check acquisition phase
							if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0)
							{
								GBMSAPI_Example_Util::CopyRawImageIntoBitmap(
									(IntPtr)AcquisitionPreviewBuffer,PreviewImage);

								// ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForPreview;
								this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForPreview;
								// end ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Image = PreviewImage;
								this->PreviewLabel->Visible = true;
								NextState = WAIT_PREVIEW_END_STATUS;
							}
							else
							{
								GBMSAPI_Example_Util::CopyRawImageIntoBitmap(
									(IntPtr)AcquisitionFullResBuffer,FullResImage);

								// ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForAcq;
								this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForAcq;
								// end ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Image = FullResImage;
								this->PreviewLabel->Visible = false;
								NextState = WAIT_ACQUISITION_END_STATUS;
							}
							// check image resolution
							if (((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0))
							{
								if (
									((LastEventInfo & GBMSAPI_EI_PREVIEW_RES) == 0)
									)
								{
									this->ImageResTextBox->Text = "LOW PREV RES";
								}
								else
								{
									this->ImageResTextBox->Text = "FULL PREV RES";
								}
							}
							else
							{
								this->ImageResTextBox->Text = "NA";
							}

							// show diagnostic
							this->DiagnosticMessageListBox->Items->Clear();
							if (LastDiagnosticValue != 0)
							{
								List<String ^> ^DiagList = GBMSAPI_Example_Util::GetDiagStringsFromDiagMask(LastDiagnosticValue);
								if ((DiagList != nullptr) && (DiagList->Count > 0))
								{
									for (int i = 0; i < DiagList->Count; i ++)
									{
										this->DiagnosticMessageListBox->Items->Add(DiagList[i]);
									}
								}
							}

							this->ImSXTextBox->Text = "" + LastFrameSizeX;
							this->ImSYTextBox->Text = "" + LastFrameSizeY;
							this->CurrentFrameRateTextBox->Text = "" + LastFrameCurrentFrameRate.ToString("F2");
							this->NominalFrameRateTextBox->Text = "" + LastFrameNominalFrameRate.ToString("F2");
							this->AutoCapturePhaseTextBox->Text = 
								GBMSAPI_Example_Util::GetAutoCapturePhaseStringFromValue(AutoCapturePhase);
							if (AutoCapturePhase == GBMSAPI_ACP_BEST_IMAGE_STOP)
							{
								NextState = WAIT_ACQUISITION_END_STATUS;
								this->StartStopButton->Enabled = FALSE;
							}
							this->ContrastTextBox->Text = "" + ImageContrast;
							this->SizeTextBox->Text = "" + ImageSize;
							// ver 3.1.0.1
							this->tbDryAreaPercent->Text = "" + DryFingerPercent;
							this->tbWetAreaPercent->Text = "" + WetFingerPercent;
							// end ver 3.1.0.1
							this->HLPCompletenessTextBox->Text = "" + HalfLowerPalmCompleteness;

							BusyFrame = FALSE;

							// switch to next state
							AcquisitionStatus = NextState;
						}
					}

					this->Update();
					break;
				}

			case WAIT_PREVIEW_END_STATUS:
				{
					this->WaitFirstFrameLabel->Visible = false;
					this->PreviewLabel->Visible = true;
					this->StartStopButton->Text = "Stop Preview";

					unsigned int ExternalEquipment = 0;
					int RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
							RetVal,"WAIT_ACQUISITION_END_STATUS");
						ExternalEquipment = 0;
					}

					// check errors
					if (LastErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
							LastErrorCode,"WAIT_PREVIEW_END_STATUS");
						// return to WAIT_START
						LastErrorCode = GBMSAPI_ERROR_CODE_NO_ERROR;
						AcquisitionStatus = WAIT_START_STATUS;
						if ((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
						{
							if (
								(DisplayOptionMask & GBMSAPI_DO_FINAL_SCREEN) != 0 // stop screen enabled after acquisition
							)
							{
								RetVal = GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
								if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
								{
									GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
										RetVal,"WAIT_PREVIEW_END_STATUS");
								}
								// show message
								RetVal = GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(
									"ERROR IN ACQUISITION",
									GBMSAPI_VILCD_CUSTOM_MSG_TYPE_ERROR_MSG);
								if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
								{
									GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
										RetVal,"WAIT_PREVIEW_END_STATUS");
								}
							}
							else
							{
								RetVal = GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();
								if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
								{
									GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
										RetVal,"WAIT_PREVIEW_END_STATUS");
								}
							}
						}
					}
					else if (AcquisitionButtonPressed == TRUE)
					{
						// VER2500
						if ((OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP) && 
							(IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
						)
						{
							int RetStopPreview = GBMSAPI_ROLL_StopPreview();
							if (RetStopPreview != GBMSAPI_ERROR_CODE_NO_ERROR)
							{
								GBMSAPI_StopAcquisition();
								AcquisitionStatus = WAIT_ACQUISITION_END_STATUS;
								this->StartStopButton->Enabled = false;
							}
						}
						else
						{
							GBMSAPI_StopAcquisition();
							AcquisitionStatus = WAIT_ACQUISITION_END_STATUS;
							this->StartStopButton->Enabled = false;									
						}
						//GBMSAPI_StopAcquisition();
						AcquisitionButtonPressed = FALSE;
						
					}
					else
					{
						if (BusyFrame == TRUE)
						{
							UInt32 NextState;
							// a valid frame has been acquired
							// visualize frame
							// check acquisition phase
							if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0)
							{
								GBMSAPI_Example_Util::CopyRawImageIntoBitmap(
									(IntPtr)AcquisitionPreviewBuffer,PreviewImage);

								// ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForPreview;
								this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForPreview;
								// end ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Image = PreviewImage;
								this->PreviewLabel->Visible = true;
								NextState = WAIT_PREVIEW_END_STATUS;
							}
							else
							{
								GBMSAPI_Example_Util::CopyRawImageIntoBitmap(
									(IntPtr)AcquisitionFullResBuffer,FullResImage);

								// ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForAcq;
								this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForAcq;
								// end ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Image = FullResImage;
								this->PreviewLabel->Visible = false;
								NextState = WAIT_ACQUISITION_END_STATUS;
							}
							// check image resolution
							if (((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0))
							{
								if (
									((LastEventInfo & GBMSAPI_EI_PREVIEW_RES) == 0)
									)
								{
									this->ImageResTextBox->Text = "LOW PREV RES";
								}
								else
								{
									this->ImageResTextBox->Text = "FULL PREV RES";
								}
							}
							else
							{
								this->ImageResTextBox->Text = "NA";
							}

							// show diagnostic
							this->DiagnosticMessageListBox->Items->Clear();
							if (LastDiagnosticValue != 0)
							{
								List<String ^> ^DiagList = GBMSAPI_Example_Util::GetDiagStringsFromDiagMask(LastDiagnosticValue);
								if ((DiagList != nullptr) && (DiagList->Count > 0))
								{
									for (int i = 0; i < DiagList->Count; i ++)
									{
										this->DiagnosticMessageListBox->Items->Add(DiagList[i]);
									}
								}
							}

							this->ImSXTextBox->Text = "" + LastFrameSizeX;
							this->ImSYTextBox->Text = "" + LastFrameSizeY;
							this->CurrentFrameRateTextBox->Text = "" + LastFrameCurrentFrameRate.ToString("F2");
							this->NominalFrameRateTextBox->Text = "" + LastFrameNominalFrameRate.ToString("F2");
							this->AutoCapturePhaseTextBox->Text = 
								GBMSAPI_Example_Util::GetAutoCapturePhaseStringFromValue(AutoCapturePhase);
							
							if (AutoCapturePhase == GBMSAPI_ACP_BEST_IMAGE_STOP)
							{
								NextState = WAIT_ACQUISITION_END_STATUS;
								this->StartStopButton->Enabled = FALSE;
							}
							this->ContrastTextBox->Text = "" + ImageContrast;
							this->SizeTextBox->Text = "" + ImageSize;
							// ver 3.1.0.1
							this->tbDryAreaPercent->Text = "" + DryFingerPercent;
							this->tbWetAreaPercent->Text = "" + WetFingerPercent;
							// end ver 3.1.0.1
							this->HLPCompletenessTextBox->Text = "" + HalfLowerPalmCompleteness;

							// check acquisition end
							if (AcquisitionEnded == TRUE)
							{
								// return to WAIT_START
								NextState = WAIT_START_STATUS;
								// check stop type
								if ((LastEventInfo & GBMSAPI_EI_STOP_TYPE) == 0)
								{
									// manual
									this->StopTypeTextBox->Text = "MANUAL STOP";
									if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
									{
										if ((OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP) == 0)
											ShowRotationNotCompletedMessage(); 
									}
									else if
										(
											(OptionMask & GBMSAPI_AO_AUTOCAPTURE)									
										)
									{
										MessageBox::Show("Image capture manually stopped before automatic stop: check image quality",
											"WARNING"); 
									}
								}
								else
								{
									this->StopTypeTextBox->Text = "AUTO STOP";
									if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_TIMEOUT) != 0)
									{
										MessageBox::Show("ACQUISITION TIMEOUT HAS EXPIRED","WARNING");
									}
								}

								// If LCD supported, enable OK button on stop screen									
								if ((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
								{
									// Ver 2.8.0.0
									// Intermediate limit enabling
									unsigned int LcdFeatures;
									RetVal = GBMSAPI_VUI_LCD_GetLcdFeatures(& LcdFeatures);
									if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
									{
										GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
											RetVal,"WAIT_ACQUISITION_END_STATUS,GBMSAPI_VUI_LCD_GetLcdFeatures");
									}
									if (
										(DisplayOptionMask & GBMSAPI_DO_FINAL_SCREEN) != 0 // stop screen enabled after acquisition
									)
									{
										// set image evaluation parameters
										if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
										{
											// ARTEFACTS
											RetVal = GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(
												7,
												GBMSAPI_VILCD_SMALLER_THAN_LIMIT_DIRECTION
												);
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,
													"WAIT_ACQUISITION_END_STATUS,GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen");
											}
											if ((LcdFeatures & GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT))
											{
												RetVal = GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(
													14
													);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,
														"WAIT_ACQUISITION_END_STATUS,\
														GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen");
												}
											}

											int ArtefactSize = (int)(RolledArtefactSize);

											RetVal = GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(ArtefactSize);
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
											/////////////////////
											// QUALITY
											/////////////////////
											// Quality parameter should be got by using the GBNFIQ or the
											// GBFINIMG libraries (FULL-ENHANCED package), not included in
											// the basic version of the GBMSAPI. Therefore quality will
											// be calculated by dividing contrast value by 2,56, in order
											// to have a value ranging from 0 (lowest) to 100 (highest)
											RetVal = GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(
												75,
												GBMSAPI_VILCD_GREATER_THAN_LIMIT_DIRECTION
												);
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
											if ((LcdFeatures & GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT))
											{
												RetVal = GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(
													60
													);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,
														"WAIT_ACQUISITION_END_STATUS,\
														GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen");
												}
											}
											BYTE QualityToShow = (BYTE)(((double)ImageContrast)/2.56);
											RetVal = GBMSAPI_VUI_LCD_SetQualityOnStopScreen(QualityToShow);
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
										}
										else if ((GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
										{
											// CONTRAST
											RetVal = GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(
												ContrastLimitToDisplay
												);
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
											if ((LcdFeatures & GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT))
											{
												RetVal = GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(
													60
													);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,
														"WAIT_ACQUISITION_END_STATUS,\
														GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen");
												}
											}
											RetVal = GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(ImageContrast);
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
											// COMPLETENESS
											if ((GBMSAPI_GetTypeFromObject(ObjToScan) == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM))
											{
												RetVal = GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(
													CompletenessLimitToDisplay
													);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}
												if ((LcdFeatures & GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT))
												{
													RetVal = GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(
														50
														);
													if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
													{
														GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
															RetVal,
															"WAIT_ACQUISITION_END_STATUS,\
															GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen");
													}
												}
												RetVal = GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(HalfLowerPalmCompleteness);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}
											}
										}

										RetVal = GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
										if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
										{
											GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
												RetVal,"WAIT_PREVIEW_END_STATUS");
										}
										// show message
										RetVal = GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(
											GBMSAPI_VILCD_MSG_ACQUISITION_SUCCESSFUL,
											GBMSAPI_VILCD_MSG_AREA);
										if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
										{
											GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
												RetVal,"WAIT_PREVIEW_END_STATUS");
										}
									}
									else
									{
										RetVal = GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();
										if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
										{
											GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
												RetVal,"WAIT_PREVIEW_END_STATUS");
										}
									}
								}

								AcquisitionEnded = FALSE;
							}

							BusyFrame = FALSE;

							// switch to next state
							AcquisitionStatus = NextState;
						}
					}

					this->Update();
					break;
				}

			case WAIT_ACQUISITION_END_STATUS:
				{
					this->WaitFirstFrameLabel->Visible = false;
					this->PreviewLabel->Visible = false;
					this->StartStopButton->Text = "Stop";

					unsigned int ExternalEquipment = 0;
					int RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
							RetVal,"WAIT_ACQUISITION_END_STATUS");
						ExternalEquipment = 0;
					}

					// check errors
					if (LastErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
							LastErrorCode,"WAIT_ACQUISITION_END_STATUS");
						// return to WAIT_START
						LastErrorCode = GBMSAPI_ERROR_CODE_NO_ERROR;
						AcquisitionStatus = WAIT_START_STATUS;
						if ((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
						{
							if (
								(DisplayOptionMask & GBMSAPI_DO_FINAL_SCREEN) != 0 // stop screen enabled after acquisition
							)
							{
								RetVal = GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
								if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
								{
									GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
										RetVal,"WAIT_ACQUISITION_END_STATUS");
								}
								// show message
								RetVal = GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(
									"ERROR IN ACQUISITION",
									GBMSAPI_VILCD_CUSTOM_MSG_TYPE_ERROR_MSG);
								if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
								{
									GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
										RetVal,"WAIT_ACQUISITION_END_STATUS");
								}
							}
							else
							{
								RetVal = GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();
								if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
								{
									GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
										RetVal,"WAIT_ACQUISITION_END_STATUS");
								}
							}
						}
					}
					else if (AcquisitionButtonPressed == TRUE)
					{
						// VER2500
						if (OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP && 
							IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
						{
							int RetStopPreview = GBMSAPI_ROLL_StopPreview();
							if (RetStopPreview != GBMSAPI_ERROR_CODE_NO_ERROR)
							{
								GBMSAPI_StopAcquisition();
							}
						}
						else
						{
							GBMSAPI_StopAcquisition();
						}
						//GBMSAPI_StopAcquisition();
						this->StartStopButton->Enabled = false;
						AcquisitionButtonPressed = FALSE;
					}
					else
					{
						if (BusyFrame == TRUE)
						{
							UInt32 NextState;
							// a valid frame has been acquired
							// visualize frame
							// check acquisition phase
							if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0)
							{
								// HEAVY ERROR: CANNOT SWITCH FROM ACQUISITION IN PROGRESS BACK TO
								// PREVIEW PHASE
								GBMSAPI_Example_Util::CopyRawImageIntoBitmap(
									(IntPtr)AcquisitionPreviewBuffer,PreviewImage);

								// ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForPreview;
								this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForPreview;
								// end ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Image = PreviewImage;
								this->PreviewLabel->Visible = true;
								GBMSAPI_StopAcquisition();
								NextState = WAIT_ACQUISITION_END_STATUS;
							}
							else
							{
								GBMSAPI_Example_Util::CopyRawImageIntoBitmap(
									(IntPtr)AcquisitionFullResBuffer,FullResImage);

								// ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Location = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxLocationForAcq;
								this->AcquiredImagePictureBox->Size = GBMSAPI_Cpp_Example_Globals::AcqWinPaintBoxSizeForAcq;
								// end ver 3.1.0.0 size and location for paint box
								this->AcquiredImagePictureBox->Image = FullResImage;
								this->PreviewLabel->Visible = false;
								NextState = WAIT_ACQUISITION_END_STATUS;
							}
							// check image resolution
							if (((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0))
							{
								if (
									((LastEventInfo & GBMSAPI_EI_PREVIEW_RES) == 0)
									)
								{
									this->ImageResTextBox->Text = "LOW PREV RES";
								}
								else
								{
									this->ImageResTextBox->Text = "FULL PREV RES";
								}
							}
							else
							{
								this->ImageResTextBox->Text = "NA";
							}

							// show diagnostic
							this->DiagnosticMessageListBox->Items->Clear();
							if (LastDiagnosticValue != 0)
							{
								List<String ^> ^DiagList = GBMSAPI_Example_Util::GetDiagStringsFromDiagMask(LastDiagnosticValue);
								if ((DiagList != nullptr) && (DiagList->Count > 0))
								{
									for (int i = 0; i < DiagList->Count; i ++)
									{
										this->DiagnosticMessageListBox->Items->Add(DiagList[i]);
									}
								}
							}

							this->ImSXTextBox->Text = "" + LastFrameSizeX;
							this->ImSYTextBox->Text = "" + LastFrameSizeY;
							this->CurrentFrameRateTextBox->Text = "" + LastFrameCurrentFrameRate.ToString("F2");
							this->NominalFrameRateTextBox->Text = "" + LastFrameNominalFrameRate.ToString("F2");
							this->AutoCapturePhaseTextBox->Text = 
								GBMSAPI_Example_Util::GetAutoCapturePhaseStringFromValue(AutoCapturePhase);
							if (AutoCapturePhase == GBMSAPI_ACP_BEST_IMAGE_STOP)
							{
								NextState = WAIT_ACQUISITION_END_STATUS;
								this->StartStopButton->Enabled = FALSE;
							}
							this->ContrastTextBox->Text = "" + ImageContrast;
							this->SizeTextBox->Text = "" + ImageSize;
							// ver 3.1.0.1
							this->tbDryAreaPercent->Text = "" + DryFingerPercent;
							this->tbWetAreaPercent->Text = "" + WetFingerPercent;
							// end ver 3.1.0.1
							this->HLPCompletenessTextBox->Text = "" + HalfLowerPalmCompleteness;

							// check acquisition end
							if (AcquisitionEnded == TRUE)
							{
								// return to WAIT_START
								NextState = WAIT_START_STATUS;
								// check stop type
								if ((LastEventInfo & GBMSAPI_EI_STOP_TYPE) == 0)
								{
									// manual
									this->StopTypeTextBox->Text = "MANUAL STOP";
									if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
									{
										if ((OptionMask & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP) == 0)
											ShowRotationNotCompletedMessage();  
									}
									else if
										(
											(OptionMask & GBMSAPI_AO_AUTOCAPTURE)									
										)
									{
										MessageBox::Show("Image capture manually stopped before automatic stop: check image quality",
											"WARNING"); 
									}
								}
								else
								{
									this->StopTypeTextBox->Text = "AUTO STOP";
									if ((LastEventInfo & GBMSAPI_EI_ACQUISITION_TIMEOUT) != 0)
									{
										MessageBox::Show("ACQUISITION TIMEOUT HAS EXPIRED","WARNING");
									}
								}

								DrawRollInfo = true;
								this->ArtefactsSizeTextBox->Text = "" + RolledArtefactSize;

								// If LCD supported, enable OK button on stop screen
								if ((ExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
								{
									try
									{
										if (
											(DisplayOptionMask & GBMSAPI_DO_FINAL_SCREEN) != 0 // stop screen enabled after acquisition
										)
										{
											// show message
											RetVal = GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(
												GBMSAPI_VILCD_MSG_ACQUISITION_SUCCESSFUL,
												GBMSAPI_VILCD_MSG_AREA);
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
											// set image evaluation parameters
											if (IsRolled(GBMSAPI_GetTypeFromObject(ObjToScan)))
											{
												// ARTEFACTS
												RetVal = GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(
													7,
													GBMSAPI_VILCD_SMALLER_THAN_LIMIT_DIRECTION
													);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}

												int ArtefactSize = (int)(RolledArtefactSize);

												RetVal = GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(ArtefactSize);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}
												/////////////////////
												// QUALITY
												/////////////////////
												// Quality parameter should be got by using the GBNFIQ or the
												// GBFINIMG libraries (FULL-ENHANCED package), not included in
												// the basic version of the GBMSAPI. Therefore quality will
												// be calculated by dividing contrast value by 2,56, in order
												// to have a value ranging from 0 (lowest) to 100 (highest)
												RetVal = GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(
													75,
													GBMSAPI_VILCD_GREATER_THAN_LIMIT_DIRECTION
													);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}
												BYTE QualityToShow = (BYTE)(((double)ImageContrast)/2.56);
												RetVal = GBMSAPI_VUI_LCD_SetQualityOnStopScreen(QualityToShow);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}
											}
											else if ((GBMSAPI_GetTypeFromObject(ObjToScan) != GBMSAPI_SBT_PHOTO))
											{
												// CONTRAST
												RetVal = GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(
													ContrastLimitToDisplay
													);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}
												RetVal = GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(ImageContrast);
												if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
												{
													GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
														RetVal,"WAIT_ACQUISITION_END_STATUS");
												}
												// COMPLETENESS
												if ((GBMSAPI_GetTypeFromObject(ObjToScan) == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM))
												{
													RetVal = GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(
														CompletenessLimitToDisplay
														);
													if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
													{
														GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
															RetVal,"WAIT_ACQUISITION_END_STATUS");
													}
													RetVal = GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(HalfLowerPalmCompleteness);
													if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
													{
														GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
															RetVal,"WAIT_ACQUISITION_END_STATUS");
													}
												}
											}

											// enable OK button on stop screen
											RetVal = GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
										}
										else
										{
											RetVal = GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();
											if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
											{
												GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
													RetVal,"WAIT_ACQUISITION_END_STATUS");
											}
										}
									}
									catch (Exception ^ex)
									{
										MessageBox::Show("Exception in WAIT_ACQUISITION_END_STATUS: " + ex->Message);
										return;
									}
								}

								AcquisitionEnded = FALSE;
							}

							BusyFrame = FALSE;

							// switch to next state
							AcquisitionStatus = NextState;
						}
					}

					this->Update();
					break;
				}
			}
		
		}			
		catch (Exception ^ex)
		{
			MessageBox::Show("Exception in AcquisitionStateManagement: " + ex->Message);
			GBMSAPI_StopAcquisition();
			this->Close();
		}
	}
}