#pragma once
#include "windows.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "GBMSAPIExampleMainForm.h"
#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
#include "GBMSAPIExample_Util.h"
#include "AcquisitionCallback.h"
#include "DSBeep.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Runtime::InteropServices;

///////////////////////////////////////////////////
// GLOBALS FOR INFO EXCHANGING WITH THE CALLBACK
///////////////////////////////////////////////////

/*************************************************
* Acquisition Options
*************************************************/
extern unsigned int ObjToScan;
extern unsigned int OptionMask;
extern unsigned int DisplayOptionMask;
extern unsigned char ContrastLimitToDisplay;
extern unsigned char CompletenessLimitToDisplay;
extern unsigned int ClipRegionW;
extern unsigned int ClipRegionH;
extern unsigned int RollPreviewTimeout;
extern unsigned char ArtefactCleaningDepth;
extern unsigned int PreviewImSX,PreviewImSY,FullResImSX,FullResImSY;
extern unsigned char ContrastIntermediateLimitToDisplay;
extern unsigned char CompletenessIntermediateLimitToDisplay;

// ver 2.10.0.0
extern unsigned int AcquisitionScanArea;
extern bool IsRollAreaStandardGa;
// end ver 2.10.0.0
// ver 3.2.0.0
extern bool RollBlockCompositionIsEnabled;
// end ver 3.2.0.0
extern bool UseImageFinalization;

/*************************************************
* Acquisition State Indicators
*************************************************/
extern unsigned int AcquisitionStatus;
extern unsigned int PreviousAcquisitionStatus;
extern BOOL AcquisitionButtonPressed;
extern BOOL ScannerStarted;
extern BOOL AcquisitionEnded;
extern BOOL BusyFrame;

/**************************************************
* Acquired Image Info
**************************************************/
extern int LastErrorCode;
extern unsigned int LastEventInfo;
extern int LastFrameSizeX;
extern int LastFrameSizeY;
extern double LastFrameCurrentFrameRate;
extern double LastFrameNominalFrameRate;
extern unsigned int LastDiagnosticValue;
extern int AutoCapturePhase;
extern int ClippingRegionPosX, ClippingRegionPosY;
extern unsigned int ClippingRegionSizeX, ClippingRegionSizeY;
extern unsigned int	RolledArtefactSize;
extern unsigned char *MarkerFrame;
extern unsigned char *NotWipedArtefactFrame;
extern unsigned int FlatFingerprintSize;
extern unsigned char ImageContrast;
extern unsigned int ImageSize;
extern unsigned char HalfLowerPalmCompleteness;
extern BOOL SkipRequested;
extern BOOL PreviewEnded;

extern unsigned long AcquiredFramesNumber, LostFramesNumber;
// ver 3.1.0.1
extern unsigned char DryFingerPercent;
extern unsigned char WetFingerPercent;
// end ver 3.1.0.1

/*************************************************
* Acquisition Buffers
*************************************************/
extern unsigned char *AcquisitionPreviewBuffer;
extern unsigned char *AcquisitionFullResBuffer;
extern unsigned char *ClippedBuffer;


namespace GBMSAPICExample {

	/// <summary>
	/// Summary for AcquisitionForm
	///
	/// </summary>
	public ref class AcquisitionForm : public System::Windows::Forms::Form
	{
	public:
		AcquisitionForm(void)
		{
			InitializeComponent();
			//
			//TODO: Add the constructor code here
			//
			// Normally the other constructor should be used
			this->Close();
		}		

	private: System::Windows::Forms::Timer^  AcquisitionManagementGeneralTimer;
	private: System::Windows::Forms::Button^  StartStopButton;
	private: System::Windows::Forms::PictureBox^  AcquiredImagePictureBox;
	private: System::Windows::Forms::ListBox^  DiagnosticMessageListBox;
	private: System::Windows::Forms::Label^  WaitFirstFrameLabel;
	private: System::Windows::Forms::Label^  PreviewLabel;
	private: System::Windows::Forms::GroupBox^  groupBox1;
	private: System::Windows::Forms::Label^  label1;
	private: System::Windows::Forms::TextBox^  ImageResTextBox;
	private: System::Windows::Forms::TextBox^  StopTypeTextBox;
	private: System::Windows::Forms::Label^  label2;
	private: System::Windows::Forms::TextBox^  ImSYTextBox;

	private: System::Windows::Forms::Label^  label3;
	private: System::Windows::Forms::TextBox^  ImSXTextBox;

	private: System::Windows::Forms::Label^  label4;
	private: System::Windows::Forms::TextBox^  NominalFrameRateTextBox;
	private: System::Windows::Forms::Label^  label5;
	private: System::Windows::Forms::TextBox^  CurrentFrameRateTextBox;
	private: System::Windows::Forms::Label^  label6;
	private: System::Windows::Forms::GroupBox^  groupBox2;
	private: System::Windows::Forms::Label^  label7;
	private: System::Windows::Forms::Label^  label11;
	private: System::Windows::Forms::Label^  label10;




	private: System::Windows::Forms::Label^  FullResolutionPreviewAcceptedLabel;
	private: System::Windows::Forms::Label^  AcquireFlatObjectOnRollAreaAcceptedLabel;




	private: System::Windows::Forms::Label^  AutoCaptureAccettedLabel;
	private: System::Windows::Forms::TextBox^  ObjectNameTextBox;
	private: System::Windows::Forms::TextBox^  AutoCapturePhaseTextBox;
	private: System::Windows::Forms::Label^  label12;
	private: System::Windows::Forms::TextBox^  ContrastTextBox;
	private: System::Windows::Forms::Label^  label13;
	private: System::Windows::Forms::TextBox^  SizeTextBox;
	private: System::Windows::Forms::Label^  label14;
	private: System::Windows::Forms::TextBox^  ArtefactsSizeTextBox;
	private: System::Windows::Forms::Label^  label15;
	private: System::Windows::Forms::TextBox^  HLPCompletenessTextBox;

	private: System::Windows::Forms::Label^  label16;

	private: System::Windows::Forms::Button^  AcceptButton;

	private: System::Windows::Forms::Label^  label18;
	private: System::Windows::Forms::Label^  label17;
	private: System::Windows::Forms::Panel^  panel1;
	private: System::Windows::Forms::TextBox^  RollPreviewModeTextBox;

	private: System::Windows::Forms::Label^  label20;
	private: System::Windows::Forms::TextBox^  AcquiredFramesNumTextBox;
	private: System::Windows::Forms::Label^  label8;
	private: System::Windows::Forms::TextBox^  LostFramesNumTextBox;
	private: System::Windows::Forms::Label^  label9;
	private: System::Windows::Forms::CheckBox^  cbExcludeFinalization;
	private: System::Windows::Forms::Button^  bUpdateBackgroundImage;
	private: System::Windows::Forms::TextBox^  tbDryAreaPercent;
	private: System::Windows::Forms::Label^  label19;
	private: System::Windows::Forms::TextBox^  tbWetAreaPercent;
	private: System::Windows::Forms::Label^  label21;






	private: System::ComponentModel::IContainer^  components;

	public:
		AcquisitionForm(GBMSAPIExampleMainForm ^CallingWindow);
	protected:
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		~AcquisitionForm()
		{
			GBMSAPI_StopAcquisition();

			this->AcquisitionManagementGeneralTimer->Stop();
			if (components)
			{
				delete components;
			}

			GBMSAPI_VUI_LCD_SetLogoScreen();
		}

		/// <summary>
		/// Required designer variable.
		/// </summary>
	protected:
		GBMSAPIExampleMainForm	^AcquisitionFormCallingWindow;
		static Bitmap ^PreviewImage;
		static Bitmap ^FullResImage;
		Boolean DrawRollInfo;
		System::DateTime DelayBeforeStartTimer;
		Boolean FirstStart;

	public:
		static const UInt32 WAIT_START_STATUS = 0x00000001;
		static const UInt32 WAIT_SCANNER_STATUS = 0x00000002;
		static const UInt32 WAIT_FIRST_FRAME_STATUS = 0x00000004;
		static const UInt32 WAIT_PREVIEW_END_STATUS = 0x00000008;
		static const UInt32 WAIT_ACQUISITION_END_STATUS = 0x00000010;

	protected:
		
		


#pragma region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->components = (gcnew System::ComponentModel::Container());
			System::ComponentModel::ComponentResourceManager^  resources = (gcnew System::ComponentModel::ComponentResourceManager(AcquisitionForm::typeid));
			this->AcquiredImagePictureBox = (gcnew System::Windows::Forms::PictureBox());
			this->DiagnosticMessageListBox = (gcnew System::Windows::Forms::ListBox());
			this->AcquisitionManagementGeneralTimer = (gcnew System::Windows::Forms::Timer(this->components));
			this->StartStopButton = (gcnew System::Windows::Forms::Button());
			this->WaitFirstFrameLabel = (gcnew System::Windows::Forms::Label());
			this->PreviewLabel = (gcnew System::Windows::Forms::Label());
			this->groupBox1 = (gcnew System::Windows::Forms::GroupBox());
			this->AcquiredFramesNumTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label8 = (gcnew System::Windows::Forms::Label());
			this->LostFramesNumTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label9 = (gcnew System::Windows::Forms::Label());
			this->HLPCompletenessTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label16 = (gcnew System::Windows::Forms::Label());
			this->ArtefactsSizeTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label15 = (gcnew System::Windows::Forms::Label());
			this->SizeTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label14 = (gcnew System::Windows::Forms::Label());
			this->ContrastTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label13 = (gcnew System::Windows::Forms::Label());
			this->AutoCapturePhaseTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label12 = (gcnew System::Windows::Forms::Label());
			this->NominalFrameRateTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label5 = (gcnew System::Windows::Forms::Label());
			this->CurrentFrameRateTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label6 = (gcnew System::Windows::Forms::Label());
			this->ImSYTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label3 = (gcnew System::Windows::Forms::Label());
			this->ImSXTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label4 = (gcnew System::Windows::Forms::Label());
			this->StopTypeTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label2 = (gcnew System::Windows::Forms::Label());
			this->ImageResTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label1 = (gcnew System::Windows::Forms::Label());
			this->groupBox2 = (gcnew System::Windows::Forms::GroupBox());
			this->RollPreviewModeTextBox = (gcnew System::Windows::Forms::TextBox());
			this->label20 = (gcnew System::Windows::Forms::Label());
			this->label18 = (gcnew System::Windows::Forms::Label());
			this->label17 = (gcnew System::Windows::Forms::Label());
			this->FullResolutionPreviewAcceptedLabel = (gcnew System::Windows::Forms::Label());
			this->AcquireFlatObjectOnRollAreaAcceptedLabel = (gcnew System::Windows::Forms::Label());
			this->AutoCaptureAccettedLabel = (gcnew System::Windows::Forms::Label());
			this->label11 = (gcnew System::Windows::Forms::Label());
			this->label10 = (gcnew System::Windows::Forms::Label());
			this->label7 = (gcnew System::Windows::Forms::Label());
			this->panel1 = (gcnew System::Windows::Forms::Panel());
			this->ObjectNameTextBox = (gcnew System::Windows::Forms::TextBox());
			this->AcceptButton = (gcnew System::Windows::Forms::Button());
			this->cbExcludeFinalization = (gcnew System::Windows::Forms::CheckBox());
			this->bUpdateBackgroundImage = (gcnew System::Windows::Forms::Button());
			this->tbDryAreaPercent = (gcnew System::Windows::Forms::TextBox());
			this->label19 = (gcnew System::Windows::Forms::Label());
			this->tbWetAreaPercent = (gcnew System::Windows::Forms::TextBox());
			this->label21 = (gcnew System::Windows::Forms::Label());
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->AcquiredImagePictureBox))->BeginInit();
			this->groupBox1->SuspendLayout();
			this->groupBox2->SuspendLayout();
			this->SuspendLayout();
			// 
			// AcquiredImagePictureBox
			// 
			this->AcquiredImagePictureBox->BackColor = System::Drawing::Color::White;
			this->AcquiredImagePictureBox->Location = System::Drawing::Point(12, 12);
			this->AcquiredImagePictureBox->Name = L"AcquiredImagePictureBox";
			this->AcquiredImagePictureBox->Size = System::Drawing::Size(600, 600);
			this->AcquiredImagePictureBox->TabIndex = 0;
			this->AcquiredImagePictureBox->TabStop = false;
			this->AcquiredImagePictureBox->Paint += gcnew System::Windows::Forms::PaintEventHandler(this, &AcquisitionForm::AcquiredImagePictureBox_Paint);
			// 
			// DiagnosticMessageListBox
			// 
			this->DiagnosticMessageListBox->Font = (gcnew System::Drawing::Font(L"Arial Black", 12, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->DiagnosticMessageListBox->ForeColor = System::Drawing::Color::Red;
			this->DiagnosticMessageListBox->FormattingEnabled = true;
			this->DiagnosticMessageListBox->ItemHeight = 23;
			this->DiagnosticMessageListBox->Location = System::Drawing::Point(627, 493);
			this->DiagnosticMessageListBox->Name = L"DiagnosticMessageListBox";
			this->DiagnosticMessageListBox->Size = System::Drawing::Size(483, 119);
			this->DiagnosticMessageListBox->TabIndex = 1;
			// 
			// AcquisitionManagementGeneralTimer
			// 
			this->AcquisitionManagementGeneralTimer->Interval = 5;
			this->AcquisitionManagementGeneralTimer->Tick += gcnew System::EventHandler(this, &AcquisitionForm::AcquisitionManagementGeneralTimer_Tick);
			// 
			// StartStopButton
			// 
			this->StartStopButton->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 12, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->StartStopButton->Location = System::Drawing::Point(875, 46);
			this->StartStopButton->Name = L"StartStopButton";
			this->StartStopButton->Size = System::Drawing::Size(235, 34);
			this->StartStopButton->TabIndex = 2;
			this->StartStopButton->Text = L"START";
			this->StartStopButton->UseVisualStyleBackColor = true;
			this->StartStopButton->Click += gcnew System::EventHandler(this, &AcquisitionForm::StartStopButton_Click);
			// 
			// WaitFirstFrameLabel
			// 
			this->WaitFirstFrameLabel->AutoSize = true;
			this->WaitFirstFrameLabel->BackColor = System::Drawing::Color::White;
			this->WaitFirstFrameLabel->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 22, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->WaitFirstFrameLabel->ForeColor = System::Drawing::SystemColors::ActiveCaption;
			this->WaitFirstFrameLabel->Location = System::Drawing::Point(184, 223);
			this->WaitFirstFrameLabel->Name = L"WaitFirstFrameLabel";
			this->WaitFirstFrameLabel->Size = System::Drawing::Size(233, 36);
			this->WaitFirstFrameLabel->TabIndex = 3;
			this->WaitFirstFrameLabel->Text = L"Don\'t put finger";
			this->WaitFirstFrameLabel->Visible = false;
			// 
			// PreviewLabel
			// 
			this->PreviewLabel->AutoSize = true;
			this->PreviewLabel->BackColor = System::Drawing::Color::White;
			this->PreviewLabel->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 22, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->PreviewLabel->ForeColor = System::Drawing::SystemColors::ActiveCaption;
			this->PreviewLabel->Location = System::Drawing::Point(28, 24);
			this->PreviewLabel->Name = L"PreviewLabel";
			this->PreviewLabel->Size = System::Drawing::Size(128, 36);
			this->PreviewLabel->TabIndex = 4;
			this->PreviewLabel->Text = L"Preview";
			this->PreviewLabel->Visible = false;
			// 
			// groupBox1
			// 
			this->groupBox1->Controls->Add(this->tbDryAreaPercent);
			this->groupBox1->Controls->Add(this->label19);
			this->groupBox1->Controls->Add(this->tbWetAreaPercent);
			this->groupBox1->Controls->Add(this->label21);
			this->groupBox1->Controls->Add(this->AcquiredFramesNumTextBox);
			this->groupBox1->Controls->Add(this->label8);
			this->groupBox1->Controls->Add(this->LostFramesNumTextBox);
			this->groupBox1->Controls->Add(this->label9);
			this->groupBox1->Controls->Add(this->HLPCompletenessTextBox);
			this->groupBox1->Controls->Add(this->label16);
			this->groupBox1->Controls->Add(this->ArtefactsSizeTextBox);
			this->groupBox1->Controls->Add(this->label15);
			this->groupBox1->Controls->Add(this->SizeTextBox);
			this->groupBox1->Controls->Add(this->label14);
			this->groupBox1->Controls->Add(this->ContrastTextBox);
			this->groupBox1->Controls->Add(this->label13);
			this->groupBox1->Controls->Add(this->AutoCapturePhaseTextBox);
			this->groupBox1->Controls->Add(this->label12);
			this->groupBox1->Controls->Add(this->NominalFrameRateTextBox);
			this->groupBox1->Controls->Add(this->label5);
			this->groupBox1->Controls->Add(this->CurrentFrameRateTextBox);
			this->groupBox1->Controls->Add(this->label6);
			this->groupBox1->Controls->Add(this->ImSYTextBox);
			this->groupBox1->Controls->Add(this->label3);
			this->groupBox1->Controls->Add(this->ImSXTextBox);
			this->groupBox1->Controls->Add(this->label4);
			this->groupBox1->Controls->Add(this->StopTypeTextBox);
			this->groupBox1->Controls->Add(this->label2);
			this->groupBox1->Controls->Add(this->ImageResTextBox);
			this->groupBox1->Controls->Add(this->label1);
			this->groupBox1->Location = System::Drawing::Point(627, 4);
			this->groupBox1->Name = L"groupBox1";
			this->groupBox1->Size = System::Drawing::Size(235, 475);
			this->groupBox1->TabIndex = 5;
			this->groupBox1->TabStop = false;
			this->groupBox1->Text = L"Image Info";
			// 
			// AcquiredFramesNumTextBox
			// 
			this->AcquiredFramesNumTextBox->Location = System::Drawing::Point(113, 356);
			this->AcquiredFramesNumTextBox->Name = L"AcquiredFramesNumTextBox";
			this->AcquiredFramesNumTextBox->ReadOnly = true;
			this->AcquiredFramesNumTextBox->Size = System::Drawing::Size(100, 20);
			this->AcquiredFramesNumTextBox->TabIndex = 29;
			// 
			// label8
			// 
			this->label8->AutoSize = true;
			this->label8->Location = System::Drawing::Point(7, 359);
			this->label8->Name = L"label8";
			this->label8->Size = System::Drawing::Size(86, 13);
			this->label8->TabIndex = 28;
			this->label8->Text = L"Acquired Frames";
			// 
			// LostFramesNumTextBox
			// 
			this->LostFramesNumTextBox->Location = System::Drawing::Point(112, 386);
			this->LostFramesNumTextBox->Name = L"LostFramesNumTextBox";
			this->LostFramesNumTextBox->ReadOnly = true;
			this->LostFramesNumTextBox->Size = System::Drawing::Size(100, 20);
			this->LostFramesNumTextBox->TabIndex = 27;
			// 
			// label9
			// 
			this->label9->AutoSize = true;
			this->label9->Location = System::Drawing::Point(7, 393);
			this->label9->Name = L"label9";
			this->label9->Size = System::Drawing::Size(64, 13);
			this->label9->TabIndex = 26;
			this->label9->Text = L"Lost Frames";
			// 
			// HLPCompletenessTextBox
			// 
			this->HLPCompletenessTextBox->Location = System::Drawing::Point(113, 295);
			this->HLPCompletenessTextBox->Name = L"HLPCompletenessTextBox";
			this->HLPCompletenessTextBox->ReadOnly = true;
			this->HLPCompletenessTextBox->Size = System::Drawing::Size(100, 20);
			this->HLPCompletenessTextBox->TabIndex = 21;
			// 
			// label16
			// 
			this->label16->AutoSize = true;
			this->label16->Location = System::Drawing::Point(7, 298);
			this->label16->Name = L"label16";
			this->label16->Size = System::Drawing::Size(97, 13);
			this->label16->TabIndex = 20;
			this->label16->Text = L"HLP Completeness";
			// 
			// ArtefactsSizeTextBox
			// 
			this->ArtefactsSizeTextBox->Location = System::Drawing::Point(112, 325);
			this->ArtefactsSizeTextBox->Name = L"ArtefactsSizeTextBox";
			this->ArtefactsSizeTextBox->ReadOnly = true;
			this->ArtefactsSizeTextBox->Size = System::Drawing::Size(100, 20);
			this->ArtefactsSizeTextBox->TabIndex = 19;
			// 
			// label15
			// 
			this->label15->AutoSize = true;
			this->label15->Location = System::Drawing::Point(7, 328);
			this->label15->Name = L"label15";
			this->label15->Size = System::Drawing::Size(72, 13);
			this->label15->TabIndex = 18;
			this->label15->Text = L"Artefacts Size";
			// 
			// SizeTextBox
			// 
			this->SizeTextBox->Location = System::Drawing::Point(113, 260);
			this->SizeTextBox->Name = L"SizeTextBox";
			this->SizeTextBox->ReadOnly = true;
			this->SizeTextBox->Size = System::Drawing::Size(100, 20);
			this->SizeTextBox->TabIndex = 17;
			// 
			// label14
			// 
			this->label14->AutoSize = true;
			this->label14->Location = System::Drawing::Point(7, 263);
			this->label14->Name = L"label14";
			this->label14->Size = System::Drawing::Size(27, 13);
			this->label14->TabIndex = 16;
			this->label14->Text = L"Size";
			// 
			// ContrastTextBox
			// 
			this->ContrastTextBox->Location = System::Drawing::Point(113, 227);
			this->ContrastTextBox->Name = L"ContrastTextBox";
			this->ContrastTextBox->ReadOnly = true;
			this->ContrastTextBox->Size = System::Drawing::Size(100, 20);
			this->ContrastTextBox->TabIndex = 15;
			// 
			// label13
			// 
			this->label13->AutoSize = true;
			this->label13->Location = System::Drawing::Point(7, 230);
			this->label13->Name = L"label13";
			this->label13->Size = System::Drawing::Size(46, 13);
			this->label13->TabIndex = 14;
			this->label13->Text = L"Contrast";
			// 
			// AutoCapturePhaseTextBox
			// 
			this->AutoCapturePhaseTextBox->Location = System::Drawing::Point(113, 192);
			this->AutoCapturePhaseTextBox->Name = L"AutoCapturePhaseTextBox";
			this->AutoCapturePhaseTextBox->ReadOnly = true;
			this->AutoCapturePhaseTextBox->Size = System::Drawing::Size(100, 20);
			this->AutoCapturePhaseTextBox->TabIndex = 13;
			// 
			// label12
			// 
			this->label12->AutoSize = true;
			this->label12->Location = System::Drawing::Point(7, 195);
			this->label12->Name = L"label12";
			this->label12->Size = System::Drawing::Size(102, 13);
			this->label12->TabIndex = 12;
			this->label12->Text = L"Auto-Capture Phase";
			// 
			// NominalFrameRateTextBox
			// 
			this->NominalFrameRateTextBox->Location = System::Drawing::Point(113, 162);
			this->NominalFrameRateTextBox->Name = L"NominalFrameRateTextBox";
			this->NominalFrameRateTextBox->ReadOnly = true;
			this->NominalFrameRateTextBox->Size = System::Drawing::Size(100, 20);
			this->NominalFrameRateTextBox->TabIndex = 11;
			// 
			// label5
			// 
			this->label5->AutoSize = true;
			this->label5->Location = System::Drawing::Point(7, 165);
			this->label5->Name = L"label5";
			this->label5->Size = System::Drawing::Size(103, 13);
			this->label5->TabIndex = 10;
			this->label5->Text = L"Nominal Frame Rate";
			// 
			// CurrentFrameRateTextBox
			// 
			this->CurrentFrameRateTextBox->Location = System::Drawing::Point(113, 133);
			this->CurrentFrameRateTextBox->Name = L"CurrentFrameRateTextBox";
			this->CurrentFrameRateTextBox->ReadOnly = true;
			this->CurrentFrameRateTextBox->Size = System::Drawing::Size(100, 20);
			this->CurrentFrameRateTextBox->TabIndex = 9;
			// 
			// label6
			// 
			this->label6->AutoSize = true;
			this->label6->Location = System::Drawing::Point(7, 136);
			this->label6->Name = L"label6";
			this->label6->Size = System::Drawing::Size(99, 13);
			this->label6->TabIndex = 8;
			this->label6->Text = L"Current Frame Rate";
			// 
			// ImSYTextBox
			// 
			this->ImSYTextBox->Location = System::Drawing::Point(113, 105);
			this->ImSYTextBox->Name = L"ImSYTextBox";
			this->ImSYTextBox->ReadOnly = true;
			this->ImSYTextBox->Size = System::Drawing::Size(100, 20);
			this->ImSYTextBox->TabIndex = 7;
			// 
			// label3
			// 
			this->label3->AutoSize = true;
			this->label3->Location = System::Drawing::Point(7, 108);
			this->label3->Name = L"label3";
			this->label3->Size = System::Drawing::Size(69, 13);
			this->label3->TabIndex = 6;
			this->label3->Text = L"Image Size Y";
			// 
			// ImSXTextBox
			// 
			this->ImSXTextBox->Location = System::Drawing::Point(113, 76);
			this->ImSXTextBox->Name = L"ImSXTextBox";
			this->ImSXTextBox->ReadOnly = true;
			this->ImSXTextBox->Size = System::Drawing::Size(100, 20);
			this->ImSXTextBox->TabIndex = 5;
			// 
			// label4
			// 
			this->label4->AutoSize = true;
			this->label4->Location = System::Drawing::Point(7, 79);
			this->label4->Name = L"label4";
			this->label4->Size = System::Drawing::Size(69, 13);
			this->label4->TabIndex = 4;
			this->label4->Text = L"Image Size X";
			// 
			// StopTypeTextBox
			// 
			this->StopTypeTextBox->Location = System::Drawing::Point(113, 49);
			this->StopTypeTextBox->Name = L"StopTypeTextBox";
			this->StopTypeTextBox->ReadOnly = true;
			this->StopTypeTextBox->Size = System::Drawing::Size(100, 20);
			this->StopTypeTextBox->TabIndex = 3;
			// 
			// label2
			// 
			this->label2->AutoSize = true;
			this->label2->Location = System::Drawing::Point(7, 52);
			this->label2->Name = L"label2";
			this->label2->Size = System::Drawing::Size(56, 13);
			this->label2->TabIndex = 2;
			this->label2->Text = L"Stop Type";
			// 
			// ImageResTextBox
			// 
			this->ImageResTextBox->Location = System::Drawing::Point(113, 20);
			this->ImageResTextBox->Name = L"ImageResTextBox";
			this->ImageResTextBox->ReadOnly = true;
			this->ImageResTextBox->Size = System::Drawing::Size(100, 20);
			this->ImageResTextBox->TabIndex = 1;
			// 
			// label1
			// 
			this->label1->AutoSize = true;
			this->label1->Location = System::Drawing::Point(7, 23);
			this->label1->Name = L"label1";
			this->label1->Size = System::Drawing::Size(89, 13);
			this->label1->TabIndex = 0;
			this->label1->Text = L"Image Resolution";
			// 
			// groupBox2
			// 
			this->groupBox2->Controls->Add(this->RollPreviewModeTextBox);
			this->groupBox2->Controls->Add(this->label20);
			this->groupBox2->Controls->Add(this->label18);
			this->groupBox2->Controls->Add(this->label17);
			this->groupBox2->Controls->Add(this->FullResolutionPreviewAcceptedLabel);
			this->groupBox2->Controls->Add(this->AcquireFlatObjectOnRollAreaAcceptedLabel);
			this->groupBox2->Controls->Add(this->AutoCaptureAccettedLabel);
			this->groupBox2->Controls->Add(this->label11);
			this->groupBox2->Controls->Add(this->label10);
			this->groupBox2->Controls->Add(this->label7);
			this->groupBox2->Controls->Add(this->panel1);
			this->groupBox2->Location = System::Drawing::Point(875, 171);
			this->groupBox2->Name = L"groupBox2";
			this->groupBox2->Size = System::Drawing::Size(235, 205);
			this->groupBox2->TabIndex = 6;
			this->groupBox2->TabStop = false;
			this->groupBox2->Text = L"Acquisition Options";
			// 
			// RollPreviewModeTextBox
			// 
			this->RollPreviewModeTextBox->Location = System::Drawing::Point(112, 150);
			this->RollPreviewModeTextBox->Name = L"RollPreviewModeTextBox";
			this->RollPreviewModeTextBox->ReadOnly = true;
			this->RollPreviewModeTextBox->Size = System::Drawing::Size(101, 20);
			this->RollPreviewModeTextBox->TabIndex = 23;
			// 
			// label20
			// 
			this->label20->AutoSize = true;
			this->label20->Location = System::Drawing::Point(6, 153);
			this->label20->Name = L"label20";
			this->label20->Size = System::Drawing::Size(96, 13);
			this->label20->TabIndex = 22;
			this->label20->Text = L"Roll Preview Mode";
			// 
			// label18
			// 
			this->label18->BackColor = System::Drawing::Color::Green;
			this->label18->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 10, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->label18->Location = System::Drawing::Point(10, 37);
			this->label18->Name = L"label18";
			this->label18->Size = System::Drawing::Size(142, 17);
			this->label18->TabIndex = 23;
			this->label18->Text = L"Green: Accepted";
			// 
			// label17
			// 
			this->label17->BackColor = System::Drawing::Color::White;
			this->label17->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 10, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->label17->Location = System::Drawing::Point(10, 20);
			this->label17->Name = L"label17";
			this->label17->Size = System::Drawing::Size(142, 17);
			this->label17->TabIndex = 22;
			this->label17->Text = L"White: Not requested";
			// 
			// FullResolutionPreviewAcceptedLabel
			// 
			this->FullResolutionPreviewAcceptedLabel->BackColor = System::Drawing::Color::White;
			this->FullResolutionPreviewAcceptedLabel->Location = System::Drawing::Point(203, 93);
			this->FullResolutionPreviewAcceptedLabel->Name = L"FullResolutionPreviewAcceptedLabel";
			this->FullResolutionPreviewAcceptedLabel->Size = System::Drawing::Size(10, 13);
			this->FullResolutionPreviewAcceptedLabel->TabIndex = 19;
			// 
			// AcquireFlatObjectOnRollAreaAcceptedLabel
			// 
			this->AcquireFlatObjectOnRollAreaAcceptedLabel->BackColor = System::Drawing::Color::White;
			this->AcquireFlatObjectOnRollAreaAcceptedLabel->Location = System::Drawing::Point(203, 123);
			this->AcquireFlatObjectOnRollAreaAcceptedLabel->Name = L"AcquireFlatObjectOnRollAreaAcceptedLabel";
			this->AcquireFlatObjectOnRollAreaAcceptedLabel->Size = System::Drawing::Size(10, 13);
			this->AcquireFlatObjectOnRollAreaAcceptedLabel->TabIndex = 18;
			// 
			// AutoCaptureAccettedLabel
			// 
			this->AutoCaptureAccettedLabel->BackColor = System::Drawing::Color::White;
			this->AutoCaptureAccettedLabel->Location = System::Drawing::Point(203, 65);
			this->AutoCaptureAccettedLabel->Name = L"AutoCaptureAccettedLabel";
			this->AutoCaptureAccettedLabel->Size = System::Drawing::Size(10, 13);
			this->AutoCaptureAccettedLabel->TabIndex = 17;
			// 
			// label11
			// 
			this->label11->AutoSize = true;
			this->label11->Location = System::Drawing::Point(6, 123);
			this->label11->Name = L"label11";
			this->label11->Size = System::Drawing::Size(160, 13);
			this->label11->TabIndex = 16;
			this->label11->Text = L"Acquire Flat Object On Roll Area";
			// 
			// label10
			// 
			this->label10->AutoSize = true;
			this->label10->Location = System::Drawing::Point(7, 93);
			this->label10->Name = L"label10";
			this->label10->Size = System::Drawing::Size(128, 13);
			this->label10->TabIndex = 15;
			this->label10->Text = L"Full Resolution in Preview";
			// 
			// label7
			// 
			this->label7->AutoSize = true;
			this->label7->Location = System::Drawing::Point(7, 65);
			this->label7->Name = L"label7";
			this->label7->Size = System::Drawing::Size(69, 13);
			this->label7->TabIndex = 12;
			this->label7->Text = L"Auto-Capture";
			// 
			// panel1
			// 
			this->panel1->BackColor = System::Drawing::Color::Silver;
			this->panel1->Location = System::Drawing::Point(7, 17);
			this->panel1->Name = L"panel1";
			this->panel1->Size = System::Drawing::Size(171, 37);
			this->panel1->TabIndex = 25;
			// 
			// ObjectNameTextBox
			// 
			this->ObjectNameTextBox->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 15, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->ObjectNameTextBox->Location = System::Drawing::Point(875, 12);
			this->ObjectNameTextBox->Name = L"ObjectNameTextBox";
			this->ObjectNameTextBox->Size = System::Drawing::Size(235, 30);
			this->ObjectNameTextBox->TabIndex = 7;
			this->ObjectNameTextBox->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			// 
			// AcceptButton
			// 
			this->AcceptButton->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 10, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->AcceptButton->Location = System::Drawing::Point(875, 88);
			this->AcceptButton->Name = L"AcceptButton";
			this->AcceptButton->Size = System::Drawing::Size(235, 30);
			this->AcceptButton->TabIndex = 9;
			this->AcceptButton->Text = L"ACCEPT";
			this->AcceptButton->UseVisualStyleBackColor = true;
			this->AcceptButton->Click += gcnew System::EventHandler(this, &AcquisitionForm::AcceptButton_Click);
			// 
			// cbExcludeFinalization
			// 
			this->cbExcludeFinalization->AutoSize = true;
			this->cbExcludeFinalization->Location = System::Drawing::Point(875, 393);
			this->cbExcludeFinalization->Name = L"cbExcludeFinalization";
			this->cbExcludeFinalization->Size = System::Drawing::Size(119, 17);
			this->cbExcludeFinalization->TabIndex = 10;
			this->cbExcludeFinalization->Text = L"Exclude Finalization";
			this->cbExcludeFinalization->UseVisualStyleBackColor = true;
			this->cbExcludeFinalization->CheckedChanged += gcnew System::EventHandler(this, &AcquisitionForm::cbExcludeFinalization_CheckedChanged);
			// 
			// bUpdateBackgroundImage
			// 
			this->bUpdateBackgroundImage->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 10, System::Drawing::FontStyle::Bold, 
				System::Drawing::GraphicsUnit::Point, static_cast<System::Byte>(0)));
			this->bUpdateBackgroundImage->Location = System::Drawing::Point(875, 122);
			this->bUpdateBackgroundImage->Name = L"bUpdateBackgroundImage";
			this->bUpdateBackgroundImage->Size = System::Drawing::Size(235, 45);
			this->bUpdateBackgroundImage->TabIndex = 21;
			this->bUpdateBackgroundImage->Text = L"UPDATE BACKGROUND IMAGE";
			this->bUpdateBackgroundImage->UseVisualStyleBackColor = true;
			this->bUpdateBackgroundImage->Click += gcnew System::EventHandler(this, &AcquisitionForm::bUpdateBackgroundImage_Click);
			// 
			// tbDryAreaPercent
			// 
			this->tbDryAreaPercent->Location = System::Drawing::Point(113, 419);
			this->tbDryAreaPercent->Name = L"tbDryAreaPercent";
			this->tbDryAreaPercent->ReadOnly = true;
			this->tbDryAreaPercent->Size = System::Drawing::Size(100, 20);
			this->tbDryAreaPercent->TabIndex = 33;
			// 
			// label19
			// 
			this->label19->AutoSize = true;
			this->label19->Location = System::Drawing::Point(7, 422);
			this->label19->Name = L"label19";
			this->label19->Size = System::Drawing::Size(88, 13);
			this->label19->TabIndex = 32;
			this->label19->Text = L"Dry Area Percent";
			// 
			// tbWetAreaPercent
			// 
			this->tbWetAreaPercent->Location = System::Drawing::Point(112, 449);
			this->tbWetAreaPercent->Name = L"tbWetAreaPercent";
			this->tbWetAreaPercent->ReadOnly = true;
			this->tbWetAreaPercent->Size = System::Drawing::Size(100, 20);
			this->tbWetAreaPercent->TabIndex = 31;
			// 
			// label21
			// 
			this->label21->AutoSize = true;
			this->label21->Location = System::Drawing::Point(6, 452);
			this->label21->Name = L"label21";
			this->label21->Size = System::Drawing::Size(92, 13);
			this->label21->TabIndex = 30;
			this->label21->Text = L"Wet Area Percent";
			// 
			// AcquisitionForm
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->ClientSize = System::Drawing::Size(1134, 637);
			this->Controls->Add(this->bUpdateBackgroundImage);
			this->Controls->Add(this->cbExcludeFinalization);
			this->Controls->Add(this->AcceptButton);
			this->Controls->Add(this->ObjectNameTextBox);
			this->Controls->Add(this->groupBox2);
			this->Controls->Add(this->groupBox1);
			this->Controls->Add(this->PreviewLabel);
			this->Controls->Add(this->WaitFirstFrameLabel);
			this->Controls->Add(this->StartStopButton);
			this->Controls->Add(this->DiagnosticMessageListBox);
			this->Controls->Add(this->AcquiredImagePictureBox);
			this->Icon = (cli::safe_cast<System::Drawing::Icon^  >(resources->GetObject(L"$this.Icon")));
			this->MaximizeBox = false;
			this->MinimizeBox = false;
			this->Name = L"AcquisitionForm";
			this->Text = L"Fingerprint Image Acquisition";
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->AcquiredImagePictureBox))->EndInit();
			this->groupBox1->ResumeLayout(false);
			this->groupBox1->PerformLayout();
			this->groupBox2->ResumeLayout(false);
			this->groupBox2->PerformLayout();
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion
	private:
		// General timer for window management: here images are visualized in order
		// to make the AcquisitionCallback to be as fast as possible
		System::Void AcquisitionManagementGeneralTimer_Tick(System::Object^  sender, System::EventArgs^  e);
	private:
		System::Void StartStopButton_Click(System::Object^  sender, System::EventArgs^  e);
	private:
		void AcquisitionStateManagement(void);
	private:
		System::Void AcquiredImagePictureBox_Paint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e);
	private:
		System::Void AcceptButton_Click(System::Object^  sender, System::EventArgs^  e)
		{
			this->Close();
		}

	private: 
		System::Void cbExcludeFinalization_CheckedChanged(System::Object^  sender, System::EventArgs^  e)
		{
			UseImageFinalization = !(this->cbExcludeFinalization->Checked);
		}
	private:
		System::Void bUpdateBackgroundImage_Click(System::Object^  sender, System::EventArgs^  e);
	// ver 3.1.0.0
	public:
		static System::Void AcquisitionForm::ShowRotationNotCompletedMessage();
	// end ver 3.1.0.0
};
		
	// ver 3.1.0.0
	public ref class GBMSAPI_Cpp_Example_Globals
	{
	public:
		static Size AcqWinPaintBoxOriginalSize = Size(600, 600);
		static Point AcqWinPaintBoxOriginalLocation = Point(3, 21);
		static Size AcqWinPaintBoxSizeForPreview = Size();
		static Point AcqWinPaintBoxLocationForPreview = Point();
		static Size AcqWinPaintBoxSizeForAcq = Size();
		static Point AcqWinPaintBoxLocationForAcq = Point();
	};
	// end ver 3.1.0.0
}
