#include "stdafx.h"
#include "windows.h"
#include "GBMSAPIExampleMainForm.h"
#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
#include "GBMSAPIExample_Util.h"
#include "SelectDeviceForm.h"
#include "CalibrationForm.h"
#include "VUI_LED_form.h"
#include "AcquisitionForm.h"
#include "PermanentDataStorageForm.h"



using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Collections::Generic;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;

using namespace GBMSAPICExample;

//////////////////////////////////
// GLOBALS FOR IMAGE ACQUISITION
//////////////////////////////////
extern unsigned char *AcquisitionPreviewBuffer;
extern unsigned char *AcquisitionFullResBuffer;
extern unsigned char *ClippedBuffer;

//////////////////////////////////
// GLOBALS FOR FRAME RATE SETTING
//////////////////////////////////
//extern unsigned int FrameRateOptions;

char DllNameBuffer[100];

namespace GBMSAPICExample {
	
	void GBMSAPIExampleMainForm::GBMSAPIExample_InitMainForm(void)
	{
		try
		{
			//////////////////////////////
			// DISABLE POSSIBILITY TO
			// SAVE IMAGE (we don't have
			// any image to be saved)
			//////////////////////////////
			this->SaveFileStripButton->Enabled = false;

			//////////////////////////////
			// GET LIBRARY VERSION
			//////////////////////////////
			unsigned char VersionField1,VersionField2,VersionField3,VersionField4;
			GBMSAPI_GetMultiScanAPIVersion(&VersionField1,&VersionField2,&VersionField3,&VersionField4);

			this->GBMSAPIVersionTextBox->Text = "" + VersionField1 + "." + VersionField2 + "." + VersionField3 + "." + VersionField4;

			int RetVal = GBMSAPI_GetUnderlyingLibraryDllVersion(
				&VersionField1,&VersionField2,&VersionField3,&VersionField4,
				DllNameBuffer);

			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetUnderlyingLibraryDllVersion");
				return;
			}

			this->DeviceDllNameTextBox->Text = gcnew String(DllNameBuffer)
				+ " Ver:" + VersionField1 + "." + VersionField2 + "." + VersionField3 + "." + VersionField4;

            //////////////////////////////
            // GET DEVICE STATISTICS
            //////////////////////////////
            unsigned int Counter, ProductionDateInSec;
            GBMSAPI_GetScannerStatistics(& Counter, & ProductionDateInSec);

			TimeSpan TimeFromProductionDate = TimeSpan::FromSeconds((double)ProductionDateInSec);
            DateTime Reference = DateTime(1970, 1, 1);
			DateTime ProductionDate = Reference.Add(TimeFromProductionDate);

            this->DeviceProductionDateTextBox->Text = ProductionDate.ToString();
            this->DeviceNumberOfUseTextBox->Text = "" + Counter;

            //////////////////////////////
            // GET DEVICE FW INFO
            //////////////////////////////
            wchar_t *FwInfoWCharArray = new wchar_t[GBMSAPI_MAX_CHARS_IN_DEVICE_NAME_AND_INFO];
			RetVal = GBMSAPI_GetDeviceNameAndVersion(FwInfoWCharArray);
			if (RetVal == GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED)
			{
				this->FwInfoTextBox->Text = "FEATURE NOT SUPPORTED";
			}
			else if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,
					"InitMainForm,GBMSAPI_GetDeviceNameAndVersion");
			}
			else
			{
				this->FwInfoTextBox->Text = gcnew String(FwInfoWCharArray);
			}

			

			//////////////////////////////
			// GET DEVICE FEATURES
			//////////////////////////////
			/*******************************
			* Device Features
			*******************************/
			unsigned int DeviceFeaturesMask;
			RetVal = GBMSAPI_GetDeviceFeatures(&DeviceFeaturesMask);

			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetDeviceFeatures");
				return;
			}

			// Multiple USB Connections
			if (DeviceFeaturesMask & GBMSAPI_DF_MULTIPLE_USB_CONNECTIONS_ALLOWED)
			{
				this->MultipleUSBConnectionsLabel->BackColor = Color::Green;
			}
			else
			{
				this->MultipleUSBConnectionsLabel->BackColor = Color::Red;
			}

			// 1000 DPI
			if (DeviceFeaturesMask & GBMSAPI_DF_1000DPI_RESOLUTION)
			{
				this->ResolutionLabel->Text = "Image Resolution: 1000 DPI";
			}
			else
			{
				this->ResolutionLabel->Text = "Image Resolution: 500 DPI";
			}

			// Auto capture blocking
			if (DeviceFeaturesMask & GBMSAPI_DF_AUTO_CAPTURE_BLOCKING)
			{
				this->AutoCaptureBlockLabel->BackColor = Color::Green;
			}
			else
			{
				this->AutoCaptureBlockLabel->BackColor = Color::Red;
			}

			// Image Rotation
			if (DeviceFeaturesMask & GBMSAPI_DF_IMAGE_ROTATION)
			{
				this->ImageRotationLabel->BackColor = Color::Green;
				this->ImageRotationGroupBox->Enabled = true;
				this->RotationRadioButton_0->Checked = true;
			}
			else
			{
				this->ImageRotationLabel->BackColor = Color::Red;
				this->ImageRotationGroupBox->Enabled = false;
			}

			// VER 2700
			// Permanent Data Storage
			if (DeviceFeaturesMask & GBMSAPI_DF_PERMANENT_USER_DATA_STORAGE)
			{
				this->PermanentDataStorageLabel->BackColor = Color::Green;
				this->PermanentDataStorageToolStripButton->Enabled = true;
			}
			else
			{
				this->PermanentDataStorageLabel->BackColor = Color::Red;
				this->PermanentDataStorageToolStripButton->Enabled = false;
			}

            // VER 2700
            // Frame rate setting
            if ((DeviceFeaturesMask & GBMSAPI_DF_FRAME_RATE_SETTING))
            {
				this->FrameRateSettingDFLabel->BackColor = Color::Green;
                this->FrameRateGroupBox->Enabled = true;
            }
            else
            {
                this->FrameRateSettingDFLabel->BackColor = Color::Red;
                this->FrameRateGroupBox->Enabled = false;
                this->lFrameRatePreviewRange->Text = ("");
                this->tbFrameRatePreview->Text = "";
            }

			// VER 2700
            // FW Retrieve info
			if (DeviceFeaturesMask & GBMSAPI_DF_FW_INFO_RETRIEVE)
			{
				this->FwInfoRetrievingLabel->BackColor = Color::Green;
			}
			else
			{
				this->FwInfoRetrievingLabel->BackColor = Color::Red;
			}

			// VER 2800
            if ((DeviceFeaturesMask & GBMSAPI_DF_ROLL_AREA_IQS) != 0)
            {
                this->RollIQSSupportDFLabel->BackColor = Color::Green;
            }
            else
            {
                this->RollIQSSupportDFLabel->BackColor = Color::Red;
            }

			// VER 2800
            this->RollStandardCheckBox->Enabled = false;
            if ((DeviceFeaturesMask & GBMSAPI_DF_ROLL_AREA_GA) != 0)
            {
                this->RollGASupportDFLabel->BackColor = Color::Green;
                this->RollStandardCheckBox->Enabled = true;
            }
            else
            {
                this->RollGASupportDFLabel->BackColor = Color::Red;
                this->RollStandardCheckBox->Enabled = false;
            }

			// VER 2.9.0.0
			if ((DeviceFeaturesMask & GBMSAPI_DF_DRY_SKIN_IMG_ENHANCE) != 0)
			{
				this->DrySkinEnhancementSupportDFLabel->BackColor = Color::Green;
				this->EnhanceDrySkinImgCheckBox->Enabled = true;
				this->EnhanceDrySkinImgCheckBox->Checked = false;
			}
			else
			{
				this->DrySkinEnhancementSupportDFLabel->BackColor = Color::Red;
				this->EnhanceDrySkinImgCheckBox->Enabled = false;
				this->EnhanceDrySkinImgCheckBox->Checked = false;
			}

			// VER 3.1.0.0
			if ((DeviceFeaturesMask & GBMSAPI_DF_ROLL_OBJECT_STRIPE) != 0)
			{
				this->lSliceSettingSupport->BackColor = Color::Green;
				this->gbStripeOptions->Enabled = true;
                // VER 3.1.0.1
				this->gbStripeOptions->Visible = true;
                // end VER 3.1.0.1
			}
			else
			{
				this->lSliceSettingSupport->BackColor = Color::Red;
				this->gbStripeOptions->Enabled = false;
                // VER 3.1.0.1
				this->gbStripeOptions->Visible = false;
                // end VER 3.1.0.1
			}
			// end VER 3.1.0.0

            // VER 3.2.0.0
            if ((DeviceFeaturesMask & GBMSAPI_DF_ENABLE_BLOCK_ROLL_COMPOSITION) != 0)
            {
                this->cbEnableRollCompositionBlock->Enabled = true;
            }
            else
            {
                this->cbEnableRollCompositionBlock->Enabled = false;
            }
            // end VER 3.2.0.0

			

			/*******************************
			* Supported Scan Options
			*******************************/
			unsigned int SupportedScanOptions;
			RetVal = GBMSAPI_GetSupportedScanOptions(&SupportedScanOptions);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"InitMainForm,GBMSAPI_GetSupportedScanOptions");
				return;
			}

			// Auto capture
			if (SupportedScanOptions & GBMSAPI_AO_AUTOCAPTURE)
			{
				this->AutoCaptureLabel->BackColor = Color::Green;
			}
			else
			{
				this->AutoCaptureLabel->BackColor = Color::Red;
			}

			// Full Resolution
			if (SupportedScanOptions & GBMSAPI_AO_FULL_RES_PREVIEW)
			{
				this->FullResolutionPreviewLabel->BackColor = Color::Green;
			}
			else
			{
				this->FullResolutionPreviewLabel->BackColor = Color::Red;
			}

			// Flat single finger on roll scan area
			if (SupportedScanOptions & GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA)
			{
				this->FlatSingleFingerOnRollAreaLabel->BackColor = Color::Green;
			}
			else
			{
				this->FlatSingleFingerOnRollAreaLabel->BackColor = Color::Red;
			}

			// Manual roll preview stop
			if (SupportedScanOptions & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP)
			{
				this->PedalRollPreviewStopLabel->BackColor = Color::Green;
			}
			else
			{
				this->PedalRollPreviewStopLabel->BackColor = Color::Red;
			}

			// No roll preview
			if (SupportedScanOptions & GBMSAPI_AO_NO_ROLL_PREVIEW)
			{
				this->NoRollPreviewLabel->BackColor = Color::Green;
			}
			else
			{
				this->NoRollPreviewLabel->BackColor = Color::Red;
			}

			// High Speed Preview
			if (SupportedScanOptions & GBMSAPI_AO_HIGH_SPEED_PREVIEW)
			{
				this->HighSpeedPreviewLabel->BackColor = Color::Green;
			}
			else
			{
				this->HighSpeedPreviewLabel->BackColor = Color::Red;
			}

			// VER 3.1.0.0
			if (SupportedScanOptions & GBMSAPI_AO_ADAPT_ROLL_AREA_POSITION)
			{
				this->lAutoRollPositioning->BackColor = Color::Green;
			}
			else
			{
				this->lAutoRollPositioning->BackColor = Color::Red;
			}
			
			if (SupportedScanOptions & GBMSAPI_AO_FORCE_ROLL_TO_LEFT)
			{
				this->lAoForceRollToLeft->BackColor = Color::Green;
			}
			else
			{
				this->lAoForceRollToLeft->BackColor = Color::Red;
			}
			
			if (SupportedScanOptions & GBMSAPI_AO_FORCE_ROLL_TO_RIGHT)
			{
				this->lAoForceRollToRight->BackColor = Color::Green;
			}
			else
			{
				this->lAoForceRollToRight->BackColor = Color::Red;
			}
			
			if (SupportedScanOptions & GBMSAPI_AO_EXTERNAL_ROLL_COMPOSITION)
			{
				this->lAoExternalRollProcedure->BackColor = Color::Green;
			}
			else
			{
				this->lAoExternalRollProcedure->BackColor = Color::Red;
			}
			// VER 3.1.0.0

			/*******************************
			* Image Info
			*******************************/
			unsigned int AvailableImageInfo;
			// Contrast and size: if available, they are valid for FLAT SINGLE FINGER
			RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_SBT_FLAT_SINGLE_FINGER,&AvailableImageInfo);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{				
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"InitMainForm,GBMSAPI_GetAvailableImageInfo");
				return;
			}

			if ((AvailableImageInfo & GBMSAPI_AII_FINGERPRINT_CONTRAST) != 0)
			{
				this->FPContrastAvailableLabel->BackColor = Color::Green;
			}
			else
			{
				this->FPContrastAvailableLabel->BackColor = Color::Red;
			}

			if ((AvailableImageInfo & GBMSAPI_AII_FINGERPRINT_SIZE) != 0)
			{
				this->FPSizeAvailableLabel->BackColor = Color::Green;
			}
			else
			{
				this->FPSizeAvailableLabel->BackColor = Color::Red;
			}

			// HLP Completeness: if available, it's valid for LOWER HALF PALM
			unsigned int ScannableTypesMask;
			RetVal = GBMSAPI_GetScannableTypes(&ScannableTypesMask);

			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetScannableTypes");
				return;
			}

			if (ScannableTypesMask & GBMSAPI_SBT_FLAT_LOWER_HALF_PALM)
			{
				RetVal = GBMSAPI_GetAvailableImageInfo(GBMSAPI_SBT_FLAT_LOWER_HALF_PALM,&AvailableImageInfo);
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{				
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal,"InitMainForm,GBMSAPI_GetAvailableImageInfo");
					return;
				}

				if ((AvailableImageInfo & GBMSAPI_AII_LOWER_HALF_PALM_COMPLETENESS) != 0)
				{
					this->HLPCompletenessAvailable->BackColor = Color::Green;
				}
				else
				{
					this->HLPCompletenessAvailable->BackColor = Color::Red;
				}
			}
			else
			{
				this->HLPCompletenessAvailable->BackColor = Color::Red;
			}

			/*******************************
			* External Equipment
			*******************************/
			unsigned int OptionalExternalEquipment;
			RetVal = GBMSAPI_GetOptionalExternalEquipment(&OptionalExternalEquipment);

			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetOptionalExternalEquipment");
				return;
			}

			// Indicator LEDs
			if ((OptionalExternalEquipment & GBMSAPI_OED_VUI_LED) != 0)
			{
				this->VUILEDControlStripButton->Enabled = true;
				this->IndicatorLEDAvailableLabel->BackColor = Color::Green;
			}
			else
			{
				this->VUILEDControlStripButton->Enabled = false;
				this->IndicatorLEDAvailableLabel->BackColor = Color::Red;
			}

			// Pedal
			if ((OptionalExternalEquipment & GBMSAPI_OED_PEDAL) != 0)
			{
				this->PedalAvailableLabel->BackColor = Color::Green;
			}
			else
			{
				this->PedalAvailableLabel->BackColor = Color::Red;
			}

			// Sound
			if ((OptionalExternalEquipment & GBMSAPI_OED_SOUND) != 0)
			{
				this->SoundAvailableLabel->BackColor = Color::Green;
			}
			else
			{
				this->SoundAvailableLabel->BackColor = Color::Red;
			}

			// LCD
			this->cbLcdLanguage->Items->Clear();
			if ((OptionalExternalEquipment & GBMSAPI_OED_VUI_LCD) != 0)
			{
				this->LCDAvailableLabel->BackColor = Color::Green;
				// Ver 2.8.0.0
				unsigned int LcdFeatures;
				RetVal = GBMSAPI_VUI_LCD_GetLcdFeatures(&LcdFeatures);
				if (LcdFeatures & GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT)
				{
					this->lLcdIntermediateLimitSetting->BackColor = Color::Green;
				}
				else
				{
					this->lLcdIntermediateLimitSetting->BackColor = Color::Red;
				}
				if (LcdFeatures & GBMSAPI_VILCD_LF_LANGUAGE_SETTING)
                {
					this->lLcdLanguageSetting->BackColor = Color::Green;
                    this->cbLcdLanguage->Enabled = true;
                    this->cbLcdLanguage->Items->Add("en");
                    this->cbLcdLanguage->Items->Add("zh");
                    this->cbLcdLanguage->SelectedIndex = 0;
                }
                else
                {
                    this->cbLcdLanguage->Enabled = false;
					this->lLcdLanguageSetting->BackColor = Color::Red;
                }

				GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();
				GBMSAPI_VUI_LCD_SetLogoScreen();
			}
			else
			{
				this->LCDAvailableLabel->BackColor = Color::Red;
				this->lLcdIntermediateLimitSetting->BackColor = Color::Red;
				this->lLcdLanguageSetting->BackColor = Color::Red;
			}

			//////////////////////////////////
			// GET SCANNABLE TYPES
			//////////////////////////////////
			this->ScannableObjectTypesListBox->Items->Clear();			

			List<String ^> ^ScannableTypesList = GBMSAPI_Example_Util::GetScannableTypesListFromMask(ScannableTypesMask);
			if (ScannableTypesList != nullptr && ScannableTypesList->Count > 0)
			{
				for (int i = 0; i < ScannableTypesList->Count; i ++)
				{
					this->ScannableObjectTypesListBox->Items->Add(ScannableTypesList[i]);
				}
			}

			

			/////////////////////////////////////
			// GET MAX IMAGE SIZE
			/////////////////////////////////////
			unsigned int ImgMaxSizeX,ImgMaxSizeY;
			RetVal = GBMSAPI_GetMaxImageSize(&ImgMaxSizeX,&ImgMaxSizeY);

			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetMaxImageSize");
				return;
			}

			// allocate buffers for acquisition
			AcquisitionPreviewBuffer = new unsigned char [ImgMaxSizeX * ImgMaxSizeY];
			AcquisitionFullResBuffer = new unsigned char [ImgMaxSizeX * ImgMaxSizeY];
			ClippedBuffer = new unsigned char [ImgMaxSizeX * ImgMaxSizeY];
			if (AcquisitionPreviewBuffer == NULL || 
				AcquisitionFullResBuffer == NULL ||
				ClippedBuffer == NULL)
			{
				MessageBox::Show("GBMSAPIExample_InitMainForm: Memory allocation failed","FATAL ERROR");
				return;
			}

			/////////////////////////////////////
			// GET CALIBRATION IMAGE SIZE
			/////////////////////////////////////
			// ver 2.10.0.0: allocate buffer into calibration form directly
			//unsigned int CalImgSizeX,CalImgSizeY;
			//RetVal = GBMSAPI_GetCalibrationImageSize(TRUE,&CalImgSizeX,&CalImgSizeY);

			//if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			//{
			//	GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetCalibrationImageSize");
			//	return;
			//}
			//CalibrationBuffer = new unsigned char [CalImgSizeX * CalImgSizeY];
			//DummyCalibrationBuffer = new unsigned char [CalImgSizeX * CalImgSizeY];
			//if (CalibrationBuffer == NULL || DummyCalibrationBuffer == NULL)
			//{
			//	MessageBox::Show("GBMSAPIExample_InitMainForm: Memory allocation failed","FATAL ERROR");
			//	return;
			//}
			if (CalibrationBuffer != NULL) free(CalibrationBuffer);
			if (DummyCalibrationBuffer != NULL) free(DummyCalibrationBuffer);
			// end ver 2.10.0.0: allocate buffer into calibration form directly

			// VER 2.10.0.0 Scan Areas
			//////////////////////////////
			// GET Scan Areas
			//////////////////////////////
			unsigned int ScanAreas;
			RetVal = GBMSAPI_GetSupportedScanAreas(&ScanAreas);

			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"InitMainForm,GBMSAPI_GetSupportedScanAreas");
				return;
			}

			// Full Frame
			if (ScanAreas & GBMSAPI_SA_FULL_FRAME)
			{
				this->lScanAreaFullFrame->BackColor = Color::Green;
			}
			else
			{
				this->lScanAreaFullFrame->BackColor = Color::Red;
			}

			// ROLL IQS
			if (ScanAreas & GBMSAPI_SA_ROLL_IQS)
			{
				this->lScanAreaRollIqs->BackColor = Color::Green;
			    // VER 3.1.0.0
                this->cbStripeIqsArea->Checked = false;
			    // end VER 3.1.0.0
			}
			else
			{
				this->lScanAreaRollIqs->BackColor = Color::Red;
			}

			// ROLL GA
			if (ScanAreas & GBMSAPI_SA_ROLL_GA)
			{
				this->lScanAreaRollGa->BackColor = Color::Green;
			}
			else
			{
				this->lScanAreaRollGa->BackColor = Color::Red;
			}

			// PHOTO
			if (ScanAreas & GBMSAPI_SA_PHOTO)
			{
				this->lScanAreaPhoto->BackColor = Color::Green;
			}
			else
			{
				this->lScanAreaPhoto->BackColor = Color::Red;
			}

			// ROLL JOINT
			if (ScanAreas & GBMSAPI_SA_ROLL_JOINT)
			{
				this->lScanAreaRollJoint->BackColor = Color::Green;
			    // VER 3.1.0.0
                this->cbStripeJointArea->Checked = false;
			    // end VER 3.1.0.0
			}
			else
			{
				this->lScanAreaRollJoint->BackColor = Color::Red;
			}

			// ROLL THENAR
			if (ScanAreas & GBMSAPI_SA_ROLL_THENAR)
			{
				this->lScanAreaRollThenar->BackColor = Color::Green;
			    // VER 3.1.0.0
                this->cbStripeThenarArea->Checked = false;
			    // end VER 3.1.0.0
			}
			else
			{
				this->lScanAreaRollThenar->BackColor = Color::Red;
			}
			// end VER 2.10.0.0 Scan Areas

			///////////////////////////////////////
			// LOAD OBJECTS TO SCAN LIST
			///////////////////////////////////////
			// VER 2000: clear Objects to Scan combo
			this->ObjectToScanComboBox->Items->Clear();
			// check all bits of ScannableTypesMask
			for (int i = 0; i < 32; i ++)
			{
				unsigned int Mask = ((unsigned int)1) << i;

				Mask &= ScannableTypesMask;

				if (Mask != 0)
				{
					// bit is set
					array <String ^> ^ ObjList = GBMSAPI_Example_Util::GetObjectsToScanListFromObjectType(Mask);
					for each (String ^ObjToScan in ObjList)
					{
						this->ObjectToScanComboBox->Items->Add(ObjToScan);
					}
				}
			}
			if (this->ObjectToScanComboBox->Items->Count > 0)
			{
				this->ObjectToScanComboBox->SelectedIndex = 0;
				this->ObjectToScanComboBox_SelectedIndexChanged(nullptr,nullptr);
			}
		}
		catch(Exception ^ex)
		{
			MessageBox::Show("Exception in GBMSAPIExample_InitMainForm: " + ex->Message);
		}
        finally
        {
        }
	}

	void GBMSAPIExampleMainForm::ObjectToScanComboBox_SelectedIndexChanged(System::Object^  sender, System::EventArgs^  e)
	{
		this->RefreshControls();
	}

	void GBMSAPIExampleMainForm::CalibrateDeviceToolStripButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		try
		{
			CalibrationForm ^CalWindow = gcnew CalibrationForm();
			if (CalWindow != nullptr)
			{
				CalWindow->ShowDialog();
			}
		}
		catch(Exception ^ex)
		{
			MessageBox::Show("Exception in CalibrateDeviceToolStripButton_Click: " + ex->Message);
		}
	}

	void GBMSAPIExampleMainForm::VUILEDControlStripButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		try
		{
			VUI_LED ^DlgToOpen = gcnew VUI_LED();
			if (DlgToOpen != nullptr )
			{
				DlgToOpen->ShowDialog();
			}
		}
		catch(Exception ^ex)
		{
			MessageBox::Show("Exception in VUILEDControlStripButton_Click: " + ex->Message);
		}
	}

	void GBMSAPIExampleMainForm::SaveFileStripButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		// copy image data
		unsigned char *SourcePtr = ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0) ?
			AcquisitionPreviewBuffer :
			AcquisitionFullResBuffer;

		// create bitmap
		Bitmap ^LastAcqImage = gcnew Bitmap(LastFrameSizeX,LastFrameSizeY,LastFrameSizeX,
			System::Drawing::Imaging::PixelFormat::Format8bppIndexed,
			(IntPtr)SourcePtr);

		Imaging::ColorPalette ^pal = LastAcqImage->Palette;
		for (int i = 0; i < pal->Entries->Length; i++)
		{
			pal->Entries[i] = Color::FromArgb(255, i, i, i);
		}
		LastAcqImage->Palette = pal;

		LastAcqImage->Save("FullResImage.bmp",Imaging::ImageFormat::Bmp);


		// check image resolution
		unsigned int DevFeatures;
		int RetVal = GBMSAPI_GetDeviceFeatures(&DevFeatures);
		if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
		{
			GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
				RetVal,"SaveFileStripButton_Click,GBMSAPI_GetDeviceFeatures");
			return;
		}
		if ((DevFeatures & GBMSAPI_DF_1000DPI_RESOLUTION) != 0)
		{
			System::Windows::Forms::DialogResult res =
			MessageBox::Show("Saved Image is in 1000 dpi resolution: would you like to save it at a 500 dpi also?",
				"1000 DPI option",
				MessageBoxButtons::YesNo);

			if (res == System::Windows::Forms::DialogResult::Yes)
			{
				//////////////////
				// save in 500 dpi
				//////////////////
				unsigned char *DestinationPointer = ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0) ?
					AcquisitionFullResBuffer:
					AcquisitionPreviewBuffer;

				// convert to 500 DPI
				int RetVal = GBMSAPI_ConvertImageFrom1000to500DPI(
					SourcePtr,
					DestinationPointer,
					LastFrameSizeX,
					LastFrameSizeY
					);
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal,"SaveFileStripButton_Click,GBMSAPI_ConvertImageFrom1000to500DPI");
					return;
				}

				// create bitmap
				Bitmap ^DecimatedAcqImage = gcnew Bitmap(LastFrameSizeX/2,LastFrameSizeY/2,LastFrameSizeX/2,
					System::Drawing::Imaging::PixelFormat::Format8bppIndexed,
					(IntPtr)DestinationPointer);

				Imaging::ColorPalette ^DestinationPal = DecimatedAcqImage->Palette;
				for (int i = 0; i < DestinationPal->Entries->Length; i++)
				{
					DestinationPal->Entries[i] = Color::FromArgb(255, i, i, i);
				}
				DecimatedAcqImage->Palette = DestinationPal;

				DecimatedAcqImage->Save("500DPIImage.bmp");
			}
		}
	}

	void GBMSAPIExampleMainForm::PermanentDataStorageToolStripButton_Click(System::Object ^sender, System::EventArgs ^e)
	{
		try
		{
			PermanentDataStorageForm ^PermWindow = gcnew PermanentDataStorageForm();
			if (PermWindow != nullptr)
			{
				PermWindow->ShowDialog();
			}
		}
		catch(Exception ^ex)
		{
			MessageBox::Show("Exception in PermanentDataStorageToolStripButton_Click: " + ex->Message);
		}
	}

	void GBMSAPIExampleMainForm::UpdateListButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		this->Enabled = false;
		Cursor->Current = Cursors::WaitCursor;
		Application::DoEvents();
		try
		{
			this->DeviceTypeComboBox->Items->Clear();
			////////////////////////////////
			///// Get device list
			////////////////////////////////
			GBMSAPI_DeviceInfoStruct AttachedDeviceList[GBMSAPI_MAX_PLUGGED_DEVICE_NUM];
			int AttachedDeviceNumber;
			unsigned long USBErrorCode;

			int RetVal = GBMSAPI_GetAttachedDeviceList(AttachedDeviceList,&AttachedDeviceNumber,&USBErrorCode);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR || AttachedDeviceNumber <= 0)
			{
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,USBErrorCode,"UpdateListButton_Click");
				}
				return;
			}

			////////////////////////////////
			//// Store device list
			////////////////////////////////
			for (int i = 0; i < AttachedDeviceNumber; i ++)
			{
				String ^StrToAdd = GBMSAPI_Example_Util::GBMSAPI_Example_GetDevNameFromDevID(
					AttachedDeviceList[i].DeviceID);
				StrToAdd += " " + gcnew String(AttachedDeviceList[i].DeviceSerialNumber);
				this->DeviceTypeComboBox->Items->Add(StrToAdd);
			}
			if (AttachedDeviceNumber > 0)
			{
				this->DeviceTypeComboBox->SelectedIndex = 0;
			}
		}
		catch (Exception ^ex)
		{
			MessageBox::Show("Error in UpdateListButton_Click function: " + ex->Message);
			this->Close();
		}
        finally
        {
            Cursor->Current = Cursors::Default;
            this->Enabled = true;
        }
	}

	void GBMSAPIExampleMainForm::SelectDeviceButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		
		String ^CurrentChoosenDev = (String ^)(this->DeviceTypeComboBox->SelectedItem);

		if (CurrentChoosenDev != nullptr)
		{
			// Parse string: <DevName><Space character><DevSerialNumber>
			int SpacePosition = CurrentChoosenDev->IndexOf(" ");
			String ^DevName = CurrentChoosenDev->Substring(0,SpacePosition);

			String ^DevSerialNumber = CurrentChoosenDev->Substring(SpacePosition + 1);

			if (DevName != nullptr && DevSerialNumber!= nullptr)
			{
				unsigned char DevID = GBMSAPI_Example_Util::GBMSAPI_Example_GetDevIDFromDevName(DevName);
				char *ANSIDevSerialNumber = (char *)(Marshal::StringToHGlobalAnsi(DevSerialNumber)).ToPointer();

				if (DevID != 0)
				{
					this->Enabled = false;
					Cursor->Current = Cursors::WaitCursor;
					Application::DoEvents();
					int RetVal = GBMSAPI_SetCurrentDevice(DevID,ANSIDevSerialNumber);

					Cursor->Current = Cursors::Default;
					this->Enabled = true;

					if (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						this->GBMSAPIExample_InitMainForm();
					}
					else
					{
						GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"SelectDeviceForm OK Button");
					}
				}
			}
		}

		
	}

	void GBMSAPIExampleMainForm::PedalRollPreviewStopModeRadioButton_CheckedChanged(System::Object^  sender, System::EventArgs^  e)
	{
		if (this->PedalRollPreviewStopModeRadioButton->Checked == true)
		{
			this->AutomaticRollPreviewTimeoutTextBox->Enabled = false;
		}
		else
		{
			this->AutomaticRollPreviewTimeoutTextBox->Enabled = true;
		}
	}

	void GBMSAPIExampleMainForm::ClipEnableCheckBox_CheckedChanged(System::Object^  sender, System::EventArgs^  e)
	{
		if (this->ClipEnableCheckBox->Checked == true)
		{
			this->ClipRegionSizeXTextBox->Enabled = true;
			this->ClipRegionSizeYTextBox->Enabled = true;
		}
		else
		{
			this->ClipRegionSizeXTextBox->Enabled = false;
			this->ClipRegionSizeYTextBox->Enabled = false;
		}
	}

	void GBMSAPIExampleMainForm::ClipRegionSizeXTextBox_Leave(System::Object^  sender, System::EventArgs^  e)
	{
		try
		{
			int ClSX = Int32::Parse(this->ClipRegionSizeXTextBox->Text);
			if (ClSX < 200)
			{
				MessageBox::Show("Min Clip Size X is 200: changing it to the min value","WARNING");
				this->ClipRegionSizeXTextBox->Text = "200";
			}
		}
		catch (Exception ^)
		{
		}
	}

	void GBMSAPIExampleMainForm::ClipRegionSizeYTextBox_Leave(System::Object^  sender, System::EventArgs^  e) {
		try
		{
			int ClSY = Int32::Parse(this->ClipRegionSizeYTextBox->Text);
			if (ClSY < 200)
			{
				MessageBox::Show("Min Clip Size Y is 200: changing it to the min value","WARNING");
				this->ClipRegionSizeYTextBox->Text = "200";
			}
		}
		catch (Exception ^)
		{
		}
	}

	void GBMSAPIExampleMainForm::NoRollPreviewCheckBox_EnabledChanged(System::Object^  sender, System::EventArgs^  e)
	{
		if (this->NoRollPreviewCheckBox->Enabled == false)
		{
			this->RollPreviewOptionsGroupBox->Enabled = true;
            // ver 3.1.0.0
			unsigned int SupportedScanOptions;
			GBMSAPI_GetSupportedScanOptions(&SupportedScanOptions);
            this->cbRollAutoPositioningInPreview->Enabled =
                ((SupportedScanOptions & GBMSAPI_AO_ADAPT_ROLL_AREA_POSITION)? true : false);
            this->cbExternalRollProcedure->Enabled =
                ((SupportedScanOptions & GBMSAPI_AO_EXTERNAL_ROLL_COMPOSITION)? true : false);
            // end ver 3.1.0.0
		}
	}

	void GBMSAPIExampleMainForm::NoRollPreviewCheckBox_CheckedChanged(System::Object^  sender, System::EventArgs^  e)
	{
		if (this->NoRollPreviewCheckBox->Checked == false)
		{
			this->RollPreviewOptionsGroupBox->Enabled = true;
            // ver 3.1.0.0
			unsigned int SupportedScanOptions;
			GBMSAPI_GetSupportedScanOptions(&SupportedScanOptions);
            this->cbRollAutoPositioningInPreview->Enabled =
                ((SupportedScanOptions & GBMSAPI_AO_ADAPT_ROLL_AREA_POSITION)? true : false);
            this->cbExternalRollProcedure->Enabled =
                ((SupportedScanOptions & GBMSAPI_AO_EXTERNAL_ROLL_COMPOSITION)? true : false);
            // end ver 3.1.0.0
		}
		else
		{
			this->RollPreviewOptionsGroupBox->Enabled = false;
            // ver 3.1.0.0
            this->cbExternalRollProcedure->Enabled = false;
            this->cbExternalRollProcedure->Checked = false;
            this->cbRollAutoPositioningInPreview->Enabled = false;
            this->cbRollAutoPositioningInPreview->Checked = false;
            // end ver 3.1.0.0
		}
	}

	System::Void GBMSAPIExampleMainForm::FullRresolutionInPreviewCheckBox_CheckedChanged(System::Object^  sender, System::EventArgs^  e)
	{
		RefreshControls();
	}

	System::Void GBMSAPIExampleMainForm::LoadFrameRateRange()
	{
		double prevMaxFR, prevMinFR, prevDefFR, fullresMaxFR, fullresMinFR, fullresDefFR;

        prevMaxFR = prevMinFR = prevDefFR = fullresMaxFR = fullresMinFR = fullresDefFR = 0;
		///////////////////////////
		// Load Options
		///////////////////////////
        unsigned int ObjToScanType, scanArea, AcqOptionMask, FrameRateOptions;
        if (GetAcquisitionSettingsOptions(& ObjToScanType, & AcqOptionMask, & scanArea, &FrameRateOptions))
        {
            ///////////////////////////
            // Get Frame Rate Range
            ///////////////////////////
			unsigned char devId; 
			char devSerNum[GBMSAPI_MAX_SN_LENGHT];
			int RetVal = GBMSAPI_GetCurrentDevice(&devId, devSerNum);
            if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
            {
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                    RetVal, "LoadFrameRateRange,GBMSAPI_GetCurrentDevice");
                this->lFrameRatePreviewRange->Text = ("");
                this->tbFrameRatePreview->Text = "";
                return;
            }
			FrameRateOptions &= ~(GBMSAPI_FRO_FULL_RESOLUTION_MODE);
            RetVal = GBMSAPI_GetFrameRateRange2(
                devId, FrameRateOptions, scanArea, & prevMaxFR, & prevMinFR, & prevDefFR
                );
            if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
            {
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                    RetVal, "LoadFrameRateRange,GBMSAPI_GetFrameRateRange3");
                this->lFrameRatePreviewRange->Text = ("");
                this->tbFrameRatePreview->Text = "";
                return;
            }
			FrameRateOptions |= (GBMSAPI_FRO_FULL_RESOLUTION_MODE);
            RetVal = GBMSAPI_GetFrameRateRange2(
                devId, FrameRateOptions, scanArea, & fullresMaxFR,
                & fullresMinFR, & fullresDefFR
                );
            if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
            {
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                    RetVal, "LoadFrameRateRange,GBMSAPI_GetFrameRateRange3");
                this->lFrameRatePreviewRange->Text = ("");
                this->tbFrameRatePreview->Text = "";
                return;
            }
            
            this->lFrameRatePreviewRange->Text = ("" + prevMinFR +
                    " < FR < " + prevMaxFR);
            this->tbFrameRatePreview->Text = "" + prevDefFR;
            this->lFrameRateFullResRange->Text = ("" + fullresMinFR +
                    " < FR < " + fullresMaxFR);
            this->tbFrameRateFullRes->Text = "" + fullresDefFR;
        }
	}

	System::Void GBMSAPIExampleMainForm::SetFrameRateButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		
		if (this->tbFrameRatePreview->Enabled)
        {
            double prevMaxFR, prevMinFR, prevDefFR, fullresMaxFR, fullresMinFR, fullresDefFR;

            prevMaxFR = prevMinFR = prevDefFR = fullresMaxFR = fullresMinFR = fullresDefFR = 0;
            ///////////////////////////
            // Load Options
            ///////////////////////////
            unsigned int ObjToScanType, scanArea, AcqOptionMask, FrameRateOptions;
			if (GetAcquisitionSettingsOptions(& ObjToScanType, & AcqOptionMask, & scanArea, &FrameRateOptions))
            {
                ///////////////////////////
                // Get Frame Rate Range
                ///////////////////////////
				unsigned char devId; 
				char devSerNum[GBMSAPI_MAX_SN_LENGHT];
				int RetVal = GBMSAPI_GetCurrentDevice(&devId, devSerNum);
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal, "SetFrameRateButton_Click,GBMSAPI_GetCurrentDevice");
					this->lFrameRatePreviewRange->Text = ("");
					this->tbFrameRatePreview->Text = "";
					return;
				}
				FrameRateOptions &= ~(GBMSAPI_FRO_FULL_RESOLUTION_MODE);
				RetVal = GBMSAPI_GetFrameRateRange2(
					devId, FrameRateOptions, scanArea, & prevMaxFR, & prevMinFR, & prevDefFR
					);
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal, "LoadFrameRateRange,GBMSAPI_GetFrameRateRange3");
					this->lFrameRatePreviewRange->Text = ("");
					this->tbFrameRatePreview->Text = "";
					return;
				}
				FrameRateOptions |= (GBMSAPI_FRO_FULL_RESOLUTION_MODE);
				RetVal = GBMSAPI_GetFrameRateRange2(
					devId, FrameRateOptions, scanArea, & fullresMaxFR,
					& fullresMinFR, & fullresDefFR
					);
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal, "LoadFrameRateRange,GBMSAPI_GetFrameRateRange3");
					this->lFrameRatePreviewRange->Text = ("");
					this->tbFrameRatePreview->Text = "";
					return;
				}

                Double previewFrameRate, fullresFrameRate;
				if (!double::TryParse(this->tbFrameRatePreview->Text, previewFrameRate))
                {
					MessageBox::Show("Preview Frame Rate Value must be a number");
                    this->tbFrameRatePreview->Text = "";
                    return;
                }
                if (previewFrameRate > prevMaxFR || previewFrameRate < prevMinFR)
                {
                    MessageBox::Show("Preview Frame Rate Value out of range");
                    this->tbFrameRatePreview->Text = "";
                    return;
                }
                if (!double::TryParse(this->tbFrameRateFullRes->Text, fullresFrameRate))
                {
                    MessageBox::Show("Full Resolution Frame Rate Value must be a number");
                    this->tbFrameRateFullRes->Text = "";
                    return;
                }
                if (fullresFrameRate > fullresMaxFR || fullresFrameRate < fullresMinFR)
                {
                    MessageBox::Show("Full Resolution Frame Rate Value out of range");
                    this->tbFrameRateFullRes->Text = "";
                    return;
                }

				FrameRateOptions &= ~(GBMSAPI_FRO_FULL_RESOLUTION_MODE);
                RetVal = GBMSAPI_SetFrameRate2(
					scanArea, FrameRateOptions, previewFrameRate);
                if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
                {
                    GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                        RetVal, "SetFrameRateButton_Click,GBMSAPI_NET_SetFrameRate3");
                    this->tbFrameRatePreview->Text = "";
                    this->tbFrameRateFullRes->Text = "";
                    return;
                }
				FrameRateOptions |= (GBMSAPI_FRO_FULL_RESOLUTION_MODE);
                RetVal = GBMSAPI_SetFrameRate2(
					scanArea, FrameRateOptions, fullresFrameRate);
                if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
                {
                    GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                        RetVal, "SetFrameRateButton_Click,GBMSAPI_NET_SetFrameRate3");
                    this->tbFrameRatePreview->Text = "";
                    this->tbFrameRateFullRes->Text = "";
                    return;
                }
                MessageBox::Show("Frame Rate correctly set");
            }
            else
            {
                MessageBox::Show("Cannot load options for frame rate setting");
            }
        }
	}

	System::Void GBMSAPIExampleMainForm::FlatSingleFingerOnRollAreaCheckBox_CheckedChanged(System::Object^  sender, System::EventArgs^  e) 
	{
		RefreshControls();
	}
	
	System::Void GBMSAPIExampleMainForm::RotationRadioButton_0_CheckedChanged(
			System::Object^  sender, System::EventArgs^  e)
	{
		if (this->RotationRadioButton_0->Checked == true)
		{
			int RetVal = GBMSAPI_SetImageRotation(GBMSAPI_IMG_ROTATE_NO);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"RotationRadioButton_0_CheckedChanged");
			}
		}
	}
	
	System::Void GBMSAPIExampleMainForm::RotationRadioButton_90_CheckedChanged(
			System::Object^  sender, System::EventArgs^  e)
	{
		if (this->RotationRadioButton_90->Checked == true)
		{
			int RetVal = GBMSAPI_SetImageRotation(GBMSAPI_IMG_ROTATE_90_CW);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"RotationRadioButton_90_CheckedChanged");
			}
		}
	}
	
	System::Void GBMSAPIExampleMainForm::RotationRadioButton_180_CheckedChanged(
			System::Object^  sender, System::EventArgs^  e)
	{
		if (this->RotationRadioButton_180->Checked == true)
		{
			int RetVal = GBMSAPI_SetImageRotation(GBMSAPI_IMG_ROTATE_180);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"RotationRadioButton_180_CheckedChanged");
			}
		}
	}
	
	System::Void GBMSAPIExampleMainForm::RotationRadioButton_270_CheckedChanged(
			System::Object^  sender, System::EventArgs^  e)
	{
		if (this->RotationRadioButton_270->Checked == true)
		{
			int RetVal = GBMSAPI_SetImageRotation(GBMSAPI_IMG_ROTATE_90_CCW);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"RotationRadioButton_270_CheckedChanged");
			}
		}
	}

	System::Void GBMSAPIExampleMainForm::SelectImageTimeoutTextBox_Leave(
		System::Object^  sender, System::EventArgs^  e)
	{
		try
		{
			UInt32 TimeoutToSelect = UInt32::Parse(this->SelectImageTimeoutTextBox->Text);

			int RetVal = GBMSAPI_SetSelectImageTimeout(TimeoutToSelect);
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"SelectImageTimeoutTextBox_Leave");
			}
		}
		catch (Exception ^)
		{
		}
	}

	System::Void GBMSAPIExampleMainForm::RollStandardCheckBox_CheckedChanged(System::Object^  sender, System::EventArgs^  e)
	{
		RefreshControls();
	}

	System::Void GBMSAPIExampleMainForm::RollStandardCheckBox_EnabledChanged(System::Object^  sender, System::EventArgs^  e)
	{
        if (this->RollStandardCheckBox->Enabled)
        {
            this->RollStandardCheckBox->Checked = true;
        }
	}

	System::Void GBMSAPIExampleMainForm::cbLcdLanguage_SelectedIndexChanged(System::Object^  sender, System::EventArgs^  e)
	{
		try
        {
            String ^LanguageToSet = (String^)this->cbLcdLanguage->SelectedItem;
            if (LanguageToSet != nullptr)
            {
				char *ANSILanguageToSet = (char *)(Marshal::StringToHGlobalAnsi(LanguageToSet)).ToPointer();
                GBMSAPI_VUI_LCD_SetLanguage(ANSILanguageToSet);
            }
        }
        catch (Exception ^ex)
        {
			MessageBox::Show("Exception in cbLcdLanguage_SelectedIndexChanged: " + ex->Message);
        }
	}

	// VER 2.9.0.0
	System::Void GBMSAPIExampleMainForm::EnhanceDrySkinImgCheckBox_CheckedChanged(System::Object^  sender, System::EventArgs^  e)
	{
		try
		{
			if (this->EnhanceDrySkinImgCheckBox->Checked)
			{
				GBMSAPI_EnableDrySkinImgEnhance(TRUE);
			}
			else
			{
				GBMSAPI_EnableDrySkinImgEnhance(FALSE);
			}
		}
        catch (Exception ^ex)
        {
			MessageBox::Show("Exception in EnhanceDrySkinImgCheckBox_CheckedChanged: " + ex->Message);
        }
	}

	// VER 3.1.0.0
	System::Void GBMSAPIExampleMainForm::bEnableRollStripeAcquisition_Click(System::Object^  sender, System::EventArgs^  e)
	{		
        int RetVal;
        ///////////////////////////////////
        // IQS AREA
        ///////////////////////////////////
        if (this->cbStripeIqsArea->Checked)
            RetVal = GBMSAPI_ROLL_EnableRollStripeAcquisition(GBMSAPI_SA_ROLL_IQS, TRUE);
        else
            RetVal = GBMSAPI_ROLL_EnableRollStripeAcquisition(GBMSAPI_SA_ROLL_IQS, FALSE);
        if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
        {
			GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                RetVal, "bEnableRollStripeAcquisition_Click,GBMSAPI_NET_ROLL_EnableRollStripeAcquisition");
            return;
        }
        ///////////////////////////////////
        // JOINT AREA
        ///////////////////////////////////
        if (this->cbStripeJointArea->Checked)
            RetVal = GBMSAPI_ROLL_EnableRollStripeAcquisition(GBMSAPI_SA_ROLL_JOINT, TRUE);
        else
            RetVal = GBMSAPI_ROLL_EnableRollStripeAcquisition(GBMSAPI_SA_ROLL_JOINT, FALSE);
        if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
        {
			GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                RetVal, "bEnableRollStripeAcquisition_Click,GBMSAPI_NET_ROLL_EnableRollStripeAcquisition");
            return;
        }
        ///////////////////////////////////
        // THENAR AREA
        ///////////////////////////////////
        if (this->cbStripeThenarArea->Checked)
            RetVal = GBMSAPI_ROLL_EnableRollStripeAcquisition(GBMSAPI_SA_ROLL_THENAR, TRUE);
        else
            RetVal = GBMSAPI_ROLL_EnableRollStripeAcquisition(GBMSAPI_SA_ROLL_THENAR, FALSE);
        if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
        {
			GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
                RetVal, "bEnableRollStripeAcquisition_Click,GBMSAPI_NET_ROLL_EnableRollStripeAcquisition");
            return;
        }
		MessageBox::Show("Done");
	}

	bool GBMSAPIExampleMainForm::GetOptionMask(unsigned int *AcqOptionMask, unsigned int *FrameRateOptions)
    {
        *AcqOptionMask = 0;
		unsigned int fro = 0;
        // Auto-Capture
        if (this->AutoCaptureCheckBox->Enabled == true &&
            this->AutoCaptureCheckBox->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_AUTOCAPTURE;
        }

        // No roll preview stop
        if (this->NoRollPreviewCheckBox->Enabled == true &&
            this->NoRollPreviewCheckBox->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_NO_ROLL_PREVIEW;
        }

        // Ver 3->1->0->0
        if (this->cbRollAutoPositioningInPreview->Enabled == true &&
            this->cbRollAutoPositioningInPreview->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_ADAPT_ROLL_AREA_POSITION;
			(fro) |= GBMSAPI_FRO_ADAPT_ROLL_AREA_POSITION;
        }
        if (this->cbExternalRollProcedure->Enabled == true &&
            this->cbExternalRollProcedure->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_EXTERNAL_ROLL_COMPOSITION;
        }
        if (this->gbRollDirection->Enabled)
        {
            if (this->rbForceRollToLeft->Checked)
            {
                (*AcqOptionMask) |= GBMSAPI_AO_FORCE_ROLL_TO_LEFT;
            }
            else if (this->rbForceRollToRight->Checked)
            {
                (*AcqOptionMask) |= GBMSAPI_AO_FORCE_ROLL_TO_RIGHT;
            }
        }
        // end Ver 3->1->0->0

        // Roll Preview stop mode
        if (this->PedalRollPreviewStopModeRadioButton->Enabled == true &&
            this->PedalRollPreviewStopModeRadioButton->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP;
        }

        // Roll automatic preview type: only if automatic roll preview mode
        if (this->CenterRollPreviewStopModeRadioButton->Enabled == true &&
            this->CenterRollPreviewStopModeRadioButton->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_ROLL_PREVIEW_TYPE;
        }

        // Full res preview image resolution
        if (this->FullRresolutionInPreviewCheckBox->Enabled == true &&
            this->FullRresolutionInPreviewCheckBox->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_FULL_RES_PREVIEW;
			(fro) |= GBMSAPI_FRO_HIGH_RES_IN_PREVIEW;
        }

        // VER 2600:
        // check HighSpeed Preview
        if (this->HighSpeedCheckBox->Enabled == true &&
            this->HighSpeedCheckBox->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_HIGH_SPEED_PREVIEW;
        }

        // Flat object on roll scan area
        if (this->FlatSingleFingerOnRollAreaCheckBox->Enabled == true &&
            this->FlatSingleFingerOnRollAreaCheckBox->Checked == true)
        {
            (*AcqOptionMask) |= GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA;
        }
		if (FrameRateOptions != NULL) *FrameRateOptions = fro;

        return true;
    }

	bool GBMSAPIExampleMainForm::GetAcquisitionSettingsOptions(unsigned int * ObjectToScanType, 
		unsigned int *  AcqOptionMask, unsigned int *  scanArea, unsigned int *FrameRateOptions)
        {
            *AcqOptionMask = 0;
            *scanArea = 0;
            *ObjectToScanType = GBMSAPI_SBT_NO_OBJECT_TYPE;
			*FrameRateOptions = 0;

            String ^ObjToScanName = (String^)(this->ObjectToScanComboBox->SelectedItem);
			unsigned int ObjToScan = GBMSAPI_Example_Util::GetObjectToScanIDFromName(ObjToScanName);
            *ObjectToScanType = GBMSAPI_GetTypeFromObject(ObjToScan);
            if (ObjectToScanType != GBMSAPI_SBT_NO_OBJECT_TYPE)
            {
                /*******************************
                 * Get Scan Area
                 * ****************************/
                bool IsUsedGA = this->RollStandardCheckBox->Enabled && this->RollStandardCheckBox->Checked;
                bool IsFlatOnRoll = this->FlatSingleFingerOnRollAreaCheckBox->Enabled &&
                    this->FlatSingleFingerOnRollAreaCheckBox->Checked;
				*scanArea = GBMSAPI_Example_Util::GetScanAreaFromObjectType((*ObjectToScanType),
                    IsFlatOnRoll,
                    IsUsedGA
                );
                if ((*scanArea) == 0) return false;

                return GetOptionMask(AcqOptionMask, FrameRateOptions);
            }
            return false;
        }

	System::Void GBMSAPIExampleMainForm::RefreshControls()
	{		
		try
		{			
			unsigned int ObjToScanType, scanArea, AcqOptionMask, FrameRateOptions;
            if (GetAcquisitionSettingsOptions(& ObjToScanType, & AcqOptionMask,& scanArea, &FrameRateOptions))
            {
                ///////////////////////////////
				// LOAD IMAGE SIZE
				///////////////////////////////
				this->ObjToScanImageSizeListBox->Items->Clear();
                unsigned int  PreviewImgMaxSizeX, PreviewImgMaxSizeY;
                unsigned int  FullResImgMaxSizeX, FullResImgMaxSizeY;

                /*******************************
                 * Get image size
                 * ****************************/
				int RetVal = GBMSAPI_GetImageSize3(ObjToScanType,AcqOptionMask,scanArea,
					&FullResImgMaxSizeX,&FullResImgMaxSizeY,
					&PreviewImgMaxSizeX,&PreviewImgMaxSizeY
					);
                // end ver 3.1.0.0
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal,"ObjectToScanComboBox_SelectedIndexChanged,GBMSAPI_GetImageSize3");
					return;
				}
				this->ObjToScanImageSizeListBox->Items->Add(
					gcnew String("PREVIEW SIZE X = " + PreviewImgMaxSizeX + "; PREVIEW SIZE Y = " + PreviewImgMaxSizeY)
					);
				this->ObjToScanImageSizeListBox->Items->Add(
					gcnew String("FULL SIZE X = " + FullResImgMaxSizeX + "; FULL SIZE Y = " + FullResImgMaxSizeY)
					);

				////////////////////////////////////
				// LOAD SCAN OPTIONS
				////////////////////////////////////
				unsigned int SupportedScanOptions;
				RetVal = GBMSAPI_GetSupportedScanOptions(&SupportedScanOptions);
				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal,"ObjectToScanComboBox_SelectedIndexChanged,GBMSAPI_GetSupportedScanOptions");
					return;
				}

				this->HighSpeedCheckBox->Enabled = (SupportedScanOptions & GBMSAPI_AO_HIGH_SPEED_PREVIEW)? true : false;	

				// check if Roll or Flat GroupBox should be enabled
				if (IsRolled(ObjToScanType))
				{
					this->FlatObjectOptionsGroupBox->Enabled = false;
					this->RollObjectOptionsGroupBox->Enabled = true;

					// no roll preview
					this->NoRollPreviewCheckBox->Enabled = 
						(SupportedScanOptions & GBMSAPI_AO_NO_ROLL_PREVIEW)? true : false;
					this->NoRollPreviewCheckBox->Checked = false;

                    // Ver 3.1.0.0: Roll Autoposition
                    this->cbRollAutoPositioningInPreview->Enabled =
                        ((SupportedScanOptions & GBMSAPI_AO_ADAPT_ROLL_AREA_POSITION)? true : false);
                    this->cbExternalRollProcedure->Enabled =
                        ((SupportedScanOptions & GBMSAPI_AO_EXTERNAL_ROLL_COMPOSITION)? true : false);
					this->gbRollDirection->Enabled = 
						((SupportedScanOptions & GBMSAPI_AO_FORCE_ROLL_TO_LEFT)? true : false);
                    // end Ver 3.1.0.0: Roll Autoposition

					// manual roll preview stop mode
					this->PedalRollPreviewStopModeRadioButton->Enabled = 
						(SupportedScanOptions & GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP)? true : false;
					this->PedalRollPreviewStopModeRadioButton->Checked = false;

					this->SideRollPreviewStopModeRadioButton->Enabled = true;
					this->CenterRollPreviewStopModeRadioButton->Enabled = true;

					// by default, side roll preview is enabled
					this->SideRollPreviewStopModeRadioButton->Checked = true;
				}
				else
				{
					this->FlatObjectOptionsGroupBox->Enabled = true;
					this->RollObjectOptionsGroupBox->Enabled = false;

					// Auto-Capture
					this->AutoCaptureCheckBox->Enabled = (SupportedScanOptions & GBMSAPI_AO_AUTOCAPTURE)? true : false;

					// Flat single finger on Roll area
					if (ObjToScanType == GBMSAPI_SBT_FLAT_SINGLE_FINGER &&
						(SupportedScanOptions & GBMSAPI_AO_FLAT_SINGLE_FINGER_ON_ROLL_AREA)
						)
					{
						this->FlatSingleFingerOnRollAreaCheckBox->Enabled = true;
					}
					else
					{
						this->FlatSingleFingerOnRollAreaCheckBox->Enabled = false;
					}
				}

                if (this->FrameRateGroupBox->Enabled)
                {
                    LoadFrameRateRange();
                }


				/////////////////////////
				// LOAD DISPLAY OPTIONS
				/////////////////////////
				unsigned int ExternalDevicesMask;
				RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalDevicesMask);

				if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
				{
					GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
						RetVal,"ObjectToScanComboBox_SelectedIndexChanged,GBMSAPI_GetOptionalExternalEquipment");
					return;
				}

				if (ExternalDevicesMask & GBMSAPI_OED_VUI_LCD)
				{
					// Ver 2.8.0.0
                    // Intermediate limit enabling
                    unsigned int LcdFeatures;
                    RetVal = GBMSAPI_VUI_LCD_GetLcdFeatures(& LcdFeatures);

					if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
							RetVal,"ObjectToScanComboBox_SelectedIndexChanged,GBMSAPI_VUI_LCD_GetLcdFeatures");
						return;
					}

					// Screen after acquisition
					this->StopScreenAfterAcquisitionCheckBox->Enabled = true;

					// contrast
					if (!IsRolled(ObjToScanType))
					{
						this->ShowCustomizedContrastCheckBox->Enabled = true;
						this->ContrastLimitTextBox->Enabled = true;
						this->ContrastLimitTextBox->Text = "120";
						if ((LcdFeatures & GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT))
						{
	                        this->tbContrastIntermediateLimit->Enabled = true;
							this->tbContrastIntermediateLimit->Text = "90";							
						}
						else
						{
	                        this->tbContrastIntermediateLimit->Enabled = false;	                        
						}
						
					}
					else
					{
						this->ShowCustomizedContrastCheckBox->Enabled = false;
						this->ContrastLimitTextBox->Enabled = false;
						this->tbContrastIntermediateLimit->Enabled = false;
					}

					// completeness
					if (ObjToScanType == GBMSAPI_SBT_FLAT_LOWER_HALF_PALM)
					{
						this->ShowCustomizedCompletenessCheckBox->Enabled = true;
						this->CompletenessLimitTextBox->Enabled = true;
						this->CompletenessLimitTextBox->Text = "90";
						if ((LcdFeatures & GBMSAPI_VILCD_LF_INTERMEDIATE_LIMIT))
						{
	                        this->tbCompleteIntermediateLimit->Enabled = true;    
							this->tbCompleteIntermediateLimit->Text = "60";							
						}
						else
						{
	                        this->tbCompleteIntermediateLimit->Enabled = false;	                        
						}
						
					}
					else
					{
						this->ShowCustomizedCompletenessCheckBox->Enabled = false;
						this->CompletenessLimitTextBox->Enabled = false;
						this->tbCompleteIntermediateLimit->Enabled = false;
					}
				}
				else
				{
					this->ShowCustomizedCompletenessCheckBox->Enabled = false;
					this->ShowCustomizedContrastCheckBox->Enabled = false;
					this->ContrastLimitTextBox->Enabled = false;
					this->CompletenessLimitTextBox->Enabled = false;
					this->StopScreenAfterAcquisitionCheckBox->Enabled = false;
					this->tbCompleteIntermediateLimit->Enabled = false;
					this->tbContrastIntermediateLimit->Enabled = false;
				}

				////////////////////////////
				// LOAD CLIP OPTIONS
				////////////////////////////
				// Clipping
				if (ObjToScanType != GBMSAPI_SBT_PHOTO)
				{
					this->ClipEnableCheckBox->Enabled = true;
				}
				else
				{
					this->ClipEnableCheckBox->Enabled = false;
				}
				this->ClipEnableCheckBox_CheckedChanged(nullptr,nullptr);

				////////////////////////////
				// LOAD DrySkin Enhance
				// option
				////////////////////////////
				this->EnhanceDrySkinImgCheckBox_CheckedChanged(nullptr, nullptr);
			}
		}
		catch(Exception ^ex)
		{
			MessageBox::Show("Exception in RefreshControls: " + ex->Message);
		}
	}
    // end VER 3.1.0.0
}