#include "stdafx.h"
#include "windows.h"
#include "GBMSAPIExampleMainForm.h"
#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
#include "GBMSAPIExample_Util.h"
#include "AcquisitionForm.h"
#include "DSBeep.h"

#define DesiredPictureBoxSize ((int)500)

namespace GBMSAPICExample {

	using namespace System;
	using namespace System::ComponentModel;
	using namespace System::Collections;
	using namespace System::Collections::Generic;
	using namespace System::Windows::Forms;
	using namespace System::Data;
	using namespace System::Drawing;

	using namespace GBMSAPICExample;

	void GBMSAPIExampleMainForm::AcquireImageButton_Click(System::Object^  sender, System::EventArgs^  e)
	{
		AcquisitionForm ^DlgToOpen = gcnew AcquisitionForm(this);
		DSInit(COINIT_APARTMENTTHREADED,FALSE);
		try
		{			
			try
			{
				if (DlgToOpen != nullptr )
				{
					DlgToOpen->ShowDialog();
				}

				// copy image data
				unsigned char *SourcePtr = ((LastEventInfo & GBMSAPI_EI_ACQUISITION_PHASE) == 0) ?
					AcquisitionPreviewBuffer :
					AcquisitionFullResBuffer;
				Bitmap ^LastAcqImage = nullptr;
				if (ClipRegionW > 0 && ClipRegionH > 0)
				{
					LastAcqImage = GBMSAPI_Example_Util::ClipFrame((IntPtr)SourcePtr,
						LastFrameSizeX,LastFrameSizeY,
						ClippingRegionPosX,ClippingRegionPosY,
						ClippingRegionSizeX,ClippingRegionSizeY,
						(IntPtr)ClippedBuffer);
				}
				else
				{
					LastAcqImage = gcnew Bitmap(LastFrameSizeX,LastFrameSizeY,LastFrameSizeX,
						System::Drawing::Imaging::PixelFormat::Format8bppIndexed,
						(IntPtr)SourcePtr);

					Imaging::ColorPalette ^pal = LastAcqImage->Palette;
					for (int i = 0; i < pal->Entries->Length; i++)
					{
						pal->Entries[i] = Color::FromArgb(255, i, i, i);
					}
					LastAcqImage->Palette = pal;
				}

				// Rescale Acquired image picture box, holding its width,
				// and varying the height (until a 300 pixel threshold, then fix width)
				this->AcquiredImagePictureBox->Width = DesiredPictureBoxSize;
				this->AcquiredImagePictureBox->Height = (int)(
					((double)(this->AcquiredImagePictureBox->Width)) *
					((double)(LastAcqImage->Height)/(double)(LastAcqImage->Width))
					);
				if (this->AcquiredImagePictureBox->Height > DesiredPictureBoxSize)
				{
					this->AcquiredImagePictureBox->Height = DesiredPictureBoxSize;
					this->AcquiredImagePictureBox->Width = (int)(
						((double)(this->AcquiredImagePictureBox->Height)) *
						((double)(LastAcqImage->Width)/(double)(LastAcqImage->Height))
						);
				}

				this->AcquiredImagePictureBox->SizeMode = PictureBoxSizeMode::StretchImage;
				this->AcquiredImagePictureBox->Image = LastAcqImage;

				this->ImSXTextBox->Text = "" + LastAcqImage->Width;
				this->ImSYTextBox->Text = "" + LastAcqImage->Height;
				this->ContrastTextBox->Text = "" + ImageContrast;
				this->SizeTextBox->Text = "" + ImageSize;
				this->ArtefactsSizeTextBox->Text = "" + RolledArtefactSize;

				// clear resources
				DlgToOpen->~AcquisitionForm();

				// enable possibility to save
				this->SaveFileStripButton->Enabled = true;

				this->Update();
			}
			catch (System::ObjectDisposedException ^ex)
			{
				// next instruction just to avoid compiler warnings
				(void)ex;
				MessageBox::Show("No scanners found or some error occurred in AcquisitionForm");
				DlgToOpen->~AcquisitionForm();
			}
		}
		catch (Exception ^ex)
		{
			MessageBox::Show("Exception in AcquireImageButton_Click: " + ex->Message);
			DlgToOpen->~AcquisitionForm();
		}
	}

	Boolean GBMSAPIExampleMainForm::GetAcquisitionOptions(
		unsigned int *ObjToScan,
		unsigned int *OptionMask,
		unsigned int *DisplayOptionMask,
		unsigned char *ContrastLimitToDisplay,
		unsigned char *CompletenessLimitToDisplay,
		unsigned int *ClipRegionW,
		unsigned int *ClipRegionH,
		unsigned int *RollPreviewTimeout,
		unsigned char *ArtefactCleaningDepth,
		unsigned char *ContrastIntermediateLimitToDisplay,
		unsigned char *CompletenessIntermediateLimitToDisplay,
		// ver 2.10.0.0
		bool *RollAreaStandardGa,
		// end ver 2.10.0.0
        // ver 3.2.0.0
        bool *EnableRollCompositionBlock
        // end ver 3.2.0.0
		)
	{
		*ObjToScan = 0;
		*OptionMask = 0;
		*DisplayOptionMask = 0;
		*ContrastLimitToDisplay = 0;
		*CompletenessLimitToDisplay = 0;
		*ClipRegionW = 0;
		*ClipRegionH = 0;
		*RollPreviewTimeout = 0;
		*ArtefactCleaningDepth = 0;
		// ver 2.10.0.0
		*RollAreaStandardGa = false;
		// end ver 2.10.0.0
        // ver 3.2.0.0
		*EnableRollCompositionBlock = false;
        // end ver 3.2.0.0

		try
		{
			// ver 2.10.0.0
			/////////////////////////////
			// ROLL AREA STANDARD
			/////////////////////////////
			if (
				this->RollStandardCheckBox->Enabled &&
				this->RollStandardCheckBox->Checked
			)
			{
				*RollAreaStandardGa = true;
			}
			else
			{
				*RollAreaStandardGa = false;
			}
			// end ver 2.10.0.0
			/////////////////////////////
			// GET OBJECT TO SCAN
			/////////////////////////////
			String ^ObjToScanName = (String ^)(this->ObjectToScanComboBox->SelectedItem);
			*ObjToScan = GBMSAPI_Example_Util::GetObjectToScanIDFromName(ObjToScanName);

			/////////////////////////////
			// GET OPTION MASK
			/////////////////////////////
			(*OptionMask) = 0;
			this->GetOptionMask(OptionMask, NULL);

			

			/////////////////////////////////////
			// GET DISPLAY OPTION MASK
			/////////////////////////////////////
			*DisplayOptionMask = 0;

			// check if supported LCD
			unsigned int ExternalDevicesMask;
			int RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalDevicesMask);

			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(RetVal,"GetAcquisitionOptions");
				return false;
			}

			if ((ExternalDevicesMask & GBMSAPI_OED_VUI_LCD) != 0)
			{
				// Stop screen after acquisition
				if (this->StopScreenAfterAcquisitionCheckBox->Enabled == true &&
					this->StopScreenAfterAcquisitionCheckBox->Checked == true)
				{
					(*DisplayOptionMask) |= GBMSAPI_DO_FINAL_SCREEN;
				}

				// customized contrast
				if (this->ShowCustomizedContrastCheckBox->Enabled == true &&
					this->ShowCustomizedContrastCheckBox->Checked == true)
				{
					// read limit value
					try
					{
						Byte ContrastLimit = Byte::Parse(this->ContrastLimitTextBox->Text);
						*ContrastLimitToDisplay = ContrastLimit;
					}
					catch (Exception ^ReadEx)
					{
						if ((ReadEx->GetType())->ToString() == (ArgumentNullException::typeid)->ToString() ||
							(ReadEx->GetType())->ToString() == (FormatException::typeid)->ToString() ||
							(ReadEx->GetType())->ToString() == (OverflowException::typeid)->ToString()
							)
						{
							// default value
							*ContrastLimitToDisplay = 120;
						}
						else
						{
							MessageBox::Show("Exception in GetAcquisitionOptions: " + ReadEx->Message);
							return false;
						}
					}
					(*DisplayOptionMask) |= GBMSAPI_DO_CUSTOMIZED_CONTRAST;
				}

				// customized completeness
				if (this->ShowCustomizedCompletenessCheckBox->Enabled == true &&
					this->ShowCustomizedCompletenessCheckBox->Checked == true)
				{
					// read limit value
					try
					{
						Byte CompletenessLimit = Byte::Parse(this->CompletenessLimitTextBox->Text);
						*CompletenessLimitToDisplay = CompletenessLimit;
					}
					catch (Exception ^ReadEx)
					{
						if ((ReadEx->GetType())->ToString() == (ArgumentNullException::typeid)->ToString() ||
							(ReadEx->GetType())->ToString() == (FormatException::typeid)->ToString() ||
							(ReadEx->GetType())->ToString() == (OverflowException::typeid)->ToString()
							)
						{
							// default value
							*CompletenessLimitToDisplay = 90;
						}
						else
						{
							MessageBox::Show("Exception in GetAcquisitionOptions: " + ReadEx->Message);
							return false;
						}
					}
					(*DisplayOptionMask) |= GBMSAPI_DO_CUSTOMIZED_COMPLETENESS;
				}
			}

			///////////////////////////////////////
			// GET CLIP DATA
			///////////////////////////////////////
			*ClipRegionW = 0;
			*ClipRegionH = 0;
			if (this->ClipEnableCheckBox->Enabled == true &&
				this->ClipEnableCheckBox->Checked == true)
			{
				// read clip values
				try
				{
					UInt32 ClipSizeX = UInt32::Parse(this->ClipRegionSizeXTextBox->Text);
					UInt32 ClipSizeY = UInt32::Parse(this->ClipRegionSizeYTextBox->Text);
					*ClipRegionW = ClipSizeX;
					*ClipRegionH = ClipSizeY;
				}
				catch (Exception ^ReadEx)
				{
					if ((ReadEx->GetType())->ToString() == (ArgumentNullException::typeid)->ToString() ||
						(ReadEx->GetType())->ToString() == (FormatException::typeid)->ToString() ||
						(ReadEx->GetType())->ToString() == (OverflowException::typeid)->ToString()
						)
					{
						// default value
						*ClipRegionW = 0;
						*ClipRegionH = 0;
					}
					else
					{
						MessageBox::Show("Exception in GetAcquisitionOptions: " + ReadEx->Message);
						return false;
					}
				}
			}

			///////////////////////////////////////////
			// GET ROLL OPTIONS
			///////////////////////////////////////////
			// Roll Preview Timeout
			*RollPreviewTimeout = 0;
			if (this->AutomaticRollPreviewTimeoutTextBox->Enabled == true)
			{
				// read timeout
				try
				{
					UInt32 Timeout = UInt32::Parse(this->AutomaticRollPreviewTimeoutTextBox->Text);
					*RollPreviewTimeout = Timeout;
				}
				catch (Exception ^ReadEx)
				{
					if ((ReadEx->GetType())->ToString() == (ArgumentNullException::typeid)->ToString() ||
						(ReadEx->GetType())->ToString() == (FormatException::typeid)->ToString() ||
						(ReadEx->GetType())->ToString() == (OverflowException::typeid)->ToString()
						)
					{
						// default value
						RollPreviewTimeout = 0;
					}
					else
					{
						MessageBox::Show("Exception in GetAcquisitionOptions: " + ReadEx->Message);
						return false;
					}
				}
			}
			// Artefact Cleaning Depth
			*ArtefactCleaningDepth = 0;
			if (this->ArtefactCleaningDepthTextBox->Enabled == true)
			{
				// read Clean Depth
				try
				{
					Byte CleanDepth = Byte::Parse(this->ArtefactCleaningDepthTextBox->Text);
					if (CleanDepth > GBMSAPI_ARTEFACT_CLEANING_DEPTH_MAX &&
						CleanDepth != GBMSAPI_ARTEFACT_CLEANING_DEPTH_UNLIMITED)
					{
						MessageBox::Show("Clean depth greater than maximum allowed value; will be automatically set to max","Warning");
						CleanDepth = GBMSAPI_ARTEFACT_CLEANING_DEPTH_MAX;
					}
					*ArtefactCleaningDepth = CleanDepth;
				}
				catch (Exception ^ReadEx)
				{
					if ((ReadEx->GetType())->ToString() == (ArgumentNullException::typeid)->ToString() ||
						(ReadEx->GetType())->ToString() == (FormatException::typeid)->ToString() ||
						(ReadEx->GetType())->ToString() == (OverflowException::typeid)->ToString()
						)
					{
						// default value
						ArtefactCleaningDepth = 0;
					}
					else
					{
						MessageBox::Show("Exception in GetAcquisitionOptions: " + ReadEx->Message);
						return false;
					}
				}
			}

            // ver 3.2.0.0
            // EnableRollCompositionBlock
            if (this->cbEnableRollCompositionBlock->Checked) *EnableRollCompositionBlock = true;
            // end ver 3.2.0.0

			return true;
		}
		catch (Exception ^ex)
		{
			MessageBox::Show("Exception in GetAcquisitionOptions: " + ex->Message);
			return false;
		}
	}
}