#pragma once

#include "windows.h"
#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
#include "GBMSAPIExample_Util.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::IO;
using namespace System::Runtime::InteropServices;


namespace GBMSAPICExample {

	/// <summary>
	/// Riepilogo per PermanentDataStorageForm
	///
	/// AVVISO: se si modifica il nome della classe, sar necessario modificare la
	///          propriet 'Nome file di risorse' relativa allo strumento Compilatore di risorse gestite
	///          associato a tutti i file RESX da cui dipende la classe. In caso contrario,
	///          le finestre di progettazione non saranno in grado di interagire correttamente con le
	///          risorse localizzate associate al form.
	/// </summary>
	public ref class PermanentDataStorageForm : public System::Windows::Forms::Form
	{
	public:
		PermanentDataStorageForm(void)
		{
			InitializeComponent();
			//
			//TODO: aggiungere qui il codice del costruttore.
			//
			int RetVal, Appoggio;
			RetVal =  GBMSAPI_GetUserDataSize(&Appoggio);

			
			
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"PermanentDataStorageForm,GBMSAPI_GetUserDataSize");
				this->Close();
			}
			else
			{
				UserDataSize = Appoggio;
				this->StorageSizeLabel->Text = "Available bytes = " + UserDataSize;
			}
		}

	protected:
		/// <summary>
		/// Liberare le risorse in uso.
		/// </summary>
		~PermanentDataStorageForm()
		{
			if (components)
			{
				delete components;
			}
		}
	protected:
		int UserDataSize;
	private: System::Windows::Forms::Button^  WriteButton;
	protected: 
	private: System::Windows::Forms::Button^  ReadButton;
	private: System::Windows::Forms::Label^  StorageSizeLabel;

	protected: 

	private:
		/// <summary>
		/// Variabile di progettazione necessaria.
		/// </summary>
		System::ComponentModel::Container ^components;

#pragma region Windows Form Designer generated code
		/// <summary>
		/// Metodo necessario per il supporto della finestra di progettazione. Non modificare
		/// il contenuto del metodo con l'editor di codice.
		/// </summary>
		void InitializeComponent(void)
		{
			this->WriteButton = (gcnew System::Windows::Forms::Button());
			this->ReadButton = (gcnew System::Windows::Forms::Button());
			this->StorageSizeLabel = (gcnew System::Windows::Forms::Label());
			this->SuspendLayout();
			// 
			// WriteButton
			// 
			this->WriteButton->Location = System::Drawing::Point(12, 29);
			this->WriteButton->Name = L"WriteButton";
			this->WriteButton->Size = System::Drawing::Size(181, 39);
			this->WriteButton->TabIndex = 0;
			this->WriteButton->Text = L"Write From File";
			this->WriteButton->UseVisualStyleBackColor = true;
			this->WriteButton->Click += gcnew System::EventHandler(this, &PermanentDataStorageForm::WriteButton_Click);
			// 
			// ReadButton
			// 
			this->ReadButton->Location = System::Drawing::Point(12, 97);
			this->ReadButton->Name = L"ReadButton";
			this->ReadButton->Size = System::Drawing::Size(181, 39);
			this->ReadButton->TabIndex = 1;
			this->ReadButton->Text = L"Read To File";
			this->ReadButton->UseVisualStyleBackColor = true;
			this->ReadButton->Click += gcnew System::EventHandler(this, &PermanentDataStorageForm::ReadButton_Click);
			// 
			// StorageSizeLabel
			// 
			this->StorageSizeLabel->AutoSize = true;
			this->StorageSizeLabel->Location = System::Drawing::Point(13, 162);
			this->StorageSizeLabel->Name = L"StorageSizeLabel";
			this->StorageSizeLabel->Size = System::Drawing::Size(0, 13);
			this->StorageSizeLabel->TabIndex = 2;
			// 
			// PermanentDataStorageForm
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->ClientSize = System::Drawing::Size(209, 213);
			this->Controls->Add(this->StorageSizeLabel);
			this->Controls->Add(this->ReadButton);
			this->Controls->Add(this->WriteButton);
			this->Name = L"PermanentDataStorageForm";
			this->Text = L"PermanentDataStorageForm";
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion
	private: 
		System::Void WriteButton_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			OpenFileDialog ^ OFDlg = gcnew OpenFileDialog();
			OFDlg->Title = "Open file containing data";
			if (OFDlg->ShowDialog() != ::DialogResult::OK )
			{
				MessageBox::Show("Error in Opening File");
				return;
			}
			String ^fname = OFDlg->FileName;
			array<unsigned char,1> ^DataToBeWrittenArray = File::ReadAllBytes(fname);

			if (DataToBeWrittenArray->Length > (UserDataSize - 1)) // I need at least a byte at the end of the buffer for CRC calculation
			{
				MessageBox::Show("Warning: Data To be written too much long; they will be truncated to the " + UserDataSize + "-th byte");
			}

			int BytesToBeWritten;

			if (DataToBeWrittenArray->Length > (UserDataSize - 1))
			{
				BytesToBeWritten = (UserDataSize - 1);
			}
			else
			{
				BytesToBeWritten = DataToBeWrittenArray->Length;
			}

			// allocate memory
			IntPtr BufferToWrite = Marshal::AllocHGlobal(BytesToBeWritten);
			if (BufferToWrite == IntPtr::Zero)
			{
				MessageBox::Show("Error in Buffer allocation");
				return;
			}

			Marshal::Copy(DataToBeWrittenArray,0,BufferToWrite,BytesToBeWritten);

			int RetVal = GBMSAPI_WriteUserData(0,BytesToBeWritten,(void *)(BufferToWrite.ToPointer()));

			// free allocated memory
			Marshal::FreeHGlobal(BufferToWrite);			
			
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"PermanentDataStorageForm,GBMSAPI_WriteUserData");
			}
			else
			{
				MessageBox::Show("Written " + BytesToBeWritten + "bytes");
			}
		}
	private:
		System::Void ReadButton_Click(System::Object^  sender, System::EventArgs^  e)
		{
			IntPtr BufferToRead = Marshal::AllocHGlobal(UserDataSize);
			if (BufferToRead == IntPtr::Zero)
			{
				MessageBox::Show("Error in Buffer allocation","ReadButton_Click");
				return;
			}

			array<unsigned char,1> ^DataToBeWrittenArray = gcnew array<unsigned char,1> (UserDataSize);

			int RetVal = GBMSAPI_ReadUserData(0,UserDataSize,(void *)(BufferToRead.ToPointer()));		
			
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"PermanentDataStorageForm,GBMSAPI_WriteUserData");
			}

			Marshal::Copy(BufferToRead,DataToBeWrittenArray,0,UserDataSize);

			Marshal::FreeHGlobal(BufferToRead);			
			
			if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
			{
				GBMSAPI_Example_Util::GBMSAPI_Example_ManageErrors(
					RetVal,"PermanentDataStorageForm,GBMSAPI_WriteUserData");
			}

			SaveFileDialog ^ SFDlg = gcnew SaveFileDialog();
			SFDlg->Title = "Open file for saving data";
			if (SFDlg->ShowDialog() != ::DialogResult::OK )
			{
				MessageBox::Show("Error in Opening File");
				return;
			}
			String ^fname = SFDlg->FileName;

			File::WriteAllBytes(fname,DataToBeWrittenArray);
		}
	};
}
