// GBMSAPI_GettingStarted_Cpp_Example.cpp : definisce il punto di ingresso dell'applicazione console.
//

#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "GBMSAPI_Defines.h"
#include "GBMSAPI_Library.h"
//#include "DSBeep.h"

//////////////////////////////////////////
// UTILITIES
//////////////////////////////////////////
void DoEventsCE(void)
{
	MSG msg;
	// Get message from queue and dispatch it
	for (int i=0; i<20; i++) {
		if (PeekMessage(&msg, 0, 0, 0, PM_REMOVE)) {
			TranslateMessage (&msg);
			DispatchMessage (&msg);
		}
	}
	return;
}

//////////////////////////////////////////
// GLOBAL VARIABLES
//////////////////////////////////////////
UINT32	MaxImageSizeX, MaxImageSizeY;
UINT32	FullResolutionSizeX, FullResolutionSizeY;
BYTE	*FullResolutionImage;

UINT32	LastErrorCode, FrameNumber, ImageSize, LastDiagnosticValue;
BOOL	AcquisitionEnded;
BYTE	ImageContrast;

UINT32	ExternalEquipment;
BOOL	FirstFrameAcquired;

void SaveAsRawImage(char *FileName, BYTE *FrameArray, UINT32 SX, UINT32 SY)
{
	FILE *fp;

	fp = fopen(FileName,"wb");
	if (fp != NULL)
	{
		fwrite(FrameArray,sizeof(BYTE),SX * SY, fp);
		fclose(fp);
	}
	else
	{
		printf ("Impossible to open the %s file\n",FileName);
	}
}

void ResetCaptureGlobals()
{
	AcquisitionEnded = FALSE;
    FrameNumber = 0;
    ImageContrast = 0;
    ImageSize = 0;
    LastDiagnosticValue = 0;
    LastErrorCode = GBMSAPI_ERROR_CODE_NO_ERROR;
	FirstFrameAcquired = FALSE;
}

BOOL __stdcall AcquisitionCallback(
    unsigned int OccurredEventCode,
	int GetFrameErrorCode,
	unsigned int EventInfo,
	unsigned char *FramePtr,
	int FrameSizeX,
	int FrameSizeY,
	double CurrentFrameRate,
	double NominalFrameRate,
	unsigned int GB_Diagnostic,
	void *UserDefinedParameters
)
{    
	////////////////
	// check error
	////////////////
	if (GetFrameErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
		LastErrorCode = GetFrameErrorCode;
		AcquisitionEnded = TRUE;
		return TRUE;
	}

	//////////////////////////////
	// check event:
	// ERROR EVENT ALREADY CHECKED
	//////////////////////////////
	switch (OccurredEventCode)
	{
		case GBMSAPI_AE_SCANNER_STARTED:
			{
				/////////
				// ....
				/////////
				break;
			}

		case GBMSAPI_AE_PREVIEW_PHASE_END:
			{
				// Sound to advise operator that the preview has ended
				if ((ExternalEquipment & GBMSAPI_OED_SOUND) != 0)
				{
					GBMSAPI_Sound(12,2,1);
				}
				else
				{
					//DSBeep(4000,200);
				}
				break;
			}

		case GBMSAPI_AE_VALID_FRAME_ACQUIRED:
			{
				FrameNumber++;

				// ver 3.3.0.0
				static unsigned int indMask = 0, oldIndMask = 0;
				static int color = 0, oldColor = 0;
				if (FrameNumber < 100)
				{
					indMask = GBMSAPI_VIL_THUMB_RIGHT;
					color = GBMSAPI_VIL_COLOR_GREEN;
				}
				else if (FrameNumber < 120)
				{
					indMask = GBMSAPI_VIL_THUMB_RIGHT | GBMSAPI_VIL_INDEX_RIGHT;
					color = GBMSAPI_VIL_COLOR_RED;
				}
				else if (FrameNumber < 140)
				{
					indMask = GBMSAPI_VIL_MIDDLE_RIGHT;
					color = GBMSAPI_VIL_COLOR_YELLOW;
				}
				else
				{
					indMask = GBMSAPI_VIL_INDEX_LEFT;
					color = GBMSAPI_VIL_COLOR_GREEN;
				}
				if (FrameNumber > 50)
				{
					if ((indMask != oldIndMask) || (color != oldColor))
					{
						oldIndMask = indMask;
						oldColor = color;
						GBMSAPI_SetFingerIndicatorVUIState(0xFFFFFFFF, GBMSAPI_VIL_COLOR_OFF,FALSE);
						GBMSAPI_SetFingerIndicatorVUIState(indMask,color,FALSE);
					}
				}
				printf("F=%d, i=%d, oi=%d, c=%d, oc=%d\n",FrameNumber, indMask, oldIndMask, color, oldColor);
				// end ver 3.3.0.0

				// check diagnostic
				if (GB_Diagnostic != 0 && GB_Diagnostic != GBMSAPI_DM_VSROLL_ROLL_DIRECTION_RIGHT
					&& GB_Diagnostic != GBMSAPI_DM_VSROLL_ROLL_DIRECTION_LEFT)
				{
					printf("DIAGNOSTIC: %d\n",GB_Diagnostic);
				}
                if (!FirstFrameAcquired)
                {
                    FirstFrameAcquired = TRUE;
                    printf("\nPUT THE OBJECT ON THE SCANNER\n\n");
                }

				// Get contrast and size: if not supported, they keep their value at 0,
				// without checking return value
				ImageContrast = 0;
				ImageSize = 0;
				GBMSAPI_GetFingerprintContrast(
					&ImageContrast);

				GBMSAPI_GetFingerprintSize(
					&ImageSize);
				LastDiagnosticValue = GB_Diagnostic;
				break;
			}

		case GBMSAPI_AE_ACQUISITION_END:
			{
				AcquisitionEnded = TRUE;

				// Sound to advise operator that acquisition has ended
				if ((ExternalEquipment & GBMSAPI_OED_SOUND) != 0)
				{
					GBMSAPI_Sound(12,2,1);
				}
				else
				{
					//DSBeep(4000,200);
				}

				// finalize image
				GBMSAPI_ImageFinalization(FramePtr);

				// copy frame
				memcpy(FullResolutionImage,FramePtr,FrameSizeX * FrameSizeY);
				FullResolutionSizeX = FrameSizeX;
				FullResolutionSizeY = FrameSizeY;

				// Get contrast and size: if not supported, they keep their value at 0,
				// without checking return value
				ImageContrast = 0;
				ImageSize = 0;
				GBMSAPI_GetFingerprintContrast(
					&ImageContrast);

				GBMSAPI_GetFingerprintSize(
					&ImageSize);
				LastDiagnosticValue = GB_Diagnostic;

				break;
			}
	}

	return 1;
}

int CaptureObject(UINT32 ObjectName)
{
    ///////////////////
    // check parameters
    ///////////////////
    UINT32 ObjType = GBMSAPI_GetTypeFromObject(ObjectName);
    // check if supported
    UINT32 ScannableTypesMask;
    GBMSAPI_GetScannableTypes(
        &ScannableTypesMask);

    if ((ObjType & ScannableTypesMask) == 0)
    {
        printf("Object not supported\n");
        return GBMSAPI_ERROR_CODE_PARAMETER;
    }
    
    ResetCaptureGlobals();

    printf("Starting Acquisition for object\n");

	// ver 3.3.0.0
	GBMSAPI_EnableLEDColorSettingDuringAcquisition(TRUE);
	// end ver 3.3.0.0

    // Set autocapture: if not supported it's not considered by the
    // GBMSAPI library
    int RetVal = GBMSAPI_StartAcquisition(
        ObjectName,
        GBMSAPI_AO_AUTOCAPTURE,
        AcquisitionCallback,
        NULL,
        0,
        0,
        0
    );
    if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        printf("Error in GBMSAPI_StartAcquisition: %d\n",RetVal);
        return RetVal;
    }

    // enter a cycle waiting for the capture
    int CurrentFrameNumber = 0;
    printf("Acquisition Started\n");
    while (!AcquisitionEnded)
    {
        //////////////////////////////////////////////////
        // VERY IMPORTANT TO CALL THE DoEvents FUNCTION,
        // OTHERWISE OS MESSAGES ARE NOT PROCESSED!!!!!
        //////////////////////////////////////////////////
        DoEventsCE();

        if (CurrentFrameNumber != FrameNumber)
        {
            // a new frame has been acquired
            CurrentFrameNumber = FrameNumber;
        }

        if (CurrentFrameNumber > 200 &&
            ((ObjType & GBMSAPI_SBT_ROLL_SINGLE_FINGER) == 0)
            )
        {
            // Manually stop capture
            GBMSAPI_StopAcquisition();
        }
    }

    printf("Acquisition Ended for object. Result:\n");
    if (LastErrorCode != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        printf("Error: %d\n", LastErrorCode);
    }
    else
    {
        printf("Acquisition ok\n");
    }

    return LastErrorCode;
}

int _tmain(int argc, _TCHAR* argv[])
{
	////////////////////////////////////////////////////
    // CHECK SCANNER PRESENCE
    ////////////////////////////////////////////////////
    GBMSAPI_DeviceInfoStruct AttachedDeviceList [GBMSAPI_MAX_PLUGGED_DEVICE_NUM] ;
    int AttachedDeviceNumber;
    UINT32 USBErrorCode, ScannableTypesMask;

    int RetVal = GBMSAPI_GetAttachedDeviceList(
        AttachedDeviceList, &AttachedDeviceNumber, (unsigned long *)(&USBErrorCode));
    if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        printf("Error in GBMSAPI_NET_GetAttachedDeviceList: %d\n",RetVal);
        printf("Press any key to close the application\n");
        getchar();
        getchar();
        return 1;
    }
    if (AttachedDeviceNumber == 0)
    {
        printf("No scanners detected\n");
        printf("Press any key to close the application\n");
        getchar();
        getchar();
        return 1;
    }

    ////////////////////////////////////////////////////
    // SET DEVICE TO BE USED
    ////////////////////////////////////////////////////
    /***************************************************
     * Here the first device found will be used.
     * If a specified device (discriminated by type, or
     * serial number, or whatever else) should be
     * used, put your code here.
    ***************************************************/
    RetVal = GBMSAPI_SetCurrentDevice(
        AttachedDeviceList[0].DeviceID, AttachedDeviceList[0].DeviceSerialNumber);
    if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        printf("Error in GBMSAPI_SetCurrentDevice: %d\n",RetVal);
        printf("Press any key to close the application\n");
        getchar();
        getchar();
        return 1;
    }

    ////////////////////////////////////////////////////
    // ALLOCATE MEMORY
    ////////////////////////////////////////////////////
    RetVal = GBMSAPI_GetMaxImageSize(
        &MaxImageSizeX, &MaxImageSizeY);
    if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        printf("Error in GBMSAPI_GetMaxImageSize: %d\n",RetVal);
        printf("Press any key to close the application\n");
        getchar();
        getchar();
        return 1;
    }

    FullResolutionImage = new BYTE[MaxImageSizeX * MaxImageSizeY];
    if (FullResolutionImage == NULL)
    {
        printf("Cannot allocate memory: press any key to close the application\n");
        getchar();
        getchar();
        return 1;
    }

    //////////////////////////////////////////////////
    // GET SUPPORTED OBJECTS
    //////////////////////////////////////////////////
    RetVal = GBMSAPI_GetScannableTypes(
        &ScannableTypesMask);
    if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        printf("Error in GBMSAPI_GetScannableTypes: %d\n",RetVal);
        printf("Press any key to close the application\n");
        getchar();
        getchar();
        return 1;
    }

	//////////////////////////////////////////////////////
    // Check scanner internal beeper
    //////////////////////////////////////////////////////
    RetVal = GBMSAPI_GetOptionalExternalEquipment(&ExternalEquipment);
	if (RetVal != GBMSAPI_ERROR_CODE_NO_ERROR)
	{
        printf("Error in GBMSAPI_GetOptionalExternalEquipment: %d\n",RetVal);
        printf("Press any key to close the application\n");
        getchar();
        getchar();
        return 1;
	}
	//DSInit(COINIT_APARTMENTTHREADED,FALSE);

    /*****************************************************
     * Here developer can change settings in order to
     * fit his needs (that means, what object he wants
     * to be captured)
    *****************************************************/
    //////////////////////////////////////////////////////
    // ACQUIRE PALM
    //////////////////////////////////////////////////////
    if ((ScannableTypesMask & GBMSAPI_SBT_FLAT_LOWER_HALF_PALM) != 0)
    {
		printf ("\n-----------------------------------------");
		printf ("\n      ACQUIRING PALM");
		printf ("\n-----------------------------------------\n");

        RetVal = CaptureObject(GBMSAPI_SO_LOWER_HALF_PALM_LEFT);
        if (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR)
        {
            // SAVE IN BMP
            SaveAsRawImage("PALM_IMAGE.raw", FullResolutionImage,
                FullResolutionSizeX, FullResolutionSizeY);
        }
    }
    //////////////////////////////////////////////////////
    // ACQUIRE SLAP-4
    //////////////////////////////////////////////////////
    if ((ScannableTypesMask & GBMSAPI_SBT_FLAT_SLAP_4) != 0)
    {
		printf ("\n-----------------------------------------");
		printf ("\n      ACQUIRING SLAP-4");
		printf ("\n-----------------------------------------\n");
        RetVal = CaptureObject(GBMSAPI_SO_SLAP_4_LEFT);
        if (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR)
        {
            // SAVE IN BMP
            SaveAsRawImage("SLAP_4_IMAGE.raw", FullResolutionImage,
                FullResolutionSizeX, FullResolutionSizeY);
        }
    }
	printf ("\n-----------------------------------------");
	Sleep(1000);
	printf ("\n-----------------------------------------");
	printf ("\n");
	printf ("\n");
	printf ("\n");
    //////////////////////////////////////////////////////
    // ACQUIRE THHUMBS-2
    //////////////////////////////////////////////////////
    if ((ScannableTypesMask & GBMSAPI_SBT_FLAT_THUMBS_2) != 0)
    {
		printf ("\n-----------------------------------------");
		printf ("\n      ACQUIRING THHUMBS-2");
		printf ("\n-----------------------------------------\n");
        RetVal = CaptureObject(GBMSAPI_SO_SLAP_2_THUMBS);
        if (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR)
        {
            // SAVE IN BMP
            SaveAsRawImage("THUMBS_2_IMAGE.raw", FullResolutionImage,
                FullResolutionSizeX, FullResolutionSizeY);
        }
    }
	printf ("\n-----------------------------------------");
	Sleep(1000);
	printf ("\n-----------------------------------------");
	printf ("\n");
	printf ("\n");
	printf ("\n");
    //////////////////////////////////////////////////////
    // ACQUIRE SINGLE FLAT FINGER
    //////////////////////////////////////////////////////
    if ((ScannableTypesMask & GBMSAPI_SBT_FLAT_SINGLE_FINGER) != 0)
    {
		printf ("\n-----------------------------------------");
		printf ("\n      ACQUIRING SINGLE FLAT FINGER");
		printf ("\n-----------------------------------------\n");
        RetVal = CaptureObject(GBMSAPI_SO_FLAT_LEFT_INDEX);
        if (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR)
        {
            // SAVE IN BMP
            SaveAsRawImage("FLAT_SINGLE_IMAGE.raw", FullResolutionImage,
                FullResolutionSizeX, FullResolutionSizeY);
        }
    }
	printf ("\n-----------------------------------------");
	Sleep(1000);
	printf ("\n-----------------------------------------");
	printf ("\n");
	printf ("\n");
	printf ("\n");
    //////////////////////////////////////////////////////
    // ACQUIRE ROLL FINGER
    //////////////////////////////////////////////////////
    if ((ScannableTypesMask & GBMSAPI_SBT_ROLL_SINGLE_FINGER) != 0)
    {
		printf ("\n-----------------------------------------");
		printf ("\n      ACQUIRING ROLL FINGER");
		printf ("\n-----------------------------------------\n");
        RetVal = CaptureObject(GBMSAPI_SO_ROLL_LEFT_INDEX);
		if (RetVal == GBMSAPI_ERROR_CODE_NO_ERROR)
		{
			// SAVE IN BMP
			SaveAsRawImage("ROLL_SINGLE_IMAGE.raw", FullResolutionImage,
				FullResolutionSizeX, FullResolutionSizeY);
		}
    }
	printf ("\n-----------------------------------------");
	Sleep(1000);
	printf ("\n-----------------------------------------");
	printf ("\n");
	printf ("\n");
	printf ("\n");

	delete [] FullResolutionImage;

	printf("Press any key to end the application\n");
    getchar();
    getchar();
	return 0;
}

