﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading;

using GBMSAPI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_VisualInterfaceLEDsDefines;

namespace GBMSAPI_GettingStarted_CS_Example
{
    public class CaptureUtils
    {
        public static void SaveInBmp(string FileName, byte[] FrameArray, uint SX, uint SY)
        {
            /////////////////////////////////////////////
            // ALLOCATE BITMAP AND ITS INTERNAL FRAME
            /////////////////////////////////////////////
            IntPtr FramePtr = Marshal.AllocHGlobal((int)(SX * SY));

            if (FramePtr == IntPtr.Zero)
            {
                Console.WriteLine("Cannot allocate memory: press any key to close the application");
                return;
            }
            Bitmap bmp = new Bitmap((int)SX, (int)SY, (int)SX, PixelFormat.Format8bppIndexed, FramePtr);
            if (bmp == null)
            {
                Console.WriteLine("Cannot allocate memory: press any key to close the application");
                return;
            }
            ColorPalette pal = bmp.Palette;
            for (int i = 0; i < pal.Entries.Length; i++)
            {
                pal.Entries[i] = Color.FromArgb(255, i, i, i);
            }
            bmp.Palette = pal;

            /////////////////////////////////////////////////////////
            // FILL BMP INTERNAL FRAME AND SAVE
            /////////////////////////////////////////////////////////
            Marshal.Copy(FrameArray, 0, FramePtr, (int)(SX * SY));
            bmp.Save(FileName, ImageFormat.Bmp);
            Marshal.FreeHGlobal(FramePtr);
        }

        public static int AcquisitionCallback(
            uint OccurredEventCode,
            int GetFrameErrorCode,
            uint EventInfo,
            byte[] FramePtr,
            int FrameSizeX,
            int FrameSizeY,
            double CurrentFrameRate,
            double NominalFrameRate,
            uint GB_Diagnostic,
            System.IntPtr UserDefinedParameters
            )
        {
            try
            {
                ////////////////
                // check error
                ////////////////
                if (GetFrameErrorCode != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    CaptureGlobals.LastErrorCode = GetFrameErrorCode;
                    Console.WriteLine("AcquisitionCallback: GetFrameErrorCode = " + GetFrameErrorCode);
                    CaptureGlobals.AcquisitionEnded = true;
                    return 1;
                }

                //////////////////////////////
                // check event:
                // ERROR EVENT ALREADY CHECKED
                //////////////////////////////
                switch (OccurredEventCode)
                {
                    case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_SCANNER_STARTED:
                        {
                            /////////
                            // ....
                            /////////
                            break;
                        }

                    case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_PREVIEW_PHASE_END:
                        {
                            // Sound to advise operator that the preview has ended
                            if ((CaptureGlobals.ExternalEquipment & GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_SOUND) != 0)
                            {
                                Console.WriteLine("END PREVIEW");
                                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_Sound(12, 2, 1);
                            }
                            else
                            {
                                //Console.Beep(4000, 200);
                                CaptureGlobals.DSBeep(4000, 200);
                            }
                            break;
                        }

                    case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_VALID_FRAME_ACQUIRED:
                        {
                            CaptureGlobals.FrameNumber++;

                            // check diagnostic
                            if (GB_Diagnostic != 0 && 
                                GB_Diagnostic != GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_ROLL_DIRECTION_LEFT &&
                                GB_Diagnostic != GBMSAPI_NET_DiagnosticMessages.GBMSAPI_NET_DM_VSROLL_ROLL_DIRECTION_RIGHT)
                            {
                                Console.WriteLine("DIAGNOSTIC: " + GB_Diagnostic);
                            }
                            if (!CaptureGlobals.FirstFrameAcquired)
                            {
                                CaptureGlobals.FirstFrameAcquired = true;
                                Console.WriteLine();
                                Console.WriteLine("PUT THE OBJECT ON THE SCANNER");
                                Console.WriteLine();
                            }

                            // Get contrast and size: if not supported, they keep their value at 0,
                            // without checking return value
                            CaptureGlobals.ImageContrast = 0;
                            CaptureGlobals.ImageSize = 0;
                            GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_GetFingerprintContrast(
                                out CaptureGlobals.ImageContrast);

                            GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_GetFingerprintSize(
                                out CaptureGlobals.ImageSize);
                            CaptureGlobals.LastDiagnosticValue = GB_Diagnostic;
                            break;
                        }

                    case GBMSAPI_NET_AcquisitionEvents.GBMSAPI_NET_AE_ACQUISITION_END:
                        {

                            // Sound to advise operator that the fingerprint can be
                            // released
                            if ((CaptureGlobals.ExternalEquipment & GBMSAPI_NET_OptionalExternalEquipment.GBMSAPI_NET_OED_SOUND) != 0)
                            {
                                Console.WriteLine("Calling GBMSAPI_NET_Sound at the end of acquisition");
                                GBMSAPI_NET_ExternalDevicesControlRoutines.GBMSAPI_NET_Sound(12, 4, 2);
                            }
                            else
                            {
                                //Console.Beep(4000, 200);
                                CaptureGlobals.DSBeep(4000, 200);
                            }

                            // finalize image
                            GBMSAPI_NET_EndAcquisitionRoutines.GBMSAPI_NET_ImageFinalization(FramePtr);

                            // copy frame
                            FramePtr.CopyTo(CaptureGlobals.FullResolutionImage, 0);
                            CaptureGlobals.FullResolutionSizeX = (uint)FrameSizeX;
                            CaptureGlobals.FullResolutionSizeY = (uint)FrameSizeY;

                            // diagnostic
                            CaptureGlobals.LastDiagnosticValue = GB_Diagnostic;

                            // Get contrast and size: if not supported, they keep their value at 0,
                            // without checking return value
                            CaptureGlobals.ImageContrast = 0;
                            CaptureGlobals.ImageSize = 0;
                            GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_GetFingerprintContrast(
                                out CaptureGlobals.ImageContrast);

                            GBMSAPI_NET_ValidFrameAcquiredRoutines.GBMSAPI_NET_GetFingerprintSize(
                                out CaptureGlobals.ImageSize);
                            CaptureGlobals.AcquisitionEnded = true;

                            break;
                        }
                }


                return 1;
            }
            catch (Exception ex)
            {
                GBMSAPI_NET_ScanningRoutines.GBMSAPI_NET_StopAcquisition();
                Console.WriteLine("Exception in acquisitioncallback: " + ex.Message);
                return 0;
            }
        }

        public static void ResetCaptureGlobals()
        {
            CaptureGlobals.AcquisitionEnded = false;
            CaptureGlobals.FrameNumber = 0;
            CaptureGlobals.ImageContrast = 0;
            CaptureGlobals.ImageSize = 0;
            CaptureGlobals.LastDiagnosticValue = 0;
            CaptureGlobals.LastErrorCode = GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR;
            CaptureGlobals.FirstFrameAcquired = false;
        }
    }
    public class CaptureGlobals
    {
        public static byte[] PreviewImage;
        public static byte[] FullResolutionImage;

        public static uint MaxImageSizeX, MaxImageSizeY;
        public static uint FullResolutionSizeX, FullResolutionSizeY;


        public static bool AcquisitionEnded;
        public static uint LastDiagnosticValue;
        public static uint ImageSize;
        public static byte ImageContrast;
        public static int FrameNumber;
        public static int LastErrorCode;
        public static uint ExternalEquipment;
        public static bool FirstFrameAcquired;

        /**************************************
         * DSInit
        **************************************/
        public const uint COINIT_APARTMENTTHREADED = (uint)(0x02);
        //[DllImport("DSBeep.dll", EntryPoint = "DSInit")]
        //public static extern bool DSInit(uint CoInit, bool ForceDS);
        public static bool DSInit(uint x, bool y)
        {
            return true;
        }

        /**************************************
         * DSInit
        **************************************/
        //[DllImport("DSBeep.dll", EntryPoint = "DSBeep")]
        //public static extern bool DSBeep(int Frequency, int Duration);
        public static bool DSBeep(int Frequency, int Duration)
        {
            return true;
        }
    }
    class Program
    {
        /// <summary>
        /// This function captures the object whose name is
        /// passed as parameter
        /// </summary>
        /// <param name="ObjectName">
        /// Objetc to be captured
        /// </param>
        /// <returns>
        /// see GBMSAPI_NET_ErrorCodes
        /// </returns>
        static int CaptureObject(uint ObjectName)
        {
            ///////////////////
            // check parameters
            ///////////////////
            uint ObjType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ObjectName);
            // check if supported
            uint SupportedObjMask;
            GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetScannableTypes(out SupportedObjMask);

            if ((ObjType & SupportedObjMask) == 0)
            {
                Console.WriteLine("Object not supported");
                return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_PARAMETER;
            }
            
            CaptureUtils.ResetCaptureGlobals();

            Console.WriteLine("Starting Acquisition for object");

            // Set autocapture: if not supported it's not considered by the
            // GBMSAPI library
            int RetVal = GBMSAPI_NET_ScanningRoutines.GBMSAPI_NET_StartAcquisition(
                                    ObjectName,
                                    GBMSAPI_NET_AcquisitionOptions.GBMSAPI_NET_AO_AUTOCAPTURE,
                                    CaptureUtils.AcquisitionCallback,
                                    IntPtr.Zero,
                                    0,
                                    0,
                                    0
                                    );
            if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                Console.WriteLine("GBMSAPI_NET_StartAcquisition returned: " + RetVal);
                return RetVal;
            }

            // enter a cycle waiting for the capture
            int FrameNumber = 0;
            Console.WriteLine("Acquisition Started");
            while (!CaptureGlobals.AcquisitionEnded)
            {
                //////////////////////////////////////////////////
                // VERY IMPORTANT TO CALL THE DoEvents FUNCTION,
                // OTHERWISE OS MESSAGES ARE NOT PROCESSED!!!!!
                //////////////////////////////////////////////////
                Application.DoEvents();

                if (FrameNumber != CaptureGlobals.FrameNumber)
                {
                    // a new frame has been acquired
                    FrameNumber = CaptureGlobals.FrameNumber;
                }

                if (FrameNumber > 50 &&
                    ((ObjType & GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLL_SINGLE_FINGER) == 0)
                    )
                {
                    // Manually stop capture
                    GBMSAPI_NET_ScanningRoutines.GBMSAPI_NET_StopAcquisition();
                }
            }
            //for (int i = 0; i < 500; i++)
            //{
            //    Application.DoEvents();
            //    Thread.Sleep(1);
            //}
            Console.WriteLine("Acquisition Ended for object. Result:");
            if (CaptureGlobals.LastErrorCode != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: " + CaptureGlobals.LastErrorCode);
            }
            else
            {
                Console.WriteLine("Acquisition ok");
            }

            return CaptureGlobals.LastErrorCode;
        }

        static void Main(string[] args)
        {
            try
            {
                ////////////////////////////////////////////////////
                // CHECK SCANNER PRESENCE
                ////////////////////////////////////////////////////
                GBMSAPI_NET_DeviceInfoStruct[] AttachedDeviceList;
                int AttachedDeviceNumber;
                uint USBErrorCode, ScannableTypesMask;

                AttachedDeviceList = new GBMSAPI_NET_DeviceInfoStruct[GBMSAPI_NET_DeviceInfoConstants.GBMSAPI_NET_MAX_PLUGGED_DEVICE_NUM];

                if (AttachedDeviceList == null)
                {
                    Console.WriteLine("Cannot allocate memory: press any key to close the application");
                    Console.Read();
                    return;
                }
                for (int i = 0; i < GBMSAPI_NET_DeviceInfoConstants.GBMSAPI_NET_MAX_PLUGGED_DEVICE_NUM; i++)
                {
                    AttachedDeviceList[i] = new GBMSAPI_NET_DeviceInfoStruct();
                    if (AttachedDeviceList[i] == null)
                    {
                        Console.WriteLine("Cannot allocate memory: press any key to close the application");
                        Console.Read();
                        return;
                    }
                }
                int RetVal = GBMSAPI_NET_DeviceSettingRoutines.GBMSAPI_NET_GetAttachedDeviceList(
                    AttachedDeviceList, out AttachedDeviceNumber, out USBErrorCode);
                if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    Console.WriteLine("Error in GBMSAPI_NET_GetAttachedDeviceList: " + RetVal);
                    Console.WriteLine("Press any key to close the application");
                    Console.Read();
                    return;
                }
                if (AttachedDeviceNumber == 0)
                {
                    Console.WriteLine("No scanners detected");
                    Console.WriteLine("Press any key to close the application");
                    Console.Read();
                    return;
                }

                ////////////////////////////////////////////////////
                // SET DEVICE TO BE USED
                ////////////////////////////////////////////////////
                /***************************************************
                 * Here the first device found will be used.
                 * If a specified device (discriminated by type, or
                 * serial number, or whatever else) should be
                 * used, put your code here.
                ***************************************************/
                RetVal = GBMSAPI_NET_DeviceSettingRoutines.GBMSAPI_NET_SetCurrentDevice(
                    AttachedDeviceList[0].DeviceID, AttachedDeviceList[0].DeviceSerialNumber);
                if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    Console.WriteLine("Error in GBMSAPI_NET_SetCurrentDevice: " + RetVal);
                    Console.WriteLine("Press any key to close the application");
                    Console.Read();
                    return;
                }

                ////////////////////////////////////////////////////
                // ALLOCATE MEMORY
                ////////////////////////////////////////////////////
                RetVal = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetMaxImageSize(
                    out CaptureGlobals.MaxImageSizeX, out CaptureGlobals.MaxImageSizeY);
                if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    Console.WriteLine("Error in GBMSAPI_NET_GetMaxImageSize: " + RetVal);
                    Console.WriteLine("Press any key to close the application");
                    Console.Read();
                    return;
                }

                CaptureGlobals.PreviewImage = new byte[CaptureGlobals.MaxImageSizeX * CaptureGlobals.MaxImageSizeY];
                if (CaptureGlobals.PreviewImage == null)
                {
                    Console.WriteLine("Cannot allocate memory: press any key to close the application");
                    Console.Read();
                    return;
                }
                CaptureGlobals.FullResolutionImage = new byte[CaptureGlobals.MaxImageSizeX * CaptureGlobals.MaxImageSizeY];
                if (CaptureGlobals.FullResolutionImage == null)
                {
                    Console.WriteLine("Cannot allocate memory: press any key to close the application");
                    Console.Read();
                    return;
                }

                //////////////////////////////////////////////////
                // GET SUPPORTED OBJECTS
                //////////////////////////////////////////////////
                RetVal = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetScannableTypes(
                    out ScannableTypesMask);
                if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    Console.WriteLine("Error in GBMSAPI_NET_GetScannableTypes: " + RetVal);
                    Console.WriteLine("Press any key to close the application");
                    return;
                }

                //////////////////////////////////////////////////////
                // Check scanner internal beeper
                //////////////////////////////////////////////////////
                RetVal = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetOptionalExternalEquipment(
                    out CaptureGlobals.ExternalEquipment);
                if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                {
                    Console.WriteLine("Error in GBMSAPI_NET_GetOptionalExternalEquipment: " + RetVal);
                    Console.WriteLine("Press any key to close the application");
                    return;
                }
                CaptureGlobals.DSInit(CaptureGlobals.COINIT_APARTMENTTHREADED, false);

                /*****************************************************
                 * Here developer can change settings in order to
                 * fit his needs (that means, what object he wants
                 * to be captured)
                *****************************************************/
                //////////////////////////////////////////////////////
                // ACQUIRE PALM
                //////////////////////////////////////////////////////
                if ((ScannableTypesMask & GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_LOWER_HALF_PALM) != 0)
                {
                    Console.WriteLine();
                    Console.WriteLine("-----------------------------------------");
                    Console.WriteLine("      ACQUIRING PALM");
                    Console.WriteLine("-----------------------------------------");
                    RetVal = CaptureObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT);
                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        // SAVE IN BMP
                        CaptureUtils.SaveInBmp("PALM_IMAGE.bmp", CaptureGlobals.FullResolutionImage,
                            CaptureGlobals.FullResolutionSizeX, CaptureGlobals.FullResolutionSizeY);
                    }
                }
                //////////////////////////////////////////////////////
                // ACQUIRE SLAP-4
                //////////////////////////////////////////////////////
                if ((ScannableTypesMask & GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SLAP_4) != 0)
                {
                    Console.WriteLine();
                    Console.WriteLine("-----------------------------------------");
                    Console.WriteLine("       ACQUIRING SLAP-4");
                    Console.WriteLine("-----------------------------------------");
                    RetVal = CaptureObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT);
                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        // SAVE IN BMP
                        CaptureUtils.SaveInBmp("SLAP_4_IMAGE.bmp", CaptureGlobals.FullResolutionImage,
                            CaptureGlobals.FullResolutionSizeX, CaptureGlobals.FullResolutionSizeY);
                    }
                }
                
                //////////////////////////////////////////////////////
                // ACQUIRE THHUMBS-2
                //////////////////////////////////////////////////////
                if ((ScannableTypesMask & GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_THUMBS_2) != 0)
                {
                    Console.WriteLine();
                    Console.WriteLine("-----------------------------------------");
                    Console.WriteLine("       ACQUIRING THHUMBS-2");
                    Console.WriteLine("-----------------------------------------");
                    RetVal = CaptureObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS);
                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        // SAVE IN BMP
                        CaptureUtils.SaveInBmp("THUMBS_2_IMAGE.bmp", CaptureGlobals.FullResolutionImage,
                            CaptureGlobals.FullResolutionSizeX, CaptureGlobals.FullResolutionSizeY);
                    }
                }
                //////////////////////////////////////////////////////
                // ACQUIRE SINGLE FLAT FINGER
                //////////////////////////////////////////////////////
                if ((ScannableTypesMask & GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SINGLE_FINGER) != 0)
                {
                    Console.WriteLine();
                    Console.WriteLine("-----------------------------------------");
                    Console.WriteLine("       ACQUIRING SINGLE FLAT FINGER");
                    Console.WriteLine("-----------------------------------------");
                    RetVal = CaptureObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX);
                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        // SAVE IN BMP
                        CaptureUtils.SaveInBmp("FLAT_SINGLE_IMAGE.bmp", CaptureGlobals.FullResolutionImage,
                            CaptureGlobals.FullResolutionSizeX, CaptureGlobals.FullResolutionSizeY);
                    }
                }
                //////////////////////////////////////////////////////
                // ACQUIRE ROLL FINGER
                //////////////////////////////////////////////////////
                if ((ScannableTypesMask & GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLL_SINGLE_FINGER) != 0)
                {
                    Console.WriteLine();
                    Console.WriteLine("-----------------------------------------");
                    Console.WriteLine("       ACQUIRING ROLL FINGER");
                    Console.WriteLine("-----------------------------------------");
                    RetVal = CaptureObject(GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX);
                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        // SAVE IN BMP
                        CaptureUtils.SaveInBmp("ROLL_SINGLE_IMAGE.bmp", CaptureGlobals.FullResolutionImage,
                            CaptureGlobals.FullResolutionSizeX, CaptureGlobals.FullResolutionSizeY);
                    }
                }
                
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in main: " + ex.Message);
            }

            Console.Read();
        }
    }
}
