#include "stdafx.h"
#include "GB_WinVer.h"

#include <stdio.h>

typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);

LPFN_ISWOW64PROCESS fnIsWow64Process;

BOOL IsWow64()
{
    BOOL bIsWow64 = FALSE;

    //IsWow64Process is not available on all supported versions of Windows.
    //Use GetModuleHandle to get a handle to the DLL that contains the function
    //and GetProcAddress to get a pointer to the function if available.

    fnIsWow64Process = (LPFN_ISWOW64PROCESS) GetProcAddress(
        GetModuleHandle(TEXT("kernel32")),"IsWow64Process");

    if(NULL != fnIsWow64Process)
    {
        if (!fnIsWow64Process(GetCurrentProcess(),&bIsWow64))
        {
            //handle error
        }
    }
    return bIsWow64;
}

GB_WINVER_API int GB_GetWindowsVersion(void)
{
	OSVERSIONINFOEX osvi;

    ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

    if (!GetVersionEx((OSVERSIONINFO *)(&(osvi)))) return GB_WinVer_Unknown;

	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		if (osvi.dwMajorVersion == 4)
		{
			return GB_WinVer_WinNT40;
		}
		else if (osvi.dwMajorVersion == 5)
		{
			if (osvi.dwMinorVersion == 0)
			{
				return GB_WinVer_Win2K;
			}
			else if (osvi.dwMinorVersion == 1)
			{
				return GB_WinVer_WinXP;
			}
			else if (osvi.dwMinorVersion == 2)
			{
				return GB_WinVer_WinServ2003;
			}
			else return GB_WinVer_Unknown;
		}
		else if (osvi.dwMajorVersion == 6)
		{
			if (osvi.dwMinorVersion == 0)
			{
				if (osvi.wProductType == 1)
				{
					return GB_WinVer_WinVista;
				}
				else if (osvi.wProductType == 3)
				{
					return GB_WinVer_WinServ2008;
				}
				else return GB_WinVer_Unknown;
			}
			else if (osvi.dwMinorVersion == 1)
			{
				if (osvi.wProductType == 1)
				{
					return GB_WinVer_Win7;
				}
				else if (osvi.wProductType == 3)
				{
					return GB_WinVer_WinServ2008R2;
				}
				else return GB_WinVer_Unknown;
			}
			else return GB_WinVer_Unknown;
		}
	}
	else return GB_WinVer_Unknown;

	return GB_WinVer_Unknown;
}

GB_WINVER_API BOOL GB_GetWindowsInfo(GB_OsInfo_Struct *OsInfoToRetrieve)
{
	OsInfoToRetrieve->Version = GB_GetWindowsVersion();
	OsInfoToRetrieve->OsBits = (IsWow64())? GB_OsBits_64 : GB_OsBits_32;

	return GB_SUCCESS;
}
