using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Reflection;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;

namespace GBMSAPI_NET
{
    namespace GBMSAPI_NET_LibraryFunctions
    {
        internal delegate Int32 GBMSAPI_NET_AcquisitionEventsManagerCallbackInterface
        (
            UInt32 OccurredEventCode,
            Int32 GetFrameErrorCode,
            UInt32 Eventinfo,
            IntPtr FramePtr,
            Int32 FrameSizeX,
            Int32 FrameSizeY,
            Double CurrentFrameRate,
            Double NominalFrameRate,
            UInt32 GB_Diagnostic,
            System.IntPtr UserDefinedParameters
        );
        internal class GBMSAPI_AcquisitionCallbackInterface
        {
            public static GBMSAPI_NET_AcquisitionEventsManagerCallback FunctionToBeCalled;
            public static Int32 MyAcquisitionEventsManagerCallbackInterface
            (
                UInt32 OccurredEventCode,
                Int32 GetFrameErrorCode,
                UInt32 Eventinfo,
                IntPtr FramePtr,
                Int32 FrameSizeX,
                Int32 FrameSizeY,
                Double CurrentFrameRate,
                Double NominalFrameRate,
                UInt32 GB_Diagnostic,
                System.IntPtr UserDefinedParameters
            )
            {
                Byte[] ArrayToPass = null;
                if (FramePtr != IntPtr.Zero && FrameSizeX > 0 && FrameSizeY > 0)
                {
                    ArrayToPass = new Byte[FrameSizeX * FrameSizeY];
                    Marshal.Copy(FramePtr, ArrayToPass, 0, FrameSizeX * FrameSizeY);                    
                }
                else
                {
                    FrameSizeX = 0;
                    FrameSizeY = 0;
                    ArrayToPass = null;
                }

                GC.KeepAlive(GBMSAPI_AcquisitionCallbackInterface.FunctionToBeCalled);
                if (FunctionToBeCalled != null)
                {
                    return FunctionToBeCalled(OccurredEventCode,
                        GetFrameErrorCode,
                        Eventinfo,
                        ArrayToPass,
                        FrameSizeX,
                        FrameSizeY,
                        CurrentFrameRate,
                        NominalFrameRate,
                        GB_Diagnostic,
                        UserDefinedParameters
                    );
                }
                else return 1;
            }
        }
        internal class GBMSAPI_NET_DLL_WRAPPER
        {
            /**************************************
             * GBMSAPI_GetAttachedDeviceList
            **************************************/
            [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
            public struct GBMSAPI_DEVICE_INFO_STRUCT
            {
                public Byte DeviceID;
                [MarshalAs(UnmanagedType.ByValTStr, SizeConst = GBMSAPI_NET_DeviceInfoConstants.GBMSAPI_NET_MAX_SN_LENGHT)]
                public String DeviceSerialNumber;
            }

            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetAttachedDeviceList")]
            public static extern int GBMSAPI_GetAttachedDeviceList(
                [In, Out] GBMSAPI_DEVICE_INFO_STRUCT[] AttachedDeviceList,
                out Int32 AttachedDeviceNumber,
                out UInt32 USBErrorCode);

            /**************************************
             * GBMSAPI_SetCurrentDevice
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetCurrentDevice", CharSet = CharSet.Ansi)]
            public static extern int GBMSAPI_SetCurrentDevice(
                Byte DeviceID,
                String DeviceSerialNumber
               );

            /**************************************
             * GBMSAPI_GetCurrentDevice
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetCurrentDevice", CharSet = CharSet.Ansi)]
            public static extern int GBMSAPI_GetCurrentDevice(
                out Byte DeviceID,
                StringBuilder DeviceSerialNumber
               );

            /**************************************
             * GBMSAPI_GetDeviceNameAndVersion
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetDeviceNameAndVersion", CharSet = CharSet.Unicode)]
            public static extern int GBMSAPI_GetDeviceNameAndVersion(
                StringBuilder NameAndVersion
               );

            /**************************************
             * GBMSAPI_GetScannableTypes
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetScannableTypes")]
            public static extern int GBMSAPI_GetScannableTypes(
                out UInt32 ScannableTypesMask
                );

            /**************************************
             * GBMSAPI_GetDeviceFeatures
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetDeviceFeatures")]
            public static extern int GBMSAPI_GetDeviceFeatures(
                out UInt32 DeviceFeatures
				);

            // Ver 2.10.0.0
			/**************************************
			 * GBMSAPI_GetSupportedScanAreas
			**************************************/
			[DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetSupportedScanAreas")]
			public static extern int GBMSAPI_GetSupportedScanAreas(
				out UInt32 SupportedScanAreas
				);
            // end Ver 2.10.0.0

            // Ver 3.1.0.0
            /**************************************
             * GBMSAPI_GetSupportedScanAreasByDevice
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetSupportedScanAreasByDevice")]
            public static extern int GBMSAPI_GetSupportedScanAreasByDevice(
                byte DeviceID,
                out UInt32 SupportedScanAreas
                );
            // end Ver 3.1.0.0

            // VER 2.7.1.0
            /**************************************
             * GBMSAPI_SetRollAreaStandard
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetRollAreaStandard")]
            public static extern int GBMSAPI_SetRollAreaStandard(
                UInt32 RollAreaStandard
                );

            /**************************************
             * GBMSAPI_GetSupportedScanOptions
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetSupportedScanOptions")]
            public static extern int GBMSAPI_GetSupportedScanOptions(
                out UInt32 SupportedScanOptions
                );

            /**************************************
             * GBMSAPI_GetOptionalExternalEquipment
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetOptionalExternalEquipment")]
            public static extern int GBMSAPI_GetOptionalExternalEquipment(
                out UInt32 OptionalExternalEquipment
                );

            /**************************************
             * GBMSAPI_GetAvailableImageInfo
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetAvailableImageInfo")]
            public static extern int GBMSAPI_GetAvailableImageInfo(
                UInt32 ObjectTypeToScan,
                out UInt32 AvailableImageInfo
                );

			// Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_GetImageSize2" instead
			/**************************************
			 * GBMSAPI_GetImageSize
			**************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetImageSize")]
            public static extern int GBMSAPI_GetImageSize(
                UInt32 ObjectTypeToScan,
                Int32 FlatScanArea,
                Int32 Preview,
                out UInt32 ImageSizeX,
                out UInt32 ImageSizeY
                );

            /**************************************
             * GBMSAPI_GetImageSize2
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetImageSize2")]
            public static extern int GBMSAPI_GetImageSize2(
                UInt32 ObjectTypeToScan,
                UInt32 ScanArea,
                Int32 Preview,
                out UInt32 ImageSizeX,
                out UInt32 ImageSizeY
                );
            // end Ver 2.10.0.0

            // Ver 3.1.0.0
            /**************************************
             * GBMSAPI_GetImageSize3
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetImageSize3")]
            public static extern int GBMSAPI_GetImageSize3 (
		        UInt32	ObjectTypeToScan,
		        UInt32	OptionMask,
		        UInt32	ScanArea,
		        out UInt32	FullResImageSizeX,
		        out UInt32	FullResImageSizeY,
		        out UInt32	PreviewImageSizeX,
                out UInt32 PreviewImageSizeY
            );
            // end Ver 3.1.0.0

			// Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_GetCalibrationImageSize2" instead
			/**************************************
             * GBMSAPI_GetCalibrationImageSize
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetCalibrationImageSize")]
            public static extern int GBMSAPI_GetCalibrationImageSize(
                Int32 FlatScanArea,
                out UInt32 ImageSizeX,
                out UInt32 ImageSizeY
                );

			/**************************************
			 * GBMSAPI_GetCalibrationImageSize2
			**************************************/
			[DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetCalibrationImageSize2")]
			public static extern int GBMSAPI_GetCalibrationImageSize2(
				UInt32 ScanArea,
				out UInt32 ImageSizeX,
				out UInt32 ImageSizeY
				);
			// end Ver 2.10.0.0

            /**************************************
             * GBMSAPI_GetMaxImageSize
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetMaxImageSize")]
            public static extern int GBMSAPI_GetMaxImageSize(
                out UInt32 ImageMaxSizeX,
                out UInt32 ImageMaxSizeY
                );

            /**************************************
             * GBMSAPI_SetSelectImageTimeout
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetSelectImageTimeout")]
            public static extern int GBMSAPI_SetSelectImageTimeout(
                UInt32 SelectImageTimeout
                );

            /**************************************
             * GBMSAPI_SetLowerHalfPalmCompletenessThreshold
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetLowerHalfPalmCompletenessThreshold")]
            public static extern int GBMSAPI_SetLowerHalfPalmCompletenessThreshold(
                Byte CompletenessThreshold
                );

            // Ver 2300: SetFrameRate
            /**************************************
             * GBMSAPI_SetFrameRate
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetFrameRate")]
            public static extern int GBMSAPI_SetFrameRate(
                Int32 FlatScanArea,
                Double FrameRate
                );
            // end Ver 2300: SetFrameRate

            // Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_SetFrameRate2" instead
			// Ver 2700: GBMSAPI_SetFrameRateComplete
            /**************************************
             * GBMSAPI_SetFrameRateComplete
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetFrameRateComplete")]
            public static extern int GBMSAPI_SetFrameRateComplete(
                UInt32 FrameRateOptions,
                Double FrameRate
                );
            // end Ver 2700: GBMSAPI_SetFrameRateComplete
			
            /**************************************
             * GBMSAPI_SetFrameRate2
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetFrameRate2")]
            public static extern int GBMSAPI_SetFrameRate2(
				UInt32 ScanArea,
                UInt32 FrameRateOptions,
                Double FrameRate
                );
            // end Ver 2.10.0.0

            // Ver 2700: GBMSAPI_GetFrameRateRange
            /**************************************
             * GBMSAPI_GetFrameRateRange
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetFrameRateRange")]
            public static extern int GBMSAPI_GetFrameRateRange(
                UInt32 FrameRateOptions,
                out double MaxFrameRate,
                out double MinFrameRate,
                out double DefFrameRate
                );
            // end Ver 2700: GBMSAPI_GetFrameRateRange

			// Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_GetFrameRateRange2" instead
			// Ver 2800: GBMSAPI_GetFrameRateRange_Global
            /**************************************
             * GBMSAPI_GetFrameRateRange_Global
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetFrameRateRange_Global")]
            public static extern int GBMSAPI_GetFrameRateRange_Global(
                Byte DeviceID,
                UInt32 FrameRateOptions,
                UInt32 RollAreaStandard,
                out double MaxFrameRate,
                out double MinFrameRate,
                out double DefFrameRate
                );
            // end Ver 2800: GBMSAPI_GetFrameRateRange_Global
			
            /**************************************
             * GBMSAPI_GetFrameRateRange2
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetFrameRateRange2")]
            public static extern int GBMSAPI_GetFrameRateRange2(
                Byte DeviceID,
                UInt32 FrameRateOptions,
                UInt32 ScanArea,
                out double MaxFrameRate,
                out double MinFrameRate,
                out double DefFrameRate
                );
            // end Ver 2.10.0.0

            // Ver 2300: GBMSAPI_EnableManualLEDControl
            /**************************************
             * GBMSAPI_EnableManualLEDControl
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_EnableManualLEDControl")]
            public static extern int GBMSAPI_EnableManualLEDControl(
                Int32 Enable
                );
            // end Ver 2300: GBMSAPI_EnableManualLEDControl

            // Ver 3.3.0.0
            /**************************************
             * GBMSAPI_EnableLEDColorSettingDuringAcquisition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_EnableLEDColorSettingDuringAcquisition")]
            public static extern int GBMSAPI_EnableLEDColorSettingDuringAcquisition(
                Int32 Enable
                );
            // end Ver 3.3.0.0

            // Ver 2500: added Image rotation feature
            /**************************************
             * GBMSAPI_SetImageRotation
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetImageRotation")]
            public static extern int GBMSAPI_SetImageRotation(
                Int32 ImageRotation
                );
			// end Ver 2500: added Image rotation feature

			// VER 2.9.0.0: added dry skin image enhancement feature
			/**************************************
			 * GBMSAPI_EnableDrySkinImgEnhance
			**************************************/
			[DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_EnableDrySkinImgEnhance")]
			public static extern int GBMSAPI_EnableDrySkinImgEnhance(
				Byte Enable
				);
            // end VER 2.9.0.0: added dry skin image enhancement feature

            // VER 3.1.0.0
            /**************************************
             * GBMSAPI_ROLL_EnableRollStripeAcquisition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_EnableRollStripeAcquisition")]
            public static extern int GBMSAPI_ROLL_EnableRollStripeAcquisition(
                UInt32 ScanArea,
                Int32 Enable
                );
            // end VER 3.1.0.0

            // VER 3.2.0.0
            /**************************************
             * GBMSAPI_ROLL_EnableBlockComposition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_EnableBlockComposition")]
            public static extern int GBMSAPI_ROLL_EnableBlockComposition(Int32 Enable);
            // end VER 3.2.0.0

			// Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_StartAcquisition2" instead
			/**************************************
             * GBMSAPI_StartAcquisition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_StartAcquisition")]
            internal static extern int GBMSAPI_StartAcquisition(
                UInt32 ObjectToScan,
                UInt32 OptionMask,
                GBMSAPI_NET_AcquisitionEventsManagerCallbackInterface
                    AcquisitionEventsManagerCallbackPtr,
                IntPtr AcquisitionEventsManagerParameters,
                UInt32 DisplayOptionMask,
	            Byte ContrastLimitToDisplayOnLCD,
                Byte CompletenessLimitToDisplayOnLCD
                );
			
			/**************************************
             * GBMSAPI_StartAcquisition2
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_StartAcquisition2")]
            internal static extern int GBMSAPI_StartAcquisition2(
                UInt32 ObjectToScan,
                UInt32 OptionMask,
                UInt32 ScanArea,
                GBMSAPI_NET_AcquisitionEventsManagerCallbackInterface
                    AcquisitionEventsManagerCallbackPtr,
                IntPtr AcquisitionEventsManagerParameters,
                UInt32 DisplayOptionMask,
	            Byte ContrastLimitToDisplayOnLCD,
                Byte CompletenessLimitToDisplayOnLCD
                );
			// end Ver 2.10.0.0

            /**************************************
             * GBMSAPI_StopAcquisition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_StopPreview")]
            public static extern int GBMSAPI_ROLL_StopPreview();

            /**************************************
             * GBMSAPI_StopAcquisition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_StopAcquisition")]
            public static extern int GBMSAPI_StopAcquisition();

            /**************************************
             * GBMSAPI_SetClippingRegionSize
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetClippingRegionSize")]
            public static extern int GBMSAPI_SetClippingRegionSize(
                UInt32 ClippingRegionWidth,
	            UInt32 ClippingRegionHeight
                );

			// Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_SetDiagnosticSensitivity2" instead
			/**************************************
             * GBMSAPI_SetDiagnosticSensitivity
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetDiagnosticSensitivity")]
            public static extern int GBMSAPI_SetDiagnosticSensitivity(
                Int32 FlatScanArea,
	            Int32 ParameterType,
                IntPtr ParameterValuePtr
                );

			/**************************************
			 * GBMSAPI_SetDiagnosticSensitivity2
			**************************************/
			[DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetDiagnosticSensitivity2")]
			public static extern int GBMSAPI_SetDiagnosticSensitivity2(
				UInt32 FlatScanArea,
				Int32 ParameterType,
				IntPtr ParameterValuePtr
			);
			// end Ver 2.10.0.0

            /**************************************
             * GBMSAPI_ROLL_SetDiagnosticSensitivity
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_SetDiagnosticSensitivity")]
            public static extern int GBMSAPI_ROLL_SetDiagnosticSensitivity(
                Int32 ParameterType,
                IntPtr ParameterValuePtr
                );

            /**************************************
             * GBMSAPI_VUI_LED_BlinkDuringAcquisition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LED_BlinkDuringAcquisition")]
            public static extern int GBMSAPI_VUI_LED_BlinkDuringAcquisition(
                Int32 Blink
                );

            /**************************************
             * GBMSAPI_ROLL_SetPreviewTimeout
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_SetPreviewTimeout")]
            public static extern int GBMSAPI_ROLL_SetPreviewTimeout(
                UInt32 Timeout
                );

            /**************************************
             * GBMSAPI_ROLL_SetArtefactCleaningDepth
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_SetArtefactCleaningDepth")]
            public static extern int GBMSAPI_ROLL_SetArtefactCleaningDepth(
                Byte ArtefactCleaningDepth
                );

            // ver 3.1.0.2
            /**************************************
             * GBMSAPI_GetDryWetFingerAreaPercent
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetDryWetFingerAreaPercent")]
            public static extern int GBMSAPI_GetDryWetFingerAreaPercent(
                out byte Dry,
                out byte Wet
                );
            // end ver 3.1.0.2

            /**************************************
             * GBMSAPI_GetAutoCapturePhase
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetAutoCapturePhase")]
            public static extern int GBMSAPI_GetAutoCapturePhase(
                out Int32 AcquisitionPhase
                );

            /**************************************
             * GBMSAPI_SetAutoCaptureBlocking
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetAutoCaptureBlocking")]
            public static extern int GBMSAPI_SetAutoCaptureBlocking(
                Int32 Blocking
                );

            /**************************************
             * GBMSAPI_GetFingerprintContrast
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetFingerprintContrast")]
            public static extern int GBMSAPI_GetFingerprintContrast(
                out Byte Contrast
                );

            /**************************************
             * GBMSAPI_GetFingerprintSize
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetFingerprintSize")]
            public static extern int GBMSAPI_GetFingerprintSize(
                out UInt32 FingerprintSize
                );

            /**************************************
             * GBMSAPI_GetLowerHalfPalmCompleteness
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetLowerHalfPalmCompleteness")]
            public static extern int GBMSAPI_GetLowerHalfPalmCompleteness(
                out Byte LowerHalfPalmCompleteness
                );

            /**************************************
             * GBMSAPI_GetClippingRegionPosition
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetClippingRegionPosition")]
            public static extern int GBMSAPI_GetClippingRegionPosition(
                out Int32 ClippingRegionPosX,
                out Int32 ClippingRegionPosY,
                out UInt32 ClippingRegionSizeX,
                out UInt32 ClippingRegionSizeY
                );

            // VER 3.1.0.0
            /**************************************
             * GBMSAPI_ROLL_GetCurrentStripeCoord
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_GetCurrentStripeCoord")]
            public static extern int GBMSAPI_ROLL_GetCurrentStripeCoord(
                out Int32 LeftX,
                out Int32 RightX
               );

            /**************************************
             * GBMSAPI_UpdateBackgroundImage
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_UpdateBackgroundImage")]
            public static extern int GBMSAPI_UpdateBackgroundImage(
                IntPtr FramePtr,
                out UInt32 Diagnostic
               );
            // end VER 3.1.0.0

            /**************************************
             * GBMSAPI_ImageFinalization
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ImageFinalization")]
            public static extern int GBMSAPI_ImageFinalization(
                IntPtr FrameToFinalize
                );

            /**************************************
             * GBMSAPI_ColorReconstruction
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ColorReconstructionInByteArray")]
            public static extern int GBMSAPI_ColorReconstructionInByteArray(
                IntPtr MonochromeImage,
                Int32 SizeX, Int32 SizeY,
                IntPtr ColorImage,
                Double RedKoef, Double GreenKoef, Double BlueKoef
                );

            /**************************************
             * GBMSAPI_GetFrameStatistic
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetFrameStatistic")]
            public static extern int GBMSAPI_GetFrameStatistic(
                out UInt32 AcquiredFrames,
                out UInt32 LostFrames
                );

            // Ver 2.10.0.0
            // CONSIDER AS OBSOLETE: use "GBMSAPI_ROLL_GetCompositeImageInfo2" instead
            /**************************************
             * GBMSAPI_ROLL_GetCompositeImageInfo
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_GetCompositeImageInfo")]
            public static extern int GBMSAPI_ROLL_GetCompositeImageInfo(
                out UInt32 RolledArtefactSize,
                ref IntPtr MarkerFrame,
                ref IntPtr NotWipedArtefactFrame,
                out UInt32 CompositeFingerprintSize,
                out Byte CompositeFingerprintContrast,
                out UInt32 FlatFingerprintSize
                );

            /**************************************
             * GBMSAPI_ROLL_GetCompositeImageInfo2
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ROLL_GetCompositeImageInfo2")]
			public static extern int GBMSAPI_ROLL_GetCompositeImageInfo2(
				out UInt32 RolledArtefactSize,
				ref IntPtr MarkerFrame,
				ref IntPtr NotWipedArtefactFrame,
				out UInt32 CompositeFingerprintSize,
				out Byte CompositeFingerprintContrast,
				out UInt32 FlatFingerprintSize,
				out UInt32 ImageSizeX,
				out UInt32 ImageSizeY
				);
			// end Ver 2.10.0.0

			// Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_SetCalibrationImage2" instead
			/**************************************
             * GBMSAPI_SetCalibrationImage
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetCalibrationImage")]
            public static extern int GBMSAPI_SetCalibrationImage(
                Int32 FlatScanArea,
                IntPtr CalibrationImage
				);

			/**************************************
             * GBMSAPI_SetCalibrationImage2
            **************************************/
			[DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetCalibrationImage2")]
			public static extern int GBMSAPI_SetCalibrationImage2(
				UInt32 ScanArea,
				IntPtr CalibrationImage
			);
			// end Ver 2.10.0.0

			// Ver 2.10.0.0
			// CONSIDER AS OBSOLETE: use "GBMSAPI_GetCalibrationImage2" instead
			/**************************************
             * GBMSAPI_GetCalibrationImage
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetCalibrationImage")]
            public static extern int GBMSAPI_GetCalibrationImage(
                Int32 FlatScanArea,
                IntPtr CalibrationImage,
                out UInt32 CalibrationDiagnostic
                );

			/**************************************
			 * GBMSAPI_GetCalibrationImage2
			**************************************/
			[DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetCalibrationImage2")]
			public static extern int GBMSAPI_GetCalibrationImage2(
				UInt32 ScanArea,
				IntPtr CalibrationImage,
				out UInt32 CalibrationDiagnostic
				);
			// end Ver 2.10.0.0

            /**************************************
             * GBMSAPI_GetUnderlyingLibraryDllVersion
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetUnderlyingLibraryDllVersion", CharSet=CharSet.Ansi )]
            public static extern int GBMSAPI_GetUnderlyingLibraryDllVersion(
                out Byte VersionField1,
                out Byte VersionField2,
                out Byte VersionField3,
                out Byte VersionField4,
                StringBuilder LowLevelDllName
                );

            /**************************************
             * GBMSAPI_GetUSBErrorCode
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetUSBErrorCode")]
            public static extern int GBMSAPI_GetUSBErrorCode(
                out UInt32 USBErrorCode
                );

            /**************************************
             * GBMSAPI_GetScannerStatistics
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetScannerStatistics")]
            public static extern int GBMSAPI_GetScannerStatistics(
                out UInt32 Counter,
                out UInt32 ProductionDate
                );

            /**************************************
             * GBMSAPI_Sound
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_Sound")]
            public static extern int GBMSAPI_Sound(
                Byte Tone,
                Byte Duration, 
                Byte Iterations
                );

            // Ver 2700
            /**************************************
             * GBMSAPI_GetUserDataSize
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_GetUserDataSize")]
            public static extern int GBMSAPI_GetUserDataSize(
                out Int32 UserDataSize
                );

            // Ver 2700
            /**************************************
             * GBMSAPI_ReadUserData
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ReadUserData")]
            public static extern int GBMSAPI_ReadUserData(
		        UInt32 Address, 
                UInt32 Size,
                IntPtr Data
                );

            // Ver 2700
            /**************************************
             * GBMSAPI_WriteUserData
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_WriteUserData")]
            public static extern int GBMSAPI_WriteUserData(
                UInt32 Address,
                UInt32 Size,
                IntPtr Data
                );

            /**************************************
             * GBMSAPI_SetFingerIndicatorVUIState
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_SetFingerIndicatorVUIState")]
            public static extern int GBMSAPI_SetFingerIndicatorVUIState(
                UInt32  IndicatorMask,
                Int32   Color, 
                Int32   Blink
                );

            /**************************************
             * GBMSAPI_CheckPedalState
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_CheckPedalState")]
            public static extern int GBMSAPI_CheckPedalState(
                out Byte PedalState
                );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetLanguage
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetLanguage", CharSet = CharSet.Ansi)]
            public static extern int GBMSAPI_VUI_LCD_SetLanguage(
                String ISO639Lang
               );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_GetLcdFeatures
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_GetLcdFeatures")]
            public static extern int GBMSAPI_VUI_LCD_GetLcdFeatures(
                out UInt32 CompletenessIntermediateLimit
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetLogoScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetLogoScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetLogoScreen();

            /**************************************
             * GBMSAPI_VUI_LCD_SetScannedObject
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetScannedObject")]
            public static extern int GBMSAPI_VUI_LCD_SetScannedObject(
                UInt32 ObjToScan
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(
                Byte MessageToSet,
		        Byte MessageArea
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen", CharSet=CharSet.Ansi )]
            public static extern int GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(
                String MessageToSet,
		        Byte MessageToSetType
                );

            /**************************************
             * GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen" )]
            public static extern int GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();

            /**************************************
             * GBMSAPI_VUI_LCD_SetQualityOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetQualityOnStopScreen" )]
            public static extern int GBMSAPI_VUI_LCD_SetQualityOnStopScreen(
                Byte Quality
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetArtefactOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetArtefactOnStopScreen" )]
            public static extern int GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(
                int Artefact
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen" )]
            public static extern int GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(
                Byte QualityLimit,
                Byte Direction
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen" )]
            public static extern int GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(
                int ArtefactLimit,
                Byte Direction
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(
                Byte Seg1,
                Byte Seg2,
                Byte Seg3,
                Byte Seg4
                );

            /**************************************
             * GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen")]
            public static extern int GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();

            /**************************************
             * GBMSAPI_VUI_LCD_GetPressedButton
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_GetPressedButton")]
            public static extern int GBMSAPI_VUI_LCD_GetPressedButton(
                out Byte PressedButton
                );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit")]
            public static extern int GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit(
                Byte CompletenessIntermediateLimit
                );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastIntermediateLimit
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetContrastIntermediateLimit")]
            public static extern int GBMSAPI_VUI_LCD_SetContrastIntermediateLimit(
                Byte ContrastIntermediateLimit
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen(
                Byte ContrastValue
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen(
                Byte CompletenessValue
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(
                Byte ContrastValue
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(
                Byte CompletenessValue
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(
                Byte ContrastLimit
                );

            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(
                Byte CompletenessLimit
                );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(
                Byte QualityIntermediateLimit
                );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(
                Int32 ArtefactIntermediateLimit
                );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(
                Byte ContrastIntermediateLimit
                );

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen")]
            public static extern int GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(
                Byte CompletenessIntermediateLimit
				);

			// VER 2.9.0.1
			/**************************************
			 * GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen
			**************************************/
			[DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen")]
			public static extern int GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(
				Byte DiagMessageToVisualize
				);
            // end VER 2.9.0.1

            /**************************************
             * GBMSAPI_ConvertImageFrom1000to500DPI
            **************************************/
            [DllImport("GBMSAPI.dll", EntryPoint = "GBMSAPI_ConvertImageFrom1000to500DPI")]
            public static extern int GBMSAPI_ConvertImageFrom1000to500DPI(
                IntPtr Input1000DPIFrame,
                IntPtr Output500DPIFrame,
                UInt32 InputSizeX,
                UInt32 InputSizeY
                );

        }

        public class GBMSAPI_NET_DeviceSettingRoutines
        {
            /**************************************
             * GBMSAPI_GetAttachedDeviceList
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetAttachedDeviceList(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="AttachedDeviceList"></param>
            /// <param name="AttachedDeviceNumber"></param>
            /// <param name="USBErrorCode"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetAttachedDeviceList(
                GBMSAPI_NET_DeviceInfoStruct[] AttachedDeviceList,
                out Int32 AttachedDeviceNumber,
                out UInt32 USBErrorCode)
            {
                // check parameters
                if (AttachedDeviceList == null || AttachedDeviceList.Length < GBMSAPI_NET_DeviceInfoConstants.GBMSAPI_NET_MAX_PLUGGED_DEVICE_NUM)
                {
                    USBErrorCode = 0;
                    AttachedDeviceNumber = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_PARAMETER;
                }

                try
                {
                    GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_DEVICE_INFO_STRUCT[] WrapAttDevList = new GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_DEVICE_INFO_STRUCT
                        [GBMSAPI_NET_DeviceInfoConstants.GBMSAPI_NET_MAX_PLUGGED_DEVICE_NUM];
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetAttachedDeviceList(
                        WrapAttDevList, out AttachedDeviceNumber, out USBErrorCode);

                    for (int i = 0; i < AttachedDeviceNumber; i++)
                    {
                        AttachedDeviceList[i].DeviceID = WrapAttDevList[i].DeviceID;
                        AttachedDeviceList[i].DeviceSerialNumber = "" + WrapAttDevList[i].DeviceSerialNumber;
                    }

                    return RetVal;
                }
                catch (Exception ex)
                {
                    USBErrorCode = 0;
                    AttachedDeviceNumber = 0;
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetAttachedDeviceList: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_SetCurrentDevice
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetCurrentDevice(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="DeviceID"></param>
            /// <param name="DeviceSerialNumber"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetCurrentDevice(
                Byte DeviceID,
                String DeviceSerialNumber
               )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetCurrentDevice(
                        DeviceID, DeviceSerialNumber);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_SetCurrentDevice: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetCurrentDevice
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetCurrentDevice(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="DeviceID"></param>
            /// <param name="DeviceSerialNumber"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetCurrentDevice(
                out Byte DeviceID,
                out String DeviceSerialNumber
                )
            {
                try
                {
                    Int32 RetVal;

                    StringBuilder SB_LL_DLLName = new StringBuilder(GBMSAPI_NET_DeviceInfoConstants.GBMSAPI_NET_MAX_SN_LENGHT + 1);
                    if (SB_LL_DLLName == null)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_GetCurrentDevice: cannot alloc memory";
                        DeviceID = 0;
                        DeviceSerialNumber = null;

                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }

                    RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetCurrentDevice(
                        out DeviceID,
                        SB_LL_DLLName
                    );

                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        DeviceSerialNumber = SB_LL_DLLName.ToString();
                    }
                    else
                    {
                        DeviceID = 0;
                        DeviceSerialNumber = null;
                    }

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetCurrentDevice: " + ex.Message;
                    DeviceID = 0;
                    DeviceSerialNumber = null;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetDeviceNameAndVersion
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetDeviceNameAndVersion(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="DeviceID"></param>
            /// <param name="DeviceSerialNumber"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetDeviceNameAndVersion(
                out String NameAndVersion
                )
            {
                try
                {
                    Int32 RetVal;

                    StringBuilder SB_LL_DLLName = new StringBuilder(GBMSAPI_NET_DeviceInfoConstants.GBMSAPI_NET_MAX_CHARS_IN_DEVICE_NAME_AND_INFO);
                    if (SB_LL_DLLName == null)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_GetDeviceNameAndVersion: cannot alloc memory";
                        NameAndVersion = null;

                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }

                    RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetDeviceNameAndVersion(
                        SB_LL_DLLName
                    );

                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        NameAndVersion = SB_LL_DLLName.ToString();
                    }
                    else
                    {
                        NameAndVersion = null;
                    }

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetDeviceNameAndVersion: " + ex.Message;
                    NameAndVersion = null;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

        }

        public class GBMSAPI_NET_DeviceCharacteristicsRoutines
        {

            /**************************************
             * GBMSAPI_GetScannableTypes
            **************************************/
            /// <summary> 
            /// See GBMSAPI_GetScannableTypes(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ScannableTypesMask"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetScannableTypes(
                out UInt32 ScannableTypesMask
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetScannableTypes(out ScannableTypesMask);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetScannableTypes: " + ex.Message;
                    ScannableTypesMask = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			// Ver 2.10.0.0
			/**************************************
             * GBMSAPI_GetSupportedScanAreas
            **************************************/
			/// <summary>
			/// See GBMSAPI_GetSupportedScanAreas(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="SupportedScanAreas"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_GetSupportedScanAreas(
				out UInt32 SupportedScanAreas
                )
            {
                try
                {
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetSupportedScanAreas(out SupportedScanAreas);

                    return RetVal;
                }
                catch (Exception ex)
                {
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetSupportedScanAreas: " + ex.Message;
					SupportedScanAreas = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
			// end Ver 2.10.0.0

            // Ver 3.1.0.0
            /**************************************
             * GBMSAPI_GetSupportedScanAreasByDevice
            **************************************/
            public static Int32 GBMSAPI_NET_GetSupportedScanAreasByDevice(
                byte DeviceID,
                out UInt32 SupportedScanAreas
                )
            {
                SupportedScanAreas = 0;
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetSupportedScanAreasByDevice(DeviceID, out SupportedScanAreas);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetSupportedScanAreasByDevice: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end Ver 3.1.0.0
			
            /**************************************
             * GBMSAPI_GetDeviceFeatures
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetDeviceFeatures(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="DeviceFeatures"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetDeviceFeatures(
                out UInt32 DeviceFeatures
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetDeviceFeatures(out DeviceFeatures);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetDeviceFeatures: " + ex.Message;
                    DeviceFeatures = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }



            // VER 2.7.1.0
            /**************************************
             * GBMSAPI_SetRollAreaStandard
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetRollAreaStandard(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="DeviceFeatures"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetRollAreaStandard(
                UInt32 RollAreaStandard
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetRollAreaStandard(RollAreaStandard);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_SetRollAreaStandard: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetSupportedScanOptions
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetSupportedScanOptions(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="SupportedScanOptions"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetSupportedScanOptions(
                out UInt32 SupportedScanOptions
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetSupportedScanOptions(out SupportedScanOptions);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetSupportedScanOptions: " + ex.Message;
                    SupportedScanOptions = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetOptionalExternalEquipment
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetOptionalExternalEquipment(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="OptionalExternalEquipment"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetOptionalExternalEquipment(
                out UInt32 OptionalExternalEquipment
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetOptionalExternalEquipment(out OptionalExternalEquipment);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetOptionalExternalEquipment: " + ex.Message;
                    OptionalExternalEquipment = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
            * GBMSAPI_GetAvailableImageInfo
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetAvailableImageInfo(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ObjectTypeToScan"></param>
            /// <param name="AvailableImageInfo"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetAvailableImageInfo(
                UInt32 ObjectTypeToScan,
                out UInt32 AvailableImageInfo
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetAvailableImageInfo(ObjectTypeToScan, out AvailableImageInfo);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetAvailableImageInfo: " + ex.Message;
                    AvailableImageInfo = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "GBMSAPI_NET_GetImageSize2" instead
            /**************************************
             * GBMSAPI_GetImageSize
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetImageSize(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ObjectTypeToScan"></param>
            /// <param name="FlatScanArea"></param>
            /// <param name="Preview"></param>
            /// <param name="ImageSizeX"></param>
            /// <param name="ImageSizeY"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetImageSize(
                UInt32 ObjectTypeToScan,
                Boolean FlatScanArea,
                Boolean Preview,
                out UInt32 ImageSizeX,
                out UInt32 ImageSizeY
                )
            {
                try
                {
                    Int32 FSA = (FlatScanArea == true) ? 1 : 0;
                    Int32 PVw = (Preview == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetImageSize(
                        ObjectTypeToScan,
                        FSA,
                        PVw,
                        out ImageSizeX,
                        out ImageSizeY
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetImageSize: " + ex.Message;
                    ImageSizeX = 0;
                    ImageSizeY = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
			}

			/**************************************
			 * GBMSAPI_GetImageSize2
			**************************************/
			/// <summary>
			/// See GBMSAPI_GetImageSize2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="ObjectTypeToScan"></param>
			/// <param name="ScanArea"></param>
			/// <param name="Preview"></param>
			/// <param name="ImageSizeX"></param>
			/// <param name="ImageSizeY"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_GetImageSize2(
				UInt32 ObjectTypeToScan,
				UInt32 ScanArea,
				Boolean Preview,
				out UInt32 ImageSizeX,
				out UInt32 ImageSizeY
				)
			{
				try
				{
					Int32 PVw = (Preview == true) ? 1 : 0;
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetImageSize2(
						ObjectTypeToScan,
						ScanArea,
						PVw,
						out ImageSizeX,
						out ImageSizeY
						);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetImageSize2: " + ex.Message;
					ImageSizeX = 0;
					ImageSizeY = 0;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
			// end Ver 2.10.0.0

            // Ver 3.1.0.0
            /**************************************
             * GBMSAPI_GetImageSize3
            **************************************/
            public static Int32 GBMSAPI_NET_GetImageSize3(
                UInt32 ObjectTypeToScan,
                UInt32 OptionMask,
                UInt32 ScanArea,
                out UInt32 FullResImageSizeX,
                out UInt32 FullResImageSizeY,
                out UInt32 PreviewImageSizeX,
                out UInt32 PreviewImageSizeY
            )
            {
                FullResImageSizeX = 0;
                FullResImageSizeY = 0;
                PreviewImageSizeX = 0;
                PreviewImageSizeY = 0;
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetImageSize3(
                        ObjectTypeToScan,
                        OptionMask,
                        ScanArea,
                        out FullResImageSizeX,
                        out FullResImageSizeY,
                        out PreviewImageSizeX,
                        out PreviewImageSizeY
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetImageSize3: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end Ver 3.1.0.0

			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "GBMSAPI_GetCalibrationImageSize2" instead
			/**************************************
			 * GBMSAPI_GetCalibrationImageSize
			**************************************/
            /// <summary>
            /// See GBMSAPI_GetCalibrationImageSize(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="ImageSizeX"></param>
            /// <param name="ImageSizeY"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetCalibrationImageSize(
                Boolean FlatScanArea,
                out UInt32 ImageSizeX,
                out UInt32 ImageSizeY
                )
            {
                try
                {
                    Int32 FSA = (FlatScanArea == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetCalibrationImageSize(
                        FSA,
                        out ImageSizeX,
                        out ImageSizeY
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetCalibrationImageSize: " + ex.Message;
                    ImageSizeX = 0;
                    ImageSizeY = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			/**************************************
			 * GBMSAPI_GetCalibrationImageSize2
			**************************************/
			/// <summary>
			/// See GBMSAPI_GetCalibrationImageSize2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="ScanArea"></param>
			/// <param name="ImageSizeX"></param>
			/// <param name="ImageSizeY"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_GetCalibrationImageSize2(
				UInt32 ScanArea,
				out UInt32 ImageSizeX,
				out UInt32 ImageSizeY
				)
			{
				try
				{
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetCalibrationImageSize2(
						ScanArea,
						out ImageSizeX,
						out ImageSizeY
						);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetCalibrationImageSize2: " + ex.Message;
					ImageSizeX = 0;
					ImageSizeY = 0;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
			// end Ver 2.10.0.0

            /**************************************
             * GBMSAPI_GetMaxImageSize
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetMaxImageSize(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ImageMaxSizeX"></param>
            /// <param name="ImageMaxSizeY"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetMaxImageSize(
                out UInt32 ImageMaxSizeX,
                out UInt32 ImageMaxSizeY
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetMaxImageSize(
                        out ImageMaxSizeX,
                        out ImageMaxSizeY
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_GetMaxImageSize: " + ex.Message;
                    ImageMaxSizeX = 0;
                    ImageMaxSizeY = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }


        }

        public class GBMSAPI_NET_ScanSettingsRoutines
        {
            
            /**************************************
             * GBMSAPI_SetSelectImageTimeout
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetSelectImageTimeout(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="SelectImageTimeout"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetSelectImageTimeout(
                UInt32 SelectImageTimeout
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetSelectImageTimeout(
                        SelectImageTimeout
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING = "GBMSAPI_NET_SetSelectImageTimeout: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /*********************************************
             * GBMSAPI_SetLowerHalfPalmCompletenessThreshold
            *********************************************/
            /// <summary>
            /// See GBMSAPI_SetLowerHalfPalmCompletenessThreshold(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="CompletenessThreshold"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetLowerHalfPalmCompletenessThreshold(
                Byte CompletenessThreshold
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetLowerHalfPalmCompletenessThreshold(
                        CompletenessThreshold
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetLowerHalfPalmCompletenessThreshold: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // Ver 2300: SetFrameRate
            /**************************************
             * GBMSAPI_SetFrameRate
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetFrameRate(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="FrameRate"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetFrameRate(
                Boolean FlatScanArea,
                Double FrameRate
                )
            {
                try
                {
                    Int32 FSA = (FlatScanArea == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetFrameRate(
                        FSA,FrameRate
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetFrameRate: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
			// end Ver 2300: SetFrameRate

			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "SetFrameRate2" instead
            // Ver 2700: GBMSAPI_SetFrameRateComplete
            /**************************************
             * GBMSAPI_SetFrameRateComplete
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetFrameRateComplete(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="FrameRate"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetFrameRateComplete(
                UInt32 FrameRateOptions,
                Double FrameRate
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetFrameRateComplete(
                        FrameRateOptions, FrameRate
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetFrameRateComplete: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end Ver 2700: GBMSAPI_SetFrameRateComplete

			/**************************************
			 * GBMSAPI_SetFrameRate2
			**************************************/
			/// <summary>
			/// See GBMSAPI_SetFrameRate2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="FlatScanArea"></param>
			/// <param name="FrameRate"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_SetFrameRate2(
				UInt32 ScanArea,
				UInt32 FrameRateOptions,
				Double FrameRate
				)
			{
				try
				{
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetFrameRate2(
						ScanArea, FrameRateOptions, FrameRate
						);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_SetFrameRate2: " + ex.Message;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
            // end Ver 2.10.0.0

			// VER 2800: CONSIDER THIS FUNCTION AS OBSOLETE: use GBMSAPI_GetFrameRateRange_Global instead
			// Ver 2700: GBMSAPI_NET_GetFrameRateRange
			/**************************************
			 * GBMSAPI_GetFrameRateRange
			**************************************/
            /// <summary>
            /// See GBMSAPI_GetFrameRateRange(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="FrameRate"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetFrameRateRange(
                UInt32 FrameRateOptions,
                out double MaxFrameRate,
                out double MinFrameRate,
                out double DefFrameRate
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetFrameRateRange(
                        FrameRateOptions,
                        out MaxFrameRate,
                        out MinFrameRate,
                        out DefFrameRate
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetFrameRateRange: " + ex.Message;
                    MaxFrameRate = 0;
                    MinFrameRate = 0;
                    DefFrameRate = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end Ver 2700: GBMSAPI_GetFrameRateRange

			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "GBMSAPI_GetFrameRateRange2" instead
			// VER 2800: added GBMSAPI_GetFrameRateRange_Global(...)
            /**************************************
             * GBMSAPI_GetFrameRateRange_Global
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetFrameRateRange(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="FrameRate"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetFrameRateRange_Global(
                Byte DeviceID,
                UInt32 FrameRateOptions,
                UInt32 RollAreaStandard,
                out double MaxFrameRate,
                out double MinFrameRate,
                out double DefFrameRate
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetFrameRateRange_Global(
                        DeviceID,
                        FrameRateOptions,
                        RollAreaStandard,
                        out MaxFrameRate,
                        out MinFrameRate,
                        out DefFrameRate
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetFrameRateRange: " + ex.Message;
                    MaxFrameRate = 0;
                    MinFrameRate = 0;
                    DefFrameRate = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end VER 2800: added GBMSAPI_GetFrameRateRange_Global(...)

			/**************************************
			 * GBMSAPI_GetFrameRateRange2
			**************************************/
			/// <summary>
			/// See GBMSAPI_GetFrameRateRange2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="ScanArea"></param>
			/// <param name="FrameRate"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_GetFrameRateRange2(
				Byte DeviceID,
				UInt32 FrameRateOptions,
				UInt32 ScanArea,
				out double MaxFrameRate,
				out double MinFrameRate,
				out double DefFrameRate
				)
			{
				try
				{
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetFrameRateRange2(
						DeviceID,
						FrameRateOptions,
						ScanArea,
						out MaxFrameRate,
						out MinFrameRate,
						out DefFrameRate
						);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_GetFrameRateRange2: " + ex.Message;
					MaxFrameRate = 0;
					MinFrameRate = 0;
					DefFrameRate = 0;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
            // end Ver 2.10.0.0

            // Ver 2300: GBMSAPI_EnableManualLEDControl
            /**************************************
             * GBMSAPI_EnableManualLEDControl
            **************************************/
            /// <summary>
            /// See GBMSAPI_EnableManualLEDControl(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Enable"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_EnableManualLEDControl(
                bool Enable
                )
            {
                try
                {
                    Int32 EnInt32 = (Enable == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_EnableManualLEDControl(
                        EnInt32
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_EnableManualLEDControl: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end Ver 2300: GBMSAPI_EnableManualLEDControl

            // Ver 3.3.0.0
            /**************************************
             * GBMSAPI_EnableLEDColorSettingDuringAcquisition
            **************************************/
            /// <summary>
            /// See GBMSAPI_EnableLEDColorSettingDuringAcquisition(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Enable"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_EnableLEDColorSettingDuringAcquisition(
                bool Enable
                )
            {
                try
                {
                    Int32 EnInt32 = (Enable == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_EnableLEDColorSettingDuringAcquisition(
                        EnInt32
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_EnableLEDColorSettingDuringAcquisition: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end Ver 3.3.0.0

            // Ver 2500: added Image rotation feature
            /**************************************
             * GBMSAPI_SetImageRotation
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetImageRotation(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Enable"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetImageRotation(
                Int32 ImageRotation
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetImageRotation(
                        ImageRotation
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetImageRotation: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end Ver 2500: added Image rotation feature

			// VER 2.9.0.0: added dry skin image enhancement feature
			/**************************************
			 * GBMSAPI_EnableDrySkinImgEnhance
			**************************************/
			/// <summary>
			/// See GBMSAPI_EnableDrySkinImgEnhance(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="Enable"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_EnableDrySkinImgEnhance(
				bool Enable
			)
			{
				try
				{
					Byte bEnable = Enable ? (Byte)1 : (Byte)0;
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_EnableDrySkinImgEnhance(
						bEnable
						);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_EnableDrySkinImgEnhance: " + ex.Message;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
            // end VER 2.9.0.0: added dry skin image enhancement feature

            // VER 3.1.0.0
            /**************************************
             * GBMSAPI_ROLL_EnableRollStripeAcquisition
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_EnableRollStripeAcquisition(...) function in GBMSAPI_Library.h
            /// </summary>
            /// <param name="ScanArea"></param>
            /// <param name="MinimumSlicePercent"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_EnableRollStripeAcquisition(
                UInt32 ScanArea,
                bool Enable
            )
            {
                try
                {
                    Int32 enInt = Enable ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_EnableRollStripeAcquisition(
                        ScanArea, enInt
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_EnableRollStripeAcquisition: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end VER 3.1.0.0

            // VER 3.2.0.0
            /**************************************
             * GBMSAPI_NET_ROLL_EnableBlockComposition
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_EnableBlockComposition(...) function in GBMSAPI_Library.h
            /// </summary>
            /// <param name="Enable"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_EnableBlockComposition(bool Enable)
            {
                try
                {
                    Int32 enInt = Enable ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_EnableBlockComposition(enInt);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_EnableBlockComposition: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end VER 3.2.0.0
        }

        public class GBMSAPI_NET_ScanningRoutines
        {
            internal static GBMSAPI_NET_AcquisitionEventsManagerCallbackInterface Callback = null;

			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "GBMSAPI_StartAcquisition2" instead
			/**************************************
             * GBMSAPI_StartAcquisition
            **************************************/
            /// <summary>
            /// See GBMSAPI_StartAcquisition(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ObjectToScan"></param>
            /// <param name="OptionMask"></param>
            /// <param name="AcquisitionEventsManagerCallbackPtr"></param>
            /// <param name="AcquisitionEventsManagerParameters"></param>
            /// <param name="DisplayOptionMask"></param>
            /// <param name="ContrastLimitToDisplayOnLCD"></param>
            /// <param name="CompletenessLimitToDisplayOnLCD"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_StartAcquisition(
                UInt32 ObjectToScan,
                UInt32 OptionMask,
                GBMSAPI_NET_AcquisitionEventsManagerCallback AcquisitionEventsManagerCallbackPtr,
                IntPtr AcquisitionEventsManagerParameters,
                UInt32 DisplayOptionMask,
                Byte ContrastLimitToDisplayOnLCD,
                Byte CompletenessLimitToDisplayOnLCD
                )
            {
                try
                {
                    GBMSAPI_NET_ScanningRoutines.Callback = new 
                        GBMSAPI_NET_AcquisitionEventsManagerCallbackInterface(
                        GBMSAPI_AcquisitionCallbackInterface.MyAcquisitionEventsManagerCallbackInterface);
                    GBMSAPI_AcquisitionCallbackInterface.FunctionToBeCalled = AcquisitionEventsManagerCallbackPtr;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_StartAcquisition(
                        ObjectToScan,
                        OptionMask,
                        GBMSAPI_NET_ScanningRoutines.Callback,
                        AcquisitionEventsManagerParameters,
                        DisplayOptionMask,
                        ContrastLimitToDisplayOnLCD,
                        CompletenessLimitToDisplayOnLCD
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_StartAcquisition: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			/**************************************
			 * GBMSAPI_StartAcquisition2
			**************************************/
			/// <summary>
			/// See GBMSAPI_StartAcquisition2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="ObjectToScan"></param>
			/// <param name="OptionMask"></param>
			/// <param name="ScanArea"></param>
			/// <param name="AcquisitionEventsManagerCallbackPtr"></param>
			/// <param name="AcquisitionEventsManagerParameters"></param>
			/// <param name="DisplayOptionMask"></param>
			/// <param name="ContrastLimitToDisplayOnLCD"></param>
			/// <param name="CompletenessLimitToDisplayOnLCD"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_StartAcquisition2(
				UInt32 ObjectToScan,
				UInt32 OptionMask,
				UInt32 ScanArea,
				GBMSAPI_NET_AcquisitionEventsManagerCallback AcquisitionEventsManagerCallbackPtr,
				IntPtr AcquisitionEventsManagerParameters,
				UInt32 DisplayOptionMask,
				Byte ContrastLimitToDisplayOnLCD,
				Byte CompletenessLimitToDisplayOnLCD
				)
			{
				try
				{
					GBMSAPI_NET_ScanningRoutines.Callback = new
						GBMSAPI_NET_AcquisitionEventsManagerCallbackInterface(
						GBMSAPI_AcquisitionCallbackInterface.MyAcquisitionEventsManagerCallbackInterface);
					GBMSAPI_AcquisitionCallbackInterface.FunctionToBeCalled = AcquisitionEventsManagerCallbackPtr;
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_StartAcquisition2(
						ObjectToScan,
						OptionMask,
						ScanArea,
						GBMSAPI_NET_ScanningRoutines.Callback,
						AcquisitionEventsManagerParameters,
						DisplayOptionMask,
						ContrastLimitToDisplayOnLCD,
						CompletenessLimitToDisplayOnLCD
					);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_StartAcquisition2: " + ex.Message;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
			// Ver 2.10.0.0

            /**************************************
             * GBMSAPI_ROLL_StopPreview
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_StopPreview(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_StopPreview()
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_StopPreview();

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_StopPreview: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_StopAcquisition
            **************************************/
            /// <summary>
            /// See GBMSAPI_StopAcquisition(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_StopAcquisition()
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_StopAcquisition();

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_StopAcquisition: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
        }

        public class GBMSAPI_NET_ScannerStartedRoutines
        {
            /**************************************
             * GBMSAPI_SetClippingRegionSize
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetClippingRegionSize(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ClippingRegionWidth"></param>
            /// <param name="ClippingRegionHeight"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetClippingRegionSize(
                UInt32 ClippingRegionWidth,
                UInt32 ClippingRegionHeight
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetClippingRegionSize(
                        ClippingRegionWidth,
                        ClippingRegionHeight
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetClippingRegionSize: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "GBMSAPI_SetDiagnosticSensitivity2" instead
			/**************************************
             * GBMSAPI_SetDiagnosticSensitivity
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetDiagnosticSensitivity(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="ParameterType"></param>
            /// <param name="ParameterValuePtr"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetDiagnosticSensitivity(
                Int32 FlatScanArea,
                Int32 ParameterType,
                IntPtr ParameterValuePtr
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetDiagnosticSensitivity(
                        FlatScanArea,
                        ParameterType,
                        ParameterValuePtr
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetDiagnosticSensitivity: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			/**************************************
			 * GBMSAPI_SetDiagnosticSensitivity2
			**************************************/
			/// <summary>
			/// See GBMSAPI_SetDiagnosticSensitivity2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="ScanArea"></param>
			/// <param name="ParameterType"></param>
			/// <param name="ParameterValuePtr"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_SetDiagnosticSensitivity2(
				UInt32 ScanArea,
				Int32 ParameterType,
				IntPtr ParameterValuePtr
				)
			{
				try
				{
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetDiagnosticSensitivity2(
						ScanArea,
						ParameterType,
						ParameterValuePtr
					);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_SetDiagnosticSensitivity2: " + ex.Message;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
			// end Ver 2.10.0.0

            /**************************************
             * GBMSAPI_ROLL_SetDiagnosticSensitivity
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_SetDiagnosticSensitivity(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ParameterType"></param>
            /// <param name="ParameterValuePtr"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_SetDiagnosticSensitivity(
                Int32 ParameterType,
                IntPtr ParameterValuePtr
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_SetDiagnosticSensitivity(
                        ParameterType,
                        ParameterValuePtr
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_SetDiagnosticSensitivity: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_ROLL_SetPreviewTimeout
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_SetPreviewTimeout(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Timeout"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_SetPreviewTimeout(
                UInt32 Timeout
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_SetPreviewTimeout(
                        Timeout
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_SetPreviewTimeout: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_ROLL_SetArtefactCleaningDepth
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_SetArtefactCleaningDepth(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ArtefactCleaningDepth"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_SetArtefactCleaningDepth(
                Byte ArtefactCleaningDepth
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_SetArtefactCleaningDepth(
                        ArtefactCleaningDepth
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_SetArtefactCleaningDepth: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
        }

        public class GBMSAPI_NET_ValidFrameAcquiredRoutines
        {
            /**************************************
             * GBMSAPI_VUI_LED_BlinkDuringAcquisition
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LED_BlinkDuringAcquisition(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Blink"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LED_BlinkDuringAcquisition(
                Int32 Blink
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LED_BlinkDuringAcquisition(
                        Blink
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LED_BlinkDuringAcquisition: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }



            // ver 3.1.0.2
            /**************************************
             * GBMSAPI_GetDryWetFingerAreaPercent
            **************************************/
            public static Int32 GBMSAPI_NET_GetDryWetFingerAreaPercent(
                out byte Dry,
                out byte Wet
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetDryWetFingerAreaPercent(
                        out Dry, out Wet
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetDryWetFingerAreaPercent: " + ex.Message;
                    Dry = 0;
                    Wet = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end ver 3.1.0.2

            /**************************************
             * GBMSAPI_GetAutoCapturePhase
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetAutoCapturePhase(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="AcquisitionPhase"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetAutoCapturePhase(
                out Int32 AcquisitionPhase
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetAutoCapturePhase(
                        out AcquisitionPhase
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetAutoCapturePhase: " + ex.Message;
                    AcquisitionPhase = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_SetAutoCaptureBlocking
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetAutoCaptureBlocking(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Blocking"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetAutoCaptureBlocking(
                Boolean Blocking
                )
            {
                try
                {
                    Int32 BlockParam = (Blocking == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetAutoCaptureBlocking(
                        BlockParam
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetAutoCaptureBlocking: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetFingerprintContrast
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetFingerprintContrast(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Contrast"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetFingerprintContrast(
                out Byte Contrast
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetFingerprintContrast(
                        out Contrast
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetFingerprintContrast: " + ex.Message;
                    Contrast = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetFingerprintSize
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetFingerprintSize(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FingerprintSize"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetFingerprintSize(
                out UInt32 FingerprintSize
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetFingerprintSize(
                        out FingerprintSize
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetFingerprintSize: " + ex.Message;
                    FingerprintSize = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetLowerHalfPalmCompleteness
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetLowerHalfPalmCompleteness(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="LowerHalfPalmCompleteness"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetLowerHalfPalmCompleteness(
                out Byte LowerHalfPalmCompleteness
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetLowerHalfPalmCompleteness(
                        out LowerHalfPalmCompleteness
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetLowerHalfPalmCompleteness: " + ex.Message;
                    LowerHalfPalmCompleteness = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetClippingRegionPosition
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetClippingRegionPosition(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ClippingRegionPosX"></param>
            /// <param name="ClippingRegionPosY"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetClippingRegionPosition(
                out Int32 ClippingRegionPosX,
                out Int32 ClippingRegionPosY,
                out UInt32 ClippingRegionSizeX,
                out UInt32 ClippingRegionSizeY
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetClippingRegionPosition(
                        out ClippingRegionPosX,
                        out ClippingRegionPosY,
                        out ClippingRegionSizeX,
                        out ClippingRegionSizeY
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetClippingRegionPosition: " + ex.Message;
                    ClippingRegionPosX = 0;
                    ClippingRegionPosY = 0;
                    ClippingRegionSizeX = 0;
                    ClippingRegionSizeY = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 3.1.0.0
            /**************************************
             * GBMSAPI_ROLL_GetCurrentStripeCoord
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_GetCurrentStripeCoord(...) function in GBMSAPI_Library.h
            /// </summary>
            /// <param name="LeftX"></param>
            /// <param name="RightX"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_GetCurrentStripeCoord(
                out Int32 LeftX,
                out Int32 RightX
            )
            {
                LeftX = -1;
                RightX = -1;
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_GetCurrentStripeCoord(
                        out LeftX, out RightX
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_GetCurrentStripeCoord: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }



            /**************************************
             * GBMSAPI_UpdateBackgroundImage
            **************************************/
            public static Int32 GBMSAPI_NET_UpdateBackgroundImage(
                byte [] BackgroundImage,
                out UInt32 Diagnostic
            )
            {
                Diagnostic = 0;
                try
                {
                    IntPtr FramePtr = Marshal.AllocHGlobal(BackgroundImage.Length);
                    if (FramePtr == IntPtr.Zero)
                    {
                        throw new Exception("Cannot alloc space for FramePtr");
                    }
                    Marshal.Copy(BackgroundImage, 0, FramePtr, BackgroundImage.Length);
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_UpdateBackgroundImage(
                        FramePtr, out Diagnostic
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_UpdateBackgroundImage: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
            // end VER 3.1.0.0
        }

        public class GBMSAPI_NET_EndAcquisitionRoutines
        {
            /**************************************
             * GBMSAPI_ImageFinalization
            **************************************/
            /// <summary>
            /// See GBMSAPI_ImageFinalization(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FrameToFinalize"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ImageFinalization(
                Byte[] FrameToFinalize
                )
            {
                try
                {
                    IntPtr ParamForDLL = Marshal.AllocHGlobal(FrameToFinalize.Length);
                    if (ParamForDLL == IntPtr.Zero)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_ImageFinalization: Cannot alloc memory";
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }
                    Marshal.Copy(FrameToFinalize, 0, ParamForDLL, FrameToFinalize.Length);
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ImageFinalization(
                       ParamForDLL
                    );

                    Marshal.Copy(ParamForDLL, FrameToFinalize, 0, FrameToFinalize.Length);
                    Marshal.FreeHGlobal(ParamForDLL);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ImageFinalization: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_ColorReconstruction
            **************************************/
            /// <summary>
            /// Returns a byte array in RGB format. For other parameters
            /// see GBMSAPI_ColorReconstruction(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="MonochromeImage">
            /// input image
            /// </param>
            /// <param name="SizeX"></param>
            /// <param name="SizeY"></param>
            /// <param name="ColorImage">
            /// returned byte array: no need of external allocation, its
            /// size will be >= 3* <paramref name="SizeX"/> * <paramref name="SizeY"/>
            /// </param>
            /// <param name="RedKoef"></param>
            /// <param name="GreenKoef"></param>
            /// <param name="BlueKoef"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ColorReconstruction(
                Byte[] MonochromeImage,
                Int32 SizeX, Int32 SizeY,
                out Byte [] ColorImage,
                Double RedKoef, Double GreenKoef, Double BlueKoef
                )
            {
                try
                {
                    if (MonochromeImage == null || SizeX <= 0 || SizeY <= 0)
                    {
                        ColorImage = null;
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_PARAMETER;
                    }

                    // alloc memory for struct
                    IntPtr Dest = Marshal.AllocHGlobal(3 * SizeX * SizeY);

                    // transform from array to IntPtr
                    IntPtr Src = Marshal.AllocHGlobal(MonochromeImage.Length);
                    if (Src == IntPtr.Zero || Dest == IntPtr.Zero)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                               "GBMSAPI_NET_ColorReconstruction: Cannot alloc memory";
                        ColorImage = null;
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }
                    Marshal.Copy(MonochromeImage, 0, Src, MonochromeImage.Length);

                    // call function
                    int RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ColorReconstructionInByteArray(
                        Src, SizeX, SizeY, Dest, RedKoef, GreenKoef, BlueKoef);
                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        ColorImage = new Byte[3 * SizeX * SizeY];
                        if (ColorImage == null)
                        {
                            GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                                   "GBMSAPI_NET_ColorReconstruction: Cannot alloc memory";
                            ColorImage = null;
                            return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                        }
                        Marshal.Copy(Dest, ColorImage, 0, ColorImage.Length);
                    }
                    else
                    {
                        ColorImage = null;
                    }

                    Marshal.FreeHGlobal(Src);
                    Marshal.FreeHGlobal(Dest);
                    
                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ColorReconstruction: " + ex.Message;
                    ColorImage = null;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }



            /**************************************
             * GBMSAPI_GetFrameStatistic
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetFrameStatistic(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="AcquiredFrames"></param>
            /// <param name="LostFrames"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetFrameStatistic(
                out UInt32 AcquiredFrames,
                out UInt32 LostFrames
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetFrameStatistic(
                        out AcquiredFrames,
                        out LostFrames
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetFrameStatistic: " + ex.Message;
                    AcquiredFrames = 0;
                    LostFrames = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_ROLL_GetCompositeImageInfo
            **************************************/
            /// <summary>
            /// See GBMSAPI_ROLL_GetCompositeImageInfo(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="RolledArtefactSize"></param>
            /// <param name="MarkerFrame"></param>
            /// <param name="NotWipedArtefactFrame"></param>
            /// <param name="CompositeFingerprintSize"></param>
            /// <param name="CompositeFingerprintContrast"></param>
            /// <param name="FlatFingerprintSize"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ROLL_GetCompositeImageInfo(
                out UInt32 RolledArtefactSize,
                out Byte[] MarkerFrame,
                out Byte[] NotWipedArtefactFrame,
                out UInt32 CompositeFingerprintSize,
                out Byte CompositeFingerprintContrast,
                out UInt32 FlatFingerprintSize
                )
            {
                try
                {
                    //////////////////////////
                    // Get Image Size for Roll
                    //////////////////////////
                    UInt32 ImSX, ImSY;
                    GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetImageSize(
                        GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLL_SINGLE_FINGER,
                        false,
                        false,
                        out ImSX,out ImSY);

                    ///////////////////////////
                    // Prepare IntPtr
                    ///////////////////////////
                    IntPtr MarkerFramePtr = new IntPtr(), NotWipedArtefactFramePtr = new IntPtr();

                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_GetCompositeImageInfo(
                        out RolledArtefactSize,
                        ref MarkerFramePtr,
                        ref NotWipedArtefactFramePtr,
                        out CompositeFingerprintSize,
                        out CompositeFingerprintContrast,
                        out FlatFingerprintSize
                    );
                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        MarkerFrame = new Byte[ImSX * ImSY];
                        NotWipedArtefactFrame = new Byte[ImSX * ImSY];

                        if (MarkerFrame == null || NotWipedArtefactFrame == null)
                        {
                            GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                                "GBMSAPI_NET_ROLL_GetCompositeImageInfo: Cannot alloc memory";
                            RolledArtefactSize = 0;
                            MarkerFrame = null;
                            NotWipedArtefactFrame = null;
                            CompositeFingerprintSize = 0;
                            CompositeFingerprintContrast = 0;
                            FlatFingerprintSize = 0;
                            return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                        }

                        Marshal.Copy(MarkerFramePtr, MarkerFrame, 0, (int)(ImSY * ImSX));
                        Marshal.Copy(NotWipedArtefactFramePtr, NotWipedArtefactFrame, 0, (int)(ImSY * ImSX));
                    }
                    else
                    {
                        RolledArtefactSize = 0;
                        MarkerFrame = null;
                        NotWipedArtefactFrame = null;
                        CompositeFingerprintSize = 0;
                        CompositeFingerprintContrast = 0;
                        FlatFingerprintSize = 0;
                    }

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ROLL_GetCompositeImageInfo: " + ex.Message;
                    RolledArtefactSize = 0;
                    MarkerFrame = null;
                    NotWipedArtefactFrame = null;
                    CompositeFingerprintSize = 0;
                    CompositeFingerprintContrast = 0;
                    FlatFingerprintSize = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			// Ver 2.10.0.0
			/**************************************
			 * GBMSAPI_ROLL_GetCompositeImageInfo2
			**************************************/
			/// <summary>
			/// See GBMSAPI_ROLL_GetCompositeImageInfo2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="RolledArtefactSize"></param>
			/// <param name="MarkerFrame"></param>
			/// <param name="NotWipedArtefactFrame"></param>
			/// <param name="CompositeFingerprintSize"></param>
			/// <param name="CompositeFingerprintContrast"></param>
			/// <param name="FlatFingerprintSize"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_ROLL_GetCompositeImageInfo2(
				out UInt32 RolledArtefactSize,
				out Byte[] MarkerFrame,
				out Byte[] NotWipedArtefactFrame,
				out UInt32 CompositeFingerprintSize,
				out Byte CompositeFingerprintContrast,
				out UInt32 FlatFingerprintSize,
				out UInt32 ImageSizeX,
				out UInt32 ImageSizeY
				)
			{
				try
				{
					///////////////////////////
					// Prepare IntPtr
					///////////////////////////
					IntPtr MarkerFramePtr = new IntPtr(), NotWipedArtefactFramePtr = new IntPtr();

					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ROLL_GetCompositeImageInfo2(
						out RolledArtefactSize,
						ref MarkerFramePtr,
						ref NotWipedArtefactFramePtr,
						out CompositeFingerprintSize,
						out CompositeFingerprintContrast,
						out FlatFingerprintSize,
						out ImageSizeX,
						out ImageSizeY
					);
					if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
					{
						MarkerFrame = new Byte[(int)(ImageSizeX * ImageSizeY)];
						NotWipedArtefactFrame = new Byte[(int)(ImageSizeX * ImageSizeY)];

						if (MarkerFrame == null || NotWipedArtefactFrame == null)
						{
							GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
								"GBMSAPI_NET_ROLL_GetCompositeImageInfo: Cannot alloc memory";
							RolledArtefactSize = 0;
							MarkerFrame = null;
							NotWipedArtefactFrame = null;
							CompositeFingerprintSize = 0;
							CompositeFingerprintContrast = 0;
							FlatFingerprintSize = 0;
							return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
						}

						Marshal.Copy(MarkerFramePtr, MarkerFrame, 0, (int)(ImageSizeX * ImageSizeY));
						Marshal.Copy(NotWipedArtefactFramePtr, NotWipedArtefactFrame, 0, (int)(ImageSizeX * ImageSizeY));
					}
					else
					{
						RolledArtefactSize = 0;
						MarkerFrame = null;
						NotWipedArtefactFrame = null;
						CompositeFingerprintSize = 0;
						CompositeFingerprintContrast = 0;
						FlatFingerprintSize = 0;
						ImageSizeX = 0;
						ImageSizeY = 0;
					}

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_ROLL_GetCompositeImageInfo: " + ex.Message;
					RolledArtefactSize = 0;
					MarkerFrame = null;
					NotWipedArtefactFrame = null;
					CompositeFingerprintSize = 0;
					CompositeFingerprintContrast = 0;
					FlatFingerprintSize = 0;
					ImageSizeX = 0;
					ImageSizeY = 0;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
			// end Ver 2.10.0.0

            

        }

        public class GBMSAPI_NET_AuxiliaryRoutines
        {
            // Ver 2700
            /**************************************
             * GBMSAPI_GetUserDataSize
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetUserDataSize(...) function in GBMSAPI_Library.h file
            /// </summary>
            public static Int32 GBMSAPI_NET_GetUserDataSize(out Int32 UserDataSize)
            {
                try
                {
                    return GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetUserDataSize(out UserDataSize);
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetUserDataSize: " + ex.Message;
                    UserDataSize = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // Ver 2700
            /**************************************
             * GBMSAPI_ReadUserData
            **************************************/
            /// <summary>
            /// See GBMSAPI_ReadUserData(...) function in GBMSAPI_Library.h file
            /// </summary>
            public static Int32 GBMSAPI_NET_ReadUserData(
                UInt32 Address,
                UInt32 Size,
                Byte [] Data
            )
            {
                try
                {
                    IntPtr DataPtr = IntPtr.Zero;
                    if (Data != null && Data.Length >= (int)Size)
                    {
                        DataPtr = Marshal.AllocHGlobal(Data.Length);
                        if (DataPtr == IntPtr.Zero)
                        {
                            GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                                "GBMSAPI_NET_ReadUserData: cannot alloc memory";
                            return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                        }
                        int RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ReadUserData(Address, Size, DataPtr);
                        Marshal.Copy(DataPtr, Data, 0, (int)Size);

                        Marshal.FreeHGlobal(DataPtr);

                        return RetVal;
                    }
                    else
                    {
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_PARAMETER;
                    }
                    
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_ReadUserData: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // Ver 2700
            /**************************************
             * GBMSAPI_WriteUserData
            **************************************/
            /// <summary>
            /// See GBMSAPI_WriteUserData(...) function in GBMSAPI_Library.h file
            /// </summary>
            public static Int32 GBMSAPI_NET_WriteUserData(
                UInt32 Address,
                UInt32 Size,
                Byte[] Data
            )
            {
                try
                {
                    // check params
                    if (Data == null || Data.Length < Size)
                    {
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_PARAMETER;
                    }
                    IntPtr DataPtr = Marshal.AllocHGlobal((int)Size);
                    if (DataPtr == IntPtr.Zero)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_WriteUserData: cannot alloc memory";
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }
                    Marshal.Copy(Data, 0, DataPtr, (int)Size);

                    int RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_WriteUserData(Address, Size, DataPtr);
                    Marshal.FreeHGlobal(DataPtr);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_WriteUserData: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "GBMSAPI_SetCalibrationImage2" instead
			/**************************************
             * GBMSAPI_SetCalibrationImage
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetCalibrationImage(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="CalibrationImage"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetCalibrationImage(
                Boolean FlatScanArea,
                Byte[] CalibrationImage
                )
            {
                try
                {
                    IntPtr CalFrame = IntPtr.Zero;
                    if (CalibrationImage != null && CalibrationImage.Length > 0)
                    {
                        CalFrame = Marshal.AllocHGlobal(CalibrationImage.Length);
                        if (CalFrame == IntPtr.Zero)
                        {
                            GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                                "GBMSAPI_NET_SetCalibrationImage: cannot alloc memory";
                            return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                        }
                        Marshal.Copy(CalibrationImage, 0, CalFrame, CalibrationImage.Length);
                    }
                    Int32 FSA = (FlatScanArea == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetCalibrationImage(FSA, CalFrame);
                    if (CalFrame != IntPtr.Zero)
                    {
                        Marshal.FreeHGlobal(CalFrame);
                    }
                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetCalibrationImage: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			/**************************************
			 * GBMSAPI_SetCalibrationImage2
			**************************************/
			/// <summary>
			/// See GBMSAPI_SetCalibrationImage2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="ScanArea"></param>
			/// <param name="CalibrationImage"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_SetCalibrationImage2(
				UInt32 ScanArea,
				Byte[] CalibrationImage
				)
			{
				try
				{
					IntPtr CalFrame = IntPtr.Zero;
					if (CalibrationImage != null && CalibrationImage.Length > 0)
					{
						CalFrame = Marshal.AllocHGlobal(CalibrationImage.Length);
						if (CalFrame == IntPtr.Zero)
						{
							GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
								"GBMSAPI_NET_SetCalibrationImage2: cannot alloc memory";
							return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
						}
						Marshal.Copy(CalibrationImage, 0, CalFrame, CalibrationImage.Length);
					}
					Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetCalibrationImage2(ScanArea, CalFrame);
					if (CalFrame != IntPtr.Zero)
					{
						Marshal.FreeHGlobal(CalFrame);
					}
					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_SetCalibrationImage2: " + ex.Message;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
			// end Ver 2.10.0.0


			// Ver 2.10.0.0
			// CONSIDER THIS FUNCTION AS OBSOLETE: use "GBMSAPI_GetCalibrationImage2" instead
			/**************************************
             * GBMSAPI_GetCalibrationImage
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetCalibrationImage(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="FlatScanArea"></param>
            /// <param name="CalibrationImage"></param>
            /// <param name="CalibrationDiagnostic"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetCalibrationImage(
                Boolean FlatScanArea,
                out Byte[] CalibrationImage,
                out UInt32 CalibrationDiagnostic
                )
            {
                try
                {
                    UInt32 SX, SY;
                    Int32 RetVal;
                    RetVal = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetCalibrationImageSize(
                            FlatScanArea,
                            out SX,
                            out SY
                            );
                    if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_GetCalibrationImage: cannot get image size; GBMSAPI_NET_GetImageSize returned " + RetVal;
                        CalibrationImage = null;
                        CalibrationDiagnostic = 0;
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }
                    // alloc memory
                    IntPtr CalFrame = Marshal.AllocHGlobal((int)(SX * SY));
                    if (CalFrame == IntPtr.Zero)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_GetCalibrationImage: cannot alloc memory";
                        CalibrationImage = null;
                        CalibrationDiagnostic = 0;
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }

                    Int32 FSA = (FlatScanArea == true) ? 1 : 0;
                    RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetCalibrationImage(FSA, CalFrame, out CalibrationDiagnostic);
                    if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        CalibrationImage = null;
                        CalibrationDiagnostic = 0;
                    }
                    else
                    {
                        CalibrationImage = new Byte[SX * SY];
                        if (CalibrationImage == null)
                        {
                            Marshal.FreeHGlobal(CalFrame);
                            GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_GetCalibrationImage: cannot alloc memory";
                            CalibrationImage = null;
                            CalibrationDiagnostic = 0;
                            return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                        }
                        Marshal.Copy(CalFrame, CalibrationImage, 0, CalibrationImage.Length);
                    }

                    Marshal.FreeHGlobal(CalFrame);
                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetCalibrationImage: " + ex.Message;
                    CalibrationImage = null;
                    CalibrationDiagnostic = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

			/**************************************
			 * GBMSAPI_GetCalibrationImage2
			**************************************/
			/// <summary>
			/// See GBMSAPI_GetCalibrationImage2(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <param name="ScanArea"></param>
			/// <param name="CalibrationImage"></param>
			/// <param name="CalibrationDiagnostic"></param>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_GetCalibrationImage2(
				UInt32 ScanArea,
				out Byte[] CalibrationImage,
				out UInt32 CalibrationDiagnostic
				)
			{
				try
				{
					UInt32 SX, SY;
					Int32 RetVal;
					RetVal = GBMSAPI_NET_DeviceCharacteristicsRoutines.GBMSAPI_NET_GetCalibrationImageSize2(
							ScanArea,
							out SX,
							out SY
							);
					if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
					{
						GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
							"GBMSAPI_NET_GetCalibrationImage2: cannot get image size; GBMSAPI_NET_GetCalibrationImageSize2 returned " + RetVal;
						CalibrationImage = null;
						CalibrationDiagnostic = 0;
						return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
					}
					// alloc memory
					IntPtr CalFrame = Marshal.AllocHGlobal((int)(SX * SY));
					if (CalFrame == IntPtr.Zero)
					{
						GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
							"GBMSAPI_NET_GetCalibrationImage: cannot alloc memory";
						CalibrationImage = null;
						CalibrationDiagnostic = 0;
						return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
					}

					RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetCalibrationImage2(ScanArea, CalFrame, out CalibrationDiagnostic);
					if (RetVal != GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
					{
						CalibrationImage = null;
						CalibrationDiagnostic = 0;
					}
					else
					{
						CalibrationImage = new Byte[SX * SY];
						if (CalibrationImage == null)
						{
							Marshal.FreeHGlobal(CalFrame);
							GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
							"GBMSAPI_NET_GetCalibrationImage2: cannot alloc memory";
							CalibrationImage = null;
							CalibrationDiagnostic = 0;
							return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
						}
						Marshal.Copy(CalFrame, CalibrationImage, 0, CalibrationImage.Length);
					}

					Marshal.FreeHGlobal(CalFrame);
					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_NET_GetCalibrationImage2: " + ex.Message;
					CalibrationImage = null;
					CalibrationDiagnostic = 0;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}
			// end Ver 2.10.0.0

            /**************************************
             * GBMSAPI_GetUnderlyingLibraryDllVersion
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetUnderlyingLibraryDllVersion(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="VersionField1"></param>
            /// <param name="VersionField2"></param>
            /// <param name="VersionField3"></param>
            /// <param name="VersionField4"></param>
            /// <param name="LowLevelDllName"></param>
            /// <returns></returns>
            public static void GBMSAPI_NET_GetMultiScanAPIVersion(
                out Byte VersionField1,
                out Byte VersionField2,
                out Byte VersionField3,
                out Byte VersionField4
                )
            {
                Assembly asm =  Assembly.GetExecutingAssembly();
                Version v = (asm.GetName().Version);
                VersionField1 = (Byte)v.Major;
                VersionField2 = (Byte)v.Minor;
                VersionField3 = (Byte)v.Build;
                VersionField4 = (Byte)v.Revision;
            }

            /**************************************
             * GBMSAPI_GetUnderlyingLibraryDllVersion
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetUnderlyingLibraryDllVersion(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="VersionField1"></param>
            /// <param name="VersionField2"></param>
            /// <param name="VersionField3"></param>
            /// <param name="VersionField4"></param>
            /// <param name="LowLevelDllName"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetUnderlyingLibraryDllVersion(
                out Byte VersionField1,
                out Byte VersionField2,
                out Byte VersionField3,
                out Byte VersionField4,
                out String LowLevelDllName
                )
            {
                try
                {
                    Int32 RetVal, DllNameStringLen = 200;
                    
                    StringBuilder SB_LL_DLLName = new StringBuilder(DllNameStringLen);
                    if (SB_LL_DLLName == null)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "GBMSAPI_NET_GetUnderlyingLibraryDllVersion: cannot alloc memory";
                        VersionField1 = 0;
                        VersionField2 = 0;
                        VersionField3 = 0;
                        VersionField4 = 0;
                        LowLevelDllName = null;

                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }

                    RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetUnderlyingLibraryDllVersion(
                        out VersionField1,
                        out VersionField2,
                        out VersionField3,
                        out VersionField4,
                        SB_LL_DLLName
                    );

                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        LowLevelDllName = SB_LL_DLLName.ToString();
                    }
                    else
                    {
                        VersionField1 = 0;
                        VersionField2 = 0;
                        VersionField3 = 0;
                        VersionField4 = 0;
                        LowLevelDllName = null;
                    }

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetUnderlyingLibraryDllVersion: " + ex.Message;
                    VersionField1 = 0;
                    VersionField2 = 0;
                    VersionField3 = 0;
                    VersionField4 = 0;
                    LowLevelDllName = null;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetUSBErrorCode
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetUSBErrorCode(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="USBErrorCode"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetUSBErrorCode(
               out UInt32 USBErrorCode
               )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetUSBErrorCode(
                        out USBErrorCode
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetUSBErrorCode: " + ex.Message;
                    USBErrorCode = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_GetScannerStatistics
            **************************************/
            /// <summary>
            /// See GBMSAPI_GetScannerStatistics(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Counter"></param>
            /// <param name="ProductionDate"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_GetScannerStatistics(
                out UInt32 Counter,
                out UInt32 ProductionDate
               )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_GetScannerStatistics(
                        out Counter,
                        out ProductionDate
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_GetScannerStatistics: " + ex.Message;
                    Counter = 0;
                    ProductionDate = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
        }

        public class GBMSAPI_NET_ExternalDevicesControlRoutines
        {
            /// <summary>
            /// See GBMSAPI_Sound(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Tone"></param>
            /// <param name="Duration"></param>
            /// <param name="Iterations"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_Sound(
                Byte Tone,
                Byte Duration,
                Byte Iterations
               )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_Sound(
                        Tone,
                        Duration,
                        Iterations
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_Sound: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_SetFingerIndicatorVUIState
            **************************************/
            /// <summary>
            /// See GBMSAPI_SetFingerIndicatorVUIState(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="IndicatorMask"></param>
            /// <param name="Color"></param>
            /// <param name="Blink"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_SetFingerIndicatorVUIState(
                UInt32  IndicatorMask,
                Int32   Color,
                Boolean Blink
               )
            {
                try
                {
                    Int32 ShouldBlink = (Blink == true) ? 1 : 0;
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_SetFingerIndicatorVUIState(
                        IndicatorMask,
                        Color,
                        ShouldBlink
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_SetFingerIndicatorVUIState: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_CheckPedalState
            **************************************/
            /// <summary>
            /// See GBMSAPI_CheckPedalState(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="PedalState"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_CheckPedalState(
                out Boolean PedalState
               )
            {
                try
                {
                    Byte ReturnFromLL;
                    
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_CheckPedalState(
                        out ReturnFromLL
                        );

                    PedalState = (ReturnFromLL == 0) ? false : true;

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_CheckPedalState: " + ex.Message;
                    PedalState = false;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetLanguage
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetLanguage(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ScreenID"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetLanguage(String ISO639Lang)
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetLanguage(ISO639Lang);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetLanguage: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_GetLcdFeatures
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_GetLcdFeatures(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ScreenID"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_GetLcdFeatures(out UInt32 LcdFeatures)
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_GetLcdFeatures(out LcdFeatures);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    LcdFeatures = 0;
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_GetLcdFeatures: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }



            /**************************************
             * GBMSAPI_VUI_LCD_SetLogoScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ScreenID"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetLogoScreen()
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetLogoScreen();

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetLogoScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetScannedObject
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetScannedObject(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="ObjToScan"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetScannedObject(
                UInt32 ObjToScan
               )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetScannedObject(
                        ObjToScan
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetScannedObject: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="MessageToSet"></param>
            /// <param name="MessageArea"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen(
                Byte MessageToSet,
                Byte MessageArea
               )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetPreloadedMessageOnStopScreen(
                        MessageToSet,
                        MessageArea
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetPreloadedMessageOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="MessageToSet"></param>
            /// <param name="MessageToSetType"></param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetCustomerMessageOnStopScreen(
                String MessageToSet,
                Byte MessageToSetType
               )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetCustomerMessageOnStopScreen(
                        MessageToSet,
                        MessageToSetType
                        );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetCustomerMessageOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_EnableOKButtonOnStopScreen()
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_EnableOKButtonOnStopScreen();

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_EnableOKButtonOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetQualityLimitOnStopScreen(
                Byte QualityLimit,
                Byte Direction
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetQualityLimitOnStopScreen(
                        QualityLimit,
                        Direction
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetQualityLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(
                Byte Quality
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetQualityIntermediateLimitOnStopScreen(Quality);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetQualityIntermediateLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetQualityOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetQualityOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetQualityOnStopScreen(
                Byte Quality
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetQualityOnStopScreen(Quality);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetQualityOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetArtefactLimitOnStopScreen(
                int ArtefactLimit,
                Byte Direction
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetArtefactLimitOnStopScreen(
                        ArtefactLimit,
                        Direction
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetArtefactLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(
                int ArtefactLimit
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen(
                        ArtefactLimit
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetArtefactIntermediateLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetArtefactOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetArtefactOnStopScreen(
                int Artefact
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetArtefactOnStopScreen(Artefact);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetArtefactOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetSegmentationOnStopScreen(
                Byte Seg1,
                Byte Seg2,
                Byte Seg3,
                Byte Seg4
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetSegmentationOnStopScreen(
                        Seg1,
                        Seg2,
                        Seg3,
                        Seg4
                    );

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetSegmentationOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_EnableStartButtonOnLogoScreen()
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_EnableStartButtonOnLogoScreen();

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_EnableStartButtonOnLogoScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_GetPressedButton
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_GetPressedButton(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_GetPressedButton(
                out Byte PressedButton
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_GetPressedButton(out PressedButton);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_EnableStartButtonOnLogoScreen: " + ex.Message;
                    PressedButton = 0;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastIntermediateLimit
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetContrastIntermediateLimit(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetContrastIntermediateLimit(
                Byte ContrastValue
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetContrastIntermediateLimit(ContrastValue);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetContrastIntermediateLimit: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetContrastValueOnAcquisitionScreen(
                Byte ContrastValue
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetContrastValueOnAcquisitionScreen(ContrastValue);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetContrastValueOnAcquisitionScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetCompletenessIntermediateLimit(
                Byte ContrastValue
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimit(ContrastValue);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetCompletenessIntermediateLimit: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetCompletenessValueOnAcquisitionScreen(
                Byte CompletenessValue
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetCompletenessValueOnAcquisitionScreen(CompletenessValue);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetCompletenessValueOnAcquisitionScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetContrastLimitOnStopScreen(
                Byte ContrastLimit
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetContrastLimitOnStopScreen(ContrastLimit);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetContrastLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(
                Byte ContrastLimit
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetContrastIntermediateLimitOnStopScreen(ContrastLimit);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetContrastIntermediateLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetContrastValueOnStopScreen(
                Byte ContrastValue
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetContrastValueOnStopScreen(ContrastValue);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetContrastValueOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetCompletenessLimitOnStopScreen(
                Byte CompletenessLimit
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetCompletenessLimitOnStopScreen(CompletenessLimit);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetCompletenessLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

            // VER 2.8.0.0
            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(
                Byte CompletenessLimit
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(CompletenessLimit);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
			}

			// VER 2.9.0.1
			/**************************************
			 * GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen
			**************************************/
			/// <summary>
			/// See GBMSAPI_VUI_LCD_SetCompletenessIntermediateLimitOnStopScreen(...) function in GBMSAPI_Library.h file
			/// </summary>
			/// <returns></returns>
			public static Int32 GBMSAPI_NET_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(
				Byte DiagMessageToVisualize
				)
			{
				try
				{
					Int32 RetVal = 
						GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen(DiagMessageToVisualize);

					return RetVal;
				}
				catch (Exception ex)
				{
					GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
						"GBMSAPI_VUI_LCD_SetWetDryFingerDiagnosticIconOnStopScreen: " + ex.Message;
					return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
				}
			}

            /**************************************
             * GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen
            **************************************/
            /// <summary>
            /// See GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_VUI_LCD_SetCompletenessValueOnStopScreen(
                Byte CompletenessValue
                )
            {
                try
                {
                    Int32 RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_VUI_LCD_SetCompletenessValueOnStopScreen(CompletenessValue);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_VUI_LCD_SetCompletenessValueOnStopScreen: " + ex.Message;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }

        }

        public class GBMSAPI_NET_ImageProcessingUtilities
        {

            /**************************************
             * GBMSAPI_ConvertImageFrom1000to500DPI
            **************************************/
            /// <summary>
            /// See GBMSAPI_ConvertImageFrom1000to500DPI(...) function in GBMSAPI_Library.h file
            /// </summary>
            /// <param name="Output500DPIFrame">
            /// It is allocated by the functions. Its size is the multiple of 4 closest to
            /// <paramref name="InputSizeX"/> * <paramref name="InputSizeY"/>
            /// </param>
            /// <returns></returns>
            public static Int32 GBMSAPI_NET_ConvertImageFrom1000to500DPI(
                Byte[] Input1000DPIFrame,
                out Byte[] Output500DPIFrame,
                UInt32 InputSizeX,
                UInt32 InputSizeY
                )
            {
                try
                {
                    if (Input1000DPIFrame.Length < (InputSizeX * InputSizeY))
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                            "Length specified for input frame is more than <Input1000DPIFrame> parameter length";
                        Output500DPIFrame = null;
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }
                    IntPtr InputFrame = Marshal.AllocHGlobal((Int32)(InputSizeX * InputSizeY));
                    if (InputFrame == IntPtr.Zero)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                               "Cannot alloc memory";
                        Output500DPIFrame = null;
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }
                    Marshal.Copy(Input1000DPIFrame, 0, InputFrame, (int)(InputSizeX * InputSizeY));
                    if ((InputSizeX & 0x00000001) != 0) InputSizeX++;
                    if ((InputSizeY & 0x00000001) != 0) InputSizeY++;

                    IntPtr OutFrame = Marshal.AllocHGlobal((int)(InputSizeX * InputSizeY));
                    if (OutFrame == IntPtr.Zero)
                    {
                        GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                               "Cannot alloc memory";
                        Output500DPIFrame = null;
                        Marshal.FreeHGlobal(InputFrame);
                        return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                    }

                    int RetVal = GBMSAPI_NET_DLL_WRAPPER.GBMSAPI_ConvertImageFrom1000to500DPI(
                        InputFrame,
                        OutFrame,
                        InputSizeX,
                        InputSizeY
                    );

                    if (RetVal == GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_NO_ERROR)
                    {
                        Output500DPIFrame = new Byte[(InputSizeY * InputSizeX) / 4];
                        if (Output500DPIFrame == null)
                        {
                            GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                               "Cannot alloc memory";
                            Output500DPIFrame = null;
                            Marshal.FreeHGlobal(InputFrame);
                            Marshal.FreeHGlobal(OutFrame);
                            return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                        }
                        Marshal.Copy(OutFrame, Output500DPIFrame, 0, Output500DPIFrame.Length);
                    }
                    else
                    {
                        Output500DPIFrame = null;
                    }

                    Marshal.FreeHGlobal(InputFrame);
                    Marshal.FreeHGlobal(OutFrame);

                    return RetVal;
                }
                catch (Exception ex)
                {
                    GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION_STRING =
                        "GBMSAPI_NET_ConvertImageFrom1000to500DPI: " + ex.Message;
                    Output500DPIFrame = null;
                    return GBMSAPI_NET_ErrorCodes.GBMSAPI_NET_ERROR_CODE_EXCEPTION;
                }
            }
        }
    }
}