#include <stdafx.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <an2k_dll.h>
#include "GB_AN2K.h"

#include "imageprocessing.h"
#include "GB_Segmentation.h"


// to avoid warning with strcpy and so on
#pragma warning( disable : 4996 4793)

using namespace System;
using namespace System::IO;
using namespace System::Windows::Forms;

/****************************************************************************/
/************* GLOBAL VARIABLES *********************************************/
/****************************************************************************/

// Finger amputation list for left slap
FING_AMP_LIST *LeftSlapAmpList;

// Finger NIST quality list
// It's declared as a global variable 'cause it's built, filled and
// saved in different phases of the image processing process
FOUR_FING_QUALITY *NIST_QualityList = nullptr;

/****************************************************************************/
/************* ROUTINES *****************************************************/
/****************************************************************************/


int		CreateRec2FromRollAmpList(ANSI_NIST *ansi_nist)
{
	RECORD *record;
	if (LeftSlapAmpList == nullptr)
	{
		Console::WriteLine("Cannot create Record 2: Amputation list missing...");
		return GB_EXAMPLE_GENER_ERROR;
	}

	int RetVal = CreateETFStype_2(
		&record, // output record
		"GreenBit", // vendor
		GB_MOD, // model
		GB_PSN, // serial number
		LeftSlapAmpList // amputation list
		);
	if (RetVal != AN2K_DLL_NO_ERROR)
	{
		// show error and exit
		Console::WriteLine("Cannot create Record 2: CreateETFStype_2 returned " + RetVal);
		Console::Read();
		return GB_EXAMPLE_GENER_ERROR;
	}
	Console::WriteLine("Record 2 successfully created..." );

	// insert record
	RetVal = insert_ANSI_NIST_record_ETFS(1, record, ansi_nist);
	if (RetVal != AN2K_DLL_NO_ERROR)
	{
		Console::WriteLine ("Cannot insert record 2 in list...");
		return GB_EXAMPLE_GENER_ERROR;
	}

	return GB_EXAMPLE_NO_ERROR;
}

int		InitAmputationList(void)
{
	// alloc memory
	LeftSlapAmpList = (FING_AMP_LIST *)malloc(sizeof(FING_AMP_LIST));
	if (LeftSlapAmpList == nullptr)
	{
		return GB_EXAMPLE_MEM_ERROR;
	}
	LeftSlapAmpList->fingers = (FING_AMP **)malloc(sizeof(FING_AMP *));
	if (LeftSlapAmpList->fingers == nullptr)
	{
		return GB_EXAMPLE_MEM_ERROR;
	}
	/* alloc and fillthe first FING_AMP field:
	 this operation should be done for each missing 
	 finger (for example in a for cycle) */
	// alloc
	LeftSlapAmpList->fingers[0] = (FING_AMP *)malloc(sizeof(FING_AMP));
	if (LeftSlapAmpList->fingers[0] == nullptr)
	{
		return GB_EXAMPLE_MEM_ERROR;
	}
	// fill
	LeftSlapAmpList->num_fingers = 1;
	strcpy(LeftSlapAmpList->fingers[0]->amp_code,UNABLE_TO_PRINT);
	strcpy(LeftSlapAmpList->fingers[0]->fing_id,ETFS_14_FGP_LEFT_RING);

	return GB_EXAMPLE_NO_ERROR;
}

int		BuildQualityListFromObjID(int ObjID)
{
	// check input
	if (ObjID != LEFT_SLAP	&& 	ObjID != RIGHT_SLAP	&& 	ObjID != TWO_THUMBS)
	{
		return GB_EXAMPLE_PARAM_ERROR;
	}
	/* alloc memory */
	// alloc list pointer
	if (NIST_QualityList != nullptr)
	{
		free_FOUR_FING_QUALITY(NIST_QualityList);
	}
	NIST_QualityList = (FOUR_FING_QUALITY *)malloc(sizeof(FOUR_FING_QUALITY));
	if(NIST_QualityList == nullptr)
	{
		return GB_EXAMPLE_MEM_ERROR;
	}
	// set number of segments
	if (ObjID == RIGHT_SLAP)
	{
		NIST_QualityList->num_fingers = 4;
	}
	else if (ObjID == LEFT_SLAP)
	{
		NIST_QualityList->num_fingers = 3;
	}
	else
	{
		// 2 thumbs
		NIST_QualityList->num_fingers = 2;
	}
	// alloc segments
	NIST_QualityList->fingers = (FING_QUALITY **)malloc((NIST_QualityList->num_fingers) * (sizeof(FING_QUALITY *)));
	if(NIST_QualityList->fingers == nullptr)
	{
		// free allocated memory
		NIST_QualityList->num_fingers = 0; // in this way I can call free_FOUR_FING_QUALITY
		free_FOUR_FING_QUALITY(NIST_QualityList);
		NIST_QualityList = nullptr;
		return GB_EXAMPLE_MEM_ERROR;
	}
	for (int i=0; i < NIST_QualityList->num_fingers; i++)
	{
		NIST_QualityList->fingers[i] = (FING_QUALITY *)malloc((sizeof(FING_QUALITY)));
		if(NIST_QualityList->fingers == nullptr)
		{
			NIST_QualityList->num_fingers = i; // in this way I can call free_FOUR_FING_QUALITY
			free_FOUR_FING_QUALITY(NIST_QualityList);
			NIST_QualityList = nullptr;
			return GB_EXAMPLE_MEM_ERROR;
		}
	}

	return GB_EXAMPLE_NO_ERROR;
}

int		BuildBoundaryBoxListFromObjID(FOUR_FING_BOUND **pBoundaries,int ObjID)
{
	if (ObjID != LEFT_SLAP	&& 	ObjID != RIGHT_SLAP	&& 	ObjID != TWO_THUMBS)
	{
		return GB_EXAMPLE_PARAM_ERROR;
	}

	// alloc memory for FING_SEGMENT list
	FOUR_FING_BOUND *Boundaries = (FOUR_FING_BOUND *)malloc(sizeof(FOUR_FING_BOUND));
	if (Boundaries == nullptr)
	{
		*pBoundaries = nullptr;
		return GB_EXAMPLE_MEM_ERROR;
	}
	// calculate segment number
	if (ObjID == RIGHT_SLAP)
	{
		Boundaries->num_fingers = 4;
		
	}
	else if (ObjID == LEFT_SLAP)
	{
		Boundaries->num_fingers = 3;
	}
	else
	{
		Boundaries->num_fingers = 2;
	}
	// alloc memory for segments
	Boundaries->fingers = (FING_SEGMENT **)malloc((Boundaries->num_fingers) * sizeof(FING_SEGMENT *));
	if (Boundaries->fingers == nullptr)
	{
		*pBoundaries = nullptr;
		free(Boundaries);
		return GB_EXAMPLE_MEM_ERROR;
	}
	for (int i = 0; i < Boundaries->num_fingers; i ++)
	{
		Boundaries->fingers[i] = (FING_SEGMENT *)malloc(sizeof(FING_SEGMENT));
		if (Boundaries->fingers[i] == nullptr)
		{
			// free memory
			for (int j=0; j<i; j++)
			{
				free(Boundaries->fingers[j]);
			}
			free(Boundaries->fingers);
			free(Boundaries);
			*pBoundaries = nullptr;
			return GB_EXAMPLE_MEM_ERROR;
		}
	}
	// now fill
	// index
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying bounding box coordinates for Index...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_INDEX);
		if (IndexInList >= 0)
		{
			(ObjID == LEFT_SLAP) ? 
				(strcpy(Boundaries->fingers[0]->fing_id,ETFS_14_FGP_LEFT_INDEX)) :
				(strcpy(Boundaries->fingers[0]->fing_id,ETFS_14_FGP_RIGHT_INDEX));
			// bottom
			itoa(CompleteSlapList[IndexInList].BoundingBoxB,Boundaries->fingers[0]->bottom_bound,10);
			// top
			itoa(CompleteSlapList[IndexInList].BoundingBoxT,Boundaries->fingers[0]->top_bound,10);
			// left
			itoa(CompleteSlapList[IndexInList].BoundingBoxL,Boundaries->fingers[0]->left_bound,10);
			// right
			itoa(CompleteSlapList[IndexInList].BoundingBoxR,Boundaries->fingers[0]->right_bound,10);
		}
		else
		{
			free_FOUR_FING_BOUND(Boundaries);
			*pBoundaries = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}

	// middle
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying bounding box coordinates for Middle...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_MIDDLE);
		if (IndexInList >= 0)
		{
			(ObjID == LEFT_SLAP) ? 
				(strcpy(Boundaries->fingers[1]->fing_id,ETFS_14_FGP_LEFT_MIDDLE)) :
				(strcpy(Boundaries->fingers[1]->fing_id,ETFS_14_FGP_RIGHT_MIDDLE));
			// bottom
			itoa(CompleteSlapList[IndexInList].BoundingBoxB,Boundaries->fingers[1]->bottom_bound,10);
			// top
			itoa(CompleteSlapList[IndexInList].BoundingBoxT,Boundaries->fingers[1]->top_bound,10);
			// left
			itoa(CompleteSlapList[IndexInList].BoundingBoxL,Boundaries->fingers[1]->left_bound,10);
			// right
			itoa(CompleteSlapList[IndexInList].BoundingBoxR,Boundaries->fingers[1]->right_bound,10);
		}
		else
		{
			free_FOUR_FING_BOUND(Boundaries);
			*pBoundaries = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}

	// ring
	if (ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying bounding box coordinates for Ring...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_RING);
		if (IndexInList >= 0)
		{
			(strcpy(Boundaries->fingers[2]->fing_id,ETFS_14_FGP_RIGHT_RING));
			// bottom
			itoa(CompleteSlapList[IndexInList].BoundingBoxB,Boundaries->fingers[2]->bottom_bound,10);
			// top
			itoa(CompleteSlapList[IndexInList].BoundingBoxT,Boundaries->fingers[2]->top_bound,10);
			// left
			itoa(CompleteSlapList[IndexInList].BoundingBoxL,Boundaries->fingers[2]->left_bound,10);
			// right
			itoa(CompleteSlapList[IndexInList].BoundingBoxR,Boundaries->fingers[2]->right_bound,10);
		}
		else
		{
			free_FOUR_FING_BOUND(Boundaries);
			*pBoundaries = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}

	// little
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying bounding box coordinates for Little...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_LITTLE);
		if (IndexInList >= 0)
		{
			int BoundIndex = ((ObjID == LEFT_SLAP) ? 2 : 3);
			(ObjID == LEFT_SLAP) ? 
				(strcpy(Boundaries->fingers[BoundIndex]->fing_id,ETFS_14_FGP_LEFT_LITTLE)) :
				(strcpy(Boundaries->fingers[BoundIndex]->fing_id,ETFS_14_FGP_RIGHT_LITTLE));
			
			// bottom
			itoa(CompleteSlapList[IndexInList].BoundingBoxB,Boundaries->fingers[BoundIndex]->bottom_bound,10);
			// top
			itoa(CompleteSlapList[IndexInList].BoundingBoxT,Boundaries->fingers[BoundIndex]->top_bound,10);
			// left
			itoa(CompleteSlapList[IndexInList].BoundingBoxL,Boundaries->fingers[BoundIndex]->left_bound,10);
			// right
			itoa(CompleteSlapList[IndexInList].BoundingBoxR,Boundaries->fingers[BoundIndex]->right_bound,10);
		}
		else
		{
			free_FOUR_FING_BOUND(Boundaries);
			*pBoundaries = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}

	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP) // we are finished
	{
		*pBoundaries = Boundaries;
		return GB_EXAMPLE_NO_ERROR;
	}

	// left thumb
	Console::WriteLine("Copying bounding box coordinates for left thumb...");
	int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_THUMB,GBFINIMG_HAND_LEFT);
	if (IndexInList >= 0)
	{
		(strcpy(Boundaries->fingers[0]->fing_id,ETFS_14_FGP_LEFT_THUMB));
		
		// bottom
		itoa(CompleteSlapList[IndexInList].BoundingBoxB,Boundaries->fingers[0]->bottom_bound,10);
		// top
		itoa(CompleteSlapList[IndexInList].BoundingBoxT,Boundaries->fingers[0]->top_bound,10);
		// left
		itoa(CompleteSlapList[IndexInList].BoundingBoxL,Boundaries->fingers[0]->left_bound,10);
		// right
		itoa(CompleteSlapList[IndexInList].BoundingBoxR,Boundaries->fingers[0]->right_bound,10);
	}
	else
	{
		free_FOUR_FING_BOUND(Boundaries);
		*pBoundaries = nullptr;
		return GB_EXAMPLE_GENER_ERROR;
	}

	// right thumb
	Console::WriteLine("Copying bounding box coordinates for right thumb...");
	IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_THUMB,GBFINIMG_HAND_RIGHT);
	if (IndexInList >= 0)
	{
		(strcpy(Boundaries->fingers[1]->fing_id,ETFS_14_FGP_RIGHT_THUMB));
		
		// bottom
		itoa(CompleteSlapList[IndexInList].BoundingBoxB,Boundaries->fingers[1]->bottom_bound,10);
		// top
		itoa(CompleteSlapList[IndexInList].BoundingBoxT,Boundaries->fingers[1]->top_bound,10);
		// left
		itoa(CompleteSlapList[IndexInList].BoundingBoxL,Boundaries->fingers[1]->left_bound,10);
		// right
		itoa(CompleteSlapList[IndexInList].BoundingBoxR,Boundaries->fingers[1]->right_bound,10);
	}
	else
	{
		free_FOUR_FING_BOUND(Boundaries);
		*pBoundaries = nullptr;
		return GB_EXAMPLE_GENER_ERROR;
	}


	*pBoundaries = Boundaries;
	return GB_EXAMPLE_NO_ERROR;
}
int		BuildSegmentationQualityListFromObjID(ALT_QUAL_LIST **pQualList,int ObjID)
{
	if (ObjID != LEFT_SLAP	&& 	ObjID != RIGHT_SLAP	&& 	ObjID != TWO_THUMBS)
	{
		return GB_EXAMPLE_PARAM_ERROR;
	}

	// alloc memory for ALT_QUAL_LIST list
	ALT_QUAL_LIST *QualList = (ALT_QUAL_LIST *)malloc(sizeof(ALT_QUAL_LIST));
	if (QualList == nullptr)
	{
		*pQualList = nullptr;
		return GB_EXAMPLE_MEM_ERROR;
	}
	// calculate segment number
	if (ObjID == RIGHT_SLAP)
	{
		QualList->num_fingers = 4;
		
	}
	else if (ObjID == LEFT_SLAP)
	{
		QualList->num_fingers = 3;
	}
	else
	{
		QualList->num_fingers = 2;
	}
	// alloc memory for segments
	QualList->fingers = (ALT_QUAL **)malloc((QualList->num_fingers) * sizeof(ALT_QUAL *));
	if (QualList->fingers == nullptr)
	{
		*pQualList = nullptr;
		free(QualList);
		return GB_EXAMPLE_MEM_ERROR;
	}
	for (int i = 0; i < QualList->num_fingers; i ++)
	{
		QualList->fingers[i] = (ALT_QUAL *)malloc(sizeof(ALT_QUAL));
		if (QualList->fingers[i] == nullptr)
		{
			// free memory
			QualList->num_fingers = i;
			free_ANSI_NIST_alt_qual_list(QualList);
			*pQualList = nullptr;
			return GB_EXAMPLE_MEM_ERROR;
		}
		// fill with IBIA parameters
		itoa(GB_QUAL_ALG_VENDOR_ID,QualList->fingers[i]->Vendor_ID,10);
		itoa(GB_QUAL_ALG_PRODUCT_ID,QualList->fingers[i]->Algorithm_ID,10);
	}
	// now fill
	// index
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying GB quality for Index...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_INDEX);
		if (IndexInList >= 0)
		{
			(ObjID == LEFT_SLAP) ? 
				(strcpy(QualList->fingers[0]->fing_id,ETFS_14_FGP_LEFT_INDEX)) :
				(strcpy(QualList->fingers[0]->fing_id,ETFS_14_FGP_RIGHT_INDEX));
			// quality
			itoa(CompleteSlapList[IndexInList].Quality,QualList->fingers[0]->quality,10);
		}
		else
		{
			free_ANSI_NIST_alt_qual_list(QualList);
			*pQualList = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}

	// middle
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying GB quality for Middle...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_MIDDLE);
		if (IndexInList >= 0)
		{
			(ObjID == LEFT_SLAP) ? 
				(strcpy(QualList->fingers[1]->fing_id,ETFS_14_FGP_LEFT_MIDDLE)) :
				(strcpy(QualList->fingers[1]->fing_id,ETFS_14_FGP_RIGHT_MIDDLE));
			// quality
			itoa(CompleteSlapList[IndexInList].Quality,QualList->fingers[1]->quality,10);
		}
		else
		{
			free_ANSI_NIST_alt_qual_list(QualList);
			*pQualList = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}

	// ring
	if (ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying GB quality for Ring...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_RING);
		if (IndexInList >= 0)
		{
			(strcpy(QualList->fingers[2]->fing_id,ETFS_14_FGP_RIGHT_RING));
			// quality
			itoa(CompleteSlapList[IndexInList].Quality,QualList->fingers[2]->quality,10);
		}
		else
		{
			free_ANSI_NIST_alt_qual_list(QualList);
			*pQualList = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}

	// little
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		Console::WriteLine("Copying GB quality for Little...");
		int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_LITTLE);
		if (IndexInList >= 0)
		{
			int BoundIndex = ((ObjID == LEFT_SLAP) ? 2 : 3);
			(ObjID == LEFT_SLAP) ? 
				(strcpy(QualList->fingers[BoundIndex]->fing_id,ETFS_14_FGP_LEFT_LITTLE)) :
				(strcpy(QualList->fingers[BoundIndex]->fing_id,ETFS_14_FGP_RIGHT_LITTLE));
			
			// quality
			itoa(CompleteSlapList[IndexInList].Quality,QualList->fingers[BoundIndex]->quality,10);
		}
		else
		{
			free_ANSI_NIST_alt_qual_list(QualList);
			*pQualList = nullptr;
			return GB_EXAMPLE_GENER_ERROR;
		}
	}
	
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP) // we are finished
	{
		*pQualList = QualList;
		return GB_EXAMPLE_NO_ERROR;
	}

	// left thumb
	Console::WriteLine("Copying GB quality for left thumb...");
	int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_THUMB,GBFINIMG_HAND_LEFT);
	if (IndexInList >= 0)
	{
		(strcpy(QualList->fingers[0]->fing_id,ETFS_14_FGP_LEFT_THUMB));

		// quality
		itoa(CompleteSlapList[IndexInList].Quality,QualList->fingers[0]->quality,10);
	}
	else
	{
		free_ANSI_NIST_alt_qual_list(QualList);
		*pQualList = nullptr;
		return GB_EXAMPLE_GENER_ERROR;
	}

	// right thumb
	Console::WriteLine("Copying GB quality for right thumb...");
	IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FINGER_THUMB,GBFINIMG_HAND_RIGHT);
	if (IndexInList >= 0)
	{
		(strcpy(QualList->fingers[1]->fing_id,ETFS_14_FGP_RIGHT_THUMB));

		// quality
		itoa(CompleteSlapList[IndexInList].Quality,QualList->fingers[1]->quality,10);
	}
	else
	{
		free_ANSI_NIST_alt_qual_list(QualList);
		*pQualList = nullptr;
		return GB_EXAMPLE_GENER_ERROR;
	}


	*pQualList = QualList;
	return GB_EXAMPLE_NO_ERROR;
}



int		getFingerPosForRec4FromObjID(Byte ObjID)
{
	// check input
	if (ObjID < LEFT_THUMB || ObjID > RIGHT_LITTLE)
	{
		return -1;
	}

	// find finger
	switch (ObjID)
	{
	case LEFT_THUMB:
		return atoi(ETFS_14_FGP_LEFT_THUMB);
		break;
	case LEFT_INDEX:
		return atoi(ETFS_14_FGP_LEFT_INDEX);
		break;
	case LEFT_MIDDLE:
		return atoi(ETFS_14_FGP_LEFT_MIDDLE);
		break;
	case LEFT_RING:
		return atoi(ETFS_14_FGP_LEFT_RING);
		break;
	case LEFT_LITTLE:
		return atoi(ETFS_14_FGP_LEFT_LITTLE);
		break;
	case RIGHT_THUMB:
		return atoi(ETFS_14_FGP_RIGHT_THUMB);
		break;
	case RIGHT_INDEX:
		return atoi(ETFS_14_FGP_RIGHT_INDEX);
		break;
	case RIGHT_MIDDLE:
		return atoi(ETFS_14_FGP_RIGHT_MIDDLE);
		break;
	case RIGHT_RING:
		return atoi(ETFS_14_FGP_RIGHT_RING);
		break;
	case RIGHT_LITTLE:
		return atoi(ETFS_14_FGP_RIGHT_LITTLE);
		break;
	}

	// finger not found: never should arrive here
	return -1;
}


int		CreateRec4FromObjID(Byte ObjID, RECORD **pRecord,Byte *ImgCompressed,int ImgCompLen,int SizeX, int SizeY, char *ImgCompAlg)
{
	int RetVal;

	// get finger position
	int FingPos = getFingerPosForRec4FromObjID(ObjID);
	if (FingPos > 0)
	{
		Console::WriteLine ("");
		Console::WriteLine ("Creating record 4...");
		RetVal = image2ETFStype_4(
			pRecord, // output record
			ImgCompressed,ImgCompLen, // image pointer and size
			ETFS_14_LIVE_SCAN_ROLLED, // image impression
			FingPos, // finger position
			1, // max resolution
			SizeX,SizeY, // image dimensions
			ImgCompAlg // compression alg
			);
		if (RetVal == AN2K_DLL_NO_ERROR)
		{
			// no error: set RetVal
			Console::WriteLine ("Record 4 succesfully created...");
			RetVal = GB_EXAMPLE_NO_ERROR;
		}
		else
		{
			// error: show retval and then set it to generic error
			Console::WriteLine("image2ETFStype_4 returned " + RetVal + " : cannot fill record 14...");
			*pRecord = nullptr;
			RetVal = GB_EXAMPLE_GENER_ERROR;
		}
	}
	else
	{
		RetVal = GB_EXAMPLE_GENER_ERROR;
		*pRecord = nullptr;
		Console::WriteLine ("");
		Console::WriteLine ("Warning: Cannot get finger position: record 4 will not be filled...");
	}

	return RetVal;
}

int		CreateRec14FromObjID(int ObjID, RECORD **pRecord,Byte *ImgCompressed,int ImgCompLen,int SizeX, int SizeY, char *ImgCompAlg)
{
	int RetVal;

	Console::WriteLine("Creating Lists for Record 14...");

	// get finger position
	char finger_position[3];
	if (ObjID == LEFT_SLAP)
	{
		strcpy(finger_position,ETFS_14_FGP_LEFT_4_FING);
	}
	else if (ObjID == RIGHT_SLAP)
	{
		strcpy(finger_position,ETFS_14_FGP_RIGHT_4_FING);
	}
	else
	{
		strcpy(finger_position,ETFS_14_FGP_2_THUMBS);
	}

	// get amputation list
	FING_AMP_LIST *amp_list = nullptr;
	if (ObjID == LEFT_SLAP)
	{
		amp_list = LeftSlapAmpList;
	}

	// build bounding box list
	FOUR_FING_BOUND *boundaries;
	RetVal = BuildBoundaryBoxListFromObjID(&boundaries,ObjID);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		boundaries = nullptr;
	}

	// build Green Bit quality list
	ALT_QUAL_LIST * SegmentationQualityList;
	RetVal = BuildSegmentationQualityListFromObjID(&SegmentationQualityList,ObjID);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		SegmentationQualityList = nullptr;
	}

	Console::WriteLine("Lists for Record 14 created...");

	Console::WriteLine("");
	Console::WriteLine("Creating Record 14...");
	
	/* Create Record 14 */
	RetVal = image2ETFStype_14(
		pRecord, // pointer to record
		ImgCompressed, ImgCompLen, (int)SizeX, (int)SizeY, // image info
		PIXEL_DEPTH_8BPP, GB_RES_PPMM, // image pixel depth and resolution
		ImgCompAlg, // image compression
		ETFS_14_LIVE_SCAN_PLAN, // image impression
		"Green Bit Scanner", // source
		nullptr, // comment
		finger_position, // finger position
		amp_list, // amputation list, needed for left slap in our simulation
		boundaries, // boundary boxes
		NIST_QualityList, // NIST quality for each finger in slap
		SegmentationQualityList, // segmentation quality for each finger in slap
		nullptr // alternate quality
		);

	// check RetVal
	if (RetVal == AN2K_DLL_NO_ERROR)
	{
		// no error: set RetVal
		Console::WriteLine("Record 14 successfully created...");
		RetVal = GB_EXAMPLE_NO_ERROR;
	}
	else
	{
		// error: show retval and then set it to generic error
		Console::WriteLine("image2ETFStype_14 returned " + RetVal + " : cannot fill record 14...");
		*pRecord = nullptr;
		RetVal = GB_EXAMPLE_GENER_ERROR;
	}

	// free memory
	if (boundaries != nullptr )
	{
		free_FOUR_FING_BOUND(boundaries);
	}
	if (NIST_QualityList != nullptr )
	{
		free_FOUR_FING_QUALITY(NIST_QualityList);
		NIST_QualityList = nullptr;
	}
	if (SegmentationQualityList != nullptr)
	{
		free_ANSI_NIST_alt_qual_list(SegmentationQualityList);
	}

	return RetVal;

}