#ifndef GB_AN2K_H
#define GB_AN2K_H

using namespace System;

/********************************************************************************************/
// Initializes the LeftSlapAmpList
// Shows how should be filled a FING_AMP_LIST structure to be useful for Green Bit an2k 
// software library
/********************************************************************************************/
int		InitAmputationList(void);

/********************************************************************************************/
// This function allocs memory for storing the informations about the NIST quality for
// each segment extracted from a slap. Memory is allocated for the following global
// FOUR_FING_QUALITY structure:
extern FOUR_FING_QUALITY *NIST_QualityList;
//
// Parameters:
// - ObjID	: ID of the slap to be processed
/********************************************************************************************/
int		BuildQualityListFromObjID(int ObjID);

/********************************************************************************************/
// This function allocs memory and shows how the informations aboout the position of each
// segment inside a slap should be stored into a FOUR_FING_BOUND structure.
//
// Parameters:
// - pBoundaries: points to the FOUR_FING_BOUND structure to be filled (output)
// - ObjID	: ID of the slap to be processed (input)
/********************************************************************************************/
int		BuildBoundaryBoxListFromObjID(FOUR_FING_BOUND **pBoundaries,int ObjID);

/********************************************************************************************/
// This function allocs memory and shows how the informations aboout the GB quality of each
// segment inside a slap should be stored into a ALT_QUAL_LIST structure.
//
// Parameters:
// - pQualList: points to the ALT_QUAL_LIST structure to be filled (output)
// - ObjID	: ID of the slap to be processed (input)
/********************************************************************************************/
int		BuildSegmentationQualityListFromObjID(ALT_QUAL_LIST **pQualList,int ObjID);

/********************************************************************************************/
// This function returns an integer representing the finger position to be passed to the
// image2ETFStype_4 (...) function of the AN2K tool. Unlike the image2ETFStype_14 (...)
// function, it wants an integer instead of a string representing the finger position
//
// Parameters:
// - ObjID	: ID of the slap to be processed (input)
//
// Return:
// - a value >= 0 if valid finger position
// - a value < 0 in case of error
/********************************************************************************************/
int		getFingerPosForRec4FromObjID(Byte ObjID);

/********************************************************************************************/
// This function shows how a record 4 should correctly be built starting from input
// parameters. No preliminary operations are needed, except for getting finger position from
// the ID of the object (see getFingerPosForRec4FromObjID(...)).
//
// Parameters:
// - ObjID			: ID of the slap to be processed (input)
// - pRecord		: points to the record to be created (output)
// - ImgCompressed	: buffer of the image to be stored (input)
// - ImgCompLen		: size of the ImgCompressed buffer (input)
// - SizeX, SizeY	: dimensions of the ImgCompressed buffer (input)
// - ImgCompAlg		: AN2K (see an2k_dll.h) Id of the compression algorithm (input)
/********************************************************************************************/
int		CreateRec4FromObjID(Byte ObjID,
							RECORD **pRecord,
							Byte *ImgCompressed,int ImgCompLen,int SizeX, int SizeY,
							char *ImgCompAlg);

/********************************************************************************************/
// This function shows how a record 14 should correctly be built starting from input
// parameters.
// Operations performed:
//
// 1) Get Finger Position (a string)
// 2) Get current amputation list:
extern	FING_AMP_LIST *LeftSlapAmpList; // for left slap
//		NULL for right slap and two thumbs images)
// 3) Build bounding box list from segmentation info (see BuildBoundaryBoxListFromObjID(...))
// 4) Build GB alternate quality from segmentation info (see
//		BuildSegmentationQualityListFromObjID(...))
// 5) Create record 14
//
// Parameters:
// - ObjID			: ID of the slap to be processed (input)
// - pRecord		: points to the record to be created (output)
// - ImgCompressed	: buffer of the image to be stored (input)
// - ImgCompLen		: size of the ImgCompressed buffer (input)
// - SizeX, SizeY	: dimensions of the ImgCompressed buffer (input)
// - ImgCompAlg		: AN2K (see an2k_dll.h) Id of the compression algorithm (input)
/********************************************************************************************/
int		CreateRec14FromObjID(int ObjID,
							 RECORD **pRecord,
							 Byte *ImgCompressed,int ImgCompLen,int SizeX, int SizeY,
							 char *ImgCompAlg);

/********************************************************************************************/
// Creates a type-2 record with amputation list and insert it into an ANSI/NIST file. 
// Amputation list is defined this way:
// extern FING_AMP_LIST *LeftSlapAmpList;
//
// Parameters:
// - ansi_nist	: Points to the ANSI/NIST structure where the record created in this function
//		must be added
// - pRecord	: Points to the record to be added
/********************************************************************************************/
int		CreateRec2FromRollAmpList(ANSI_NIST *ansi_nist);

#endif	GB_AN2K_H