#include <stdafx.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <lfs_dll.h>
#include <Bozorth_dll.h>
#include "imageprocessing.h"
#include "GB_Segmentation.h"
#include "GB_Minutiae.h"


// to avoid warning with strcpy and so on
#pragma warning( disable : 4996 4793)

using namespace System;
using namespace System::IO;
using namespace System::Windows::Forms;

/****************************************************************************/
/************* GLOBAL VARIABLES *********************************************/
/****************************************************************************/
// minutiae for each flat
MINUTIAE *FlatMinutiae[10];

/****************************************************************************/
/************* ROUTINES *****************************************************/
/****************************************************************************/
// ObjID must be one of the defined in ImageProcessing.h
// returns the index in FlatMinutiae array
int		GetFlatMinutiaeIndexFromObjID(int ObjID)
{
	if (ObjID < LEFT_THUMB || ObjID > RIGHT_LITTLE) return - 1;
	return (ObjID - 3);
}

void	CalcMinForFingerInSlap(int FingerID, int GBFINIMG_FingerID)
{
	int RetVal, MinIndex;
	MINUTIAE **pMin;
	Byte *buffer;

	MinIndex = GetFlatMinutiaeIndexFromObjID(FingerID);
	if (MinIndex < 0)
	{
		// wrong FingerID
		Console::WriteLine("Cannot Calc Minutiae ...");
		return;
	}
	
	pMin = &(FlatMinutiae[MinIndex]);
	// if thumb, I need to know also the hand id
	int HandID = -1; // default value
	if (FingerID == LEFT_THUMB)
	{
		HandID = GBFINIMG_HAND_LEFT;
	}
	if (FingerID == RIGHT_THUMB)
	{
		HandID = GBFINIMG_HAND_RIGHT;
	}
	int IndexInList = SearchSlapIndexFromFingerID(GBFINIMG_FingerID,HandID);
	if (IndexInList < 0)
	{
		// wrong GBFINIMG_FingerID
		Console::WriteLine("Cannot Calc Minutiae ...");
		return;
	}
	
	buffer = CompleteSlapList[IndexInList].SegmentImagePtr;
	RetVal = get_minutiae(pMin, buffer, SEGM_SIZE_X, SEGM_SIZE_Y, 8, GB_RES_PPMM);
	// check retval
	if (RetVal != 0)
	{
		*pMin = nullptr;
		Console::WriteLine("Cannot Calc Minutiae ...");
	}
	else
	{
		Console::WriteLine("Minutiae succesfully calculated...");
	}

	return;
}

int		CalculateSlapMinutiae(unsigned char ObjID)
{
	// check input
	if (ObjID != LEFT_SLAP && ObjID != RIGHT_SLAP && ObjID != TWO_THUMBS)
	{
		return GB_EXAMPLE_PARAM_ERROR;
	}

	int FingerID;

	// calculate for index
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		FingerID = ((ObjID == LEFT_SLAP)?(LEFT_INDEX):(RIGHT_INDEX));
		Console::WriteLine("\n\tIndex...");
		CalcMinForFingerInSlap(FingerID,GBFINIMG_FINGER_INDEX);
	}
	
	// calculate for middle
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		FingerID = ((ObjID == LEFT_SLAP)?(LEFT_MIDDLE):(RIGHT_MIDDLE));
		Console::WriteLine("\n\tMiddle...");
		CalcMinForFingerInSlap(FingerID,GBFINIMG_FINGER_MIDDLE);
	}
	// calculate for ring: only right slap
	if (ObjID == RIGHT_SLAP)
	{
		FingerID = RIGHT_RING;
		Console::WriteLine("\n\tRing...");
		CalcMinForFingerInSlap(FingerID,GBFINIMG_FINGER_RING);
	}
	// calculate for little
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		FingerID = ((ObjID == LEFT_SLAP)?(LEFT_LITTLE):(RIGHT_LITTLE));
		Console::WriteLine("\n\tLittle...");
		CalcMinForFingerInSlap(FingerID,GBFINIMG_FINGER_LITTLE);
	}	

	// if left or right slap, we can return
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		return GB_EXAMPLE_NO_ERROR;
	}

	// left thumb
	FingerID = LEFT_THUMB;
	Console::WriteLine("\n\tLeft Thumb...");
	CalcMinForFingerInSlap(FingerID,GBFINIMG_FINGER_THUMB);

	// right thumb
	FingerID = RIGHT_THUMB;
	Console::WriteLine("\n\tRight Thumb...");
	CalcMinForFingerInSlap(FingerID,GBFINIMG_FINGER_THUMB);
	
	
	return GB_EXAMPLE_NO_ERROR;
}


void	CalculateRollMinutiaeAndCompare(Byte *frame,int SizeX,int SizeY, int ObjID)
{
	int RetVal;
	MINUTIAE *RollMin;

	RetVal = get_minutiae(&RollMin, // minutiae list (output)
		frame,SizeX,SizeY, // image frame and size
		8, // image pixel depth
		19.67 // image resolution in ppmm (19.67 corresponds to 500 dpi)
		);
	// check RetVal
	if (RetVal != 0)
	{
		Console::WriteLine("Cannot get minutiae for this image...");
		return;
	}
	
	Console::WriteLine("Minutiae for this image have been calculated...\n\nPerform sequence check...");
	/* perform sequence check */
	// get slap minutiae
	int FlatMinIndex = GetFlatMinutiaeIndexFromObjID(ObjID); // index inside FlatMinutiae array
	if (FlatMinIndex >= 0)
	{
		// call comparison algorithm
		int Score = boz_direct_call(200, // max number of minutiae for comparison
			RollMin, // minutiae to be compared
			FlatMinutiae[FlatMinIndex] // minutiae gallery
		);

		// show results
		if (Score > GB_MIN_COMPARISON_SCORE)
		{
			Console::WriteLine("Comparison OK: sequence check successful for this image...");
		}
		else
		{
			Console::WriteLine("Very Low matching score: verify that the roll image corresponds to the flat one...");
		}
	}
	else
	{
		Console::WriteLine("Cannot perform sequence check for this image: flat minutiae not found...");
	}
	

	return;
}