#include <stdafx.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gbnfiq.h>
#include "ImageProcessing.h"
#include "GB_Segmentation.h"
// To avoid compilation errors, redefine the
// followings: FAR, PASCAL and BYTE
#ifndef FAR
#define GB_FAR_DEFINED
#define FAR
#define PASCAL __stdcall
#define BYTE unsigned char
#endif	FAR
#include <GBFINIMG.h>
#ifdef GB_FAR_DEFINED // if are defined externally keep them defined
#undef FAR
#undef PASCAL
#undef BYTE
#endif GB_FAR_DEFINED


// to avoid warning with strcpy and so on
#pragma warning( disable : 4996 4793)

using namespace System;
using namespace System::IO;
using namespace System::Windows::Forms;

/****************************************************************************/
/************* GLOBAL VARIABLES *********************************************/
/****************************************************************************/

// segments
GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR CompleteSlapList[4];
// segments external info for left slap
GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO LeftSlapExternInfoList[3];

// NIST quality list to be stored in record 14
extern FOUR_FING_QUALITY *NIST_QualityList;

/****************************************************************************/
/************* ROUTINES *****************************************************/
/****************************************************************************/

int		SearchSlapIndexFromFingerID(int FingerID,int HandID)
{
	// check input
	if ( (FingerID == GBFINIMG_FINGER_THUMB) &&
		(HandID != GBFINIMG_HAND_LEFT && HandID != GBFINIMG_HAND_RIGHT)
		)
	{
		return -1;
	}
	// ok input
	int RetVal = -1;
	int MaxSearch = (FingerID == GBFINIMG_FINGER_THUMB)?2:4;
	for (int i = 0;i < MaxSearch; i ++)
	{
		if (CompleteSlapList[i].Finger == FingerID)
		{
			if ( (FingerID == GBFINIMG_FINGER_THUMB))
			{
				if (CompleteSlapList[i].Hand == HandID)
				{
					// found
					RetVal = i;
					i = 4; // exit from cycle
				}
			}
			else
			{
				// if not thumb, don't care hand
				RetVal = i;
				i = 4; // exit from cycle
			}
		}
	}

	return RetVal;
}

void	InitExternalInfoList(void)
{
	// little
	LeftSlapExternInfoList[0].Hand			= GBFINIMG_HAND_LEFT;
	LeftSlapExternInfoList[0].Finger		= GBFINIMG_FINGER_LITTLE;
	LeftSlapExternInfoList[0].FingerCenterX	= LITTLE_CENTER_X;
	LeftSlapExternInfoList[0].FingerCenterY	= LITTLE_CENTER_Y;

	// middle
	LeftSlapExternInfoList[1].Hand			= GBFINIMG_HAND_LEFT;
	LeftSlapExternInfoList[1].Finger		= GBFINIMG_FINGER_MIDDLE;
	LeftSlapExternInfoList[1].FingerCenterX	= MIDDLE_CENTER_X;
	LeftSlapExternInfoList[1].FingerCenterY	= MIDDLE_CENTER_Y;

	// index
	LeftSlapExternInfoList[2].Hand			= GBFINIMG_HAND_LEFT;
	LeftSlapExternInfoList[2].Finger		= GBFINIMG_FINGER_INDEX;
	LeftSlapExternInfoList[2].FingerCenterX	= INDEX_CENTER_X;
	LeftSlapExternInfoList[2].FingerCenterY	= INDEX_CENTER_Y;
}


int		InitSegmentDescriptorList(void)
{
	CompleteSlapList[0].SegmentImagePtr = new Byte[SEGM_SIZE_X * SEGM_SIZE_Y];
	if (CompleteSlapList[0].SegmentImagePtr == nullptr)
	{
		return GB_EXAMPLE_MEM_ERROR;
	}

	CompleteSlapList[1].SegmentImagePtr = new Byte[SEGM_SIZE_X * SEGM_SIZE_Y];
	if (CompleteSlapList[1].SegmentImagePtr == nullptr)
	{
		delete [] CompleteSlapList[0].SegmentImagePtr;
		return GB_EXAMPLE_MEM_ERROR;
	}

	CompleteSlapList[2].SegmentImagePtr = new Byte[SEGM_SIZE_X * SEGM_SIZE_Y];
	if (CompleteSlapList[2].SegmentImagePtr == nullptr)
	{
		delete [] CompleteSlapList[0].SegmentImagePtr;
		delete [] CompleteSlapList[1].SegmentImagePtr;
		return GB_EXAMPLE_MEM_ERROR;
	}

	CompleteSlapList[3].SegmentImagePtr = new Byte[SEGM_SIZE_X * SEGM_SIZE_Y];
	if (CompleteSlapList[3].SegmentImagePtr == nullptr)
	{
		delete [] CompleteSlapList[0].SegmentImagePtr;
		delete [] CompleteSlapList[1].SegmentImagePtr;
		delete [] CompleteSlapList[2].SegmentImagePtr;
		return GB_EXAMPLE_MEM_ERROR;
	}

	return GB_EXAMPLE_NO_ERROR;
}


void	DisplaySegmInfoForFingerInSlap(int FingerID,int NISTQualityListIndex,char *ANSI_NIST_FingID,int HandID)
{
	int RetVal, NIST_Quality;

	int IndexInList = SearchSlapIndexFromFingerID(FingerID,HandID);
	if (IndexInList >= 0)
	{
		// check GB quality
		if (CompleteSlapList[IndexInList].Quality < GB_PATTERN_QUALITY_MIN_THRESH)
		{
			Console::WriteLine("Warning: Finger GB Image Quality = " + CompleteSlapList[IndexInList].Quality +
				", should be >= " + GB_PATTERN_QUALITY_MIN_THRESH);
		}
		// check validity, completeness and inclination
		if (CompleteSlapList[IndexInList].PatternCompleteness < GB_PATTERN_COMPLETENESS_MIN_THRESH)
		{
			Console::WriteLine("Warning: Pattern completeness = " + CompleteSlapList[IndexInList].PatternCompleteness +
				", should be >= " + GB_PATTERN_COMPLETENESS_MIN_THRESH);
		}
		if (CompleteSlapList[IndexInList].PatternValidity < GB_PATTERN_VAIDITY_MIN_THRESH)
		{
			Console::WriteLine("Warning: Pattern validity = " + CompleteSlapList[IndexInList].PatternValidity +
				", should be >= " + GB_PATTERN_VAIDITY_MIN_THRESH);
		}
		
		if (CompleteSlapList[IndexInList].Inclination < GB_PATTERN_INCLINATION_MIN_THRESH ||
			CompleteSlapList[IndexInList].Inclination > GB_PATTERN_INCLINATION_MAX_THRESH)
		{
			Console::WriteLine("Warning: Pattern inclination = " + CompleteSlapList[IndexInList].Inclination +
				", should be between " + GB_PATTERN_INCLINATION_MIN_THRESH + " and " + GB_PATTERN_INCLINATION_MAX_THRESH);
		}
		// calculate NIST quality
		RetVal = GBNFIQ_GetImageQuality(
			CompleteSlapList[IndexInList].SegmentImagePtr,SEGM_SIZE_X,SEGM_SIZE_Y, // image parameters
			&NIST_Quality, // output NIST quality
			nullptr // minutiae info
			);
		// check retval
		String ^ MessageToDisplay;
		if (RetVal == GBNFIQ_EMPTY_IMG)
		{
			// not found
			MessageToDisplay = ("Finger not found...");
		}
		if (RetVal == GBNFIQ_TOO_FEW_MINUTIAE)
		{
			// not found
			MessageToDisplay = ("Too much Low NIST Quality image...");
		}
		else
		{
			// no error
			MessageToDisplay = ("Finger NIST Image Quality = " + NIST_Quality + "...");
			// save NIST quality into NIST Quality list
			if (NIST_QualityList != nullptr && 
				NISTQualityListIndex < NIST_QualityList->num_fingers &&
				NISTQualityListIndex >= 0)
			{
				FING_QUALITY * FingQualPtr = NIST_QualityList->fingers[NISTQualityListIndex];
				itoa(NIST_Quality,FingQualPtr->quality,10);
				strcpy(FingQualPtr->fing_id,ANSI_NIST_FingID);
			}
		}
		Console::WriteLine(MessageToDisplay);
	}
	else
	{
		// not found
		Console::WriteLine("Finger not found...");
	}

	return;
}

void	ShowSegmentationData(unsigned char ObjID)
{
	// build quality list
	if (BuildQualityListFromObjID(ObjID) != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Warning: Error in creating quality list, cannot save NIST qualities into record 14");
	}
	// show index
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		int NISTQualityListIndex = 0;
		Console::WriteLine("\n\tIndex...");
		if (ObjID == LEFT_SLAP)
		{
			DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_INDEX,NISTQualityListIndex,ETFS_14_FGP_LEFT_INDEX);
		}
		else
		{
			DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_INDEX,NISTQualityListIndex,ETFS_14_FGP_RIGHT_INDEX);
		}
	}
	
	// show middle
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		int NISTQualityListIndex = 1;
		Console::WriteLine("\n\tMiddle...");
		if (ObjID == LEFT_SLAP)
		{
			DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_MIDDLE,NISTQualityListIndex,ETFS_14_FGP_LEFT_MIDDLE);
		}
		else
		{
			DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_MIDDLE,NISTQualityListIndex,ETFS_14_FGP_RIGHT_MIDDLE);
		}
	}
	// show ring
	if (ObjID == RIGHT_SLAP)
	{
		int NISTQualityListIndex = 2;
		Console::WriteLine("\n\tRing...");
		DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_RING,NISTQualityListIndex,ETFS_14_FGP_RIGHT_RING);
	}
	// show little
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		int NISTQualityListIndex;
		Console::WriteLine("\n\tLittle...");
		if (ObjID == LEFT_SLAP)
		{
			NISTQualityListIndex = 2;
			DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_LITTLE,NISTQualityListIndex,ETFS_14_FGP_LEFT_LITTLE);
		}
		else
		{
			NISTQualityListIndex = 3;
			DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_LITTLE,NISTQualityListIndex,ETFS_14_FGP_RIGHT_LITTLE);
		}
	}

	// if right or left slap at this point we can return
	if (ObjID == LEFT_SLAP || ObjID == RIGHT_SLAP)
	{
		return;
	}

	// left thumb
	Console::WriteLine("\n\tLeft Thumb...");
	DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_THUMB,0,ETFS_14_FGP_LEFT_THUMB,GBFINIMG_HAND_LEFT);

	// right thumb
	Console::WriteLine("\n\tRight Thumb...");
	DisplaySegmInfoForFingerInSlap(GBFINIMG_FINGER_THUMB,1,ETFS_14_FGP_RIGHT_THUMB,GBFINIMG_HAND_RIGHT);
	Console::WriteLine("");
}
int		CalculateManualSegmentation(unsigned char *pImage,unsigned int SizeX,unsigned int SizeY)
{
	UInt32 DetectedSegmentImageNumber,DetectionFailureDiagnostic;

	// working buffer
	Byte *TempBuffer = new Byte[GBFINIMG_MEMORY_BUFFER_SIZE];
	if (TempBuffer == nullptr)
	{
		return GB_EXAMPLE_MEM_ERROR;
	}

	int RetVal = GBFINIMG_ProcessImage(
		// input image
			pImage,	SizeX,SizeY, // IMAGE parameters
			GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4,	// specifies type of input image
		// options
			GBFINIMG_HALO_LATENT_ELIMINATION, // Enhancement
		// segment list and parameters
			CompleteSlapList, // Segments list
			SEGM_SIZE_X, SEGM_SIZE_Y, // XY-size of fingerprint segment image
			&DetectedSegmentImageNumber, // number of detected fingerprint segments
		// Diagnostic
			&DetectionFailureDiagnostic, // Diagnostic
		// External info list
			LeftSlapExternInfoList, // SEMI-AUTOMATIC mode
			3, // external info list size
		// working buffer
			TempBuffer, GBFINIMG_MEMORY_BUFFER_SIZE
		);

	// free allocated memory
	delete [] TempBuffer;

	// check retval
	if (RetVal != GBFINIMG_NO_ERROR)
	{
		if (RetVal == GBFINIMG_ERROR_PARAMETER)
		{
			Console::WriteLine("GBFINIMG_ProcessImage returned error parameter...");
		}
		else
		{
			Console::WriteLine("GBFINIMG_ProcessImage returned error " + RetVal + " ...");
		}
		return GB_EXAMPLE_GENER_ERROR;
	}

	// in manual mode do not check diagnostic	

	// check segment number
	if (DetectedSegmentImageNumber != 3)
	{
		Console::WriteLine("Detected segment number = " + DetectedSegmentImageNumber + ": should be 3");
		Console::WriteLine("Segmentation failed...");
		return GB_EXAMPLE_UNACCEPTABLE_SEGM_RES_ERROR;
	}

	return GB_EXAMPLE_NO_ERROR;
}

int		CalculateAutomaticSegmentation(unsigned char *pImage,unsigned int SizeX,unsigned int SizeY,Byte ObjID)
{
	UInt32	DetectedSegmentImageNumber,DetectionFailureDiagnostic;
	UInt32	ImageType;
	Byte	ExpectedSegmentsNumber;

	if (ObjID == RIGHT_SLAP)
	{
		ImageType = GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4;
		ExpectedSegmentsNumber = 4;
	}
	else
	{
		ImageType = GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2;
		ExpectedSegmentsNumber = 2;
	}

	// working buffer
	Byte *TempBuffer = new Byte[GBFINIMG_MEMORY_BUFFER_SIZE];
	if (TempBuffer == nullptr)
	{
		return GB_EXAMPLE_MEM_ERROR;
	}

	int RetVal = GBFINIMG_ProcessImage(
		// input image
			pImage,	SizeX,SizeY, // IMAGE parameters
			ImageType,	// specifies type of input image
		// options
			GBFINIMG_HALO_LATENT_ELIMINATION, // Enhancement
		// segment list and parameters
			CompleteSlapList, // Segments list
			SEGM_SIZE_X, SEGM_SIZE_Y, // XY-size of fingerprint segment image
			&DetectedSegmentImageNumber, // number of detected fingerprint segments
		// Diagnostic
			&DetectionFailureDiagnostic, // Diagnostic
		// External info list
			nullptr,0, // FULL-AUTOMATIC mode
		// working buffer
			TempBuffer, GBFINIMG_MEMORY_BUFFER_SIZE
		);

	// free allocated memory
	delete [] TempBuffer;

	// check retval
	if (RetVal != GBFINIMG_NO_ERROR)
	{
		if (RetVal == GBFINIMG_ERROR_PARAMETER)
		{
			Console::WriteLine("GBFINIMG_ProcessImage returned error parameter...");
		}
		else
		{
			Console::WriteLine("GBFINIMG_ProcessImage returned error " + RetVal + " ...");
		}
		return GB_EXAMPLE_GENER_ERROR;
	}

	// check diagnostic
	if (DetectionFailureDiagnostic != 0)
	{
		// show diag
		if (DetectionFailureDiagnostic & GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX)
		{
			Console::WriteLine(LEFT_RIGHT_HAND_MIX_MESSAGE);
		}
		if (DetectionFailureDiagnostic & GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP)
		{
			Console::WriteLine(INCOMPLETE_SLAP_MESSAGE);
		}
		if (DetectionFailureDiagnostic & GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP)
		{
			Console::WriteLine(AMBIGUOUS_SLAP_MESSAGE);
		}

		// check segm number
		if (DetectedSegmentImageNumber != ExpectedSegmentsNumber )
		{
			Console::WriteLine("Segmentation failed...");
			return GB_EXAMPLE_UNACCEPTABLE_SEGM_RES_ERROR;
		}
		else
		{
			Console::WriteLine("Verify that segmentation result is correct...");
			return GB_EXAMPLE_ACCEPTABLE_SEGM_RES_ERROR;
		}
	}

	// check segment number
	if (DetectedSegmentImageNumber != ExpectedSegmentsNumber)
	{
		Console::WriteLine("Detected segment number = " + DetectedSegmentImageNumber + ": should be " + ExpectedSegmentsNumber);
		Console::WriteLine("Segmentation failed...");
		return GB_EXAMPLE_UNACCEPTABLE_SEGM_RES_ERROR;
	}

	return GB_EXAMPLE_NO_ERROR;
}

