// TestAn2k.cpp : definisce il punto di ingresso dell'applicazione console.
//

#include "stdafx.h"
#include "windows.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "GB_AN_Engine.h"
#include "GB_AN2011.h"
#include "an2k_dll.h"

#pragma warning(disable: 4996)

/*****************************************
** UTILITIES
*****************************************/
void ShowError(int error, char *callfunction)
{
	printf("Function %s returned %d\n", callfunction,error);
		getchar();
		getchar();
}

void SaveOnFile(char *fname, unsigned char*outBuffer, int bufLen)
{
	FILE *fp = fopen(fname, "wb");
	if (fp == NULL){
		printf ("CANNOT OPEN %s\n",fname);
		return;
	}
	fwrite(outBuffer,sizeof(unsigned char),bufLen,fp);
	fclose(fp);

	printf ("FILE %s SAVED\n",fname);
	return;
}



#define IMG_SX	500
#define IMG_SY	500
int GB_AN_ENGINE_EXAMPLE_CreateImageBuffer(unsigned char **imgBuff, int*pImgBuffLen, int *pSx, int * pSy)
{
	unsigned char *dummyPtr, dummyCounter;
	*imgBuff = new unsigned char[IMG_SX * IMG_SY];
	if (*imgBuff == NULL)
	{
		return AN2K_DLL_MEMORY;
	}
	*pImgBuffLen = IMG_SX * IMG_SY;
	*pSx = IMG_SX;
	*pSy = IMG_SY;

	dummyPtr = *imgBuff;
	dummyCounter = 0;
	for (int i = 0; i < IMG_SX * IMG_SY; i ++)
	{
		*dummyPtr = dummyCounter; dummyPtr++; dummyCounter++;
	}
	return AN2K_DLL_NO_ERROR;
}


/*******************************************************************
Record 2 Description
FIELD 1:
	LEN
FIELD 2:
	IDC
FIELD 3:
	NME, 2 items:
	- Name (String)
	- Surname (String)
FIELD 4:
	DOB (Date Of Birth)
	- YYYYMMDD (String)
FIELD 5:
	DOC (Date Of Court Reporting)
	- YYYYMMDD (String)
FIELD 6: 
	RSN (Reason)
	- Court Reporting Reason (String)
FIELD 7:
	PLR (Old Police Records), from 0 to 2 subfields made this way:
	- DOR (Date of Record), YYYYMMDD (String)
	- RSR (Reason of Record) (String)
********************************************************************/
typedef struct 
{
	GB_AN2011_DateStruct dor;
	char rsr[50];
}GB_AN_ENGINE_EXAMPLE_PoliceRecordStruct;
////////////////////////////////////////////
// RECORD CREATION
////////////////////////////////////////////
#define GB_AN_ENGINE_EXAMPLE_REC2_NAME_FIELD_ID	3
#define GB_AN_ENGINE_EXAMPLE_REC2_DOB_FIELD_ID	4
#define GB_AN_ENGINE_EXAMPLE_REC2_DOC_FIELD_ID	5
#define GB_AN_ENGINE_EXAMPLE_REC2_RSN_FIELD_ID	6
#define GB_AN_ENGINE_EXAMPLE_REC2_PLR_FIELD_ID	7
int GB_AN_ENGINE_EXAMPLE_CreateRecordType2(
	RECORD **pRec,
	int idc,
	GB_AN_ENGINE_String name,
	GB_AN_ENGINE_String surname,
	GB_AN2011_DateStruct *dateOfBirth,
	GB_AN2011_DateStruct *dateOfCourtReporting,
	GB_AN_ENGINE_String courtReportingReason,
	GB_AN_ENGINE_EXAMPLE_PoliceRecordStruct *policeRecordsArray, int policeRecordsNum
	)
{
	RECORD *record = NULL;
	FIELD *dummyField;
	SUBFIELD *dummySubfield;
	int Retval;
	char dummyString[200];
	char *strArray[10];

	///////////////////////////
	// check parameters
	///////////////////////////
	if (pRec == NULL)
	{
		return AN2K_DLL_BAD_PARAMETER;
	}
	if (name == NULL)
	{
		return AN2K_DLL_BAD_PARAMETER;
	}
	if (surname == NULL)
	{
		return AN2K_DLL_BAD_PARAMETER;
	}
	if (dateOfBirth == NULL)
	{
		return AN2K_DLL_BAD_PARAMETER;
	}
	if (courtReportingReason == NULL)
	{
		return AN2K_DLL_BAD_PARAMETER;
	}
	if (dateOfCourtReporting == NULL)
	{
		return AN2K_DLL_BAD_PARAMETER;
	}
	if (policeRecordsArray != NULL && policeRecordsNum > 0)
	{
		if (policeRecordsNum > 2)
		{
			return AN2K_DLL_BAD_PARAMETER;
		}
	}

	/////////////////////////////////////////
	// CREATE RECORD
	/////////////////////////////////////////
	Retval = GB_AN_ENGINE_CreateEmptyTaggedRecord(&record, 2, idc);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_ENGINE_CreateEmptyTaggedRecord");
		return Retval;
	}

	/////////////////////////////////////////
	// INSERT NAME FIELD
	/////////////////////////////////////////
	strArray[0] = name;
	strArray[1] = surname;
	Retval = GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords(&dummyField,2,GB_AN_ENGINE_EXAMPLE_REC2_NAME_FIELD_ID,strArray,2);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, NULL, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords");
		return Retval;
	}
	Retval = GB_AN_ENGINE_InsertFieldIntoTaggedRecord(record,dummyField);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, dummyField, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_InsertFieldIntoTaggedRecord");
		return Retval;
	}

	/////////////////////////////////////////
	// INSERT DATE OF BIRTH FIELD
	/////////////////////////////////////////
	sprintf(dummyString, "%04d%02d%02d", dateOfBirth->Year, dateOfBirth->Month, dateOfBirth->Day);
	strArray[0] = dummyString;
	Retval = GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords(&dummyField,2,GB_AN_ENGINE_EXAMPLE_REC2_DOB_FIELD_ID,strArray,1);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, NULL, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords");
		return Retval;
	}
	Retval = GB_AN_ENGINE_InsertFieldIntoTaggedRecord(record,dummyField);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, dummyField, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_InsertFieldIntoTaggedRecord");
		return Retval;
	}

	/////////////////////////////////////////
	// INSERT DATE OF REPORTING FIELD
	/////////////////////////////////////////
	sprintf(dummyString, "%04d%02d%02d", dateOfCourtReporting->Year, dateOfCourtReporting->Month, dateOfCourtReporting->Day);
	strArray[0] = dummyString;
	Retval = GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords(&dummyField,2,GB_AN_ENGINE_EXAMPLE_REC2_DOC_FIELD_ID,strArray,1);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, NULL, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords");
		return Retval;
	}
	Retval = GB_AN_ENGINE_InsertFieldIntoTaggedRecord(record,dummyField);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, dummyField, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_InsertFieldIntoTaggedRecord");
		return Retval;
	}

	/////////////////////////////////////////
	// INSERT COURT REPORTING REASON FIELD
	/////////////////////////////////////////
	strArray[0] = courtReportingReason;
	Retval = GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords(&dummyField,2,GB_AN_ENGINE_EXAMPLE_REC2_RSN_FIELD_ID,strArray,1);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, NULL, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords");
		return Retval;
	}
	Retval = GB_AN_ENGINE_InsertFieldIntoTaggedRecord(record,dummyField);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		GB_AN_ENGINE_Free(NULL, record, dummyField, NULL, NULL);
		ShowError(Retval,"GB_AN_ENGINE_InsertFieldIntoTaggedRecord");
		return Retval;
	}

	/////////////////////////////////////////
	// INSERT COURT REPORTING REASON FIELD
	/////////////////////////////////////////
	if (policeRecordsArray != NULL && policeRecordsNum > 0)
	{
		int ret = GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords(&dummyField,2, GB_AN_ENGINE_EXAMPLE_REC2_PLR_FIELD_ID);
		if(ret != AN2K_DLL_NO_ERROR)
		{
			GB_AN_ENGINE_Free(NULL, record, NULL, NULL, NULL);
			ShowError(Retval,"GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords");
			return ret;
		}
		for (int i = 0; i < policeRecordsNum; i ++)
		{		
			sprintf(dummyString, "%04d%02d%02d",
				(policeRecordsArray[i].dor).Year, (policeRecordsArray[i].dor).Month, (policeRecordsArray[i].dor).Day);
			strArray[0] = dummyString;
			strArray[1] = policeRecordsArray[i].rsr;

			ret = GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords(&dummySubfield, strArray, 2);
			if(ret != AN2K_DLL_NO_ERROR)
			{
				GB_AN_ENGINE_Free(NULL, record, dummyField, NULL, NULL);
				ShowError(Retval,"GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords");
				return ret;
			}

			ret = GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords(dummyField, dummySubfield);
			if(ret != AN2K_DLL_NO_ERROR)
			{
				GB_AN_ENGINE_Free(NULL, record, dummyField, dummySubfield, NULL);
				ShowError(Retval,"GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords");
				return ret;
			}
		}

		// append field to record
		ret = GB_AN_ENGINE_InsertFieldIntoTaggedRecord(record, dummyField);
		if(ret != AN2K_DLL_NO_ERROR)
		{
			GB_AN_ENGINE_Free(NULL, record, dummyField, NULL, NULL);
			ShowError(Retval,"GB_AN_ENGINE_InsertFieldIntoTaggedRecord");
			return ret;
		}
	}

	*pRec = record;
	return AN2K_DLL_NO_ERROR;
}
////////////////////////////////////////////
// RECORD GET INFO
////////////////////////////////////////////
int GB_AN_ENGINE_EXAMPLE_GetType2Info(
	RECORD *recToRead,
	int *pIdc,
	GB_AN_ENGINE_String* pName, 
	GB_AN_ENGINE_String* pSurname,
	GB_AN2011_DateStruct *pDateOfBirth,// if !NULL, will be filled
	GB_AN2011_DateStruct *pDateOfCourtReporting,// if !NULL, will be filled
	GB_AN_ENGINE_String *pCourtReportingReason,
	GB_AN_ENGINE_EXAMPLE_PoliceRecordStruct **pPoliceRecordsArray, int *pPoliceRecordsNum
	)
{
	int RetVal;
	FIELD *dummyField;
	GB_AN_ENGINE_StringArray Values;
	int Num;

	///////////////////////////
	// check parameters
	///////////////////////////
	if (recToRead == NULL)
	{
		return AN2K_DLL_BAD_PARAMETER;
	}

	///////////////////////////////////
	// IDC
	///////////////////////////////////
	if (pIdc != NULL)
	{
		RetVal = GB_AN_ENGINE_SearchFieldInTaggedRecord(recToRead, 2, &dummyField);
		if(RetVal != AN2K_DLL_NO_ERROR)
		{
			ShowError(RetVal,"GB_AN_ENGINE_SearchFieldInTaggedRecord");
			return RetVal;
		}
		if (dummyField != NULL)
		{
			RetVal = GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(dummyField, &Values, &Num);
			if (RetVal != AN2K_DLL_NO_ERROR)
			{
				ShowError(RetVal,"GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords");
				return RetVal;
			}
			if (Num < 1 || Values[0] == NULL)
			{
				*pIdc = -1;
			}
			else
			{
				*pIdc = atoi(Values[0]);
			}
			GB_AN_ENGINE_FreeStringArray(Values,Num);
		}
		else
		{
			*pIdc = -1;
		}
	}

	///////////////////////////////////
	// NAME-SURNAME
	///////////////////////////////////
	if (pName != NULL && pSurname != NULL)
	{
		RetVal = GB_AN_ENGINE_SearchFieldInTaggedRecord(recToRead, GB_AN_ENGINE_EXAMPLE_REC2_NAME_FIELD_ID, &dummyField);
		if(RetVal != AN2K_DLL_NO_ERROR)
		{
			ShowError(RetVal,"GB_AN_ENGINE_SearchFieldInTaggedRecord");
			return RetVal;
		}
		if (dummyField != NULL)
		{
			RetVal = GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(dummyField, &Values, &Num);
			if (RetVal != AN2K_DLL_NO_ERROR)
			{
				ShowError(RetVal,"GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords");
				return RetVal;
			}
			if (Num < 2
				|| Values[0] == NULL
				|| Values[1] == NULL
			)
			{
				*pName = NULL;
				*pSurname = NULL;
			}
			else
			{
				*pName = (char *)malloc(strlen(Values[0]) + 1);
				if (*pName == NULL)
				{
					GB_AN_ENGINE_FreeStringArray(Values,Num);
					return AN2K_DLL_MEMORY;
				}
				strcpy(*pName,Values[0]);
				*pSurname = (char *)malloc(strlen(Values[1]) + 1);
				if (*pSurname == NULL)
				{
					free(*pName);
					GB_AN_ENGINE_FreeStringArray(Values,Num);
					return AN2K_DLL_MEMORY;
				}
				strcpy(*pSurname,Values[1]);
			}
			GB_AN_ENGINE_FreeStringArray(Values,Num);
		}
		else
		{
			*pName = NULL;
			*pSurname = NULL;
		}
	}

	///////////////////////////////////
	// DOB
	///////////////////////////////////
	if (pDateOfBirth != NULL)
	{
		RetVal = GB_AN_ENGINE_SearchFieldInTaggedRecord(recToRead, GB_AN_ENGINE_EXAMPLE_REC2_DOB_FIELD_ID, &dummyField);
		if (RetVal != AN2K_DLL_NO_ERROR)
		{
			ShowError(RetVal,"GB_AN_ENGINE_SearchFieldInTaggedRecord");
			return RetVal;
		}

		if (dummyField != NULL)
		{
			RetVal = GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(dummyField, &Values, &Num);
			if (RetVal != AN2K_DLL_NO_ERROR)
			{
				ShowError(RetVal,"GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords");
				return RetVal;
			}

			if (Num != 1 || Values[0] == NULL)
			{
				pDateOfBirth->Year = 0;
				pDateOfBirth->Month = 0;
				pDateOfBirth->Day = 0;
			}
			else if (strlen(Values[0]) != 8)
			{
				pDateOfBirth->Year = 0;
				pDateOfBirth->Month = 0;
				pDateOfBirth->Day = 0;
			}
			else
			{
				///////////////////////////
				// parse date
				///////////////////////////
				char pDest[5], *pSource = Values[0];
				// year
				memcpy(pDest, pSource, 4);
				pDest[4] = 0;
				pDateOfBirth->Year = atoi(pDest);
				// month
				pSource += 4;
				memcpy(pDest, pSource, 2);
				pDest[2] = 0;
				pDateOfBirth->Month = atoi(pDest);
				// day
				pSource += 2;
				memcpy(pDest, pSource, 2);
				pDest[2] = 0;
				pDateOfBirth->Day = atoi(pDest);
			}

			GB_AN_ENGINE_FreeStringArray(Values,Num);
		}
		else
		{
			pDateOfBirth->Year = 0;
			pDateOfBirth->Month = 0;
			pDateOfBirth->Day = 0;
		}
	}
	///////////////////////////////////
	// DOC
	///////////////////////////////////
	if (pDateOfCourtReporting != NULL)
	{
		RetVal = GB_AN_ENGINE_SearchFieldInTaggedRecord(recToRead, GB_AN_ENGINE_EXAMPLE_REC2_DOC_FIELD_ID, &dummyField);
		if (RetVal != AN2K_DLL_NO_ERROR)
		{
			ShowError(RetVal,"GB_AN_ENGINE_SearchFieldInTaggedRecord");
			return RetVal;
		}

		if (dummyField != NULL)
		{
			RetVal = GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(dummyField, &Values, &Num);
			if (RetVal != AN2K_DLL_NO_ERROR)
			{
				ShowError(RetVal,"GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords");
				return RetVal;
			}

			if (Num != 1 || Values[0] == NULL)
			{
				pDateOfCourtReporting->Year = 0;
				pDateOfCourtReporting->Month = 0;
				pDateOfCourtReporting->Day = 0;
			}
			else if (strlen(Values[0]) != 8)
			{
				pDateOfCourtReporting->Year = 0;
				pDateOfCourtReporting->Month = 0;
				pDateOfCourtReporting->Day = 0;
			}
			else
			{
				///////////////////////////
				// parse date
				///////////////////////////
				char pDest[5], *pSource = Values[0];
				// year
				memcpy(pDest, pSource, 4);
				pDest[4] = 0;
				pDateOfCourtReporting->Year = atoi(pDest);
				// month
				pSource += 4;
				memcpy(pDest, pSource, 2);
				pDest[2] = 0;
				pDateOfCourtReporting->Month = atoi(pDest);
				// day
				pSource += 2;
				memcpy(pDest, pSource, 2);
				pDest[2] = 0;
				pDateOfCourtReporting->Day = atoi(pDest);
			}

			GB_AN_ENGINE_FreeStringArray(Values,Num);
		}
		else
		{
			pDateOfCourtReporting->Year = 0;
			pDateOfCourtReporting->Month = 0;
			pDateOfCourtReporting->Day = 0;
		}
	}
	///////////////////////////////////
	// COURT REPORTING REASON
	///////////////////////////////////
	if (pCourtReportingReason != NULL)
	{
		RetVal = GB_AN_ENGINE_SearchFieldInTaggedRecord(recToRead, GB_AN_ENGINE_EXAMPLE_REC2_RSN_FIELD_ID, &dummyField);
		if(RetVal != AN2K_DLL_NO_ERROR)
		{
			ShowError(RetVal,"GB_AN_ENGINE_SearchFieldInTaggedRecord");
			return RetVal;
		}
		if (dummyField != NULL)
		{
			RetVal = GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(dummyField, &Values, &Num);
			if (RetVal != AN2K_DLL_NO_ERROR)
			{
				ShowError(RetVal,"GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords");
				return RetVal;
			}
			if (Num < 1
				|| Values[0] == NULL
			)
			{
				*pCourtReportingReason = NULL;
			}
			else
			{
				*pCourtReportingReason = (char *)malloc(strlen(Values[0]) + 1);
				if (*pCourtReportingReason == NULL)
				{
					GB_AN_ENGINE_FreeStringArray(Values,Num);
					return AN2K_DLL_MEMORY;
				}
				strcpy(*pCourtReportingReason,Values[0]);
			}
			GB_AN_ENGINE_FreeStringArray(Values,Num);
		}
		else
		{
			*pCourtReportingReason = NULL;
		}
	}

	if (pPoliceRecordsArray != NULL && pPoliceRecordsNum != NULL)
	{
		RetVal = GB_AN_ENGINE_SearchFieldInTaggedRecord(recToRead, GB_AN_ENGINE_EXAMPLE_REC2_PLR_FIELD_ID, &dummyField);
		if (RetVal != AN2K_DLL_NO_ERROR)
		{
			ShowError(RetVal,"GB_AN_ENGINE_SearchFieldInTaggedRecord");
			return RetVal;
		}
		if (dummyField != NULL && dummyField->num_subfields > 0 && dummyField->num_subfields <= 2)
		{
			*pPoliceRecordsArray = (GB_AN_ENGINE_EXAMPLE_PoliceRecordStruct *)malloc(
				(dummyField->num_subfields) * sizeof(GB_AN_ENGINE_EXAMPLE_PoliceRecordStruct));
			if (*pPoliceRecordsArray == NULL)
			{
				*pPoliceRecordsNum = 0;
				return AN2K_DLL_MEMORY;
			}
			*pPoliceRecordsNum = dummyField->num_subfields;
			for (int i = 0; i < dummyField->num_subfields; i ++)
			{
				SUBFIELD *dummySf = dummyField->subfields[i];
				if (dummySf == NULL)
				{
					*pPoliceRecordsNum = 0;
					free(*pPoliceRecordsArray);
					*pPoliceRecordsArray = NULL;
					return AN2K_DLL_BAD_PARAMETER;
				}

				RetVal = GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords(dummySf, &Values, &Num);
				if (RetVal != AN2K_DLL_NO_ERROR)
				{
					ShowError(RetVal,"GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords");
					*pPoliceRecordsNum = 0;
					free(*pPoliceRecordsArray);
					*pPoliceRecordsArray = NULL;
					return RetVal;
				}

				if (Num != 2 || Values[0] == NULL || Values[1] == NULL)
				{
					*pPoliceRecordsNum = 0;
					free(*pPoliceRecordsArray);
					*pPoliceRecordsArray = NULL;
					GB_AN_ENGINE_FreeStringArray(Values,Num);
					return AN2K_DLL_BAD_PARAMETER;
				}

				// DOR
				///////////////////////////
				// parse date
				///////////////////////////
				char pDest[5], *pSource = Values[0];
				// year
				memcpy(pDest, pSource, 4);
				pDest[4] = 0;
				(*pPoliceRecordsArray)[i].dor.Year = atoi(pDest);
				// month
				pSource += 4;
				memcpy(pDest, pSource, 2);
				pDest[2] = 0;
				(*pPoliceRecordsArray)[i].dor.Month = atoi(pDest);
				// day
				pSource += 2;
				memcpy(pDest, pSource, 2);
				pDest[2] = 0;
				(*pPoliceRecordsArray)[i].dor.Day = atoi(pDest);

				// RSR
				if (strlen(Values[1]) > 49)
				{
					Values[1][49] = '\0';
				}
				strcpy(((*pPoliceRecordsArray)[i]).rsr,Values[1]);
				
				GB_AN_ENGINE_FreeStringArray(Values,Num);
			}

		}
		else
		{
			*pPoliceRecordsArray = NULL;
			*pPoliceRecordsNum = 0;
		}
	}

	return AN2K_DLL_NO_ERROR;
}


/*********************************************
** MAIN
*********************************************/
int _tmain(int argc, _TCHAR* argv[])
{
	ANSI_NIST *ansinist = NULL;
	RECORD *dummyRecord = NULL;
	int idc = 0, positionInAnsiNist = 1;
	///////////////////////////////////////////////////
	// CREATE ANSI NIST
	///////////////////////////////////////////////////
	printf("///////////////////////////////////////////////////\n");
	printf("// CREATE ANSI NIST WITH AN2011 TYPE-1 RECORD\n");
	printf("///////////////////////////////////////////////////\n");
	GB_AN2011_DateStruct DateOfTransaction;
	GB_AN2011_ApplicationProfileStruct aps[2];
	GB_AN2011_GmtStruct gmt;

	DateOfTransaction.Year = 2013;
	DateOfTransaction.Month = 1;
	DateOfTransaction.Day = 29;

	aps[0].AppProfName = "apn0";
	aps[0].AppProfOrg = "apo0";
	aps[0].AppProfVer = "apv0";
	aps[1].AppProfName = "apn1";
	aps[1].AppProfOrg = "apo1";
	aps[1].AppProfVer = "apv1";

	gmt.Year = 2014;
	gmt.Month = 2;
	gmt.Day = 21;
	gmt.hour = 9;
	gmt.minute = 57;
	gmt.second = 25;

	int Retval = GB_AN2011_CreateAnsiNistStruct(
		&ansinist,"TestTot",&DateOfTransaction,8,"DestAg","OrAg","TCN1234","TCR1234",(float)19.69,(float)19.69,
		"DomName","DV1.0.0.0",&gmt,0,GB_TABLE_AN2011_4_CHAR_ENCODING_NAME_ASCII,"1.0",aps,2,"DAN","OAN");

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN2011_CreateAnsiNistStruct");
		return Retval;
	}

	int Ver1_1, Priority1_1, charIndex1_1,apsNum1_1;
	char *Tot1_1, *DestAgency1_1, *OrigAgency1_1, *TransControlNum1_1, *TransControlRef1_1, *DomN1_1, *DomV1_1,
		*charName1_1, *charVersion1_1, *destAgName1_1, *origAgName1_1;
	float NSR1_1, NTR1_1;
	GB_AN2011_GmtStruct Gmt1_1;
	GB_AN2011_DateStruct DateOfTransaction1_1;
	GB_AN2011_ApplicationProfileStruct *aps1_1;

	Retval = GB_AN2011_GetType1RecordInfo(
		ansinist->records[0], 
		&Ver1_1,
		&Tot1_1,
		&DateOfTransaction1_1,
		&Priority1_1,
		&DestAgency1_1,
		&OrigAgency1_1,
		&TransControlNum1_1,
		&TransControlRef1_1, 
		&NSR1_1, &NTR1_1,
		&DomN1_1,&DomV1_1,
		&Gmt1_1,
		&charIndex1_1, &charName1_1, &charVersion1_1,
		&aps1_1, &apsNum1_1,
		&destAgName1_1, &origAgName1_1
	);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN2011_GetType1RecordInfo");
		return Retval;
	}

	printf("Version = %d\nTot = %s\nDate = %d-%d-%d\nPriority = %d\n",
		Ver1_1, Tot1_1,
		DateOfTransaction1_1.Year, DateOfTransaction1_1.Month, DateOfTransaction1_1.Day,
		Priority1_1);

	printf("DestAgency = %s\nOrigAgency = %s\n", DestAgency1_1, OrigAgency1_1);

	printf("TransControlNum = %s\nTransControlRef = %s\n", TransControlNum1_1, TransControlRef1_1);

	printf("NSR = %f\nNTR = %f\nDomN = %s\nDomV = %s\n", NSR1_1, NTR1_1, DomN1_1, DomV1_1);

	printf("charName = %s\ncharVersion = %s\n", charName1_1, charVersion1_1);

	for (int i = 0; i < apsNum1_1; i ++)
	{
		printf("\taps1[%d].AppProfOrg = %s\n", i,aps1_1[i].AppProfOrg);
		printf("\taps1[%d].AppProfName = %s\n", i,aps1_1[i].AppProfName);
		printf("\taps1[%d].AppProfVer = %s\n", i,aps1_1[i].AppProfVer);
	}

	printf("destAgName = %s\norigAgName = %s\n", destAgName1_1, origAgName1_1);

	GB_AN_ENGINE_FreeGenericMem(DestAgency1_1);
	GB_AN_ENGINE_FreeGenericMem(OrigAgency1_1);
	GB_AN_ENGINE_FreeGenericMem(TransControlNum1_1);
	GB_AN_ENGINE_FreeGenericMem(TransControlRef1_1);
	GB_AN_ENGINE_FreeGenericMem(DomN1_1);
	GB_AN_ENGINE_FreeGenericMem(DomV1_1);
	GB_AN_ENGINE_FreeGenericMem(charName1_1);
	GB_AN_ENGINE_FreeGenericMem(charVersion1_1);
	GB_AN_ENGINE_FreeGenericMem(aps1_1);
	GB_AN_ENGINE_FreeGenericMem(destAgName1_1);
	GB_AN_ENGINE_FreeGenericMem(origAgName1_1);

	printf("\n");
	printf("\n");

	///////////////////////////////////////////////////
	// CUSTOM TYPE-2 RECORD
	///////////////////////////////////////////////////
	printf("///////////////////////////////////////////////////\n");
	printf("// CREATE CUSTOM TYPE-2 RECORD\n");
	printf("///////////////////////////////////////////////////\n");

	GB_AN2011_DateStruct dob, doc;
	GB_AN_ENGINE_EXAMPLE_PoliceRecordStruct polRecArray[2];

	dob.Year = 1984; dob.Month = 2; dob.Day = 22;
	doc.Year = 2013; doc.Month = 10; doc.Day = 11;
	polRecArray[0].dor.Year = 2000; polRecArray[0].dor.Month = 5; polRecArray[0].dor.Day = 7;
	sprintf(polRecArray[0].rsr,"Drug Possession");
	polRecArray[1].dor.Year = 2009; polRecArray[1].dor.Month = 7; polRecArray[1].dor.Day = 27;
	sprintf(polRecArray[1].rsr,"Drug Dealing");
	Retval = GB_AN_ENGINE_EXAMPLE_CreateRecordType2(&dummyRecord,idc ++,"John","Smith",
		&dob,&doc,"Robbery",polRecArray,2);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_ENGINE_EXAMPLE_CreateRecordType2");
		return -1;
	}

	Retval = GB_AN_ENGINE_DeleteFieldFromTaggedRecord(dummyRecord, 7);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_ENGINE_DeleteFieldFromTaggedRecord");
		return -1;
	}

	Retval = GB_AN_ENGINE_InsertRecordInAnsiNist(dummyRecord,ansinist,positionInAnsiNist ++,FALSE);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_Engine_InsertRecordInAnsiNist");
		return -1;
	}

	Retval = GB_AN_ENGINE_DeleteFieldFromTaggedRecord(dummyRecord, 6);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_ENGINE_DeleteFieldFromTaggedRecord");
		return -1;
	}

	int idc_2, prs_2; 
	GB_AN_ENGINE_String name_2, surname_2, crr_2;
	GB_AN_ENGINE_EXAMPLE_PoliceRecordStruct *prs;

	Retval = GB_AN_ENGINE_EXAMPLE_GetType2Info(dummyRecord, &idc_2, &name_2, &surname_2,
		&dob, & doc, &crr_2, &prs, &prs_2);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_ENGINE_EXAMPLE_GetType2Info");
		return -1;
	}
	printf("idc_2 = %d\n", idc_2);
	printf("name_2 = %s\n", name_2);
	printf("surname_2 = %s\n", surname_2);
	printf("Date Of Birth = %d-%d-%d\n", dob.Year, dob.Month, dob.Day);
	printf("Date Of court reporting = %d-%d-%d\n", doc.Year, doc.Month, doc.Day);
	printf("Court reporting reason = %s\n\n", crr_2);
	for (int i = 0; i < prs_2; i ++)
	{
		printf("Date Of court reportin = %d-%d-%d\n", prs[i].dor.Year, prs[i].dor.Month, prs[i].dor.Day);
		printf("Court reporting reason = %s\n\n", prs[i].rsr);
	}

	printf("\n\n");
	///////////////////////////////////////////////////
	// TYPE-4 RECORD
	///////////////////////////////////////////////////
	printf("///////////////////////////////////////////////////\n");
	printf("// CREATE TYPE-4 RECORD\n");
	printf("///////////////////////////////////////////////////\n");
	unsigned char *imgBuff;
	int bufLen, sx, sy;
	unsigned char fgp[6];

	fgp[0] = GB_TABLE_AN2011_8_RIGHT_LITTLE;
	for (int i = 1; i < 6; i ++)
	{
		fgp[i] = 255;
	}

	Retval = GB_AN_ENGINE_EXAMPLE_CreateImageBuffer(&imgBuff, &bufLen, &sx, &sy);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"CreateImageBuffer");
		return Retval;
	}
	Retval = GB_AN_ENGINE_CreateBinaryRecord(
		&dummyRecord,
		4, // rec type
		idc ++, // idc
		GB_TABLE_AN2011_7_FRIT_LS_UNKNOWN_ROLLED, // impression type
		fgp,
		FALSE, // ISR: preferred is minimum, that is a value of 0 (paragraph 8.4.5 of the an2011 standard)
		sx,sy, // HLL and VLL
		GB_TABLE_AN2011_15_CODE_UNCOMPRESSED, // compression algorithm
		imgBuff,bufLen
	);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_ENGINE_CreateBinaryRecord");
		return Retval;
	}

	Retval = GB_AN_ENGINE_InsertRecordInAnsiNist(dummyRecord,ansinist,positionInAnsiNist ++,FALSE);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_ENGINE_InsertRecordInAnsiNist");
		return -1;
	}

	int recordType_4, idc_4, impressionType_4, isr_4, hll_4, vll_4, cga_4, imgBuffLen_4;
	unsigned char fgp_4[6], *imgbuff_4;

	Retval = GB_AN_ENGINE_GetBinaryRecordInfo(
		dummyRecord,&recordType_4, &idc_4,&impressionType_4, fgp_4, &isr_4, & hll_4, &vll_4, &cga_4, 
		&imgbuff_4, &imgBuffLen_4);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN2011_GetType4RecordInfo");
		return -1;
	}
	printf("idc = %d\n", idc_4);
	printf("impressionType = %d\n", impressionType_4);
	printf("isr = %d\n", isr_4);
	printf("hll = %d\n", hll_4);
	printf("vll = %d\n", vll_4);
	printf("cga = %d\n", cga_4);
	printf("imgBuffLen1 = %d\n", imgBuffLen_4);
	for (int i = 0; i < 6; i ++)
	{
		printf("fgp[%d] = %d\n", fgp_4[i], i);
	}
	GB_AN_ENGINE_FreeGenericMem(imgbuff_4);


	printf("\n");
	printf("\n");
	///////////////////////////////////////////////////
	// TYPE-14 RECORD
	///////////////////////////////////////////////////
	printf("///////////////////////////////////////////////////\n");
	printf("// CREATE TYPE-14 RECORD\n");
	printf("///////////////////////////////////////////////////\n");
	GB_AN2011_DateStruct DateOfType_14;
	DateOfType_14.Year = 2013;
	DateOfType_14.Month = 6;
	DateOfType_14.Day = 5;

	GB_AN2011_PrintPositionDescriptorsStruct ppd_14;
	ppd_14.DecimalFingerPosition = GB_TABLE_AN2011_8_RIGHT_MIDDLE;
	sprintf(ppd_14.FingerImageCode,GB_TABLE_AN2011_9_FULL_FINGER_LEFT_SIDE);

	GB_AN2011_PrintPositionCoordinatesStruct ppc_14[2];
	sprintf(ppc_14[0].FullFingerView,GB_TABLE_AN2011_9_FULL_FINGER_LEFT_SIDE);
	sprintf(ppc_14[0].LocationOfASegment,GB_TABLE_AN2011_9_FVC_ONLY_NA);
	ppc_14[0].LeftHorizontalCoordinate = 11;
	ppc_14[0].RightHorizontalCoordinate = 22;
	ppc_14[0].TopVerticalCoordinate = 30;
	ppc_14[0].BottomVerticalCoordinate = 31;

	sprintf(ppc_14[1].FullFingerView,GB_TABLE_AN2011_9_FVC_ONLY_NA);
	sprintf(ppc_14[1].LocationOfASegment,GB_TABLE_AN2011_9_LOS_DISTAL);
	ppc_14[1].LeftHorizontalCoordinate = 11;
	ppc_14[1].RightHorizontalCoordinate = 22;
	ppc_14[1].TopVerticalCoordinate = 30;
	ppc_14[1].BottomVerticalCoordinate = 31;

	GB_AN2011_FingerSegmentPositionStruct seg_14[2];
	seg_14[0].FrictionRidgePosition = GB_TABLE_AN2011_8_LEFT_INDEX;
	seg_14[0].LeftHorizontalCoordinate = 11;
	seg_14[0].RightHorizontalCoordinate = 22;
	seg_14[0].TopVerticalCoordinate = 30;
	seg_14[0].BottomVerticalCoordinate = 31;
	seg_14[1].FrictionRidgePosition = GB_TABLE_AN2011_8_LEFT_MIDDLE;
	seg_14[1].LeftHorizontalCoordinate = 11;
	seg_14[1].RightHorizontalCoordinate = 22;
	seg_14[1].TopVerticalCoordinate = 30;
	seg_14[1].BottomVerticalCoordinate = 31;

	GB_AN2011_NistQualityMetricStruct nqm_14[2];
	nqm_14[0].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_THUMB;
	nqm_14[0].Quality = 3;
	nqm_14[1].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_INDEX;
	nqm_14[1].Quality = 3;

	GB_AN2011_FingerOrSegmentQualityMetricStruct sqm_14[2];
	sqm_14[0].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_THUMB;
	sqm_14[0].QualityValue = 2;
	sqm_14[0].AlgorithmVendorId = 3;
	sqm_14[0].AlgorithmProductId = 4;
	sqm_14[1].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_INDEX;
	sqm_14[1].QualityValue = 12;
	sqm_14[1].AlgorithmVendorId = 13;
	sqm_14[1].AlgorithmProductId = 14;

	GB_AN2011_AlternateSegmentPositionStruct aseg_14[2];
	aseg_14[0].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_THUMB;
	aseg_14[0].NumberOfPoints = 3;
	aseg_14[0].Points = new GB_AN2011_PointOffsetStruct[aseg_14[0].NumberOfPoints];
	for (int i = 0; i < aseg_14[0].NumberOfPoints; i ++)
	{
		aseg_14[0].Points[i].HorizontalPointOffset = i + aseg_14[0].FrictionRidgePosition;
		aseg_14[0].Points[i].VerticalPointOffset = i + aseg_14[0].FrictionRidgePosition;
	}
	aseg_14[1].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_INDEX;
	aseg_14[1].NumberOfPoints = 4;
	aseg_14[1].Points = new GB_AN2011_PointOffsetStruct[aseg_14[1].NumberOfPoints];
	for (int i = 0; i < aseg_14[1].NumberOfPoints; i ++)
	{
		aseg_14[1].Points[i].HorizontalPointOffset = i + aseg_14[1].FrictionRidgePosition;
		aseg_14[1].Points[i].VerticalPointOffset = i + aseg_14[1].FrictionRidgePosition;
	}

	GB_AN2011_AmputatedBandagedCodeStruct amp_14[2];
	amp_14[0].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_THUMB;
	sprintf(amp_14[0].AmpBandCode,"XX");
	amp_14[1].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_INDEX;
	sprintf(amp_14[1].AmpBandCode,"UP");

	Retval = GB_AN2011_CreateType14Record(
		&dummyRecord,//	RECORD **pRecord,
		idc ++,//int idc,
		GB_TABLE_AN2011_7_FRIT_LS_UNKNOWN_PLAIN,//int impressionType,
		"sAgency",//GB_AN_ENGINE_String sourceAgency,
		&DateOfType_14,//GB_AN2011_DateStruct *fingerprintCaptureDate,
		sx,sy,//int horizontalNumberOfPixels,//int verticalNumberOfPixels,
		1,//int scaleUnits,
		500,500,//int transmittedHorizontalPixelScale,//int transmittedVerticalPixelScale,
		GB_TABLE_AN2011_15_CODE_UNCOMPRESSED,8,//int compressionAlgorithm,//int bitsPerPixel,
		GB_TABLE_AN2011_8_EJI_OR_TIP,//int fingerprintPosition,
		&ppd_14, //GB_AN2011_PrintPositionDescriptorsStruct *ppd,
		ppc_14, 2, //GB_AN2011_PrintPositionCoordinatesStruct *ppc, int ppcNum,
		-1,-1,// SHPS, SVPS
		amp_14, 2,//GB_AN2011_AmputatedBandagedCodeStruct *amp, int ampNum,
		"GB_AN_ENGINE_String comment",
		seg_14, 2,//GB_AN2011_FingerSegmentPositionStruct *seg, int segNum,
		nqm_14, 2,//GB_AN2011_NistQualityMetricStruct *nqm, int nqmNum,
		sqm_14, 2,//GB_AN2011_FingerOrSegmentQualityMetricStruct *sqm, int sqmNum,
		sqm_14, 2,//GB_AN2011_FingerOrSegmentQualityMetricStruct *fqm, int fqmNum,
		aseg_14, 2,//GB_AN2011_AlternateSegmentPositionStruct *aseg, int asegNum,
		3, //int simultaneousCaptureFingerprint,
		TRUE, //BOOL stitchedImageFlag,
		GB_TABLE_AN2011_5_DMM_ASSISTED,//GB_AN_ENGINE_String deviceMonitoringMode,
		GB_TABLE_AN2011_12_FAP_40, //int fingerprintAcquisitionProfile,
		imgBuff,bufLen //unsigned char *imageBuffer, int imageBufferLen
	);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN2011_CreateType14Record");
		return -1;
	}

	Retval = GB_AN_ENGINE_InsertRecordInAnsiNist(dummyRecord,ansinist,positionInAnsiNist ++,FALSE);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_Engine_InsertRecordInAnsiNist");
		return -1;
	}

	int idc1_14, imp1_14, hnp1_14, vnp1_14, scl1_14, thps1_14, tvps1_14, cga1_14, bps1_14, fgp1_14, ppcnum1_14,
		shps1_14, svps1_14, ampnum1_14, segnum1_14, nqmnum1_14, sqmnum1_14, fqmnum1_14, asegnum1_14,
		scf1_14, fap1_14, imglen1_14;
	BOOL sif1_14;
	char *sa1_14, *com1_14, *dmm1_14;
	unsigned char *img1_14;
	GB_AN2011_DateStruct fcd1_14;
	GB_AN2011_PrintPositionDescriptorsStruct ppd1_14;
	GB_AN2011_PrintPositionCoordinatesStruct *ppc1_14;
	GB_AN2011_AmputatedBandagedCodeStruct *amp1_14;
	GB_AN2011_FingerSegmentPositionStruct *seg1_14;
	GB_AN2011_NistQualityMetricStruct *nqm1_14;
	GB_AN2011_FingerOrSegmentQualityMetricStruct *sqm1_14, *fqm1_14;
	GB_AN2011_AlternateSegmentPositionStruct *aseg1_14;
	Retval = GB_AN2011_GetType14RecordInfo(
		dummyRecord, //RECORD *recordToRead,
		&idc1_14, //int *pIdc,
		&imp1_14, //int *pImpressionType,
		&sa1_14, //GB_AN_ENGINE_String *pSourceAgency,
		&fcd1_14, //GB_AN2011_DateStruct *fingerprintCaptureDate, // da fare
		&hnp1_14, //int *pHorizontalNumberOfPixels,
		&vnp1_14, //int *pVerticalNumberOfPixels,
		&scl1_14, //int *pScaleUnits,
		&thps1_14, //int *pTransmittedHorizontalPixelScale,
		&tvps1_14, //int *pTransmittedVerticalPixelScale,
		&cga1_14, //int *pCompressionAlgorithm,
		&bps1_14, //int *pBitsPerPixel,
		&fgp1_14, //int *pFingerprintPosition,
		&ppd1_14, //GB_AN2011_PrintPositionDescriptorsStruct *ppd,
		&ppc1_14, &ppcnum1_14,//GB_AN2011_PrintPositionCoordinatesStruct **pPpc, int *pPpcNum,
		&shps1_14,//int *pScannedHorizontalPixelScale,
		&svps1_14,//int *pScannedVerticalPixelScale,
		&amp1_14, &ampnum1_14,//GB_AN2011_AmputatedBandagedCodeStruct **pAmp, int *pAmpNum,
		&com1_14, //GB_AN_ENGINE_String *pComment,
		&seg1_14, &segnum1_14, //GB_AN2011_FingerSegmentPositionStruct **pSeg, int *pSegNum,
		&nqm1_14, &nqmnum1_14,//GB_AN2011_NistQualityMetricStruct **pNqm, int *pNqmNum,
		&sqm1_14, &sqmnum1_14,//GB_AN2011_FingerOrSegmentQualityMetricStruct **pSqm, int *pSqmNum,
		&fqm1_14, &fqmnum1_14,//GB_AN2011_FingerOrSegmentQualityMetricStruct **pFqm, int *pFqmNum,
		&aseg1_14, &asegnum1_14,//GB_AN2011_AlternateSegmentPositionStruct **pAseg, int *pAsegNum,
		&scf1_14,//int *pSimultaneousCaptureFingerprint,
		&sif1_14,//BOOL *pStitchedImageFlag,
		&dmm1_14,//GB_AN_ENGINE_String *pDeviceMonitoringMode,
		&fap1_14,//int *pFingerprintAcquisitionProfile,
		&img1_14, &imglen1_14//unsigned char **pImageBuffer, int *pImageBufferLen
	);

	printf("idc for record type-14 = %d\n", idc1_14);
	printf("ImpressionType for record type-14 = %d\n", imp1_14);
	if (sa1_14 != NULL) printf("SourceAgency for record type-14 = %s\n", sa1_14);
	else printf("SourceAgency for record type-14 NULL\n");
	printf("fingerprintCaptureDate for record type-14 = %d-%d-%d\n", fcd1_14.Year, fcd1_14.Month, fcd1_14.Day);
	printf("HorizontalNumberOfPixels for record type-14 = %d\n", hnp1_14);
	printf("VerticalNumberOfPixels for record type-14 = %d\n", vnp1_14);
	printf("ScaleUnits for record type-14 = %d\n", scl1_14);
	printf("TransmittedHorizontalPixelScale for record type-14 = %d\n", thps1_14);
	printf("TransmittedVerticalPixelScale for record type-14 = %d\n", tvps1_14);
	printf("CompressionAlgorithm for record type-14 = %d\n", cga1_14);
	printf("FingerprintPosition for record type-14 = %d\n", fgp1_14);
	printf("PrintPositionDescriptors for record type-14:\n");
	printf("\tFingerImageCode = %s\n", ppd1_14.FingerImageCode);
	printf("\tDecimalFingerPosition = %d\n", ppd1_14.DecimalFingerPosition);
	if (ppc1_14 != NULL && ppcnum1_14 > 0)
	{
		for (int i = 0; i < ppcnum1_14; i ++)
		{
			printf("PrintPositionCoordinates[%d] for record type-14:\n", i);
			printf("\tFullFingerView = %s\n", ppc1_14[i].FullFingerView);
			printf("\tLocationOfASegment = %s\n", ppc1_14[i].LocationOfASegment);
			printf("\tLeftHorizontalCoordinate = %d\n", ppc1_14[i].LeftHorizontalCoordinate);
			printf("\tRightHorizontalCoordinate = %d\n", ppc1_14[i].RightHorizontalCoordinate);
			printf("\tTopVerticalCoordinate = %d\n", ppc1_14[i].TopVerticalCoordinate);
			printf("\tBottomVerticalCoordinate = %d\n", ppc1_14[i].BottomVerticalCoordinate);
		}
	}
	else printf("PrintPositionCoordinates for record type-14 NULL\n");
	printf("ScannedHorizontalPixelScale for record type-14 = %d\n", shps1_14);
	printf("ScannedVerticalPixelScale for record type-14 = %d\n", svps1_14);
	if (amp1_14 != NULL && ampnum1_14 > 0)
	{
		for (int i = 0; i < ampnum1_14; i ++)
		{
			printf("AmputatedBandagedCode[%d] for record type-14:\n", i);
			printf("\tFrictionRidgePosition = %d\n", amp1_14[i].FrictionRidgePosition);
			printf("\tAmpBandCode = %s\n", amp1_14[i].AmpBandCode);
		}
	}
	else printf("AmputatedBandagedCode for record type-14 NULL\n");
	if (sa1_14 != NULL) printf("Comment for record type-14 = %s\n", com1_14);
	else printf("Comment for record type-14 NULL\n");	
	if (seg1_14 != NULL && segnum1_14 > 0)
	{
		for (int i = 0; i < segnum1_14; i ++)
		{
			printf("FingerSegmentPosition[%d] for record type-14:\n", i);
			printf("\tFrictionRidgePosition = %d\n", seg1_14[i].FrictionRidgePosition);
			printf("\tLeftHorizontalCoordinate = %d\n", seg1_14[i].LeftHorizontalCoordinate);
			printf("\tRightHorizontalCoordinate = %d\n", seg1_14[i].RightHorizontalCoordinate);
			printf("\tTopVerticalCoordinate = %d\n", seg1_14[i].TopVerticalCoordinate);
			printf("\tBottomVerticalCoordinate = %d\n", seg1_14[i].BottomVerticalCoordinate);
		}
	}
	else printf("FingerSegmentPosition for record type-14 NULL\n");	
	if (nqm1_14 != NULL && nqmnum1_14 > 0)
	{
		for (int i = 0; i < nqmnum1_14; i ++)
		{
			printf("NistQualityMetric[%d] for record type-14:\n", i);
			printf("\tFrictionRidgePosition = %d\n", nqm1_14[i].FrictionRidgePosition);
			printf("\tQuality = %d\n", nqm1_14[i].Quality);
		}
	}
	else printf("NistQualityMetric for record type-14 NULL\n");
	if (sqm1_14 != NULL && sqmnum1_14 > 0)
	{
		for (int i = 0; i < sqmnum1_14; i ++)
		{
			printf("SegmentQualityMetric[%d] for record type-14:\n", i);
			printf("\tFrictionRidgePosition = %d\n", sqm1_14[i].FrictionRidgePosition);
			printf("\tQuality = %d\n", sqm1_14[i].QualityValue);
			printf("\tAlgorithmVendorId = %d\n", sqm1_14[i].AlgorithmVendorId);
			printf("\tAlgorithmProductId = %d\n", sqm1_14[i].AlgorithmProductId);
		}
	}
	else printf("SegmentQualityMetric for record type-14 NULL\n");
	if (fqm1_14 != NULL && fqmnum1_14 > 0)
	{
		for (int i = 0; i < fqmnum1_14; i ++)
		{
			printf("FingerQualityMetric[%d] for record type-14:\n", i);
			printf("\tFrictionRidgePosition = %d\n", fqm1_14[i].FrictionRidgePosition);
			printf("\tQuality = %d\n", fqm1_14[i].QualityValue);
			printf("\tAlgorithmVendorId = %d\n", fqm1_14[i].AlgorithmVendorId);
			printf("\tAlgorithmProductId = %d\n", fqm1_14[i].AlgorithmProductId);
		}
	}
	else printf("FingerQualityMetric for record type-14 NULL\n");
	if (aseg1_14 != NULL && asegnum1_14 > 0)
	{
		for (int i = 0; i < asegnum1_14; i ++)
		{
			printf("AlternateSegmentPosition[%d] for record type-14:\n", i);
			printf("\tFrictionRidgePosition = %d\n", aseg1_14[i].FrictionRidgePosition);
			printf("\tNumberOfPoints = %d\n", aseg1_14[i].NumberOfPoints);
			for (int j = 0; j < aseg1_14[i].NumberOfPoints; j ++)
			{
				printf("\t\tPoint[%d].HorizontalPointOffset = %d\n", j, aseg1_14[i].Points[j].HorizontalPointOffset);
				printf("\t\tPoint[%d].VerticalPointOffset = %d\n", j, aseg1_14[i].Points[j].VerticalPointOffset);
			}
		}
	}
	else printf("AlternateSegmentPosition for record type-14 NULL\n");
	printf("SimultaneousCaptureFingerprint for record type-14 = %d\n", scf1_14);	
	if (sif1_14) printf("StitchedImageFlag for record type-14 = TRUE\n");
	else printf("StitchedImageFlag for record type-14 = FALSE\n");
	if (dmm1_14 != NULL) printf("DeviceMonitoringMode for record type-14 = %s\n", dmm1_14);
	else printf("DeviceMonitoringMode for record type-14 NULL\n");
	printf("FingerprintAcquisitionProfile for record type-14 = %d\n", fap1_14);	
	printf("ImageBufferLen for record type-14 = %d\n", imglen1_14);	

	GB_AN_ENGINE_FreeGenericMem(sa1_14);
	GB_AN_ENGINE_FreeGenericMem(ppc1_14);
	GB_AN_ENGINE_FreeGenericMem(amp1_14);
	GB_AN_ENGINE_FreeGenericMem(com1_14);
	GB_AN_ENGINE_FreeGenericMem(seg1_14);
	GB_AN_ENGINE_FreeGenericMem(nqm1_14);
	GB_AN_ENGINE_FreeGenericMem(sqm1_14);
	GB_AN_ENGINE_FreeGenericMem(fqm1_14);
	GB_AN2011_FreeAsegArray(aseg1_14,asegnum1_14);
	GB_AN_ENGINE_FreeGenericMem(dmm1_14);
	GB_AN_ENGINE_FreeGenericMem(img1_14);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN2011_GetType14RecordInfo");
		return -1;
	}
	///////////////////////////////////////////////////
	// TYPE-15 RECORD
	///////////////////////////////////////////////////
	printf("///////////////////////////////////////////////////\n");
	printf("// CREATE TYPE-15 RECORD\n");
	printf("///////////////////////////////////////////////////\n");
	GB_AN2011_DateStruct DateOfType_15;
	DateOfType_15.Year = 2013;
	DateOfType_15.Month = 6;
	DateOfType_15.Day = 5;

	GB_AN2011_FingerOrSegmentQualityMetricStruct pqm_15[2];
	pqm_15[0].FrictionRidgePosition = GB_TABLE_AN2011_8_LEFT_WRITERS_PALM;
	pqm_15[0].QualityValue = 2;
	pqm_15[0].AlgorithmVendorId = 3;
	pqm_15[0].AlgorithmProductId = 4;
	pqm_15[1].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_WRITERS_PALM;
	pqm_15[1].QualityValue = 12;
	pqm_15[1].AlgorithmVendorId = 13;
	pqm_15[1].AlgorithmProductId = 15;

	GB_AN2011_AmputatedBandagedCodeStruct amp_15[2];
	amp_15[0].FrictionRidgePosition = GB_TABLE_AN2011_8_LEFT_UPPER_PALM;
	sprintf(amp_15[0].AmpBandCode,"XX");
	amp_15[1].FrictionRidgePosition = GB_TABLE_AN2011_8_RIGHT_UPPER_PALM;
	sprintf(amp_15[1].AmpBandCode,"UP");

	Retval = GB_AN2011_CreateType15Record(
		&dummyRecord,//	RECORD **pRecord,
		idc ++,//int idc,
		GB_TABLE_AN2011_7_FRIT_LS_UNKNOWN_PALM,//int impressionType,
		"sAgency",//GB_AN_ENGINE_String sourceAgency,
		&DateOfType_15,//GB_AN2011_DateStruct *fingerprintCaptureDate,
		sx,sy,//int horizontalNumberOfPixels,//int verticalNumberOfPixels,
		1,//int scaleUnits,
		500,500,//int transmittedHorizontalPixelScale,//int transmittedVerticalPixelScale,
		GB_TABLE_AN2011_15_CODE_UNCOMPRESSED,8,//int compressionAlgorithm,//int bitsPerPixel,
		GB_TABLE_AN2011_8_RIGHT_FULL_PALM,//int fingerprintPosition,
		-1,-1,// SHPS, SVPS
		amp_15, 2,//GB_AN2011_AmputatedBandagedCodeStruct *amp, int ampNum,
		"GB_AN_ENGINE_String comment",
		pqm_15, 2,//GB_AN2011_FingerOrSegmentQualityMetricStruct *sqm, int sqmNum,
		GB_TABLE_AN2011_5_DMM_ASSISTED,//GB_AN_ENGINE_String deviceMonitoringMode,
		imgBuff,bufLen //unsigned char *imageBuffer, int imageBufferLen
	);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN2011_CreateType15Record");
		return -1;
	}

	Retval = GB_AN_ENGINE_InsertRecordInAnsiNist(dummyRecord,ansinist,positionInAnsiNist ++,FALSE);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN_Engine_InsertRecordInAnsiNist");
		return -1;
	}

	int idc1_15, imp1_15, hnp1_15, vnp1_15, scl1_15, thps1_15, tvps1_15, cga1_15, bps1_15, fgp1_15,
		shps1_15, svps1_15, ampnum1_15, pqmnum1_15, imglen1_15;
	char *sa1_15, *com1_15, *dmm1_15;
	unsigned char *img1_15;
	GB_AN2011_DateStruct fcd1_15;
	GB_AN2011_AmputatedBandagedCodeStruct *amp1_15;
	GB_AN2011_FingerOrSegmentQualityMetricStruct *pqm1_15;

	Retval = GB_AN2011_GetType15RecordInfo(
		dummyRecord, //RECORD *recordToRead,
		&idc1_15, //int *pIdc,
		&imp1_15, //int *pImpressionType,
		&sa1_15, //GB_AN_ENGINE_String *pSourceAgency,
		&fcd1_15, //GB_AN2011_DateStruct *fingerprintCaptureDate, // da fare
		&hnp1_15, //int *pHorizontalNumberOfPixels,
		&vnp1_15, //int *pVerticalNumberOfPixels,
		&scl1_15, //int *pScaleUnits,
		&thps1_15, //int *pTransmittedHorizontalPixelScale,
		&tvps1_15, //int *pTransmittedVerticalPixelScale,
		&cga1_15, //int *pCompressionAlgorithm,
		&bps1_15, //int *pBitsPerPixel,
		&fgp1_15, //int *pFingerprintPosition,
		&shps1_15,//int *pScannedHorizontalPixelScale,
		&svps1_15,//int *pScannedVerticalPixelScale,
		&amp1_15, &ampnum1_15,//GB_AN2011_AmputatedBandagedCodeStruct **pAmp, int *pAmpNum,
		&com1_15, //GB_AN_ENGINE_String *pComment,
		&pqm1_15, &pqmnum1_15,//GB_AN2011_FingerOrSegmentQualityMetricStruct **pSqm, int *pSqmNum,
		&dmm1_15,//GB_AN_ENGINE_String *pDeviceMonitoringMode,
		&img1_15, &imglen1_15//unsigned char **pImageBuffer, int *pImageBufferLen
	);

	printf("idc for record type-15 = %d\n", idc1_15);
	printf("ImpressionType for record type-15 = %d\n", imp1_15);
	if (sa1_15 != NULL) printf("SourceAgency for record type-15 = %s\n", sa1_15);
	else printf("SourceAgency for record type-15 NULL\n");
	printf("fingerprintCaptureDate for record type-15 = %d-%d-%d\n", fcd1_15.Year, fcd1_15.Month, fcd1_15.Day);
	printf("HorizontalNumberOfPixels for record type-15 = %d\n", hnp1_15);
	printf("VerticalNumberOfPixels for record type-15 = %d\n", vnp1_15);
	printf("ScaleUnits for record type-15 = %d\n", scl1_15);
	printf("TransmittedHorizontalPixelScale for record type-15 = %d\n", thps1_15);
	printf("TransmittedVerticalPixelScale for record type-15 = %d\n", tvps1_15);
	printf("CompressionAlgorithm for record type-15 = %d\n", cga1_15);
	printf("FingerprintPosition for record type-15 = %d\n", fgp1_15);
	printf("ScannedHorizontalPixelScale for record type-15 = %d\n", shps1_15);
	printf("ScannedVerticalPixelScale for record type-15 = %d\n", svps1_15);
	if (amp1_15 != NULL && ampnum1_15 > 0)
	{
		for (int i = 0; i < ampnum1_15; i ++)
		{
			printf("AmputatedBandagedCode[%d] for record type-15:\n", i);
			printf("\tFrictionRidgePosition = %d\n", amp1_15[i].FrictionRidgePosition);
			printf("\tAmpBandCode = %s\n", amp1_15[i].AmpBandCode);
		}
	}
	else printf("AmputatedBandagedCode for record type-15 NULL\n");
	if (sa1_15 != NULL) printf("Comment for record type-15 = %s\n", com1_15);
	else printf("Comment for record type-15 NULL\n");
	if (pqm1_15 != NULL && pqmnum1_15 > 0)
	{
		for (int i = 0; i < pqmnum1_15; i ++)
		{
			printf("SegmentQualityMetric[%d] for record type-15:\n", i);
			printf("\tFrictionRidgePosition = %d\n", pqm1_15[i].FrictionRidgePosition);
			printf("\tQuality = %d\n", pqm1_15[i].QualityValue);
			printf("\tAlgorithmVendorId = %d\n", pqm1_15[i].AlgorithmVendorId);
			printf("\tAlgorithmProductId = %d\n", pqm1_15[i].AlgorithmProductId);
		}
	}
	else printf("SegmentQualityMetric for record type-15 NULL\n");
	if (dmm1_15 != NULL) printf("DeviceMonitoringMode for record type-15 = %s\n", dmm1_15);
	else printf("DeviceMonitoringMode for record type-15 NULL\n");
	printf("ImageBufferLen for record type-15 = %d\n", imglen1_15);	

	GB_AN_ENGINE_FreeGenericMem(sa1_15);
	GB_AN_ENGINE_FreeGenericMem(amp1_15);
	GB_AN_ENGINE_FreeGenericMem(com1_15);
	GB_AN_ENGINE_FreeGenericMem(pqm1_15);
	GB_AN_ENGINE_FreeGenericMem(dmm1_15);
	GB_AN_ENGINE_FreeGenericMem(img1_15);

	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"GB_AN2011_GetType15RecordInfo");
		return -1;
	}
	//////////////////////////////////////////////////////////////
	// PRINT ANSI NIST ON FILE
	//////////////////////////////////////////////////////////////
	printf("///////////////////////////////////////////////////\n");
	printf("// PRINT ANSI NIST ON FILE\n");
	printf("///////////////////////////////////////////////////\n");
	unsigned char *outBuffer;
	int anBufLen;
	Retval = write_ANSI_NIST_2_buffer(&outBuffer, &anBufLen, ansinist);
	if (Retval != AN2K_DLL_NO_ERROR)
	{
		ShowError(Retval,"write_ANSI_NIST_2_buffer");
		return -1;
	}
	SaveOnFile("TestAnsinist.an2",outBuffer,anBufLen);

	printf("\n");

	printf("///////////////////////////////////////////////////\n");
	printf("// FREE ANSI NIST\n");
	printf("///////////////////////////////////////////////////\n");
	GB_AN_ENGINE_Free(ansinist,NULL, NULL, NULL, NULL);
	printf("END OF TEST\nPress any key until exit");
	getchar();
	getchar();
	return 0;
}
