﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using GBFIR_NET_WRAPPER;

namespace TestGBFIR_GUI_CS
{
    public partial class Form1 : Form
    {
        public class TestFIR_Globals
        {
            public static string ISOIEC = "ISO-IEC 19794-4";
            public static string ANSI = "ANSI/INCITS 381";
        }

        public void ShowError(string Function, int RetVal)
        {
            string MessageToShow = "Function " + Function + " returned " + RetVal;
            MessageBox.Show(MessageToShow);
            this.MessagesListBox.Items.Add(MessageToShow);
        }

        public void ShowException(string Function, string exMessage)
        {
            string MessageToShow = "Exception in function " + Function + ": " + exMessage;
            MessageBox.Show(MessageToShow);
            this.MessagesListBox.Items.Add(MessageToShow);
        }

        public Form1 ()
        {
        	InitializeComponent ();

            this.StorageOptionsComboBox.Items.Clear ();
        	this.StorageOptionsComboBox.Items.Add (TestFIR_Globals.ISOIEC);
        	this.StorageOptionsComboBox.Items.Add (TestFIR_Globals.ANSI);
        	this.StorageOptionsComboBox.SelectedIndex = 0;
        }
		
		private void LogMessage(String messageToLog)
		{
			this.MessagesListBox.Items.Add(messageToLog);
		}

        private void OpenImageFileButton_Click (object sender, EventArgs e)
        {
        	try
            {
        		OpenFileDialog FD = new OpenFileDialog ();

                FD.ShowDialog ();

                String fn = FD.FileName;
        		if (fn != null && fn != "")
                {
        			this.MessagesListBox.Items.Clear ();
        			/////////////////////////////////////////////
        			// check if it is a .bmp
        			/////////////////////////////////////////////
        			String DirectoryName = Path.GetDirectoryName (FD.FileName);
        			String PureFileName = Path.GetFileNameWithoutExtension (FD.FileName);
        			String Extension = Path.GetExtension (FD.FileName);
     
					IntPtr bpmBuffer = IntPtr.Zero;
        			byte[] TempBuffer = null;
        			byte CompressionAlgorithm = GBFIR_CompressionAlgorithms.GBFIR_CA_UNCOMPRESSED_NO_BIT_PACKING;
        			Bitmap x = new Bitmap (fn);
        			///////////////////////////////////////////////
        			// CHECK PIXEL FORMAT: should be 8 pixel
        			///////////////////////////////////////////////				
        			LogMessage ("PixelFormat: " + x.PixelFormat.ToString ());
        			if (x.PixelFormat != PixelFormat.Format8bppIndexed) {
        				ShowException ("OpenImageFileButton_Click", "ERROR IN LOADING IMAGE, INCORRECT PIXEL FORMAT");
        				return;
        			}
     
					
					if (Extension == ".bmp")
					{
        				///////////////////////////////////////////////
        				// GET IMAGE PIXELS ONLY
        				///////////////////////////////////////////////				
        				BitmapData bmpData = x.LockBits (new Rectangle (0, 0, x.Width, x.Height), ImageLockMode.ReadOnly, PixelFormat.Format8bppIndexed);
        				bpmBuffer = Marshal.AllocHGlobal (x.Width * x.Height);
        				TempBuffer = new byte[x.Height * x.Width];
        				Marshal.Copy (bmpData.Scan0, TempBuffer, 0, TempBuffer.Length);
        				Marshal.Copy (TempBuffer, 0, bpmBuffer, TempBuffer.Length);
        				x.UnlockBits (bmpData);
      
						CompressionAlgorithm = GBFIR_CompressionAlgorithms.GBFIR_CA_UNCOMPRESSED_NO_BIT_PACKING;
        			}
					else 
					{
        				if (Extension != ".jpg" && Extension != ".png")
						{
        					ShowException ("OpenImageFileButton_Click", "ERROR IN LOADING IMAGE, RAW FORMAT NOT SUPPORTED");
        					return;
        				}
      
						///////////////////////////////////////////////
        				// GET WHOLE FILE CONTENTS
        				///////////////////////////////////////////////
        				TempBuffer = File.ReadAllBytes (FD.FileName);
        				bpmBuffer = Marshal.AllocHGlobal (TempBuffer.Length);
        				Marshal.Copy (TempBuffer, 0, bpmBuffer, TempBuffer.Length);
        				if (Extension == ".jpg")
						{
        					CompressionAlgorithm = GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_JPEG;
        				}
						else
						{
							CompressionAlgorithm = GBFIR_CompressionAlgorithms.GBFIR_CA_PNG;
						}
					}                    

                    ///////////////////////////////////////////////
                    // CREATE FIR
                    ///////////////////////////////////////////////				
                    int FirType = (this.StorageOptionsComboBox.SelectedIndex == 0) ? GBFIR_Formats.GBFIR_FORMAT_ISO : GBFIR_Formats.GBFIR_FORMAT_ANSI;
					
					int RetVal = GBFIR_DllWrapper.GBFIR_CreateFIR(
                        FirType,
                        (ushort)31,
                        IntPtr.Zero,
                        (ushort)0,
                        GBFIR_ScaleUnits.GBFIR_SU_PIXEL_PER_INCH,
                        (ushort)500,
                        (ushort)500,
                        (ushort)500,
                        (ushort)500,
                        (byte)8,
                        CompressionAlgorithm
                        );

                    if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                    {
                        ShowError("GBFIR_CreateFIR", RetVal);
                        Marshal.FreeHGlobal(bpmBuffer);
                        return;
                    }
                    else
                    {
                        this.MessagesListBox.Items.Add("FIR created succesfully");
                    }

                    ///////////////////////////////////////////////
                    // ADD IMAGE TO FIR
                    ///////////////////////////////////////////////				
                    RetVal = GBFIR_DllWrapper.GBFIR_AddImageToFIR(
                        bpmBuffer,
                        TempBuffer.Length,
                        x.Width, x.Height,
                        1, 1, 100,
                        GBFIR_FingerPositions.GBFIR_FP_UNKNOWN,
                        GBFIR_ImpressionTypes.GBFIR_IT_LIVE_SCAN_PLAIN);

                    if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                    {
                        ShowError("GBFIR_AddImageToFIR", RetVal);
                        Marshal.FreeHGlobal(bpmBuffer);
                        return;
                    }
                    else
                    {
                        this.MessagesListBox.Items.Add("image added to FIR succesfully");
                    }
                    Marshal.FreeHGlobal(bpmBuffer);

                    ///////////////////////////////////////////////
                    // SAVE FIR ON FILE
                    ///////////////////////////////////////////////	
                    int FIRBuffLen = GBFIR_DllWrapper.GBFIR_GetFIRLen();
                    this.MessagesListBox.Items.Add("FIRBuffLen = " + FIRBuffLen);

                    bpmBuffer = Marshal.AllocHGlobal(FIRBuffLen);

                    RetVal = GBFIR_DllWrapper.GBFIR_WriteFIRToBuffer(bpmBuffer);

                    if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                    {
                        ShowError("GBFIR_WriteFIRToBuffer", RetVal);
                        Marshal.FreeHGlobal(bpmBuffer);
                        return;
                    }
                    else
                    {
                        this.MessagesListBox.Items.Add("FIR written to buffer succesfully");
                    }
                    GBFIR_DllWrapper.GBFIR_FreeFIR();
                    String extension = (this.StorageOptionsComboBox.SelectedIndex == 0) ? ".fir-iso" : ".fir-ansi";

                    String outputFileName = DirectoryName + "/"
                        + PureFileName +
                        extension;

                    TempBuffer = new byte[FIRBuffLen];
                    Marshal.Copy(bpmBuffer, TempBuffer, 0, FIRBuffLen);
                    File.WriteAllBytes(outputFileName, TempBuffer);
                    this.MessagesListBox.Items.Add("FIR saved in " + outputFileName);
                }
                else
                {
                    ShowException("OpenImageFileButton_Click", "Cannot open file");
                }
            }
            catch (Exception ex)
            {
                ShowException("OpenImageFileButton_Click", ex.Message);
            }
        }

        private void OpenFIRButton_Click(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog FD = new OpenFileDialog();

                FD.ShowDialog();

                String fn = FD.FileName;
                if (fn != null && fn != "")
                {
                    String extension = Path.GetExtension(FD.FileName);
                    this.MessagesListBox.Items.Add("extension: " + extension);
                    int FIRType;
                    if (extension == ".fir-iso")
                    {
                        FIRType = GBFIR_Formats.GBFIR_FORMAT_ISO;
                    }
                    else if (extension == ".fir-ansi")
                    {
                        FIRType = GBFIR_Formats.GBFIR_FORMAT_ANSI;
                    }
                    else
                    {
                        ShowException("OpenFIRButton_Click", "file extension not allowed");
                        return;
                    }

                    byte[] TempBuffer = File.ReadAllBytes(FD.FileName);
                    IntPtr FIRBuffer = Marshal.AllocHGlobal(TempBuffer.Length);
                    Marshal.Copy(TempBuffer, 0, FIRBuffer, TempBuffer.Length);
                    int RetVal = GBFIR_DllWrapper.GBFIR_ReadFIRFromBuffer(FIRBuffer, TempBuffer.Length, FIRType);

                    if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                    {
                        ShowError("GBFIR_ReadFIRFromBuffer", RetVal);
                        Marshal.FreeHGlobal(FIRBuffer);
                        return;
                    }
                    else
                    {
                        this.MessagesListBox.Items.Add("FIR read from buffer succesfully");
                    }

                    ushort ImageAcquisitionLevel, CaptureDeviceID,
                        ScanResolutionH, ScanResolutionV,
                        ImageResolutionH, ImageResolutionV;
                    IntPtr CBEFFProductId = Marshal.AllocHGlobal(GBFIR_Constants.CBEFF_PRODUCT_ID_LEN);
                    byte ScaleUnits, PixelDepth, ImageCompressionAlgorithm;

                    RetVal = GBFIR_DllWrapper.GBFIR_GetFIRHeaderFields(
                        out ImageAcquisitionLevel,
                        CBEFFProductId,
                        out CaptureDeviceID,
                        out ScaleUnits,
                        out ScanResolutionH,
                        out ScanResolutionV,
                        out ImageResolutionH,
                        out ImageResolutionV,
                        out PixelDepth,
                        out ImageCompressionAlgorithm);

                    if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                    {
                        ShowError("GBFIR_GetFIRHeaderFields", RetVal);
                        Marshal.FreeHGlobal(FIRBuffer);
                        return;
                    }
                    else
                    {
                        this.MessagesListBox.Items.Add("FIR fields got succesfully");
                    }

                    this.MessagesListBox.Items.Add("FIR ImageAcquisitionLevel: " + ImageAcquisitionLevel);
                    this.MessagesListBox.Items.Add("FIR CaptureDeviceID: " + CaptureDeviceID);
                    this.MessagesListBox.Items.Add("FIR ScaleUnits: " + ScaleUnits);
                    this.MessagesListBox.Items.Add("FIR ScanResolutionH: " + ScanResolutionH);
                    this.MessagesListBox.Items.Add("FIR ScanResolutionV: " + ScanResolutionV);
                    this.MessagesListBox.Items.Add("FIR ImageResolutionH: " + ImageResolutionH);
                    this.MessagesListBox.Items.Add("FIR ImageResolutionV: " + ImageResolutionV);
                    this.MessagesListBox.Items.Add("FIR PixelDepth: " + PixelDepth);
                    this.MessagesListBox.Items.Add("FIR ImageCompressionAlgorithm: " + ImageCompressionAlgorithm);

                    Marshal.FreeHGlobal(FIRBuffer);

                    int NumFingers = GBFIR_DllWrapper.GBFIR_GetNumberOfFingers();
                    int ImageBufferSize;
                    IntPtr ImageBuffer;
                    int ImageWidth, ImageHeight, ImageQuality, FingerPosition, ImpressionType;
                    byte CountView, ViewNumber;
                    
                    this.MessagesListBox.Items.Add("Number of records = " + (NumFingers));
                    
                    for (int i=0; i<NumFingers; i++)
                    {
                        this.MessagesListBox.Items.Add("### Record " + (i));
                        
                        RetVal = GBFIR_DllWrapper.GBFIR_GetFingerRecordHeaderFields(i, 
			                out CountView,
			                out ViewNumber,
			                out ImageQuality,
			                out FingerPosition,
			                out ImpressionType);
        

                        if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                        {
                            ShowError("GBFIR_GetFingerRecordHeaderFields", RetVal);
                            Marshal.FreeHGlobal(FIRBuffer);
                            return;
                        }

		                this.MessagesListBox.Items.Add("    Count View = " + (CountView));
		                this.MessagesListBox.Items.Add("    View Number = " + (ViewNumber));
		                this.MessagesListBox.Items.Add("    Image Quality = " + (ImageQuality));
		                this.MessagesListBox.Items.Add("    FingerPosition = " + (FingerPosition));
		                this.MessagesListBox.Items.Add("    ImpressionType = " + (ImpressionType));

                        RetVal = GBFIR_DllWrapper.GBFIR_GetFingerImageSize(i, out ImageBufferSize);
                        if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                        {
                            ShowError("GBFIR_GetFingerImageSize", RetVal);
                            Marshal.FreeHGlobal(FIRBuffer);
                            return;
                        }

                        ImageBuffer = Marshal.AllocHGlobal(ImageBufferSize);
                        RetVal = GBFIR_DllWrapper.GBFIR_GetFingerImage(i, ImageBuffer, out ImageWidth, out ImageHeight);
                        if (RetVal != GBFIR_ReturnCodes.GBFIR_RET_SUCCESS)
                        {
                            ShowError("GBFIR_GetFingerImage", RetVal);
                            Marshal.FreeHGlobal(FIRBuffer);
                            return;
                        }

		                this.MessagesListBox.Items.Add("    Image Width = " + (ImageWidth));
		                this.MessagesListBox.Items.Add("    Image Height = " + (ImageHeight));

                        
                        String outFName = Path.GetDirectoryName(FD.FileName) + '/' + 
                                Path.GetFileNameWithoutExtension(FD.FileName) + '_' + i;

                        if (ImageCompressionAlgorithm == GBFIR_CompressionAlgorithms.GBFIR_CA_UNCOMPRESSED_NO_BIT_PACKING)
                        {
                            Bitmap x = new Bitmap(ImageWidth, ImageHeight, ImageWidth, PixelFormat.Format8bppIndexed, ImageBuffer);
                            ColorPalette pal = x.Palette;
                            for (int j = 0; j < pal.Entries.Length; j++)
                                pal.Entries[j] = Color.FromArgb(j, j, j);
                            x.Palette = pal;
                            outFName += ".bmp";
                            x.Save(outFName, ImageFormat.Bmp);

                            this.MessagesListBox.Items.Add("    Image file saved: " + outFName);
                        }
                        else
                        {
                            if (ImageCompressionAlgorithm == GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_JPEG)
                            {
                                outFName += ".jpg";
                            }
                            else if (ImageCompressionAlgorithm == GBFIR_CompressionAlgorithms.GBFIR_CA_PNG)
                            {
                                outFName += ".png";
                            }
                            else if (ImageCompressionAlgorithm == GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_WSQ)
                            {
                                outFName += ".wsq";
                            }
                            else if (ImageCompressionAlgorithm == GBFIR_CompressionAlgorithms.GBFIR_CA_COMPRESSED_JPEG2000)
                            {
                                outFName += ".j2k";
                            }

                            TempBuffer = new byte[ImageBufferSize];
                            Marshal.Copy(ImageBuffer, TempBuffer, 0, TempBuffer.Length);

                            File.WriteAllBytes(outFName, TempBuffer);

                            this.MessagesListBox.Items.Add("    Image file saved: " + outFName);
                        }
                        Marshal.FreeHGlobal(ImageBuffer);
                    }
                    
                    GBFIR_DllWrapper.GBFIR_FreeFIR();
                }
                else
                {
                    ShowException("OpenFIRButton_Click", "Cannot open file");
                }
            }
            catch (Exception ex)
            {
                ShowException("OpenFIRButton_Click", ex.Message);
            }
        }
    }
}
