#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "GBMSAPI_Library.h"
#include "WSQPack.h"
#include "GBJpeg.h"

#include <QDebug>
#include <QSettings>
#include <QDesktopWidget>
#include <QStyle>

// defines for file format
#define FF_BITMAP       1
#define FF_JPEG         2
#define FF_JPEG2000     3
#define FF_WSQ          4

#define DEFAULT_SESSION_OPTIONS         GBMSGUI_SESSION_OPTION_SEQUENCE_CHECK
#define DEFAULT_ACQUISITION_OPTIONS     (GBMSGUI_ACQUISITION_OPTION_SEGMENTATION | \
                                        GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE | \
                                        GBMSGUI_ACQUISITION_OPTION_SOUND | \
                                        GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT | \
                                        GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN | \
                                        GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN | \
                                        GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION | \
                                        GBMSGUI_ACQUISITION_OPTION_ROTATE_FINGER | \
                                        GBMSGUI_ACQUISITION_OPTION_SOUND_ON_ROLL_PREVIEW_END)

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // center window
    setGeometry(QStyle::alignedRect(Qt::LeftToRight,Qt::AlignCenter,
            size(), qApp->desktop()->availableGeometry()));

    ui->frameFrameRate->setEnabled(false);

    // menu actions
    //connect (ui->action_Exit, SIGNAL(triggered()), this, SLOT(OnExit()));

	connect(ui->optFROFullRes, SIGNAL(clicked()), this, SLOT(OnFROptionChange()));
	connect(ui->optFROPreview, SIGNAL(clicked()), this, SLOT(OnFROptionChange()));
	connect(ui->chkFROAdaptiveRoll, SIGNAL(clicked()), this, SLOT(OnFROptionChange()));
	connect(ui->chkFROHiResPreview, SIGNAL(clicked()), this, SLOT(OnFROptionChange()));
	connect(ui->lstScanArea, SIGNAL(currentRowChanged(int)), this, SLOT(on_lstScanArea_changed(int)));

	//connect(ui->chkRollAreaGA, SIGNAL(clicked()), this, SLOT(OnRollAreaChange()));

	connect(ui->chkAdaptRollAreaPosition, SIGNAL(clicked()), this, SLOT(OnAdapRollAreaPositionChange()));
	connect(ui->chkEnableBlockComposition, SIGNAL(clicked()), this, SLOT(OnEnableBlockCompositionChange()));
	connect(ui->chkBlockAutocaptureLedColorFeedback, SIGNAL(clicked()), this, SLOT(OnEnableBlockAutocaptureLedColorFeedback()));

    int i;
    QListWidgetItem *Item;
    for (i=0; i<ui->lstUnavailableFingersLeft->count(); i++)
    {
        Item = ui->lstUnavailableFingersLeft->item(i);
        Item->setFlags(Item->flags() | Qt::ItemIsUserCheckable);
        Item->setCheckState(Qt::Unchecked);
        Item->setSizeHint(QSize(0, 14));
    }

    for (i=0; i<ui->lstUnavailableFingersRight->count(); i++)
    {
        Item = ui->lstUnavailableFingersRight->item(i);
        Item->setFlags(Item->flags() | Qt::ItemIsUserCheckable);
        Item->setCheckState(Qt::Unchecked);
        Item->setSizeHint(QSize(0, 14));
    }

    for (i=0; i<ui->lstUnavailableFingersThumbs->count(); i++)
    {
        Item = ui->lstUnavailableFingersThumbs->item(i);
        Item->setFlags(Item->flags() | Qt::ItemIsUserCheckable);
        Item->setCheckState(Qt::Unchecked);
        Item->setSizeHint(QSize(0, 14));
    }

    ImageBuffer = NULL;

    GBMSGUI_Load();

    int V1, V2, V3, V4; 
    GBMSGUI_GetVersion(&V1, &V2, &V3, &V4);
    //qDebug() << "GBMSGUI Version " << V1 << "." << V2 << "." << V3 << "." << V4;

    ImagesPath.setPath("./Images");
    if (!ImagesPath.exists())
    {
        QDir().mkdir(ImagesPath.path());
    }

#ifndef WIN32
    int ret;
    ret = GBMSAPI_LoadLibrary();
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_LoadLibrary");
        return;
    }
#endif

    // read INI file
    ReadIni();

	memset (ItemSettings, 0, MAX_ITEM_INDEX*sizeof(ItemSetting));

    LoadDevicesList();

    GrayTable.resize(256);
    for( int i = 0; i < 256; i++ )
    {
       GrayTable[i] = qRgb( i, i, i );
    }

    // add items to Quality Algorithm combo
    ui->cmbQualityAlgorithm->addItem("NFIQ", GBMSGUI_NFIQ_ALGORITHM);
    ui->cmbQualityAlgorithm->addItem("Green Bit", GBMSGUI_GB_ALGORITHM);

    // add items to Output format combo
    ui->cmbOutputFormat->addItem("Bitmap", FF_BITMAP);
    ui->cmbOutputFormat->addItem("Jpeg", FF_JPEG);
    ui->cmbOutputFormat->addItem("Jpeg 2000", FF_JPEG2000);
    ui->cmbOutputFormat->addItem("WSQ", FF_WSQ);

    // settings from INI file
    // session options
    ui->chkSequenceCheck->setChecked(CheckMask(SessionOptions, GBMSGUI_SESSION_OPTION_SEQUENCE_CHECK));
    ui->chkAskUnavailabilityReason->setChecked(CheckMask(SessionOptions, GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON));
    ui->chkNoAdvancedView->setChecked(CheckMask(SessionOptions, GBMSGUI_SESSION_OPTION_NO_ADVANCED_VIEW));
    ui->chkDryFingerImageEnhancement->setChecked(CheckMask(SessionOptions, GBMSGUI_SESSION_OPTION_DRY_FINGER_IMAGE_ENHANCEMENT));

    // acquisition options
    ui->chkSegmentation->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_SEGMENTATION));
    ui->chkAutocapture->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE));
    ui->chkBlockAutocapture->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE));
    ui->chkSound->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_SOUND));
    ui->chkAutoAccept->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_AUTOACCEPT));
    ui->chkRemoveHalo->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT));
    ui->chkDetectInvalidPattern->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN));
    ui->chkDetectIncompletePattern->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN));
    ui->chkDetectInclination->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION));
    ui->chkRotate->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ROTATE_FINGER));
    ui->chkFullResolutionPreview->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW));
    ui->chkSoundOnRollPreviewEnd->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_SOUND_ON_ROLL_PREVIEW_END));
    ui->chkPalmPrintQualityCalculation->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION));
    ui->chkNoArtefacts->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_NO_ARTEFACTS_DISPLAY));
    ui->chkHighSpeedPreview->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_HIGH_SPEED_PREVIEW));
    ui->chkAutoClearOutsideRoll->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_AUTO_CLEAR_OUTSIDE_ROLL));
	ui->chkSegmentsQualityEval->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_LIVE_SEGM_EVAL));
	ui->chkIncompleteSlapNaming->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_AUTO_INCOMPLETE_SLAP_SEGM));
	ui->chkAdaptRollAreaPosition->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ADAPT_ROLL_AREA_POSITION));
	//ui->chkEnableRollStripeAcquisition->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_ACQUISITION));
	// Stripe acquisition not yet certified
    AcquisitionOptions &= ~GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_ACQUISITION;
	ui->chkEnableRollStripeAcquisition->setChecked(false);
	ui->chkEnableRollStripeAcquisition->setVisible(false);
	//ui->chkEnableRollStripeFeedback->setChecked(CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_FEEDBACK));
    if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_NO_ROLL_PREVIEW))
        ui->optNoRollPreview->setChecked(true);
    else if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_MANUAL_STOP))
        ui->optRollPreviewManual->setChecked(true);
    else if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_TYPE))
        ui->optRollPreviewCenter->setChecked(true);
    else
        ui->optRollPreviewSide->setChecked(true);

    ui->txtRollArtefactThreshold1->setText(QString().setNum(RollArtefactThreshold1));
    ui->txtRollArtefactThreshold2->setText(QString().setNum(RollArtefactThreshold2));
    ui->txtInclinationThreshold->setText(QString().setNum(InclinationThreshold));
    ui->txtPalmInclinationThreshold->setText(QString().setNum(PalmInclinationThreshold));
    ui->cmbQualityAlgorithm->setCurrentIndex(ui->cmbQualityAlgorithm->findData(QualityAlgorithm));
	ui->txtLiveSegmQualityEvalTimeout->setText(QString().setNum(LiveSegmQualityEvalTimeout));

    ui->grpAuto->setChecked(true);
    ui->grpManual->setChecked(false);

    ui->cmbLanguage->addItem("English", "en");
    ui->cmbLanguage->addItem("Italiano", "it");
    ui->cmbLanguage->addItem("Chinese", "zh");
	connect(ui->cmbLanguage, SIGNAL(currentIndexChanged(int)), this, SLOT(OnLanguageChange(int)));
	ui->cmbLanguage->setCurrentIndex(0);

    // add items to Finger contact evaluation mode combo
    ui->cmbFingerContactEvaluationMode->addItem("Dry/Wet warning", GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING);
    ui->cmbFingerContactEvaluationMode->addItem("Contrast", GBMSGUI_FINGER_CONTACT_EVAL_MODE_CONTRAST);

	ui->txtSegmentsWidth->setText("500");
	ui->txtSegmentsHeight->setText("500");

	ui->optRollToCenter->setChecked(true);

	// To be defined - for now disabled
	//ui->chkSegmentsQualityEval->setVisible(false);
}

MainWindow::~MainWindow()
{
    if (ImageBuffer)
    {
        free(ImageBuffer);
        ImageBuffer = NULL;
    }

    WriteIni();

    GrayTable.clear();
#ifndef WIN32
    GBMSAPI_UnloadLibrary();
#endif
    GBMSGUI_Unload();    
    delete ui;
}

bool __stdcall MainWindow::AfterStartCallback(void *Param)    // STATIC
{
    // e.g. use param to keep trace of current ObjectID
    unsigned int ObjID = *(unsigned int *)Param;

    //qDebug() << "AfterStartCallback called; ObjId = " << ObjID;

    // for example
    // roll setting
    if (GBMSGUI_IsRolled(ObjID))
    {
        int ret = GBMSAPI_ROLL_SetArtefactCleaningDepth(GBMSAPI_ARTEFACT_CLEANING_DEPTH_ZERO);

        if (ret == GBMSAPI_ERROR_CODE_NO_ERROR)
            return true;
        else
            return false;
    }

    return true;
}

void MainWindow::DisplayGBMSAPIError(int ErrorCode, QString FunctionName)
{
    QString Title = QApplication::instance()->applicationName();
    QString ErrMsg;
    unsigned long UsbErr;

    switch (ErrorCode)
    {
        case GBMSAPI_ERROR_CODE_USB_DRIVER:
            GBMSAPI_GetUSBErrorCode(&UsbErr);
            if (UsbErr == 0xE0001005)
            {
                ErrMsg = "Driver is busy. Another device currently uses USB device driver";
            }
            else
                ErrMsg = QString("USB Driver error %1").arg(UsbErr, 8, 16);
            break;
        case GBMSAPI_ERROR_CODE_DEVICE_NOT_FOUND:
            ErrMsg = "Device not found";
            break;
        case GBMSAPI_ERROR_CODE_USB_THREAD:
            ErrMsg = "USB Thread";
            break;
        case GBMSAPI_ERROR_CODE_PARAMETER:
            ErrMsg = "Parameter error";
            break;
        case GBMSAPI_ERROR_CODE_SCANNER_NOT_CONFIGURED:
            ErrMsg = "Scanner not configured";
            break;
        case GBMSAPI_ERROR_CODE_DEVICE_NOT_RESPONDING:
            ErrMsg = "Device not responding";
            break;
        case GBMSAPI_ERROR_CODE_SCANNER_COMMUNICATION:
            ErrMsg = "Scanner communication error";
            break;
        case GBMSAPI_ERROR_CODE_UNAVAILABLE_OPTION:
            ErrMsg = "Unavailable option";
            break;
        case GBMSAPI_ERROR_CODE_INTERNAL:
            ErrMsg = "Internal error";
            break;
        case GBMSAPI_ERROR_CODE_SPECIFIC_DLL_NOT_LOADED:
            ErrMsg = "Specific Dll not loaded";
            break;
        case GBMSAPI_ERROR_CODE_METHOD_NOT_SUPPORTED:
            ErrMsg = "Method not supported";
            break;
        case GBMSAPI_ERROR_CODE_OBJECT_TYPE_NOT_SUPPORTED:
            ErrMsg = "Object type not supported";
            break;
        case GBMSAPI_ERROR_CODE_SCAN_AREA_NOT_SUPPORTED:
            ErrMsg = "Scan area not supported";
            break;
        case GBMSAPI_ERROR_CODE_ACQUISITION_THREAD:
            ErrMsg = "Acquisition thread error";
            break;
        case GBMSAPI_ERROR_CODE_ACQUISITION_ALREADY_STARTED:
            ErrMsg = "Acquisition already started";
            break;
        case GBMSAPI_ERROR_CODE_FEATURE_NOT_SUPPORTED:
            ErrMsg = "Feature not supported";
            break;
        case GBMSAPI_ERROR_CODE_CURRENT_DEV_NOT_SET:
            ErrMsg = "Current device not set";
            break;
        case GBMSAPI_ERROR_CODE_MEMORY_ALLOCATION:
            ErrMsg = "Memory allocation error";
            break;
        case GBMSAPI_ERROR_CODE_GENERIC:
            ErrMsg = "Generic error";
            break;
    }

    QString Message = QString("%1 (%2)").arg(ErrMsg).arg(FunctionName);

    QMessageBox::critical(this, Title, Message, QMessageBox::Ok);
}

void MainWindow::DisplayGBMSGUIError(QString FunctionName)
{
    QString Title = QApplication::instance()->applicationName();
    const wchar_t *pMsg = GBMSGUI_GetErrorMessage();
    QString ErrMsg = QString::fromWCharArray(pMsg);
    int ErrorCode = GBMSGUI_GetErrorCode();
    QString Message = QString("%1 (%2)").arg(ErrMsg).arg(FunctionName);

    QMessageBox::critical(this, Title, Message, QMessageBox::Ok);
}

void MainWindow::LoadDevicesList()
{
    int ret;
    unsigned long USBErrorCode;

    ui->cmbDevices->clear();

    ret = GBMSAPI_GetAttachedDeviceList(DeviceList, &DevicesNumber, &USBErrorCode);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_GetAttachedDeviceList");
        return;
    }

    for (int i=0; i<DevicesNumber; i++)
    {
        ui->cmbDevices->addItem(QString("%1 (%2)").arg(GetDeviceName(DeviceList[i].DeviceID)).arg(DeviceList[i].DeviceSerialNumber));
    }
}

void MainWindow::LoadItemsList()
{
    unsigned int ScannableTypes;

    GBMSAPI_GetScannableTypes(&ScannableTypes);

    ui->lstItems->clear();

    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_SLAP_4_LEFT)))
        AddItemToList("FLAT_FOUR_FINGERS_LEFT", GBMSAPI_SO_SLAP_4_LEFT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_SLAP_4_RIGHT)))
        AddItemToList("FLAT_FOUR_FINGERS_RIGHT", GBMSAPI_SO_SLAP_4_RIGHT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_SLAP_2_THUMBS)))
        AddItemToList("FLAT_TWO_THUMBS", GBMSAPI_SO_SLAP_2_THUMBS);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_SLAP_2_LEFT)))
        AddItemToList("FLAT_TWO_FINGERS_LEFT", GBMSAPI_SO_SLAP_2_LEFT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_SLAP_2_RIGHT)))
        AddItemToList("FLAT_TWO_FINGERS_RIGHT", GBMSAPI_SO_SLAP_2_RIGHT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_SLAP_2_INDEXES)))
        AddItemToList("FLAT_TWO_INDEXES", GBMSAPI_SO_SLAP_2_INDEXES);

    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_UPPER_HALF_PALM_LEFT)))
        AddItemToList("UPPER_HALF_PALM_LEFT", GBMSAPI_SO_UPPER_HALF_PALM_LEFT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_UPPER_HALF_PALM_RIGHT)))
        AddItemToList("UPPER_HALF_PALM_RIGHT", GBMSAPI_SO_UPPER_HALF_PALM_RIGHT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_LOWER_HALF_PALM_LEFT)))
        AddItemToList("LOWER_HALF_PALM_LEFT", GBMSAPI_SO_LOWER_HALF_PALM_LEFT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_LOWER_HALF_PALM_RIGHT)))
        AddItemToList("LOWER_HALF_PALM_RIGHT", GBMSAPI_SO_LOWER_HALF_PALM_RIGHT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_WRITER_PALM_LEFT)))
        AddItemToList("WRITER_PALM_LEFT", GBMSAPI_SO_WRITER_PALM_LEFT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_WRITER_PALM_RIGHT)))
        AddItemToList("WRITER_PALM_RIGHT", GBMSAPI_SO_WRITER_PALM_RIGHT);

    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_LEFT_THUMB)))
        AddItemToList("ROLLED_THUMB_LEFT", GBMSAPI_SO_ROLL_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_LEFT_INDEX)))
        AddItemToList("ROLLED_INDEX_LEFT", GBMSAPI_SO_ROLL_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_LEFT_MIDDLE)))
        AddItemToList("ROLLED_MIDDLE_LEFT", GBMSAPI_SO_ROLL_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_LEFT_RING)))
        AddItemToList("ROLLED_RING_LEFT", GBMSAPI_SO_ROLL_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_LEFT_LITTLE)))
        AddItemToList("ROLLED_LITTLE_LEFT", GBMSAPI_SO_ROLL_LEFT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_RIGHT_THUMB)))
        AddItemToList("ROLLED_THUMB_RIGHT", GBMSAPI_SO_ROLL_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_RIGHT_INDEX)))
        AddItemToList("ROLLED_INDEX_RIGHT", GBMSAPI_SO_ROLL_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_RIGHT_MIDDLE)))
        AddItemToList("ROLLED_MIDDLE_RIGHT", GBMSAPI_SO_ROLL_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_RIGHT_RING)))
        AddItemToList("ROLLED_RING_RIGHT", GBMSAPI_SO_ROLL_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLL_RIGHT_LITTLE)))
        AddItemToList("ROLLED_LITTLE_RIGHT", GBMSAPI_SO_ROLL_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_LEFT_THUMB)))
        AddItemToList("FLAT_THUMB_LEFT", GBMSAPI_SO_FLAT_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_LEFT_INDEX)))
        AddItemToList("FLAT_INDEX_LEFT", GBMSAPI_SO_FLAT_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_LEFT_MIDDLE)))
        AddItemToList("FLAT_MIDDLE_LEFT", GBMSAPI_SO_FLAT_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_LEFT_RING)))
        AddItemToList("FLAT_RING_LEFT", GBMSAPI_SO_FLAT_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_LEFT_LITTLE)))
        AddItemToList("FLAT_LITTLE_LEFT", GBMSAPI_SO_FLAT_LEFT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_RIGHT_THUMB)))
        AddItemToList("FLAT_THUMB_RIGHT", GBMSAPI_SO_FLAT_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_RIGHT_INDEX)))
        AddItemToList("FLAT_INDEX_RIGHT", GBMSAPI_SO_FLAT_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_RIGHT_MIDDLE)))
        AddItemToList("FLAT_MIDDLE_RIGHT", GBMSAPI_SO_FLAT_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_RIGHT_RING)))
        AddItemToList("FLAT_RING_RIGHT", GBMSAPI_SO_FLAT_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_FLAT_RIGHT_LITTLE)))
        AddItemToList("FLAT_LITTLE_RIGHT", GBMSAPI_SO_FLAT_RIGHT_LITTLE);
	// MS527
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_LEFT_THUMB)))
        AddItemToList("ROLLED_JOINT_LEFT_THUMB", GBMSAPI_SO_ROLLED_JOINT_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_LEFT_INDEX)))
        AddItemToList("ROLLED_JOINT_LEFT_INDEX", GBMSAPI_SO_ROLLED_JOINT_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_LEFT_MIDDLE)))
        AddItemToList("ROLLED_JOINT_LEFT_MIDDLE", GBMSAPI_SO_ROLLED_JOINT_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_LEFT_RING)))
        AddItemToList("ROLLED_JOINT_LEFT_RING", GBMSAPI_SO_ROLLED_JOINT_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_LEFT_LITTLE)))
        AddItemToList("ROLLED_JOINT_LEFT_LITTLE", GBMSAPI_SO_ROLLED_JOINT_LEFT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_RIGHT_THUMB)))
        AddItemToList("ROLLED_JOINT_RIGHT_THUMB", GBMSAPI_SO_ROLLED_JOINT_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_RIGHT_INDEX)))
        AddItemToList("ROLLED_JOINT_RIGHT_INDEX", GBMSAPI_SO_ROLLED_JOINT_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_RIGHT_MIDDLE)))
        AddItemToList("ROLLED_JOINT_RIGHT_MIDDLE", GBMSAPI_SO_ROLLED_JOINT_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_RIGHT_RING)))
        AddItemToList("ROLLED_JOINT_RIGHT_RING", GBMSAPI_SO_ROLLED_JOINT_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_RIGHT_LITTLE)))
        AddItemToList("ROLLED_JOINT_RIGHT_LITTLE", GBMSAPI_SO_ROLLED_JOINT_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_THUMB)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_LEFT_THUMB", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_INDEX)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_LEFT_INDEX", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_MIDDLE)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_LEFT_MIDDLE", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_RING)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_LEFT_RING", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_LITTLE)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_LEFT_LITTLE", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_THUMB)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_RIGHT_THUMB", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_INDEX)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_RIGHT_INDEX", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_MIDDLE)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_RIGHT_MIDDLE", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_RING)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_RIGHT_RING", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_LITTLE)))
        AddItemToList("PLAIN_JOINT_LEFT_SIDE_RIGHT_LITTLE", GBMSAPI_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_THUMB)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_LEFT_THUMB", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_INDEX)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_LEFT_INDEX", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_MIDDLE)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_LEFT_MIDDLE", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_RING)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_LEFT_RING", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_LITTLE)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_LEFT_LITTLE", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_THUMB)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_RIGHT_THUMB", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_INDEX)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_RIGHT_INDEX", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_MIDDLE)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_RIGHT_MIDDLE", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_RING)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_RIGHT_RING", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_LITTLE)))
        AddItemToList("PLAIN_JOINT_RIGHT_SIDE_RIGHT_LITTLE", GBMSAPI_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_THUMB)))
        AddItemToList("ROLLED_JOINT_CENTER_LEFT_THUMB", GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_INDEX)))
        AddItemToList("ROLLED_JOINT_CENTER_LEFT_INDEX", GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_MIDDLE)))
        AddItemToList("ROLLED_JOINT_CENTER_LEFT_MIDDLE", GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_RING)))
        AddItemToList("ROLLED_JOINT_CENTER_LEFT_RING", GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_LITTLE)))
        AddItemToList("ROLLED_JOINT_CENTER_LEFT_LITTLE", GBMSAPI_SO_ROLLED_JOINT_CENTER_LEFT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_THUMB)))
        AddItemToList("ROLLED_JOINT_CENTER_RIGHT_THUMB", GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_INDEX)))
        AddItemToList("ROLLED_JOINT_CENTER_RIGHT_INDEX", GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_MIDDLE)))
        AddItemToList("ROLLED_JOINT_CENTER_RIGHT_MIDDLE", GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_RING)))
        AddItemToList("ROLLED_JOINT_CENTER_RIGHT_RING", GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_LITTLE)))
        AddItemToList("ROLLED_JOINT_CENTER_RIGHT_LITTLE", GBMSAPI_SO_ROLLED_JOINT_CENTER_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_LEFT_THUMB)))
        AddItemToList("ROLLED_TIP_LEFT_THUMB", GBMSAPI_SO_ROLLED_TIP_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_LEFT_INDEX)))
        AddItemToList("ROLLED_TIP_LEFT_INDEX", GBMSAPI_SO_ROLLED_TIP_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_LEFT_MIDDLE)))
        AddItemToList("ROLLED_TIP_LEFT_MIDDLE", GBMSAPI_SO_ROLLED_TIP_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_LEFT_RING)))
        AddItemToList("ROLLED_TIP_LEFT_RING", GBMSAPI_SO_ROLLED_TIP_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_LEFT_LITTLE)))
        AddItemToList("ROLLED_TIP_LEFT_LITTLE", GBMSAPI_SO_ROLLED_TIP_LEFT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_RIGHT_THUMB)))
        AddItemToList("ROLLED_TIP_RIGHT_THUMB", GBMSAPI_SO_ROLLED_TIP_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_RIGHT_INDEX)))
        AddItemToList("ROLLED_TIP_RIGHT_INDEX", GBMSAPI_SO_ROLLED_TIP_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_RIGHT_MIDDLE)))
        AddItemToList("ROLLED_TIP_RIGHT_MIDDLE", GBMSAPI_SO_ROLLED_TIP_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_RIGHT_RING)))
        AddItemToList("ROLLED_TIP_RIGHT_RING", GBMSAPI_SO_ROLLED_TIP_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_TIP_RIGHT_LITTLE)))
        AddItemToList("ROLLED_TIP_RIGHT_LITTLE", GBMSAPI_SO_ROLLED_TIP_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_THENAR_LEFT)))
        AddItemToList("ROLLED_THENAR_LEFT", GBMSAPI_SO_ROLLED_THENAR_LEFT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_THENAR_RIGHT)))
        AddItemToList("ROLLED_THENAR_RIGHT", GBMSAPI_SO_ROLLED_THENAR_RIGHT);
	// 1.5.0.0 - Hypotenars
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_HYPOTHENAR_LEFT)))
        AddItemToList("ROLLED_HYPOTHENAR_LEFT", GBMSAPI_SO_ROLLED_HYPOTHENAR_LEFT);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_HYPOTHENAR_RIGHT)))
        AddItemToList("ROLLED_HYPOTHENAR_RIGHT", GBMSAPI_SO_ROLLED_HYPOTHENAR_RIGHT);
	// MS527 - end

	// 1.4.0.0 - moved palms up (for sequence check)

	// MC517
	// Roll up
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_RIGHT_THUMB)))
        AddItemToList("ROLLED_UP_RIGHT_THUMB", GBMSAPI_SO_ROLLED_UP_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_RIGHT_INDEX)))
        AddItemToList("ROLLED_UP_RIGHT_INDEX", GBMSAPI_SO_ROLLED_UP_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_RIGHT_MIDDLE)))
        AddItemToList("ROLLED_UP_RIGHT_MIDDLE", GBMSAPI_SO_ROLLED_UP_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_RIGHT_RING)))
        AddItemToList("ROLLED_UP_RIGHT_RING", GBMSAPI_SO_ROLLED_UP_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_RIGHT_LITTLE)))
        AddItemToList("ROLLED_UP_RIGHT_LITTLE", GBMSAPI_SO_ROLLED_UP_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_LEFT_THUMB)))
        AddItemToList("ROLLED_UP_LEFT_THUMB", GBMSAPI_SO_ROLLED_UP_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_LEFT_INDEX)))
        AddItemToList("ROLLED_UP_LEFT_INDEX", GBMSAPI_SO_ROLLED_UP_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_LEFT_MIDDLE)))
        AddItemToList("ROLLED_UP_LEFT_MIDDLE", GBMSAPI_SO_ROLLED_UP_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_LEFT_RING)))
        AddItemToList("ROLLED_UP_LEFT_RING", GBMSAPI_SO_ROLLED_UP_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_UP_LEFT_LITTLE)))
        AddItemToList("ROLLED_UP_LEFT_LITTLE", GBMSAPI_SO_ROLLED_UP_LEFT_LITTLE);
	// Roll down
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_RIGHT_THUMB)))
        AddItemToList("ROLLED_DOWN_RIGHT_THUMB", GBMSAPI_SO_ROLLED_DOWN_RIGHT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_RIGHT_INDEX)))
        AddItemToList("ROLLED_DOWN_RIGHT_INDEX", GBMSAPI_SO_ROLLED_DOWN_RIGHT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_RIGHT_MIDDLE)))
        AddItemToList("ROLLED_DOWN_RIGHT_MIDDLE", GBMSAPI_SO_ROLLED_DOWN_RIGHT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_RIGHT_RING)))
        AddItemToList("ROLLED_DOWN_RIGHT_RING", GBMSAPI_SO_ROLLED_DOWN_RIGHT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_RIGHT_LITTLE)))
        AddItemToList("ROLLED_DOWN_RIGHT_LITTLE", GBMSAPI_SO_ROLLED_DOWN_RIGHT_LITTLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_LEFT_THUMB)))
        AddItemToList("ROLLED_DOWN_LEFT_THUMB", GBMSAPI_SO_ROLLED_DOWN_LEFT_THUMB);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_LEFT_INDEX)))
        AddItemToList("ROLLED_DOWN_LEFT_INDEX", GBMSAPI_SO_ROLLED_DOWN_LEFT_INDEX);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_LEFT_MIDDLE)))
        AddItemToList("ROLLED_DOWN_LEFT_MIDDLE", GBMSAPI_SO_ROLLED_DOWN_LEFT_MIDDLE);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_LEFT_RING)))
        AddItemToList("ROLLED_DOWN_LEFT_RING", GBMSAPI_SO_ROLLED_DOWN_LEFT_RING);
    if (CheckMask(ScannableTypes, GBMSAPI_GetTypeFromObject(GBMSAPI_SO_ROLLED_DOWN_LEFT_LITTLE)))
        AddItemToList("ROLLED_DOWN_LEFT_LITTLE", GBMSAPI_SO_ROLLED_DOWN_LEFT_LITTLE);
}

void MainWindow::AddItemToList(QString Name, int Value)
{
    QListWidgetItem *Item;

    Item = new QListWidgetItem(Name);
    Item->setData(Qt::UserRole, Value);
    Item->setFlags(Item->flags() | Qt::ItemIsUserCheckable);
    Item->setCheckState(Qt::Unchecked);
    ui->lstItems->addItem(Item);

	// set item setting
	ItemSettings[Value].ImageWidth = DefaultImageWidth(Value);
	ItemSettings[Value].ImageHeight = DefaultImageHeight(Value);
	ItemSettings[Value].QualityTh1 = DefaultQualityThreshold1(Value, QualityAlgorithm);
	ItemSettings[Value].QualityTh2 = DefaultQualityThreshold2(Value, QualityAlgorithm);
	ItemSettings[Value].ContrastTh1 = DefaultContrastThreshold1(Value);
	ItemSettings[Value].ContrastTh2 = DefaultContrastThreshold2(Value);
}

void MainWindow::on_cmbDevices_currentIndexChanged(int index)
{
    int ret;

    if (index == -1)
        return;

    ret = GBMSAPI_SetCurrentDevice(DeviceList[index].DeviceID,
                    DeviceList[index].DeviceSerialNumber);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_SetCurrentDevice");
        return;
    }

    ret = GBMSAPI_GetMaxImageSize(&MaxImageSizeX, &MaxImageSizeY);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_GetMaxImageSize");
        return;
    }

	// fill scan area list
	LoadScanAreaList();

    unsigned int DeviceFeatures;
    GBMSAPI_GetDeviceFeatures(&DeviceFeatures);
    if (CheckMask(DeviceFeatures, GBMSAPI_DF_FRAME_RATE_SETTING))
    {
        ui->frameFrameRate->setEnabled(true);
        OnFROptionChange();
    }
    else
        ui->frameFrameRate->setEnabled(false);

    // MC517 supports both, MC500 GA only, everything else IQS only
	if (CheckMask(DeviceFeatures, GBMSAPI_DF_ROLL_AREA_GA))
	{
		if (!CheckMask(DeviceFeatures, GBMSAPI_DF_ROLL_AREA_IQS))
		{
			ui->chkRollAreaGA->setChecked(true);
			ui->chkRollAreaGA->setEnabled(false);
		}
		else
			ui->chkRollAreaGA->setEnabled(true);
	}
	else
	{
		ui->chkRollAreaGA->setChecked(false);
		ui->chkRollAreaGA->setEnabled(false);
    }

	if (CheckMask(DeviceFeatures, GBMSAPI_DF_ENABLE_BLOCK_ROLL_COMPOSITION))
		ui->chkEnableBlockComposition->setEnabled(true);
	else
		ui->chkEnableBlockComposition->setEnabled(false);

    LoadItemsList();
}

void MainWindow::SetSessionOptions()
{
    SessionOptions = 0;

    if (ui->chkSequenceCheck->isChecked())
        // enable sequence check
        SessionOptions |= GBMSGUI_SESSION_OPTION_SEQUENCE_CHECK;
    if (ui->chkAskUnavailabilityReason->isChecked())
        // enable sequence check
        SessionOptions |= GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON;
    if (ui->chkNoAdvancedView->isChecked())
        SessionOptions |= GBMSGUI_SESSION_OPTION_NO_ADVANCED_VIEW;
    if (ui->chkDryFingerImageEnhancement->isChecked())
		SessionOptions |= GBMSGUI_SESSION_OPTION_DRY_FINGER_IMAGE_ENHANCEMENT;
}

void MainWindow::SetAcquisitionOptions()
{
    AcquisitionOptions = 0;

    if (ui->chkSegmentation->isChecked())
        // enable segmentation
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_SEGMENTATION;
    if (ui->chkAutocapture->isChecked())
        // enable segmentation
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE;
    if (ui->chkSound->isChecked())
        // enable segmentation
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_SOUND;
    if (ui->chkAutoAccept->isChecked())
        // enable auto-accept
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_AUTOACCEPT;
    if (ui->chkBlockAutocapture->isChecked())
        // enable block autocapture
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE;
    if (ui->chkRemoveHalo->isChecked())
        // enable "Halo effect" removal
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT;
    if (ui->chkDetectInvalidPattern->isChecked())
        // enable invalid pattern detection
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN;
    if (ui->chkDetectIncompletePattern->isChecked())
        // enable incomplete pattern detection
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN;
    if (ui->chkDetectInclination->isChecked())
        // enable inclination detection
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION;
    if (ui->chkRotate->isChecked())
        // enable rotation
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_ROTATE_FINGER;
    if (ui->chkFullResolutionPreview->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW;
    if (ui->chkSoundOnRollPreviewEnd->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_SOUND_ON_ROLL_PREVIEW_END;
    if (ui->chkPalmPrintQualityCalculation->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION;
    if (ui->chkNoArtefacts->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_NO_ARTEFACTS_DISPLAY;
    if (ui->chkHighSpeedPreview->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_HIGH_SPEED_PREVIEW;
    if (ui->chkAutoClearOutsideRoll->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_AUTO_CLEAR_OUTSIDE_ROLL;
	if (ui->chkSegmentsQualityEval->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_LIVE_SEGM_EVAL;
	if (ui->chkIncompleteSlapNaming->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_AUTO_INCOMPLETE_SLAP_SEGM;
	if (ui->chkAdaptRollAreaPosition->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_ADAPT_ROLL_AREA_POSITION;
	if (ui->chkEnableRollStripeAcquisition->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_ACQUISITION;
	//if (ui->chkEnableRollStripeFeedback->isChecked())
    //    AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_FEEDBACK;
	if (ui->chkDeletePalmFingerSegments->isChecked())
		AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_DELETE_PALM_FINGER_SEGMENTS;

    // Roll preview
    if (ui->optNoRollPreview->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_NO_ROLL_PREVIEW;

    if (ui->optRollPreviewManual->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_MANUAL_STOP;

    if (ui->optRollPreviewCenter->isChecked())
        AcquisitionOptions |= GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_TYPE;

    GBMSGUI_SetFlatFingerOnRollArea(ui->chkFlatFingerOnRollArea->isChecked());
}

bool MainWindow::StartSession(int AcquisitionMode)
{
    int ret;

    EmptyImagesFolder();

    SetSessionOptions();

    GBMSGUI_SetWindowSizeAndPosition(AcqWndLeft, AcqWndTop, AcqWndWidth, AcqWndHeight, AcqWndMaximized);

    ImageBufferSize = MaxImageSizeX * MaxImageSizeY;
    // allocate buffer for the image
    if (ImageBuffer)
        free(ImageBuffer);
    ImageBuffer = (unsigned char *)calloc(ImageBufferSize, sizeof(unsigned char));
    if (ImageBuffer == NULL)
    {
        QMessageBox::critical(this, "TestGBMSGUI", "Not enough memory", QMessageBox::Ok);
        return false;
    }

    int Len = ui->lblPersonID->text().length();
    wchar_t *PersonID = (wchar_t *)calloc(Len + 1, sizeof(wchar_t));
    ui->lblPersonID->text().toWCharArray(PersonID);
    PersonID[Len] = 0;

    ret = GBMSGUI_StartSession(AcquisitionMode, PersonID, SessionOptions);
    free(PersonID);
    PersonID = NULL;
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("GBMSGUI_StartSession");
        return false;
    }

    return true;
}

int MainWindow::Acquire(int ObjectID, QString ObjectName)
{
    int ret;
    int ImageSizeX, ImageSizeY, Dpi;

    SetAcquisitionOptions();
    SetRollArtefactsThresholds();
    SetInclinationThreshold();
	SetPalmInclinationThreshold();
	SetLiveSegmEvalTimeout();

	CurrentObjectID = ObjectID;
    GBMSGUI_SetAfterStartCallback(AfterStartCallback, (void *)&CurrentObjectID);

    if (GBMSGUI_IsRolled(ObjectID))
    {
		if (ui->chkRollAreaGA->isChecked())
			ret = GBMSGUI_SetRollArea(GBMSGUI_ROLL_AREA_GA);
		else
			ret = GBMSGUI_SetRollArea(GBMSGUI_ROLL_AREA_IQS);
		if (ret != GBMSGUI_RET_SUCCESS)
		{
			return ret;
		}
    }

    // for slaps
	// 1.4.0.0 - and upper palms
    if (GBMSGUI_IsSlap(ObjectID) || GBMSGUI_IsUpperPalm(ObjectID))
    {
		ret = SetSegmentsImageSize(ObjectID);
        if (ret == GBMSGUI_RET_FAILURE)
            return ret;

        ret = SetSegmentsQualityThresholds(ObjectID);
        if (ret == GBMSGUI_RET_FAILURE)
            return ret;

        //set eventual missing fingers
        ret = SetMissingFingers(ObjectID);
        if (ret == GBMSGUI_RET_FAILURE)
            return ret;
    }

    // call set functions
	ret = GBMSGUI_SetImageSize(ObjectID, ItemSettings[ObjectID].ImageWidth, ItemSettings[ObjectID].ImageHeight);
    if (ret != GBMSGUI_RET_SUCCESS)
    {
        return ret;
    }

    ret = GBMSGUI_SetQualityThresholds(ObjectID, ItemSettings[ObjectID].QualityTh1, ItemSettings[ObjectID].QualityTh2);
    if (ret != GBMSGUI_RET_SUCCESS)
    {
        return ret;
    }

    ret = GBMSGUI_SetContrastThresholds(ObjectID, ItemSettings[ObjectID].ContrastTh1, ItemSettings[ObjectID].ContrastTh2);
    if (ret != GBMSGUI_RET_SUCCESS)
    {
        return ret;
    }

	GBMSGUI_SelectFingerContactEvaluationMode(ui->cmbFingerContactEvaluationMode->itemData(ui->cmbFingerContactEvaluationMode->currentIndex()).toInt());

    if (CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_ADAPT_ROLL_AREA_POSITION))
	{
		int RollDirection;
		if (ui->optRollToLeft->isChecked())
			RollDirection = GBMSGUI_ADAPTIVE_ROLL_DIRECTION_TO_LEFT;
		else if (ui->optRollToRight->isChecked())
			RollDirection = GBMSGUI_ADAPTIVE_ROLL_DIRECTION_TO_RIGHT;
		else
			RollDirection = GBMSGUI_ADAPTIVE_ROLL_DIRECTION_TO_CENTER;

		GBMSGUI_SetAdaptiveRollDirection(RollDirection);
	}

    ret = GBMSGUI_Acquire(ObjectID, AcquisitionOptions, ImageBuffer, ImageBufferSize, &ImageSizeX, &ImageSizeY, &Dpi);
    if (ret != GBMSGUI_RET_SUCCESS)
    {
        return ret;
    }

    // save image to file
    //QString FileName = ImagesPath.path() + "/" + ObjectName;
    QString FileName = ImagesPath.path() + "/";
	if (ui->grpFileName->isChecked())
		FileName += ui->txtFileName->text();
	else
		FileName += ObjectName;

    SaveImage(FileName, ImageBuffer, ImageSizeX, ImageSizeY);

    // for Slaps, save segments
	// 1.4.0.0 - and upper palms
    if ((GBMSGUI_IsSlap(ObjectID) || GBMSGUI_IsUpperPalm(ObjectID)) && 
		CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
        SaveSegments(ObjectID, ObjectName);

	// 1.3.0.0
	// for Joints, get segment bounding boxes
    if (GBMSGUI_IsJoint(ObjectID) && CheckMask(AcquisitionOptions, GBMSGUI_ACQUISITION_OPTION_SEGMENTATION))
		GetJointBoundingBoxes(ObjectID);

    return ret;
}

void MainWindow::StopSession()
{
    int ret;

    if (ImageBuffer)
    {
        free(ImageBuffer);
        ImageBuffer = NULL;
    }

    ret = GBMSGUI_StopSession();
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("GBMSGUI_StopSession");
        return;
    }

    GBMSGUI_GetWindowSizeAndPosition(&AcqWndLeft, &AcqWndTop, &AcqWndWidth, &AcqWndHeight, &AcqWndMaximized);
}

void MainWindow::on_btnSequence_clicked()
{
    int ObjectID;
    QString ObjectName;
    int ret;
    QListWidgetItem *pItem;

    if (!StartSession(GBMSGUI_AM_MULTIPLE_ACQUISITION))
        return;

    for (int i=0; i<ui->lstItems->count(); i++)
    {
        pItem = ui->lstItems->item(i);
        if (pItem->checkState() == Qt::Unchecked)
            continue;

        ObjectID = pItem->data(Qt::UserRole).toInt();
        ObjectName = pItem->text();

        ret = Acquire(ObjectID, ObjectName);
        if (ret == GBMSGUI_RET_FAILURE)
        {
            DisplayGBMSGUIError("Acquire");
            break;
        }
        else if (ret == GBMSGUI_RET_BREAK_SEQUENCE)
        {
            break;
        }
        else if (ret == GBMSGUI_RET_SKIP_IMAGE)
        {
            continue;
        }
        else if (ret == GBMSGUI_RET_GO_BACK)
        {
            if (i > 0)
            {
                i--;
                i--;
                continue;
            }
            else
                break;
        }

    }

    StopSession();
}

void MainWindow::SaveToBmp(QString FileName, unsigned char *Buffer, int SizeX, int SizeY)
{
    QImage *ImgFrame = new QImage(SizeX, SizeY, QImage::Format_Indexed8);
	unsigned char *pImage = Buffer;
	for (int y=0; y<SizeY; y++, pImage += SizeX)
		memcpy(ImgFrame->scanLine(y), pImage, SizeX);

    // set gray palette
    ImgFrame->setColorTable(GrayTable);
	// set DPI
	int dpm = 500 / 0.0254;
	ImgFrame->setDotsPerMeterX(dpm);
	ImgFrame->setDotsPerMeterY(dpm);

    ImgFrame->save(FileName, "BMP");
    delete ImgFrame;
}

void MainWindow::on_btnSelectAll_clicked()
{
    for (int i=0; i<ui->lstItems->count(); i++)
        ui->lstItems->item(i)->setCheckState(Qt::Checked);
}

void MainWindow::on_btnUnselectAll_clicked()
{
    for (int i=0; i<ui->lstItems->count(); i++)
        ui->lstItems->item(i)->setCheckState(Qt::Unchecked);
}

void MainWindow::on_btnStartSession_clicked()
{
    if (!StartSession(GBMSGUI_AM_SINGLE_ACQUISITION))
        return;

    ui->btnStartSession->setEnabled(false);
    ui->btnAcquireItem->setEnabled(true);
    ui->btnEndSession->setEnabled(true);
    ui->frameObjectSettings->setEnabled(true);
}

void MainWindow::on_btnAcquireItem_clicked()
{
    int ObjectID;
    QString ObjectName;
    int ret;

    QListWidgetItem *pItem = ui->lstItems->currentItem();
    if (!pItem)
    {
        QMessageBox::warning(this, "Acquire", "Select an item from the list first!", QMessageBox::Ok);
        return;
    }

    SetAcquisitionOptions();

    ObjectID = pItem->data(Qt::UserRole).toInt();
    ObjectName = pItem->text();

    ret = Acquire(ObjectID, ObjectName);
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("Acquire");
    }
}

void MainWindow::on_btnEndSession_clicked()
{
    StopSession();

    ui->btnStartSession->setEnabled(true);
    ui->btnAcquireItem->setEnabled(false);
    ui->btnEndSession->setEnabled(false);
    ui->frameObjectSettings->setEnabled(false);
}


void MainWindow::on_grpAuto_toggled(bool arg1)
{
    ui->grpManual->setChecked(!arg1);
}

void MainWindow::on_grpManual_toggled(bool arg1)
{
    ui->grpAuto->setChecked(!arg1);

    if (arg1)
    {
        ui->btnStartSession->setEnabled(true);
        ui->btnAcquireItem->setEnabled(false);
        ui->btnEndSession->setEnabled(false);
    }
}

void MainWindow::on_btnRefresh_clicked()
{
    setCursor(Qt::WaitCursor);
    LoadDevicesList();
    setCursor(Qt::ArrowCursor);
}

QString MainWindow::GetDeviceName(unsigned char DeviceID)
{
    QString DeviceName;

    switch (DeviceID)
    {
        case GBMSAPI_DN_DS40:
            DeviceName = "DactyScan40";
            break;
        case GBMSAPI_DN_DS84:
            DeviceName = "DactyScan84";
            break;
        case GBMSAPI_DN_MS1000:
            DeviceName = "MultiScan1000";
            break;
        case GBMSAPI_DN_MS500:
            DeviceName = "MultiScan500";
            break;
        case GBMSAPI_DN_PS2:
            DeviceName = "PoliScan2";
            break;
        case GBMSAPI_DN_VS3:
            DeviceName = "Visascan3";
            break;
        case GBMSAPI_DN_DS26:
            DeviceName = "DactyScan26";
            break;
        case GBMSAPI_DN_MC500:
            DeviceName = "MC500";
            break;
        case GBMSAPI_DN_MSC500:
            DeviceName = "MSC500";
            break;
        case GBMSAPI_DN_DS40I:
            DeviceName = "DactyScan40i";
            break;
		case GBMSAPI_DN_DS84C:
            DeviceName = "DactyScan84C";
            break;
        case GBMSAPI_DN_MC517:
            DeviceName = "MC517";
            break;
        case GBMSAPI_DN_MSC517:
            DeviceName = "MSC517";
            break;
		case GBMSAPI_DN_DS32:
            DeviceName = "DactyScan32";
            break;
		case GBMSAPI_DN_MS527:
            DeviceName = "MS527";
            break;
        default:
            DeviceName = "";
            break;
    }

    return DeviceName;
}

bool MainWindow::SaveSegments(int ObjectID, QString ObjectName)
{
    int i, num_fingers;
    int ret;
    bool SegmentAvailable;
    int Left, Top, Right, Bottom;
    int UnavailabilityReason;
    int FingerprintQuality;
    unsigned char *SegmImagePtr;
    int SegmImageSize;
    int ImageWidth, ImageHeight;
    int Dpi;
    QString SegmFileName;
    int QualityAlgorithmID = GBMSGUI_NFIQ_ALGORITHM;

	// 1.4.0.0 - also upper palms
    if ((ObjectID == GBMSAPI_SO_SLAP_4_LEFT) || (ObjectID == GBMSAPI_SO_SLAP_4_RIGHT) ||
		(ObjectID == GBMSAPI_SO_UPPER_HALF_PALM_LEFT) || (ObjectID == GBMSAPI_SO_UPPER_HALF_PALM_RIGHT))
        num_fingers = 4;
    else
        num_fingers = 2;

    // allocate only one buffer for image, maximum size
    SegmImageSize = ImageBufferSize;
    SegmImagePtr = (unsigned char *)malloc(SegmImageSize);
    if (SegmImagePtr == NULL)
    {
        QMessageBox::critical(this, "Error", "Insufficient memory", QMessageBox::Ok);
        return false;
    }

    for (i=0; i<num_fingers; i++)
    {
        ret = GBMSGUI_GetSegmentationResult(i,      // index of finger
            QualityAlgorithmID,                     // quality algorithm
            &SegmentAvailable,                      // segment availability flag
            &Left, &Top, &Right, &Bottom,           // bounding box coordinates
            &UnavailabilityReason,                  // reason of finger unavailability (if so)
            &FingerprintQuality,                    // fingerprint quality
            SegmImagePtr,                               // pointer to image
            SegmImageSize,                        //
            &ImageWidth,                            // image size x
            &ImageHeight,                           // image size y
            &Dpi);                                  // resolution
        if (ret == GBMSGUI_RET_FAILURE)
        {
            DisplayGBMSGUIError("GBMSGUI_GetSegmentationResult");
        }
        else
        {
            // if the finger is available, save the image
            if (SegmentAvailable)
            {
                // build the file name
				if (ui->grpFileName->isChecked())
					SegmFileName = ImagesPath.path() + "/" + QString("%1_%2").arg(ui->txtFileName->text()).arg(i);
				else
	                SegmFileName = ImagesPath.path() + "/" + QString("%1_%2").arg(ObjectName).arg(i);
                // save the image
                SaveImage(SegmFileName, SegmImagePtr, ImageWidth, ImageHeight);
            }
        }
    }

    free (SegmImagePtr);

    return true;
}

// return the flat finger id corresponding to an index of a slap
int MainWindow::SlapIdIndex2ObjectID(int SlapID, int Index)
{
    switch (SlapID)
    {
        case GBMSAPI_SO_SLAP_4_LEFT:
        case GBMSAPI_SO_UPPER_HALF_PALM_LEFT:	// 1.4.0.0
            switch (Index)
            {
                case 0:
                    return GBMSAPI_SO_FLAT_LEFT_INDEX;
                case 1:
                    return GBMSAPI_SO_FLAT_LEFT_MIDDLE;
                case 2:
                    return GBMSAPI_SO_FLAT_LEFT_RING;
                case 3:
                    return GBMSAPI_SO_FLAT_LEFT_LITTLE;
            }
            break;

        case GBMSAPI_SO_SLAP_4_RIGHT:
        case GBMSAPI_SO_UPPER_HALF_PALM_RIGHT:	// 1.4.0.0
            switch (Index)
            {
                case 0:
                    return GBMSAPI_SO_FLAT_RIGHT_INDEX;
                case 1:
                    return GBMSAPI_SO_FLAT_RIGHT_MIDDLE;
                case 2:
                    return GBMSAPI_SO_FLAT_RIGHT_RING;
                case 3:
                    return GBMSAPI_SO_FLAT_RIGHT_LITTLE;
            }
            break;

        case GBMSAPI_SO_SLAP_2_THUMBS:
            switch (Index)
            {
                case 0:
                    return GBMSAPI_SO_FLAT_LEFT_THUMB;
                case 1:
                    return GBMSAPI_SO_FLAT_RIGHT_THUMB;
            }
            break;

        case GBMSAPI_SO_SLAP_2_LEFT:
            switch (Index)
            {
                case 0:
                    return GBMSAPI_SO_FLAT_LEFT_MIDDLE;
                case 1:
                    return GBMSAPI_SO_FLAT_LEFT_INDEX;
            }
            break;
        case GBMSAPI_SO_SLAP_2_RIGHT:
            switch (Index)
            {
                case 0:
                    return GBMSAPI_SO_FLAT_RIGHT_INDEX;
                case 1:
                    return GBMSAPI_SO_FLAT_RIGHT_MIDDLE;
            }
            break;
        case GBMSAPI_SO_SLAP_2_INDEXES:
            switch (Index)
            {
                case 0:
                    return GBMSAPI_SO_FLAT_LEFT_INDEX;
                case 1:
                    return GBMSAPI_SO_FLAT_RIGHT_INDEX;
            }
            break;
    }

    return 0;
}

void MainWindow::OnExit()   // this is a slot
{
    close();
}


void MainWindow::on_lstItems_currentItemChanged(QListWidgetItem *current, QListWidgetItem *previous)
{
    if (current)
    {
        ui->lblSelectedObject->setText(current->text());
        int ObjectID = current->data(Qt::UserRole).toInt();
        ui->txtWidth->setText(QString().setNum(ItemSettings[ObjectID].ImageWidth));
        ui->txtHeight->setText(QString().setNum(ItemSettings[ObjectID].ImageHeight));
		ui->txtQualityTh1->setText(QString().setNum(ItemSettings[ObjectID].QualityTh1));
        ui->txtQualityTh2->setText(QString().setNum(ItemSettings[ObjectID].QualityTh2));
		ui->txtContrastTh1->setText(QString().setNum(ItemSettings[ObjectID].ContrastTh1));
        ui->txtContrastTh2->setText(QString().setNum(ItemSettings[ObjectID].ContrastTh2));

		CurrentObjectID = ObjectID;
		ShowDefaultFrameRate();
    }
    else
    {
        ui->lblSelectedObject->setText("(no object selected)");
        ui->txtWidth->setText("");
        ui->txtHeight->setText("");
        ui->txtQualityTh1->setText("");
        ui->txtQualityTh2->setText("");
        ui->txtContrastTh1->setText("");
        ui->txtContrastTh2->setText("");

		CurrentObjectID = 0;
		ui->txtFrameRate->setText("");
		ui->lblFrameRateRange->setText("");
    }
}

void MainWindow::on_btnSetObjectSettings_clicked()
{
    QListWidgetItem *pItem = ui->lstItems->currentItem();
    if (!pItem)
    {
        QMessageBox::warning(this, "Set Image Size", "Select an item from the list first!", QMessageBox::Ok);
        return;
    }

    int ObjectID = pItem->data(Qt::UserRole).toInt();
    int ImgWidth, ImgHeight, QualityTh1, QualityTh2, ContrastTh1, ContrastTh2;

    // check values
    bool ok;
    ImgWidth = ui->txtWidth->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Image Size", "Invalid value!", QMessageBox::Ok);
        ui->txtWidth->setFocus();
        return;
    }

    ImgHeight = ui->txtHeight->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Image Size", "Invalid value!", QMessageBox::Ok);
        ui->txtHeight->setFocus();
        return;
    }

    QualityTh1 = ui->txtQualityTh1->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Quality threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtQualityTh1->setFocus();
        return;
    }

    QualityTh2 = ui->txtQualityTh2->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Quality threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtQualityTh2->setFocus();
        return;
    }

    ContrastTh1 = ui->txtContrastTh1->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Contrast threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtContrastTh1->setFocus();
        return;
    }

    ContrastTh2 = ui->txtContrastTh2->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Contrast threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtContrastTh2->setFocus();
        return;
    }

	// save values
	ItemSettings[ObjectID].ImageWidth = ImgWidth;
	ItemSettings[ObjectID].ImageHeight = ImgHeight;
	ItemSettings[ObjectID].QualityTh1 = QualityTh1;
	ItemSettings[ObjectID].QualityTh2 = QualityTh2;
	ItemSettings[ObjectID].ContrastTh1 = ContrastTh1;
	ItemSettings[ObjectID].ContrastTh2 = ContrastTh2;

    QMessageBox::information(this, "Set Image Size", "Set succeded", QMessageBox::Ok);
}

void MainWindow::ReadIni()
{
    QSettings Configuration("./TestGBMSGUI.ini", QSettings::IniFormat);

    SessionOptions = Configuration.value("Settings/SessionOptions", DEFAULT_SESSION_OPTIONS).toUInt();
    AcquisitionOptions = Configuration.value("Settings/AcquisitionOptions", DEFAULT_ACQUISITION_OPTIONS).toUInt();
    QualityAlgorithm = Configuration.value("Settings/QualityAlgorithm", GBMSGUI_NFIQ_ALGORITHM).toInt();
    RollArtefactThreshold1 = Configuration.value("Settings/RollArtefactThreshold1", 7).toInt();
    RollArtefactThreshold2 = Configuration.value("Settings/RollArtefactThreshold2", 14).toInt();
    InclinationThreshold = Configuration.value("Settings/InclinationThreshold", 35).toInt();
	PalmInclinationThreshold = Configuration.value("Settings/PalmInclinationThreshold", 15).toInt();
	LiveSegmQualityEvalTimeout = Configuration.value("Settings/LiveSegmQualityEvalTimeout", 5).toInt();
    AcqWndLeft = Configuration.value("AcquisitionWindow/Left", 0).toInt();
    AcqWndTop = Configuration.value("AcquisitionWindow/Top", 0).toInt();
    AcqWndWidth = Configuration.value("AcquisitionWindow/Width", 1024).toInt();
    AcqWndHeight = Configuration.value("AcquisitionWindow/Height", 768).toInt();
    AcqWndMaximized = Configuration.value("AcquisitionWindow/Maximized", false).toBool();
}

void MainWindow::WriteIni()
{
    QSettings Configuration("./TestGBMSGUI.ini", QSettings::IniFormat);

    Configuration.setValue("Settings/SessionOptions", SessionOptions);
    Configuration.setValue("Settings/AcquisitionOptions", AcquisitionOptions);
    Configuration.setValue("Settings/QualityAlgorithm", QualityAlgorithm);
    Configuration.setValue("Settings/RollArtefactThreshold1", RollArtefactThreshold1);
    Configuration.setValue("Settings/RollArtefactThreshold2", RollArtefactThreshold2);
    Configuration.setValue("Settings/InclinationThreshold", InclinationThreshold);
    Configuration.setValue("Settings/PalmInclinationThreshold", PalmInclinationThreshold);
	Configuration.setValue("Settings/LiveSegmQualityEvalTimeout", LiveSegmQualityEvalTimeout);
    Configuration.setValue("AcquisitionWindow/Left", AcqWndLeft);
    Configuration.setValue("AcquisitionWindow/Top", AcqWndTop);
    Configuration.setValue("AcquisitionWindow/Width", AcqWndWidth);
    Configuration.setValue("AcquisitionWindow/Height", AcqWndHeight);
    Configuration.setValue("AcquisitionWindow/Maximized", AcqWndMaximized);
}

int MainWindow::DefaultImageWidth(int ObjectID)
{
    int Width;

    if (GBMSGUI_IsFlatSingleFinger(ObjectID))
        Width = 500;
	else if (GBMSGUI_IsJoint(ObjectID))		// 1.3.0.0
		Width = 800;
	else if (GBMSGUI_IsRolledThenar(ObjectID)		// 1.3.0.0
		|| GBMSGUI_IsRolledHypothenar(ObjectID))	// 1.8.0.0
		Width = 1500;
    else if (GBMSGUI_IsRolled(ObjectID))
    {
		if (ui->chkRollAreaGA->isChecked())
	        Width = 640;
		else
			Width = 800;
        if (Width > MaxImageSizeX)
            Width = MaxImageSizeX;
    }
    else
        Width = MaxImageSizeX;

    return Width;
}

int MainWindow::DefaultImageHeight(int ObjectID)
{
    int Height;

    if (GBMSGUI_IsFlatSingleFinger(ObjectID))
        Height = 500;
	else if (GBMSGUI_IsJoint(ObjectID))		// 1.3.0.0
		Height = 2500;
	else if (GBMSGUI_IsRolledThenar(ObjectID)		// 1.3.0.0
		|| GBMSGUI_IsRolledHypothenar(ObjectID))	// 1.8.0.0
		Height = 2250;
	else if (GBMSGUI_IsRolledTip(ObjectID))		// 1.3.0.0
		Height = 500;
    else if (GBMSGUI_IsRolled(ObjectID))
    {
		if (ui->chkRollAreaGA->isChecked())
	        Height = 640;
		else
	        Height = 750;
        if (Height > MaxImageSizeY)
            Height = MaxImageSizeY;
    }
    else
        Height = MaxImageSizeY;

    return Height;
}

int MainWindow::DefaultQualityThreshold1(int ObjectID, int AlgorithmID)
{
    if (AlgorithmID == GBMSGUI_NFIQ_ALGORITHM)
        return 2;
    else
        return 50;
}

int MainWindow::DefaultQualityThreshold2(int ObjectID, int AlgorithmID)
{
    if (AlgorithmID == GBMSGUI_NFIQ_ALGORITHM)
        return 3;
    else
        return 70;
}

int MainWindow::DefaultContrastThreshold1(int ObjectID)
{
    return 120;
}

int MainWindow::DefaultContrastThreshold2(int ObjectID)
{
    return 150;
}

void MainWindow::SetQualityAlgorithm()
{
    int ret;
    ret = GBMSGUI_SetQualityAlgorithm(QualityAlgorithm);
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("GBMSGUI_SetQualityAlgorithm");
        return;
    }

    // unselect object, for resetting edit boxes
    ui->lstItems->setCurrentItem(NULL);
}

void MainWindow::SetDefaultQualityThresholds()
{
    // set thresholds according to the selected algorithm
    int Threshold1, Threshold2;
    if (QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
    {
        Threshold1 = 2;
        Threshold2 = 3;
    }
    else
    {
        Threshold1 = 50;
        Threshold2 = 70;
    }
	for (int i=1; i<MAX_ITEM_INDEX; i++)
    {
		ItemSettings[i].QualityTh1 = Threshold1;
		ItemSettings[i].QualityTh2 = Threshold2;
    }
}

int MainWindow::SetSegmentsQualityThresholds(int ObjectID)
{
    int Threshold1, Threshold2;
    if (QualityAlgorithm == GBMSGUI_NFIQ_ALGORITHM)
    {
        Threshold1 = 2;
        Threshold2 = 3;
    }
    else
    {
        Threshold1 = 50;
        Threshold2 = 70;
    }

    int num;

	// 1.4.0.0 - and upper palms
    if ((ObjectID == GBMSAPI_SO_SLAP_4_LEFT) || (ObjectID == GBMSAPI_SO_SLAP_4_RIGHT) ||
		(ObjectID == GBMSAPI_SO_UPPER_HALF_PALM_LEFT) || (ObjectID == GBMSAPI_SO_UPPER_HALF_PALM_RIGHT))
        num = 4;
    else
        num = 2;

    int ret;
    for (int i=0; i<num; i++)
    {
        ret = GBMSGUI_SetSegmentQualityThresholds(ObjectID, i, Threshold1, Threshold2);
        if (ret == GBMSGUI_RET_FAILURE)
            return ret;
    }

    return ret;
}

void MainWindow::SetRollArtefactsThresholds()
{
    int ret;
    bool ok;
    int Threshold1 = ui->txtRollArtefactThreshold1->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Roll Artefact Threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtRollArtefactThreshold1->setFocus();
        return;
    }

    int Threshold2 = ui->txtRollArtefactThreshold2->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Roll Artefact Threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtRollArtefactThreshold2->setFocus();
        return;
    }

    ret = GBMSGUI_SetArtefactsThresholds(Threshold1, Threshold2);
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("GBMSGUI_SetArtefactsThresholds");
		return;
    }

	RollArtefactThreshold1 = Threshold1;
	RollArtefactThreshold2 = Threshold2;
}

void MainWindow::SetInclinationThreshold()
{
    int ret;
    bool ok;
    int Threshold = ui->txtInclinationThreshold->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Inclination Threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtInclinationThreshold->setFocus();
        return;
    }

    ret = GBMSGUI_SetInclinationThreshold(Threshold);
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("GBMSGUI_SetInclinationThreshold");
		return;
    }

	InclinationThreshold = Threshold;
}

void MainWindow::SetPalmInclinationThreshold()
{
    int ret;
    bool ok;
    int Threshold = ui->txtPalmInclinationThreshold->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Palm Inclination Threshold", "Invalid value!", QMessageBox::Ok);
        ui->txtInclinationThreshold->setFocus();
        return;
    }

    ret = GBMSGUI_SetPalmInclinationThreshold(Threshold);
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("GBMSGUI_SetPalmInclinationThreshold");
		return;
    }

	PalmInclinationThreshold = Threshold;
}

void MainWindow::SetLiveSegmEvalTimeout()
{
    int ret;
    bool ok;
    int Timeout = ui->txtLiveSegmQualityEvalTimeout->text().toInt(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Live Segm Eval Timeout", "Invalid value!", QMessageBox::Ok);
        ui->txtLiveSegmQualityEvalTimeout->setFocus();
        return;
    }

    ret = GBMSGUI_SetLiveSegmEvalTimeout(Timeout);
    if (ret == GBMSGUI_RET_FAILURE)
    {
        DisplayGBMSGUIError("GBMSGUI_SetLiveSegmEvalTimeout");
		return;
    }

	LiveSegmQualityEvalTimeout = Timeout;
}

void MainWindow::on_cmbQualityAlgorithm_currentIndexChanged(int index)
{
    QualityAlgorithm = ui->cmbQualityAlgorithm->itemData(index).toInt();
    SetQualityAlgorithm();
	SetDefaultQualityThresholds();
}

void MainWindow::EmptyImagesFolder()
{
    QStringList list = ImagesPath.entryList(QDir::NoDotAndDotDot|QDir::AllEntries);

    for (int i=0; i<list.count(); i++)
        ImagesPath.remove(list[i]);
}


void MainWindow::SaveImage(QString FileName, unsigned char *Buffer, int SizeX, int SizeY)
{
    int FileFormat = ui->cmbOutputFormat->itemData(ui->cmbOutputFormat->currentIndex()).toInt();

    switch (FileFormat)
    {
        case FF_BITMAP:
            SaveToBmp(FileName + ".bmp", Buffer, SizeX, SizeY);
            break;
        case FF_JPEG:
            SaveToJPEG(FileName + ".jpg", Buffer, SizeX, SizeY);
            break;
        case FF_JPEG2000:
            SaveToJPEG2000(FileName + ".jp2", Buffer, SizeX, SizeY);
            break;
        case FF_WSQ:
            SaveToWSQ(FileName + ".wsq", Buffer, SizeX, SizeY);
            break;
    }
}

void MainWindow::SaveToWSQ(QString FileName, unsigned char *Buffer, int SizeX, int SizeY)
{
    int ret;
    unsigned char *CompressedBuffer;
    int CompressedBufferLen;
    QString ErrMsg;

    try
    {
        ret = WSQPACK_Compress(&CompressedBuffer, &CompressedBufferLen,
                    0.75,//15:1
                    Buffer, SizeX, SizeY, 8, 500, NULL);
    }
    catch( char *str )
    {
        ErrMsg = QString("WSQPACK_Compress exception: %1").arg(str);
        QMessageBox::critical(this, "Save image", ErrMsg, QMessageBox::Ok);
        return;
    }

    if (ret != WSQPACK_OK)
    {
        QMessageBox::critical(this, "Save image", "WSQPACK_Compress Error", QMessageBox::Ok);
        return;
    }

    QFile File(FileName);
    if (File.open(QIODevice::WriteOnly))
    {
        File.write((const char *)CompressedBuffer, CompressedBufferLen);
        File.close();
    }

    WSQPACK_Free(CompressedBuffer);
}

void MainWindow::SaveToJPEG(QString FileName, unsigned char *Buffer, int SizeX, int SizeY)
{
    int ret;
    unsigned char *CompressedBuffer;
    unsigned long CompressedBufferLen;

    ret = GBJPEG_JPG_Encode(&CompressedBuffer, &CompressedBufferLen,
        Buffer, (unsigned long)SizeX, (unsigned long)SizeY,
        8, 500, 100);

    if(ret != GBJPEG_OK)
    {
        QMessageBox::critical(this, "Save image", "GBJPEG_JPG_Encode Error", QMessageBox::Ok);
        return;
    }

    QFile File(FileName);
    if (File.open(QIODevice::WriteOnly))
    {
        File.write((const char *)CompressedBuffer, CompressedBufferLen);
        File.close();
    }

    GBJPEG_Free(CompressedBuffer);
}

void MainWindow::SaveToJPEG2000(QString FileName, unsigned char *Buffer, int SizeX, int SizeY)
{
    int ret;
    unsigned char *CompressedBuffer;
    unsigned long  CompressedBufferLen;

    ret = GBJPEG_JP2_Encode(&CompressedBuffer, &CompressedBufferLen,
            Buffer,
            (unsigned long)SizeX, (unsigned long)SizeY,
            8,
            14);	// 14:1 compression

    if(ret != GBJPEG_OK)
    {
        QMessageBox::critical(this, "Save image", "GBJPEG_JP2_Encode Error", QMessageBox::Ok);
        return;
    }

    QFile File(FileName);
    if (File.open(QIODevice::WriteOnly))
    {
        File.write((const char *)CompressedBuffer, CompressedBufferLen);
        File.close();
    }

    GBJPEG_Free(CompressedBuffer);
}

void MainWindow::OnFROptionChange() // this is a slot
{
    ShowDefaultFrameRate();
}

void MainWindow::ShowDefaultFrameRate()
{
    int ret;
    // show default value
    double FRMax, FRMin, FRDefault;
    int DeviceID = DeviceList[ui->cmbDevices->currentIndex()].DeviceID;
	unsigned int FrameRateOptions = 0;
	unsigned int ScanArea;
	
	if (!ui->lstScanArea->currentItem())
		return;

	ScanArea = ui->lstScanArea->currentItem()->data(Qt::UserRole).toInt();

	if (ui->optFROFullRes->isChecked())
		FrameRateOptions |= GBMSAPI_FRO_FULL_RESOLUTION_MODE;
	if (ui->chkFROAdaptiveRoll->isChecked())
		FrameRateOptions |= GBMSAPI_FRO_ADAPT_ROLL_AREA_POSITION;
	if (ui->chkFROHiResPreview->isChecked())
		FrameRateOptions |= GBMSAPI_FRO_HIGH_RES_IN_PREVIEW;

	ret = GBMSAPI_GetFrameRateRange2(DeviceID, FrameRateOptions, ScanArea, 
		&FRMax, &FRMin, &FRDefault);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_GetFrameRateRange2");
        return;
    }

    ui->txtFrameRate->setText(QString("%1").arg(FRDefault, 0, 'f', 1));
    // show range
    ui->lblFrameRateRange->setText(QString("(%1-%2)").arg(FRMin, 0, 'f', 1).arg(FRMax, 0, 'f', 1));
}

void MainWindow::SetFrameRate()
{
    int ret;
    double FrameRate;
    int DeviceID = DeviceList[ui->cmbDevices->currentIndex()].DeviceID;
    unsigned int FrameRateOptions = 0;
	int ScanArea = ui->lstScanArea->currentItem()->data(Qt::UserRole).toInt();

	if (ui->optFROFullRes->isChecked())
		FrameRateOptions |= GBMSAPI_FRO_FULL_RESOLUTION_MODE;
	if (ui->chkFROAdaptiveRoll->isChecked())
		FrameRateOptions |= GBMSAPI_FRO_ADAPT_ROLL_AREA_POSITION;
	if (ui->chkFROHiResPreview->isChecked())
		FrameRateOptions |= GBMSAPI_FRO_HIGH_RES_IN_PREVIEW;

    // flat
    bool ok;
    FrameRate = ui->txtFrameRate->text().toFloat(&ok);
    if (!ok)
    {
        QMessageBox::critical(this, "Set Frame Rate", "Invalid Frame rate value!", QMessageBox::Ok);
        ui->txtFrameRate->setFocus();
        return;
    }

    ret = GBMSAPI_SetFrameRate2(ScanArea, FrameRateOptions, FrameRate);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_SetFrameRate2");
        return;
    }
/*
	ret = GBMSAPI_SetFrameRateComplete(FrameRateOptions, FrameRate);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_SetFrameRateComplete");
        return;
    }
*/
}

int MainWindow::SetMissingFingers(int ObjectID)
{
    int ret = GBMSGUI_RET_SUCCESS;
    int i;
    QListWidget *lstUnavailableFingers;

    switch (ObjectID) {
        case GBMSAPI_SO_SLAP_4_LEFT:
        case GBMSAPI_SO_SLAP_2_LEFT:
        case GBMSAPI_SO_UPPER_HALF_PALM_LEFT:	// 1.4.0.0
            lstUnavailableFingers = ui->lstUnavailableFingersLeft;
            break;

        case GBMSAPI_SO_SLAP_4_RIGHT:
        case GBMSAPI_SO_SLAP_2_RIGHT:
        case GBMSAPI_SO_UPPER_HALF_PALM_RIGHT:	// 1.4.0.0
            lstUnavailableFingers = ui->lstUnavailableFingersRight;
            break;

        case GBMSAPI_SO_SLAP_2_THUMBS:
        case GBMSAPI_SO_SLAP_2_INDEXES:
            lstUnavailableFingers = ui->lstUnavailableFingersThumbs;
            break;
    }

    for (i=0; i<lstUnavailableFingers->count(); i++)
    {
        if (lstUnavailableFingers->item(i)->checkState() == Qt::Checked)
        {
            ret = GBMSGUI_SetSegmentUnavailabilityReason(ObjectID, i, GBMSGUI_UR_UNPRINTABLE);
            if (ret == GBMSGUI_RET_FAILURE)
            {
                return ret;
            }
        }
    }

    return ret;
}

void MainWindow::OnLanguageChange(int Index)		// this is a slot
{
	if (Index == -1)
		return;

	QString Lang = ui->cmbLanguage->itemData(Index).toString();
	GBMSGUI_SetLanguage(Lang.toAscii());
}

void MainWindow::on_btnSelectObject_clicked()
{
    int ret;
    int ScannedObjectID;

    ret = GBMSGUI_SelectScannedObject(&ScannedObjectID);
    if (ret == GBMSGUI_RET_FAILURE) 
	{
        DisplayGBMSGUIError("GBMSGUI_SelectScannedObject");
        return;
    }

	for (int i=0; i<ui->lstItems->count(); i++)
	{
		if (ui->lstItems->item(i)->data(Qt::UserRole).toInt() == ScannedObjectID)
		{
			ui->lstItems->setCurrentRow(i);
			ui->lstItems->item(i)->setCheckState(Qt::Checked);
			break;
		}
	}
}

void MainWindow::OnRollAreaChange() // this is a slot
{
    if (ui->chkRollAreaGA->isChecked())
	{
		//GBMSAPI_SetRollAreaStandard(GBMSAPI_DF_ROLL_AREA_GA);
		// moved in Acquire, and call funcion of GBMSGUI
		// adapt rolled ojects size
		SetRollSizeGA(true);
	}
	else
	{
		//GBMSAPI_SetRollAreaStandard(GBMSAPI_DF_ROLL_AREA_IQS);
		SetRollSizeGA(false);
	}

	// if an item in the list was selected, refresh the view
	if (ui->lstItems->currentItem())
		on_lstItems_currentItemChanged(ui->lstItems->currentItem(), ui->lstItems->currentItem());
}

void MainWindow::SetRollSizeGA(bool GAMode)
{
	int RollWidth;
	int RollHeight;

	if (GAMode)
	{
		RollWidth = 640;
		RollHeight = 640;
	}
	else
	{
		RollWidth = 800;
		RollHeight = 750;
	}

	for (int i=1; i<MAX_ITEM_INDEX; i++)
	{
		if (GBMSGUI_IsRolled(i) && !GBMSGUI_IsJoint(i) && !GBMSGUI_IsRolledThenar(i) && !GBMSGUI_IsRolledHypothenar(i))
		{
			ItemSettings[i].ImageWidth = RollWidth;
			ItemSettings[i].ImageHeight = RollHeight;
		}
	}
}

int MainWindow::SetSegmentsImageSize(int ObjectID)
{
	bool ok;

	SegmSizeX = ui->txtSegmentsWidth->text().toInt(&ok);
	if (!ok)
	{
		QMessageBox::critical(this, "Segments image width", "Invalid value!", QMessageBox::Ok);
		ui->txtSegmentsWidth->setFocus();
		SegmSizeX = 500;
		return GBMSGUI_RET_SUCCESS;
	}

	SegmSizeY = ui->txtSegmentsHeight->text().toInt(&ok);
	if (!ok)
	{
		QMessageBox::critical(this, "Segments image height", "Invalid value!", QMessageBox::Ok);
		ui->txtSegmentsHeight->setFocus();
		SegmSizeY = 500;
		return GBMSGUI_RET_SUCCESS;
	}

    int num;
	// 1.4.0.0 - and upper palms
    if ((ObjectID == GBMSAPI_SO_SLAP_4_LEFT) || (ObjectID == GBMSAPI_SO_SLAP_4_RIGHT) ||
		(ObjectID == GBMSAPI_SO_UPPER_HALF_PALM_LEFT) || (ObjectID == GBMSAPI_SO_UPPER_HALF_PALM_RIGHT))
        num = 4;
    else
        num = 2;

    int ret;
    for (int i=0; i<num; i++)
    {
		ret = GBMSGUI_SetSegmentImageSize(ObjectID, i, SegmSizeX, SegmSizeY);
        if (ret == GBMSGUI_RET_FAILURE)
            return ret;
    }

    return ret;
}

void MainWindow::GetJointBoundingBoxes(int ObjectID)
{
    int i, num_segments;
    int ret;
    bool SegmentAvailable;
    int Left, Top, Right, Bottom;
    int UnavailabilityReason;
    int FingerprintQuality;
    unsigned char *SegmImagePtr;
    int SegmImageSize;
    int ImageWidth, ImageHeight;
    int Dpi;
	QString Msg = "";

    if (GBMSGUI_IsThumbJoint(ObjectID))
        num_segments = 2;
    else
        num_segments = 3;

    for (i=0; i<num_segments; i++)
    {
        ret = GBMSGUI_GetSegmentationResult(i,      // index of segment
            0,                     // quality algorithm
            &SegmentAvailable,                      // segment availability flag
            &Left, &Top, &Right, &Bottom,           // bounding box coordinates
            &UnavailabilityReason,                  // reason of finger unavailability (if so)
            &FingerprintQuality,                    // fingerprint quality
            NULL, 0, 
			&ImageWidth, &ImageHeight, &Dpi);
        if (ret == GBMSGUI_RET_FAILURE)
        {
            DisplayGBMSGUIError("GBMSGUI_GetSegmentationResult");
        }
        else
        {
            // if the finger is available, save the image
            if (SegmentAvailable)
				Msg.append(QString("Left=%1 Top=%2 Right=%3 Bottom=%4\n").arg(Left).arg(Top).arg(Right).arg(Bottom));
        }
    }

	QMessageBox::information(this, "Joint segments", Msg, QMessageBox::Ok);
}

void MainWindow::OnAdapRollAreaPositionChange() // this is a slot
{
	//ui->grpRollDirection->setEnabled(ui->chkAdaptRollAreaPosition->isChecked());
	ui->toolBoxOptions->setItemEnabled(3, ui->chkAdaptRollAreaPosition->isChecked());

    // don't allow no roll preview
    if (ui->chkAdaptRollAreaPosition->isChecked())
    {
		if (ui->optNoRollPreview->isChecked())
        {
			ui->optNoRollPreview->setChecked(false);
            ui->optRollPreviewSide->setChecked(true);
        }
		ui->optNoRollPreview->setEnabled(false);
    }
    else
        ui->optNoRollPreview->setEnabled(true);
}

void MainWindow::on_btnSetFrameRate_clicked() // this is a slot
{
	SetFrameRate();
}

void MainWindow::LoadScanAreaList()
{
	int ret;
	unsigned int SupportedScanAreas;

	ui->lstScanArea->clear();

	ret = GBMSAPI_GetSupportedScanAreas(&SupportedScanAreas);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_GetSupportedScanAreas");
        return;
    }

    QListWidgetItem *Item;

	if (CheckMask(SupportedScanAreas, GBMSAPI_SA_FULL_FRAME))
	{
	    Item = new QListWidgetItem("Full Frame");
		Item->setData(Qt::UserRole, GBMSAPI_SA_FULL_FRAME);
		ui->lstScanArea->addItem(Item);
	}
	if (CheckMask(SupportedScanAreas, GBMSAPI_SA_ROLL_IQS))
	{
	    Item = new QListWidgetItem("Roll IQS");
		Item->setData(Qt::UserRole, GBMSAPI_SA_ROLL_IQS);
		ui->lstScanArea->addItem(Item);
	}
	if (CheckMask(SupportedScanAreas, GBMSAPI_SA_ROLL_GA))
	{
	    Item = new QListWidgetItem("Roll GA");
		Item->setData(Qt::UserRole, GBMSAPI_SA_ROLL_GA);
		ui->lstScanArea->addItem(Item);
	}
	if (CheckMask(SupportedScanAreas, GBMSAPI_SA_ROLL_THENAR))
	{
	    Item = new QListWidgetItem("Roll Thenar");
		Item->setData(Qt::UserRole, GBMSAPI_SA_ROLL_THENAR);
		ui->lstScanArea->addItem(Item);
	}
	if (CheckMask(SupportedScanAreas, GBMSAPI_SA_ROLL_JOINT))
	{
	    Item = new QListWidgetItem("Roll Joint");
		Item->setData(Qt::UserRole, GBMSAPI_SA_ROLL_JOINT);
		ui->lstScanArea->addItem(Item);
	}

	// select the first
	if (ui->lstScanArea->count())
		ui->lstScanArea->setCurrentRow(0);
}

void MainWindow::on_lstScanArea_changed(int currentRow)
{
	OnFROptionChange();
}

void MainWindow::OnEnableBlockCompositionChange() // this is a slot
{
	int ret;
	int Enable;

	if (ui->chkEnableBlockComposition->isChecked())
		Enable = 1;
	else
		Enable = 0;

	ret = GBMSAPI_ROLL_EnableBlockComposition(Enable);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSAPI_ROLL_EnableBlockComposition");
        return;
    }
}

void MainWindow::OnEnableBlockAutocaptureLedColorFeedback() // this is a slot
{
	int ret;
	int Enable;

	if (ui->chkBlockAutocaptureLedColorFeedback->isChecked())
		Enable = 1;
	else
		Enable = 0;

	ret = GBMSGUI_EnableBlockAutocaptureLedColorFeedback(Enable);
    if (ret != GBMSAPI_ERROR_CODE_NO_ERROR)
    {
        DisplayGBMSAPIError(ret, "GBMSGUI_EnableBlockAutocaptureLedColorFeedback");
        return;
    }
}
