#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QMessageBox>
#include <QListWidgetItem>
#include <QDir>
#include "GBMSAPI_Library.h"
#include "GBMSGUI_Qt.h"

namespace Ui {
    class MainWindow;
}

typedef struct ItemSetting
{
	int ImageWidth;
	int ImageHeight;
	int QualityTh1;
	int QualityTh2;
	int ContrastTh1;
	int ContrastTh2;
};
#define MAX_ITEM_INDEX	107	// corresponds to the max Sannable Object ID (GBMSAPI_Defines.h)

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    static bool __stdcall AfterStartCallback(void *Param);

private slots:
    void on_cmbDevices_currentIndexChanged(int index);
    void on_btnSequence_clicked();
    void on_btnSelectAll_clicked();
    void on_btnStartSession_clicked();
    void on_btnAcquireItem_clicked();
    void on_btnEndSession_clicked();
    void on_btnUnselectAll_clicked();
    void on_grpAuto_toggled(bool arg1);
    void on_grpManual_toggled(bool arg1);
    void on_btnRefresh_clicked();
    void on_btnSelectObject_clicked();
    void OnExit();
    void on_lstItems_currentItemChanged(QListWidgetItem *current, QListWidgetItem *previous);
    void on_btnSetObjectSettings_clicked();
    void on_cmbQualityAlgorithm_currentIndexChanged(int index);
    void OnFROptionChange();
	void OnLanguageChange(int Index);
	void OnRollAreaChange();
	void OnAdapRollAreaPositionChange();
	void on_btnSetFrameRate_clicked();
	void on_lstScanArea_changed(int currentRow);
	void OnEnableBlockCompositionChange();
	void OnEnableBlockAutocaptureLedColorFeedback();

private:
    Ui::MainWindow *ui;

    GBMSAPI_DeviceInfoStruct DeviceList[GBMSAPI_MAX_PLUGGED_DEVICE_NUM];
    int DevicesNumber;
    unsigned int MaxImageSizeX;
    unsigned int  MaxImageSizeY;
    unsigned char *ImageBuffer;
    int ImageBufferSize;
    QVector<QRgb> GrayTable;
    QDir ImagesPath;
	int CurrentObjectID;
	ItemSetting ItemSettings[MAX_ITEM_INDEX+1];
	int SegmSizeX;
	int SegmSizeY;

    // INI file values
    int AcqWndLeft;
    int AcqWndTop;
    int AcqWndWidth;
    int AcqWndHeight;
    bool AcqWndMaximized;
    unsigned int SessionOptions;
    unsigned int AcquisitionOptions;
    int QualityAlgorithm;
    int RollArtefactThreshold1;
    int RollArtefactThreshold2;
    int InclinationThreshold;
    int PalmInclinationThreshold;
	int LiveSegmQualityEvalTimeout;

    bool StartSession(int AcquisitionMode);
    int Acquire(int ObjectID, QString ObjectName);
    void StopSession();
    void DisplayGBMSAPIError(int ErrorCode, QString FunctionName);
    void DisplayGBMSGUIError(QString FunctionName);
    void LoadDevicesList();
    void LoadItemsList();
    void AddItemToList(QString Name, int Value);
    void SetSessionOptions();
    void SetAcquisitionOptions();
    void SaveToBmp(QString FileName, unsigned char *Buffer, int SizeX, int SizeY);
    QString GetDeviceName(unsigned char DeviceID);
    bool SaveSegments(int ObjectID, QString ObjectName);
    int SlapIdIndex2ObjectID(int SlapID, int Index);
    void ReadIni();
    void WriteIni();
    int DefaultImageWidth(int ObjectID);
    int DefaultImageHeight(int ObjectID);
    int DefaultQualityThreshold1(int ObjectID, int AlgorithmID);
    int DefaultQualityThreshold2(int ObjectID, int AlgorithmID);
    int DefaultContrastThreshold1(int ObjectID);
    int DefaultContrastThreshold2(int ObjectID);
    void SetQualityAlgorithm();
    void SetDefaultQualityThresholds();
    void SetRollArtefactsThresholds();
    void SetInclinationThreshold();
	void SetPalmInclinationThreshold();
	void SetLiveSegmEvalTimeout();
    void EmptyImagesFolder();
    void SaveImage(QString FileName, unsigned char *Buffer, int SizeX, int SizeY);
    void SaveToWSQ(QString FileName, unsigned char *Buffer, int SizeX, int SizeY);
    void SaveToJPEG(QString FileName, unsigned char *Buffer, int SizeX, int SizeY);
    void SaveToJPEG2000(QString FileName, unsigned char *Buffer, int SizeX, int SizeY);
    void ShowDefaultFrameRate();
    void SetFrameRate();
    int SetMissingFingers(int ObjectID);
    int SetSegmentsQualityThresholds(int ObjectID);
	void SetRollSizeGA(bool GAMode);
	int SetSegmentsImageSize(int ObjectID);
	void GetJointBoundingBoxes(int ObjectID);
	void LoadScanAreaList();
};

#endif // MAINWINDOW_H
