/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package GBFINIMG_JAVA_Defines;

/**
 *
 * @author andrea
 */
public class GBFINIMG_ProcessingOptions {

    public static final int GBFINIMG_HALO_LATENT_ELIMINATION = 0x00000001;
    // Wipes halo-effect/latent-prints in <InputImage> buffer around fingerprint segments.
    public static final int GBFINIMG_SEGMENT_VERTICAL_ROTATION = 0x00000002;
    //Rotates segmented images in <SegmentImageList[].SegmentImagePtr> to the vertical position.
    //NOTE: this operation requires "GBImgTran.dll" to be allocated
    //      in the same directory with "GBFINIMG.DLL".
    //VER3000: for half-palm images the rotation will not be applied 
    //		   in a case when the system is unable detect if a hand is left or right:
    //         (GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.Hand == GBFINIMG_HAND_UNDEFINED)
    public static final int GBFINIMG_PALM_PRINT_IMAGE_DELETE_FINGER_SEGMENTS = 0x00000004; //VER3000
    // Deletes eventual fingerprint segments found around palm-prints in <InputImage> buffer. 
    // Recommended to be used.
    // This option is effective just for palm-print image types and only together with
    //  option GBFINIMG_HALO_LATENT_ELIMINATION
    public static final int GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION = 0x00000008; //VER3000
    // Enables complete palm-print image processing that provides parameters of quality of
    //	 palm-print image (GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.PatternValidity and  
    //	 GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR.Quality)
    // ATTENTION: due to big size of palm-prints, activating this option will result in 
    //			  significantly longer processing time.
    public static final int GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING = 0x00000010; //VER3000-1000DPI
    // Allows to process 1000-dpi print images. If this option is set:  
    //	- the resolution of <InputImage> will be considered as 1000 dpi instead of 500 dpi
    //	- <InputSizeX> is limited by GBFINIMG_MAX_INPUT_SIZE_X_1000DPI 
    //	- <InputSizeY> is limited by GBFINIMG_MAX_INPUT_SIZE_Y_1000DPI 
    //  If this option is set together with GBFINIMG_SEGMENT_VERTICAL_ROTATION, an auxiliary 
    //	   buffer will be required for rotation operation, see GBFINIMG_SetRotationBuffer1000DPI()
    //	   function description.
    public static final int GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY = 0x00000020; //VER3002
    // Allows to process fingerprint images in a faster way omitting some processing steps.
    // Attention: if this option is set: 
    // 1. application can use only  <*DetectedSegmentImageNumber> and <*DetectionFailureDiagnostic> 
    //    output parameters (with one exception - <*DetectionFailureDiagnostic> output parameters 
    //    in this case CAN NOT be used for writer's palm).
    // 2. Output parameter <SegmentImageList> CAN NOT BE USED for any kind of <InputImageType>!!!!
    // 3. The following other options will be ignored:
    //		- GBFINIMG_HALO_LATENT_ELIMINATION
    //		- GBFINIMG_SEGMENT_VERTICAL_ROTATION
    //		- GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION
    public static final int GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK = 0x00000040;//VER3002
    // Can be useful for the cases when application/operator itself ensures that applied hand 
    //  corresponds to the requested one (there is no surely left-right hand mix-up).
    // If this option is set, GBFINIMG_ProcessImage() function does not try to verify the hipotesis
    //	of hand mix-up and this fact improves the performnace of GBFINIMG_GetFingerprintCounter()
    //  function (decreases error rate) especially in cases of non-complete slaps (when one or more 
    //	fingers of a slap are absent/bandaged). 
    //  This option is supported only for the following <InputImageType>:
    //			- GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4 
    //			- GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4 
    public static final int GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA = 0x00000080;
    // Allows to process fingerprint images in a faster way omitting some processing steps (similar to 
    //   GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY option).
    // Attention: if this option is set: 
    // 1. application can use <*DetectedSegmentImageNumber> and <*DetectionFailureDiagnostic> 
    //    output parameters (with one exception - <*DetectionFailureDiagnostic> output parameters 
    //    in this case CAN NOT be used for writer's palm) and output parameter <SegmentImageList>.
    //   In difference from GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY, if this option is set,
    //    <SegmentImageList> data will be generated and it will contain valid information 
    //    for all fields except for these: 
    //			SegmentImageList[].SegmentImagePtr			- separate segment image is NOT AVAILABLE
    //			SegmentImageList[].PatternValidity			- NOT AVAILABLE
    //			SegmentImageList[].PatternCompleteness		- NOT AVAILABLE
    //	  Note: all available fields in SegmentImageList are results of "fast" processing, so, in some cases "fast" results
    //			can differ from "complete" processing results.
    // 2. The following other options will be ignored:
    //      - GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
    //		- GBFINIMG_HALO_LATENT_ELIMINATION
    //		- GBFINIMG_SEGMENT_VERTICAL_ROTATION
    //		- GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION 
    public static final int GBFINIMG_ALLOW_INCOMPLETE_SLAP_SEGMENTS_DATA_GENERATION = 0x00000100;
    // This option allows to activate automatic fingerprint segment naming even in case of 
    // incomplete slap.
    // If this option is NOT set the default behaviour of GBFINIMG_ProcessImage() function 
    // will be the following - 
    //
    //  in case of missing finger(s) in slap/thumbs and with no external info available
    //  about which fingers are missing (GBFINIMG_Slap4_MissingFingers() not called) in
    //  FULLY-AUTOMATIC mode output parameter <DetectedSegmentImageNumber> of 
    //  GBFINIMG_ProcessImage() returns 0 (segmentation result is negative).
    //
    // Instead, if this option is set, GBFINIMG_ProcessImage() function will 
    // automatically try to perform detected fingerprint segment naming, that is, to 
    // determine to which finger a concrete fingeprint segment belongs to. In this case
    // output parameter <DetectedSegmentImageNumber> returns actual detected fingerprint
    // segment number and <SegmentImageList> contains fingerprint segments information
    // with finger names associated to the segments by special automatic naming procedure
    // that analyses mutual finger positions and position on scanning window for better
    // results.
    //
    // Attention: when system has a high-level of confidence about automatic segmentation
    //          result of non-complete slap, diagnostic message
    //          GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP will not be set; otherwise, 
    //          when confidence level is not high enough, diagnostic message
    //          GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP will be present and in this case
    //          it is strongly recommended to check the results by human operator to avoid
    //          fingerprint naming errors that can lead in future to wrong finger identification.
    //
    // This option is supported only for the following <InputImageType>:
    //			- GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4 
    //			- GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4 
    //			- GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2
    //
    // This option must be obligatory used together with previously defined 
    // GBFINIMG_DISABLE_LEFT_RIGHT_HAND_MIXUP_CHECK option (see above), otherwise, it will be
    // ignored. 
    // That means that in this case system will not check left-right hand mix and will rely
    // completely on the information about hand passed by application. So, application/human
    // operator should ensure appropriate hand applying for correct result.
    //
    // This option will be ignored if at least one of the previously defined options is
    // present:
    //			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
    //			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATAGBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
    
    // Ver 3.3.0.0
    public static final int GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE = 0x00000200; //VER3500
    // IMPORTANT NOTE: DO NOT USE THIS OPTION FOR APPLICATIONS THAT REQUIRE FBI IQS APPENDIX F CERTIFICATION
    //                 (like fingerprint/palmprint forensic or civil-id enrollment)
    // Enables high-quality specially-designed software algorithm to restore/refine fingerprint ridges 
    //  especially of dry/underpressed fingerprint image containing in <InputImage> buffer to improve 
    //  fingerprint quality.
    // This option will be ignored if at least one of the previously defined options is present:
    //			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
    //			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
    //			- GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING
    // Attention: this is CPU-resource consuming option.
    // Note: if this option set for <InputImageType> == GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM or 
    //  GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM, automatically internally also 
    //	GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION option will be set to allow palm image enhancement.
    public static final int GBFINIMG_REFINE_WET_FINGERPRINT_IMAGE = 0x00000400; //VER3500
    // IMPORTANT NOTE: DO NOT USE THIS OPTION FOR APPLICATIONS THAT REQUIRE FBI IQS APPENDIX F CERTIFICATION
    //                 (like fingerprint/palmprint forensic or civil-id enrollment)
    // Enables high-quality specially-designed software algorithm to restore/refine fingerprint ridges 
    //  especially of wet/overpressed fingerprint image containing in <InputImage> buffer to improve 
    //  fingerprint quality.
    //  This option will be ignored if at least one of the previously defined options is present:
    //			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
    //			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
    //			- GBFINIMG_1000DPI_PRINT_IMAGE_PROCESSING
    // Attention: this is CPU-resource consuming option.
    // Note: if this option set for <InputImageType> == GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_HALF_PALM or 
    //  GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_HALF_PALM, automatically internally also 
    //	GBFINIMG_PALM_PRINT_IMAGE_QUALITY_CALCULATION option will be set to allow palm image enhancement.
    public static final int GBFINIMG_REFINE_FINGERPRINT_CONTRAST_AND_CLEAN_BACKGROUND = 0x00000800; //VER3500
    // Enables high-quality specially-designed software algorithm to improve overall contrast by
    //  expanding dynamic range of a fingerprint image containing in <InputImage> buffer to improve 
    //  fingerprint quality and also cleans image areas around fingerprint(s) to delete eventual residual 
    //  latent prints and halo-effects.
    //  This option will be ignored if at least one of the previously defined options is present:
    //			- GBFINIMG_FAST_PROCESSING_SEGMENT_NUMBER_ONLY
    //			- GBFINIMG_FAST_PROCESSING_PRELIMINARY_SEGMENT_DATA
    // Note: this option is similar to GBFINIMG_HALO_LATENT_ELIMINATION, but additionally 
    //  it improves fingerprint image contrast and provides white valleys within fingerprint area 
    //  for better seamless conjuctiuon with white background around fingerprint.
    public static final int GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE_AGE_CREASES_UNTOUCHED = 0x00001000; //VER3510
    // This option has to be used in conjuction with GBFINIMG_REFINE_DRY_FINGERPRINT_IMAGE option, 
    //	otherwise, it will be ignored. If this option is present, age creases on dry-skin fingerprint image 
    //  will not be corrected - ridges disconnected by age creases will not be reconnected.
    public static final int GBFINIMG_PAPER_CARD_INK_PRINT = 0x00002000; //VER4000
    // This option has to be used to improve performance when ink print from paper card has to be elaboarted.
    public static final int GBFINIMG_JOINT_FINGER_IS_THUMB = 0x00004000; //VER4000
    // This option helps to manage the elaboration of lower phalange in case of thumb (note that thumb has only proximal 
    //  joint segments and has no medial joint). Use this option to inform GBFINIMG_ProcessImage() function that processed 
    //  finger is a THUMB in cases when <InputImageType> is one of the following types:
    //		- GBFINIMG_INPUT_IMAGE_TYPE_ROLLED_JOINT_FINGER_FV1	
    //		- GBFINIMG_INPUT_IMAGE_TYPE_FLAT_JOINT_FINGER_FV2		
    //		- GBFINIMG_INPUT_IMAGE_TYPE_FLAT_JOINT_FINGER_FV3
    //		- GBFINIMG_INPUT_IMAGE_TYPE_FLAT_JOINT_FINGER_FV4
    //  If this option is set, GBFINIMG_GetFullFingerMedialProximalJointBoundingBox() function will return 
    //   proximal joint bounding box coordinates and will leave medial joint coordinates undefined.
    public static final int GBFINIMG_SEGMENT_IMAGE_AS_BOUNDING_BOX = 0x00008000; //VER4001
    // The size of segment image generated in <SegmentImageList[n].SegmentImagePtr> will be determined automatically by system for each finger 
    //	 segment and will correspond to segment's Bounding Box width and height. 
    //  Segment image width can be calculated as (SegmentImageList[n].BoundingBoxR-SegmentImageList[n].BoundingBoxL) for "n"-th segment.
    //   If bounding box width is bigger than <SegmentImageSizeX>, it will be limited by <SegmentImageSizeX>.
    //  Segment image height can be calculated as (SegmentImageList[n].BoundingBoxB-SegmentImageList[n].BoundingBoxT) for "n"-th segment.
    //   If bounding box height is bigger than <SegmentImageSizeY>, it will be limited by <SegmentImageSizeY>.
    // end Ver 3.3.0.0
}
