#include "ClipRectItem.h"

//QClipRectItem::QClipRectItem(QColor PenColor, int PenWidth, Qt::PenStyle PenStyle)
QClipRectItem::QClipRectItem(QColor PenColor, int PenWidth, Qt::PenStyle PenStyle, bool SemiTransparent)
{
    QGraphicsRectItem::QGraphicsRectItem();

    Text = "";
    QualityValue = 0;
    QualityColor = Qt::lightGray;
	AmpCode = "";
	// mancavano queste inizializzazioni
	resizing = false;
	mEnabled = true;
	mSizeOperation = SizeOperationNone;
	mSizeOption = None;
	DiagIcon = NULL;

    mPen.setColor(PenColor);
    mPen.setWidth(PenWidth);
    mPen.setStyle(PenStyle);
    setPen(mPen);

    // non serve??? e' transparent di default???
    //setBrush(QBrush(Qt::transparent));

	mMinSize = QSize(0, 0);
	mMaxSize = QSize(65535, 65535);

	if (SemiTransparent)
	{
		// use same color as border, semitransparent
		QColor BrushColor = QColor(PenColor.red(), PenColor.green(), PenColor.blue(), 32);
		mBrush.setColor(BrushColor);
		mBrush.setStyle(Qt::BrushStyle::SolidPattern);
		setBrush(mBrush);
	}

    setAcceptHoverEvents(true);
    setFlag(QGraphicsItem::ItemSendsGeometryChanges, true);
}

QClipRectItem::~QClipRectItem()
{
}

//void QClipRectItem::SetSizeOption(SizeOptions SizeOption)
void QClipRectItem::SetSizeOption(unsigned int SizeOption)
{
    mSizeOption = SizeOption;
}

QVariant QClipRectItem::itemChange ( GraphicsItemChange change, const QVariant & value )
{
    if (change == ItemPositionChange && scene())
    {
        // value is the new position.
        QPointF newPos = value.toPointF();
        QRectF rect = scene()->sceneRect();
        //if (!rect.contains(newPos))
        // check not only the top-left, but the whole rectangle...
        if (!rect.contains(QRectF(newPos, this->rect().size())))
        {
            // Keep the item inside the scene rect.
            newPos.setX(qMin(rect.right()-this->rect().width(), qMax(newPos.x(), rect.left())));
            newPos.setY(qMin(rect.bottom()-this->rect().height(), qMax(newPos.y(), rect.top())));
            return newPos;
        }
    }

    if (change == ItemPositionHasChanged)
    {
    }

    return QGraphicsItem::itemChange(change, value);
}

#define BORDER	2
void QClipRectItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{
    qreal x = event->pos().x();
    qreal y = event->pos().y();

    if (mEnabled)
    {
        // check cursor position for resize
        if ((mSizeOption & AllowMove) && (x > BORDER) && (x < rect().width() - BORDER) && (y > BORDER) && (y < rect().height() - BORDER))
        {
            this->setCursor(Qt::OpenHandCursor);
            mSizeOperation = SizeOperationMove;
        }
		// PROVA - corners
		else if ((mSizeOption & AllowResize) && (x >= rect().width() - BORDER) && (x <= rect().width() + BORDER) && (y >= - BORDER) && (y <= BORDER))
		{
            this->setCursor(Qt::SizeBDiagCursor);
            mSizeOperation = SizeOperationNE;
		}
		else if ((mSizeOption & AllowResize) && (x >= -BORDER) && (x <= BORDER) && (y >= rect().height() - BORDER) && (y <= rect().height() + BORDER))
		{
            this->setCursor(Qt::SizeBDiagCursor);
            mSizeOperation = SizeOperationSW;
		}
		else if ((mSizeOption & AllowResize) && (x >= -BORDER) && (x <= BORDER) && (y >= - BORDER) && (y <= BORDER))
		{
            this->setCursor(Qt::SizeFDiagCursor);
            mSizeOperation = SizeOperationNW;
		}
		else if ((mSizeOption & AllowResize) && (x >= rect().width() - BORDER) && (x <= rect().width() + BORDER)  && (y >= rect().height() - BORDER) && (y <= rect().height() + BORDER))
		{
            this->setCursor(Qt::SizeFDiagCursor);
            mSizeOperation = SizeOperationSE;
		}
		// fine PROVA - corners
        else if ((mSizeOption & AllowResize) && (x >= -BORDER) && (x <= BORDER))
        {
            this->setCursor(Qt::SizeHorCursor);
            mSizeOperation = SizeOperationLeft;
        }
        else if ((mSizeOption & AllowResize) && (x >= rect().width() - BORDER) && (x <= rect().width() + BORDER))
        {
            this->setCursor(Qt::SizeHorCursor);
            mSizeOperation = SizeOperationRight;
        }
        else if ((mSizeOption & AllowResize) && (y >= - BORDER) && (y <= BORDER))
        {
            this->setCursor(Qt::SizeVerCursor);
            mSizeOperation = SizeOperationTop;
        }
        else if ((mSizeOption & AllowResize) && (y >= rect().height() - BORDER) && (y <= rect().height() + BORDER))
        {
            this->setCursor(Qt::SizeVerCursor);
            mSizeOperation = SizeOperationBottom;
        }
        else
        {
            //this->setCursor(Qt::ArrowCursor);
            this->unsetCursor();
            mSizeOperation = SizeOperationNone;
        }
    }

    QGraphicsItem::hoverMoveEvent(event);
}

void QClipRectItem::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
    if (!mEnabled)
	{
		// prova - passo comunque l'evento
	    QGraphicsItem::mousePressEvent(event);
        return;
	}

    if ((mSizeOperation != SizeOperationNone) && (mSizeOperation != SizeOperationMove))
        resizing = true;
    else
        resizing = false;

    QGraphicsItem::mousePressEvent(event);
}

void QClipRectItem::mouseMoveEvent ( QGraphicsSceneMouseEvent *event )
{
    if (!mEnabled)
        return;

    if(resizing)
    {
        // keep inside the scene
        QPointF scenePos = mapToScene(event->pos());

        if ((scenePos.x() < 0) || (scenePos.y() < 0) ||
                (scenePos.x() > scene()->width()) || (scenePos.y() > scene()->height()))
            return;

        qreal XShift = event->pos().x() - event->lastPos().x();
        qreal YShift = event->pos().y() - event->lastPos().y();
        QRectF newRect = rect();
        QPointF newPos = pos();
        //QRectF newRect = boundingRect();
		double Ratio = rect().width()/rect().height();

        switch (mSizeOperation)
        {
            case SizeOperationLeft:
                //newRect.setLeft(rect().left() + XShift);
                newPos.setX(pos().x() + XShift);
                newRect.setWidth(rect().width() - XShift);
                break;
            case SizeOperationRight:
                newRect.setRight(rect().right() + XShift);
                break;
            case SizeOperationTop:
                //newRect.setTop(rect().top() + YShift);
                newPos.setY(pos().y() + YShift);
                newRect.setHeight(rect().height() - YShift);
                break;
            case SizeOperationBottom:
                newRect.setBottom(rect().bottom() + YShift);
                break;
			// PROVA - corners
			case SizeOperationNE:
                newRect.setRight(rect().right() + XShift);
                newPos.setY(pos().y() + YShift);
                newRect.setHeight(rect().height() - YShift);
                break;
			case SizeOperationSE:
                newRect.setRight(rect().right() + XShift);
                newRect.setBottom(rect().bottom() + YShift);
                break;
			case SizeOperationSW:
                newPos.setX(pos().x() + XShift);
                newRect.setWidth(rect().width() - XShift);
                newRect.setBottom(rect().bottom() + YShift);
                break;
			case SizeOperationNW:
                newPos.setX(pos().x() + XShift);
                newRect.setWidth(rect().width() - XShift);
                newPos.setY(pos().y() + YShift);
                newRect.setHeight(rect().height() - YShift);
                break;
        }

		// PROVA - proportional resize
		// 1.1.1.0 - tolto
		/*
		if (newRect.width() != rect().width())
			newRect.setHeight(floor(newRect.width() / Ratio + 0.5));
		else
			newRect.setWidth(floor(newRect.height() * Ratio + 0.5));
		*/

        // prevent to have a negative size
        //if ((newRect.width() < 0) || (newRect.height() < 0))
        //    return;
        // check constraints
		if ((newRect.width() < mMinSize.width()) || (newRect.height() < mMinSize.height())
			|| (newRect.width() > mMaxSize.width()) || (newRect.height() > mMaxSize.height()))
            return;

		// check that item remains inside the scene
        if (!scene()->sceneRect().contains(QRectF(newPos, newRect.size())))
			return;

        prepareGeometryChange();
        setRect(newRect);
        setPos(newPos);
        update();

        //qDebug() << "newRect" << newRect;
    }
    else
    {
        QGraphicsItem::mouseMoveEvent(event);
    }
}

void QClipRectItem::mouseReleaseEvent ( QGraphicsSceneMouseEvent *event )
{
    if (!mEnabled)
	{
		// prova - passo comunque l'evento
	    QGraphicsItem::mouseReleaseEvent(event);
        return;
	}

    resizing = false;

    emit Changed();

    QGraphicsItem::mouseReleaseEvent(event);
}

void QClipRectItem::SetEnabled(bool Enabled)
{
    mEnabled = Enabled;

    // la setFlag funziona solo se la chiamo DOPO
    setFlag(QGraphicsItem::ItemIsMovable, Enabled);
	// PROVA
    //setFlag(QGraphicsItem::ItemIsSelectable, true);

    //setSelected(Enabled); perche' c'era questo?
}

void QClipRectItem::SetSelectable(bool Selectable)
{
    setFlag(QGraphicsItem::ItemIsSelectable, Selectable);
}

void QClipRectItem::SetText(QString Value)
{
    Text = Value;
}

void QClipRectItem::paint ( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget )
{
    if (Text != "")
    {
        //painter->setFont(QFont(painter->font().family(), 11, QFont::Bold));
		// metto il font un po' piu' piccolo
        painter->setFont(QFont(painter->font().family(), 9, QFont::Bold));
        //QPoint p(rect().x() + 2, rect().y() - 2);
		// PROVA - disegno il testo dentro
        QPoint p(rect().x() + 4, rect().y() + 15);

        // draw shadow
        //painter->setPen(Qt::darkGray);
        //painter->drawText(p, Text);

        // draw text
        painter->setPen(Qt::blue);
        p.setX(p.x() - 1);
        p.setY(p.y() - 1);
        painter->drawText(p, Text);
    }

    if (QualityValue)
    {
        QRect QualityRect(rect().topLeft().toPoint(), rect().bottomRight().toPoint());
        QualityRect.setLeft(QualityRect.right()-20);
        QualityRect.setTop(QualityRect.bottom()-20);

        // draw rectangle
        painter->fillRect(QualityRect, (QBrush(QualityColor)));

        // draw value
        QString strValue;
        if ((QualityValue == 254) || (QualityValue == 255))
            strValue = "-";
        else
            strValue.setNum(QualityValue);
        painter->setFont(QFont(painter->font().family(), 8, QFont::Bold));
        painter->setPen(Qt::black);
        painter->drawText(QualityRect, Qt::AlignCenter, strValue);
    }

    if (!DiagIcon.isNull())
    {
        QPointF DiagPoint = rect().bottomLeft();
        DiagPoint.setY(DiagPoint.y() - DiagIcon.height());
        painter->drawPixmap(DiagPoint, DiagIcon);
    }

	if (AmpCode != "")
	{
        QRect AmpCodeRect(rect().topLeft().toPoint(), rect().bottomRight().toPoint());
        //AmpCodeRect.setLeft(AmpCodeRect.right()-100);	// TODO - sarebbe da calcolare la lunghezza del testo...
		AmpCodeRect.setLeft(0);	// tanto l'immagine e' vuota
        AmpCodeRect.setTop(AmpCodeRect.bottom()-20);

        // draw rectangle
		painter->fillRect(AmpCodeRect, (QBrush(Qt::red)));

        // draw value
        painter->setFont(QFont(painter->font().family(), 8, QFont::Bold));
        painter->setPen(Qt::black);
        painter->drawText(AmpCodeRect, Qt::AlignCenter, AmpCode);
	}

    QGraphicsRectItem::paint(painter, option, widget);
}

void QClipRectItem::SetQuality(int Value, QColor Color)
{
    QualityValue = Value;
    QualityColor = Color;
}

void QClipRectItem::SetDiagIcon(QPixmap Icon)
{
    DiagIcon = Icon;

    if (!DiagIcon.isNull())
    {
        // set transparency
        DiagIcon.setMask(DiagIcon.createMaskFromColor(Qt::magenta));
    }
}

void QClipRectItem::SetMinSize(QSize Size)
{
	mMinSize = Size;
}

void QClipRectItem::SetMaxSize(QSize Size)
{
	mMaxSize = Size;
}

// PROVA
void QClipRectItem::SetAmpCode(QString AmpCode)
{
	this->AmpCode = AmpCode;
}
