#ifndef ACQUISITIONFORM_H
#define ACQUISITIONFORM_H

#include <QDialog>
#include <QCloseEvent>
#include "ClipRectItem.h"
#include "GBMSAPI_Library.h"
#include <QtOpenGL>
#include <QGLWidget>
#include "gbmsgui_class.h"
#include "GBFINIMG.h"
#include "FrameItem.h"
#include <QTime>
#include <QTimer>

// forward declarations
struct ScannedObject;
class GBMSGUI;

// 1.7.0.0
// definitions for other hand parts, used in Finger field of SegmentData structure
// start after GBFINIMG_FINGER_LITTLE (5)
#define HAND_PART_INTERDIGITAL			6
#define HAND_PART_FINGER_PHALANGES		7

// data for segments
struct SegmentData
{
    int Finger;
    int Hand;
    int FingerCenterX;   // center of the finger inside image
    int FingerCenterY;
	QString FingerText;
    QRect RealBoundingRect;     // segment bounding box
    QClipRectItem *BoundingRectItem;    // bounding box on Graphics View
    int NFIQQuality;
    int GBQuality;
    QColor QualityColor;  // evaluation of quality
    bool IncompletePatternDetected;
    bool InvalidPatternDetected;
    bool InclinationDetected;
    int UnavailabilityReason;
    bool Found;  // if the procedure found the segment
};

#define SEGM_IMG_MAX_NUM    4

// used by ListenerTimer
struct PedalInformation
{
    int PressedTime;
    int ReleasedTime;
    unsigned char PedalPressed;
    QTime tStart;
    int Condition;
};
// condition to test
enum PedatTestConditions
{
    ReleasePedal = 1,
    PedalPressing = 2
};

namespace Ui {
    class AcquisitionForm;
}

class AcquisitionForm : public QDialog
{
    Q_OBJECT

public:
    //explicit AcquisitionForm(QWidget *parent = 0);
    explicit AcquisitionForm(GBMSGUI *Ref, QWidget *parent = 0);
    ~AcquisitionForm();

    static BOOL __stdcall AcquisitionEventsManagerCallback(
            unsigned int OccurredEventCode,
            int GetFrameErrorCode,
            unsigned int EventInfo,
            unsigned char *FramePtr,
            int FrameSizeX,
            int FrameSizeY,
            double CurrentFrameRate,
            double NominalFrameRate,
            unsigned int GB_Diagnostic,
            void *UserDefinedParameters);
    int Acquire();
    void AcquireEnd();
    bool Finished();
    int GetScanResult();
    void SetForceClose();
    void GetAcquiredImageBuffer(unsigned char *ImageBuff, int *ImgWidth, int *ImgHeight);
    void SetScannedObject(ScannedObject *pObject);
    void SetPersonID(QString Value);
    bool IsAcquiring();
    void SetAcquisitionEventsManagerRet(int Value);
    int GetAcquisitionEventsManagerRet();
    int GetSegmentationResult(int Index, int QualityAlgorithmID,
        bool *SegmentAvailable, int *Left, int *Top, int *Right, int *Bottom,
        int *UnavailabilityReason, int *FingerprintQuality,
        unsigned char *ImagePtr, int *ImageSizeX, int *ImageSizeY);
	void SaveToBmp(QString FileName, unsigned char *Buffer, int SizeX, int SizeY); // DEBUG
	// DEBUG
    static int __stdcall VSRollBeforeFinalizationCallback(unsigned char *Frame, int SizeX, int SizeY, void *Ctx);

public slots:
    BOOL AcquisitionEventsManager(
            unsigned int OccurredEventCode,
            int GetFrameErrorCode,
            unsigned int EventInfo,
            unsigned char *FramePtr,
            int FrameSizeX,
            int FrameSizeY,
            double CurrentFrameRate,
            double NominalFrameRate,
            unsigned int GB_Diagnostic);
    void done(int r);

signals:
    void OnAcquisitionEvent(
            unsigned int OccurredEventCode,
            int GetFrameErrorCode,
            unsigned int EventInfo,
            unsigned char *FramePtr,
            int FrameSizeX,
            int FrameSizeY,
            double CurrentFrameRate,
            double NominalFrameRate,
            unsigned int GB_Diagnostic);

private slots:
    void on_btnAccept_clicked();
    void on_btnStop_clicked();
    void on_btnRepeat_clicked();
    void on_btnBreak_clicked();
    void on_btnHelpDiag_clicked();
    void on_lstDiag_itemClicked(QListWidgetItem *item);
    void on_lstDiag_itemDoubleClicked(QListWidgetItem *item);
    void on_btnSkip_clicked();
    void on_btnBack_clicked();
    void on_btnAdvancedView_toggled(bool checked);
    void PopUpSegmMenu(const QPoint& pos);
    void OnSegmentSelected(int Index);
    void OnViewSegment(int Index);
    void OnRemoveSegment(int Index);
    void OnClearOutside();
    void OnSegmBoxChanged(int Index);
    void OnClipRectChanged();
	void OnListenerTimer();
	void OnIncompleteSlap();

    void on_chkDryFingerImageEnhancement_toggled(bool checked);

private:
    Ui::AcquisitionForm *ui;
    int ScanResult; // form result
    bool Acquiring;
    unsigned char *FramePtr;
    unsigned int ImageSizeX;
	unsigned int ImageSizeY;
	unsigned int PreviewSizeX;
	unsigned int PreviewSizeY;
    int FrameSizeX;
    int FrameSizeY;
    QVector<QRgb> GrayTable;
    QGraphicsScene scene;
    QFrameItem FrameItem;
    QClipRectItem *ClipRectItem;
    QRect RealClipRect;
    bool mForceClose;
    ScannedObject *mScannedObject;
    unsigned int LastDiagnostic;
	unsigned int PrevDiagnostic;
    QString PersonID;
    unsigned int ScanOptions;       // options for current scanning
    bool PreviewPhase;
    bool FullResImage;
    float PreviewScale;
    unsigned char Contrast;
    unsigned char PalmCompleteness;
    GBMSGUI *MSGUIRef;  // reference to GBMSGUI class
    int AcquisitionEventsManagerRet;
    unsigned char *MarkerFrame;
    unsigned char *SegmTempBuff;    // buffer for GBFINIMG
    unsigned int NumSegments;
    bool IncompletePatternDetected;
    bool InvalidPatternDetected;
    bool InclinationDetected;
    bool SegmentationResult;
    bool ProcessImageResult;
	bool SequenceCheckResult;	// 1.7.0.0
    // additional diagnostics for displaying (post-process)
    unsigned int PostProcessDiag;
    unsigned int PrevPostProcessDiag;
    GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR SegmImgDescr[SEGM_IMG_MAX_NUM];
    int SegmSizeX;
    int SegmSizeY;
    QList<SegmentData> SegmData;
    QSignalMapper *FindSegmentSignalMapper;
    QSignalMapper *ViewSegmentSignalMapper;
    QSignalMapper *RemoveSegmentSignalMapper;
    QSignalMapper *SegmBoxChangeSignalMapper;
    QPointF SegmMenuPoint;
	PedalInformation PedalInfo;
	QTimer *ListenerTimer;
	unsigned char *RotationBuffer1000DPI;
	bool ToDoFinalization;
	unsigned int ScanArea;			// 1.6.0.0
    QClipRectItem *StripeRectItem;	// 1.6.0.0
    QRect RealStripeRect;			// 1.6.0.0
	QTime LiveSegmStartTime;		// 1.8.0.0

	// DEBUG - ini settings
	bool DontDrawFrame;
	bool DrawScaled;

    void StartScanning();
    void StopScanning();
    int OnValidFrame(unsigned int EventInfo,
            unsigned char *FramePtr,
            int FrameSizeX,
            int FrameSizeY,
            double CurrentFrameRate,
            double NominalFrameRate,
            unsigned int GB_Diagnostic,
            bool FinalFrame);
    int OnScannerStarted();
    int OnPreviewPhaseEnd();
    int OnAcquisitionEnd(bool AutomaticStop, bool Timeout);
    int OnAcquisitionError(int GetFrameErrorCode);
    void ShowClipRectangle();
    QString GetScannedObjectName();
    void ShowScannedObjectPictogram();
	void BlockAutoCapture();
	void UnblockAutoCapture();
    void DisplayImage();
    void DisplayMessage(QString Message, bool IsError);
    void DisplayDiagnostic();
	void CleanDiagnostic();
	void DeleteDiagItem(QString DiagText);
    void SetScanningOptions();
	void ShowScanningOptions();	// 1.6.0.0
    void SetPreviewScale();
    void ShowFrameRate(double CurrentFrameRate, double NominalFrameRate);
    void AddMessageToList(QString Message, QIcon Icon);
    void RemoveMessageFromList(QString Message);
    void GetRolledInfo();
    int GetNFIQQuality(unsigned char *ImagePtr, int ImageSizeX, int ImageSizeY);
    //void DrawMarkerFrame(QPixmap *origPixmap);
    void DrawMarkerFrame();
    void SetQualityThresholds(int Threshold1, int Threshold2, int Dir);
    bool EvalIAIFSQuality();
    void ResetIAFISQuality();
    void SetArtefactsThresholds(int Threshold1, int Threshold2, int Dir);
    bool EvalRolledArtefactsSize();
    void ResetArtefactsSize();
    bool CheckResult();
    unsigned int SetIndicatorMask(unsigned int ScanObjID);
    void TurnOffLeds();
    bool DoSegmentation(bool ManualMode, bool IncompleteSlap);
    GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO* BuildExtInfo(unsigned int *Num);
    void GetMaxSegmentsSize(int *SizeX, int *SizeY);
    //void ResetSegments();
	// 1.8.0.0
    void ResetSegments(bool PreserveQuality=false);
    void SetSegments();
    bool InitSegmImgDescr();
    void FreeSegmImgDescr();
    int SegmDataIndex(int FingID, int HandID);
    int SegmImgDescrIndex(int FingID, int HandID);
    int GBSegmId2ObjectID(int FingerID, int HandID);
    void ShowBoundingBoxes();
    bool EvalSegmentIAFISQuality(int Index);
    void DisplayGBFinImgError(int ErrCode);
    //bool CheckSequence();
	// 1.7.0.0
	bool CheckSequence(int Finger, int Hand, unsigned char *Img, int ImgSizeX, int ImgSizeY);
    void CutImage(unsigned char* SrcImg, unsigned int SrcSizeX, unsigned int SrcSizeY,
                              unsigned int CutStartX, unsigned int CutStartY,
                              unsigned char* DstImg, unsigned int DstSizeX, unsigned int DstSizeY);
    bool CheckUnavailableFingers();
	bool CheckAllUnavailableFingers();
    bool ProcessImage();
    void CenterClipWithBoxes();
    bool CheckSegmentBoundingBoxPositions();
    bool CheckSegmentBoundingBoxOverlap();
    bool AreRectOverlapping(QRect r1, QRect r2, int Limit);
    void CutBoundingBoxPositions();
    bool AreSegmentsAllFound();
    bool AskUnavailabilityReason();
    void FillImage(unsigned char *ImgBuff, int SizeX, int SizeY,
        int ExcludedLeft, int ExcludedTop, int ExcludedRight, int ExcludedBottom,
        unsigned char FillColor);
    void ClearOutsideBoundingBox();
    int GBQualityToNFIQ(int GBQuality);
	void StartListenPedal(int Condition);
	bool CheckPedal();
	void OnPedalReleased();
	bool OnShortPedalPressing();
	bool OnLongPedalPressing();
	void OnReleasePedal();
	bool CheckLCDButtons();
	void ReadIni();	// DEBUG
	bool LiveSegmentation();
	bool SetDryFingerImageEnhancementOption(bool Option);
	bool JointSegmentation();
	void ShowStripeRectangle();
	void FindInterdigitalArea();
	void FindFingerPhalangesArea();
	void ScanObjIDToFingerHand(unsigned int ObjectID, int *Finger, int *Hand);

protected:
    void closeEvent(QCloseEvent *event);
    void resizeEvent(QResizeEvent *event);
    void showEvent(QShowEvent * event);
    void changeEvent ( QEvent * event );
};

#endif // ACQUISITIONFORM_H
