#ifdef GBMSGUI_EXPORTS
#define GBMSGUI_API __declspec(dllexport)
#else
#define GBMSGUI_API __declspec(dllimport)
#endif

//------------------------------------------------------------------------------
//
// GBMSGUI.DLL is simply a wrapper who offers standard C interface for calling 
// the .Net class library GBMSGUI_NET.DLL.
// This header file serves as documentation for both.
// Note that for acquiring images from Green Bit scanner, also GBMSAPI.dll
// must be used: for enumerating and selecting device, getting device features
// and information about image size for memory allocation.
//
//------------------------------------------------------------------------------

// Return codes for functions
#define GBMSGUI_RET_SUCCESS			0
#define GBMSGUI_RET_FAILURE			1
#define GBMSGUI_RET_SKIP_IMAGE		2
#define GBMSGUI_RET_BREAK_SEQUENCE	3
#define GBMSGUI_RET_BUSY_ACQUIRING	4
#define GBMSGUI_RET_GO_BACK			5

// Scanned object ID's: for constants for scanned object ID's, refer to 
// GBMSAPI_Defines.h, section SCANNABLE OBJECTS


extern "C" {

//------------------------------------------------------------------------------
// GBMSGUI_SetLanguage
//      Set the language for the graphic interface. Call this function before
//      any other GBMSGUI function. If not called, english will be used.
//      Translations are located in external resource dlls, named GBMSGUI_NET.resources.dll,
//      located in subdirectories of the application.
//		Each subdirectory is named as the culture name (e.g. "it" for Italian).
//      If file for requested language is not present, by default english is used.
// Parameters:
//      CultureName: two-letter Culture/Language Name as defined in CultureInfo Class of 
//		the .NET Framework 
//		(see http://msdn.microsoft.com/en-us/library/system.globalization.cultureinfo.aspx)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors 
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetLanguage(const char *CultureName);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_GetVersion
//      The function returns GBMSGUI_NET.DLL version numbers.
// Parameters:
//      Major:		(output) Major version number
//		Minor:		(output) Minor version number
//		Build:		(output) Build number
//		Revision:	(output) Revision number
// Return values:
//      None
//------------------------------------------------------------------------------
GBMSGUI_API void __stdcall GBMSGUI_GetVersion(int *Major, int *Minor, int *Build, int *Revision);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetWindowSizeAndPosition
//      Sets the size and position and state of acquisition window.
//		Call this function before GBMSGUI_StartSession.
//		If not called, by default window is shown in maximized state.
// Parameters:
//      Left, Top: coordinates of the top left corner of the window
//      Width, Height: size of the window
//		Maximized: true if the window is wanted in maximized state
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetWindowSizeAndPosition(int Left, int Top, int Width, int Height, bool Maximized);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetArtefactsThresholds
//      Set the value of the Artefacts Thresholds for rolled acquisition.
//      Rolled artefact size value is dislpayed in user interface at the end
//      of rolling procedure, using the color red, yellow or green.
//      Default thresholds are T1=7 and T2=14, and the evaluation scheme is:
//          Size <=T1	        green (good)
//          Size >T1 and <=T2	yellow (medium)
//          Size >T2	        red    (unacceptable)
//		To have only one threshold, set them both to the same value.
//      In this case, only red and green colors will be used.
//      It is also possible to set both thresholds to -1, in this case no evaluation
//      is performed (no colors, gray is used).
// Parameters:
//		Threshold1, Threshold2:		values of the thresholds, in mm2
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetArtefactsThresholds(int Threshold1, int Threshold2);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetQualityAlgorithm
//      Selects the algorithm to be used for the fingerprint quality assessment
//		in acquisition window. Internally, both value are calculated, and is possible
//		to retrieve passing the relative AlgorithmID to GBMSGUI_GetIAFISQuality
//		and GBMSGUI_GetSegmentationResult.
//      If this function is not called, NFIQ algorithm is used by default.
//      After calling this function, be sure to set appropriate thresholds
//      calling GBMSGUI_SetQualityThresholds and GBMSGUI_SetSegmentQualityThresholds 
//		if GBMSGUI_ACQUISITION_OPTION_SEGMENTATION is active.
// Parameters:
//      QualityAlgorithmID:	ID of the algorithm:
#define GBMSGUI_NFIQ_ALGORITHM	1	// use NFIQ algorithm as defined in NISTIR 7151
#define GBMSGUI_GB_ALGORITHM	2	// use Green Bit fingerprint quality algorithm
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetQualityAlgorithm(int AlgorithmID);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetPatternValidityThreshold
//      Set the value of the Pattern Validity Threshold.
//		Fingerprint pattern validity estimation is used to distinguish a normal fingerprint 
//			image from an image of a glove,rubber, cloth, etc.
//      If the pattern validity value calculated for the fingerprint is less
//      than the threshold, a warning message will be displayed in the user
//      interface.
//		The validity check is performed for all objects (for slaps, applies
//		to segments) except the upper palm, and if 
//		GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN is active.
//		For the lower palm, is performed only if also
//		GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION is active.
//      Default threshold is 75.
// Parameters:
//		Threshold:		value of the threshold (0-100)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetPatternValidityThreshold(int Threshold);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetPatternCompletenessThreshold
//      Set the value of the Pattern Completeness Threshold.
//		Fingerprint pattern completeness estimation is used to detect improper finger 
//			presentation (finger tip, finger side, etc.) by checking presence of main 
//			fingerprint flow components (finger core, basic/distal flows, etc.)
//      If the Pattern Completeness value calculated for the fingerprint is less
//      than the threshold, a warning message will be displayed in the user
//      interface.
//		The completeness check is performed for all objects (for slaps, applies
//		to segments) except the upper palm and the lower palm (for the latter, a 
//		separate evaluation exixts, see GBMSGUI_SetLowerPalmCompletenessThresholds), 
//		and if GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN is active.
//      Default threshold is 85.
// Parameters:
//		Threshold:		value of the threshold (0-100)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetPatternCompletenessThreshold(int Threshold);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetLowerPalmCompletenessThresholds
//      Set the value of the Lower Palm Completeness Threshold.
//		Palm completeness value refers to dimensions of palm central "empty" zone
//		that appears due to the fact that concave palm surface is scanned on flat 
//		scanner surface.
//		If Autocapture is active, procedure does not stops until the Threshold2
//		value is reached.
//      If acquisition is stopped in manual mode, if the pattern completeness value
//		is less than the Threshold2, a warning message will be displayed in the user
//      interface.
//      Default thresholds are Threshold1=80, Threshold2=90.
// Parameters:
//		Threshold1, Threshold2:		values of the thresholds (0-100)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetLowerPalmCompletenessThresholds(int Threshold1, int Threshold2);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetInclinationThreshold
//      Set the value of the inclination Threshold.
//      Fingerprint inclination check is used to prevent improper finger presentation
//		when one or more fingers are significantly inclined in respect to the 
//		vertical axis.
//		The inclination check is performed for all objects (for slaps, applies
//		to segments) except the upper palm, and if GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION
//		is active.
//      Default threshold is 35.
// Parameters:
//		Threshold:		value of the threshold, in degrees ranging from 0 to 55
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetInclinationThreshold(int Threshold);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetBlockAutoCaptureMask
//      Set the value of the BlockAutoCaptureMask.
//		BlockAutoCaptureMask describes conditions that prevent execution of auto-capture.
//      Put in this mask diagnostic bits (see GBMSAPI_Defines.h, section "DIAGNOSTIC MESSAGES") 
//		that must block autocapture (takes effect if GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE
//		option in GBMSGUI_Acquire is active).
//      If not called, by default value is:
//			GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA |
//			GBMSAPI_DM_FLAT_FINGER_SLIDING | 
//			GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_LEFT |
//			GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_RIGHT | 
//			GBMSAPI_DM_FLAT_FINGER_OUT_OF_REGION_TOP |
//			GBMSAPI_DM_FLAT_FINGER_DISPLACED_DOWN |
//			GBMSAPI_DM_DRY_FINGER |
//			GBMSAPI_DM_WET_FINGER
// Parameters:
//		Mask:		value of the mask
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetBlockAutoCaptureMask(unsigned int Mask);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetBlockAutoCaptureContrast
//      Set if the value of the contrast affects the autocapture behavior.
//		If set to true, a contrast value lower than Threshold2 (see GBMSGUI_SetContrastThresholds)
//		blocks the autocapture until the value is reached.
//		Takes effect only if GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE
//		option in GBMSGUI_Acquire is active.
//		If not called, by default is false.
// Parameters:
//		Flag:		true or false
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetBlockAutoCaptureContrast(bool Flag);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetIgnoreDiagnosticMask
//      Set diagnostic bit that should be ignored (not displayed in user interface).
//      The following diagnostics can't be masked:
//          GBMSAPI_DM_SCANNER_SURFACE_NOT_NORMA
//          GBMSAPI_DM_SCANNER_FAILURE
// Parameters:
//      IgnoreDiagnosticMask: value of the mask
// Return values:
//      GBMSGUI_SUCCESS - no errors
//      GBMSGUI_FAILURE - some errors occured, call GBMSGUI_GetLastError()
//                      or GBMSGUI_GetLastErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetIgnoreDiagnosticMask(
    unsigned int IgnoreDiagnosticMask);
//------------------------------------------------------------------------------





//------------------------------------------------------------------------------
// GBMSGUI_SetFlatFingerOnRollArea
//      Set this value to true to allow acquisition of flat finger on roll area,
//      on scanner that supports (MultiScan500, VisaScan3, DactyScan84).
//      Call this function before GBMSGUI_GetImageSize because this affects
//      image size.
//      Note that autocapture feature is not available for flat fingers on
//      roll area.
// Parameters:
//      FlatFingerOnRollArea: true or false
// Return values:
//      GBMSGUI_SUCCESS - no errors
//      GBMSGUI_FAILURE - some errors occured, call GBMSGUI_GetLastError()
//                      or GBMSGUI_GetLastErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetFlatFingerOnRollArea(
    bool FlatFingerOnRollArea);
//------------------------------------------------------------------------------





//------------------------------------------------------------------------------
// GBMSGUI_StartSession
//      The function starts an acquisition session for a person. Within a session
//      all required images of fingers/slaps/palms must be acquired by sequential
//      calls of GBMSGUI_Acquire. Call GBMSGUI_StopSession to close the session.
// Parameters:
//      AcquisitionMode:
#define GBMSGUI_AM_SINGLE_ACQUISITION	0    // Acquisition window is
//                      shown and hidden within every call of GBMSGUI_Acquire.
#define GBMSGUI_AM_MULTIPLE_ACQUISITION	1    // Acquisition window is always
//                      visible during session.
//
//      PersonID:       Pointer to null-terminated string containing Person ID
//
//      Options:        describes optional operations.
#define GBMSGUI_SESSION_OPTION_SEQUENCE_CHECK				0x00000001 // set this bit to perform
//                      sequence check of acquired fingerprints. In sequence check,
//                      rolled fingerprints are compared against corresponding
//                      fingerprints extracted from slaps: so the acquisition
//                      sequence must respect this order: first slaps, then rolled.
//						To proper functionality of this option it is required that
//						GBMSGUI_ACQUISITION_OPTION_SEGMENTATION is specified in GBMSGUI_Acquire
//						for slap acquisitions; otherwise, a warning message is displayed in the
//						acquisition window
//                      If the option is requested and the order is not respected
//                      (roll requested first), a warning message is displayed in the
//						acquisition window.
#define GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON	0x00000002 // set this bit to
//						force the operator to specify the reason of finger/slap/palm
//						unavailability when a finger/slap/palm acquisition is skipped,
//					    or a segment in a slap is not assigned (see also
//                      GBMSGUI_GetUnavailabilityReason or GBMSGUI_GetSegmentationResult)
#define GBMSGUI_SESSION_OPTION_NO_ADVANCED_VIEW 			 0x00000004  // set this to hide
//                      the "Advanced View" button in acquisition window
#define GBMSGUI_SESSION_OPTION_DRY_FINGER_IMAGE_ENHANCEMENT	0x00000008	// set this to enable 
//						enhancement of dry finger image; this option can be activated also 
//						directly in the user interface wiht a checkbox; not all scanners support this option, 
//						see GBMSAPI function GBMSAPI_GetDeviceFeatures, option GBMSAPI_DF_DRY_SKIN_IMG_ENHANCE;
//						if the option is not supported, the checkbox in the user interface will appear grayed.
//
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//      GBMSGUI_RET_BUSY_ACQUIRING - an acquisition is still in progress
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_StartSession(int AcquisitionMode, wchar_t *PersonID, unsigned int Options);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetImageSize
//      Set the size of the final image produced by GBMSGUI_Acquire function
//      for the given Scanned Object ID.
//		Call this function after GBMSGUI_StartSession, before GBMSGUI_Acquire.
//		If this function is not called, sizes defined in ANSI/NIST ITL 1-2007
//		standard are used by default:
//          Palms (upper and lower)				2500 x 2500 pixel (5000 x 5000 at 1000 Dpi)
//			Writer palms						900 x 2500 pixel (1800 x 5000 at 1000 Dpi)
//          Slaps (four fingers and two thumbs) 1600 x 1500 pixel (3200 x 3000 at 1000 Dpi) 
//          Flat thumbs (single)                500 x 1000 pixel (1000 x 2000 at 1000 Dpi)
//          Rolled finger                       800 x 750 pixel (1600 x 1500 at 1000 Dpi)
//      For objects not defined in standard (flat index, middle, ring and little)
//      a default size of 500x500 pixel (1000 x 1000 at 1000 Dpi) is used.
//		Values are initialized internally to default each time GBMSGUI_StartSession is called,
//		according to the resolution of the currently selected scanner.
//      If the specified size is smaller than the maximum scanning area, 
//		a blue frame will be shown over the image in the user interface 
//		automatically positioned following the actual position of fingerprint 
//		within the original image.
//      Minimal possible image size is 200x200 (400 x 400 at 1000 Dpi).
//		REMARK: because image size are passed in pixel, be careful to adapt image
//		size to the resolution of currently selected scanner.
//		To set the size of fingerprint segment images, call GBMSGUI_SetSegmentImageSize
//		instead.
// Parameters:
//      ScannedObjectID:	ID of the object
//		Width, Height:		size of the image in pixel
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetImageSize(unsigned int ScannedObjectID, unsigned int Width, unsigned int Height);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetContrastThresholds
//      Set the value of the thresholds for contrast evaluation for the given
//      Scanned Object ID.
//		Contrast is dislpayed in user interface during scanning phase using the
//      color red, yellow or green, to help the operator to achieve better image
//      quality.
//		Call this function after GBMSGUI_StartSession, before GBMSGUI_Acquire.
//      If this function is not called, no evaluation is performed.
//      Values range is from 0 to 255 and the evaluation scheme is:
//          Contrast < T1 				Red    (unacceptable)
//          Contrast >= T1 and < Th2 	Yellow (medium)
//          Contrast >= T2 			    Green  (good)
//		To have only one threshold set them both to the same value.
//		If not called, dafult thresholds are 120, 150.
// Parameters:
//      ScannedObjectID:	        ID of the object
//		Threshold1, Threshold2:		values of the thresholds, ranged from 0 to 255
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetContrastThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetQualityThresholds
//      Set the values of the thresholds for the fingerprint quality assessment
//      for the given Scanned Object ID.
//		Quality evaluation is performed only for single finger images, lower palms
//		and writer's palms (not for slaps and upper palms).
//		The range of values and evaluation scheme depend on the fingerprint quality
//      algorithm selected (see GBMSGUI_SetQualityAlgorithm function).
//      Fingerprint quality value is dislpayed in user interface using the color 
//		red, yellow or green.
//
//      For GBMSGUI_NFIQ_ALGORITHM, the value range is from 1 (best quality)
//      to 5 (worst quality) and the evaluation scheme is:
//          Quality <=T1	        green  (good)
//          Quality >T1 and <=T2	yellow (medium)
//          Quality > T2	        red    (unacceptable)
//      Default thresholds are T1=2 and T2=3
//
//      For GBMSGUI_GB_ALGORITHM algorithm the value range is from 1 (worst quality) to
//      100 (best quality) and the evaluation scheme is:
//          Quality <=T1	        red    (unacceptable)
//          Quality >T1 and <=T2	yellow (medium)
//          Quality > T2            green  (good)
//      Recommended thresholds are T1=50 and T2=70
//
//		To work with only one threshold, set them both to the same value.
//      In this case, only red and green colors will be used.
//		Call this function after GBMSGUI_StartSession, before GBMSGUI_Acquire.
// Parameters:
//      ScannedObjectID:	        ID of the object
//		Threshold1, Threshold2:		values of the thresholds
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetQualityThresholds(unsigned int ScannedObjectID, int Threshold1, int Threshold2);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetSegmentImageSize
//      Sets size of finger segments.
//		This function affects only acquisition of slap images (four fingers and
//		two thumbs) and only if GBMSGUI_ACQUISITION_OPTION_SEGMENTATION is active.
//		Call this function before GBMSGUI_Acquire of the affected slap.
//		If not called, default size for segments is 500x500 pixel (1000 x 1000
//		at 1000 Dpi).
// Parameters:
//      ScannedObjectID:    identifier of the slap object
//      Index of finger, as follows:
//          for four (or two) fingers acquisition:
//              0 - index
//              1 - middle
//              2 - ring
//              3 - little
//          for two thumbs (or two indexes) acquisition:
//              0 - left
//              1 - right
//		Width, Height: image size, in pixel; 
//		REMARK: because image size are passed in pixel, be careful to adapt image
//		size to the resolution of currently selected scanner.
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetSegmentImageSize(unsigned int ScannedObjectID, int Index, unsigned int Width, unsigned int Height);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetSegmentQualityThresholds
//      Sets threshold for quality evaluation on finger segments.
//		This function affects only acquisition of slap images (four fingers and
//		two thumbs) and only if GBMSGUI_ACQUISITION_OPTION_SEGMENTATION is active.
//		The range of values and evaluation scheme depend on the fingerprint quality
//      algorithm selected (see GBMSGUI_SetQualityAlgorithm function).
//		Call this function before GBMSGUI_Acquire of the affected slap.
//		If not called, default threshold are 2 and 3.
// Parameters:
//      ScannedObjectID:    identifier of the slap object
//      Index of finger, as follows:
//          for four (or two) fingers acquisition:
//              0 - index
//              1 - middle
//              2 - ring
//              3 - little
//          for two thumbs (or two indexes) acquisition:
//              0 - left
//              1 - right
//		Threshold1, Threshold2: threshod values
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetSegmentQualityThresholds(unsigned int ScannedObjectID, int Index, int Threshold1, int Threshold2);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_SetSegmentUnavailabilityReason
//      Sets unavailability of a specific finger before the acquisition.
//		This function affects only acquisition of slap images (four fingers and
//		two thumbs) and only if GBMSGUI_ACQUISITION_OPTION_SEGMENTATION is active.
//		Call this function before GBMSGUI_Acquire of the affected slap.
//      Calling this funcion will mark specified segment as unavailable, and
//      will force segmentation to be performed in manual way (operator must point
//      the finger segments inside the image).
// Parameters:
//      ScannedObjectID:    identifier of the slap object
//      Index of finger, as follows:
//          for four (or two) fingers acquisition:
//              0 - index
//              1 - middle
//              2 - ring
//              3 - little
//          for two thumbs (or two indexes) acquisition:
//              0 - left
//              1 - right
//		Reason: reason of the unavailability:
#define GBMSGUI_UR_AMPUTATED	1    // finger is missing
#define GBMSGUI_UR_UNPRINTABLE	2	 // other reasons (e.g. bandaged)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetSegmentUnavailabilityReason(unsigned int ScannedObjectID, int Index, int Reason);
//------------------------------------------------------------------------------




//------------------------------------------------------------------------------
// SelectFingerContactEvaluationMode
//      Set this value to select which method of finger contact evaluation
//		show in user interface; note that not all scanners supports dry/wet warning.
//		For scanner that supports it, GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING
//		is used by default.
//      Call this function after GBMSGUI_StartSession.
// Parameters:
//      Mode: one of the following values:
#define GBMSGUI_FINGER_CONTACT_EVAL_MODE_DRY_WET_WARNING	1	// use dry/wet warning
#define GBMSGUI_FINGER_CONTACT_EVAL_MODE_CONTRAST			2	// use contrast value evaluation
// Return values:
//      GBMSGUI_SUCCESS - no errors
//      GBMSGUI_FAILURE - some errors occured, call GBMSGUI_GetLastError()
//                      or GBMSGUI_GetLastErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SelectFingerContactEvaluationMode(int Mode);
//------------------------------------------------------------------------------


// 1.12.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetAdaptiveRollDirection
//		Call this function if you want to force the roll direction in a 
//		specific direction, when GBMSGUI_ACQUISITION_OPTION_ADAPT_ROLL_AREA_POSITION
//		is set.
//		If not called, GBMSGUI_ADAPTIVE_ROLL_DIRECTION_TO_CENTER is the default.
//		For more details, see "ADAPTIVE ROLL AREA POSITIONING SYSTEM" in GBMSAPI_Defines.h.
//      Call this function after GBMSGUI_StartSession.
// Parameters:
//      Direction: one of the following values:
#define GBMSGUI_ADAPTIVE_ROLL_DIRECTION_TO_CENTER	1	// default
#define GBMSGUI_ADAPTIVE_ROLL_DIRECTION_TO_LEFT		2
#define GBMSGUI_ADAPTIVE_ROLL_DIRECTION_TO_RIGHT	3
// Return values:
//      GBMSGUI_SUCCESS - no errors
//      GBMSGUI_FAILURE - some errors occured, call GBMSGUI_GetLastError()
//                      or GBMSGUI_GetLastErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetAdaptiveRollDirection(int Direction);
//------------------------------------------------------------------------------


// 1.12.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetRollArea
//		Call this function to select area where to acquire rolled fingeprint,
//		for scanner supporting both IQS and GA roll area.
//		Check which area the current scanner support calling 
//		GBMSAPI_GetSupportedScanAreas function (see GBMSAPI_Library.h)
//      Call this function after GBMSGUI_StartSession.
// Parameters:
//      RollArea: one of the following values:
#define GBMSGUI_ROLL_AREA_IQS	1
#define GBMSGUI_ROLL_AREA_GA	2
// Return values:
//      GBMSGUI_SUCCESS - no errors
//      GBMSGUI_FAILURE - some errors occured, call GBMSGUI_GetLastError()
//                      or GBMSGUI_GetLastErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetRollArea(int RollArea);
//------------------------------------------------------------------------------


// 1.13.0.0
//------------------------------------------------------------------------------
// GBMSGUI_EnableBlockAutocaptureLedColorFeedback
//		Enables special leds behavior with block/unblock autocapture (for 
//		scanners with leds interface).
//		By configuring appropriately the conditions that fires the BlockAutocapture
//		(see GBMSGUI_SetBlockAutoCaptureMask), the led can inform about the proper
//		finger placement on the scanner.
//		By default this is the behavior of the scanned object related led:
//			- when acquisition starts, led is turned on with fixed yellow light
//			- if BlockAutocapture fires, led blinks (always yellow)
//			- when BlockAutocapture conditions disappear, led stop blinking
//		Calling this function, the led behavior with BlockAutocapture changes in the following way:
//			- when acquisition starts, led is turned on with fixed yellow light
//			- if no BlockAutocapture conditions are met, led blinks green
//			- if BlockAutocapture fires, led blinks red
//			- if object is removed from the scanner, led turns back to yellow fixed
//		1.14.0.0 - NOTE: this setting is IGNORED if GBMSGUI_ACQUISITION_OPTION_LIVE_SEGM_EVAL
//						is active.
// Parameters:
//      Enable: 1=enable; 0=disable
// Return values:
//      GBMSGUI_SUCCESS - no errors
//      GBMSGUI_FAILURE - some errors occured, call GBMSGUI_GetLastError()
//                      or GBMSGUI_GetLastErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_EnableBlockAutocaptureLedColorFeedback(int Enable);
//------------------------------------------------------------------------------


// 1.13.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetUpperLowerPalmIdentityCheckThreshold
//		Set the minimum value of the matching score for the identity check 
//		of interdigital area between upper and lower palm.
//		Identity check is performed if GBMSGUI_SESSION_OPTION_SEQUENCE_CHECK is active.
//		If not called, default value is 12.
//		Setting to 0 equals to no check.
// Parameters:
//		Threshold:		value of the threshold, ranged from 0 to 200.
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetUpperLowerPalmIdentityCheckThreshold(int Threshold);
//------------------------------------------------------------------------------



// 1.13.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetPalmInclinationThreshold
//      Set the value of the inclination Threshold for palm objects (upper, lower
//		and writers palm) and rolled hypothenar.
//      Inclination check is used to prevent improper palm presentation
//		when the palm image is significantly inclined in respect to the 
//		vertical axis.
//		The inclination check is performed if GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION
//		is active.
//      Default threshold is 15.
// Parameters:
//		Threshold:		value of the threshold, in degrees ranging from 0 to 55
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetPalmInclinationThreshold(int Threshold);
//------------------------------------------------------------------------------



// 1.13.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetInterdigitalCompletenessThreshold
//      Set the value of the completeness threshold for the interdigital area of
//		upper and lower palm.
//      Default threshold is 75.
// Parameters:
//		Threshold:		value of the threshold (0-100)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetInterdigitalCompletenessThreshold(int Threshold);



// 1.13.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetFingerPhalangesCompletenessThreshold
//      Set the value of the completeness threshold for the area of medial and
//		proximal finger phalanges of upper palm.
//      Default threshold is 75.
// Parameters:
//		Threshold:		value of the threshold (0-100)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetFingerPhalangesCompletenessThreshold(int Threshold);
//------------------------------------------------------------------------------

// 1.13.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetInterdigitalQualityThresholds
//      Set the values of the thresholds for the quality assessment for the
//      interdigital area of upper and lower palm.
//		See GBMSGUI_SetQualityThresholds for explanation of how thresholds
//		are intepreted.
// Parameters:
//		Threshold1, Threshold2:		values of the thresholds
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetInterdigitalQualityThresholds(int Threshold1, int Threshold2);
//------------------------------------------------------------------------------

// 1.13.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetFingerPhalangesQualityThresholds
//      Set the values of the thresholds for the quality assessment for the 
//		area of medial and proximal finger phalanges of upper palm.
//		See GBMSGUI_SetQualityThresholds for explanation of how thresholds
//		are intepreted.
// Parameters:
//		Threshold1, Threshold2:		values of the thresholds
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetFingerPhalangesQualityThresholds(int Threshold1, int Threshold2);
//------------------------------------------------------------------------------


// 1.14.0.0
//------------------------------------------------------------------------------
// GBMSGUI_SetLiveSegmEvalTimeout
//		Set the timeout for Live Segments Quality Evaluation, when GBMSGUI_ACQUISITION_OPTION_LIVE_SEGM_EVAL
//		option is active.
//		If no good quality can be achieved for the available segments, when
//		timeout expires the image is acquired anyway (BlockAutocapture is no more called).
//		Setting timeout value to 0 means no timeout: if no good quality can be achieved for the segments,
//		autocapture will not fire, and acquisition can be only stopped manually by operator.
// Parameters:
//		Timeout:		timeout, in seconds; default is 7;
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_SetLiveSegmEvalTimeout(int Timeout);
//------------------------------------------------------------------------------



typedef bool (FAR PASCAL *GBMSGUI_AfterStartFuncPtr)(void *AfterStartParam);

//------------------------------------------------------------------------------
// GBMSGUI_SetAfterStartCallback
//		Sets a pointer to a function to be called in acquisition procedure after 
//		the scanning process is started (see GBMSAPI_AE_SCANNER_STARTED in GBMSAPI_Library.h).
//		Developer can use this function to call GBMSAPI functions
//      GBMSAPI_ROLL_SetPreviewTimeout, GBMSAPI_SetDiagnosticSensitivity, 
//		GBMSAPI_ROLL_SetArtefactCleaningDepth and GBMSAPI_ROLL_SetDiagnosticSensitivity.
//      If the function returns true, the acquisition process will continue,
//      otherwise the process will be interrupted and GBMSGUI_RET_FAILURE returned.
//		Pass NULL to clear the setting.
// Parameters:
//		AfterStartFunc: pointer to a function of type GBMSGUI_AfterStartFuncPtr
//		AfterStartParam: parameter that will be passed to AfterStartFunc
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API void __stdcall GBMSGUI_SetAfterStartCallback(GBMSGUI_AfterStartFuncPtr AfterStartFunc,
    void *AfterStartParam);
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// GBMSGUI_Acquire
//      This function is called to produce final image corresponding to
//      required Scanned Object ID within an acquisition session opened by
//      GBMSGUI_StartSession() function.
//      Acquisition window displays image flow from scanner in real time
//		(for rolled fingers, a composite fingerprint image is displayed).
//      An operator can in real-time evaluate scanned image and the corresponding
//      image evaluation parameters.
//      If GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE option is activated, acquisition
//      is stopped automatically when the best image contrast and fingerprint size
//      are achieved (for lower palm, also palm completeness value must reach the
//		threshold set); for rolled fingerprints, acquisition process is stopped
//      automatically when the finger is removed.
//      Acquisition process can always be stopped by Stop button on user interface
//		or on LCD touchscreen (if present) or by pressing pedal (if present);
//		for rolled fingerprint acquisition, stopping the acquisition before the
//      automatic stop will result in an error.
//      After acquisition is stopped, an operator should evaluate final image, the
//      corresponding image quality value (for rolled fingerprints also rolling
//      artefact area should be evaluated, and completeness for lower palm) 
//		and possible diagnostic messages.
//      To return final image to the calling application, the user can:
//			- press "Accept" button on user interface or on LCD (if present)
//			- press pedal shortly (if present)
//      (if GBMSGUI_ACQUISITION_OPTION_AUTOACCEPT option is active, this is done 
//      automatically - see GBMSGUI_ACQUISITION_OPTION_AUTOACCEPT description below).
//      "Accept" button is disabled in some conditions:
//          - rolled acquisition interrupted before the automatic stop
//          - no fingerprint detected on scanner surface after manual stop
//            (Contrast = 0)
//          - slap segmentation failed in automatic mode (e.g. if some finger
//            is missing)
//		When "Accept" button is clicked, if segmentation of slap image is active
//		(GBMSGUI_ACQUISITION_OPTION_SEGMENTATION is specified) and 
//		GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON is active for session, 
//		completeness check is performed - all finger segments are found or their 
//		unavailability reason is specified.
//      To repeat the acquisition, the user can:
//			- press Repeat button on user interface or on LCD (if present)
//			- keep the pedal pressed for more than 2 seconds (if present)
//      Press Skip for skipping the current object acquisition and passing to
//      the next in the sequence. If Skip is clicked, the function returns
//      GBMSGUI_RET_SKIP_IMAGE (see return code description).
//		Press "Break sequence"/Cancel to cancel acquisition of a person (see
//      return code GBMSGUI_RET_BREAK_SEQUENCE).
//
// Parameters:
//      ScannedObjectID:    identifier of object to be scanned
//      Options:            describes optional operations
#define GBMSGUI_ACQUISITION_OPTION_SEGMENTATION						0x00000001  // set this bit to enable
//                          segmentation of slap images containing 2/4 fingers
//                          simultaneously, or of individual phalanges of jonts.
//							For Joints, only bounding boxes are provided (no images), 
//							and for this BETA version is not yet fully automatic (needed manual adjustment by operator).
//							This option is valid only for slap and joints objects and ignored for all others.
//                          Call GBMSGUI_GetSegmentationResult after GBMSGUI_Acquire
//                          to get the results of segmentation
#define GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE						0x00000002  // set this bit to
//							activate autocapture. This option is valid only for
//							flat objects aquisition, for rolled print autocapture
//                          is always active.
#define GBMSGUI_ACQUISITION_OPTION_SOUND							0x00000004  // set this bit to
//                          enable sound. Device or PC can produce sound in the following events:
//		                            - on acquisition end
//		                            - on "repeat" and "accept" events
//		                            - on error messages in message area
//							If current scanner supports sound (DactyScan84), the device will sound,
//							otherwise, the PC will.
#define GBMSGUI_ACQUISITION_OPTION_AUTOACCEPT						0x00000008  // set this bit to allow
//                          an acquisition to be automatically accepted (no
//                          operator intervention) if all requirements are
//                          satisfied:
//                              (all objects) - no error conditions or diagnostic appears
//                              (single finger, flat or rolled) - fingerprint quality
//                                      is good (green) (depends on the algorithm,
//                                      for NFIQ Quality <= T1)
//                              (rolled) - artefacts size is less or equal than T1 (green)
//										  and sequence check is positive (if active).
//                              (slaps) - automatic segmentation is completed
//                                        successfully and single segments quality
//                                        is good (green) (depends on the algorithm,
//                                        for NFIQ Quality <= T1)
//								(lower palm) - palm completeness is good (green)
//								Note: If image quality assessment is not active,
//                                    image quality conditions are believed unconditionally
//                                    satisfied. So, it is recommended to activate
//                                    image quality assessment (that is, specify thresholds) when
//                                    GBMSGUI_ACQUISITION_OPTION_AUTOACCEPT option is active to
//                                    prevent automatic acceptance of bad quality images.
#define GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE				0x00000010  // set this bit to block
//                          autocapture in case when a diagnostic message appears
//                          due to improper user behaviour (finger on scanner before scanning
//                          starts, finger displaced, finger sliding) (see also
//							GBMSGUI_SetBlockAutoCaptureMask) or in case of low contrast images
//							(see GBMSGUI_SetBlockAutoCaptureContrast). This option
//                          is valid only if GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE is also active.
#define GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT				0x00000020  // set this bit to enable
//                          "halo effect" and residual latent prints removal on the acquired image
#define GBMSGUI_ACQUISITION_OPTION_DETECT_INVALID_PATTERN			0x00000040  // set this bit to enable
//                          pattern validity estimation (see also GBMSGUI_SetPatternValidityThreshold)
#define GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN		0x00000080  // set this bit to enable
//                          pattern completeness estimation (see also GBMSGUI_SetPatternCompletenessThreshold)
#define GBMSGUI_ACQUISITION_OPTION_DETECT_INCLINATION				0x00000100  // set this bit to enable
//                          significant finger inclination detection (see also GBMSGUI_SetInclinationThreshold)
#define GBMSGUI_ACQUISITION_OPTION_ROTATE_FINGER					0x00000200  // set this bit to rotate
//                          inclined fingeprints to vertical position; this option affetcs only
//							segments of slaps.
#define GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW					0x00000400  // corresponds
//							to GBMSAPI_AO_FULL_RES_PREVIEW option (see GBMSAPI_Defines.h)
#define GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_MANUAL_STOP			0x00000800  // corresponds
//							to GBMSAPI_AO_MANUAL_ROLL_PREVIEW_STOP option (see GBMSAPI_Defines.h)
#define GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_TYPE				0x00001000  // corresponds
//							to GBMSAPI_AO_ROLL_PREVIEW_TYPE option (see GBMSAPI_Defines.h);
//							is valid only if GBMSGUI_ACQUISITION_OPTION_ROLL_PREVIEW_MANUAL_STOP is not set.
#define GBMSGUI_ACQUISITION_OPTION_PALM_PRINT_QUALITY_CALCULATION	0x00002000  // set this to enable
//							quality calculation on palm prints (requires long time, about 3-7 sec.)
#define GBMSGUI_ACQUISITION_OPTION_SOUND_ON_ROLL_PREVIEW_END		0x00004000  // set this to enable
//							sound to notify roll preview end; recommended to be set
#define GBMSGUI_ACQUISITION_OPTION_NO_ROLL_PREVIEW					0x00008000  // set this to disable
//							roll preview phase
#define GBMSGUI_ACQUISITION_OPTION_NO_ARTEFACTS_DISPLAY				0x00010000  // set this to disable
//                          drawing of artefacts area over the image
#define GBMSGUI_ACQUISITION_OPTION_HIGH_SPEED_PREVIEW				0x00020000  // corresponds
//							to GBMSAPI_AO_HIGH_SPEED_PREVIEW option (see GBMSAPI_Defines.h)
#define GBMSGUI_ACQUISITION_OPTION_AUTO_CLEAR_OUTSIDE_ROLL			0x00040000  // set this to enable
//							automatic clear of area outside the bounding box of the rolled finger (remove foreing objects)
#define GBMSGUI_ACQUISITION_OPTION_SEGM_QUALITY_EVAL				0x00080000  // Set this to enable
//							segmentation during preview, with single segments quality evaluation. 
//							This option is valid only if GBMSGUI_ACQUISITION_OPTION_SEGMENTATION and 
//							GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW are active. 
//							If GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE is active and supported, 
//							autocapture is blocked until all segments quality is good.
//							1.14.0.0 - This option is NO MORE ACTIVE, replaced by GBMSGUI_ACQUISITION_OPTION_LIVE_SEGM_EVAL
#define GBMSGUI_ACQUISITION_OPTION_AUTO_INCOMPLETE_SLAP_SEGM		0x00100000  // Set this to enable
//							automatic attempt to perform segmentation of incomplete slaps.
//							If this option is set, system will not check left-right hand mix and will rely completely 
//     						on the information about hand passed by application. So, application/human operator should ensure
//	   						appropriate hand applying for correct result.
//							Even if this option is not set, when segmentation fails, it's always possible to try 
//							automatic segmentation from the context menu right-clicking on the image.
#define GBMSGUI_ACQUISITION_OPTION_ADAPT_ROLL_AREA_POSITION			0x00200000	// 1.12.0.0
//							this option activates the "ADAPTIVE ROLL AREA POSITIONING SYSTEM" (see GBMSAPI_defines.h)
#define GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_ACQUISITION	0x00400000	// 1.12.0.0
//							this option activates Roll Stripe Acquisition (see GBMSAPI_ROLL_EnableRollStripeAcquisition GBMSAPI_library.h)
#define GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_FEEDBACK		0x00800000	// 1.12.0.0
//							this option enable the visual feedback (a rectangle showing the current stripe area acquired) when Roll Stripe Acquisition is active;
//							1.12.1.0 - this option is now ignored, stripe rectangle is always shown if
//							GBMSGUI_ACQUISITION_OPTION_ENABLE_ROLL_STRIPE_ACQUISITION is active;
#define GBMSGUI_ACQUISITION_OPTION_LIVE_SEGM_EVAL					0x01000000	// 1.8.0.0
//							this option checks that all fingerprint segments found in image are complete and of good quality,
//                          and led feedback is provided (if supported by the scanner); if GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE
//                          is active, autocapture is blocked if segments are incomplete or of bad quality;
//                          this option automatically activates also: GBMSGUI_ACQUISITION_OPTION_SEGMENTATION,
//                          GBMSGUI_ACQUISITION_OPTION_FULL_RES_PREVIEW and GBMSGUI_ACQUISITION_OPTION_DETECT_INCOMPLETE_PATTERN;
//                          if GBMSGUI_ACQUISITION_OPTION_AUTOCAPTURE is active, GBMSGUI_ACQUISITION_OPTION_BLOCK_AUTOCAPTURE is activated automatically and 
//                          autocapture is blocked if segments are incomplete or of bad quality;
//                          this means that live segments evaluation can be used in both autocapture or manual stop mode;
//                          this option is valid only for slap objects types (GBMSAPI_SBT_FLAT_SLAP_4, GBMSAPI_SBT_FLAT_THUMBS_2, GBMSAPI_SBT_FLAT_SLAP_2);
#define GBMSGUI_ACQUISITION_OPTION_DELETE_PALM_FINGER_SEGMENTS		0x02000000	// 1.14.0.0
//							Deletes eventual fingerprint segments found around palm-prints; valid only if GBMSGUI_ACQUISITION_OPTION_REMOVE_HALO_LATENT
//							is also active, and only for palm objects (except upper palm); leaves thumbs untouched;
//      Image:				pointer to buffer where to store acquired image in raw format
//							(8 bits greyscale); must be already allocated.
//		ImageBufferSize:	size in bytes of the allocated Image buffer
//		ImageSizeX:			(output) width in pixel of the acquired image
//		ImageSizeY:			(output) height in pixel of the acquired image
//		Dpi:				(output) resolution of the acquired image (can be 500 or 1000)
// Return values:
//      GBMSGUI_RET_SUCCESS - image is successfully generated
//      GBMSGUI_RET_FAILURE - image is not generated, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//      GBMSGUI_RET_SKIP_IMAGE - image is not generated. "Skip" button was pressed
//							by operator. If GBMSGUI_StartSession was
//                          called with the GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON
//                          option, a dialog box will prompt the operator to
//                          specifiy the reason of the finger(s) unavailability.
//							To get finger unavailability reason application should:
//								- call GBMSGUI_GetSegmentationResult for slap
//								- call GBMSGUI_GetUnavailabilityReason for others
//      GBMSGUI_RET_BREAK_SEQUENCE - image is not generated. "Break sequence"/"Cancel"
//                            button was pressed by operator; application must
//                            call GBMSGUI_StopSession
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_Acquire(unsigned int ScannedObjectID, unsigned int Options, 
										  unsigned char *Image, unsigned long ImageBufferSize, 
										  int *ImageSizeX, int *ImageSizeY, int *Dpi);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_GetIAFISQuality
//      Returns quality of acquired fingerprint/palm image, according to the
//		algorithm passed as parameter (see GBMSGUI_SetQualityAlgorithm).
//      Call this function after GBMSGUI_Acquire only for acquisition
//		of single fingerprint (flat or rolled), lower and writer's palm.
//		To get fingerprint quality of segmented images from slap captures, call
//		GBMSGUI_GetSegmentationResult instead.
// Parameters:
//		ScannedObjectID:	ID of the object
//		QualityAlgorithmID: ID of the quality algorithm
//		Quality:			(output) value of fingerprint image quality.
//							A value of 254 indicates that no quality was ever computed
//							A value of 255 indicate a failed attempt to calculate the quality
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_GetIAFISQuality(unsigned int ScannedObjectID, int QualityAlgorithm, int *Quality);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_GetUnavailabilityReason
//      Returns the reason of a finger/palm unavailability.
//      Call this function after GBMSGUI_Acquire if it returns GBMSGUI_RET_SKIP_IMAGE.
//      Option GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON in GBMSGUI_StartSession
//      must been activated.
//		To get unavailability reason of fingers for segmented images from
//		slap captures, call GBMSGUI_GetSegmentationResult instead.
// Parameters:
//		ScannedObjectID:	ID of the object
//		Reason:				(output) GBMSGUI_UR_AMPUTATED or GBMSGUI_UR_UNPRINTABLE.
//							-1 if the vaule is not available
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_GetUnavailabilityReason(unsigned int ScannedObjectID, int *Reason);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_GetSegmentationResult
//      Returns information about finger segments produced by the segmentation.
//      Call this function after GBMSGUI_Acquire for slap and joints objects,
//		and only if GBMSGUI_ACQUISITION_OPTION_SEGMENTATION is active.
// Parameters:
//      Index of segment, as follows:
//          for four (or two) fingers acquisition:
//              0 - index
//              1 - middle
//              2 - ring
//              3 - little
//          for two thumbs (or two indexes) acquisition:
//              0 - left
//              1 - right
//			for joint thumbs
//				0 - distal
//				1 - proximal
//			for other joints
//				0 - distal
//				1 - medial
//				2 - proximal
//      QualityAlgorithmID:	ID of the quality algorithm; not supported for joints, pass 0
//	    SegmentAvailable:	(output) true, false
//      Left, Top, Right, Bottom:	(output) coordinates in pixels of the bounding box of
//				the segment within image returned by GBMSGUI_Acquire;
//				the values are set to -1 if not available
//		UnavailabilityReason: (output) this value can be interpreted just when SegmentAvailable
//              flag is false.
//				Values can be: GBMSGUI_UR_AMPUTATED or GBMSGUI_UR_UNPRINTABLE
//				if GBMSGUI_StartAcquisitionSession was called with
//				GBMSGUI_SESSION_OPTION_ASK_UNAVAILABILITY_REASON option; otherwise -1.
//				Not applicable for Joints
//		FingerprintQuality: (output) quality value according to QualityAlgorithmID
//                (see GBMSGUI_SetQualityAlgorithm);
//                A value of 254 indicates that no quality was ever computed
//                A value of 255 indicate a failed attempt to calculate the quality
//				Not applicable for Joints
//      ImageBuffer: pointer to a buffer to store the segment image in raw format
//                (8 bits greyscale). The buffer must be already allocated by
//                the application. 
//                The segment image is available only if SegmentAvailable flag is true.
//                Set this parameter to NULL if the image should not be returned
//                (e.g. single segments images are not needed).
//				Not applicable for Joints
//		ImageBufferSize: size in bytes of the allocated Image buffer
//		ImageSizeX:		(output) width in pixel of the segment image
//		ImageSizeY:		(output) height in pixel of the segment image
//		Dpi:			(output) resolution of the segment image (can be 500 or 1000)
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_GetSegmentationResult(int Index, int QualityAlgorithmID,
            bool *SegmentAvailable, int *Left, int *Top, int *Right, int *Bottom,
            int *UnavailabilityReason, int *FingerprintQuality,
            unsigned char *ImageBuffer, int ImageBufferSize, int *ImageSizeX, int *ImageSizeY, int *Dpi);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_StopSession
//      Closes current session and closes Acquisition window.
// Parameters:
//      None
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_BUSY_ACQUIRING - an acquisition is still in progress
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_StopSession();
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_GetWindowSizeAndPosition
//      Gets the current size and position and state of acquisition window.
//		Call this function after GBMSGUI_StopSession to know
//      actual size and position of the window. 
//      Could be useful to save the current acquisition window layout.
// Parameters:
//      Left, Top: (output) coordinates of the top left corner of the window
//      Width, Height: (output) size of the window
//		Maximized: (output) true if the window is in maximized state
// Return values:
//      GBMSGUI_RET_SUCCESS - no errors
//      GBMSGUI_RET_FAILURE - some errors occured, call GBMSGUI_GetErrorMessage()
//                      or GBMSGUI_GetErrorCode() for details
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_GetWindowSizeAndPosition(int *Left, int *Top, int *Width, int *Height, bool *Maximized);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_GetErrorMessage
//      Returns a pointer to null-terminated string containing error description
//      call this function when GBMSGUI_RET_FAILURE is returned.
// Parameters:
//      None
// Return values:
//      Pointer to a null-terminated string containing error description
//------------------------------------------------------------------------------
GBMSGUI_API const wchar_t * __stdcall GBMSGUI_GetErrorMessage();
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBMSGUI_GetErrorCode
//      Returns last error code; call this function when GBMSGUI_RET_FAILURE is
//      returned
// Parameters:
//      None
// Return values:
//      Last error code; can be one of the GBMSAPI error codes defined in 
//		GBMSAPI_Defines.h, section "ERROR CODES", or a specific GBMSGUI error:
#define GBMSGUI_ERR_INVALID_PARAMETER					101
#define GBMSGUI_ERR_SESSION_ALREADY_OPEN				102
#define GBMSGUI_ERR_SESSION_NOT_OPEN					103
#define GBMSGUI_ERR_SEGMENTATION_REQUIRED				104
#define GBMSGUI_ERR_AFTERSTARTCALLBACK_FAILED			105
#define GBMSGUI_ERR_FULL_ENHANCED_SDK_NOT_SUPPORTED		106
//------------------------------------------------------------------------------
GBMSGUI_API int __stdcall GBMSGUI_GetErrorCode();
//------------------------------------------------------------------------------


// UTILITY FUNCTIONS


//------------------------------------------------------------------------------
// GBMSGUI_GetMSAPIErrorMessage
//      Returns a pointer to null-terminated string containing error description
//      of the passed GBMSAPI error code.
// Parameters:
//      None
// Return values:
//      Pointer to a null-terminated string containing error description
//------------------------------------------------------------------------------
GBMSGUI_API const wchar_t * __stdcall GBMSGUI_GetMSAPIErrorMessage(wchar_t *FuncName, int ErrCode);
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// The followinfg functions are useful to test the type of a scanned object
//------------------------------------------------------------------------------
GBMSGUI_API bool __stdcall GBMSGUI_IsSlap(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsRolled(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsSingleFinger(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsFlatSingleFinger(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsPalm(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsLowerPalm(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsUpperPalm(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsWritersPalm(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsRolledJoint(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsFlatJoint(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsFlatJointSide(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsJoint(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsRolledTip(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsRolledThenar(unsigned int ScanObjID);
GBMSGUI_API bool __stdcall GBMSGUI_IsThumbJoint(unsigned int ScanObjID);
// 1.14.0.0
GBMSGUI_API bool __stdcall GBMSGUI_IsRolledHypothenar(unsigned int ScanObjID);

}	// extern "C"
