using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Resources;
using System.Globalization;
using System.Reflection;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using GBMSAPI_NET;
using GBMSAPI_NET.GBMSAPI_NET_Defines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_DeviceCharacteristicsDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_ErrorCodesDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_RollFunctionalityDefines;
using GBMSAPI_NET.GBMSAPI_NET_Defines.GBMSAPI_NET_AcquisitionProcessDefines;
using GBMSAPI_NET.GBMSAPI_NET_LibraryFunctions;
using AN2K_NET_WRAPPER;
using An2k_2011_NetWrapper;

namespace GBMSGUI_NET
{
    // utility functions of GBMSGUI
    public partial class GBMSGUI
    {
        // Copy a raw 8 bit image buffer into a bitmap;
        // bmpImage must be already created of the right dimension and with grayscale palette;
        // for use during scanning, to avoid to create a new bitmap object for every frame
        public static void CopyRawImageIntoBitmap(Byte[] RawImage, ref Bitmap bmpImage)
        {
            try
            {
                BitmapData bmData;
                Rectangle bmpRect = new Rectangle(0, 0, bmpImage.Width, bmpImage.Height);

                bmData = bmpImage.LockBits(bmpRect,
                        ImageLockMode.WriteOnly,
                        PixelFormat.Format8bppIndexed);

                int diff = bmData.Stride - bmpImage.Width;
                if (diff == 0)
                {
                    Marshal.Copy(RawImage, 0, bmData.Scan0, bmpImage.Width * bmpImage.Height);
                }
                else
                {
                    int RawIndex = 0;
                    int Ptr = (int)bmData.Scan0;
                    for (int i = 0; i < bmpImage.Height; i++, RawIndex += bmpImage.Width)
                    {
                        Marshal.Copy(RawImage, RawIndex, (IntPtr)Ptr, bmpImage.Width);
                        Ptr += bmData.Stride;
                    }
                }

                // Unlock the bits
                bmpImage.UnlockBits(bmData);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in CopyRawImageIntoBitmap", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        // Create a bitmap from a raw 8 bit image buffer
        // for use not during scanning
        public static Bitmap RawImageToBitmap(Byte[] RawImage, int SizeX, int SizeY)
        {
            Bitmap bmpImage = new Bitmap(SizeX, SizeY, PixelFormat.Format8bppIndexed);
            GBMSGUI.SetGrayPalette(ref bmpImage);
            CopyRawImageIntoBitmap(RawImage, ref bmpImage);

            return bmpImage;
        }

        // gets the raw image buffer from a bitmap object
        public static void BitmapToRawImage(Bitmap bmpImage, Byte[] RawImage)
        {
            BitmapData bmData;
            Rectangle bmpRect = new Rectangle(0, 0, bmpImage.Width, bmpImage.Height);

            bmData = bmpImage.LockBits(bmpRect,
                    ImageLockMode.ReadOnly,
                    PixelFormat.Format8bppIndexed);

            int diff = bmData.Stride - bmpImage.Width;
            if (diff == 0)
            {
                Marshal.Copy(bmData.Scan0, RawImage, 0, bmpImage.Width * bmpImage.Height);
            }
            else
            {
                int RawIndex = 0;
                int Ptr = (int)bmData.Scan0;
                for (int i = 0; i < bmpImage.Height; i++, RawIndex += bmpImage.Width)
                {
                    Marshal.Copy((IntPtr)Ptr, RawImage, RawIndex, bmpImage.Width);
                    Ptr += bmData.Stride;
                }
            }

            // Release the memory 
            bmpImage.UnlockBits(bmData);
        }

        public static void SetGrayPalette(ref Bitmap bmpImage)
        {
            ColorPalette cp = bmpImage.Palette;

            for (int i = 0; i < cp.Entries.Length; i++)
            {
                cp.Entries[i] = Color.FromArgb(i, i, i);

            }
            bmpImage.Palette = cp;
        }

        // used for checking bitmasks
        public static bool CheckMask(uint Val, uint Flag)
        {
            if ((Val & Flag) != 0)
                return true;

            return false;
        }

        internal static bool IsValidScannedObjectID(uint ScannedObjectID)
        {
            bool ret;

            switch (ScannedObjectID)
            {
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_LEFT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_RIGHT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT:
                    ret = true;
                    break;
                //V1.1
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_LEFT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_RIGHT:
                    ret = true;
                    break;
                //V1.2
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_INDEXES:
                    ret = true;
                    break;
                //V1.10
	            case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_THENAR_LEFT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_THENAR_RIGHT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_TIP_RIGHT_LITTLE:
                    ret = true;
                    break;
                // 1.12.3.0
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_UP_RIGHT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_LEFT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_LEFT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_LEFT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_LEFT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_LEFT_LITTLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_RIGHT_THUMB:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_RIGHT_INDEX:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_RIGHT_MIDDLE:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_RIGHT_RING:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_DOWN_RIGHT_LITTLE:
                    ret = true;
                    break;
                // 1.14.0.0
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_HYPOTHENAR_LEFT:
                    ret = true;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_HYPOTHENAR_RIGHT:
                    ret = true;
                    break;
                default:
                    ret = false;
                    break;
            }

            return ret;
        }

        public static bool IsSlap(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SLAP_4) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_THUMBS_2) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SLAP_2) ||    //V1.1
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_INDEXES_2))    //V1.2
                return true;

            return false;
        }

        public static bool IsRolled(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLL_SINGLE_FINGER) ||
                //V1.10
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_TIP) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT_CENTER) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_THENAR) ||
                // 1.12.3.0
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_UP) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_DOWN) ||
                // 1.14.0.0
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_HYPOTHENAR)
                )
                return true;

            return false;
        }

        public static bool IsSingleFinger(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLL_SINGLE_FINGER) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SINGLE_FINGER) ||
                //V1.10
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_TIP) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT_CENTER) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_LEFT_SIDE) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_RIGHT_SIDE) ||
                // 1.12.3.0
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_UP) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_DOWN)
                )
                return true;

            return false;
        }

        public static bool IsFlatSingleFinger(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_SINGLE_FINGER)
                return true;

            return false;
        }

        public static bool IsPalm(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_LOWER_HALF_PALM) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_UPPER_HALF_PALM) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_WRITER_PALM))
                return true;

            return false;
        }

        public static bool IsLowerPalm(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_LOWER_HALF_PALM)
                return true;

            return false;
        }

        public static bool IsUpperPalm(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_UPPER_HALF_PALM)
                return true;

            return false;
        }

        public static bool IsWritersPalm(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_FLAT_WRITER_PALM)
                return true;

            return false;
        }

        //V1.10
        public static bool IsRolledJoint(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT_CENTER))
                return true;

            return false;
        }

        //V1.10
        public static bool IsFlatJoint(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_LEFT_SIDE) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_RIGHT_SIDE))
                return true;

            return false;
        }

        //V1.10
        public static bool IsFlatJointSide(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_LEFT_SIDE) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_RIGHT_SIDE))
                return true;

            return false;
        }

        //V1.10
        public static bool IsJoint(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if ((ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_LEFT_SIDE) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_PLAIN_JOINT_RIGHT_SIDE) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT) ||
                (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_JOINT_CENTER))
                return true;

            return false;
        }

        //V1.10
        public static bool IsRolledTip(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_TIP)
                return true;

            return false;
        }

        //V1.10
        public static bool IsRolledThenar(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_THENAR)
                return true;

            return false;
        }

        // 1.11.0.0
        public static bool IsThumbJoint(uint ScanObjID)
        {
            if ((ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_LEFT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_LEFT_SIDE_RIGHT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_LEFT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_PLAIN_JOINT_RIGHT_SIDE_RIGHT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_THUMB) ||
                (ScanObjID == GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_THUMB))
                return true;

            return false;
        }

        // 1.14.0.0
        public static bool IsRolledHypothenar(uint ScanObjID)
        {
            uint ObjectType = GBMSAPI_NET_ScanObjectsUtilities.GBMSAPI_NET_GetTypeFromObject(ScanObjID);

            if (ObjectType == GBMSAPI_NET_ScannableBiometricTypes.GBMSAPI_NET_SBT_ROLLED_HYPOTHENAR)
                return true;

            return false;
        }

        // converts ScanObjID to AN2K finger position
        public static String ScannedObjectIDToAN2KFingerID(uint ScanObjID)
        {
            String FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN;

            switch (ScanObjID)
            {
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_INDEX:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_LITTLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_MIDDLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_RING:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_LEFT_THUMB:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_INDEX:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_LITTLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_MIDDLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_RING:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLL_RIGHT_THUMB:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_2_THUMBS;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_4_FING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_4_FING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_LEFT:
                    FingerID = NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_LEFT_UPPER_PALM;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_RIGHT:
                    FingerID = NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_RIGHT_UPPER_PALM;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT:
                    FingerID = NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_LEFT_LOWER_PALM;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT:
                    FingerID = NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_RIGTH_LOWER_PALM;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT:
                    FingerID = NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_LEFT_WRITER_PALM;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT:
                    FingerID = NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_RIGHT_WRITER_PALM;
                    break;
                // 1.11.0.0 - needed for sequence check
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_INDEX:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_INDEX:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_LITTLE:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_LITTLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_MIDDLE:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_MIDDLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_RING:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_RING:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_LEFT_THUMB:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_LEFT_THUMB:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_INDEX:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_INDEX:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_LITTLE:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_LITTLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_MIDDLE:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_MIDDLE:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_RING:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_RING:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING;
                    break;
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_RIGHT_THUMB:
                case GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_ROLLED_JOINT_CENTER_RIGHT_THUMB:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB;
                    break;
                default:
                    FingerID = NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_UNKNOWN;
                    break;
            }

            return FingerID;
        }

        // converts AN2K finger position to ScanObjID
        public static uint AN2KFingerIDToScannedObjectID(String AN2KFingerID)
        {
            uint ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SBT_NO_OBJECT;

            if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_INDEX;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_LITTLE;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_MIDDLE;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_RING;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_LEFT_THUMB;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_INDEX;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_LITTLE;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_MIDDLE;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_RING;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_FLAT_RIGHT_THUMB;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_2_THUMBS)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_2_THUMBS;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_4_FING)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_LEFT;
            else if (AN2KFingerID == NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_4_FING)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_SLAP_4_RIGHT;
            else if (AN2KFingerID == NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_LEFT_UPPER_PALM)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_LEFT;
            else if (AN2KFingerID == NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_RIGHT_UPPER_PALM)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_UPPER_HALF_PALM_RIGHT;
            else if (AN2KFingerID == NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_LEFT_LOWER_PALM)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_LEFT;
            else if (AN2KFingerID == NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_RIGTH_LOWER_PALM)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_LOWER_HALF_PALM_RIGHT;
            else if (AN2KFingerID == NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_LEFT_WRITER_PALM)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_LEFT;
            else if (AN2KFingerID == NW_AN2K_PALM_POSITIONS.NW_ETFS_15_PLP_RIGHT_WRITER_PALM)
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SO_WRITER_PALM_RIGHT;
            else
                ScanObjID = GBMSAPI_NET_ScannableObjects.GBMSAPI_NET_SBT_NO_OBJECT;

            return ScanObjID;
        }

        //V1.10
        public static void CopyRawImageRegionIntoRawImage(Byte[] RawImageSrc, int SrcWidth, int SrcHeight,
            int SrcRegionStartX, int SrcRegionStartY, int SrcRegionWidth, int SrcRegionHeight,
            ref Byte[] RawImageDest, int DestWidth, int DestHeight, int DestPositionX, int DestPositionY)
        {
            try
            {
                int SrcY, DestY;

                DestY = DestPositionY;
                for (SrcY = SrcRegionStartY; SrcY < (SrcRegionHeight + SrcRegionStartY); SrcY++, DestY++)
                {
                    Buffer.BlockCopy(RawImageSrc, SrcY * SrcWidth + SrcRegionStartX, 
                        RawImageDest, DestY * DestWidth + DestPositionX, SrcRegionWidth);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " in CopyRawImageRegionIntoRawImage", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        // 1.12.0.0
        public static bool IsOldScanner(byte DeviceID)
        {
            if ((DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_VS3) || (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_PS2)
                || (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_DS26) || (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MS500)
                || (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MC500) || (DeviceID == GBMSAPI_NET_DeviceName.GBMSAPI_NET_DN_MSC500))
		         return true;

	        return false;
        }

    }
}
