using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace GBMSGUI_NET
{
    // implements a progressbar with value shown inside it and with thresholds
    public partial class ThresholdProgressBar : UserControl
    {
        private int Threshold1;
        private int Threshold2;
        private int Value;
        public int MaxValue;

        public ThresholdProgressBar()
        {
            InitializeComponent();

            DoubleBuffered = true;
            MaxValue = 100;
        }

        public void SetThresholds(int Th1, int Th2)
        {
            Threshold1 = Th1;
            Threshold2 = Th2;

            Refresh();
        }

        public void SetValue(int Val)
        {
            Value = Val;
            Refresh();
        }

        private void ThresholdProgressBar_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            Color newColor = new Color();
            Pen BorderPen = new Pen(SystemColors.ControlDark, 1);

            // thresholds
            if (Value < Threshold1)
                newColor = Color.Red;
            else if (((Value >= Threshold1) && (Value < Threshold2))
                && (Threshold1 != Threshold2))
                newColor = Color.Yellow;
            else if (Value >= Threshold2)
                newColor = Color.Lime;

            if ((Threshold1 == 0) && (Threshold2 == 0))
                newColor = SystemColors.Highlight;// Color.Silver;

            // draw border
            Rectangle ContainerRect = new Rectangle(panProgress.Left, panProgress.Top,
                panProgress.Width, panProgress.Height);
            g.DrawRectangle(BorderPen, ContainerRect);

            Font drawFont = new Font(Parent.Font, FontStyle.Bold);

            // draw progressbar
            SolidBrush drawBrush = new SolidBrush(newColor);
            Rectangle ProgressRect = new Rectangle();
            ProgressRect.X = ContainerRect.X + 1;
            ProgressRect.Y = ContainerRect.Y + 1;
            ProgressRect.Width = ContainerRect.Width - 1;
            ProgressRect.Height = ContainerRect.Height - 1;

            ProgressRect.Width = ProgressRect.Width * Value / MaxValue;

            g.FillRectangle(drawBrush, ProgressRect);

            StringFormat stringFormat = new StringFormat();
            stringFormat.Alignment = StringAlignment.Center;
            stringFormat.LineAlignment = StringAlignment.Center;

            if ((Threshold1 != 0) || (Threshold2 != 0))
            {
                // draw thresholds
                Point P1 = new Point();
                Point P2 = new Point();

                P1.X = ContainerRect.X + ContainerRect.Width * Threshold1 / MaxValue;
                P1.Y = ContainerRect.Y;
                P2.X = P1.X;
                P2.Y = ContainerRect.Y + ContainerRect.Height;
                g.DrawLine(BorderPen, P1, P2);
                P2.Y += 10;
                g.DrawString(Threshold1.ToString(), Parent.Font,
                    SystemBrushes.WindowText, P2, stringFormat);

                P1.X = ContainerRect.X + ContainerRect.Width * Threshold2 / MaxValue;
                P1.Y = ContainerRect.Y;
                P2.X = P1.X;
                P2.Y = ContainerRect.Y + ContainerRect.Height;
                g.DrawLine(BorderPen, P1, P2);
                P2.Y += 10;
                g.DrawString(Threshold2.ToString(), Parent.Font,
                    SystemBrushes.WindowText, P2, stringFormat);
            }

            // draw value
            g.DrawString(Value.ToString(), drawFont, SystemBrushes.WindowText, ContainerRect, stringFormat);

        }
    }
}
