using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace GBMSGUI_NET
{
    // Form for Unavailability Reason selection
    internal partial class UnavailabilityReasonForm : Form
    {
        // list of item in the grid; filled from AskUnavailabilityReason function in AcquisitionForm
        public List<GBMSGUI.UnavailabilityReasonListItem> UnavailabilityReasonList = new List<GBMSGUI.UnavailabilityReasonListItem>();

        // class for combobox item
        private class UnavailabilityReasonComboItem
        {
            private string mText;
            private int mValue;

            public UnavailabilityReasonComboItem(int Value, String Text)
            {
                this.mValue = Value;
                this.mText = Text;
            }

            // for combobox column
            public int Value
            {
                get { return mValue; }
                set { mValue = value; }
            }
            public string Text
            {
                get { return mText; }
                set { mText = value; }
            }
        }

        private List<UnavailabilityReasonComboItem> ReasonList = new List<UnavailabilityReasonComboItem>();

        private AcquisitionForm AcquisitionFormRef;
        public UnavailabilityReasonForm(AcquisitionForm AcquisitionFormRef)
        {
            InitializeComponent();

            // set captions
            Text = AcquisitionFormRef.MSGUIRef.GetLocalizedString("Specify_Unavailability_Reason");
            
            lblFingerCaption.Text = AcquisitionFormRef.MSGUIRef.GetLocalizedString("Finger");
            lblReasonCaption.Text = AcquisitionFormRef.MSGUIRef.GetLocalizedString("Unavailability_Reason");
            btnOk.Text = AcquisitionFormRef.MSGUIRef.GetLocalizedString("btnOk");
            btnCancel.Text = AcquisitionFormRef.MSGUIRef.GetLocalizedString("btnCancel");

            // set reference to AcquisitionForm
            this.AcquisitionFormRef = AcquisitionFormRef;
        }

        private void UnavailabilityReasonForm_Load(object sender, EventArgs e)
        {
            UnavailabilityReasonComboItem UnavailReason;

            // prepare values for combobox
            // set an empty value
            UnavailReason = new UnavailabilityReasonComboItem(0, "");
            ReasonList.Add(UnavailReason);

            UnavailReason = new UnavailabilityReasonComboItem(GBMSGUI.UnavailabilityReason.Amputated, 
                AcquisitionFormRef.MSGUIRef.GetLocalizedString("Amputated"));
            ReasonList.Add(UnavailReason);

            UnavailReason = new UnavailabilityReasonComboItem(GBMSGUI.UnavailabilityReason.Unprintable, 
                AcquisitionFormRef.MSGUIRef.GetLocalizedString("Bandaged"));
            ReasonList.Add(UnavailReason);

            // show necessary rows
            panUnavail2.Visible = (UnavailabilityReasonList.Count > 1);
            panUnavail3.Visible = (UnavailabilityReasonList.Count > 2);
            panUnavail4.Visible = (UnavailabilityReasonList.Count > 3);

            // show fingers names
            lblFinger1.Text = UnavailabilityReasonList[0].ObjectName;
            if (panUnavail2.Visible)
                lblFinger2.Text = UnavailabilityReasonList[1].ObjectName;
            if (panUnavail3.Visible)
                lblFinger3.Text = UnavailabilityReasonList[2].ObjectName;
            if (panUnavail4.Visible)
                lblFinger4.Text = UnavailabilityReasonList[3].ObjectName;

            // set comboboxes
            cmbReason1.BindingContext = new BindingContext();
            cmbReason1.DataSource = ReasonList;
            cmbReason1.DisplayMember = "Text";
            cmbReason1.ValueMember = "Value";
            cmbReason1.DataBindings.Add("SelectedValue", UnavailabilityReasonList[0], "Reason");
            if (panUnavail2.Visible)
            {
                cmbReason2.BindingContext = new BindingContext();
                cmbReason2.DataSource = ReasonList;
                cmbReason2.DisplayMember = "Text";
                cmbReason2.ValueMember = "Value";
                cmbReason2.DataBindings.Add("SelectedValue", UnavailabilityReasonList[1], "Reason");
            }
            if (panUnavail3.Visible)
            {
                cmbReason3.BindingContext = new BindingContext();
                cmbReason3.DataSource = ReasonList;
                cmbReason3.DisplayMember = "Text";
                cmbReason3.ValueMember = "Value";
                cmbReason3.DataBindings.Add("SelectedValue", UnavailabilityReasonList[2], "Reason");
            }
            if (panUnavail4.Visible)
            {
                cmbReason4.BindingContext = new BindingContext();
                cmbReason4.DataSource = ReasonList;
                cmbReason4.DisplayMember = "Text";
                cmbReason4.ValueMember = "Value";
                cmbReason4.DataBindings.Add("SelectedValue", UnavailabilityReasonList[3], "Reason");
            }
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            bool AllSpecified = true;

            for (int i=0; i<UnavailabilityReasonList.Count; i++)
            {
                if (UnavailabilityReasonList[i].Reason == 0)
                {
                    AllSpecified = false;
                    break;
                }
            }

            if (!AllSpecified)
            {
                MessageBox.Show(AcquisitionFormRef.MSGUIRef.GetLocalizedString("Specify_Unavailability_Reason"),
                    AcquisitionFormRef.MSGUIRef.GetLocalizedString("Dlg_Caption_Confirm"), 
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            DialogResult = DialogResult.OK;
        }
    }
}