﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using AN2K_NET_WRAPPER;
using An2k_Net_Wrapper_Utilities;
using An2k_Engine_Net_Wrapper_Utilities;
using An2k_Engine_Net_Wrapper;
using An2k_2011_NetWrapper;
using An2k_2011_Net_Wrapper_Utilities;

namespace An2k_Engine_Net_Wrapper_Utilities
{
	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
	public struct An2k_Subfield
	{
		public int num_bytes;
		public int num_items;
		public int alloc_items;
		public IntPtr items; // ITEM**
		public int rs_char;
	}

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
	public struct An2k_Field
	{
		public String id;
		public UInt32 record_type;
		public UInt32 field_int;
		public int num_bytes;
		public int num_subfields;
		public int alloc_subfields;
		public IntPtr subfields; // SUBFIELD **
		public int gs_char;
	}

	/// <summary>
	/// 
	/// </summary>
	public class An2k_Engine_NetWrapper
	{
		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords(
			IntPtr outSubField, // SUBFIELD **
			String[] valuesToInsertArray,
			int numOfValuesToInsert
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords(
			IntPtr outField, // FIELD** 
			int recordType,
			int fieldId
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords(
			IntPtr fieldToUpdate, // FIELD*
			IntPtr subfieldToInsert // SUBFIELD *
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords(
			IntPtr outField, // FIELD** 
			int recordType,
			int fieldId,
			String[] valuesToInsertArray,
			int numOfValuesToInsert
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_CreateImageFieldForTaggedRecord", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_CreateImageFieldForTaggedRecord(
			IntPtr outField, // FIELD** 
			int recordType,
			IntPtr imageBuffer, // unsigned char *
			int imageBufferLen
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_CreateEmptyTaggedRecord", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_CreateEmptyTaggedRecord(
			IntPtr outRecord, // RECORD** 
			int recordType,
			int Idc
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_InsertFieldIntoTaggedRecord", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_InsertFieldIntoTaggedRecord(
			IntPtr recordToUpdate, // RECORD* 
			IntPtr fieldToInsert // FIELD* 
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_CreateBinaryRecord", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_CreateBinaryRecord(
			IntPtr outRecord, // RECORD** 
			int recordType,
			int Idc,
			int impressionType,
			IntPtr fgp, // unsigned char *
			int imageScanResolution,
			int horizontalNumberOfPixels,
			int verticalNumberOfPixels,
			int compressionAlgorithm,
			IntPtr imageBuffer, // unsigned char *
			int imageBufferLen
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_CreateEmptyAnsiNistStruct", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_CreateEmptyAnsiNistStruct(
			IntPtr pAnsinistToBeCreated // ANSI_NIST** 
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_InsertRecordInAnsiNist", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_InsertRecordInAnsiNist(
			IntPtr RecordToBeInserted, // RECORD*
			IntPtr ansi_nist, // ANSI_NIST*
			int RecordIndex,
			int AutomaticIDC // BOOL
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_SearchFieldInTaggedRecord", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_SearchFieldInTaggedRecord(
			IntPtr recordToLookup, // RECORD*
			int FieldId,
			IntPtr FoundField // FIELD** 
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(
			IntPtr inField, // FIELD*
			IntPtr pValuesToGet, // GB_AN_ENGINE_StringArray* (char ***)
			out int pNumOfValuesToGet
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords(
			IntPtr inSubField, // SUBFIELD *
			IntPtr pValuesToGet, // GB_AN_ENGINE_StringArray* (char ***)
			out int pNumOfValuesToGet
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_GetImageFieldFromTaggedRecord", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_GetImageFieldFromTaggedRecord(
			IntPtr recordToRead, // RECORD *
			IntPtr pImageBuffer, // unsigned char **
			out int imgLen
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_FreeStringArray", CharSet = CharSet.Ansi)]
		public static extern int GB_AN_ENGINE_FreeStringArray(
			IntPtr ValuesToFree,
			int numOfValuesToFree
			);

		[DllImport("AN2K_LIB.dll", EntryPoint = "GB_AN_ENGINE_Free", CharSet = CharSet.Ansi)]
		public static extern void GB_AN_ENGINE_Free(
			IntPtr anStructToBeFreed, // ANSI_NIST* 
			IntPtr recordToBeFreed, // RECORD*
			IntPtr fieldToBeFreed,// FIELD*
			IntPtr subfieldToBeFreed, // SUBFIELD*
			IntPtr itemToBeFreed // ITEM* 
			);
	}


	public class An2kEngineNetWrapperUtilities
	{
		/// <summary>
		/// gets the string from a single-item field
		/// </summary>
		/// <param name="recToRead"></param>
		/// <param name="fieldId"></param>
		/// <returns></returns>
		public static String GetStringFromTaggedRecordField(An2kEngineNw_TaggedRecord recToRead, UInt32 fieldId)
		{
			try
			{
				An2kEngineNw_FieldForTaggedRecord dummyField = recToRead.SearchField((int)fieldId);
				if (dummyField != null)
				{
					string[] strArray;
					int RetVal = dummyField.GetValuesFromItems(out strArray);
					if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
					{
						NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "GetStringFromTaggedRecordField returned " + RetVal +
							" for field " + fieldId;
						throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
					}
					if (strArray == null || strArray.Length < 1)
					{
						return null;
					}
					return strArray[0];
				}
				return null;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2k2011_Type1Record.GetInfo: " + ex.Message;
				throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
			}

		}
	}
}

namespace An2k_Engine_Net_Wrapper
{
	/// <summary>
	/// This class represents a subfield. It's to be used for tagged records only.
	/// It corresponds to the SUBFIELD struct in C library 
	/// </summary>
	public class An2kEngineNw_SubfieldForTaggedRecord
	{
		/// <summary>
		/// it's a pointer to the corresponding SUBFIELD struct in C library
		/// </summary>
		protected IntPtr buffer;

		/// <summary>
		/// initializes class properties
		/// </summary>
		public void Init()
		{
			buffer = IntPtr.Zero;
		}

		/// <summary>
		/// constructor
		/// </summary>
		public An2kEngineNw_SubfieldForTaggedRecord()
		{
			Init();
		}

		/// <summary>
		/// gets a list of values from subfield items
		/// see "GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords" in C library
		/// </summary>
		/// <param name="pValuesToGet">list of values</param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int GetValuesFromItems(out String[] pValuesToGet)
		{
			pValuesToGet = null;
			try
			{
				int numOfValues;
				IntPtr valuesToGetPtr = new IntPtr();
				IntPtr pValuesToGetPtr = An2k_Utils.MarshalToPointer(valuesToGetPtr);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords(
					this.buffer, pValuesToGetPtr, out numOfValues);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					pValuesToGet = null;
					return RetVal;
				}
				valuesToGetPtr = (IntPtr)An2k_Utils.MarshalToStruct(pValuesToGetPtr, typeof(IntPtr));
				if (valuesToGetPtr != IntPtr.Zero && numOfValues > 0)
				{
					pValuesToGet = new string[numOfValues];
					// create pointers
					Byte[] App = new Byte[(IntPtr.Size) * (numOfValues)];
					Marshal.Copy(valuesToGetPtr, App, 0, App.Length);
					for (int i = 0; i < numOfValues; i++)
					{
						IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
						Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
						IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

						if (AppPtr != IntPtr.Zero)
						{
							pValuesToGet[i] = An2k_Utils.GetStringFromIntPtr(AppPtr, 0);
						}
						else
						{
							pValuesToGet[i] = null;
						}

						Marshal.FreeHGlobal(pAppPtr);
					}
					An2k_Engine_NetWrapper.GB_AN_ENGINE_FreeStringArray(valuesToGetPtr, numOfValues);
				}
				else
				{
					pValuesToGet = null;
				}
				Marshal.FreeHGlobal(pValuesToGetPtr);

				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_SubfieldForTaggedRecord.GetValuesFromItems: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// creates a subfield from the list of values of its
		/// items (they must be in the order as they appear in the
		/// specification)
		/// see "GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords" in C library
		/// </summary>
		/// <param name="values"></param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int CreateFromValues(String[] values)
		{
			try
			{
				this.buffer = new IntPtr();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(this.buffer);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords(
					pBuffer, values, values.Length);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					return RetVal;
				}
				buffer = (IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr));
				Marshal.FreeHGlobal(pBuffer);
				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_SubfieldForTaggedRecord.CreateFromValues: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// returns buffer
		/// </summary>
		/// <returns>An2kEngineNw_SubfieldForTaggedRecord.buffer</returns>
		public IntPtr GetBuffer()
		{
			return buffer;
		}

		/// <summary>
		/// sets buffer
		/// </summary>
		/// <param name="Buf"></param>
		public void SetBuffer(IntPtr Buf)
		{
			buffer = Buf;
		}
	}

	/// <summary>
	/// This class represents a field. It's to be used for tagged records only.
	/// It corresponds to the FIELD struct in C library
	/// </summary>
	public class An2kEngineNw_FieldForTaggedRecord
	{
		/// <summary>
		/// type of the record the field belongs to
		/// </summary>
		public UInt32 recordType;
		/// <summary>
		/// id of the field (number that appears in specification)
		/// </summary>
		public UInt32 fieldId;
		/// <summary>
		/// it's a pointer to the corresponding FIELD struct in C library
		/// </summary>
		protected IntPtr buffer;

		/// <summary>
		/// initializes class properties
		/// </summary>
		public void Init()
		{
			fieldId = 0;
			recordType = 0;
			buffer = IntPtr.Zero;
		}

		/// <summary>
		/// constructor
		/// </summary>
		public An2kEngineNw_FieldForTaggedRecord()
		{
			Init();
		}

		/// <summary>
		/// returns buffer
		/// </summary>
		/// <returns>An2kEngineNw_FieldForTaggedRecord.buffer</returns>
		public IntPtr GetBuffer()
		{
			return buffer;
		}

		/// <summary>
		/// sets buffer
		/// </summary>
		/// <param name="Buf"></param>
		public void SetBuffer(IntPtr buf)
		{
			buffer = buf;
		}

		/// <summary>
		/// gets a list of values from field items.
		/// NOTE: the field must contain no more than one subfield
		/// see "GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords" in C library
		/// </summary>
		/// <param name="pValuesToGet">list of values</param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int GetValuesFromItems(out String[] pValuesToGet)
		{
			pValuesToGet = null;
			try
			{
				int numOfValues;
				IntPtr valuesToGetPtr = new IntPtr();
				IntPtr pValuesToGetPtr = An2k_Utils.MarshalToPointer(valuesToGetPtr);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(
					this.buffer, pValuesToGetPtr, out numOfValues);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					pValuesToGet = null;
					return RetVal;
				}
				valuesToGetPtr = (IntPtr)An2k_Utils.MarshalToStruct(pValuesToGetPtr, typeof(IntPtr));
				if (valuesToGetPtr != IntPtr.Zero && numOfValues > 0)
				{
					pValuesToGet = new string[numOfValues];
					// create pointers
					Byte[] App = new Byte[(IntPtr.Size) * (numOfValues)];
					Marshal.Copy(valuesToGetPtr, App, 0, App.Length);
					for (int i = 0; i < numOfValues; i++)
					{
						IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
						Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
						IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

						if (AppPtr != IntPtr.Zero)
						{
							pValuesToGet[i] = An2k_Utils.GetStringFromIntPtr(AppPtr, 0);
						}
						else
						{
							pValuesToGet[i] = null;
						}

						Marshal.FreeHGlobal(pAppPtr);
					}
					An2k_Engine_NetWrapper.GB_AN_ENGINE_FreeStringArray(valuesToGetPtr, numOfValues);
				}
				else
				{
					pValuesToGet = null;
				}
				Marshal.FreeHGlobal(pValuesToGetPtr);
				
				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_FieldForTaggedRecord.GetValuesFromItems: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// Creates the image field for those tagged records that contain
		/// images (type 13, 14, 15, etc)
		/// see "GB_AN_ENGINE_CreateImageFieldForTaggedRecord" in C library
		/// </summary>
		/// <param name="type">type of the record the field belongs to</param>
		/// <param name="imageBuffer">image buffer</param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int CreateFromImage(UInt32 type, byte[] imageBuffer)
		{
			try
			{
				this.buffer = new IntPtr();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(this.buffer);
				IntPtr imagePtr = Marshal.AllocHGlobal(imageBuffer.Length);
				if (imagePtr == IntPtr.Zero)
				{
					Marshal.FreeHGlobal(pBuffer);
					return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
				}
				Marshal.Copy(imageBuffer, 0, imagePtr, imageBuffer.Length);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_CreateImageFieldForTaggedRecord(
					pBuffer, (int)type, imagePtr, imageBuffer.Length);
				Marshal.FreeHGlobal(imagePtr);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					this.recordType = 0;
					this.fieldId = 0;
					return RetVal;
				}
				buffer = (IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr));
				this.recordType = type;
				this.fieldId = 999;

				Marshal.FreeHGlobal(pBuffer);
				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_FieldForTaggedRecord.CreateAsImageField: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// creates a field from the list of values of its
		/// items (they must be in the order as they appear in the
		/// specification). The field will be created with a subfield
		/// only
		/// see "GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords" in C library
		/// </summary>
		/// <param name="values"></param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int CreateFromValues(UInt32 type, UInt32 id, String[] values)
		{
			try
			{
				this.buffer = new IntPtr();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(this.buffer);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords(
					pBuffer, (int)type, (int)id, values, values.Length);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					this.recordType = 0;
					this.fieldId = 0;
					return RetVal;
				}
				buffer = (IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr));
				Marshal.FreeHGlobal(pBuffer);
				this.recordType = type;
				this.fieldId = id;

				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_FieldForTaggedRecord.CreateFromValues: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// creates an empty field, containing type and id only
		/// see "GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords" in C library
		/// </summary>
		/// <param name="type"></param>
		/// <param name="id"></param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int CreateEmpty(UInt32 type, UInt32 id)
		{
			try
			{
				this.buffer = new IntPtr();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(this.buffer);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords(
					pBuffer, (int)type, (int)id);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					this.recordType = 0;
					this.fieldId = 0;
					return RetVal;
				}
				buffer = (IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr));
				Marshal.FreeHGlobal(pBuffer);
				this.recordType = type;
				this.fieldId = id;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_FieldForTaggedRecord.CreateEmpty: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// adds a subfield into the field, at the end of subfields list
		/// see "GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords" in C library
		/// </summary>
		/// <param name="subfieldToInsert"></param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int InsertSubfield(An2kEngineNw_SubfieldForTaggedRecord subfieldToInsert)
		{
			try
			{
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords(
					this.buffer, subfieldToInsert.GetBuffer());
				return RetVal;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_FieldForTaggedRecord.InsertSubfield: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// gets the list of subfields contained into the field
		/// </summary>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public An2kEngineNw_SubfieldForTaggedRecord[] GetSubfields()
		{
			try
			{
				An2k_Field field = (An2k_Field)An2k_Utils.MarshalToStruct(buffer, typeof(An2k_Field));
				Byte[] App = new Byte[(IntPtr.Size) * (field.num_subfields)];
				Marshal.Copy(field.subfields, App, 0, App.Length);
				An2kEngineNw_SubfieldForTaggedRecord[] ret = new An2kEngineNw_SubfieldForTaggedRecord[field.num_subfields];
				// create pointers
				for (int i = 0; i < field.num_subfields; i++)
				{
					IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
					Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
					IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

					An2k_Subfield sbf = (An2k_Subfield)(An2k_Utils.MarshalToStruct(AppPtr, typeof(An2k_Subfield)));
					ret[i] = new An2kEngineNw_SubfieldForTaggedRecord();
					ret[i].SetBuffer(AppPtr);

					Marshal.FreeHGlobal(pAppPtr);
				}
				return ret;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_FieldForTaggedRecord.GetSubfields: " + ex.Message;
				throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
			}
		}
	}

	/// <summary>
	/// This interface represents a generic record.
	/// It corresponds to the RECORD struct in C library
	/// </summary>
	public class An2kEngineNw_Record
	{
		/// <summary>
		/// type of the record
		/// </summary>
		public UInt32 recordType;
		/// <summary>
		/// IDC of the record
		/// </summary>
		public int recordIdc;
		/// <summary>
		/// it's a pointer to the corresponding RECORD struct in C library
		/// </summary>
		protected IntPtr buffer;

		/// <summary>
		/// returns buffer
		/// </summary>
		/// <returns></returns>
		public IntPtr GetBuffer() { return buffer; }

		/// <summary>
		/// sets buffer
		/// </summary>
		/// <param name="ptr"></param>
		public void SetBuffer(IntPtr ptr) { buffer = ptr; }

		/// <summary>
		/// sets record type
		/// </summary>
		/// <param name="type"></param>
		public void SetRecordType(UInt32 type) { recordType = type; }

		/// <summary>
		/// sets idc
		/// </summary>
		/// <param name="idc"></param>
		public void SetIdc(int idc) { recordIdc = idc; }
	}

	/// <summary>
	/// derived from the An2kEngineNw_Record interface, represents a
	/// tagged record
	/// </summary>
	public class An2kEngineNw_TaggedRecord : An2kEngineNw_Record
	{
		/// <summary>
		/// initializes class properties
		/// </summary>
		public virtual void Init()
		{
			recordIdc = -1;
			recordType = 0;
			buffer = IntPtr.Zero;
		}

		/// <summary>
		/// constructor
		/// </summary>
		public An2kEngineNw_TaggedRecord()
		{
			Init();
		}

		/// <summary>
		/// gets the image from the record. The 999 field must be present
		/// see "GB_AN_ENGINE_GetImageFieldFromTaggedRecord" function in C library
		/// </summary>
		/// <param name="imageBuffer">output: buffer of the image</param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int GetImage(out byte[] imageBuffer)
		{
			imageBuffer = null;
			try
			{
				IntPtr FingImage = IntPtr.Zero;
				IntPtr pFingImage = An2k_Utils.MarshalToPointer(FingImage);
				int imgLen;

				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_GetImageFieldFromTaggedRecord(
					this.buffer, pFingImage, out imgLen);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					Marshal.FreeHGlobal(pFingImage);
					imageBuffer = null;
					return RetVal;
				}
				if (imgLen > 0)
				{
					FingImage = (IntPtr)An2k_Utils.MarshalToStruct(pFingImage, typeof(IntPtr));
					Marshal.FreeHGlobal(pFingImage);
					if (FingImage != IntPtr.Zero)
					{
						imageBuffer = new Byte[imgLen];
						Marshal.Copy(FingImage, imageBuffer, 0, imgLen);
						An2k2011_NetWrapper.GB_AN_ENGINE_FreeGenericMem(FingImage);
					}
					else
					{
						imageBuffer = null;
					}
				}
				else
				{
					imageBuffer = null;
				}
				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_TaggedRecord.GetImage: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// creates an empty record from its type and idc
		/// see "GB_AN_ENGINE_CreateEmptyTaggedRecord" function in C library
		/// </summary>
		/// <param name="type"></param>
		/// <param name="idc"></param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int CreateEmpty(UInt32 type, int idc)
		{
			try
			{
				this.buffer = new IntPtr();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(this.buffer);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_CreateEmptyTaggedRecord(
					pBuffer, (int)type, idc);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					this.recordType = 0;
					this.recordIdc = -1;
					return RetVal;
				}
				buffer = (IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr));
				Marshal.FreeHGlobal(pBuffer);
				this.recordType = type;
				this.recordIdc = idc;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_TaggedRecord.CreateEmpty: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// inserts a field into the record's fields list
		/// see "GB_AN_ENGINE_InsertFieldIntoTaggedRecord" function in C library
		/// </summary>
		/// <param name="fieldToInsert"></param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int InsertField(An2kEngineNw_FieldForTaggedRecord fieldToInsert)
		{
			try
			{
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_InsertFieldIntoTaggedRecord(
					this.buffer, fieldToInsert.GetBuffer());
				return RetVal;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_TaggedRecord.InsertField: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// searchs a field into the record's fields list
		/// see "GB_AN_ENGINE_SearchFieldInTaggedRecord" function in C library
		/// </summary>
		/// <param name="fieldId">id of the field to be searched</param>
		/// <returns>the searched field; null if not found</returns>
		public An2kEngineNw_FieldForTaggedRecord SearchField(int fieldId)
		{
			try
			{
				An2kEngineNw_FieldForTaggedRecord retField = new An2kEngineNw_FieldForTaggedRecord();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(retField.GetBuffer());
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_SearchFieldInTaggedRecord(this.buffer, fieldId, pBuffer);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					return null;
				}
				retField.SetBuffer((IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr)));
				Marshal.FreeHGlobal(pBuffer);
				retField.recordType = this.recordType;
				retField.fieldId = (UInt32)fieldId;

				return retField;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_TaggedRecord.SearchField: " + ex.Message;
				throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
			}
		}
	}

	/// <summary>
	/// derived from the An2kEngineNw_Record interface, represents a
	/// binary record (type 3, 4, 5, 6, 7, 8)
	/// </summary>
	public class An2kEngineNw_BinaryRecord : An2kEngineNw_Record
	{
		/// <summary>
		/// impression type
		/// </summary>
		public int recordImpressionType;
		/// <summary>
		/// list of fingerprint positions
		/// </summary>
		public byte[] recordFgp;
		/// <summary>
		/// scan resolution (1 or 0)
		/// </summary>
		public int recordImageScanResolution;
		/// <summary>
		/// x size of the image stored into the record
		/// </summary>
		public int recordHorizontalNumberOfPixels;
		/// <summary>
		/// y size of the image stored into the record
		/// </summary>
		public int recordVerticalNumberOfPixels;
		/// <summary>
		/// compression algorithm
		/// </summary>
		public int recordCompressionAlgorithm;
		/// <summary>
		/// image stored into the record
		/// </summary>
		public byte[] recordImageBuffer;

		/// <summary>
		/// class constructor
		/// </summary>
		public An2kEngineNw_BinaryRecord()
		{
			this.Init();
		}

		/// <summary>
		/// initialization of class values to their
		/// default value
		/// </summary>
		public void Init()
		{
			buffer = IntPtr.Zero;
			recordType = 0;
			recordIdc = -1;
			recordImpressionType = -1;
			recordImageScanResolution = -1;
			recordHorizontalNumberOfPixels = -1;
			recordVerticalNumberOfPixels = -1;
			recordCompressionAlgorithm = -1;
			recordFgp = null;
			recordImageBuffer = null;
		}

		/// <summary>
		/// creates a binary record
		/// see "GB_AN_ENGINE_CreateBinaryRecord" function in C library
		/// </summary>
		/// <param name="type"></param>
		/// <param name="idc"></param>
		/// <param name="impressionType"></param>
		/// <param name="fgp"></param>
		/// <param name="imageScanResolution"></param>
		/// <param name="horizontalNumberOfPixels"></param>
		/// <param name="verticalNumberOfPixels"></param>
		/// <param name="compressionAlgorithm"></param>
		/// <param name="imageBuffer"></param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public virtual int Create(UInt32 type, int idc, int impressionType, byte[] fgp,
			int imageScanResolution, int horizontalNumberOfPixels, int verticalNumberOfPixels,
			int compressionAlgorithm, byte[] imageBuffer)
		{
			try
			{
				if (fgp.Length < 6) return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
				this.buffer = new IntPtr();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(this.buffer);
				IntPtr imagePtr = Marshal.AllocHGlobal(imageBuffer.Length);
				if (imagePtr == IntPtr.Zero)
				{
					Marshal.FreeHGlobal(pBuffer);
					return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
				}
				Marshal.Copy(imageBuffer, 0, imagePtr, imageBuffer.Length);
				IntPtr fgpArray = Marshal.AllocHGlobal(6);
				if (fgpArray == IntPtr.Zero)
				{
					Marshal.FreeHGlobal(imagePtr);
					Marshal.FreeHGlobal(pBuffer);
					return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
				}
				Marshal.Copy(fgp, 0, fgpArray, 6);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_CreateBinaryRecord(
					pBuffer, (int)type, idc, impressionType, fgpArray, imageScanResolution, horizontalNumberOfPixels,
					verticalNumberOfPixels, compressionAlgorithm, imagePtr, imageBuffer.Length);
				Marshal.FreeHGlobal(imagePtr);
				Marshal.FreeHGlobal(fgpArray);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					Marshal.FreeHGlobal(pBuffer);
					this.Init();
					return RetVal;
				}
				buffer = (IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr));
				Marshal.FreeHGlobal(pBuffer);
				recordType = type;
				recordIdc = idc;
				recordImpressionType = impressionType;
				recordImageScanResolution = imageScanResolution;
				recordHorizontalNumberOfPixels = horizontalNumberOfPixels;
				recordVerticalNumberOfPixels = verticalNumberOfPixels;
				recordCompressionAlgorithm = compressionAlgorithm;
				recordFgp = new byte[6];
				if (recordFgp == null)
				{
					return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
				}
				Array.Copy(fgp, recordFgp, 6);
				recordImageBuffer = new byte[imageBuffer.Length];
				if (recordImageBuffer == null)
				{
					return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
				}
				Array.Copy(imageBuffer, recordImageBuffer, imageBuffer.Length);
				return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_BinaryRecord.Create: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

	}

	/// <summary>
	/// This interface represents an Ansi Nist struct (practically,
	/// a collection of records).
	/// It corresponds to the ANSI_NIST struct in C library
	/// </summary>
	public class An2kEngineNw_AnsiNist
	{
		/// <summary>
		/// it's a pointer to the corresponding ANSI_NIST struct in C library
		/// </summary>
		protected IntPtr buffer;

		/// <summary>
		/// class constructor
		/// </summary>
		public An2kEngineNw_AnsiNist() { buffer = IntPtr.Zero; }

		/// <summary>
		/// creates an empty ansi_nist struct
		/// see "GB_AN_ENGINE_CreateEmptyAnsiNistStruct" function in C library
		/// </summary>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int CreateEmpty()
		{
			try
			{
				buffer = new IntPtr();
				IntPtr pBuffer = An2k_Utils.MarshalToPointer(this.buffer);
				int RetVal = An2k_Engine_NetWrapper.GB_AN_ENGINE_CreateEmptyAnsiNistStruct(pBuffer);
				if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					buffer = (IntPtr)An2k_Utils.MarshalToStruct(pBuffer, typeof(IntPtr));
				}
				Marshal.FreeHGlobal(pBuffer);
				return RetVal;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_AnsiNist.CreateEmpty: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// inserts a record into the ansi_nist's list of records
		/// see "GB_AN_ENGINE_InsertRecordInAnsiNist" function in C library
		/// </summary>
		/// <param name="recordToInsert">record to insert</param>
		/// <param name="index">index of the record</param>
		/// <param name="autoIdc">indicates if the idc must be automatically calculated</param>
		/// <returns>see NW_AN2K_ERRORS class</returns>
		public int InsertRecord(An2kEngineNw_Record recordToInsert, int index, bool autoIdc)
		{
			try
			{
				return An2k_Engine_NetWrapper.GB_AN_ENGINE_InsertRecordInAnsiNist(
					recordToInsert.GetBuffer(), this.buffer, index, autoIdc ? 1 : 0);
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_AnsiNist.InsertRecord: " + ex.Message;
				return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
			}
		}

		/// <summary>
		/// gets the record at the specified index
		/// </summary>
		/// <param name="index"></param>
		/// <returns>the record at the specified index; null in case of error</returns>
		public An2kEngineNw_Record GetRecordFromListAtIndex(int index)
		{
			AN2K_Struct ansi_nist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(buffer, typeof(AN2K_Struct)));
			if (index >= ansi_nist.num_records) return null;

			Byte[] App = new Byte[(IntPtr.Size) * (ansi_nist.num_records)];
			Marshal.Copy(ansi_nist.records, App, 0, App.Length);
			IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
			Marshal.Copy(App, 0 + index * (IntPtr.Size), pAppPtr, IntPtr.Size);
			IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));
			AN2K_Record rec = (AN2K_Record)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_Record)));

			An2kEngineNw_Record ret = new An2kEngineNw_Record();
			ret.SetBuffer(AppPtr);
			ret.SetRecordType(rec.type);
			Marshal.FreeHGlobal(pAppPtr);

			return ret;
		}

		/// <summary>
		/// generates an ANSI_NIST buffer, formatted as 
		/// indicated in the ANSI/NIST ITL-20xx specification
		/// see "write_ANSI_NIST_2_buffer" function in C library
		/// </summary>
		/// <returns>generated buffer. null in case of failure</returns>
		public byte[] WriteToBuffer()
		{
			try
			{
				AN2K_Struct ansi_nist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(buffer, typeof(AN2K_Struct)));
				IntPtr AppBuffer = new IntPtr();
				IntPtr pAppBuffer = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(AppBuffer);
				int BufLen;

				int RetVal = An2kdll_NetWrapper.write_ANSI_NIST_2_buffer(pAppBuffer, out BufLen, ref ansi_nist);
				if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
				{
					return null;
				}
				AppBuffer = (IntPtr)An2k_Utils.MarshalToStruct(pAppBuffer, typeof(IntPtr));
				Marshal.FreeHGlobal(pAppBuffer);
				byte[] ret = new byte[BufLen];
				Marshal.Copy(AppBuffer, ret, 0, BufLen);
				An2k2011_NetWrapper.GB_AN_ENGINE_FreeGenericMem(AppBuffer);

				return ret;
			}
			catch (Exception ex)
			{
				NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "An2kEngineNw_AnsiNist.WriteToBuffer: " + ex.Message;
				throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
			}
		}
	}
}

