﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Reflection;
using System.Runtime.CompilerServices;

using AN2K_NET_WRAPPER;
using An2k_Net_Wrapper_Utilities;
[assembly: AssemblyVersion("3.3.0.0")]
[assembly: AssemblyFileVersion("3.3.0.0")]

namespace An2k_Net_Wrapper_Utilities
{
    /************************************************************
     ************************************************************
     *
     * Structures definition: PACK 1 for compatibility with
     * C structures
     * 
     ************************************************************
    ************************************************************/
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_Record
    {
        public UInt32 type;
        public int total_bytes;
        public int num_bytes;
        public int num_fields;
        public int alloc_fields;
        public IntPtr fields;
        public int fs_char;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_Struct{
        public uint version;
        public int num_bytes;
        public int num_records;
        public int alloc_records;
        public IntPtr records;
    } /* records in ANSI_NIST file */

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_RecordList{
        public int num_records;
        public IntPtr records;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_FingerSegment{
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 3)]
        public String fing_id;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 5)]
        public String left_bound;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 5)]
        public String right_bound;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 5)]
        public String top_bound;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 5)]
        public String bottom_bound;
    } /* contains informations about boundaries for segmentation*/

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_FourFingBound{
        public int num_fingers;
        public IntPtr fingers;	// AN2K_FingerSegment **
    }/* list of finger segmentation coordinates */

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_FingerQuality
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 3)]
        public String fing_id;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 2)]
        public String quality;
    } /* contains informations about NFIQ Quality */

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_FourFingerQuality{
        public int num_fingers;
        public IntPtr fingers;	// AN2K_FingerQuality **
    }/* list of finger qualities */

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_FingerAmp{
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 3)]
        public String fing_id; // char[3]
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 3)]
        public String amp_code; // char[3]
    } /* contains informations about amputated or bandaged fingers */

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_FingerAmpList{
        public int num_fingers;
        public IntPtr fingers;	// AN2K_FingerAmp **
    }/* list of finger amputation informations */

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_AlternateQuality
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 3)]
        public String fing_id;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 4)]
        public String quality;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 6)]
        public String Vendor_ID;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 6)]
        public String Algorithm_ID;
    } /* contains informations about alternate and segmentation quality*/

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_AlternateQualityList{
        public int num_fingers;
        public IntPtr fingers;	// AN2K_AlternateQuality **
    }/* list of finger alternate quality values */

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_PalmQuality
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 3)]
        public String PalmID;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 4)]
        public String quality;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 6)]
        public String Vendor_ID;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 6)]
        public String Algorithm_ID;
    } /* contains informations about alternate and segmentation quality*/

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct AN2K_PalmQualityList{
        public int AlgorhythmsNumber;		// number of algorhythms; can be any
        public IntPtr Algorhythms;	// AN2K_PalmQuality **
    }/* list of palm quality structures */

    /************************************************************
     ************************************************************
     *
     * Utilities
     * 
     ************************************************************
    ************************************************************/
    public class An2k_Utils
    {
        /// <summary>
        /// from struct to IntPtr
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public static IntPtr MarshalToPointer(object data)
        {
            IntPtr buf = Marshal.AllocHGlobal(
                Marshal.SizeOf(data));
            Marshal.StructureToPtr(data,
                buf, false);
            return buf;
        }

        // --------------------------------------------------
        // Deserialization To structures
        // --------------------------------------------------
        /// <summary>
        /// from IntPtr to structure
        /// </summary>
        /// <param name="buf"></param>
        /// <param name="t"></param>
        /// <returns></returns>
        public static object MarshalToStruct(IntPtr buf, Type t)
        {
            return Marshal.PtrToStructure(
                buf, t);
        }

        /// <summary>
        /// from Byte [] to structure
        /// </summary>
        /// <param name="bufArr"></param>
        /// <param name="t"></param>
        /// <returns></returns>
        public static object DeserializeToStruct(Byte[] bufArr, Type t)
        {
            IntPtr buf = Marshal.AllocHGlobal(
                bufArr.Length);

            if (buf == IntPtr.Zero)
            {
                throw (new Exception("ByteArrayToStruct, cannot alloc memory"));
            }

            Marshal.Copy(bufArr, 0, buf, bufArr.Length);

            object Ret = MarshalToStruct(buf, t);
            Marshal.FreeHGlobal(buf);

            return Ret;
        }

        public static String GetStringFromIntPtr(IntPtr Source, int strSize)
        {
			if (strSize > 0)
			{
				String Ret = "";
				Byte[] strArray = new Byte[strSize];
				Marshal.Copy(Source, strArray, 0, strSize);
				for (int i = 0; i < strSize; i++)
				{
					if (strArray[i] != 0)
					{
						Ret += (Char)strArray[i];
					}
					else
					{
						return Ret;
					}
				}
				return Ret;
			}
			else
			{
				IntPtr ptrToSource = new IntPtr(Source.ToInt64());
				byte[] app = new byte[1];
				bool exit = false;
				strSize = 0;
				while (!exit)
				{
					Marshal.Copy(ptrToSource, app, 0, 1);
					if (app[0] == 0) exit = true;
					else
					{
						Int64 a = ptrToSource.ToInt64();
						a++;
						ptrToSource = new IntPtr(a);
						strSize++;
					}
				}
				String Ret = "";
				Byte[] strArray = new Byte[strSize];
				Marshal.Copy(Source, strArray, 0, strSize);
				for (int i = 0; i < strSize; i++)
				{
					if (strArray[i] != 0)
					{
						Ret += (Char)strArray[i];
					}
					else
					{
						return Ret;
					}
				}
				return Ret;
			}
        }

        public static AN2K_FingerAmpList CreateAmpListFromNETList(List<NW_AN2K_FING_AMP> AmpList)
        {
            AN2K_FingerAmpList a = new AN2K_FingerAmpList();
            Byte[] App = new Byte[(IntPtr.Size) * (AmpList.Count)];

            // create pointers
            for (int i = 0; i < AmpList.Count; i++)
            {
                AN2K_FingerAmp AmpListElement = new AN2K_FingerAmp();
                AmpListElement.amp_code = AmpList[i].amp_code;
                AmpListElement.fing_id = AmpList[i].fing_id;

                IntPtr AppPtr = An2k_Utils.MarshalToPointer(AmpListElement);
                IntPtr pAppPtr = An2k_Utils.MarshalToPointer(AppPtr);

                Marshal.Copy(pAppPtr, App, 0 + (IntPtr.Size) * i, IntPtr.Size);

                Marshal.FreeHGlobal(pAppPtr);
            }

            a.fingers = Marshal.AllocHGlobal(App.Length);
            a.num_fingers = (AmpList.Count);
            Marshal.Copy(App, 0, a.fingers, App.Length);

            return a;
        }

        public static List<NW_AN2K_FING_AMP> CreateNETListFromAmpList(AN2K_FingerAmpList AmpList)
        {
            Byte[] App = new Byte[(IntPtr.Size) * (AmpList.num_fingers)];
            Marshal.Copy(AmpList.fingers, App, 0, App.Length);
            List<NW_AN2K_FING_AMP> a = new List<NW_AN2K_FING_AMP>();

            // create pointers
            for (int i = 0; i < AmpList.num_fingers; i++)
            {
                IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

                AN2K_FingerAmp AmpListElement = (AN2K_FingerAmp)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_FingerAmp)));

                NW_AN2K_FING_AMP AmpNETElement = new NW_AN2K_FING_AMP(
                    AmpListElement.fing_id, AmpListElement.amp_code);

                a.Add(AmpNETElement);

                Marshal.FreeHGlobal(pAppPtr);
            }

            return a;
        }

        public static AN2K_FourFingBound CreateSegmListFromNETList(List<NW_AN2K_FING_SEGMENT> SegmList)
        {
            AN2K_FourFingBound a = new AN2K_FourFingBound();
            Byte[] App = new Byte[(IntPtr.Size) * (SegmList.Count)];

            // create pointers
            for (int i = 0; i < SegmList.Count; i++)
            {
                AN2K_FingerSegment SegmListElement = new AN2K_FingerSegment();
                SegmListElement.bottom_bound = SegmList[i].bottom_bound.ToString();
                SegmListElement.left_bound = SegmList[i].left_bound.ToString();
                SegmListElement.right_bound = SegmList[i].right_bound.ToString();
                SegmListElement.top_bound = SegmList[i].top_bound.ToString();
                SegmListElement.fing_id = SegmList[i].fing_id;

                IntPtr AppPtr = An2k_Utils.MarshalToPointer(SegmListElement);
                IntPtr pAppPtr = An2k_Utils.MarshalToPointer(AppPtr);

                Marshal.Copy(pAppPtr, App, 0 + (IntPtr.Size) * i, IntPtr.Size);

                Marshal.FreeHGlobal(pAppPtr);
            }

            a.fingers = Marshal.AllocHGlobal(App.Length);
            a.num_fingers = (SegmList.Count);
            Marshal.Copy(App, 0, a.fingers, App.Length);

            return a;
        }

        public static List<NW_AN2K_FING_SEGMENT> CreateNETListFromSegmList(AN2K_FourFingBound SegmList)
        {
            Byte[] App = new Byte[(IntPtr.Size) * (SegmList.num_fingers)];
            Marshal.Copy(SegmList.fingers, App, 0, App.Length);
            List<NW_AN2K_FING_SEGMENT> a = new List<NW_AN2K_FING_SEGMENT>();

            // create pointers
            for (int i = 0; i < SegmList.num_fingers; i++)
            {
                IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

                AN2K_FingerSegment SegmListElement = (AN2K_FingerSegment)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_FingerSegment)));

                NW_AN2K_FING_SEGMENT AmpNETElement = new NW_AN2K_FING_SEGMENT(
                    SegmListElement.fing_id,
                    Int16.Parse(SegmListElement.left_bound),
                    Int16.Parse(SegmListElement.right_bound),
                    Int16.Parse(SegmListElement.top_bound),
                    Int16.Parse(SegmListElement.bottom_bound)
                );

                a.Add(AmpNETElement);

                Marshal.FreeHGlobal(pAppPtr);
            }

            return a;
        }

        public static AN2K_FourFingerQuality CreateNfiqListFromNETList(List<NW_AN2K_FING_QUALITY> QualList)
        {
            AN2K_FourFingerQuality a = new AN2K_FourFingerQuality();

            // create pointers
            a.fingers = Marshal.AllocHGlobal((IntPtr.Size) * (QualList.Count));
            for (int i = 0; i < QualList.Count; i++)
            {
                AN2K_FingerQuality QualListElement = new AN2K_FingerQuality();
                QualListElement.quality = QualList[i].quality.ToString();
                QualListElement.fing_id = QualList[i].fing_id;

                IntPtr AppPtr = An2k_Utils.MarshalToPointer(QualListElement);
                Marshal.WriteIntPtr(a.fingers, IntPtr.Size * i, AppPtr);
            }

            a.num_fingers = (QualList.Count);
            return a;
        }

        public static List<NW_AN2K_FING_QUALITY> CreateNETListFromNfiqList(AN2K_FourFingerQuality QualList)
        {
            Byte[] App = new Byte[(IntPtr.Size) * (QualList.num_fingers)];
            Marshal.Copy(QualList.fingers, App, 0, App.Length);
            List<NW_AN2K_FING_QUALITY> a = new List<NW_AN2K_FING_QUALITY>();

            // create pointers
            for (int i = 0; i < QualList.num_fingers; i++)
            {
                IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

                AN2K_FingerQuality QualListElement = (AN2K_FingerQuality)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_FingerQuality)));

                NW_AN2K_FING_QUALITY QualNETElement = new NW_AN2K_FING_QUALITY(
                    QualListElement.fing_id,
                    Byte.Parse(QualListElement.quality)
                );

                a.Add(QualNETElement);

                Marshal.FreeHGlobal(pAppPtr);
            }

            return a;
        }

        public static AN2K_AlternateQualityList CreateAltQualListFromNETList(List<NW_AN2K_ALT_QUAL> AltQualList)
        {
            AN2K_AlternateQualityList a = new AN2K_AlternateQualityList();
            Byte[] App = new Byte[(IntPtr.Size) * (AltQualList.Count)];

            // create pointers
            for (int i = 0; i < AltQualList.Count; i++)
            {
                AN2K_AlternateQuality AltQualListElement = new AN2K_AlternateQuality();
                AltQualListElement.Algorithm_ID = AltQualList[i].Algorithm_ID.ToString();
                AltQualListElement.Vendor_ID = AltQualList[i].Vendor_ID.ToString();
                AltQualListElement.quality = AltQualList[i].quality.ToString();
                AltQualListElement.fing_id = AltQualList[i].fing_id;

                IntPtr AppPtr = An2k_Utils.MarshalToPointer(AltQualListElement);
                IntPtr pAppPtr = An2k_Utils.MarshalToPointer(AppPtr);

                Marshal.Copy(pAppPtr, App, 0 + (IntPtr.Size) * i, IntPtr.Size);

                Marshal.FreeHGlobal(pAppPtr);
            }

            a.fingers = Marshal.AllocHGlobal(App.Length);
            a.num_fingers = (AltQualList.Count);
            Marshal.Copy(App, 0, a.fingers, App.Length);

            return a;
        }

        public static List<NW_AN2K_ALT_QUAL> CreateNETListFromAltQualList(AN2K_AlternateQualityList AltQualList)
        {
            Byte[] App = new Byte[(IntPtr.Size) * (AltQualList.num_fingers)];
            Marshal.Copy(AltQualList.fingers, App, 0, App.Length);
            List<NW_AN2K_ALT_QUAL> a = new List<NW_AN2K_ALT_QUAL>();

            // create pointers
            for (int i = 0; i < AltQualList.num_fingers; i++)
            {
                IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

                AN2K_AlternateQuality AltQualListElement =
                    (AN2K_AlternateQuality)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_AlternateQuality)));

                NW_AN2K_ALT_QUAL AltQualNETElement = new NW_AN2K_ALT_QUAL(
                    AltQualListElement.fing_id,
                    Byte.Parse(AltQualListElement.quality),
                    UInt16.Parse(AltQualListElement.Vendor_ID),
                    UInt16.Parse(AltQualListElement.Algorithm_ID)
                );

                a.Add(AltQualNETElement);

                Marshal.FreeHGlobal(pAppPtr);
            }

            return a;
        }

        public static AN2K_PalmQualityList CreatePalmQualListFromNETList(List<NW_AN2K_PALM_QUAL> PalmQualList)
        {
            AN2K_PalmQualityList a = new AN2K_PalmQualityList();
            Byte[] App = new Byte[(IntPtr.Size) * (PalmQualList.Count)];

            // create pointers
            for (int i = 0; i < PalmQualList.Count; i++)
            {
                AN2K_PalmQuality PalmQualListElement = new AN2K_PalmQuality();
                PalmQualListElement.Algorithm_ID = PalmQualList[i].Algorithm_ID.ToString();
                PalmQualListElement.Vendor_ID = PalmQualList[i].Vendor_ID.ToString();
                PalmQualListElement.quality = PalmQualList[i].quality.ToString();
                PalmQualListElement.PalmID = PalmQualList[i].PalmID;

                IntPtr AppPtr = An2k_Utils.MarshalToPointer(PalmQualListElement);
                IntPtr pAppPtr = An2k_Utils.MarshalToPointer(AppPtr);

                Marshal.Copy(pAppPtr, App, 0 + (IntPtr.Size) * i, IntPtr.Size);

                Marshal.FreeHGlobal(pAppPtr);
            }

            a.Algorhythms = Marshal.AllocHGlobal(App.Length);
            a.AlgorhythmsNumber = (PalmQualList.Count);
            Marshal.Copy(App, 0, a.Algorhythms, App.Length);

            return a;
        }

        public static List<NW_AN2K_PALM_QUAL> CreateNETListFromPalmQualList(AN2K_PalmQualityList PalmQualList)
        {
            Byte[] App = new Byte[(IntPtr.Size) * (PalmQualList.AlgorhythmsNumber)];
            Marshal.Copy(PalmQualList.Algorhythms, App, 0, App.Length);
            List<NW_AN2K_PALM_QUAL> a = new List<NW_AN2K_PALM_QUAL>();

            // create pointers
            for (int i = 0; i < PalmQualList.AlgorhythmsNumber; i++)
            {
                IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

                AN2K_PalmQuality PalmQualListElement =
                    (AN2K_PalmQuality)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_PalmQuality)));

                NW_AN2K_PALM_QUAL PalmQualNETElement = new NW_AN2K_PALM_QUAL(
                    PalmQualListElement.PalmID,
                    Byte.Parse(PalmQualListElement.quality),
                    UInt16.Parse(PalmQualListElement.Vendor_ID),
                    UInt16.Parse(PalmQualListElement.Algorithm_ID)
                );

                a.Add(PalmQualNETElement);

                Marshal.FreeHGlobal(pAppPtr);
            }

            return a;
        }

        // Free memory for pointers to pointers
        public static void FreeAllocatedMemForList(IntPtr ListToFree, int NumElem)
        {
            Byte[] App = new Byte[(IntPtr.Size) * NumElem];
            Marshal.Copy(ListToFree, App, 0, App.Length);

            for (int i = 0; i < NumElem; i++)
            {
                IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr,typeof(IntPtr)));

                Marshal.FreeHGlobal(AppPtr);
            }
        }
    }

    /************************************************************
     ************************************************************
     *
     * Direct dll wrapper
     * 
     ************************************************************
    ************************************************************/
    public class An2kdll_NetWrapper
    {
        [DllImport("AN2K_LIB.dll", EntryPoint = "rec_copy")]
        public static extern int rec_copy(
            ref AN2K_Record dst,
            ref AN2K_Record src
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "Create_ANSI_NIST", CharSet = CharSet.Ansi)]
        public static extern int Create_ANSI_NIST(
            IntPtr oansi_nist, // ANSI_NIST**
            int Version,
            String TypeOfTransaction,
            int Priority,
            String DestinationAgency,
            String OriginatingAgency,
            String TransactionControlID,
            String TransactionControlReference,
            float NominalScanningResolution,
            float NominalTransmitResolution,
            String DomainNameID
            );

        [DllImport("AN2K_LIB.dll", EntryPoint = "GetEFTSType1_Info", CharSet = CharSet.Ansi)]
        public static extern int GetEFTSType1_Info(
            ref AN2K_Record record,
            out int Version,
            IntPtr TransactionType,		// pass a char[12]
            out int Priority,
            IntPtr pDestinationAgency,  // char**
            out int DestAgencyLen,
            IntPtr pOriginatingAgency,  // char**
            out int OriginatingAgencyLen,
            IntPtr pTransContrNum,      // char**
            out int TransContrNumLen,
            IntPtr pTransContrRef,      // char**
            out int TransContrRefLen,
            out float NominalScanRes,
            out float NominalTXRes,
            IntPtr pDominion,           // char**
            out int DominionLen
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "UpdateETFStype_1", CharSet = CharSet.Ansi)]
        public static extern int UpdateETFStype_1(
            ref AN2K_Record record,
            float NominalScanningResolution,
            float NominalTransmitResolution
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "image2ETFStype_14", CharSet = CharSet.Ansi)]
        public static extern int image2ETFStype_14(
            IntPtr orecord, // RECORD **
            IntPtr FingerprintImage,
            int			FPImageSize,
            int			FPImageWidth,
            int			FPImageHeight,
            int			FPImagePixelDepth,
            double		ScanResolutionPPMM,
            String      CompressionType,
            int			ImpressionType,
            String ImageSource,
            String Comment,
            String FingerPosition,
            IntPtr AmputationList, // AN2K_FingerAmpList*
            IntPtr boundaries, // AN2K_FourFingBound*
            IntPtr qualities, //AN2K_FourFingerQuality*
            IntPtr segqual, //AN2K_AlternateQualityList*
            IntPtr altqual //AN2K_AlternateQualityList*
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "UpdateETFStype_14", CharSet = CharSet.Ansi)]
        public static extern int UpdateETFStype_14(
            IntPtr record,
            IntPtr FingerprintImage,
            int FPImageSize,
            int FPImageWidth,
            int FPImageHeight,
            double ScanResolutionPPMM,
            IntPtr AmputationList, // AN2K_FingerAmpList*
            IntPtr boundaries, // AN2K_FourFingBound*
            IntPtr qualities, //AN2K_FourFingerQuality*
            IntPtr segqual, //AN2K_AlternateQualityList*
            IntPtr altqual //AN2K_AlternateQualityList*
        );

        // ver 3.3.0.0
        [DllImport("AN2K_LIB.dll", EntryPoint = "UpdateType1415ScaleUnits", CharSet = CharSet.Ansi)]
        public static extern int UpdateType1415ScaleUnits(
            IntPtr recordToUpdate, int newScale
        );
        // end ver 3.3.0.0

        [DllImport("AN2K_LIB.dll", EntryPoint = "Get_EFTStype14_Info", CharSet = CharSet.Ansi)]
        public static extern int Get_EFTStype14_Info(
            ref AN2K_Record record,
            IntPtr FingerprintImage,			// unsigned char**
            out int ImageSize,
            out int w,
            out int h,
            out int PixelDepth,
            out double	Resolution_ppmm,
            IntPtr ImageCompression,	// pass a char[10]
            out int ImageImpression,	//
            IntPtr source_str,			// pass a char[44]
            IntPtr comment,			// comment, pass a char[136]
            IntPtr finger_position_list,	// char **, pass a list of char[3] composed of 6 items
            IntPtr pamp_list,			// FING_AMP_LIST**
            IntPtr pboundaries,		// FOUR_FING_BOUND**
            IntPtr pqualities,			// FOUR_FING_QUALITY**
            IntPtr psegqual,			// ALT_QUAL_LIST**
            IntPtr paltqual			// ALT_QUAL_LIST**
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "image2ETFStype_4", CharSet = CharSet.Ansi)]
        public static extern int image2ETFStype_4(
            IntPtr orecord, // RECORD **
            IntPtr FingerprintImage,
            int			FPImageSize,
            int			ImpressionType,
            int			FingerPosition,
            int			ScanResolutionID,
            int			FPImageWidth,
            int			FPImageHeight,
            String      CompressionType
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "UpdateETFStype_4", CharSet = CharSet.Ansi)]
        public static extern int UpdateETFStype_4(
            ref AN2K_Record orecord,
            IntPtr FingerprintImage,
            int			FPImageSize,
            int			ScanResolutionID,
            int			FPImageWidth,
            int			FPImageHeight
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "Get_EFTStype4_Info", CharSet = CharSet.Ansi)]
        public static extern int Get_EFTStype4_Info(
            ref AN2K_Record orecord,
            IntPtr FingerprintImage,				// unsigned char**
            out int ImageSize,
            out int ImageImpression,	//
            [In, Out] int [] finger_position, // array of int [6]	
            out int ImageResolution,
            out int w,
            out int h,	//
            IntPtr ImageCompression		// pass a char[10]
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "CreateETFStype_2", CharSet = CharSet.Ansi)]
        public static extern int CreateETFStype_2(
            IntPtr orecord, // RECORD**
            String Mak,
            String Modl,
            String SerNo,
            IntPtr amp_list
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "UpdateETFStype_2", CharSet = CharSet.Ansi)]
        public static extern int UpdateETFStype_2(
            ref AN2K_Record orecord,
            String Mak,
            String Modl,
            String SerNo,
            AN2K_FingerAmpList amp_list
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "image2ETFStype_15", CharSet = CharSet.Ansi)]
        public static extern int image2ETFStype_15(
            IntPtr 			orecord, // RECORD*
            IntPtr  		PPImage, // unsigned char*
            int			PPImageSize,
            int			PPImageWidth,
            int			PPImageHeight, 
            int			BitsPerPixel,
            double		PPImageResolutionPPMM,
            String      CompressionType,
            int			ImpressionType,
            String ImageSource,
            String Comment,
            String PalmPosition,
            IntPtr PalmQualitiesList // PALM_QUAL_LIST*
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "Get_EFTStype15_Info", CharSet = CharSet.Ansi)]
        public static extern int Get_EFTStype15_Info(
            ref AN2K_Record record, // RECORD*
            IntPtr PPImage, // unsigned char**
            out int ImageSize, // (can be different from w x h)
            out int w,
            out int h,
            out int BitsPerPixel,
            out double Resolution_ppmm,
            IntPtr ImageCompression,	// pass a char[10]
            out int ImageImpression,	// image impression type (live-scan, roll, etc, see definitions)
            IntPtr source_str,			// pass a char[44]
            IntPtr comment,			// comment, pass a char[136]
            IntPtr PalmPosition,		// string indicating the palm position. See definitions
            IntPtr PalmQualitiesList	// PALM_QUAL_LIST**
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "UpdateETFStype_15", CharSet = CharSet.Ansi)]
        public static extern int UpdateETFStype_15(
            ref AN2K_Record orecord,
            IntPtr  		PPImage, // unsigned char*
            int			PPImageSize,
            int			PPImageWidth,
            int PPImageHeight,
            double PPImageResolutionPPMM,
            IntPtr PalmQualitiesList // PALM_QUAL_LIST*
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "Get_EFTStype2_Info", CharSet = CharSet.Ansi)]
        public static extern int Get_EFTStype2_Info(
            ref AN2K_Record orecord,
            IntPtr Mak, // max 25
            IntPtr Modl, // max 25
            IntPtr SerNo, // max 50
            IntPtr pamp_list // FING_AMP_LIST**
         );

        [DllImport("AN2K_LIB.dll", EntryPoint = "insert_ANSI_NIST_record_ETFS", CharSet = CharSet.Ansi)]
        public static extern int insert_ANSI_NIST_record_ETFS(
            int	RecordIndex,
            ref AN2K_Record RecordToBeInserted,
            IntPtr pAnsiNist // AN2K_Struct *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "delete_ANSI_NIST_record", CharSet = CharSet.Ansi)]
        public static extern int delete_ANSI_NIST_record(
            int RecordIndex,
            IntPtr pAnsiNist // AN2K_Struct *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "read_ANSI_NIST_buffer", CharSet = CharSet.Ansi)]
        public static extern int read_ANSI_NIST_buffer(
            IntPtr  buffer,
            int     bufLen,
            IntPtr oansi_nist // AN2K_Struct **
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "search_ANSI_NIST_record", CharSet = CharSet.Ansi)]
        public static extern int search_ANSI_NIST_record(
            IntPtr oRecList, // AN2K_RecordList **
            uint	RecordType,
            ref AN2K_Struct ansi_nist
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "write_ANSI_NIST_2_buffer", CharSet = CharSet.Ansi)]
        public static extern int write_ANSI_NIST_2_buffer(
            IntPtr          pOutBuffer, // char **
            out int         bufLen,
            ref AN2K_Struct ansi_nist
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_ANSI_NIST", CharSet = CharSet.Ansi)]
        public static extern void free_ANSI_NIST(
            IntPtr ansi_nist // ANSI_NIST *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_ANSI_NIST_record", CharSet = CharSet.Ansi)]
        public static extern void free_ANSI_NIST_record(
            IntPtr record
        ); // RECORD *

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_ANSI_NIST_record_list", CharSet = CharSet.Ansi)]
        public static extern void free_ANSI_NIST_record_list(
            IntPtr record_list // RECORD_LIST *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_FOUR_FING_BOUND", CharSet = CharSet.Ansi)]
        public static extern void free_FOUR_FING_BOUND(
            IntPtr boundaries // FOUR_FING_BOUND *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_FOUR_FING_QUALITY", CharSet = CharSet.Ansi)]
        public static extern void free_FOUR_FING_QUALITY(
            IntPtr qualities // FOUR_FING_QUALITY *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_ANSI_NIST_amp_list", CharSet = CharSet.Ansi)]
        public static extern void free_ANSI_NIST_amp_list(
            IntPtr amp_list // FING_AMP_LIST *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_ANSI_NIST_alt_qual_list", CharSet = CharSet.Ansi)]
        public static extern void free_ANSI_NIST_alt_qual_list(
            IntPtr alt_qual_list // ALT_QUAL_LIST *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_ANSI_NIST_palm_qual_list", CharSet = CharSet.Ansi)]
        public static extern void free_ANSI_NIST_palm_qual_list(
            IntPtr palm_qual_list // PALM_QUAL_LIST *
        );

        [DllImport("AN2K_LIB.dll", EntryPoint = "free_ANSI_NIST_buffer", CharSet = CharSet.Ansi)]
        public static extern void free_ANSI_NIST_buffer(
            IntPtr buffer // void *
        );
    }


}

namespace AN2K_NET_WRAPPER
{
    /************************************************************************/
    /************************************************************************/
    /* SECTION 1: AN2K WRAPPER CONSTANTS                                    */
    /************************************************************************/
    /************************************************************************/

    /// <summary>
    /// Refer to the "COMMON ERROR DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_ERRORS
    {
        public static Int32 NW_AN2K_DLL_NO_ERROR = 0;
        public static Int32 NW_AN2K_DLL_BAD_PARAMETER = -1;
        public static Int32 NW_AN2K_DLL_MEMORY = -2;
        public static Int32 NW_AN2K_DLL_RECORD_NOT_FOUND = -3;
        public static Int32 NW_AN2K_DLL_FIELD_NOT_FOUND = -3;
        public static Int32 NW_AN2K_DLL_GENERIC = -4;
        /// <summary>
        /// This field has no equivalents in the underlying C version of the library.
        /// This error is returned when an exception is encountered while processing. 
        /// The exception message is stored into the 
        /// <see c= "NW_AN2K_DLL_EXCEPTION_STRING"/> string.
        /// </summary>
        public static Int32 NW_AN2K_DLL_EXCEPTION = 255;
        /// <summary>
        /// When an exception is encountered while processing,
        /// its message is stored here.
        /// </summary>
        public static String NW_AN2K_DLL_EXCEPTION_STRING;
    }

    /// <summary>
    /// Refer to the "VERSION DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_VERSIONS
    {
        public static Int32 NW_VERSION_0200 = 200;
        public static Int32 NW_VERSION_0300 = 300;
        public static Int32 NW_VERSION_0400 = 400;
    }

    /// <summary>
    /// Refer to the "PIXEL DEPTH DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_PIXEL_DEPTH_DEFINITIONS
    {
        public static Int32 NW_PIXEL_DEPTH_8BPP = 8;
    }


    /// <summary>
    /// Refer to the "COMPRESSION ALGORITHMS DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_COMP_ALGORITHMS
    {
        public static String NW_COMP_NONE = ("NONE");
        public static String NW_BIN_COMP_NONE = ("0");
        public static String NW_COMP_WSQ = ("WSQ20");
        public static String NW_BIN_COMP_WSQ = ("1");
        public static String NW_COMP_JPEGB = ("JPEGB");
        public static String NW_BIN_COMP_JPEGB = ("2");
        public static String NW_COMP_JPEGL = ("JPEGL");
        public static String NW_BIN_COMP_JPEGL = ("3");
        public static String NW_JPG2K_COMP = ("JP2");
        public static String NW_BIN_COMP_JPEG2K = ("4");
        public static String NW_JPG2KL_COMP = ("JP2L");
        public static String NW_BIN_COMP_JPEG2KL = ("5");
        public static String NW_PNG_COMP = ("PNG");
        public static String NW_BIN_COMP_PNG = ("6");
    }


    /// <summary>
    /// Refer to the "IMPRESSION CODE DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_IMPRESSION_CODES
    {
        public static Int32 NW_ETFS_14_LIVE_SCAN_PLAN = 0;
        public static Int32 NW_ETFS_14_LIVE_SCAN_ROLLED = 1;
        public static Int32 NW_ETFS_14_NON_LIVE_SCAN_PLAN = 2;
        public static Int32 NW_ETFS_14_NON_LIVE_SCAN_ROLLED = 3;
        public static Int32 NW_ETFS_14_LATENT_IMPRESSION = 4;
        public static Int32 NW_ETFS_14_LATENT_TRACING = 5;
        public static Int32 NW_ETFS_14_LATENT_PHOTO = 6;
        public static Int32 NW_ETFS_14_LATENT_LIFT = 7;
    }


    /// <summary>
    /// Refer to the "FINGER POSITION DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_FINGER_POSITIONS
    {
        public static String NW_ETFS_14_FGP_UNKNOWN = "0";
        public static String NW_ETFS_14_FGP_RIGHT_THUMB = ("1");
        public static String NW_ETFS_14_FGP_RIGHT_INDEX = ("2");
        public static String NW_ETFS_14_FGP_RIGHT_MIDDLE = ("3");
        public static String NW_ETFS_14_FGP_RIGHT_RING = ("4");
        public static String NW_ETFS_14_FGP_RIGHT_LITTLE = ("5");
        public static String NW_ETFS_14_FGP_LEFT_THUMB = ("6");
        public static String NW_ETFS_14_FGP_LEFT_INDEX = ("7");
        public static String NW_ETFS_14_FGP_LEFT_MIDDLE = ("8");
        public static String NW_ETFS_14_FGP_LEFT_RING = ("9");
        public static String NW_ETFS_14_FGP_LEFT_LITTLE = ("10");
        public static String NW_ETFS_14_FGP_PLAIN_RIGHT_THUMB = ("11");
        public static String NW_ETFS_14_FGP_PLAIN_LEFT_THUMB = ("12");
        public static String NW_ETFS_14_FGP_RIGHT_4_FING = ("13");
        public static String NW_ETFS_14_FGP_LEFT_4_FING = ("14");
        public static String NW_ETFS_14_FGP_2_THUMBS = ("15");
    }

    // Ver 1.1.0.0
    /// <summary>
    /// Refer to the "PALM POSITION DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_PALM_POSITIONS
    {
        public static String NW_ETFS_15_PLP_UNKNOWN = ("20");
        public static String NW_ETFS_15_PLP_RIGTH_FULL_PALM = ("21");
        public static String NW_ETFS_15_PLP_RIGHT_WRITER_PALM = ("22");
        public static String NW_ETFS_15_PLP_LEFT_FULL_PALM = ("23");
        public static String NW_ETFS_15_PLP_LEFT_WRITER_PALM = ("24");
        public static String NW_ETFS_15_PLP_RIGTH_LOWER_PALM = ("25");
        public static String NW_ETFS_15_PLP_RIGHT_UPPER_PALM = ("26");
        public static String NW_ETFS_15_PLP_LEFT_LOWER_PALM = ("27");
        public static String NW_ETFS_15_PLP_LEFT_UPPER_PALM = ("28");
        public static String NW_ETFS_15_PLP_RIGHT_OTHER = ("29");
        public static String NW_ETFS_15_PLP_LEFT_OTHER = ("30");
        public static String NW_ETFS_15_PLP_RIGHT_INTERDIGITAL = ("31");
        public static String NW_ETFS_15_PLP_RIGHT_THENAR = ("32");
        public static String NW_ETFS_15_PLP_RIGHT_HYPOTHENAR = ("33");
        public static String NW_ETFS_15_PLP_LEFT_INTERDIGITAL = ("34");
        public static String NW_ETFS_15_PLP_LEFT_THENAR = ("35");
        public static String NW_ETFS_15_PLP_LEFT_HYPOTHENAR = ("36");
    }
    // end Ver 1.1.0.0


    /// <summary>
    /// Refer to the "AMPUTATION CODE DEFINITIONS" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_AMPUTATION_CODES
    {
        public static String NW_AMPUTATION = ("XX");
        public static String NW_UNABLE_TO_PRINT = ("UP");
    }


    /// <summary>
    /// Refer to the "ETFS type-4 parameters" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_TYPE_4_RESOLUTION_PARAMETERS
    {
        public static Int32 NW_ETFS_TYPE4_MAX_RES_VAL = 1;
        public static Int32 NW_ETFS_TYPE4_MIN_RES_VAL = 0;
    }


    /// <summary>
    /// Refer to the "ANSI-NIST 2007 records" section in the An2k_dll.h file
    /// in the underlying C version of the library.
    /// </summary>
    public class NW_AN2K_AN2007_RECORDS
    {
        public static Int32 NW_TYPE_1_ID = 1;
        public static Int32 NW_TYPE_2_ID = 2;
        public static Int32 NW_TYPE_3_ID = 3;
        public static Int32 NW_TYPE_4_ID = 4;
        public static Int32 NW_TYPE_5_ID = 5;
        public static Int32 NW_TYPE_6_ID = 6;
        public static Int32 NW_TYPE_7_ID = 7;
        public static Int32 NW_TYPE_8_ID = 8;
        public static Int32 NW_TYPE_9_ID = 9;
        public static Int32 NW_TYPE_10_ID = 10;
        public static Int32 NW_TYPE_11_ID = 11;
        public static Int32 NW_TYPE_12_ID = 12;
        public static Int32 NW_TYPE_13_ID = 13;
        public static Int32 NW_TYPE_14_ID = 14;
        public static Int32 NW_TYPE_15_ID = 15;
        public static Int32 NW_TYPE_16_ID = 16;
        public static Int32 NW_TYPE_17_ID = 17;
        public static Int32 NW_TYPE_99_ID = 99;
    }

    /************************************************************************/
    /************************************************************************/
    /* SECTION 2: AN2K WRAPPER DATA STRUCTURES                              */
    /************************************************************************/
    /************************************************************************/

    /***********************************************************************/
    /* FingerSegment structure */
    /***********************************************************************/
    /// <summary> This is the equivalent of the "FING_SEGMENT" structure in the C version of the AN2K library.
    /// Here the information about the bounding box of a segment inside a slap are collected.</summary>
    public class NW_AN2K_FING_SEGMENT
    {
        /*//////////////////////////////////
        //// CONSTRUCTORS //////////////////
        //////////////////////////////////*/

        /// <summary> Constructor overload:	set the properties with the given values	
        /// </summary>
        /// <param name ="fing_id"> Should be one within the 
        /// <see c= "NW_AN2K_FINGER_POSITIONS"/> class
        /// </param>
        /// <param name ="left_bound"> Segment left bound x coordinate from the left-top corner
        /// </param>
        /// <param name ="right_bound"> Segment right bound x coordinate from the left-top corner
        /// </param>
        /// <param name ="top_bound"> Segment top bound y coordinate from the left-top corner
        /// </param>
        /// <param name ="bottom_bound"> Segment bottom bound y coordinate from the left-top corner
        /// </param>
        public NW_AN2K_FING_SEGMENT(String in_fing_id,
            Int16 in_left_bound,
            Int16 in_right_bound,
            Int16 in_top_bound,
            Int16 in_bottom_bound
            )
        {
            Identifier = in_fing_id;
            b_left = in_left_bound;
            b_right = in_right_bound;
            b_top = in_top_bound;
            b_bottom = in_bottom_bound;
        }

        /*//////////////////////////////////
        //// PROPERTIES ////////////////////
        //////////////////////////////////*/

        /// <summary> finger id: Should be one within the 
        /// <see c= "NW_AN2K_FINGER_POSITIONS"/> class	
        /// </summary>
        public String fing_id
        {
            get
            {
                return Identifier;
            }
            set
            {
                Identifier = value;
            }
        }

        /// <summary> left bound x coordinate from the left-top corner	
        /// </summary>
        public Int16 left_bound
        {
            get
            {
                return b_left;
            }
            set
            {
                b_left = value;
            }
        }

        /// <summary> right bound x coordinate from the left-top corner	
        /// </summary>
        public Int16 right_bound
        {
            get
            {
                return b_right;
            }
            set
            {
                b_right = value;
            }
        }

        /// <summary> top bound y coordinate from the left-top corner	
        /// </summary>
        public Int16 top_bound
        {
            get
            {
                return b_top;
            }
            set
            {
                b_top = value;
            }
        }

        /// <summary> bottom bound y coordinate from the left-top corner	
        /// </summary>
        public Int16 bottom_bound
        {
            get
            {
                return b_bottom;
            }
            set
            {
                b_bottom = value;
            }
        }

        // RESERVED: where boundary box coordinates are stored
        private Int16 b_left;
        private Int16 b_right;
        private Int16 b_top;
        private Int16 b_bottom;
        private String Identifier;
    }

    /***********************************************************************/
    /* NFIQ quality structure */
    /***********************************************************************/
    /// <summary> This is the equivalent of the "FING_QUALITY" structure in the C version of the AN2K library.
    /// Structure  where NFIQ quality info are stored </summary>
    public class NW_AN2K_FING_QUALITY
    {
        ////////////////////////////////////
        //// CONSTRUCTORS //////////////////
        ////////////////////////////////////
        /// <summary> Constructor overload:	set the properties with the given values	
        /// </summary>
        /// <param name ="fing_id"> Should be one within the 
        /// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class
        /// </param>
        /// <param name ="quality"> NFIQ quality value
        /// </param>
        public NW_AN2K_FING_QUALITY(String in_fing_id, Byte in_quality)
        {
            Identifier = in_fing_id;
            NFIQ_quality = in_quality;
        }

        ////////////////////////////////////
        //// PROPERTIES ////////////////////
        ////////////////////////////////////

        /// <summary> finger id: Should be one within the 
        /// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class	
        /// </summary>
        public String fing_id
        {
            get
            {
                return Identifier;
            }
            set
            {
                Identifier = value;
            }
        }

        /// <summary> NFIQ quality value	
        /// </summary>
        public Byte quality
        {
            get
            {
                return NFIQ_quality;
            }
            set
            {
                NFIQ_quality = value;
            }
        }

        // RESERVED: quality value is stored
        private String Identifier;
        private Byte NFIQ_quality;
    };

	/***********************************************************************/
	/* Amputation data structure */
	/***********************************************************************/
	/// <summary> This is the equivalent of the "FING_AMP" structure in the C version of the AN2K library.
	/// Structure  where missing finger indication and reason are stored </summary>
	public class NW_AN2K_FING_AMP
	{
		////////////////////////////////////
		//// CONSTRUCTORS //////////////////
		////////////////////////////////////
		/// <summary> Constructor overload:	set the properties with the given values	
		/// </summary>
		/// <param name ="in_fing_id"> Should be one within the 
		/// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class
		/// </param>
		/// <param name ="in_AmpCode"> Should be one within the 
		/// <see cref = "NW_AN2K_AMPUTATION_CODES"/> class
		/// </param>
        public NW_AN2K_FING_AMP(String in_fing_id, String in_AmpCode)
        {
            Identifier = in_fing_id;
            AmputationCode = in_AmpCode;
        }

		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////

		/// <summary> finger id: Should be one within the 
		/// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class	
		/// </summary>
        public String fing_id
        {
            get
            {
                return Identifier;
            }
            set
            {
                Identifier = value;
            }
        }

		/// <summary> Amputation code: Should be one within the 
		/// <see cref = "NW_AN2K_AMPUTATION_CODES"/> class	
		/// </summary>
		public String	amp_code 
        {
            get
            {
                return AmputationCode;
            }
            set
            {
                AmputationCode = value;
            }
        }

		// RESERVED: amputation code value is stored
		private String	Identifier;
		private String	AmputationCode;
	}

	/***********************************************************************/
	/* Alternate quality structure */
	/***********************************************************************/
	/// <summary> This is the equivalent of the "ALT_QUAL" structure in the C version of the AN2K library.
	/// Structure  where quality parameters calculated with algorithms different from NFIQ are stored </summary>
	public class NW_AN2K_ALT_QUAL
	{
		////////////////////////////////////
		//// CONSTRUCTORS //////////////////
		////////////////////////////////////
		/// <summary> Constructor overload:	set the properties with the given values	
		/// </summary>
		/// <param name ="fing_id"> Should be one within the 
		/// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class
		/// </param>
		/// <param name ="quality"> Quality value
		/// </param>
		/// <param name ="Vendor_ID"> IBIA vendor_ID code (www.ibia.org)
		/// </param>
		/// <param name ="Algorithm_ID"> IBIA Algorithm_ID code (www.ibia.org)
		/// </param>
        public NW_AN2K_ALT_QUAL(String in_fing_id, Byte in_quality, UInt16 in_Vendor_ID, UInt16 in_Algorithm_ID)
        {
            Identifier = in_fing_id;
            ALT_quality = in_quality;
            Vendor = in_Vendor_ID;
            Algorithm = in_Algorithm_ID;
        }

		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////

        /// <summary> finger id: Should be one within the 
        /// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class	
        /// </summary>
        public String fing_id
        {
            get
            {
                return Identifier;
            }
            set
            {
                Identifier = value;
            }
        }

        /// <summary> NFIQ quality value	
        /// </summary>
        public Byte quality
        {
            get
            {
                return ALT_quality;
            }
            set
            {
                ALT_quality = value;
            }
        }
		
		/// <summary> IBIA vendor_ID code (www.ibia.org)	
		/// </summary>
		/// <remarks> Normally should be in range 0...65535.
		/// </remarks>
		public UInt16 Vendor_ID
        {
            get
            {
                return Vendor;
            }
            set
            {
                Vendor = value;
            }
        }
		
		/// <summary> IBIA Algorithm_ID code (www.ibia.org)	
		/// </summary>
		/// <remarks> Normally should be in range 0...65535.
		/// </remarks>
        public UInt16 Algorithm_ID
        {
            get
            {
                return Algorithm;
            }
            set
            {
                Algorithm = value;
            }
        }

		// RESERVED: alternate quality data are stored
		private String 	    Identifier;
		private Byte		ALT_quality;
		private UInt16		Vendor;			
		private UInt16		Algorithm;
	}

	// Ver 1.1.0.0: added quality structure for palmprint
	/***********************************************************************/
	/* Palm quality structure */
	/***********************************************************************/
	/// <summary> This is the equivalent of the "PALM_QUAL" structure in the C version of the AN2K library. </summary>
	public class NW_AN2K_PALM_QUAL
	{
		////////////////////////////////////
		//// CONSTRUCTORS //////////////////
		////////////////////////////////////
		/// <summary> Constructor overload:	set the properties with the given values	
		/// </summary>
		/// <param name ="PalmID"> Should be one within the 
		/// <see cref = "NW_AN2K_PALM_POSITIONS"/> class
		/// </param>
		/// <param name ="quality"> Quality value
		/// </param>
		/// <param name ="Vendor_ID"> IBIA vendor_ID code (www.ibia.org)
		/// </param>
		/// <param name ="Algorithm_ID"> IBIA Algorithm_ID code (www.ibia.org)
		/// </param>
		public NW_AN2K_PALM_QUAL(
            String in_PalmID,
            Byte in_quality,
            UInt16 in_Vendor_ID,
            UInt16 in_Algorithm_ID
		)
        {
            Identifier = in_PalmID;
            Palm_quality = in_quality;
            Vendor = in_Vendor_ID;
            Algorithm = in_Algorithm_ID;
        }

		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////		
		/// <summary> finger id: Should be one within the 
		/// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class	
		/// </summary>
        public String PalmID
        {
            get
            {
                return Identifier;
            }
            set
            {
                Identifier = value;
            }
        }

        /// <summary> NFIQ quality value	
        /// </summary>
        public Byte quality
        {
            get
            {
                return Palm_quality;
            }
            set
            {
                Palm_quality = value;
            }
        }
		
		/// <summary> IBIA vendor_ID code (www.ibia.org)	
		/// </summary>
		/// <remarks> Normally should be in range 0...65535.
		/// </remarks>
		public UInt16 Vendor_ID
        {
            get
            {
                return Vendor;
            }
            set
            {
                Vendor = value;
            }
        }

        /// <summary> IBIA Algorithm_ID code (www.ibia.org)	
        /// </summary>
        /// <remarks> Normally should be in range 0...65535.
        /// </remarks>
        public UInt16 Algorithm_ID
        {
            get
            {
                return Algorithm;
            }
            set
            {
                Algorithm = value;
            }
        }

		// RESERVED: alternate quality data are stored
		private String 	    Identifier;
		private Byte		Palm_quality;
		private UInt16		Vendor;			
		private UInt16		Algorithm;
	};

    /***********************************************************************/
	/* ANSI-NIST 2007 Generic record */
	/***********************************************************************/
	/// <summary> ANSI-NIST 2007 Generic record.
	///	It is the equivalent of the "RECORD" structure within the C version of the library. For its use see the remarks.</summary>
	/// <remarks> It is the master type where the other records defined in this namespace inherit from.
	/// NORMALLY A USER SHOULD NOT CREATE OBJECTS DIRECTLY FROM THIS CLASS, that at the moment is for internal use only,
	/// but he should use one of the other types defined inside this namespace for objects creation.
	/// From the other hand he can use the properties defined inside this class, that are common to all the records defined
	/// inside this namespace.
	/// </remarks>
	public class NW_AN2K_RECORD
	{
		////////////////////////////////////
		//// CONSTRUCTORS //////////////////
		////////////////////////////////////

		/// <summary> Constructor overload:	set RecPtr to IntPtr.Zero (NULL)	
		/// </summary>
		/// <remarks> normally this is the constructor that should be called 
		/// </remarks>
        public NW_AN2K_RECORD()
        {
            try
            {
                this.RecPtr = IntPtr.Zero;
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_RECORD(): " + ex.Message;
                throw (new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING));
            }
        }

		/// <summary> Constructor overload:	set RecPtr to a low level RECORD structure.	
		/// </summary>
		/// <remarks> ONLY FOR INTERNAL: USER IS HIGHLY RECOMMENDED TO NOT USE THIS FUNCTION AS A CONSTRUCTOR 
		/// </remarks>
        public NW_AN2K_RECORD(IntPtr Rec)
        {
            try
            {
                if (this.RecPtr != IntPtr.Zero)
                {
                    //AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));
                    An2kdll_NetWrapper.free_ANSI_NIST_record(this.RecPtr);
                }
                this.RecPtr = Rec;
            }
            catch (Exception)
            {
            }
        }
		
		////////////////////////////////////
		///// DESTRUCTORS //////////////////
		////////////////////////////////////

		/// <summary> Destructor overload	
		/// </summary>
        ~NW_AN2K_RECORD()
        {
            try
            {
                if (this.RecPtr != IntPtr.Zero)
                {
                    //AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));
                    An2kdll_NetWrapper.free_ANSI_NIST_record(this.RecPtr);
                }
            }
            catch (Exception)
            {
            }
        }

		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////

		/// <summary> Gets the record type
		/// </summary>
		public UInt32 type{
            get
            {
                try
                {
                    if (this.RecPtr != IntPtr.Zero)
                    {
                        An2k_Net_Wrapper_Utilities.AN2K_Record rec;
                        rec = (An2k_Net_Wrapper_Utilities.AN2K_Record)Marshal.PtrToStructure(
                            this.RecPtr,
                            typeof(An2k_Net_Wrapper_Utilities.AN2K_Record)
                        );
                        return (UInt32)(rec.type);
                    }
                    else
                    {
                        return 0;
                    }
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_RECORD type get: " + ex.Message;
                    throw (new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING));
                }
            } // returns the record type
		}

		/// <summary> Gets the record fields number
		/// </summary>
		public Int32 num_fields {
			get
            {
                try
                {
                    if (this.RecPtr != IntPtr.Zero)
                    {
                        An2k_Net_Wrapper_Utilities.AN2K_Record rec;
                        rec = (An2k_Net_Wrapper_Utilities.AN2K_Record)Marshal.PtrToStructure(
                            this.RecPtr,
                            typeof(An2k_Net_Wrapper_Utilities.AN2K_Record)
                        );
                        return (Int32)(rec.num_fields);
                    }
                    else
                    {
                        return 0;
                    }
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_RECORD num_fields get: " + ex.Message;
                    throw (new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING));
                }
            } // returns the record fields number
		}

		/// <summary> Pointer to a low level RECORD structure, that allows a user to access the information stored.
		/// </summary>
		/// <remarks> RESERVED FOR INTERNAL: USER IS HIGHLY RECOMMENDED TO NOT USE THIS PROPERTY
		/// </remarks>
        //property IntPtr  RecPtr {
        //    void set(IntPtr ); // RESERVED: Do not call
        //    IntPtr  get(void); // RESERVED: returns a pointer to a low level RECORD structure
        //}
		public IntPtr RecPtr{
            get
            {
                return this.Ptr;
            }
            set
            {
                this.Ptr = value;
            }
		}


        protected IntPtr Ptr; // RESERVED: where RecPtr data are stored
	};

    /***********************************************************************/
	/* ANSI-NIST 2007 TYPE-14 record */
	/***********************************************************************/
	/// <summary> ImageInfo Struct: list of info stored into a Type-14 record </summary>
	/// <remarks>
	/// This class has no equivalents inside the C version of the library, but it is used as a property of the
	/// NW_AN2K_Type14_RECORD, where methods for managing ANSI-NIST type-14 records are collected. 
	/// In particular, getting this property from the NW_AN2K_Type14_RECORD is the equivalent of using the
	/// "Get_EFTStype14_Info" defined into the C version of the library, of which this class is the equivalent of the output.
	/// Infact the members of this class are in a 1:1 correspondence (regarding their meaning) with the output of the 
	/// "Get_EFTStype14_Info" function. Therefore they keep the same name, except for:
	/// - amp_list: corresponds to pamp_list
	/// - segmentation_list:  corresponds to pboundaries
	/// - NFIQ_quality_list: corresponds to pqualities
	/// - segmentation_quality_list:  corresponds to psegqual
	/// - alternate_quality_list: corresponds to paltqual
	/// This because in C these parameters are passed as pointers to pointers, so their name reflects (with a "p" as a prefix) 
	/// this way of passing.
	/// </remarks>
	public class NW_AN2K_Type14_StoredInfo
	{
		////////////////////////////////////
		//// CONSTRUCTORS //////////////////
		////////////////////////////////////
		/// <summary> Constructor overload:	Allocates memory for the finger position list	
		/// </summary>
        public NW_AN2K_Type14_StoredInfo()
        {
            this.finger_position_list = new List<String > (6);
        }
		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////
		/// <summary> Image Frame bytes	
		/// </summary>
	    public Byte [] ImagePtr;

		/// <summary> Image size in bytes: can be different from (Width * Height)
		/// </summary>
		public Int32 ImageSize;
		
		/// <summary> Image horizontal dimension
		/// </summary>
		public Int32 w;
		
		/// <summary> Image vertical dimension
		/// </summary>
		public Int32 h;
		
		/// <summary> Image Bits per pixel
		/// </summary>
		public Int32 PixelDepth;
		
		/// <summary> Image resolution in pixel/mm
		/// </summary>		
		public Double Resolution_ppmm;
		
		/// <summary> Image compression algorithm identifier: should be one of the values defined into the 
		/// <see cref = "NW_AN2K_COMP_ALGORITHMS"/> class.
		/// </summary>		
		public String  ImageCompression;
		
		/// <summary> Image impression identifier: should be one of the values defined into the 
		/// <see cref = "NW_AN2K_IMPRESSION_CODES"/> class.
		/// </summary>		
		public Int32 ImageImpression;
		
		/// <summary> string identifying the originating source of the image: its max length should be 43.
		/// </summary>		
		public String source_str;

		/// <summary> comment: its max length should be 135.
		/// </summary>		
		public String comment;
		
		/// <summary> List of strings indicating the finger position. Item values should be within the 
		/// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class.
		/// </summary>		
		public List <String > finger_position_list;

		/// <summary> List of structures containing missing fingers indication and reason.
		/// </summary>
		public List<NW_AN2K_FING_AMP > amp_list;
		
		/// <summary> List of structures containing finger segments bounding box information.
		/// </summary>
		public List<NW_AN2K_FING_SEGMENT > segmentation_list;

		/// <summary> List of structures containing fingers NFIQ quality information.
		/// </summary>
		public List<NW_AN2K_FING_QUALITY > NFIQ_quality_list;
		
		/// <summary> List of structures containing finger segments quality information.
		/// </summary>
		public List<NW_AN2K_ALT_QUAL > segmentation_quality_list;
		
		/// <summary> List of structures containing finger alternate quality information.
		/// </summary>
		public List<NW_AN2K_ALT_QUAL > alternate_quality_list;
	};



    /// <summary> Class representing a Type-14 record.
    /// Inherits from the <see cref = "NW_AN2K_RECORD"/> class. This class, together with the information available in the 
    /// <see cref = "NW_AN2K_RECORD"/>, collects all the methods for managing a record 14 that have an equivalent in the C
    /// version of the library, such as the "Get_EFTStype14_Info" or the "UpdateETFStype_14" functions.
    ///</summary>
    public class NW_AN2K_Type14_RECORD : NW_AN2K_RECORD
    {
        ////////////////////////////////////
        //// PROPERTIES ////////////////////
        ////////////////////////////////////
        /// <summary> 
        /// <para> Gets a
        /// <see cref = "NW_AN2K_Type14_StoredInfo"/> structure where information stored in this record are collected.
        /// </para>
        /// <para> CALL <see cref = "NW_image2ETFStype_14"/> BEFORE ACCESSING THIS PROPERTY. 
        /// If a null value is returned, that means that information are not present since the
        /// <see cref = "NW_AN2K_Type14_RECORD.NW_image2ETFStype_14"/> method has never been called. In such a case see the
        /// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message
        /// </para>
        /// </summary>
        /// <remarks>
        /// Getting this property is equivalent to calling the "Get_EFTStype14_Info" defined into the C version of the library.
        /// The difference is that a user has not directly a return code, but in case something was wrong:
        /// 1) Info will be null
        /// 2) A more detailed explanation of what happened can be retrieved into the 
        ///    <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
        /// </remarks>
        public NW_AN2K_Type14_StoredInfo Info
        {
            get
            {
                try
                {
                    if (this.RecPtr != IntPtr.Zero)
                    {
                        /////////////////////////////////////////////
                        // Get Record struct from *
                        /////////////////////////////////////////////
                        AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));

                        /////////////////////////////////////////////
                        // Prepare container for fingerprint image
                        /////////////////////////////////////////////
                        IntPtr FingImage = IntPtr.Zero;
                        IntPtr pFingImage = An2k_Utils.MarshalToPointer(FingImage);
                        int ImgSize, w, h, PDepth, ImgImp;
                        double ResPpm;

                        /////////////////////////////////////////////
                        // Prepare container for strings
                        /////////////////////////////////////////////
                        IntPtr ImgComp = Marshal.AllocHGlobal(10);
                        IntPtr SourceStr = Marshal.AllocHGlobal(44);
                        IntPtr Comm = Marshal.AllocHGlobal(136);

                        /////////////////////////////////////////////
                        // Prepare container for amputation
                        /////////////////////////////////////////////
                        IntPtr AmpList = new IntPtr();
                        IntPtr pAmpList = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(AmpList);

                        /////////////////////////////////////////////
                        // Prepare container for segments
                        /////////////////////////////////////////////
                        IntPtr Bound = new IntPtr();
                        IntPtr pBound = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(Bound);

                        /////////////////////////////////////////////
                        // Prepare container for NFIQ quality
                        /////////////////////////////////////////////
                        IntPtr Qual = new IntPtr();
                        IntPtr pQual = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(Qual);

                        /////////////////////////////////////////////
                        // Prepare container for quality of
                        // segmentation
                        /////////////////////////////////////////////
                        IntPtr SegQual = new IntPtr();
                        IntPtr pSegQual = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(SegQual);

                        /////////////////////////////////////////////
                        // Prepare container for alternate quality
                        /////////////////////////////////////////////
                        IntPtr AltQual = new IntPtr();
                        IntPtr pAltQual = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(AltQual);

                        /////////////////////////////////////////////
                        // Prepare container for finger(s)
                        // position list
                        /////////////////////////////////////////////
                        IntPtr pFingPosList = Marshal.AllocHGlobal(18);

                        int RetVal = An2kdll_NetWrapper.Get_EFTStype14_Info(
                            ref x,
                            pFingImage,
                            out ImgSize, out w, out h, out PDepth,
                            out ResPpm,
                            ImgComp,
                            out ImgImp,
                            SourceStr,
                            Comm,
                            pFingPosList,
                            pAmpList, pBound, pQual, pSegQual, pAltQual);

                        if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                        {
                            NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Get_EFTStype14_Info returned: " + RetVal);
                            return null;
                        }

                        /////////////////////////////////////////////
                        // Get finger(s) position list
                        /////////////////////////////////////////////
                        Byte[] FingPosListArray = new Byte[18];
                        Marshal.Copy(pFingPosList, FingPosListArray, 0, 18);

                        List<String> FingPosList = new List<string>();
                        for (int i = 0; i < 6; i++)
                        {
                            if (FingPosListArray[i * 3] != 0)
                            {
                                String Dummy = "";
                                Dummy += (Char)(FingPosListArray[(i * 3)]);
                                Dummy += (Char)(FingPosListArray[(i * 3) + 1]);
                                Dummy += (Char)(FingPosListArray[(i * 3) + 2]);

                                FingPosList.Add(Dummy);
                            }
                            else
                            {
                                i = 6;
                            }
                        }

                        /////////////////////////////////////////////
                        // Get amputation list
                        /////////////////////////////////////////////
                        AmpList = (IntPtr)(An2k_Utils.MarshalToStruct(pAmpList, typeof(IntPtr)));
                        AN2K_FingerAmpList oa;
                        List<NW_AN2K_FING_AMP> FingAmpList;
                        if (AmpList != IntPtr.Zero)
                        {
                            oa = (AN2K_FingerAmpList)(An2k_Utils.MarshalToStruct(AmpList, typeof(AN2K_FingerAmpList)));
                            FingAmpList = An2k_Utils.CreateNETListFromAmpList(oa);

                            /////////////////////////////////////////////
                            // free
                            /////////////////////////////////////////////
                            An2kdll_NetWrapper.free_ANSI_NIST_amp_list(AmpList);
                            Marshal.FreeHGlobal(pAmpList);
                        }
                        else
                        {
                            oa = new AN2K_FingerAmpList();
                            oa.num_fingers = 0;
                            FingAmpList = new List<NW_AN2K_FING_AMP>();
                        }

                        /////////////////////////////////////////////
                        // Get Segmentation list
                        /////////////////////////////////////////////
                        Bound = (IntPtr)(An2k_Utils.MarshalToStruct(pBound, typeof(IntPtr)));
                        AN2K_FourFingBound ob;
                        List<NW_AN2K_FING_SEGMENT> FingSegmList;
                        if (Bound != IntPtr.Zero)
                        {
                            ob = (AN2K_FourFingBound)(An2k_Utils.MarshalToStruct(Bound, typeof(AN2K_FourFingBound)));
                            FingSegmList = An2k_Utils.CreateNETListFromSegmList(ob);

                            /////////////////////////////////////////////
                            // free
                            /////////////////////////////////////////////
                            An2kdll_NetWrapper.free_FOUR_FING_BOUND(Bound);
                            Marshal.FreeHGlobal(pBound);
                        }
                        else
                        {
                            ob = new AN2K_FourFingBound();
                            ob.num_fingers = 0;
                            FingSegmList = new List<NW_AN2K_FING_SEGMENT>();
                        }

                        /////////////////////////////////////////////
                        // Get Quality list
                        /////////////////////////////////////////////
                        Qual = (IntPtr)(An2k_Utils.MarshalToStruct(pQual, typeof(IntPtr)));
                        AN2K_FourFingerQuality oc;
                        List<NW_AN2K_FING_QUALITY> FingQualList;
                        if (Qual != IntPtr.Zero)
                        {
                            oc = (AN2K_FourFingerQuality)(An2k_Utils.MarshalToStruct(Qual, typeof(AN2K_FourFingerQuality)));
                            FingQualList = An2k_Utils.CreateNETListFromNfiqList(oc);

                            /////////////////////////////////////////////
                            // free
                            /////////////////////////////////////////////
                            An2kdll_NetWrapper.free_FOUR_FING_QUALITY(Qual);
                            Marshal.FreeHGlobal(pQual);
                        }
                        else
                        {
                            oc = new AN2K_FourFingerQuality();
                            oc.num_fingers = 0;
                            FingQualList = new List<NW_AN2K_FING_QUALITY>();
                        }

                        /////////////////////////////////////////////
                        // Get Segment Quality list
                        /////////////////////////////////////////////
                        SegQual = (IntPtr)(An2k_Utils.MarshalToStruct(pSegQual, typeof(IntPtr)));
                        AN2K_AlternateQualityList od;
                        List<NW_AN2K_ALT_QUAL> FingSegQualList;
                        if (SegQual != IntPtr.Zero)
                        {
                            od = (AN2K_AlternateQualityList)(An2k_Utils.MarshalToStruct(SegQual, typeof(AN2K_AlternateQualityList)));
                            FingSegQualList = An2k_Utils.CreateNETListFromAltQualList(od);

                            /////////////////////////////////////////////
                            // free
                            /////////////////////////////////////////////
                            An2kdll_NetWrapper.free_ANSI_NIST_alt_qual_list(SegQual);
                            Marshal.FreeHGlobal(pSegQual);
                        }
                        else
                        {
                            od = new AN2K_AlternateQualityList();
                            od.num_fingers = 0;
                            FingSegQualList = new List<NW_AN2K_ALT_QUAL>();
                        }

                        /////////////////////////////////////////////
                        // Get Alternate Quality list
                        /////////////////////////////////////////////
                        AltQual = (IntPtr)(An2k_Utils.MarshalToStruct(pAltQual, typeof(IntPtr)));
                        AN2K_AlternateQualityList oe;
                        List<NW_AN2K_ALT_QUAL> FingAltQualList;
                        if (AltQual != IntPtr.Zero)
                        {
                            oe = (AN2K_AlternateQualityList)(An2k_Utils.MarshalToStruct(AltQual, typeof(AN2K_AlternateQualityList)));
                            FingAltQualList = An2k_Utils.CreateNETListFromAltQualList(oe);

                            /////////////////////////////////////////////
                            // free
                            /////////////////////////////////////////////
                            An2kdll_NetWrapper.free_ANSI_NIST_alt_qual_list(AltQual);
                            Marshal.FreeHGlobal(pAltQual);
                        }
                        else
                        {
                            oe = new AN2K_AlternateQualityList();
                            oe.num_fingers = 0;
                            FingAltQualList = new List<NW_AN2K_ALT_QUAL>();
                        }

                        NW_AN2K_Type14_StoredInfo ValToRet = new NW_AN2K_Type14_StoredInfo();
                        ValToRet.alternate_quality_list = FingAltQualList;
                        ValToRet.amp_list = FingAmpList;
                        ValToRet.comment = An2k_Utils.GetStringFromIntPtr(Comm, 136);
                        ValToRet.finger_position_list = FingPosList;
                        ValToRet.h = h;
                        ValToRet.ImageCompression = An2k_Utils.GetStringFromIntPtr(ImgComp, 10);
                        ValToRet.ImageImpression = ImgImp;
                        if (ImgSize > 0)
                        {
                            FingImage = (IntPtr)An2k_Utils.MarshalToStruct(pFingImage, typeof(IntPtr));
                            if (FingImage != IntPtr.Zero)
                            {
                                ValToRet.ImagePtr = new Byte[ImgSize];
                                Marshal.Copy(FingImage, ValToRet.ImagePtr, 0, ImgSize);
                            }
                            else
                            {
                                ValToRet.ImagePtr = null;
                            }
                        }
                        else
                        {
                            ValToRet.ImagePtr = null;
                        }
                        ValToRet.ImageSize = ImgSize;
                        ValToRet.NFIQ_quality_list = FingQualList;
                        ValToRet.PixelDepth = PDepth;
                        ValToRet.Resolution_ppmm = ResPpm;
                        ValToRet.segmentation_list = FingSegmList;
                        ValToRet.segmentation_quality_list = FingSegQualList;
                        ValToRet.source_str = An2k_Utils.GetStringFromIntPtr(SourceStr, 44);
                        ValToRet.w = w;

                        ////////////////////////////////////////////////
                        // free allocated memory
                        ////////////////////////////////////////////////
                        // IMAGE
                        if (!(FingImage.Equals(IntPtr.Zero)))
                        {
                            An2kdll_NetWrapper.free_ANSI_NIST_buffer(FingImage);
                        }
                        Marshal.FreeHGlobal(pFingImage);

                        // STRINGS
                        Marshal.FreeHGlobal(ImgComp);
                        Marshal.FreeHGlobal(SourceStr);
                        Marshal.FreeHGlobal(Comm);

                        return ValToRet;
                    }
                    else
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("No info in record");
                        return null;
                    }
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Exception in getting NW_AN2K_Type14_RECORD.info: " + ex.Message);
                    return null;
                }
            }
        }
        ////////////////////////////////////
        /////// METHODS ////////////////////
        ////////////////////////////////////
        /// <summary> 
        /// <para> Creates a type-14 record from the input values. For their meaning see the
        /// <see cref = "NW_AN2K_Type14_StoredInfo"/> structure, while for their name correspondence with the parameters of
        /// the "image2ETFStype_14" function of the C version of the library see the comments below.
        /// </para>
        /// </summary>
        /// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
        /// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
        ///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
        /// </returns>
        /// <param name = "ImagePtr"> Corresponds to the "idata" parameter of "image2ETFStype_14".</param>
        /// <param name = "ImageSize"> Corresponds to the "ilen" parameter of "image2ETFStype_14".</param>
        /// <param name = "w"> Corresponds to the "iw" parameter of "image2ETFStype_14".</param>
        /// <param name = "h"> Corresponds to the "ih" parameter of "image2ETFStype_14".</param>
        /// <param name = "PixelDepth"> Corresponds to the "id" parameter of "image2ETFStype_14".</param>
        /// <param name = "Resolution_ppmm"> Corresponds to the "ppmm" parameter of "image2ETFStype_14".</param>
        /// <param name = "ImageCompression"> Corresponds to the "img_comp" parameter of "image2ETFStype_14".</param>
        /// <param name = "ImageImpression"> Corresponds to the "img_imp" parameter of "image2ETFStype_14".</param>
        /// <param name = "source_str"> Corresponds to the "source_str" parameter of "image2ETFStype_14".</param>
        /// <param name = "comment"> Corresponds to the "comment" parameter of "image2ETFStype_14".</param>
        /// <param name = "finger_position"> Even if up to 6 positions could be entered for the FGP field in the type-14 record
        /// (infact in <see cref = "NW_AN2K_Type14_StoredInfo"/> a list of strings is specified), for our purpose
        /// (tenprint capture for BioAPI 2.0), finger position must be well specified, so just one value can be	entered.
        /// Corresponds to the "finger_position" parameter of "image2ETFStype_14".</param>
        /// <param name = "amp_list"> Corresponds to the "amp_list" parameter of "image2ETFStype_14".
        /// This list must contain EXACTLY the number of amputated fingers.</param>
        /// <param name = "segmentation_list"> Corresponds to the "boundaries" parameter of "image2ETFStype_14".</param>
        /// <param name = "NFIQ_quality_list"> Corresponds to the "qualities" parameter of "image2ETFStype_14".</param>
        /// <param name = "segmentation_quality_list"> Corresponds to the "segqual" parameter of "image2ETFStype_14".</param>
        /// <param name = "alternate_quality_list"> Corresponds to the "altqual" parameter of "image2ETFStype_14".</param>
        /// <remarks>
        ///	This function is the equivalent of the "image2ETFStype_14" function of the C version of the library.
        /// </remarks>
        public Int32 NW_image2ETFStype_14(
            Byte[] ImagePtr,
            Int32 ImageSize,
            Int32 w,
            Int32 h,
            Byte PixelDepth,
            Double Resolution_ppmm,
            String ImageCompression,
            Int32 ImageImpression,
            String source_str,
            String comment,
            String finger_position,
            List<NW_AN2K_FING_AMP> amp_list,
            List<NW_AN2K_FING_SEGMENT> segmentation_list,
            List<NW_AN2K_FING_QUALITY> NFIQ_quality_list,
            List<NW_AN2K_ALT_QUAL> segmentation_quality_list,
            List<NW_AN2K_ALT_QUAL> alternate_quality_list
        )
        {
            try 
            {
                IntPtr ImgPtr;
                Int32 RetVal;
                
                if (ImageCompression == null || finger_position == null) {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Bad Image Compression or Finger Position");
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                }            
                

                //////////////////////////////////////////////
                // FINGER IMAGE
                //////////////////////////////////////////////
                if (ImagePtr != null && ImageSize > 0){
                    // check arrays
                    if (ImagePtr.Length < ImageSize) {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Number Of pixels less than ImageSize");
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                    }
                    // alloc
                    ImgPtr = Marshal.AllocHGlobal(ImageSize);
                    if (ImgPtr == IntPtr.Zero)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot allocate memory";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    // copy
                    Marshal.Copy(ImagePtr,0,ImgPtr,ImageSize);
                }
                else {
                    ImgPtr = IntPtr.Zero;
                    ImageSize = 0;
                }
                
                // Image Source is a string...
                // comment is a string...
                // Image compression is a string...
                // Finger Position is a string...
                
                ////////////////////////////////////////////////
                // AMPUTATION LIST
                ////////////////////////////////////////////////
                AN2K_FingerAmpList a;
                IntPtr inpAmpList = IntPtr.Zero;
                if (amp_list != null && amp_list.Count > 0)
                {
                    a = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateAmpListFromNETList(amp_list);
                    if (a.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create amputation list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpAmpList = An2k_Utils.MarshalToPointer(a);
                }
                else
                {
                    a = new AN2K_FingerAmpList();
                    a.num_fingers = 0;
                }
                
                ////////////////////////////////////////////////
                // SEGMENTATION LIST
                ////////////////////////////////////////////////
                AN2K_FourFingBound b;
                IntPtr inpSegmList = IntPtr.Zero;
                if (segmentation_list != null && segmentation_list.Count > 0)
                {
                    b = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateSegmListFromNETList(segmentation_list);
                    if (b.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create segmentation list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpSegmList = An2k_Utils.MarshalToPointer(b);
                }
                else
                {
                    b = new AN2K_FourFingBound();
                    b.num_fingers = 0;
                }               
                
                ////////////////////////////////////////////////
                // NFIQ quality LIST
                ////////////////////////////////////////////////
                AN2K_FourFingerQuality c;
                IntPtr inpNfiqList = IntPtr.Zero;
                if (NFIQ_quality_list != null && NFIQ_quality_list.Count > 0)
                {
                    c = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateNfiqListFromNETList(NFIQ_quality_list);
                    if (c.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        if (b.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create Nfiq quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpNfiqList = An2k_Utils.MarshalToPointer(c);
                }
                else
                {
                    c = new AN2K_FourFingerQuality();
                    c.num_fingers = 0;
                }            
                
                ////////////////////////////////////////////////
                // SEGMENTATION quality LIST
                ////////////////////////////////////////////////
                AN2K_AlternateQualityList d;
                IntPtr inpSegQualList = IntPtr.Zero;
                if (segmentation_quality_list != null && segmentation_quality_list.Count > 0)
                {
                    d = An2k_Utils.CreateAltQualListFromNETList(segmentation_quality_list);
                    if (d.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        if (b.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                        }
                        if (c.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(c.fingers, c.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create SEGMENTATION quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpSegQualList = An2k_Utils.MarshalToPointer(d);
                }
                else
                {
                    d = new AN2K_AlternateQualityList();
                    d.num_fingers = 0;
                }               
                
                ////////////////////////////////////////////////
                // ALTERNATE quality LIST
                ////////////////////////////////////////////////
                AN2K_AlternateQualityList ef;
                IntPtr inpAltQualList = IntPtr.Zero;
                if (alternate_quality_list != null && alternate_quality_list.Count > 0)
                {
                    ef = An2k_Utils.CreateAltQualListFromNETList(alternate_quality_list);
                    if (ef.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        if (b.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                        }
                        if (c.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(c.fingers, c.num_fingers);
                        }
                        if (d.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(d.fingers, d.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create ALTERNATE quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpAltQualList = An2k_Utils.MarshalToPointer(ef);
                }
                else
                {
                    ef = new AN2K_AlternateQualityList();
                    ef.num_fingers = 0;
                }

                //////////////////////////////////////////////
                // RECORD GENERATION
                //////////////////////////////////////////////
                IntPtr ppRec = Marshal.AllocHGlobal(IntPtr.Size);
                
                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                RetVal = An2kdll_NetWrapper.image2ETFStype_14(
                    ppRec, // REC **
                    ImgPtr,
                    ImageSize, w, h,
                    PixelDepth, Resolution_ppmm,
                    ImageCompression,
                    ImageImpression,
                    source_str, comment,
                    finger_position,
                    inpAmpList,
                    inpSegmList, inpNfiqList, inpSegQualList, inpAltQualList
                );

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (!(ImgPtr.Equals(IntPtr.Zero)))
                {
                    Marshal.FreeHGlobal(ImgPtr);
                }
                if (a.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                }
                if (b.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                }
                if (c.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(c.fingers, c.num_fingers);
                }
                if (d.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(d.fingers, d.num_fingers);
                }
                if (ef.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(ef.fingers, ef.num_fingers);
                }
                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level image2ETFStype_14 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                ////////////////////////////////////////////////
                // copy record
                ////////////////////////////////////////////////
                this.RecPtr = (IntPtr)(Marshal.ReadIntPtr(ppRec));
                if (this.RecPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level MarshalToStruct function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_image2ETFStype_14: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }

        /// <summary> 
        /// <para> Updates a type-14 or 15 record from the input values. For their name correspondence with the parameters of
        /// the "UpdateType1415ScaleUnits" function of the C version of the library see the comments below.
        /// </para>
        /// </summary>
        /// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
        /// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
        ///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
        /// </returns>
        /// <remarks>
        ///	This function is the equivalent of the "UpdateType1415ScaleUnits" function of the C version of the library.
        /// </remarks>
        public Int32 NW_UpdateType1415ScaleUnits(
            Int32 NewScale
        )
        {
            try
            {
                Int32 RetVal = 0;

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                //AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));
                //for (int i = 0; i < 100; i++)
                {
                    RetVal = An2kdll_NetWrapper.UpdateType1415ScaleUnits(
                        this.RecPtr, NewScale
                    );
                }

                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level UpdateType1415ScaleUnits function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "UpdateType1415ScaleUnits: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }

        /// <summary> 
        /// <para> Updates a type-14 record from the input values. For their meaning see the
        /// <see cref = "NW_AN2K_Type14_StoredInfo"/> structure, while for their name correspondence with the parameters of
        /// the "UpdateETFStype_14" function of the C version of the library see the comments below.
        /// </para>
        /// </summary>
        /// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
        /// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
        ///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
        /// </returns>
        /// <param name = "ImagePtr"> Corresponds to the "idata" parameter of "UpdateETFStype_14".</param>
        /// <param name = "ImageSize"> Corresponds to the "ilen" parameter of "UpdateETFStype_14".</param>
        /// <param name = "w"> Corresponds to the "iw" parameter of "UpdateETFStype_14".</param>
        /// <param name = "h"> Corresponds to the "ih" parameter of "UpdateETFStype_14".</param>
        /// <param name = "Resolution_ppmm"> Corresponds to the "ppmm" parameter of "UpdateETFStype_14".</param>
        /// <param name = "amp_list"> Corresponds to the "amp_list" parameter of "UpdateETFStype_14".
        /// This list must contain EXACTLY the number of amputated fingers.</param>
        /// <param name = "segmentation_list"> Corresponds to the "boundaries" parameter of "UpdateETFStype_14".</param>
        /// <param name = "NFIQ_quality_list"> Corresponds to the "qualities" parameter of "UpdateETFStype_14".</param>
        /// <param name = "segmentation_quality_list"> Corresponds to the "segqual" parameter of "UpdateETFStype_14".</param>
        /// <param name = "alternate_quality_list"> Corresponds to the "altqual" parameter of "UpdateETFStype_14".</param>
        /// <remarks>
        ///	This function is the equivalent of the "UpdateETFStype_14" function of the C version of the library.
        /// </remarks>
        public Int32 NW_UpdateETFStype_14(
            Byte[] ImagePtr,
            Int32 ImageSize,
            Int32 w,
            Int32 h,
            Double Resolution_ppmm,
            List<NW_AN2K_FING_AMP> amp_list,
            List<NW_AN2K_FING_SEGMENT> segmentation_list,
            List<NW_AN2K_FING_QUALITY> NFIQ_quality_list,
            List<NW_AN2K_ALT_QUAL> segmentation_quality_list,
            List<NW_AN2K_ALT_QUAL> alternate_quality_list
        )
        {
            try
            {
                IntPtr ImgPtr = IntPtr.Zero;
                Int32 RetVal = 0;
                //////////////////////////////////////////////
                // FINGER IMAGE
                //////////////////////////////////////////////
                if (ImagePtr != null && ImageSize > 0)
                {
                    // check arrays
                    if (ImagePtr.Length < ImageSize)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Number Of pixels less than ImageSize");
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                    }
                    // alloc
                    ImgPtr = Marshal.AllocHGlobal(ImageSize);
                    if (ImgPtr == IntPtr.Zero)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot allocate memory";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    // copy
                    Marshal.Copy(ImagePtr, 0, ImgPtr, ImageSize);
                }
                else
                {
                    ImgPtr = IntPtr.Zero;
                    ImageSize = 0;
                }

                // Image Source is a string...
                // comment is a string...
                // Image compression is a string...
                // Finger Position is a string...

                ////////////////////////////////////////////////
                // AMPUTATION LIST
                ////////////////////////////////////////////////
                AN2K_FingerAmpList a;
                IntPtr inpAmpList = IntPtr.Zero;
                a = new AN2K_FingerAmpList();
                a.num_fingers = 0;
                if (amp_list != null && amp_list.Count > 0)
                {
                    a = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateAmpListFromNETList(amp_list);
                    if (a.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create amputation list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpAmpList = An2k_Utils.MarshalToPointer(a);
                }
                else
                {
                    a = new AN2K_FingerAmpList();
                    a.num_fingers = 0;
                }

                ////////////////////////////////////////////////
                // SEGMENTATION LIST
                ////////////////////////////////////////////////
                AN2K_FourFingBound b;
                IntPtr inpSegmList = IntPtr.Zero;
                b = new AN2K_FourFingBound();
                b.num_fingers = 0;
                if (segmentation_list != null && segmentation_list.Count > 0)
                {
                    b = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateSegmListFromNETList(segmentation_list);
                    if (b.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create segmentation list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpSegmList = An2k_Utils.MarshalToPointer(b);
                }
                else
                {
                    b = new AN2K_FourFingBound();
                    b.num_fingers = 0;
                }

                ////////////////////////////////////////////////
                // NFIQ quality LIST
                ////////////////////////////////////////////////
                AN2K_FourFingerQuality c;
                IntPtr inpNfiqList = IntPtr.Zero;
                c = new AN2K_FourFingerQuality();
                c.num_fingers = 0;
                if (NFIQ_quality_list != null && NFIQ_quality_list.Count > 0)
                {
                    c = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateNfiqListFromNETList(NFIQ_quality_list);
                    if (c.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        if (b.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create Nfiq quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpNfiqList = An2k_Utils.MarshalToPointer(c);
                }
                else
                {
                    c = new AN2K_FourFingerQuality();
                    c.num_fingers = 0;
                }

                ////////////////////////////////////////////////
                // SEGMENTATION quality LIST
                ////////////////////////////////////////////////
                AN2K_AlternateQualityList d;
                IntPtr inpSegQualList = IntPtr.Zero;
                d = new AN2K_AlternateQualityList();
                d.num_fingers = 0;
                if (segmentation_quality_list != null && segmentation_quality_list.Count > 0)
                {
                    d = An2k_Utils.CreateAltQualListFromNETList(segmentation_quality_list);
                    if (d.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        if (b.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                        }
                        if (c.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(c.fingers, c.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create SEGMENTATION quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpSegQualList = An2k_Utils.MarshalToPointer(d);
                }
                else
                {
                    d = new AN2K_AlternateQualityList();
                    d.num_fingers = 0;
                }

                ////////////////////////////////////////////////
                // ALTERNATE quality LIST
                ////////////////////////////////////////////////
                AN2K_AlternateQualityList ef;
                IntPtr inpAltQualList = IntPtr.Zero;
                ef = new AN2K_AlternateQualityList();
                ef.num_fingers = 0;
                if (alternate_quality_list != null && alternate_quality_list.Count > 0)
                {
                    ef = An2k_Utils.CreateAltQualListFromNETList(alternate_quality_list);
                    if (ef.num_fingers == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        if (a.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                        }
                        if (b.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                        }
                        if (c.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(c.fingers, c.num_fingers);
                        }
                        if (d.num_fingers > 0)
                        {
                            An2k_Utils.FreeAllocatedMemForList(d.fingers, d.num_fingers);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create ALTERNATE quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpAltQualList = An2k_Utils.MarshalToPointer(ef);
                }
                else
                {
                    ef = new AN2K_AlternateQualityList();
                    ef.num_fingers = 0;
                }

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                //AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));
                //for (int i = 0; i < 100; i++)
                {
                    RetVal = An2kdll_NetWrapper.UpdateETFStype_14(
                        this.RecPtr, // REC **
                        ImgPtr,
                        ImageSize, w, h, Resolution_ppmm,
                        inpAmpList,
                        inpSegmList, inpNfiqList, inpSegQualList, inpAltQualList
                    );
                }

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (!(ImgPtr.Equals(IntPtr.Zero)))
                {
                    Marshal.FreeHGlobal(ImgPtr);
                }
                if (a.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                }
                if (b.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(b.fingers, b.num_fingers);
                }
                if (c.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(c.fingers, c.num_fingers);
                }
                if (d.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(d.fingers, d.num_fingers);
                }
                if (ef.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(ef.fingers, ef.num_fingers);
                }

                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level UpdateETFStype_14 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_image2ETFStype_14: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }
    };
	/***********************************************************************/
	/* ANSI-NIST 2007 TYPE-15 record */
	/***********************************************************************/
	/// <summary> ImageInfo Struct: list of info stored into a Type-15 record </summary>
	/// <remarks>
	/// This class has no equivalents inside the C version of the library, but it is used as a property of the
	/// NW_AN2K_Type15_RECORD, where methods for managing ANSI-NIST type-15 records are collected. 
	/// In particular, getting this property from the NW_AN2K_Type15_RECORD is the equivalent of using the
	/// "Get_EFTStype15_Info" defined into the C version of the library, of which this class is the equivalent of the output.
	/// Infact the members of this class are in a 1:1 correspondence (regarding their meaning) with the output of the 
	/// "Get_EFTStype15_Info" function. Therefore they keep the same name.
	/// </remarks>
	public class NW_AN2K_Type15_StoredInfo
	{
		////////////////////////////////////
		//// CONSTRUCTORS //////////////////
		////////////////////////////////////
	
		/// <summary> Constructor overload:	Allocates memory for the finger position list	
		/// </summary>
        public NW_AN2K_Type15_StoredInfo()
        {
        }
		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////
		/// <summary> Image Frame bytes	
		/// </summary>
		public Byte [] ImagePtr;

		/// <summary> Image size in bytes: can be different from (Width * Height)
		/// </summary>
        public Int32 ImageSize;
		
		/// <summary> Image horizontal dimension
		/// </summary>
    
        public Int32 w;
		
		/// <summary> Image vertical dimension
		/// </summary>
    
        public Int32 h;
		
		/// <summary> Image Bits per pixel
		/// </summary>
    
        public Int32 PixelDepth;
		
		/// <summary> Image resolution in pixel/mm
		/// </summary>		
    
        public Double Resolution_ppmm;
		
		/// <summary> Image compression algorithm identifier: should be one of the values defined into the 
		/// <see cref = "NW_AN2K_COMP_ALGORITHMS"/> class.
		/// </summary>		
    
        public String ImageCompression;
		
		/// <summary> Image impression identifier: should be one of the values defined into the 
		/// <see cref = "NW_AN2K_IMPRESSION_CODES"/> class.
		/// </summary>		
    
        public Int32 ImageImpression;
		
		/// <summary> string identifying the originating source of the image: its max length should be 43.
		/// </summary>		
    
        public String source_str;

		/// <summary> comment: its max length should be 135.
		/// </summary>		
    
        public String comment;

		/// <summary> PalmPosition as defined in
		/// <see cref = "NW_AN2K_PALM_POSITIONS"/> class.
		/// </summary>		
    
        public String PalmPosition;
		
		/// <summary> List of structures containing finger alternate quality information.
		/// </summary>
    
        public List<NW_AN2K_PALM_QUAL> palm_quality_list;
	};

    /// <summary> Class representing a Type-15 record.
	/// Inherits from the <see cref = "NW_AN2K_RECORD"/> class. This class, together with the information available in the 
	/// <see cref = "NW_AN2K_RECORD"/>, collects all the methods for managing a record 15 that have an equivalent in the C
	/// version of the library, such as the "Get_EFTStype15_Info" or the "UpdateETFStype_15" functions.
	///</summary>
	public class NW_AN2K_Type15_RECORD:NW_AN2K_RECORD
	{
		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////
		/// <summary> 
		/// <para> Gets a
		/// <see cref = "NW_AN2K_Type15_StoredInfo"/> structure where information stored in this record are collected.
		/// </para>
		/// <para> CALL <see cref = "NW_image2ETFStype_15"/> BEFORE ACCESSING THIS PROPERTY. 
		/// If a null value is returned, that means that information are not present since the
		/// <see cref = "NW_AN2K_Type15_RECORD.NW_image2ETFStype_15"/> method has never been called. In such a case see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message
		/// </para>
		/// </summary>
		/// <remarks>
		/// Getting this property is equivalent to calling the "Get_EFTStype15_Info" defined into the C version of the library.
		/// The difference is that a user has not directly a return code, but in case something was wrong:
		/// 1) Info will be null
		/// 2) A more detailed explanation of what happened can be retrieved into the 
		///    <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </remarks>
        /// 
		public NW_AN2K_Type15_StoredInfo Info 
        {
            get
            {
                try
                {
                    if (this.RecPtr != IntPtr.Zero)
                    {
                        /////////////////////////////////////////////
                        // Get Record struct from **
                        /////////////////////////////////////////////
                        AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));

                        /////////////////////////////////////////////
                        // Prepare container for fingerprint image
                        /////////////////////////////////////////////
                        IntPtr FingImage = IntPtr.Zero;
                        IntPtr pFingImage = An2k_Utils.MarshalToPointer(FingImage);
                        int ImgSize, w, h, PDepth, ImgImp;
                        double ResPpm;

                        /////////////////////////////////////////////
                        // Prepare container for strings
                        /////////////////////////////////////////////
                        IntPtr ImgComp = Marshal.AllocHGlobal(10);
                        IntPtr SourceStr = Marshal.AllocHGlobal(44);
                        IntPtr Comm = Marshal.AllocHGlobal(136);
                        IntPtr PalmPosition = Marshal.AllocHGlobal(10);


                        /////////////////////////////////////////////
                        // Prepare container for quality of
                        // segmentation
                        /////////////////////////////////////////////
                        IntPtr PalmQual = new IntPtr();
                        IntPtr pPalmQual = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(PalmQual);


                        int RetVal = An2kdll_NetWrapper.Get_EFTStype15_Info(
                            ref x,
                            pFingImage,
                            out ImgSize, out w, out h, out PDepth,
                            out ResPpm,
                            ImgComp,
                            out ImgImp,
                            SourceStr,
                            Comm,
                            PalmPosition,
                            PalmQual);

                        if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                        {
                            NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Get_EFTStype14_Info returned: " + RetVal);
                            return null;
                        }

                        /////////////////////////////////////////////
                        // Get Segment Quality list
                        /////////////////////////////////////////////
                        PalmQual = (IntPtr)(An2k_Utils.MarshalToStruct(pPalmQual, typeof(IntPtr)));
                        AN2K_PalmQualityList od;
                        List<NW_AN2K_PALM_QUAL> PalmQualList;
                        if (PalmQual != IntPtr.Zero)
                        {
                            od = (AN2K_PalmQualityList)(An2k_Utils.MarshalToStruct(PalmQual, typeof(AN2K_PalmQualityList)));
                            PalmQualList = An2k_Utils.CreateNETListFromPalmQualList(od);

                            /////////////////////////////////////////////
                            // free
                            /////////////////////////////////////////////
                            An2kdll_NetWrapper.free_ANSI_NIST_palm_qual_list(PalmQual);
                            Marshal.FreeHGlobal(pPalmQual);
                        }
                        else
                        {
                            od = new AN2K_PalmQualityList();
                            od.AlgorhythmsNumber = 0;
                            PalmQualList = new List<NW_AN2K_PALM_QUAL>();
                        }

                        NW_AN2K_Type15_StoredInfo ValToRet = new NW_AN2K_Type15_StoredInfo();
                        ValToRet.comment = An2k_Utils.GetStringFromIntPtr(Comm, 136);
                        ValToRet.PalmPosition = An2k_Utils.GetStringFromIntPtr(PalmPosition, 10);
                        ValToRet.palm_quality_list = PalmQualList;
                        ValToRet.h = h;
                        ValToRet.ImageCompression = An2k_Utils.GetStringFromIntPtr(ImgComp, 10);
                        ValToRet.ImageImpression = ImgImp;
                        if (ImgSize > 0)
                        {
                            FingImage = (IntPtr)An2k_Utils.MarshalToStruct(pFingImage, typeof(IntPtr));
                            if (FingImage != IntPtr.Zero)
                            {
                                ValToRet.ImagePtr = new Byte[ImgSize];
                                Marshal.Copy(FingImage, ValToRet.ImagePtr, 0, ImgSize);
                            }
                            else
                            {
                                ValToRet.ImagePtr = null;
                            }
                        }
                        else
                        {
                            ValToRet.ImagePtr = null;
                        }
                        ValToRet.ImageSize = ImgSize;
                        ValToRet.PixelDepth = PDepth;
                        ValToRet.Resolution_ppmm = ResPpm;
                        ValToRet.source_str = An2k_Utils.GetStringFromIntPtr(SourceStr, 44);
                        ValToRet.w = w;

                        ////////////////////////////////////////////////
                        // free allocated memory
                        ////////////////////////////////////////////////
                        // IMAGE
                        if (!(FingImage.Equals(IntPtr.Zero)))
                        {
                            An2kdll_NetWrapper.free_ANSI_NIST_buffer(FingImage);
                        }
                        Marshal.FreeHGlobal(pFingImage);

                        // STRINGS
                        Marshal.FreeHGlobal(ImgComp);
                        Marshal.FreeHGlobal(SourceStr);
                        Marshal.FreeHGlobal(Comm);
                        Marshal.FreeHGlobal(PalmPosition);

                        return ValToRet;
                    }
                    else
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("No info in record");
                        return null;
                    }
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Exception in getting NW_AN2K_Type15_RECORD.info: " + ex.Message);
                    return null;
                }
            }
        }
		////////////////////////////////////
		/////// METHODS ////////////////////
		////////////////////////////////////
		/// <summary> 
		/// <para> Creates a type-15 record from the input values. For their meaning see the
		/// <see cref = "NW_AN2K_Type15_StoredInfo"/> structure, while for their name correspondence with the parameters of
		/// the "image2ETFStype_15" function of the C version of the library see the comments below.
		/// </para>
		/// </summary>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		/// <param name = "ImagePtr"> Corresponds to the "PPImage" parameter of "image2ETFStype_15".</param>
		/// <param name = "ImageSize"> Corresponds to the "PPImageSize" parameter of "image2ETFStype_15".</param>
		/// <param name = "w"> Corresponds to the "PPImageWidth" parameter of "image2ETFStype_15".</param>
		/// <param name = "h"> Corresponds to the "PPImageHeight" parameter of "image2ETFStype_15".</param>
		/// <param name = "PixelDepth"> Corresponds to the "BitsPerPixel" parameter of "image2ETFStype_15".</param>
		/// <param name = "Resolution_ppmm"> Corresponds to the "PPImageResolutionPPMM" parameter of "image2ETFStype_15".</param>
		/// <param name = "ImageCompression"> Corresponds to the "CompressionType" parameter of "image2ETFStype_15".</param>
		/// <param name = "ImageImpression"> Corresponds to the "ImpressionType" parameter of "image2ETFStype_15".</param>
		/// <param name = "source_str"> Corresponds to the "ImageSource" parameter of "image2ETFStype_15".</param>
		/// <param name = "comment"> Corresponds to the "Comment" parameter of "image2ETFStype_15".</param>
		/// <param name = "PalmPosition"> Corresponds to the "PalmPosition" parameter of "image2ETFStype_15".</param>
		/// <param name = "PalmQualitiesList"> Corresponds to the "PalmQualitiesList" parameter of "image2ETFStype_15".</param>
		/// <remarks>
		///	This function is the equivalent of the "image2ETFStype_15" function of the C version of the library.
		/// </remarks>
        public Int32 NW_image2ETFStype_15(
            Byte[] ImagePtr,
            Int32 ImageSize,
            Int32 w,
            Int32 h,
            Byte PixelDepth,
            Double Resolution_ppmm,
            String ImageCompression,
            Int32 ImageImpression,
            String source_str,
            String comment,
            String PalmPosition,
            List<NW_AN2K_PALM_QUAL> PalmQualitiesList)
        {
            try
            {
                IntPtr ImgPtr;
                Int32 RetVal;

                //////////////////////////////////////////////
                // RECORD GENERATION
                //////////////////////////////////////////////
                IntPtr y = new IntPtr();
                IntPtr oRec = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(y);

                if (ImageCompression == null || PalmPosition == null)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Bad Image Compression or Palm Position");
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                }


                //////////////////////////////////////////////
                // FINGER IMAGE
                //////////////////////////////////////////////
                if (ImagePtr != null && ImageSize > 0)
                {
                    // check arrays
                    if (ImagePtr.Length < ImageSize)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Number Of pixels less than ImageSize");
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                    }
                    // alloc
                    ImgPtr = Marshal.AllocHGlobal(ImageSize);
                    if (ImgPtr == IntPtr.Zero)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot allocate memory";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    // copy
                    Marshal.Copy(ImagePtr, 0, ImgPtr, ImageSize);
                }
                else
                {
                    ImgPtr = IntPtr.Zero;
                    ImageSize = 0;
                }

                ////////////////////////////////////////////////
                // PALM quality LIST
                ////////////////////////////////////////////////
                AN2K_PalmQualityList d;
                IntPtr inpPalmQualList = IntPtr.Zero;
                if (PalmQualitiesList != null && PalmQualitiesList.Count > 0)
                {
                    d = An2k_Utils.CreatePalmQualListFromNETList(PalmQualitiesList);
                    if (d.AlgorhythmsNumber == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create PALM quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpPalmQualList = An2k_Utils.MarshalToPointer(d);
                }
                else
                {
                    d = new AN2K_PalmQualityList();
                    d.AlgorhythmsNumber = 0;
                }

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                RetVal = An2kdll_NetWrapper.image2ETFStype_15(
                    oRec, // REC **
                    ImgPtr,
                    ImageSize, w, h,
                    PixelDepth, Resolution_ppmm,
                    ImageCompression,
                    ImageImpression,
                    source_str, comment,
                    PalmPosition, inpPalmQualList
                );

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (!(ImgPtr.Equals(IntPtr.Zero)))
                {
                    Marshal.FreeHGlobal(ImgPtr);
                }
                if (d.AlgorhythmsNumber > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(d.Algorhythms, d.AlgorhythmsNumber);
                }
                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level image2ETFStype_15 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                ////////////////////////////////////////////////
                // copy record
                ////////////////////////////////////////////////
                this.RecPtr = (IntPtr)(An2k_Utils.MarshalToStruct(oRec, typeof(IntPtr))); ;
                if (this.RecPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level MarshalToStruct function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_image2ETFStype_15: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }

		/// <summary> 
		/// <para> Updates a type-15 record from the input values. For their meaning see the
		/// <see cref = "NW_AN2K_Type15_StoredInfo"/> structure, while for their name correspondence with the parameters of
		/// the "UpdateETFStype_15" function of the C version of the library see the comments below.
		/// </para>
		/// </summary>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		/// <param name = "ImagePtr"> Corresponds to the "PPImage" parameter of "UpdateETFStype_15".</param>
		/// <param name = "ImageSize"> Corresponds to the "PPImageSize" parameter of "UpdateETFStype_15".</param>
		/// <param name = "w"> Corresponds to the "PPImageWidth" parameter of "UpdateETFStype_15".</param>
		/// <param name = "h"> Corresponds to the "PPImageHeight" parameter of "UpdateETFStype_15".</param>
		/// <param name = "Resolution_ppmm"> Corresponds to the "PPImageResolutionPPMM" parameter of "UpdateETFStype_15".</param>
		/// <param name = "PalmQualitiesList"> Corresponds to the "PalmQualitiesList" parameter of "UpdateETFStype_15".</param>
		/// <remarks>
		///	This function is the equivalent of the "UpdateETFStype_15" function of the C version of the library.
		/// </remarks>

        public Int32 NW_UpdateETFStype_15(
            Byte[] ImagePtr,
            Int32 ImageSize,
            Int32 w,
            Int32 h,
            Double Resolution_ppmm,
            List<NW_AN2K_PALM_QUAL> PalmQualitiesList
        )
        {
            try
            {
                IntPtr ImgPtr = IntPtr.Zero;
                Int32 RetVal = 0;
                //////////////////////////////////////////////
                // FINGER IMAGE
                //////////////////////////////////////////////
                if (ImagePtr != null && ImageSize > 0)
                {
                    // check arrays
                    if (ImagePtr.Length < ImageSize)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Number Of pixels less than ImageSize");
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                    }
                    // alloc
                    ImgPtr = Marshal.AllocHGlobal(ImageSize);
                    if (ImgPtr == IntPtr.Zero)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot allocate memory";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    // copy
                    Marshal.Copy(ImagePtr, 0, ImgPtr, ImageSize);
                }
                else
                {
                    ImgPtr = IntPtr.Zero;
                    ImageSize = 0;
                }

                // Image Source is a string...
                // comment is a string...
                // Image compression is a string...
                // Finger Position is a string...

                ////////////////////////////////////////////////
                // PALM quality LIST
                ////////////////////////////////////////////////
                AN2K_PalmQualityList d;
                IntPtr inpPalmQualList = IntPtr.Zero;
                if (PalmQualitiesList != null && PalmQualitiesList.Count > 0)
                {
                    d = An2k_Utils.CreatePalmQualListFromNETList(PalmQualitiesList);
                    if (d.AlgorhythmsNumber == 0)
                    {
                        if (!(ImgPtr.Equals(IntPtr.Zero)))
                        {
                            Marshal.FreeHGlobal(ImgPtr);
                        }
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create PALM quality list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpPalmQualList = An2k_Utils.MarshalToPointer(d);
                }
                else
                {
                    d = new AN2K_PalmQualityList();
                    d.AlgorhythmsNumber = 0;
                }

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));
                //for (int i = 0; i < 100; i++)
                {
                    RetVal = An2kdll_NetWrapper.UpdateETFStype_15(
                        ref x, // REC **
                        ImgPtr,
                        ImageSize, w, h, Resolution_ppmm,
                        inpPalmQualList
                    );
                }

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (!(ImgPtr.Equals(IntPtr.Zero)))
                {
                    Marshal.FreeHGlobal(ImgPtr);
                }
                if (d.AlgorhythmsNumber > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(d.Algorhythms, d.AlgorhythmsNumber);
                }

                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level UpdateETFStype_15 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                ////////////////////////////////////////////////
                // copy record
                ////////////////////////////////////////////////
                this.RecPtr = (IntPtr)(An2k_Utils.MarshalToPointer(x)); ;
                if (this.RecPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level MarshalToStruct function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_image2ETFStype_15: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }
	};
	/***********************************************************************/
	/* ANSI-NIST 2007 TYPE-4 record */
	/***********************************************************************/
	/// <summary> ImageInfo Struct: list of info stored into a Type-4 record </summary>
	/// <remarks>
	/// This class has no equivalents inside the C version of the library, but it is used as a property of the
	/// NW_AN2K_Type4_RECORD, where methods for managing ANSI-NIST type-4 records are collected. 
	/// In particular, getting this property from the NW_AN2K_Type4_RECORD is the equivalent of using the
	/// "Get_EFTStype4_Info" defined into the C version of the library, of which this class is the equivalent of the output.
	/// Infact the members of this class are in a 1:1 correspondence (regarding their meaning) with the output of the 
	/// "Get_EFTStype4_Info" function. Therefore they keep the same name, except for:
	/// - finger_position_list: corresponds to "finger_position"
	/// Infact in the C version "finger_position" is passed as an integer array, because in the type-4 record specifications finger
	/// positions are represented by an integer instead of a string.
	/// </remarks>
	public class NW_AN2K_Type4_StoredInfo
	{
		////////////////////////////////////
		//// CONSTRUCTORS //////////////////
		////////////////////////////////////
		/// <summary> Constructor overload:	Allocates memory for the finger position list	
		/// </summary>
        public NW_AN2K_Type4_StoredInfo()
        {
        }

		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////
		/// <summary> Image Frame bytes	
		/// </summary>
		public Byte []ImagePtr;

		/// <summary> Image size in bytes: can be different from (Width * Height)
		/// </summary>
		public Int32 ImageSize;
		
		/// <summary> Image horizontal dimension
		/// </summary>
		public Int32 w;
		
		/// <summary> Image vertical dimension
		/// </summary>
		public Int32 h;

		/// <summary> Image resolution identifier: should be one of the values defined into the 
		/// <see cref = "NW_AN2K_TYPE_4_RESOLUTION_PARAMETERS"/> class.
		/// </summary>
		public Int32 ImageResolution;

		/// <summary> Image compression algorithm identifier: should be one of the values defined into the 
		/// <see cref = "NW_AN2K_COMP_ALGORITHMS"/> class.
		/// </summary>
		public String ImageCompression;

		/// <summary> Image impression identifier: should be one of the values defined into the 
		/// <see cref = "NW_AN2K_IMPRESSION_CODES"/> class.
		/// </summary>
		public Int32		ImageImpression;

		/// <summary> List of strings indicating the finger position. Item values should be within the 
		/// <see cref = "NW_AN2K_FINGER_POSITIONS"/> class.
		/// </summary>
		public List <String > finger_position_list;
	};

    

	/// <summary> Class representing a Type-4 record.
	/// Inherits from the <see cref = "NW_AN2K_RECORD"/> class. This class, together with the information available in the 
	/// <see cref = "NW_AN2K_RECORD"/>, collects all the methods for managing a record 4 that have an equivalent in the C
	/// version of the library, such as the "Get_EFTStype4_Info" or the "UpdateETFStype_4" functions.
	/// </summary>
	public class NW_AN2K_Type4_RECORD:NW_AN2K_RECORD
	{
		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////
		/// <summary> 
		/// <para> Gets a
		/// <see cref = "NW_AN2K_Type4_StoredInfo"/> structure where information stored in this record are collected.
		/// </para>
		/// <para> CALL <see cref = "NW_image2ETFStype_4"/> BEFORE ACCESSING THIS PROPERTY. 
		/// If a null value is returned, that means that information are not present since the
		/// <see cref = "NW_image2ETFStype_4"/> method has never been called. In such a case see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message
		/// </para>
		/// </summary>
		/// <remarks>
		/// Getting this property is equivalent to calling the "Get_EFTStype4_Info" defined into the C version of the library.
		/// The difference is that a user has not directly a return code, but in case something was wrong:
		/// 1) Info will be null
		/// 2) A more detailed explanation of what happened can be retrieved into the 
		///    <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </remarks>
		public NW_AN2K_Type4_StoredInfo Info 
        {
            get
            {
                try
                {
                    if (this.RecPtr != IntPtr.Zero)
                    {
                        /////////////////////////////////////////////
                        // Get Record struct from **
                        /////////////////////////////////////////////
                        AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));

                        /////////////////////////////////////////////
                        // Prepare container for fingerprint image
                        /////////////////////////////////////////////
                        IntPtr FingImage = IntPtr.Zero;
                        IntPtr pFingImage = An2k_Utils.MarshalToPointer(FingImage);
                        int ImgSize, w, h, ImageResolution, ImgImp;

                        /////////////////////////////////////////////
                        // Prepare container for strings
                        /////////////////////////////////////////////
                        IntPtr ImgComp = Marshal.AllocHGlobal(10);
                        int [] finger_position = new int [6];
                        
                        
                        int RetVal = An2kdll_NetWrapper.Get_EFTStype4_Info(
                            ref x,
                            pFingImage,
                            out ImgSize,
                            out ImgImp,
                            finger_position,
                            out ImageResolution,
                            out w, out h,
                            ImgComp
                        ); 

                        if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                        {
                            NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Get_EFTStype4_Info returned: " + RetVal);
                            return null;
                        }

                        NW_AN2K_Type4_StoredInfo ValToRet = new NW_AN2K_Type4_StoredInfo();
                        ValToRet.finger_position_list = new List<string>();
                        for (int i = 0; i < finger_position.Length; i++)
                        {
                            String k = "" + finger_position[i];
                            ValToRet.finger_position_list.Add(k);
                        }
                        ValToRet.h = h;
                        ValToRet.ImageCompression = An2k_Utils.GetStringFromIntPtr(ImgComp, 10);
                        ValToRet.ImageImpression = ImgImp;
                        if (ImgSize > 0)
                        {
                            FingImage = (IntPtr)An2k_Utils.MarshalToStruct(pFingImage, typeof(IntPtr));
                            if (FingImage != IntPtr.Zero)
                            {
                                ValToRet.ImagePtr = new Byte[ImgSize];
                                Marshal.Copy(FingImage, ValToRet.ImagePtr, 0, ImgSize);
                            }
                            else
                            {
                                ValToRet.ImagePtr = null;
                            }
                        }
                        else
                        {
                            ValToRet.ImagePtr = null;
                        }
                        ValToRet.ImageResolution = ImageResolution;
                        ValToRet.ImageSize = ImgSize; // int
                        ValToRet.w = w;

                        ////////////////////////////////////////////////
                        // free allocated memory
                        ////////////////////////////////////////////////
                        // IMAGE
                        if (!(FingImage.Equals(IntPtr.Zero)))
                        {
                            An2kdll_NetWrapper.free_ANSI_NIST_buffer(FingImage);
                        }
                        Marshal.FreeHGlobal(pFingImage);

                        // STRINGS
                        Marshal.FreeHGlobal(ImgComp);

                        return ValToRet;
                    }
                    else
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("No info in record");
                        return null;
                    }
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Exception in getting NW_AN2K_Type15_RECORD.info: " + ex.Message);
                    return null;
                }
            }
		}

		////////////////////////////////////
		/////// METHODS ////////////////////
		////////////////////////////////////
		/// <summary> 
		/// <para> Creates a type-4 record from the input values. For their meaning see the
		/// <see cref = "NW_AN2K_Type4_StoredInfo"/> structure, while for their name correspondence with the parameters of
		/// the "image2ETFStype_4" function of the C version of the library see the comments below.
		/// </para>
		/// </summary>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		/// <param name = "ImagePtr"> Corresponds to the "idata" parameter of "image2ETFStype_4".</param>
		/// <param name = "ImageSize"> Corresponds to the "ilen" parameter of "image2ETFStype_4".</param>
		/// <param name = "w"> Corresponds to the "ihll" parameter of "image2ETFStype_4".</param>
		/// <param name = "h"> Corresponds to the "ivll" parameter of "image2ETFStype_4".</param>
		/// <param name = "ImageResolution"> Corresponds to the "iisr" parameter of "image2ETFStype_4".</param>
		/// <param name = "ImageCompression"> Corresponds to the "igca" parameter of "image2ETFStype_4".</param>
		/// <param name = "ImageImpression"> Corresponds to the "iimp" parameter of "image2ETFStype_4".</param>
		/// <param name = "finger_position"> Even if up to 6 positions could be entered for the FGP field in the type-4 record
		/// (infact in <see cref = "NW_AN2K_Type4_StoredInfo"/> a list of strings is specified, for our purpose
		/// (tenprint capture for BioAPI 2.0), finger position must be well specified, so just one value can be	entered.
		/// Corresponds to the "ifgp" parameter of "image2ETFStype_4".</param>
		/// <remarks>
		///	This function is the equivalent of the "image2ETFStype_4" function of the C version of the library.
		/// </remarks>
		public Int32 NW_image2ETFStype_4(
			Byte [] ImagePtr,
			Int32 ImageSize,
			Int32 ImageImpression,
			String finger_position,
			Int32 ImageResolution,
			Int32 w,
			Int32 h,
			String ImageCompression
		)
        {
            try
            {
                IntPtr ImgPtr;
                Int32 RetVal;

                //////////////////////////////////////////////
                // RECORD GENERATION
                //////////////////////////////////////////////
                IntPtr y = new IntPtr();
                IntPtr oRec = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(y);

                if (ImageCompression == null)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Bad Image Compression");
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                }


                //////////////////////////////////////////////
                // FINGER IMAGE
                //////////////////////////////////////////////
                if (ImagePtr != null && ImageSize > 0)
                {
                    // check arrays
                    if (ImagePtr.Length < ImageSize)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Number Of pixels less than ImageSize");
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                    }
                    // alloc
                    ImgPtr = Marshal.AllocHGlobal(ImageSize);
                    if (ImgPtr == IntPtr.Zero)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot allocate memory";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    // copy
                    Marshal.Copy(ImagePtr, 0, ImgPtr, ImageSize);
                }
                else
                {
                    ImgPtr = IntPtr.Zero;
                    ImageSize = 0;
                }

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                
                // call low level function
                RetVal = An2kdll_NetWrapper.image2ETFStype_4(
                    oRec, // REC **
                    ImgPtr,
                    ImageSize,
                    ImageImpression,
                    Int32.Parse(finger_position),
                    ImageResolution,
                    w,h,ImageCompression
                );

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (!(ImgPtr.Equals(IntPtr.Zero)))
                {
                    Marshal.FreeHGlobal(ImgPtr);
                }
                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level image2ETFStype_4 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                ////////////////////////////////////////////////
                // copy record
                ////////////////////////////////////////////////
                this.RecPtr = (IntPtr)(An2k_Utils.MarshalToStruct(oRec, typeof(IntPtr))); ;
                if (this.RecPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level MarshalToStruct function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_image2ETFStype_4: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }

		/// <summary> 
		/// <para> Updates a type-4 record from the input values. For their meaning see the
		/// <see cref = "NW_AN2K_Type4_StoredInfo"/> structure, while for their name correspondence with the parameters of
		/// the "UpdateETFStype_4" function of the C version of the library see the comments below.
		/// </para>
		/// </summary>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		/// <param name = "ImagePtr"> Corresponds to the "idata" parameter of "UpdateETFStype_4".</param>
		/// <param name = "ImageSize"> Corresponds to the "ilen" parameter of "UpdateETFStype_4".</param>
		/// <param name = "w"> Corresponds to the "ihll" parameter of "UpdateETFStype_4".</param>
		/// <param name = "h"> Corresponds to the "ivll" parameter of "UpdateETFStype_4".</param>
		/// <param name = "ImageResolution"> Corresponds to the "iisr" parameter of "UpdateETFStype_4".</param>
		/// <remarks>
		///	This function is the equivalent of the "UpdateETFStype_4" function of the C version of the library.
		/// </remarks>
        Int32 NW_UpdateETFStype_4(
            Byte[] ImagePtr,
            Int32 ImageSize,
            Int32 ImageResolution,
            Int32 w, Int32 h
        )
        {
            try
            {
                IntPtr ImgPtr;
                Int32 RetVal;

                //////////////////////////////////////////////
                // RECORD GENERATION
                //////////////////////////////////////////////
                IntPtr y = new IntPtr();
                IntPtr oRec = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(y);


                //////////////////////////////////////////////
                // FINGER IMAGE
                //////////////////////////////////////////////
                if (ImagePtr != null && ImageSize > 0)
                {
                    // check arrays
                    if (ImagePtr.Length < ImageSize)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Number Of pixels less than ImageSize");
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                    }
                    // alloc
                    ImgPtr = Marshal.AllocHGlobal(ImageSize);
                    if (ImgPtr == IntPtr.Zero)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot allocate memory";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    // copy
                    Marshal.Copy(ImagePtr, 0, ImgPtr, ImageSize);
                }
                else
                {
                    ImgPtr = IntPtr.Zero;
                    ImageSize = 0;
                }

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));
                RetVal = An2kdll_NetWrapper.UpdateETFStype_4(
                    ref x, // REC **
                    ImgPtr,
                    ImageSize,
                    ImageResolution,
                    w, h
                );

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (!(ImgPtr.Equals(IntPtr.Zero)))
                {
                    Marshal.FreeHGlobal(ImgPtr);
                }
                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level UpdateETFStype_4 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                ////////////////////////////////////////////////
                // copy record
                ////////////////////////////////////////////////
                this.RecPtr = (IntPtr)(An2k_Utils.MarshalToPointer(x)); ;
                if (this.RecPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level MarshalToStruct function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_UpdateETFStype_4: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }
	};

	/***********************************************************************/
	/* ANSI-NIST 2007 TYPE-2 record */
	/***********************************************************************/
	/// <summary> Info Struct: list of info stored into a Type-2 record </summary>
	/// <remarks>
	/// This class has no equivalents inside the C version of the library, but it is used as a property of the
	/// NW_Type2_RECORD, where methods for managing ANSI-NIST type-2 records are collected. 
	/// In particular, getting this property from the NW_Type2_RECORD is the equivalent of using the
	/// "Get_EFTStype2_Info" defined into the C version of the library, of which this class is the equivalent of the output.
	/// Infact the members of this class are in a 1:1 correspondence (regarding their meaning) with the output of the 
	/// "Get_EFTStype2_Info" function. Therefore they keep the same name, except for:
	/// - amp_list: corresponds to pamp_list
	/// This because in C these parameters are passed as pointers to pointers, so their name reflects (with a "p" as a prefix) 
	/// this way of passing.
	/// </remarks>
	public class NW_AN2K_Type2_StoredInfo
	{
		/// <summary> A string identifying the equipment vendor </summary>
		public String Mak;
		/// <summary> A string identifying the equipment model </summary>
		public String Modl;
		/// <summary> A string identifying the equipment serial number </summary>
		public String SerNo;
		/// <summary> List of structures containing missing fingers indication and reason.
		/// </summary>
		public List<NW_AN2K_FING_AMP > amp_list;
	};

	/// <summary> Class representing a Type-2 record.
	/// Inherits from the <see cref = "NW_AN2K_RECORD"/> class. This class, together with the information available in the 
	/// <see cref = "NW_AN2K_RECORD"/>, collects all the methods for managing a record 2 that have an equivalent in the C
	/// version of the library, such as the "Get_EFTStype2_Info" or the "UpdateETFStype_2" functions.
	/// </summary>
	public class NW_AN2K_Type2_RECORD:NW_AN2K_RECORD
	{
		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////
		/// <summary> 
		/// <para> Gets a
		/// <see cref = "NW_AN2K_Type2_StoredInfo"/> structure where information stored in this record are collected.
		/// </para>
		/// <para> CALL <see cref = "NW_CreateETFStype_2"/> BEFORE ACCESSING THIS PROPERTY. 
		/// If a null value is returned, that means that information are not present since the
		/// <see cref = "NW_CreateETFStype_2"/> method has never been called. In such a case see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message
		/// </para>
		/// </summary>
		/// <remarks>
		/// Getting this property is equivalent to calling the "Get_EFTStype2_Info" defined into the C version of the library.
		/// The difference is that a user has not directly a return code, but in case something was wrong:
		/// 1) Info will be null
		/// 2) A more detailed explanation of what happened can be retrieved into the 
		///    <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </remarks>
		public NW_AN2K_Type2_StoredInfo Info {
            get
            {
                try
                {
                    if (this.RecPtr != IntPtr.Zero)
                    {
                        /////////////////////////////////////////////
                        // Get Record struct from **
                        /////////////////////////////////////////////
                        AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));

                        /////////////////////////////////////////////
                        // Prepare container for strings
                        /////////////////////////////////////////////
                        IntPtr pMak = Marshal.AllocHGlobal(25);
                        IntPtr pModl = Marshal.AllocHGlobal(25);
                        IntPtr pSerNo = Marshal.AllocHGlobal(50);

                        /////////////////////////////////////////////
                        // Prepare container for amputation
                        /////////////////////////////////////////////
                        IntPtr AmpList = new IntPtr();
                        IntPtr pAmpList = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(AmpList);

                        int RetVal = An2kdll_NetWrapper.Get_EFTStype2_Info(
                            ref x,
                            pMak,
                            pModl,
                            pSerNo,
                            pAmpList);

                        if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                        {
                            NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Get_EFTStype2_Info returned: " + RetVal);
                            return null;
                        }

                        /////////////////////////////////////////////
                        // Get amputation list
                        /////////////////////////////////////////////
                        AmpList = (IntPtr)(An2k_Utils.MarshalToStruct(pAmpList, typeof(IntPtr)));
                        AN2K_FingerAmpList oa;
                        List<NW_AN2K_FING_AMP> FingAmpList;
                        if (AmpList != IntPtr.Zero)
                        {
                            oa = (AN2K_FingerAmpList)(An2k_Utils.MarshalToStruct(AmpList, typeof(AN2K_FingerAmpList)));
                            FingAmpList = An2k_Utils.CreateNETListFromAmpList(oa);

                            /////////////////////////////////////////////
                            // free
                            /////////////////////////////////////////////
                            An2kdll_NetWrapper.free_ANSI_NIST_amp_list(AmpList);
                            Marshal.FreeHGlobal(pAmpList);
                        }
                        else
                        {
                            oa = new AN2K_FingerAmpList();
                            oa.num_fingers = 0;
                            FingAmpList = new List<NW_AN2K_FING_AMP>();
                        }

                        NW_AN2K_Type2_StoredInfo ValToRet = new NW_AN2K_Type2_StoredInfo();
                        ValToRet.amp_list = FingAmpList;
                        ValToRet.Mak = An2k_Utils.GetStringFromIntPtr(pMak, 25);
                        ValToRet.Modl = An2k_Utils.GetStringFromIntPtr(pModl, 25);
                        ValToRet.SerNo = An2k_Utils.GetStringFromIntPtr(pModl, 50);

                        ////////////////////////////////////////////////
                        // free allocated memory
                        ////////////////////////////////////////////////

                        // STRINGS
                        Marshal.FreeHGlobal(pMak);
                        Marshal.FreeHGlobal(pModl);
                        Marshal.FreeHGlobal(pModl);

                        return ValToRet;
                    }
                    else
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("No info in record");
                        return null;
                    }
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = ("Exception in getting NW_AN2K_Type2_RECORD.info: " + ex.Message);
                    return null;
                }
            }
		}
		////////////////////////////////////
		/////// METHODS ////////////////////
		////////////////////////////////////
		/// <summary> 
		/// <para> Creates a type-2 record from the input values. For their meaning see the
		/// <see cref = "NW_AN2K_Type2_StoredInfo"/> structure, while for their name correspondence with the parameters of
		/// the "CreateETFStype_2" function of the C version of the library see the comments below.
		/// </para>
		/// </summary>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		/// <param name = "Mak"> Corresponds to the "Mak" parameter of "CreateETFStype_2".</param>
		/// <param name = "Modl"> Corresponds to the "Modl" parameter of "CreateETFStype_2".</param>
		/// <param name = "SerNo"> Corresponds to the "SerNo" parameter of "CreateETFStype_2".</param>
		/// <param name = "amp_list"> Corresponds to the "amp_list" parameter of "CreateETFStype_2".
		/// This list must contain EXACTLY the number of amputated fingers.</param>
		/// <remarks>
		///	This function is the equivalent of the "CreateETFStype_2" function of the C version of the library.
		/// </remarks>
        public Int32 NW_CreateETFStype_2(
            String Mak,
            String Modl,
            String SerNo,
            List<NW_AN2K_FING_AMP> amp_list)
        {
            try
            {
                Int32 RetVal;

                //////////////////////////////////////////////
                // RECORD GENERATION
                //////////////////////////////////////////////
                IntPtr y = new IntPtr();
                IntPtr oRec = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(y);

                ////////////////////////////////////////////////
                // AMPUTATION LIST
                ////////////////////////////////////////////////
                AN2K_FingerAmpList a;
                IntPtr inpAmpList = IntPtr.Zero;
                if (amp_list != null && amp_list.Count > 0)
                {
                    a = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateAmpListFromNETList(amp_list);
                    if (a.num_fingers == 0)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create amputation list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpAmpList = An2k_Utils.MarshalToPointer(a);
                }
                else
                {
                    a = new AN2K_FingerAmpList();
                    a.num_fingers = 0;
                }

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                RetVal = An2kdll_NetWrapper.CreateETFStype_2(
                    oRec, // REC **
                    Mak,
                    Modl, SerNo, inpAmpList
                );

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (a.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                }
                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level CreateETFStype_2 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                ////////////////////////////////////////////////
                // copy record
                ////////////////////////////////////////////////
                this.RecPtr = (IntPtr)(An2k_Utils.MarshalToStruct(oRec, typeof(IntPtr))); ;
                if (this.RecPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level MarshalToStruct function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_CreateETFStype_2: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }

		
		/// <summary> 
		/// <para> Updates a type-2 record from the input values. For their meaning see the
		/// <see cref = "NW_AN2K_Type2_StoredInfo"/> structure, while for their name correspondence with the parameters of
		/// the "UpdateETFStype_2" function of the C version of the library see the comments below.
		/// </para>
		/// </summary>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		/// <param name = "Mak"> Corresponds to the "Mak" parameter of "UpdateETFStype_2".</param>
		/// <param name = "Modl"> Corresponds to the "Modl" parameter of "UpdateETFStype_2".</param>
		/// <param name = "SerNo"> Corresponds to the "SerNo" parameter of "UpdateETFStype_2".</param>
		/// <param name = "amp_list"> Corresponds to the "amp_list" parameter of "UpdateETFStype_2".
		/// This list must contain EXACTLY the number of amputated fingers.</param>
		/// <remarks>
		///	This function is the equivalent of the "UpdateETFStype_2" function of the C version of the library.
		/// </remarks>
		Int32 NW_UpdateETFStype_2(
			String Mak,
			String Modl,
			String SerNo,
            List<NW_AN2K_FING_AMP> amp_list)
        {
            try
            {
                Int32 RetVal = 0;

                // Image Source is a string...
                // comment is a string...
                // Image compression is a string...
                // Finger Position is a string...

                ////////////////////////////////////////////////
                // AMPUTATION LIST
                ////////////////////////////////////////////////
                AN2K_FingerAmpList a;
                IntPtr inpAmpList = IntPtr.Zero;
                if (amp_list != null && amp_list.Count > 0)
                {
                    a = An2k_Net_Wrapper_Utilities.An2k_Utils.CreateAmpListFromNETList(amp_list);
                    if (a.num_fingers == 0)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Cannot create amputation list";
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                    }
                    inpAmpList = An2k_Utils.MarshalToPointer(a);
                }
                else
                {
                    a = new AN2K_FingerAmpList();
                    a.num_fingers = 0;
                }

                ////////////////////////////////////////////////
                // Call low level function
                ////////////////////////////////////////////////
                // call low level function
                AN2K_Record x = (AN2K_Record)(An2k_Utils.MarshalToStruct(this.RecPtr, typeof(AN2K_Record)));
                //for (int i = 0; i < 100; i++)
                {
                    RetVal = An2kdll_NetWrapper.UpdateETFStype_2(
                        ref x,
                        Mak,
                        Modl,
                        SerNo,
                        a
                    );
                }

                ////////////////////////////////////////////////
                // free allocated memory
                ////////////////////////////////////////////////
                // IMAGE
                if (a.num_fingers > 0)
                {
                    An2k_Utils.FreeAllocatedMemForList(a.fingers, a.num_fingers);
                }

                ////////////////////////////////////////////////
                // check returned value
                ////////////////////////////////////////////////
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level UpdateETFStype_2 function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                ////////////////////////////////////////////////
                // copy record
                ////////////////////////////////////////////////
                this.RecPtr = (IntPtr)(An2k_Utils.MarshalToPointer(x)); ;
                if (this.RecPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Error in low level MarshalToStruct function";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_UpdateETFStype_2: " + ex.Message;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }
            // return normally
            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }
	};

    

	// ANSI_NIST info structure
	/***********************************************************************/
	/* ANSI-NIST 2007 structure */
	/***********************************************************************/
	
	/// <summary> Info Struct: list of info stored into a Type-1 record </summary>
	/// <remarks>
	/// This class has no equivalents inside the C version of the library, but it is used as a property of the
	/// NW_AN2K_ANSI_NIST, where methods for managing an ANSI-NIST buffer are collected. 
	/// In particular, getting this property from the NW_AN2K_ANSI_NIST is the equivalent of using the
	/// "GetEFTSType1_Info" defined into the C version of the library, of which this class is the equivalent of the output.
	/// Infact the members of this class are in a 1:1 correspondence (regarding their meaning) with the output of the 
	/// "GetEFTSType1_Info" function. Therefore they keep the same name, except for:
	/// - DestinationAgency: corresponds to pDestinationAgency
	/// - OriginatingAgency: corresponds to pOriginatingAgency	
	/// - TransactionControl: corresponds to pTransContrNum
	/// - TransactionControlReference: corresponds to pTransContrRef
	/// - Domain: corresponds to pDominion
	/// - all the parameters indicating a string length (such as "DestAgencyLen") are not included,
	///		since in .NET "Length" is already a property of the <see cref = "String"/> class
	/// This because in C these parameters are passed as pointers to pointers, so their name reflects (with a "p" as a prefix) 
	/// this way of passing.
	/// </remarks>
	public class NW_AN2K_ANSI_NIST_StoredInfo
	{
		/// <summary> An integer representing the standard version: its value should be within the
		/// <see cref = "NW_AN2K_VERSIONS"/> class.
		/// </summary>
		public Int32 Version;
		/// <summary> A string identifying the transaction type. </summary>
		public String TransactionType;
		/// <summary> An integer identifying the transaction priority. Should range between 0-9. </summary>
		public Int32 Priority;
		/// <summary> A string identifying the transaction destination agency. </summary>
		public String DestinationAgency;
		/// <summary> A string identifying the transaction originating agency. </summary>
		public String OriginatingAgency;
		/// <summary> A string identifying the transaction control. </summary>
		public String TransactionControl;
		/// <summary> A string identifying the transaction control reference. </summary>
		public String TransactionControlReference;
		/// <summary> A floating point number identifying the nominal scanning resolution. </summary>
		public Single NominalScanRes;
		/// <summary> A floating point number identifying the nominal transmitting resolution. </summary>
		public Single NominalTXRes;
		/// <summary> A string identifying the transaction domain. </summary>
		public String Domain;
	};	
	
	/// <summary> Class representing an ANSI-NIST buffer.
	/// This class, together with representing the equivalent of the "ANSI_NIST" structure,
	/// collects all the methods for managing an ANSI_NIST buffer that have an equivalent in the C
	/// version of the library, such as the "Create_ANSI_NIST" or the "write_ANSI_NIST_2_buffer" functions.
	/// </summary>
	public class NW_AN2K_ANSI_NIST
	{
		////////////////////////////////////
		//// CONSTRUCTIORS /////////////////
		////////////////////////////////////
		/// <summary> Default Constructor:
		/// Initizializes parameters
		/// </summary>
		public NW_AN2K_ANSI_NIST()
        {
            this.BinaryBuffer = null;
        }
		/// <summary> Constructor Overload. 
		/// Initializes the class with the input parameters: for their meaning see
		/// the <see cref = "NW_AN2K_ANSI_NIST_StoredInfo"/> class, while for their name correspondence with the parameters of
		/// the "Create_ANSI_NIST" function of the C version of the library see the comments below.
		/// </summary>
		/// <remarks>
		/// <para> If at the end of this function Buffer property is null see the 
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </para>
		/// <para> The class can be initialized alternatively by using the
		/// <see cref = "NW_AN2K_ANSI_NIST.NW_Create_ANSI_NIST"/> method.
		/// </para>
		/// <para>
		/// Using this constructor is equivalent to calling the "Create_ANSI_NIST" function of the C version of the library.
		/// </para>
		/// </remarks>
		/// <param name = "Version"> Corresponds to the "Ver" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "TransactionType"> Corresponds to the "Tot" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "Priority"> Corresponds to the "Pry" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "DestinationAgency"> Corresponds to the "Dai" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "OriginatingAgency"> Corresponds to the "Ori" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "TransactionControl"> Corresponds to the "Tcn" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "TransactionControlReference"> Corresponds to the "Tcr" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "NominalScanRes"> Corresponds to the "Nsr" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "NominalTXRes"> Corresponds to the "Ntr" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "Domain"> Corresponds to the "Dom" parameter of "Create_ANSI_NIST".</param>
		public NW_AN2K_ANSI_NIST(
			Int32 Version,
			String TransactionType,
			Int32 Priority,
			String DestinationAgency,
			String OriginatingAgency,
			String TransactionControl,
			String TransactionControlReference,
			Single NominalScanRes,
			Single NominalTXRes,
			String Domain
		)
        {
            Int32 RetVal = NW_Create_ANSI_NIST(
                Version, TransactionType, Priority, DestinationAgency,
                OriginatingAgency, TransactionControl, TransactionControlReference,
                NominalScanRes, NominalTXRes,
                Domain
            );
		    if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
		    {
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "Create_ANSI_NIST returned: " + RetVal;
                }
		    }
        }

		/// <summary> Constructor Overload. 
		/// Initializes the class by means of an ANSI-NIST formatted buffer.
		/// </summary>
		/// <remarks>
		/// <para> If at the end of this function Buffer property is null</para>
		/// <para>
		/// <see cref = "AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/>
		/// </para>
		/// <para>
		/// Using this constructor is equivalent to calling the "read_ANSI_NIST_buffer" function of the C version of the library.
		/// </para>
		/// </remarks>
		public NW_AN2K_ANSI_NIST(Byte []  InputBuffer)
        {
            this.BinaryBuffer = InputBuffer;
        }

		/// <summary> Default destructor
		/// </summary>
		~NW_AN2K_ANSI_NIST()
        {
        }

		////////////////////////////////////
		//// PROPERTIES ////////////////////
		////////////////////////////////////
		/// <summary> Gets and sets the ANSI-NIST formatted buffer on which the class is based.
		/// </summary>
		/// <remarks>
		/// Getting this property is the equivalent of calling the "write_ANSI_NIST_2_buffer" function, while
		/// setting this property is the equivalent of calling the "read_ANSI_NIST_buffer" function in the C version
		/// of the library.
		/// </remarks>
		public Byte []  Buffer {
            get
            {
                return this.BinaryBuffer;
            }			
            set
            {
                this.BinaryBuffer = value;
            }
		}
		/// <summary> 
		/// <para> Gets a
		/// <see cref = "NW_AN2K_ANSI_NIST_StoredInfo"/> structure where information stored in this record are collected.
		/// </para>
		/// <para>
		/// If a null value is returned, that means that information are not present since the
		/// <see cref = "NW_AN2K_ANSI_NIST.Buffer"/> is null. In such a case see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message
		/// </para>
		/// </summary>
		/// <remarks>
		/// Getting this property is the equivalent of calling the "GetEFTSType1_Info" function in the C version
		/// of the library.
		/// </remarks>
		public NW_AN2K_ANSI_NIST_StoredInfo  Info
        {
            get
            {
                int Counter = 0;
                try
                {
                    NW_AN2K_ANSI_NIST_StoredInfo  Stored = new NW_AN2K_ANSI_NIST_StoredInfo ();
                    
                    // create ANSI_NIST struct
                    AN2K_Struct ansi_nist = new AN2K_Struct();

                    Counter = 1;
                    IntPtr BuffPtr = Marshal.AllocHGlobal(this.BinaryBuffer.Length);
			        if (BuffPtr == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
				        return null;
			        }
                    Marshal.Copy(this.BinaryBuffer,0,BuffPtr,this.BinaryBuffer.Length);
                    IntPtr AnsiNistPtr, pAnsiNistPtr;

                    Counter = 2;
                    AnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToPointer(ansi_nist));
                    if (AnsiNistPtr == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
				        Marshal.FreeHGlobal(BuffPtr);
				        return null;
			        }
                    pAnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToPointer(AnsiNistPtr));
                    Counter = 3;
                    if (pAnsiNistPtr == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
				        Marshal.FreeHGlobal(BuffPtr);
				        Marshal.FreeHGlobal(AnsiNistPtr);
				        return null;
                    }
                    Counter = 4;

                    int RetVal = An2kdll_NetWrapper.read_ANSI_NIST_buffer(BuffPtr, this.BinaryBuffer.Length, pAnsiNistPtr);
                    Counter = 5;
			        if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Unable to read buffer";
				        Marshal.FreeHGlobal(BuffPtr);
				        Marshal.FreeHGlobal(AnsiNistPtr);
				        Marshal.FreeHGlobal(pAnsiNistPtr);
				        return null;
                    }
                    Counter = 6;

                    Marshal.FreeHGlobal(BuffPtr);
                    Counter = 7;

                    AnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAnsiNistPtr, typeof(IntPtr)));
                    if (AnsiNistPtr == IntPtr.Zero)
			        {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Unable to read buffer";
				        Marshal.FreeHGlobal(BuffPtr);
				        return null;
                    }
                    Counter = 10;
                    ansi_nist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(AnsiNistPtr, typeof(AN2K_Struct)));
                    Counter = 11;

                    if (ansi_nist.num_records < 1)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = 
                            "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Record1 missing or not in the correct position";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                        return null;
                    }
                    Counter = 12;

                    //////////////////////////////////
                    // Get record 1
                    //////////////////////////////////
                    Byte[] App = new Byte[(IntPtr.Size)];
                    Marshal.Copy(ansi_nist.records, App, 0, App.Length);
                    IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                    Marshal.Copy(App, 0, pAppPtr, IntPtr.Size);
                    IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));
                    AN2K_Record Record1 = (AN2K_Record)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_Record)));
                    Marshal.FreeHGlobal(pAppPtr);
                    Counter = 13;

                    if (Record1.type != AN2K_NET_WRAPPER.NW_AN2K_AN2007_RECORDS.NW_TYPE_1_ID)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                            "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Record1 type not correct";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                        return null;
                    }
                    Counter = 14;
                    
                    //////////////////////////////////
                    // get record1 info
                    //////////////////////////////////
                    int		Version,Priority;

                    IntPtr	pTransactionType, // char * only
                        pDestinationAgency = IntPtr.Zero, ppDestinationAgency,
                        pOriginatingAgency = IntPtr.Zero, ppOriginatingAgency,
                        pTransContrNum = IntPtr.Zero, ppTransContrNum,
                        pTransContrRef = IntPtr.Zero, ppTransContrRef,
                        pDominion = IntPtr.Zero, ppDominion;
                    Counter = 15;

                    pTransactionType = Marshal.AllocHGlobal(12);                    
			        if (pTransactionType == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
				        return null;
                    }
                    Counter = 16;

                    ppDestinationAgency = An2k_Utils.MarshalToPointer(pDestinationAgency);                    
			        if (ppDestinationAgency == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
				        return null;
                    }
                    Counter = 17;

                    ppOriginatingAgency = An2k_Utils.MarshalToPointer(pOriginatingAgency);                    
			        if (ppOriginatingAgency == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
				        return null;
                    }
                    Counter = 18;

                    ppTransContrNum = An2k_Utils.MarshalToPointer(pTransContrNum);                    
			        if (ppTransContrNum == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
				        return null;
                    }
                    Counter = 19;

                    ppTransContrRef = An2k_Utils.MarshalToPointer(pTransContrRef);                 
			        if (ppTransContrRef == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
				        return null;
                    }
                    Counter = 20;

                    ppDominion = An2k_Utils.MarshalToPointer(pDominion);               
			        if (ppDominion == IntPtr.Zero)
			        {
				        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Cannot allocate memory";
                        IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                        An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
				        return null;
                    }
                    Counter = 21;

			        int		DestAgencyLen,OriginatingAgencyLen,TransContrNumLen,TransContrRefLen,DominionLen;
			        float	NominalScanRes,NominalTXRes;
                    
                    RetVal = An2kdll_NetWrapper.GetEFTSType1_Info(
                        ref Record1,
				        out Version,
				        pTransactionType,
				        out Priority,
				        ppDestinationAgency,out DestAgencyLen,
				        ppOriginatingAgency,	out OriginatingAgencyLen,
				        ppTransContrNum,out TransContrNumLen,
				        ppTransContrRef,out TransContrRefLen,
				        out NominalScanRes,out NominalTXRes,
				        ppDominion,out DominionLen
                       );
                    Counter = 22;
                    IntPtr MyappPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                    An2kdll_NetWrapper.free_ANSI_NIST(MyappPtr);
                    Counter = 8;
                    Marshal.FreeHGlobal(pAnsiNistPtr);
                    Counter = 9;
                    Counter = 23;
                    Counter = 24;
                    if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                    {
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = 
                            "NW_AN2K_ANSI_NIST_StoredInfo, Low Level GetEFTSType1_Info. Code error was: " + RetVal;                        
                        return null;
                    }
                    Counter = 25;
                    
                    // copy values
                    // version
                    Stored.Version = Version;
                    // transaction type
                    Stored.TransactionType = An2k_Utils.GetStringFromIntPtr(pTransactionType,12);
                    Marshal.FreeHGlobal(pTransactionType);
                    // priority
                    Stored.Priority = Priority;
                    // Destination Agency
                    pDestinationAgency = (IntPtr)(An2k_Utils.MarshalToStruct(ppDestinationAgency,typeof(IntPtr)));
                    Marshal.FreeHGlobal(ppDestinationAgency);
                    if (pDestinationAgency != IntPtr.Zero && DestAgencyLen > 0)
                    {
                        Stored.DestinationAgency = An2k_Utils.GetStringFromIntPtr(pDestinationAgency,DestAgencyLen);
                        An2kdll_NetWrapper.free_ANSI_NIST_buffer(pDestinationAgency);
                    }
                    else
                    {
                        Stored.DestinationAgency = null;
                    }

                    // Originating Agency
                    pOriginatingAgency = (IntPtr)(An2k_Utils.MarshalToStruct(ppOriginatingAgency,typeof(IntPtr)));
                    Marshal.FreeHGlobal(ppOriginatingAgency);
                    if (pOriginatingAgency != IntPtr.Zero && DestAgencyLen > 0)
                    {
                        Stored.OriginatingAgency = An2k_Utils.GetStringFromIntPtr(pOriginatingAgency,OriginatingAgencyLen);
                        An2kdll_NetWrapper.free_ANSI_NIST_buffer(pOriginatingAgency);
                    }
                    else
                    {
                        Stored.DestinationAgency = null;
                    }

                    // TransContrNum
                    pTransContrNum = (IntPtr)(An2k_Utils.MarshalToStruct(ppTransContrNum,typeof(IntPtr)));
                    Marshal.FreeHGlobal(ppTransContrNum);
                    if (pTransContrNum != IntPtr.Zero && TransContrNumLen > 0)
                    {
                        Stored.TransactionControl = An2k_Utils.GetStringFromIntPtr(pTransContrNum,TransContrNumLen);
                        An2kdll_NetWrapper.free_ANSI_NIST_buffer(pTransContrNum);
                    }
                    else
                    {
                        Stored.TransactionControl = null;
                    }

                    // TransContrRef
                    pTransContrRef = (IntPtr)(An2k_Utils.MarshalToStruct(ppTransContrRef,typeof(IntPtr)));
                    Marshal.FreeHGlobal(ppTransContrRef);
                    if (pTransContrRef != IntPtr.Zero && TransContrRefLen > 0)
                    {
                        Stored.TransactionControlReference = An2k_Utils.GetStringFromIntPtr(pTransContrRef,TransContrRefLen);
                        An2kdll_NetWrapper.free_ANSI_NIST_buffer(pTransContrRef);
                    }
                    else
                    {
                        Stored.TransactionControlReference = null;
                    }
                    
                    // scanning resolution
                    Stored.NominalScanRes = NominalScanRes;
                    Stored.NominalTXRes = NominalTXRes;
                    
                    // Dominion
                    pDominion = (IntPtr)(An2k_Utils.MarshalToStruct(ppDominion,typeof(IntPtr)));
                    Marshal.FreeHGlobal(ppDominion);
                    if (pDominion != IntPtr.Zero && DominionLen > 0)
                    {
                        Stored.Domain = An2k_Utils.GetStringFromIntPtr(pDominion,DominionLen);
                        An2kdll_NetWrapper.free_ANSI_NIST_buffer(pDominion);
                    }
                    else
                    {
                        Stored.Domain = null;
                    }

                    // free allocated memory
                    Counter = 103;

                    // return normally
                    return Stored;
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo: " 
                        + Counter + ", " + ex.Message;
                    return null;
                }
            }
        }

		// Ver 1.2.0.0: added Update_Type1 function
		/// <summary> 
		/// <para> Updates a type-1 record from the input values. For their name correspondence with the parameters of
		/// the "UpdateETFStype_1" function of the C version of the library see the comments below.
		/// </para>
		/// </summary>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		///  <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		/// <param name = "NominalScanningResolution"> Corresponds to the "NominalScanningResolution" parameter of "UpdateETFStype_1".</param>
		/// <param name = "NominalTransmitResolution"> Corresponds to the "NominalTransmitResolution" parameter of "UpdateETFStype_1".</param>
		/// <remarks>
		///	This function is the equivalent of the "UpdateETFStype_1" function of the C version of the library.
		/// </remarks>
		public Int32 NW_UpdateETFStype_1(
			float		NominalScanningResolution,
			float		NominalTransmitResolution
		)
        {
            try
            {
                // create ANSI_NIST struct
                AN2K_Struct ansi_nist = new AN2K_Struct();

                IntPtr BuffPtr = Marshal.AllocHGlobal(this.BinaryBuffer.Length);
                if (BuffPtr == IntPtr.Zero)
                {
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
                }
                Marshal.Copy(this.BinaryBuffer, 0, BuffPtr, this.BinaryBuffer.Length);
                IntPtr AnsiNistPtr, pAnsiNistPtr;

                AnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToPointer(ansi_nist));
                if (AnsiNistPtr == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(BuffPtr);
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
                }
                pAnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToPointer(AnsiNistPtr));
                if (pAnsiNistPtr == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(BuffPtr);
                    Marshal.FreeHGlobal(AnsiNistPtr);
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
                }

                int RetVal = An2kdll_NetWrapper.read_ANSI_NIST_buffer(BuffPtr, this.BinaryBuffer.Length, pAnsiNistPtr);
                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    Marshal.FreeHGlobal(BuffPtr);
                    Marshal.FreeHGlobal(AnsiNistPtr);
                    Marshal.FreeHGlobal(pAnsiNistPtr);
                    return RetVal;
                }

                Marshal.FreeHGlobal(BuffPtr);

                AnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAnsiNistPtr, typeof(IntPtr)));
                Marshal.FreeHGlobal(pAnsiNistPtr);
                if (AnsiNistPtr == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Unable to read buffer";
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
                ansi_nist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(AnsiNistPtr, typeof(AN2K_Struct)));

                if (ansi_nist.num_records < 1)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Record1 missing or not in the correct position";
                    IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                //////////////////////////////////
                // Get record 1
                //////////////////////////////////
                Byte[] App = new Byte[(IntPtr.Size)];
                Marshal.Copy(ansi_nist.records, App, 0, App.Length);
                IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                Marshal.Copy(App, 0, pAppPtr, IntPtr.Size);
                IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));
                AN2K_Record Record1 = (AN2K_Record)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_Record)));
                Marshal.FreeHGlobal(pAppPtr);

                if (Record1.type != AN2K_NET_WRAPPER.NW_AN2K_AN2007_RECORDS.NW_TYPE_1_ID)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Record1 type not correct";
                    IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }

                RetVal = An2kdll_NetWrapper.UpdateETFStype_1(
                    ref Record1,
                    NominalScanningResolution,
                    NominalTransmitResolution
                );
			    if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
			    {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_UpdateETFStype_1, Low Level: Unable to update Type-1 record";
                    IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
				    return RetVal;
			    }

			    // write ANSI_NIST structure to buffer again
                IntPtr pBuffPtr = An2k_Utils.MarshalToPointer(BuffPtr);
			    int bufLen;
                RetVal = An2kdll_NetWrapper.write_ANSI_NIST_2_buffer(pBuffPtr, out bufLen, ref ansi_nist);
                IntPtr MyappPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                An2kdll_NetWrapper.free_ANSI_NIST(MyappPtr);

			    if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
			    {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_UpdateETFStype_1, Low Level: Unable to write buffer";
				    Marshal.FreeHGlobal(BuffPtr);
				    return RetVal;
			    }

                BuffPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pBuffPtr, typeof(IntPtr)));

			    //Marshal.Copy((IntPtr)WriteBuffer,
			    this.Buffer = new Byte[bufLen];
                Marshal.Copy(BuffPtr, this.Buffer, 0, bufLen);

                // free memory
                An2kdll_NetWrapper.free_ANSI_NIST_buffer(BuffPtr);
		    }
		    catch (Exception ex)
            {
			    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_UpdateETFStype_1: " + ex.Message;
			    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
		    }
		    return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }
		// end of Ver 1.2.0.0: added Update_Type1 function
		
		/// <summary> 
		/// <para> Gets the number of records stored within this object.
		/// </para>
		/// <para>
		/// If a negative value is returned, that means that information are not present since the
		/// <see cref = "NW_AN2K_ANSI_NIST.Buffer"/> is null. In such a case see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message
		/// </para>
		/// </summary>
		public  Int32 num_records {
            get
            {
                try
                {
                    // create ANSI_NIST struct
                    AN2K_Struct ansi_nist = new AN2K_Struct();

                    IntPtr BuffPtr = Marshal.AllocHGlobal(this.BinaryBuffer.Length);
                    if (BuffPtr == IntPtr.Zero)
                    {
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
                    }
                    Marshal.Copy(this.BinaryBuffer, 0, BuffPtr, this.BinaryBuffer.Length);
                    IntPtr AnsiNistPtr, pAnsiNistPtr;

                    AnsiNistPtr = IntPtr.Zero;
                    pAnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToPointer(AnsiNistPtr));
                    if (pAnsiNistPtr == IntPtr.Zero)
                    {
                        Marshal.FreeHGlobal(BuffPtr);
                        Marshal.FreeHGlobal(AnsiNistPtr);
                        return NW_AN2K_ERRORS.NW_AN2K_DLL_MEMORY;
                    }

                    int RetVal = An2kdll_NetWrapper.read_ANSI_NIST_buffer(BuffPtr, this.BinaryBuffer.Length, pAnsiNistPtr);
                    if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                    {
                        Marshal.FreeHGlobal(BuffPtr);
                        Marshal.FreeHGlobal(AnsiNistPtr);
                        Marshal.FreeHGlobal(pAnsiNistPtr);
                        return RetVal;
                    }

                    Marshal.FreeHGlobal(BuffPtr);

                    AnsiNistPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAnsiNistPtr, typeof(IntPtr)));
                    if (AnsiNistPtr == IntPtr.Zero)
                    {
                        Marshal.FreeHGlobal(pAnsiNistPtr);
                        NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_AN2K_ANSI_NIST_StoredInfo, Low Level: Unable to read buffer";
                        throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                    }
                    ansi_nist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(AnsiNistPtr, typeof(AN2K_Struct)));

                    Int32 ValToRet = ansi_nist.num_records;
                    IntPtr appPtr = Marshal.ReadIntPtr(pAnsiNistPtr);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                    //Marshal.FreeHGlobal(pAnsiNistPtr);

                    return ValToRet;
                }
                catch (Exception ex)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "num_records property: " + ex.Message;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
            }
		}

		////////////////////////////////////
		/////// METHODS ////////////////////
		////////////////////////////////////
		/// <summary>  
		/// Initializes or updates the class with the input parameters: for their meaning see
		/// the <see cref = "NW_AN2K_ANSI_NIST_StoredInfo"/> class, while for their name correspondence with the parameters of
		/// the "Create_ANSI_NIST" function of the C version of the library see the comments below.
		/// </summary>
		/// <remarks>
		///	This function is the equivalent of the "Create_ANSI_NIST" function of the C version of the library.
		/// </remarks>
		/// <returns> A <see cref = "System.Int32"/> value within the  <see cref = "NW_AN2K_ERRORS"/> class.
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// <para> If at the end of this function Buffer property is null see the 
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </para>
		/// </returns>
		/// <param name = "Version"> Corresponds to the "Ver" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "TransactionType"> Corresponds to the "Tot" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "Priority"> Corresponds to the "Pry" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "DestinationAgency"> Corresponds to the "Dai" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "OriginatingAgency"> Corresponds to the "Ori" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "TransactionControl"> Corresponds to the "Tcn" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "TransactionControlReference"> Corresponds to the "Tcr" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "NominalScanRes"> Corresponds to the "Nsr" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "NominalTXRes"> Corresponds to the "Ntr" parameter of "Create_ANSI_NIST".</param>
		/// <param name = "Domain"> Corresponds to the "Dom" parameter of "Create_ANSI_NIST".</param>
		public Int32 NW_Create_ANSI_NIST(
			Int32 Version,
			String TransactionType,
			Int32 Priority,
			String DestinationAgency,
			String OriginatingAgency,
			String TransactionControl,
			String TransactionControlReference,
			Single NominalScanRes,
			Single NominalTXRes,
			String Domain
		)
        {
            int ErrorCounter = 0;
            try 
            {
                int RetVal;

                ErrorCounter = 1;
                
                // check mandatory
                if (TransactionType == null || TransactionType.Length == 0 ||
                    DestinationAgency == null || DestinationAgency.Length == 0 ||
                    OriginatingAgency == null || OriginatingAgency.Length == 0 ||
                    TransactionControl == null || TransactionControl.Length == 0
                    ) 
                {
                    this.BinaryBuffer = null;
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                }

                ErrorCounter = 2;

                //////////////////////////////////////////////
                // RECORD GENERATION
                //////////////////////////////////////////////
                IntPtr pAnsiNist = new IntPtr();
                ErrorCounter = 3;
                IntPtr ppAnsiNist = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(pAnsiNist);
                ErrorCounter = 4;
                
                // now call create ansi nist
                RetVal = An2kdll_NetWrapper.Create_ANSI_NIST(
                    ppAnsiNist,
                    Version,
                    TransactionType,
                    Priority,
                    DestinationAgency,
                    OriginatingAgency,
                    TransactionControl,
                    TransactionControlReference, 
                    NominalScanRes, NominalTXRes,
                    Domain
                );

                ErrorCounter = 5;

                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    // delete allocated memory
                    // ...
                    // return error
                    this.BinaryBuffer = null;
                    return RetVal;
                }

                ErrorCounter = 6;

                pAnsiNist = (IntPtr)(An2k_Utils.MarshalToStruct(ppAnsiNist, typeof(IntPtr)));
                ErrorCounter = 7;
                if (pAnsiNist == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = 
                        "NW_Create_ANSI_NIST: a null ptr was returned by low level Create_ANSI_NIST() function";
                    this.BinaryBuffer = null;
                    ErrorCounter = 9;
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
                }
                ErrorCounter = 8;
                AN2K_Struct ansi_nist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(pAnsiNist, typeof(AN2K_Struct)));
                ErrorCounter = 10;
                IntPtr AppBuffer = new IntPtr();
                ErrorCounter = 11;
                IntPtr pAppBuffer = An2k_Net_Wrapper_Utilities.An2k_Utils.MarshalToPointer(AppBuffer);
                ErrorCounter = 12;
                int BufLen;

                RetVal = An2kdll_NetWrapper.write_ANSI_NIST_2_buffer(pAppBuffer, out BufLen, ref ansi_nist);
                ErrorCounter = 13;

                // delete allocated memory
                // ... 
                IntPtr appPtr = Marshal.ReadIntPtr(ppAnsiNist);
                ErrorCounter = 14;
                An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                ErrorCounter = 15;

                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {                    
                    // return error
                    return RetVal;
                }
                ErrorCounter = 16;
                AppBuffer = (IntPtr)(An2k_Utils.MarshalToStruct(pAppBuffer, typeof(IntPtr)));
                ErrorCounter = 17;
                this.BinaryBuffer = new Byte[BufLen];
                ErrorCounter = 18;
                Marshal.Copy(AppBuffer, this.BinaryBuffer, 0, BufLen);
                ErrorCounter = 19;
            }
            catch (Exception ex) {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_Create_ANSI_NIST: " + ErrorCounter  + ", "+ ex.Message;
                this.BinaryBuffer = null;
                return NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION;
            }

            return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
        }

		/// <summary>  
		/// Returns a list of the records contained inside this object whose type id matches with the
		/// <paramref name = "RecordType"/> input parameter.
		/// </summary>
		/// <param name = "RecordType"> Type ID of the wanted records. Corresponds to the "rec_type" parameter
		/// of the "search_ANSI_NIST_record" function.
		/// </param>
		/// <returns>
		/// A list of the records contained inside this object whose type id matches with the
		/// <paramref name = "RecordType"/> input parameter.
		/// <para> If returned value is null see the 
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </para> 
		/// </returns>
		/// <remarks>
		/// Corresponds to the "search_ANSI_NIST_record" function in the C version of the library.
		/// </remarks>
		List<NW_AN2K_RECORD >  NW_search_ANSI_NIST_record(UInt32 RecordType)
        {
            try
            {
                IntPtr pAnsiNist = IntPtr.Zero;
                IntPtr ppAnsiNist = An2k_Utils.MarshalToPointer(pAnsiNist);
                if (ppAnsiNist == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_search_ANSI_NIST_record: cannot allocate ppAnsiNist";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // read ansi_nist buffer
                IntPtr ptrToBuffer = Marshal.AllocHGlobal(this.BinaryBuffer.Length);
                if (ptrToBuffer == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_search_ANSI_NIST_record: cannot allocate ptrToBuffer";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                Marshal.Copy(this.BinaryBuffer, 0, ptrToBuffer, this.BinaryBuffer.Length);

                Int32 RetVal = An2kdll_NetWrapper.read_ANSI_NIST_buffer(ptrToBuffer, this.BinaryBuffer.Length, ppAnsiNist);
                Marshal.FreeHGlobal(ptrToBuffer);

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_search_ANSI_NIST_record, Low level: read_ANSI_NIST_buffer() returned " + RetVal;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // get ansi_nist structure
                pAnsiNist = (IntPtr)(An2k_Utils.MarshalToStruct(ppAnsiNist, typeof(IntPtr)));
                Marshal.FreeHGlobal(ppAnsiNist);
                if (pAnsiNist == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_search_ANSI_NIST_record: pAnsiNist null ptr";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                AN2K_Struct AnsiNist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(pAnsiNist, typeof(AN2K_Struct)));

                // create RecordList structure
                IntPtr pList = IntPtr.Zero;
                IntPtr ppList = An2k_Utils.MarshalToPointer(pList);
                if (ppList == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_search_ANSI_NIST_record: cannot allocate ppList";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // Load list
                RetVal = An2kdll_NetWrapper.search_ANSI_NIST_record(ppList, RecordType, ref AnsiNist);

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    IntPtr appPtr = Marshal.ReadIntPtr(ppAnsiNist);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                    Marshal.FreeHGlobal(ppList);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_search_ANSI_NIST_record, Low level: search_ANSI_NIST_record() returned " + RetVal;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // get list from **ppList
                pList = (IntPtr)(An2k_Utils.MarshalToStruct(ppList, typeof(IntPtr)));
                if (pList == IntPtr.Zero)
                {
                    IntPtr appPtr = Marshal.ReadIntPtr(ppAnsiNist);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                    Marshal.FreeHGlobal(ppList);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_search_ANSI_NIST_record: pList null ptr";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                AN2K_RecordList RecList = (AN2K_RecordList)(An2k_Utils.MarshalToStruct(pList, typeof(AN2K_RecordList)));
                if (RecList.records == IntPtr.Zero || RecList.num_records == 0)
                {
                    IntPtr appPtr = Marshal.ReadIntPtr(ppAnsiNist);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                    An2kdll_NetWrapper.free_ANSI_NIST_record_list(pList);
                    Marshal.FreeHGlobal(ppList);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_search_ANSI_NIST_record: pList null ptr";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // now copy any element of the list
                Byte[] App = new Byte[(IntPtr.Size) * (RecList.num_records)];
                Marshal.Copy(RecList.records, App, 0, App.Length);
                List<NW_AN2K_RECORD> a = new List<NW_AN2K_RECORD>();

                // create pointers
                for (int i = 0; i < RecList.num_records; i++)
                {
                    IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                    Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                    IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

                    AN2K_Record RecordListElement = (AN2K_Record)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_Record)));

                    AN2K_Record Dest = new AN2K_Record();
                    An2kdll_NetWrapper.rec_copy(ref Dest, ref RecordListElement);

                    NW_AN2K_RECORD NetListElement = new NW_AN2K_RECORD(An2k_Utils.MarshalToPointer(Dest));

                    a.Add(NetListElement);

                    Marshal.FreeHGlobal(pAppPtr);
                }

                // free memory
                IntPtr MyappPtr = Marshal.ReadIntPtr(ppAnsiNist);
                An2kdll_NetWrapper.free_ANSI_NIST(MyappPtr);
                An2kdll_NetWrapper.free_ANSI_NIST_record_list(pList);
                Marshal.FreeHGlobal(ppList);

                return a;
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_search_ANSI_NIST_record: " + ex.Message;
                throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
            }
        }

        // Ver 1.3.0.1: added possibility to get a list of records contained
        public List<NW_AN2K_RECORD> NW_GetRecordsList()
        {
            try
            {
                IntPtr pAnsiNist = IntPtr.Zero;
                IntPtr ppAnsiNist = Marshal.AllocHGlobal(IntPtr.Size);
                if (ppAnsiNist == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_GetRecordsList: cannot allocate ppAnsiNist";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // read ansi_nist buffer
                IntPtr ptrToBuffer = Marshal.AllocHGlobal(this.BinaryBuffer.Length);
                if (ptrToBuffer == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_GetRecordsList: cannot allocate ptrToBuffer";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                Marshal.Copy(this.BinaryBuffer, 0, ptrToBuffer, this.BinaryBuffer.Length);

                Int32 RetVal = An2kdll_NetWrapper.read_ANSI_NIST_buffer(ptrToBuffer, this.BinaryBuffer.Length, ppAnsiNist);
                Marshal.FreeHGlobal(ptrToBuffer);

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_GetRecordsList, Low level: read_ANSI_NIST_buffer() returned " + RetVal;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // get ansi_nist structure
                pAnsiNist = (IntPtr)(Marshal.ReadIntPtr(ppAnsiNist));
                if (pAnsiNist == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_GetRecordsList: pAnsiNist null ptr";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                AN2K_Struct AnsiNist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(pAnsiNist, typeof(AN2K_Struct)));

                // now copy any element of the list
                Byte[] App = new Byte[(IntPtr.Size) * (AnsiNist.num_records)];
                Marshal.Copy(AnsiNist.records, App, 0, App.Length);
                List<NW_AN2K_RECORD> a = new List<NW_AN2K_RECORD>();

                // create pointers
                for (int i = 0; i < AnsiNist.num_records; i++)
                {
                    IntPtr pAppPtr = Marshal.AllocHGlobal(IntPtr.Size);
                    Marshal.Copy(App, 0 + i * (IntPtr.Size), pAppPtr, IntPtr.Size);
                    IntPtr AppPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pAppPtr, typeof(IntPtr)));

                    AN2K_Record RecordListElement = (AN2K_Record)(An2k_Utils.MarshalToStruct(AppPtr, typeof(AN2K_Record)));

                    AN2K_Record Dest = new AN2K_Record();
                    An2kdll_NetWrapper.rec_copy(ref Dest, ref RecordListElement);

                    NW_AN2K_RECORD NetListElement = new NW_AN2K_RECORD(An2k_Utils.MarshalToPointer(Dest));

                    //An2kdll_NetWrapper.free_ANSI_NIST_record(AppPtr);

                    a.Add(NetListElement);

                    Marshal.FreeHGlobal(pAppPtr);
                }

                // free memory
                An2kdll_NetWrapper.free_ANSI_NIST(pAnsiNist);
                Marshal.FreeHGlobal(ppAnsiNist);

                return a;
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_GetRecordsList: " + ex.Message;
                throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
            }
        }
        // end Ver 1.3.0.1: added possibility to get a list of records contained

		
		/// <summary>  
		/// Routine used to insert the logical record passed as parameter into this object.
		/// This routine also updates the CNT field of the Type-1 record to accurately reflect the insertion.
		/// </summary>
		/// <param name = "Record"> Record to be inserted.
		/// </param>
		/// <param name = "Position"> Desired position where record should be inserted.
		/// RECOMMENDATION: since records order within list is not important,
		/// if record 2, put in position "1", otherwise in position <see cref = "num_records"/>
		/// </param>
		/// <remarks>
		/// <para>
		/// In this routine, an IDC is assigned based on the record's position in the ANSI/NIST structure.
		/// </para>
		/// <para>
		/// Corresponds to the "insert_ANSI_NIST_record_ETFS" function in the C version of the library.
		/// </para>
		/// </remarks>
		/// <returns>
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
		public Int32 NW_insert_ANSI_NIST_record_ETFS(NW_AN2K_RECORD Record,Int32 Position)
        {
            int Counter = 0;
            try
            {
                if (Record.RecPtr == IntPtr.Zero)
                {
                    return NW_AN2K_ERRORS.NW_AN2K_DLL_BAD_PARAMETER;
                }
                IntPtr pAnsiNist = IntPtr.Zero;
                IntPtr ppAnsiNist = Marshal.AllocHGlobal(IntPtr.Size);
                if (ppAnsiNist == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_insert_ANSI_NIST_record_ETFS: cannot allocate ppAnsiNist";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                Counter = 1;

                // read ansi_nist buffer
                IntPtr ptrToBuffer = Marshal.AllocHGlobal(this.BinaryBuffer.Length);
                if (ptrToBuffer == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_insert_ANSI_NIST_record_ETFS: cannot allocate ptrToBuffer";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                Marshal.Copy(this.BinaryBuffer, 0, ptrToBuffer, this.BinaryBuffer.Length);

                Int32 RetVal = An2kdll_NetWrapper.read_ANSI_NIST_buffer(ptrToBuffer, this.BinaryBuffer.Length, ppAnsiNist);

                Counter = 2;
                Marshal.FreeHGlobal(ptrToBuffer);
                Counter = 3;

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    Counter = 8;
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_insert_ANSI_NIST_record_ETFS, Low level: read_ANSI_NIST_buffer() returned " + RetVal;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Counter = 9;
                // get ansi_nist structure
                pAnsiNist = Marshal.ReadIntPtr(ppAnsiNist);
                if (pAnsiNist == IntPtr.Zero)
                {
                    Counter = 10;
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_insert_ANSI_NIST_record_ETFS: pAnsiNist null ptr";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }


                Counter = 11;
                AN2K_Record Dest = new AN2K_Record();
                AN2K_Record Source = (AN2K_Record)(An2k_Utils.MarshalToStruct(Record.RecPtr,
                     typeof(AN2K_Record)));
                Counter = 12;
                An2kdll_NetWrapper.rec_copy(ref Dest, ref Source);

                Counter = 6;
                RetVal = An2kdll_NetWrapper.insert_ANSI_NIST_record_ETFS(Position, ref Dest, pAnsiNist);
                Counter = 7;

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    An2kdll_NetWrapper.free_ANSI_NIST(pAnsiNist);
                    Counter = 4;
                    Marshal.FreeHGlobal(ppAnsiNist);
                    Counter = 5; 
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_delete_ANSI_NIST_record, Low level: insert_ANSI_NIST_record_ETFS() returned " + RetVal;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // write ANSI_NIST structure to buffer again
                IntPtr AppBuffer = new IntPtr();
                IntPtr pAppBuffer = Marshal.AllocHGlobal(IntPtr.Size);
                if (pAppBuffer == IntPtr.Zero)
                {
                    Counter = 13;
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_insert_ANSI_NIST_record_ETFS: pAppBuffer null ptr";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                int BufLen;

                AN2K_Struct AnsiNist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(pAnsiNist, typeof(AN2K_Struct)));

                RetVal = An2kdll_NetWrapper.write_ANSI_NIST_2_buffer(pAppBuffer, out BufLen, ref AnsiNist);

                // delete allocated memory
                // Last record was created inside here
                for (int i = 0; i < Position; i++ )
                {
                    IntPtr MyPtr = Marshal.ReadIntPtr(AnsiNist.records, i * (IntPtr.Size));
                    An2kdll_NetWrapper.free_ANSI_NIST_record(MyPtr);
                }
                for (int i = Position + 1; i < (AnsiNist.num_records - 1); i++)
                {
                    IntPtr MyPtr = Marshal.ReadIntPtr(AnsiNist.records, i * (IntPtr.Size));
                    An2kdll_NetWrapper.free_ANSI_NIST_record(MyPtr);
                }
                //Marshal.FreeHGlobal(pAnsiNist);

                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    // return error
                    return RetVal;
                }
                AppBuffer = Marshal.ReadIntPtr(pAppBuffer);
                this.BinaryBuffer = new Byte[BufLen];
                Marshal.Copy(AppBuffer, this.BinaryBuffer, 0, BufLen);

                An2kdll_NetWrapper.free_ANSI_NIST_buffer(AppBuffer);

                return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_insert_ANSI_NIST_record_ETFS: Count = " + Counter
                    + ", " + ex.Message;
                throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
            }
        }

		// Ver 1.2.0.0: added possibility to delete a record

		/// <summary>  
		/// Routine used to delete the logical record passed as parameter from this object.
		/// This routine also updates the CNT field of the Type-1 record to accurately
		/// reflect the insertion.
		/// </summary>
		/// <param name = "RecordIndex"> index of logical record to be deleted
		/// </param>
		/// <remarks>
		/// <para>
		/// In this routine, an IDC is assigned based on the record's position in the ANSI/NIST structure.
		/// </para>
		/// <para>
		/// Corresponds to the "insert_ANSI_NIST_record_ETFS" function in the C version of the library.
		/// </para>
		/// </remarks>
		/// <returns>
		/// If <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION"/> is returned, see the
		/// <see cref = "NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING"/> message.
		/// </returns>
        public Int32 NW_delete_ANSI_NIST_record(Int32 RecordIndex)
        {
            try
            {
                IntPtr pAnsiNist = IntPtr.Zero;
                IntPtr ppAnsiNist = An2k_Utils.MarshalToPointer(pAnsiNist);
                if (ppAnsiNist == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_delete_ANSI_NIST_record: cannot allocate ppAnsiNist";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // read ansi_nist buffer
                IntPtr ptrToBuffer = Marshal.AllocHGlobal(this.BinaryBuffer.Length);
                if (ptrToBuffer == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_delete_ANSI_NIST_record: cannot allocate ptrToBuffer";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                Marshal.Copy(this.BinaryBuffer, 0, ptrToBuffer, this.BinaryBuffer.Length);

                Int32 RetVal = An2kdll_NetWrapper.read_ANSI_NIST_buffer(ptrToBuffer, this.BinaryBuffer.Length, ppAnsiNist);
                Marshal.FreeHGlobal(ptrToBuffer);

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    Marshal.FreeHGlobal(ppAnsiNist);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_delete_ANSI_NIST_record, Low level: read_ANSI_NIST_buffer() returned " + RetVal;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // get ansi_nist structure
                pAnsiNist = (IntPtr)(An2k_Utils.MarshalToStruct(ppAnsiNist, typeof(IntPtr)));
                Marshal.FreeHGlobal(ppAnsiNist);
                if (pAnsiNist == IntPtr.Zero)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_delete_ANSI_NIST_record: pAnsiNist null ptr";
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                AN2K_Struct AnsiNist = (AN2K_Struct)(An2k_Utils.MarshalToStruct(pAnsiNist, typeof(AN2K_Struct)));

                RetVal = An2kdll_NetWrapper.delete_ANSI_NIST_record(RecordIndex, pAnsiNist);

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    IntPtr appPtr = Marshal.ReadIntPtr(ppAnsiNist);
                    An2kdll_NetWrapper.free_ANSI_NIST(appPtr);
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING =
                        "NW_delete_ANSI_NIST_record, Low level: delete_ANSI_NIST_record() returned " + RetVal;
                    throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }

                // write ANSI_NIST structure to buffer again
                IntPtr BuffPtr = IntPtr.Zero;
                IntPtr pBuffPtr = An2k_Utils.MarshalToPointer(BuffPtr);
                int bufLen;
                RetVal = An2kdll_NetWrapper.write_ANSI_NIST_2_buffer(pBuffPtr, out bufLen, ref AnsiNist);
                An2kdll_NetWrapper.free_ANSI_NIST(pAnsiNist);

                if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_delete_ANSI_NIST_record, Low Level: Unable to write buffer";
                    Marshal.FreeHGlobal(BuffPtr);
                    return RetVal;
                }

                BuffPtr = (IntPtr)(An2k_Utils.MarshalToStruct(pBuffPtr, typeof(IntPtr)));

                //Marshal.Copy((IntPtr)WriteBuffer,
                this.Buffer = new Byte[bufLen];
                Marshal.Copy(BuffPtr, this.Buffer, 0, bufLen);

                // free memory
                An2kdll_NetWrapper.free_ANSI_NIST_buffer(BuffPtr);

                return NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR;
            }
            catch (Exception ex)
            {
                NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING = "NW_delete_ANSI_NIST_record: " + ex.Message;
                throw new Exception(NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
            }
        }
		// end Ver 1.2.0.0: added possibility to delete a record
        
        private Byte [] BinaryBuffer;
	};
}
